/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_app.hpp"
#include "ff_gui.hpp"
#include "ff_modulemanager.hpp"
#include "bankaccounts/fm_bankaccounts.hpp"
#include "transfers/fm_transfers.hpp"
#include "outbox/fm_outbox.hpp"
#include "accounts/fm_accounts.hpp"
#include "partners/fm_partners.hpp"
#include "business/fm_business.hpp"
#include "vehicles/fm_vehicles.hpp"
#include "reports/fm_reports.hpp"
#include "mainwindow.hpp"

#include <aqbanking/abgui.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_gui.hpp>

#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/configmgr.h>
#include <gwenhywfar/buffer.h>
#include <gwenhywfar/directory.h>
#include <gwenhywfar/gwenhywfarapi.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/args.h>

#include <aqbanking/dlg_importer.h>


#ifdef OS_DARWIN
# include <Carbon/Carbon.h>
#endif



int main(int argc, char **argv) {
  FF_App app("AqFinance", "Martin Preuss");
  FF_Gui *gui;
  MainWindow *mw;
  FF_Module *m;
  FXString lf;
  FXString str;
  AB_BANKING *ab;
  int rv;
  bool wantRestart=false;
  bool debug;
  GWEN_DB_NODE *db;
  const char *s;
  GWEN_PASSWD_STORE *sto;
  GWEN_DB_NODE *dbPasswords;
  const GWEN_ARGS args[]={
  {
    0,                            /* flags */
    GWEN_ArgsType_Int,            /* type */
    "debug",                      /* name */
    0,                            /* minnum */
    1,                            /* maxnum */
    NULL,                         /* short option */
    "debug",                      /* long option */
    "Increase the loglevel",/* short description */
    "Increase the loglevel"
  },
  {
    GWEN_ARGS_FLAGS_HELP | GWEN_ARGS_FLAGS_LAST, /* flags */
    GWEN_ArgsType_Int,            /* type */
    "help",                       /* name */
    0,                            /* minnum */
    0,                            /* maxnum */
    "h",                          /* short option */
    "help",
    I18S("Show this help screen. For help on commands, "
	 "run aqfinance-cli <COMMAND> --help."),
    I18S("Show this help screen. For help on commands, run aqfinance-cli <COMMAND> --help.")
  }
  };

  rv=GWEN_Init();
  if (rv) {
    fprintf(stderr, "ERROR: Unable to init Gwen.\n");
    exit(2);
  }

  rv=AQDB_Init();
  if (rv) {
    fprintf(stderr, "ERROR: Unable to init AqDatabase.\n");
    exit(2);
  }

  GWEN_Logger_Open(0, "aqfinance", 0,
		   GWEN_LoggerType_Console,
		   GWEN_LoggerFacility_User);
  GWEN_Logger_SetLevel(0, GWEN_LoggerLevel_Warning);

  app.init(argc, argv);
  db=GWEN_DB_Group_new("arguments");
  rv=GWEN_Args_Check(argc, argv, 1,
		     GWEN_ARGS_MODE_ALLOW_FREEPARAM |
		     GWEN_ARGS_MODE_STOP_AT_FREEPARAM,
		     args,
		     db);
  if (rv==GWEN_ARGS_RESULT_ERROR) {
    fprintf(stderr, "ERROR: Could not parse arguments main\n");
    return 1;
  }
  else if (rv==GWEN_ARGS_RESULT_HELP) {
    GWEN_BUFFER *ubuf;

    ubuf=GWEN_Buffer_new(0, 1024, 0, 1);
    GWEN_Buffer_AppendString(ubuf,
                             I18N("Usage: "));
    GWEN_Buffer_AppendString(ubuf, argv[0]);
    GWEN_Buffer_AppendString(ubuf,
                             I18N(" [GLOBAL OPTIONS]"
                                  "[LOCAL OPTIONS]\n"));
    GWEN_Buffer_AppendString(ubuf,
                             I18N("\nGlobal Options:\n"));
    if (GWEN_Args_Usage(args, ubuf, GWEN_ArgsOutType_Txt)) {
      fprintf(stderr, "ERROR: Could not create help string\n");
      return 1;
    }
    /* end of text */
    GWEN_Buffer_AppendString(ubuf, "\n");

    fprintf(stderr, "%s\n", GWEN_Buffer_GetStart(ubuf));
    GWEN_Buffer_free(ubuf);
    return 0;
  }

  debug=(GWEN_DB_GetIntValue(db, "debug", 0, 0)!=0);

  /* init application */
  AE_Init(NULL);

  ab=AB_Banking_new("AqFinance", NULL, 0);
  app.setBanking(ab);

  /* create GUI */
  s=AE_GetUserDataFolder();
  gui=new FF_Gui(&app, s);
  str=FXString(s);
  str+=GWEN_DIR_SEPARATOR_S;
  str+="passwd.db";
  sto=GWEN_PasswordStore_new(str.text());
  gui->setPasswordStore(sto);
  dbPasswords=GWEN_DB_Group_new("TempPasswords");
  gui->setPasswordDb(dbPasswords, 0);

  GWEN_Gui_SetGui(gui->getCInterface());
  gui->setupLogging(debug);

  init_theme(&app);

  mw=new MainWindow(&app, "AqFinance", NULL, NULL, DECOR_ALL);
  app.create();

  rv=AB_Banking_Init(ab);
  if (rv<0) {
    DBG_ERROR(0, "Error initializing AqBanking (%d)", rv);
  }
  else {
    rv=AB_Banking_OnlineInit(ab);
    if (rv<0) {
      DBG_ERROR(0, "Error initializing online-part of AqBanking (%d)", rv);
      app.setBanking(NULL);
    }
  }

  /* add handling of certificate cache using AqBanking shared data */
  AB_Gui_Extend(gui->getCInterface(), ab);

  mw->openConfig();
  rv=app.loadSettings();
  if (rv<0) {
    DBG_ERROR(0, "Error loading settings: %d", rv);
  }

  /* add modules */
  m=new FM_BankAccounts(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Accounts(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Partners(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Transfers(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Business(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Vehicles(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Reports(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  m=new FM_Outbox(app.getModuleManager());
  app.getModuleManager()->addModule(m);

  mw->show();

  lf=mw->getLastOpened();
  if (!lf.empty())
    mw->openDb(lf);

#ifdef OS_DARWIN
  {
    EventRecord dummyVariable;

    /* this is to stop the dock icon froun bouncing indefinately */
    WaitNextEvent(everyEvent,&dummyVariable,0,nil);
  }
#endif

  app.run();
  wantRestart=app.isRestartRequested();
  app.fini();

  rv=app.saveSettings();
  if (rv<0) {
    DBG_ERROR(0, "Error saving settings: %d", rv);
  }

  AB_Gui_Unextend(gui->getCInterface());

  rv=AB_Banking_OnlineFini(ab);
  if (rv<0) {
    DBG_ERROR(0, "Error deinitializing online-part of AqBanking (%d)", rv);
  }
  rv=AB_Banking_Fini(ab);
  if (rv<0) {
    DBG_ERROR(0, "Error deinitializing AqBanking (%d)", rv);
  }
  AB_Banking_free(ab);

  GWEN_Gui_SetGui(NULL);
  //delete gui;

  AQDB_Fini();

  if (wantRestart)
    return 77;

  return 0;
}

