/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editaccount.hpp"
#include "ff_app.hpp"
#include "ff_accounttree.hpp"
#include "ff_selectcurrency.hpp"
#include "ff_ruleview.hpp"
#include "ff_commodityedit.hpp"

#include <aqfinance/engine/book/ae_account.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg


static struct {
  const char *name;
  AE_ACCOUNT_TYPE type;
} _accountTypes[]={

{ I18S("asset"), AE_AccountType_Asset },
{ I18S("cash"), AE_AccountType_Cash },
{ I18S("bank account"), AE_AccountType_Bank },
{ I18S("open debit notes"), AE_AccountType_OpenDebitNotes },
{ I18S("receivable"), AE_AccountType_Receivable },

{ I18S("liability"), AE_AccountType_Liability },
{ I18S("credit card"), AE_AccountType_CreditCard },
{ I18S("open transfers"), AE_AccountType_OpenTransfers },
{ I18S("payable"), AE_AccountType_Payable },

{ I18S("equity"), AE_AccountType_Equity },
{ I18S("auto equity"), AE_AccountType_AutoEquity },

{ I18S("income"), AE_AccountType_Income },
{ I18S("tax income"), AE_AccountType_TaxIncome },

{ I18S("expense"), AE_AccountType_Expense },
{ I18S("tax expense"), AE_AccountType_Expense },

{ NULL, AE_AccountType_Unknown }
};



FXDEFMAP(FF_EditAccount) FF_EditAccountMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditAccount::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditAccount::onCmdCancel),
  FXMAPFUNC(SEL_INSERTED, FF_EditAccount::ID_RULEVIEW, FF_EditAccount::onInsertedRule),
  FXMAPFUNC(SEL_REPLACED, FF_EditAccount::ID_RULEVIEW, FF_EditAccount::onReplacedRule),
};


FXIMPLEMENT(FF_EditAccount, FXDialogBox, FF_EditAccountMap, ARRAYNUMBER(FF_EditAccountMap))



FF_EditAccount::FF_EditAccount(FF_App* a,
                               FXComposite *parent,
                               const FXString& title,
                               FXuint opts,
                               FXint x, FXint y, FXint w, FXint h,
                               FXint pl, FXint pr, FXint pt, FXint pb,
                               FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_nameEdit(NULL)
,m_codeEdit(NULL)
,m_descrEdit(NULL)
,m_typeCombo(NULL)
,m_placeHolderCheck(NULL)
,m_primaryCurrencyEdit(NULL)
,m_secondaryCurrencyEdit(NULL)
,m_tabBook(NULL)
,m_accountTree(NULL)
,m_ruleView(NULL)
,m_accountId(0)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  int i;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  /* name */
  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* code */
  new FXLabel(m, I18N("Code"), NULL, LABEL_NORMAL);
  m_codeEdit=new FXTextField(m, 32, this, ID_TEXTFIELD_CODE,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* description */
  new FXLabel(m, I18N("Description"), NULL, LABEL_NORMAL);
  m_descrEdit=new FXTextField(m, 64, this, ID_TEXTFIELD_DESCR,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  /* type */
  new FXLabel(m, I18N("Type"), NULL, LABEL_NORMAL);
  m_typeCombo=new FXComboBox(m, 1, this, ID_COMBO_TYPE,
                             COMBOBOX_STATIC |
                             FRAME_SUNKEN | FRAME_THICK);
  m_typeCombo->appendItem(I18N("-- select type --"));
  for (i=0; ; i++) {
    if (_accountTypes[i].type==AE_AccountType_Unknown)
      break;
    m_typeCombo->appendItem(I18N(_accountTypes[i].name));
  }

  m_typeCombo->setNumVisible(m_typeCombo->getNumItems());

  /* primary commodity */
  new FXLabel(m, I18N("Primary Currency"), NULL, LABEL_NORMAL);
  m_primaryCurrencyEdit=new FF_CommodityEdit(m, NULL, 0, LAYOUT_FILL_X);

  /* secondary commodity */
  new FXLabel(m, I18N("Secondary Currency"), NULL, LABEL_NORMAL);
  m_secondaryCurrencyEdit=new FF_CommodityEdit(m, NULL, 0, LAYOUT_FILL_X);

  /* placeholder */
  m_placeHolderCheck=new FXCheckButton(vf, I18N("Account is a placeholder"),
				       NULL, 0, CHECKBUTTON_NORMAL | LAYOUT_FILL_X | JUSTIFY_LEFT);

  m_tabBook=new FXTabBook(vf, this, ID_TABBOOK,
                          LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new FXTabItem(m_tabBook, I18N("Parent Account"), NULL);

  /* parent account */
  m_accountTree=new FF_AccountTree(m_tabBook, this, ID_ACCOUNTTREE,
                                   LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                   FOLDINGLIST_SINGLESELECT |
                                   FOLDINGLIST_SHOWS_LINES |
                                   FOLDINGLIST_SHOWS_BOXES |
                                   FOLDINGLIST_ROOT_BOXES);

  /* matching rules */
  new FXTabItem(m_tabBook, I18N("Matching Rules"), NULL);
  m_ruleView=new FF_RuleView(m_tabBook, this, ID_RULEVIEW, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /* last line (OK / Cancel) */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}


FF_EditAccount::FF_EditAccount()
:FXDialogBox()
,m_app(NULL)
,m_nameEdit(NULL)
,m_codeEdit(NULL)
,m_descrEdit(NULL)
,m_typeCombo(NULL)
,m_placeHolderCheck(NULL)
,m_primaryCurrencyEdit(NULL)
,m_secondaryCurrencyEdit(NULL)
,m_tabBook(NULL)
,m_accountTree(NULL)
,m_ruleView(NULL)
,m_accountId(0)
{
}



FF_EditAccount::~FF_EditAccount() {
}



void FF_EditAccount::create() {
  FXDialogBox::create();
}



void FF_EditAccount::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_EditAccount");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_accountTree->getHeaderSize(i));
    m_accountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_accountTree->setHeaderArrowDir(i, TRUE);
      else
        m_accountTree->setHeaderArrowDir(i, FALSE);
    }
  }

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "ruleView");
  m_ruleView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_EditAccount::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_accountTree->getHeaderSize(i));
    b=m_accountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "ruleView");
  m_ruleView->fini(dbT);

  rv=m_app->setDialogConfig("FF_EditAccount", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_EditAccount::checkGui(FXbool quiet) {
  FXint ci=m_typeCombo->getCurrentItem();
  int i=0;
  FXString str;
  AE_ACCOUNT_TYPE t;
  AE_ACCOUNT *pacc;
  AQDB_ID id1, id2;
  bool isPlaceHolder;

  isPlaceHolder=m_placeHolderCheck->getCheck();

  for (i=0; ; i++) {
    if (_accountTypes[i].type==AE_AccountType_Unknown) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Please select the account type"));
	m_typeCombo->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    if ((i+1)==ci)
      break;
  }

  /* type */
  t=_accountTypes[i].type;

  /* parent account */
  pacc=m_accountTree->getFirstSelectedAccount();
  if (pacc) {
    AE_ACCOUNT_GROUP gchild;
    AE_ACCOUNT_GROUP gparent;

    gchild=AE_AccountType_GetGroup(t);
    gparent=AE_AccountType_GetGroup(AE_Account_GetType(pacc));

    if (!((AE_Account_GetFlags(pacc) & AE_ACCOUNT_FLAGS_PLACEHOLDER) ||
	  isPlaceHolder ||
	  gchild==gparent)) {
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Account type must be the same as in parent account or placeholder"));
	m_typeCombo->setFocus();
	return GWEN_ERROR_BAD_DATA;
      }
    }
  }

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
                          I18N("Missing account name"));
      m_nameEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  id1=m_primaryCurrencyEdit->getCommodityId();
  id2=m_secondaryCurrencyEdit->getCommodityId();
  if (id1==0 && id2==0) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Please select the primary currency"));
      m_primaryCurrencyEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }


  return 0;
}



void FF_EditAccount::loadRules(AE_RULE_LIST *rules) {
  char qbuf[256];

  snprintf(qbuf, sizeof(qbuf)-1, "($ownertype==%d)&&($ownerid==%d)",
           AE_Book_TableType_Account,
           m_accountId);
  qbuf[sizeof(qbuf)-1]=0;
  AE_Book_QueryRuleList(m_app->getBook(), AE_Book_TableType_Rule, 0, qbuf, rules);
}



void FF_EditAccount::reload() {
  AE_RULE_LIST *rules;

  rules=AE_Rule_List_new();
  loadRules(rules);
  m_ruleView->toGui(rules);
  AE_Rule_List_free(rules);
}



FXint FF_EditAccount::toGui(const AE_ACCOUNT *acc) {
  AE_BOOK *b;

  m_accountId=AE_Account_GetId(acc);

  m_primaryCurrencyEdit->reload();
  m_secondaryCurrencyEdit->reload();

  b=m_app->getBook();
  if (b) {
    const char *s;
    AQDB_ID id;
    AE_ACCOUNT_TREE *atree;
    int rv;
    int i;
    AE_ACCOUNT_TYPE t;

    s=AE_Account_GetName(acc);
    if (s && *s)
      m_nameEdit->setText(s);

    s=AE_Account_GetCode(acc);
    if (s && *s)
      m_codeEdit->setText(s);

    s=AE_Account_GetDescription(acc);
    if (s && *s)
      m_descrEdit->setText(s);

    t=AE_Account_GetType(acc);
    for (i=0; ; i++) {
      if (_accountTypes[i].type==AE_AccountType_Unknown) {
	m_typeCombo->setCurrentItem(0);
	break;
      }

      if (_accountTypes[i].type==t) {
	m_typeCombo->setCurrentItem(i+1);
	break;
      }
    }

    /* primary commodity */
    id=AE_Account_GetPrimaryCommodityId(acc);
    m_primaryCurrencyEdit->setCommodityId(id);

    /* secondary commodity */
    id=AE_Account_GetSecondaryCommodityId(acc);
    m_secondaryCurrencyEdit->setCommodityId(id);

    /* parent account */
    atree=AE_Account_Tree_new();
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, atree);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      AQDB_ID parentId;

      m_accountTree->addAccounts(atree, AE_Account_GetId(acc));
      m_accountTree->expandItems();
      parentId=AE_Account_GetParentId(acc);
      if (parentId) {
        m_accountTree->selectAccount(parentId, true);
        m_accountTree->makeAccountVisible(parentId);
      }
    }
    AE_Account_Tree_free(atree);

    m_placeHolderCheck->setCheck((AE_Account_GetFlags(acc) & AE_ACCOUNT_FLAGS_PLACEHOLDER)?TRUE:FALSE);

    /* read rules */
    reload();
  }

  return 0;
}



FXint FF_EditAccount::fromGui(AE_ACCOUNT *acc) {
  FXint ci=m_typeCombo->getCurrentItem();
  int i;
  FXString str;
  AE_ACCOUNT_TYPE t;
  AE_ACCOUNT *pacc;
  AE_RULE_LIST *rules;
  AE_RULE_LIST *dbrules;
  AE_RULE *r;
  AQDB_ID id1;
  AQDB_ID id2;
  bool isPlaceHolder;

  isPlaceHolder=m_placeHolderCheck->getCheck();

  /* type */
  for (i=0; ; i++) {
    if (_accountTypes[i].type==AE_AccountType_Unknown)
      return GWEN_ERROR_BAD_DATA;
    if ((i+1)==ci)
      break;
  }
  t=_accountTypes[i].type;
  AE_Account_SetType(acc, t);

  /* name */
  str=m_nameEdit->getText().simplify();
  if (str.empty())
    return GWEN_ERROR_NO_DATA;
  AE_Account_SetName(acc, str.text());

  /* code */
  str=m_codeEdit->getText().simplify();
  if (str.empty())
    AE_Account_SetCode(acc, NULL);
  else
    AE_Account_SetCode(acc, str.text());

  /* description */
  str=m_descrEdit->getText().simplify();
  if (str.empty())
    AE_Account_SetDescription(acc, NULL);
  else
    AE_Account_SetDescription(acc, str.text());

  /* commodity ids */
  id1=m_primaryCurrencyEdit->getCommodityId();
  id2=m_secondaryCurrencyEdit->getCommodityId();
  if (id1==0 && id2==0)
    return GWEN_ERROR_NO_DATA;
  else if (id1 && id2==0) {
    AE_Account_SetPrimaryCommodityId(acc, id1);
    AE_Account_SetSecondaryCommodityId(acc, 0);
  }
  else if (id1==0 && id2) {
    AE_Account_SetPrimaryCommodityId(acc, id2);
    AE_Account_SetSecondaryCommodityId(acc, 0);
  }
  else if (id1 && id2) {
    AE_Account_SetPrimaryCommodityId(acc, id1);
    AE_Account_SetSecondaryCommodityId(acc, id2);
  }

  /* parent account */
  pacc=m_accountTree->getFirstSelectedAccount();
  if (pacc) {
    DBG_ERROR(0, "Got parent id %08x", AE_Account_GetId(pacc));
    AE_Account_SetParentId(acc, AE_Account_GetId(pacc));
  }
  else
    AE_Account_SetParentId(acc, 0);

  /* get rules */
  rules=AE_Account_GetRules(acc);
  if (rules)
    AE_Rule_List_Clear(rules);
  else {
    rules=AE_Rule_List_new();
    AE_Account_SetRules(acc, rules);
  }
  m_ruleView->fromGui(rules);

  /* determine the rules to be deleted */
  dbrules=AE_Rule_List_new();
  loadRules(dbrules);
  r=AE_Rule_List_First(dbrules);
  while(r) {
    AE_RULE *rNext=AE_Rule_List_Next(r);

    AE_Rule_List_Del(r);
    if (AE_Rule_List_GetById(rules, AE_Rule_GetId(r))==NULL) {
      /* rule missing in new list, mark as deleted and add to new list */
      AE_Rule_AddRuntimeFlags(r, AE_RULE_RTFLAGS_DELETED);
      AE_Rule_List_Add(r, rules);
    }
    else
      AE_Rule_free(r);
    r=rNext;
  }

  if (isPlaceHolder)
    AE_Account_AddFlags(acc, AE_ACCOUNT_FLAGS_PLACEHOLDER);
  else
    AE_Account_SubFlags(acc, AE_ACCOUNT_FLAGS_PLACEHOLDER);

  return 0;
}



long FF_EditAccount::onInsertedRule(FXObject*, FXSelector, void *ptr) {
  AE_RULE *r;

  r=(AE_RULE*) ptr;
  if (r) {
    DBG_ERROR(0, "Rule inserted");
  }

  return 1;
}



long FF_EditAccount::onReplacedRule(FXObject*, FXSelector, void *ptr) {
  AE_RULE *r;

  r=(AE_RULE*) ptr;
  if (r) {
    DBG_ERROR(0, "Rule replaced");
  }

  return 1;
}



long FF_EditAccount::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditAccount::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  /*hide();*/
  return 1;
}



FXint FF_EditAccount::getDefaultHeight() {
  /* add minimum height of table to be usefull */
  return FXDialogBox::getDefaultHeight()+100;
}



bool FF_EditAccount::editAccount(FF_App* app,
                                 FXComposite *parent,
                                 const FXString &title,
                                 AE_ACCOUNT *acc) {
  FF_EditAccount dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();

  res=dlg.toGui(acc);
  if (res)
    return false;
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());
  dlg.init();
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(acc);
  if (res==0) {
    return true;
  }

  return false;
}






