/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editbankaccgroup.hpp"
#include "ff_app.hpp"
#include "ff_grouptree.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditBankAccountGroup) FF_EditBankAccountGroupMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditBankAccountGroup::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditBankAccountGroup::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditBankAccountGroup::ID_TEXTFIELD, FF_EditBankAccountGroup::onCmdTextField),
};


FXIMPLEMENT(FF_EditBankAccountGroup, FXDialogBox, FF_EditBankAccountGroupMap, ARRAYNUMBER(FF_EditBankAccountGroupMap))




FF_EditBankAccountGroup::FF_EditBankAccountGroup(FF_App* a,
                                                 FXComposite *parent,
                                                 const FXString& title,
                                                 FXuint opts,
                                                 FXint x, FXint y, FXint w, FXint h,
                                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                                 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_moduleCombo(NULL)
,m_nameEdit(NULL)
,m_groupTree(NULL){
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXIcon *ic;

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_nameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Type"), NULL, LABEL_NORMAL);
  m_moduleCombo=new FXComboBox(m, 1, this, ID_COMBOBOX, COMBOBOX_STATIC);
  m_moduleCombo->appendItem(I18N("BankAccountGroupType|default"));
  m_moduleCombo->appendItem(I18N("BankAccountGroupType|new"));
  m_moduleCombo->appendItem(I18N("BankAccountGroupType|unused"));

  gb=new FXGroupBox(vf, I18N("Parent Group"),
		    LAYOUT_FILL_X | LAYOUT_FILL_Y |
		    FRAME_LINE);
  m_groupTree=new FF_GroupTree(gb, this, ID_GROUP,
			       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                               FOLDINGLIST_SINGLESELECT |
			       FOLDINGLIST_SHOWS_LINES |
			       FOLDINGLIST_SHOWS_BOXES |
			       FOLDINGLIST_ROOT_BOXES);
  m_groupTree->appendHeader(I18N("Group Name"));
  m_groupTree->setHeaderSize(0, m_nameEdit->getDefaultWidth());

  ic=a->getIcon("minifolderopen");
  if (ic)
    m_groupTree->setIconOpen(ic);
  ic=a->getIcon("minifolder");
  if (ic)
    m_groupTree->setIconClosed(ic);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditBankAccountGroup::FF_EditBankAccountGroup()
:FXDialogBox()
,m_app(NULL)
,m_moduleCombo(NULL)
,m_nameEdit(NULL)
,m_groupTree(NULL){

}



FF_EditBankAccountGroup::~FF_EditBankAccountGroup() {
}



void FF_EditBankAccountGroup::create() {
  FXDialogBox::create();
  m_moduleCombo->create();
  m_nameEdit->create();
  m_groupTree->create();
}



FXint FF_EditBankAccountGroup::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



FXint FF_EditBankAccountGroup::checkGui(FXbool quiet) {
  FXint i=m_moduleCombo->getCurrentItem();
  FXString name;

  switch(i) {
  case 0:
  case 1:
  case 2:
    break;
  default:
    DBG_ERROR(0, "Unknown module selected");
    return GWEN_ERROR_INVALID;
  }

  name=m_nameEdit->getText().simplify();
  if (name.empty()) {
    DBG_ERROR(0, "Empty name");
    return GWEN_ERROR_INVALID;
  }

  return 0;
}



FXint FF_EditBankAccountGroup::toGui(const AE_GROUP *gr) {
  const char *s;
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_GROUP_TREE *gt;
    int rv;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_BankAccountGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else
      m_groupTree->addGroups(gt, AE_Group_GetId(gr));
    AE_Group_Tree_free(gt);

    m_groupTree->sortItems();
    m_groupTree->expandItems();
  }

  if (AE_Group_GetParentId(gr)) {
    FF_GroupTreeItem *item=NULL;

    item=m_groupTree->findGroupItem(AE_Group_GetParentId(gr));
    if (item)
      m_groupTree->setCurrentItem(item);
  }

  switch(AE_Group_GetType(gr)) {
  case AE_BankAccountGroup_New:
      m_moduleCombo->setCurrentItem(1);
      break;
  case AE_BankAccountGroup_Unused:
      m_moduleCombo->setCurrentItem(2);
      break;
  case AE_BankAccountGroup_None:
  default:
    m_moduleCombo->setCurrentItem(0);
    break;
  }

  s=AE_Group_GetName(gr);
  if (s)
    m_nameEdit->setText(FXString(s));

  return 0;
}



FXint FF_EditBankAccountGroup::fromGui(AE_GROUP *gr) {
  FXint i=m_moduleCombo->getCurrentItem();
  FXString name;
  AE_GROUP *parent;

  switch(i) {
  case 0:
    AE_Group_SetType(gr, AE_BankAccountGroup_None);
    break;
  case 1:
    AE_Group_SetType(gr, AE_BankAccountGroup_New);
    break;
  case 2:
    AE_Group_SetType(gr, AE_BankAccountGroup_Unused);
    break;
  default:
    DBG_ERROR(0, "Unknown module selected");
    return GWEN_ERROR_INVALID;
  }

  name=m_nameEdit->getText().simplify();
  if (name.empty()) {
    DBG_ERROR(0, "Empty name");
    return GWEN_ERROR_INVALID;
  }
  AE_Group_SetName(gr, name.text());

  parent=m_groupTree->getFirstSelectedGroup();
  if (parent)
    AE_Group_SetParentId(gr, AE_Group_GetId(parent));
  else
    AE_Group_SetParentId(gr, 0);

  return 0;
}



long FF_EditBankAccountGroup::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_EditBankAccountGroup::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_EditBankAccountGroup::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}




bool FF_EditBankAccountGroup::editGroup(FF_App* app,
                                        FXComposite *parent,
                                        const FXString &title,
                                        AE_GROUP *gr) {
  FF_EditBankAccountGroup dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  res=dlg.toGui(gr);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return false;
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(gr);
      if (res==0) {
        return true;
        break;
      }
    }
  }
  /* should never reach this point */
  return false;
}





