/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editcurrency.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_datepicker.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditCurrency) FF_EditCurrencyMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditCurrency::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditCurrency::onCmdCancel),

};


FXIMPLEMENT(FF_EditCurrency, FXDialogBox, FF_EditCurrencyMap, ARRAYNUMBER(FF_EditCurrencyMap))



FF_EditCurrency::FF_EditCurrency(FF_App* a,
                                 FXComposite *parent,
                                 const FXString& title,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h,
                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts,
             x, y, w, h,
             pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_codeEdit(NULL)
,m_nameEdit(NULL)
,m_symbolEdit(NULL)
,m_descrEdit(NULL)
,m_fractionEdit(NULL) {
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;

  /* description */
  m_label=new FOX16_HtmlLabel(vf, I18N("Edit the currency."),
			      LAYOUT_FILL_X);

  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL);
  m_nameEdit=new FXTextField(m, 32, this, ID_EDIT_NAME,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Code"), NULL, LABEL_NORMAL);
  m_codeEdit=new FXTextField(m, 4, this, ID_EDIT_CODE,
                             TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Symbol"), NULL, LABEL_NORMAL);
  m_symbolEdit=new FXTextField(m, 4, this, ID_EDIT_SYMBOL,
                               TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Description"), NULL, LABEL_NORMAL);
  m_descrEdit=new FXTextField(m, 32, this, ID_EDIT_DESCR,
                              TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Smallest Unit"), NULL, LABEL_NORMAL);
  m_fractionEdit=new FXTextField(m, 12, this, ID_EDIT_FRACTION,
                                 TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | JUSTIFY_RIGHT);

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);

}



FF_EditCurrency::FF_EditCurrency()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_codeEdit(NULL)
,m_nameEdit(NULL)
,m_symbolEdit(NULL)
,m_descrEdit(NULL)
,m_fractionEdit(NULL) {

}




FF_EditCurrency::~FF_EditCurrency() {
}






int FF_EditCurrency::checkGui(FXbool quiet) {
  FXString str;

  str=m_nameEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
			  I18N("Missing currency name"));
      m_nameEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  str=m_codeEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
                          I18N("Missing currency code"));
      m_codeEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }

  str=m_fractionEdit->getText().simplify();
  if (str.empty()) {
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Input Error"),
			  "%s",
                          I18N("Missing amount"));
      m_fractionEdit->setFocus();
    }
    return GWEN_ERROR_NO_DATA;
  }
  else {
    if (str.contains(',') && str.contains('.')) {
      DBG_ERROR(0, "Both types of comma detected.");
      if (!quiet) {
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Input Error"),
			    "%s",
                            I18N("Two types of komma detected, "
                                 "please specify the decimal point only"));
        m_fractionEdit->setFocus();
      }
      return GWEN_ERROR_BAD_DATA;
    }
    else {
      AQDB_VALUE *v;
      int i;

      /* replace textual comma with decimal point */
      i=str.find(',');
      if (i>=0)
        str[i]='.';

      v=AQDB_Value_fromString(str.text());
      if (v==NULL) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad input for smallest unit"));
          m_fractionEdit->setFocus();
        }
        return GWEN_ERROR_BAD_DATA;
      }
      if (AQDB_Value_IsZero(v) ||
          AQDB_Value_IsNegative(v)) {
        if (!quiet) {
          FXMessageBox::error(this, MBOX_OK,
                              I18N("Input Error"),
			      "%s",
                              I18N("Bad input for smallest unit: Must be greater than zero."));
          m_fractionEdit->setFocus();
        }

        AQDB_Value_free(v);
        return GWEN_ERROR_BAD_DATA;
      }

      AQDB_Value_free(v);
    }
  }

  return 0;
}



int FF_EditCurrency::toGui(const AE_COMMODITY *co) {
  const char *s;
  const AQDB_VALUE *v;

  s=AE_Commodity_GetName(co);
  if (s)
    m_nameEdit->setText(s);

  s=AE_Commodity_GetCode(co);
  if (s)
    m_codeEdit->setText(s);

  s=AE_Commodity_GetSymbol(co);
  if (s)
    m_symbolEdit->setText(s);

  s=AE_Commodity_GetDescription(co);
  if (s)
    m_descrEdit->setText(s);

  v=AE_Commodity_GetSmallestUnit(co);
  if (v) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AQDB_Value_toHumanReadableString(v, tbuf, 2);
    m_fractionEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
    GWEN_Buffer_free(tbuf);
  }
  else
    m_fractionEdit->setText("0.01");

  return 0;
}



int FF_EditCurrency::fromGui(AE_COMMODITY *co) {
  FXString str;

  str=m_nameEdit->getText().simplify();
  if (str.empty())
    return GWEN_ERROR_NO_DATA;
  AE_Commodity_SetName(co, str.text());

  str=m_codeEdit->getText().simplify();
  if (str.empty())
    return GWEN_ERROR_NO_DATA;
  AE_Commodity_SetCode(co, str.text());

  str=m_symbolEdit->getText().simplify();
  if (str.empty())
    AE_Commodity_SetSymbol(co, NULL);
  else
    AE_Commodity_SetSymbol(co, str.text());

  str=m_descrEdit->getText().simplify();
  if (str.empty())
    AE_Commodity_SetDescription(co, NULL);
  else
    AE_Commodity_SetDescription(co, str.text());

  str=m_fractionEdit->getText().simplify();
  if (str.empty())
    return GWEN_ERROR_NO_DATA;
  if (str.contains(',') && str.contains('.')) {
    DBG_ERROR(0, "Both types of comma detected.");
    return GWEN_ERROR_BAD_DATA;
  }
  else {
    AQDB_VALUE *v;
    int i;

    /* replace textual comma with decimal point */
    i=str.find(',');
    if (i>=0)
      str[i]='.';

    v=AQDB_Value_fromString(str.text());
    if (v==NULL)
      return GWEN_ERROR_BAD_DATA;
    if (AQDB_Value_IsZero(v) ||
        AQDB_Value_IsNegative(v)) {
      AQDB_Value_free(v);
      return GWEN_ERROR_BAD_DATA;
    }
    AE_Commodity_SetSmallestUnit(co, v);
    AQDB_Value_free(v);
  }

  return 0;
}



void FF_EditCurrency::create() {
  FXDialogBox::create();
}



long FF_EditCurrency::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditCurrency::onCmdFraction(FXObject*, FXSelector, void *ptr) {
  return 1;
}



long FF_EditCurrency::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditCurrency::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditCurrency::editCurrency(FF_App* app,
                                   FXComposite *parent,
                                   const FXString &title,
                                   AE_COMMODITY *co) {
  FF_EditCurrency dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  res=dlg.toGui(co);
  if (res)
    return false;
  dlg.place(PLACEMENT_VISIBLE);
  dlg.show();
  dlg.layout();
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return false;
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(co);
      if (res==0) {
        return true;
        break;
      }
    }
  }

  /* should never reach this point */
  return false;

}









