/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editgroup.hpp"
#include "ff_app.hpp"
#include "ff_grouptree.hpp"

#include <aqfinance/engine/book/ae_group.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_EditGroup) FF_EditGroupMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditGroup::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditGroup::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditGroup::ID_TEXTFIELD, FF_EditGroup::onCmdTextField),
};


FXIMPLEMENT(FF_EditGroup, FXDialogBox, FF_EditGroupMap, ARRAYNUMBER(FF_EditGroupMap))




FF_EditGroup::FF_EditGroup(FF_App* a,
			   FXComposite *parent,
			   const FXString& title,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr, FXint pt, FXint pb,
			   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_nameEdit(NULL)
,m_groupTree(NULL){
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  FXHorizontalFrame *hf;
  FXGroupBox *gb;
  FXIcon *ic;

  new FXLabel(m, I18N("Name"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_nameEdit=new FXTextField(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  gb=new FXGroupBox(vf, I18N("Parent Group"),
		    LAYOUT_FILL_X | LAYOUT_FILL_Y |
		    FRAME_LINE);
  m_groupTree=new FF_GroupTree(gb, this, ID_GROUP,
			       LAYOUT_FILL_X | LAYOUT_FILL_Y |
			       FOLDINGLIST_SINGLESELECT |
			       FOLDINGLIST_SHOWS_LINES |
			       FOLDINGLIST_SHOWS_BOXES |
			       FOLDINGLIST_ROOT_BOXES);
  m_groupTree->appendHeader(I18N("Group Name"));
  m_groupTree->setHeaderSize(0, m_nameEdit->getDefaultWidth());

  ic=a->getIcon("minifolderopen");
  if (ic)
    m_groupTree->setIconOpen(ic);
  ic=a->getIcon("minifolder");
  if (ic)
    m_groupTree->setIconClosed(ic);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditGroup::FF_EditGroup()
:FXDialogBox()
,m_app(NULL)
,m_nameEdit(NULL)
,m_groupTree(NULL){

}



FF_EditGroup::~FF_EditGroup() {
}



void FF_EditGroup::create() {
  FXDialogBox::create();
  m_nameEdit->create();
  m_groupTree->create();
}



FXint FF_EditGroup::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



void FF_EditGroup::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_EditGroup");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_groupTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_groupTree->getHeaderSize(i));
    m_groupTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_groupTree->setHeaderArrowDir(i, TRUE);
      else
	m_groupTree->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_EditGroup::fini() {
  GWEN_DB_NODE *db;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_groupTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_groupTree->getHeaderSize(i));
    b=m_groupTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  GWEN_DB_Group_free(db);
}



FXint FF_EditGroup::checkGui(FXbool quiet) {
  FXString name;

  name=m_nameEdit->getText().simplify();
  if (name.empty()) {
    DBG_ERROR(0, "Empty name");
    if (!quiet) {
      m_nameEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }

  return 0;
}



FXint FF_EditGroup::toGui(const AE_GROUP *gr, AE_BOOK_TABLE_TYPE tt) {
  const char *s;
  AE_BOOK *b;

  b=m_app->getBook();
  if (b) {
    AE_GROUP_TREE *gt;
    int rv;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, tt, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else
      m_groupTree->addGroups(gt, AE_Group_GetId(gr));
    AE_Group_Tree_free(gt);

    m_groupTree->sortItems();
    m_groupTree->expandItems();
  }

  if (AE_Group_GetParentId(gr)) {
    FF_GroupTreeItem *item=NULL;

    item=m_groupTree->findGroupItem(AE_Group_GetParentId(gr));
    if (item) {
      m_groupTree->setCurrentItem(item);
      item->setSelected(true);
    }
  }

  s=AE_Group_GetName(gr);
  if (s)
    m_nameEdit->setText(FXString(s));

  return 0;
}



FXint FF_EditGroup::fromGui(AE_GROUP *gr) {
  FXString name;
  AE_GROUP *parent;

  name=m_nameEdit->getText().simplify();
  if (name.empty()) {
    DBG_ERROR(0, "Empty name");
    return GWEN_ERROR_INVALID;
  }
  AE_Group_SetName(gr, name.text());

  parent=m_groupTree->getFirstSelectedGroup();
  if (parent)
    AE_Group_SetParentId(gr, AE_Group_GetId(parent));
  else
    AE_Group_SetParentId(gr, 0);

  return 0;
}



long FF_EditGroup::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_EditGroup::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditGroup::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}




bool FF_EditGroup::editGroup(FF_App* app,
			     FXComposite *parent,
			     const FXString &title,
                             AE_BOOK_TABLE_TYPE tt,
			     AE_GROUP *gr) {
  FF_EditGroup dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.place(PLACEMENT_VISIBLE);
  dlg.resize(dlg.getDefaultWidth(), dlg.getDefaultHeight());

  dlg.init();
  res=dlg.toGui(gr, tt);
  if (res)
    return false;

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(gr);
  if (res==0) {
    return true;
  }

  return false;
}





