/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_editpayment.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_valueedit.hpp"

#include <aqfinance/engine/book/ae_group.h>
#include <aqfinance/engine/book/ae_utils.h>


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_EditPayment) FF_EditPaymentMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditPayment::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditPayment::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditPayment::ID_TEXTFIELD, FF_EditPayment::onCmdTextField),
};


FXIMPLEMENT(FF_EditPayment, FXDialogBox, FF_EditPaymentMap, ARRAYNUMBER(FF_EditPaymentMap))




FF_EditPayment::FF_EditPayment(bool isBuy,
			       FF_App* a,
			       FXComposite *parent,
			       const FXString& title,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h,
			       FXint pl, FXint pr, FXint pt, FXint pb,
			       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_commodities(NULL)
,m_isBuy(isBuy)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;

  /* begin: first matrix */
  m=new FXMatrix(vf, 3, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Enter Date"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_enterDateEdit=new FF_DateEdit(m, this, ID_DATEEDIT, I18N("YYYY/MM/DD"));
  new FXSpring(m, LAYOUT_FILL_X);

  new FXLabel(m, I18N("Amount Paid"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_valueEdit=new FF_ValueEdit(m, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL);
  new FXSpring(m, LAYOUT_FILL_X);

  /* end: first matrix */


  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditPayment::FF_EditPayment()
:FXDialogBox()
,m_app(NULL)
,m_commodities(NULL)
{

}



FF_EditPayment::~FF_EditPayment() {
  AE_Commodity_List_free(m_commodities);
  m_commodities=NULL;
}



FXint FF_EditPayment::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



FXint FF_EditPayment::checkGui(FXbool quiet) {
  FXString s;
  GWEN_DATE *d;

  if (!m_enterDateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Invalid Entry Date"));
      m_enterDateEdit->setFocus();
    }
    return GWEN_ERROR_INVALID;
  }
  GWEN_Date_free(d);

  s=m_valueEdit->getText();
  if (s.empty())
    return GWEN_ERROR_INVALID;
  else {
    AQDB_VALUE *v=NULL;
    AE_COMMODITY *co=NULL;
    int rv;

    rv=AE_Utils_AmountFromString(s.text(), m_commodities, &v, &co);
    if (rv<0) {
      DBG_ERROR(0, "Bad amount");
      if (!quiet) {
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Input Error"),
			    "%s",
			    I18N("Invalid amount."));
	m_valueEdit->setFocus();
      }
      return GWEN_ERROR_INVALID;
    }
    else {
      AQDB_Value_free(v);
      if (co==NULL) {
	DBG_ERROR(0, "Bad amount");
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("Missing currency in amount."));
	  m_valueEdit->setFocus();
	}
	return GWEN_ERROR_INVALID;
      }
    }
  }


  return 0;
}



FXint FF_EditPayment::toGui(const AE_JOB *j) {
  AE_BOOK *b;
  const GWEN_DATE *d;
  const AE_ITEM_LIST *il;

  b=m_app->getBook();
  reload();

  d=AE_Job_GetEnterDate(j);
  if (j)
    m_enterDateEdit->setDate(d);

  il=AE_Job_GetItems(j);
  if (il) {
    AE_ITEM *item;

    item=AE_Item_List_First(il);
    if (item) {
      const AQDB_VALUE *v;
      AQDB_ID cid;

      if (m_isBuy)
	v=AE_Item_GetBuyPriceValue(item);
      else
	v=AE_Item_GetSellPriceValue(item);
      if (v)
	m_valueEdit->setValue(v);
      cid=AE_Item_GetCommodityId(item);
      if (v) {
	int rv;
	GWEN_BUFFER *tbuf;
  
	tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	rv=AE_Utils_AmountToString(v, cid,
				   m_commodities,
				   tbuf);
	if (rv<0) {
	}
	else {
	  m_valueEdit->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	  m_valueEdit->selectAll();
	}
	GWEN_Buffer_free(tbuf);
      }
    }
  }

  return 0;
}



FXint FF_EditPayment::fromGui(AE_JOB *j) {
  FXString txt;
  GWEN_DATE *d;
  int rv;

  if (!m_enterDateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    return GWEN_ERROR_INVALID;
  }
  AE_Job_SetEnterDate(j, d);
  GWEN_Date_free(d);

  txt=m_valueEdit->getText();
  if (txt.empty())
    return GWEN_ERROR_INVALID;
  else {
    AQDB_VALUE *v=NULL;
    AE_COMMODITY *co=NULL;

    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
    if (rv<0) {
      DBG_ERROR(0, "Bad amount");
      return GWEN_ERROR_INVALID;
    }
    else {
      AE_ITEM_LIST *items;
      AE_ITEM *t;

      items=AE_Item_List_new();
      t=AE_Item_new();
      AE_Item_SetJobId(t, AE_Job_GetId(j));
      if (m_isBuy)
	AE_Item_SetBuyPriceValue(t, v);
      else
	AE_Item_SetSellPriceValue(t, v);
      if (co)
	AE_Item_SetCommodityId(t, AE_Commodity_GetId(co));
      AE_Item_List_Add(t, items);
      AE_Job_SetItems(j, items);
    }
    AQDB_Value_free(v);
  }

  return 0;
}



void FF_EditPayment::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditPayment");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight());

  GWEN_DB_Group_free(db);
}



void FF_EditPayment::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditPayment", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



long FF_EditPayment::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_EditPayment::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditPayment::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



void FF_EditPayment::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  AE_Commodity_List_free(m_commodities);
  m_commodities=AE_Commodity_List_new();

  if (b) {
    int rv;

    /* read commodity list */
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, m_commodities);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }
}



bool FF_EditPayment::editPayment(FF_App* app,
				 FXComposite *parent,
				 const FXString &title,
				 AE_JOB *j,
				 bool isBuy) {
  FF_EditPayment dlg(isBuy, app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  res=dlg.toGui(j);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());
  dlg.init();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(j);
  if (res==0) {
    return true;
  }

  return false;
}





