/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_editrule.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_rule.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg


FXDEFMAP(FF_EditRule) FF_EditRuleMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditRule::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditRule::onCmdCancel),
};


FXIMPLEMENT(FF_EditRule, FXDialogBox,
            FF_EditRuleMap, ARRAYNUMBER(FF_EditRuleMap))



static FF_EditRule::FieldDescription _operators[]={
  {"==", I18S("FF_EditRule|equals")},
  {"!=", I18S("FF_EditRule|is not equal")},
  {">", I18S("FF_EditRule|is greater than")},
  {">=", I18S("FF_EditRule|is greater than or equal")},
  {"<", I18S("FF_EditRule|is lesser than")},
  {"<=", I18S("FF_EditRule|is lesser than or equal")},
  {"?contains", I18S("FF_EditRule|contains")},
  {"?matches", I18S("FF_EditRule|matches")},
  {NULL, NULL}
};



FF_EditRule::FF_EditRule(FF_App* a,
                         const FieldDescription *fd,
                         FXComposite *parent,
                         const FXString& title,
                         FXuint opts,
                         FXint x, FXint y, FXint w, FXint h,
                         FXint pl, FXint pr, FXint pt, FXint pb,
                         FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_fieldDescriptions(fd)
,m_okButton(NULL)
,m_abortButton(NULL)
,m_verticalFrame(NULL)
,m_checkCombineAnd(NULL) {
  FXVerticalFrame *vf;
  FXMatrix *m;
  int i;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_checkCombineAnd=new FXCheckButton(vf, I18N("All rules must match"), this, ID_CHECK_COMBINEMODE);

  m=new FXMatrix(vf, 4, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  for (i=0; i<MAX_SUBRULES; i++) {
    int j;

    new FXLabel(m, I18N("Rule"), NULL, LABEL_NORMAL);

    /* field */
    m_comboField[i]=new THEMECOMBOBOX(m, 1, this, ID_COMBO_FIELD,
				      COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK);
    m_comboField[i]->appendItem(I18N("(select)"));
    for (j=0; m_fieldDescriptions[j].name!=NULL; j++)
      m_comboField[i]->appendItem(I18N(m_fieldDescriptions[j].text));
    if (j>10)
      j=10;
    m_comboField[i]->setNumVisible(j);

    /* operator */
    m_comboOperator[i]=new THEMECOMBOBOX(m, 1, this, ID_COMBO_OPERATOR,
					 COMBOBOX_STATIC | FRAME_SUNKEN | FRAME_THICK);
    m_comboOperator[i]->appendItem(I18N("(select)"));
    for (j=0; _operators[j].name!=NULL; j++)
      m_comboOperator[i]->appendItem(I18N(_operators[j].text));
    if (j>10)
      j=10;
    m_comboOperator[i]->setNumVisible(j);

    /* operand */
    m_editOperand[i]=new FXTextField(m, 32, this, ID_TEXTFIELD_OPERAND,
                                     TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  }

  m_verticalFrame=vf;
}



FF_EditRule::FF_EditRule()
:FXDialogBox()
,m_app(NULL)
,m_okButton(NULL)
,m_abortButton(NULL)
,m_verticalFrame(NULL)
,m_checkCombineAnd(NULL) {
  int i;

  for (i=0; i<MAX_SUBRULES; i++) {
    m_comboField[i]=NULL;
    m_comboOperator[i]=NULL;
    m_editOperand[i]=NULL;
  }
}



FF_EditRule::~FF_EditRule() {
}



void FF_EditRule::addDialogLine() {
  FXHorizontalFrame *hf;

  new FXHorizontalSeparator(m_verticalFrame, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(m_verticalFrame, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  m_abortButton=new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
				BUTTON_NORMAL | LAYOUT_RIGHT);
  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
			     BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



void FF_EditRule::create() {
  FXDialogBox::create();
}



void FF_EditRule::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_EditRule");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  GWEN_DB_Group_free(db);
}



void FF_EditRule::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_EditRule", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_EditRule::checkGui(FXbool quiet) {
  return 0;
}



FXint FF_EditRule::toGui(const AE_RULE *r) {
  const char *expr;

  m_checkCombineAnd->setCheck((AE_Rule_GetCombineMode(r)==AE_RuleCombineMode_And)?TRUE:FALSE);

  expr=AE_Rule_GetExpression(r);
  if (expr) {
    char *cpy;
    char *s;
    int i=0;

    cpy=strdup(expr);
    s=cpy;
    while(*s && i<MAX_SUBRULES) {
      char *start;

      /* get field */
      start=s;
      while(*s && *s!='\t' && *s!='\n')
        s++;
      if (*s=='\t') {
        int j;

        *s=0;
        for (j=0; m_fieldDescriptions[j].name!=NULL; j++) {
          if (strcasecmp(m_fieldDescriptions[j].name, start)==0) {
            m_comboField[i]->setCurrentItem(j+1);
            break;
          }
        }
        s++;

        /* get operator */
        start=s;
        while(*s && *s!='\t' && *s!='\n')
          s++;
        if (*s=='\t') {
          char safe;

          *s=0;
          for (j=0; _operators[j].name!=NULL; j++) {
            if (strcasecmp(_operators[j].name, start)==0) {
              m_comboOperator[i]->setCurrentItem(j+1);
              break;
            }
          }
          s++;

          /* get operand */
          start=s;
          while(*s && *s!='\n')
            s++;
          safe=*s;
          *s=0;
          m_editOperand[i]->setText(start);
          *s=safe;
          i++;
        }
      }

      /* seek end of expression */
      while(*s && *s!='\n')
        s++;
      if (*s=='\n')
        s++;
    }

    free(cpy);
  }

  return 0;
}



FXint FF_EditRule::fromGui(AE_RULE *r) {
  GWEN_BUFFER *tbuf;
  int i;

  if (m_checkCombineAnd->getCheck())
    AE_Rule_SetCombineMode(r, AE_RuleCombineMode_And);
  else
    AE_Rule_SetCombineMode(r, AE_RuleCombineMode_Or);

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  for (i=0; i<MAX_SUBRULES; i++) {
    int j;

    j=m_comboField[i]->getCurrentItem();
    if (j>0) {
      GWEN_Buffer_AppendString(tbuf, m_fieldDescriptions[j-1].name);
      GWEN_Buffer_AppendByte(tbuf, '\t');
      j=m_comboOperator[i]->getCurrentItem();
      if (j>0) {
        GWEN_Buffer_AppendString(tbuf, _operators[j-1].name);
        GWEN_Buffer_AppendByte(tbuf, '\t');
        GWEN_Buffer_AppendString(tbuf, m_editOperand[i]->getText().text());
        GWEN_Buffer_AppendByte(tbuf, '\n');
      }
    }
  }
  AE_Rule_SetExpression(r, GWEN_Buffer_GetStart(tbuf));
  GWEN_Buffer_free(tbuf);

  return 0;
}



long FF_EditRule::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_EditRule::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_EditRule::editRule(FF_App* app,
                           const FieldDescription *fd,
                           FXComposite *parent,
                           const FXString &title,
                           AE_RULE *r) {
  FF_EditRule dlg(app, fd, parent, title, DECOR_ALL);
  FXuint dlgres;

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.toGui(r);

  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  dlg.fromGui(r);
  return true;
}



