/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_edittransaction.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_splittable.hpp"
#include "ff_selectpartner.hpp"
#include "ff_partneredit.hpp"

#include <aqfinance/engine/book/ae_group.h>


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



#define FF_EDITTRANSACTION_ACCOUNT_COLUMN 1



FXDEFMAP(FF_EditTransaction) FF_EditTransactionMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditTransaction::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditTransaction::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransaction::ID_TEXTFIELD, FF_EditTransaction::onCmdTextField),
  FXMAPFUNC(SEL_REPLACED, FF_EditTransaction::ID_SPLIT_TABLE, FF_EditTransaction::onRplSplitTable),
  FXMAPFUNC(SEL_RIGHTBUTTONRELEASE, FF_EditTransaction::ID_SPLIT_TABLE,
	    FF_EditTransaction::onListRightBtnReleased),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransaction::ID_CMD_DELSPLIT,
	    FF_EditTransaction::onCmdDelSplit),
};


FXIMPLEMENT(FF_EditTransaction, FXDialogBox, FF_EditTransactionMap, ARRAYNUMBER(FF_EditTransactionMap))




FF_EditTransaction::FF_EditTransaction(FF_App* a,
				       FXComposite *parent,
				       const FXString& title,
				       FXuint opts,
				       FXint x, FXint y, FXint w, FXint h,
				       FXint pl, FXint pr, FXint pt, FXint pb,
				       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m;
  FXHorizontalFrame *hf;
  FXGroupBox *fg;

  /* begin: first matrix */
  m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);

  new FXLabel(m, I18N("Date"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_dateEdit=new FF_DateEdit(m, this, ID_DATEEDIT, I18N("YYYY/MM/DD"));

  new FXLabel(m, I18N("Receipt"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_receiptEdit=new FXTextField(m, 64, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);

  new FXLabel(m, I18N("Memo"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);
  m_memoEdit=new FXTextField(m, 64, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  new FXSpring(m, LAYOUT_FILL_X);
  /* end: first matrix */

  fg=new FXGroupBox(vf, I18N("Splits"), LAYOUT_FILL_X | LAYOUT_FILL_Y | FRAME_LINE);
  m_splitTable=new FF_SplitTable(fg, this, ID_SPLIT_TABLE, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_splitTable->addColumn(FF_SplitTable::COLUMN_TYPE_MEMO);
  /* NOTE: Adjust FF_EDITTRANSACTION_ACCOUNT_COLUMN if you change the column */
  m_splitTable->addColumn(FF_SplitTable::COLUMN_TYPE_ACCOUNT);
  m_splitTable->addColumn(FF_SplitTable::COLUMN_TYPE_DEBIT_VALUE);
  m_splitTable->addColumn(FF_SplitTable::COLUMN_TYPE_CREDIT_VALUE);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  m_helpLabel=new FOX16_HtmlLabel(vf,
				  I18N("<font size=\"8\"><b>Hint</b>:"
				       "<ul>"
				       "<li><b>Assets</b>: "
				       "<font color=\"green\">Deposit</font> in <i>Debit</i> column, "
				       "<font color=\"red\">Withdrawal</font> in <i>Credit</i> column"
				       "</li>"
				       "<li><b>Liabilities</b>: "
				       "<font color=\"green\">Deposit</font> in <i>Credit</i> column, "
				       "<font color=\"red\">Withdrawal</font> in <i>Debit</i> column"
				       "</li>"
				       "<li><b>Income</b>: "
				       "<font color=\"green\">Deposit</font> in <i>Credit</i>"
				       "</li>"
				       "<li><b>Expenses</b>: "
				       "<font color=\"red\">Withdrawal</font> in <i>Debit</i> column</li>"
				       "</ul></font>"),
				  LAYOUT_FILL_X);

  /* ok/abort line */
  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_EditTransaction::FF_EditTransaction()
:FXDialogBox()
,m_app(NULL)
{

}



FF_EditTransaction::~FF_EditTransaction() {
}



void FF_EditTransaction::create() {
  FXDialogBox::create();
}



FXint FF_EditTransaction::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



FXint FF_EditTransaction::checkGui(FXbool quiet) {
  GWEN_DATE *d;
  AE_TRANSACTION *t;
  AE_SPLIT_LIST *splits;
  int cnt;
  int i;

  if (!m_dateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    if (!quiet) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Input Error"),
			  "%s",
			  I18N("Invalid Date"));
    }
    m_dateEdit->setFocus();
    return GWEN_ERROR_INVALID;
  }
  GWEN_Date_free(d);

  /* check splits */
  cnt=m_splitTable->getNumberOfRows()-1; /* ignore last line */
  splits=AE_Split_List_new();
  for (i=0; i<cnt; i++) {
    AE_SPLIT *sp;

    sp=m_splitTable->getSplitByRow(i);
    if (sp) {
      if (AE_Split_GetAccountId(sp)==0) {
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("No account assigned to split. Please select an account."));
	  m_splitTable->setCurrentItem(i, FF_EDITTRANSACTION_ACCOUNT_COLUMN, false);
	  m_splitTable->startInput(i, FF_EDITTRANSACTION_ACCOUNT_COLUMN);
	}
	AE_Split_List_free(splits);
	return GWEN_ERROR_BAD_DATA;
      }

      if (AE_Split_GetCommodityId(sp)==0) {
	if (!quiet) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("At least one row has no currency."));
	}
	AE_Split_List_free(splits);
	return GWEN_ERROR_BAD_DATA;
      }

      /* add copy to list because the split table owns the list elements */
      AE_Split_List_Add(AE_Split_dup(sp), splits);
    }
  }

  t=AE_Transaction_new();
  AE_Transaction_SetSplitList(t, splits);
  if (!m_app->checkTransaction(this, t, quiet)) {
    AE_Transaction_free(t);
    return GWEN_ERROR_BAD_DATA;
  }
  AE_Transaction_free(t);

  return 0;
}



FXint FF_EditTransaction::toGui(const AE_TRANSACTION *t) {
  const char *s;
  const GWEN_DATE *d;
  const AE_SPLIT_LIST *il;

  m_splitTable->reload();

  d=AE_Transaction_GetDate(t);
  if (t)
    m_dateEdit->setDate(d);

  s=AE_Transaction_GetReceipt(t);
  if (s && *s)
    m_receiptEdit->setText(FXString(s));

  s=AE_Transaction_GetMemo(t);
  if (s && *s)
    m_memoEdit->setText(FXString(s));


  il=AE_Transaction_GetSplitList(t);
  if (il) {
    AE_SPLIT *split;

    split=AE_Split_List_First(il);
    while(split) {
      m_splitTable->addSplit(AE_Split_dup(split));
      split=AE_Split_List_Next(split);
    }

    /* add empty split */
    split=AE_Split_new();
    m_splitTable->addSplit(split);
  }

  m_splitTable->sort();

  layout();

  return 0;
}



FXint FF_EditTransaction::fromGui(AE_TRANSACTION *t) {
  FXString s;
  GWEN_DATE *d;
  AE_SPLIT_LIST *splits;
  int cnt;
  int i;

  if (!m_dateEdit->getDate(&d)) {
    DBG_ERROR(0, "Bad date");
    return GWEN_ERROR_INVALID;
  }
  AE_Transaction_SetDate(t, d);
  GWEN_Date_free(d);

  s=m_receiptEdit->getText().simplify();
  if (s.empty())
    AE_Transaction_SetReceipt(t, NULL);
  else
    AE_Transaction_SetReceipt(t, s.text());

  s=m_memoEdit->getText().simplify();
  if (s.empty())
    AE_Transaction_SetMemo(t, NULL);
  else
    AE_Transaction_SetMemo(t, s.text());

  /* get splits */
  cnt=m_splitTable->getNumberOfRows()-1; /* ignore last line */
  splits=AE_Split_List_new();
  for (i=0; i<cnt; i++) {
    AE_SPLIT *t;

    t=m_splitTable->getSplitByRow(i);
    if (t) {
      /* add copy to list because the split table owns the list elements */
      AE_Split_List_Add(AE_Split_dup(t), splits);
    }
  }
  AE_Transaction_SetSplitList(t, splits);


  return 0;
}



void FF_EditTransaction::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_EditTransaction");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "splitsSortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_splitTable->getNumHeaders(); i++) {
    int t=GWEN_DB_GetIntValue(db, "splitsColumns", i, m_splitTable->getHeaderSize(i));
    m_splitTable->setHeaderSize(i, t);
    if (i==sortByColumn) {
      if (sortUp)
        m_splitTable->setHeaderArrowDir(i, TRUE);
      else
        m_splitTable->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_EditTransaction::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_splitTable->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "splitsColumns",
                        m_splitTable->getHeaderSize(i));
    b=m_splitTable->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "splitsSortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_EditTransaction", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



long FF_EditTransaction::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_EditTransaction::onRplSplitTable(FXObject*, FXSelector, void *ptr) {
  FXXTable::ColumnInfo *ci;

  DBG_ERROR(0, "CMD SplitTable");
  ci=(FXXTable::ColumnInfo*) ptr;

  if (ci) {
    if ((ci->getRow()+1)==m_splitTable->getNumberOfRows()) {
      AE_SPLIT *split;

      DBG_ERROR(0, "Adding empty split");
      split=AE_Split_new();
      m_splitTable->addSplit(split);
      m_splitTable->update();
    }
    else {
      DBG_ERROR(0, "Row %d is not the last row (%d)",
		ci->getRow(), m_splitTable->getNumberOfRows());
    }
  }
  else {
    DBG_ERROR(0, "No ci");
  }

  return 1;
}



long FF_EditTransaction::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
  }
  return 1;
}



long FF_EditTransaction::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_EditTransaction::onListRightBtnReleased(FXObject*, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*)ptr;

  if (!event->moved){
    FXMenuPane pmenu(this);

    new FXMenuCommand(&pmenu, I18N("Delete Split"), NULL, this, ID_CMD_DELSPLIT);

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



long FF_EditTransaction::onCmdDelSplit(FXObject*, FXSelector, void* ptr) {
  int r;

  DBG_ERROR(0, "Delete split");
  r=m_splitTable->getCurrentRow();
  if (r>=0) {
    AE_SPLIT *split;

    split=m_splitTable->getCurrentSplit();
    if (split) {
      FF_App *app;
      AE_BOOK *b;
      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
        FXuint res;

        res=FXMessageBox::question(this, MBOX_YES_NO,
				   I18N("Delete Split"),
				   "%s",
                                   I18N("Are you sure you want to delete the selected "
					"split?"));
        if (res!=MBOX_CLICKED_YES) {
          DBG_ERROR(0, "Aborted");
        }
	else {
	  m_splitTable->delRow(r);
	  m_splitTable->rebuild();
	}
      }
    }
  }

  return 1;
}



bool FF_EditTransaction::editTransaction(FF_App* app,
					 FXComposite *parent,
					 const FXString &title,
					 AE_TRANSACTION *t) {
  FF_EditTransaction dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  res=dlg.toGui(t);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());
  dlg.init();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(t);
  if (res==0) {
    return true;
  }

  return false;
}





