/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_edittransfer.hpp"
#include "ff_app.hpp"
#include "ff_transferedit.hpp"
#include "ff_picktransfer.hpp"

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)

#define MAX_PURPOSE_LINES 4



FXDEFMAP(FF_EditTransfer2) FF_EditTransferMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_EditTransfer2::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_EditTransfer2::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransfer2::ID_BUTTON_LOAD_TMPL, FF_EditTransfer2::onCmdLoadTmpl),
  FXMAPFUNC(SEL_COMMAND, FF_EditTransfer2::ID_BUTTON_SAVE_TMPL, FF_EditTransfer2::onCmdSaveTmpl)
};


FXIMPLEMENT(FF_EditTransfer2, FXDialogBox, FF_EditTransferMap, ARRAYNUMBER(FF_EditTransferMap))




FF_EditTransfer2::FF_EditTransfer2(FF_App* a,
                                   int editMode,
                                   int rules,
                                   FXComposite *parent,
                                   const FXString& title,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h,
                                   FXint pl, FXint pr, FXint pt, FXint pb,
                                   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_rules(rules)
,m_label(NULL)
,m_transferEdit(NULL)
,m_loadTmplButton(NULL)
,m_saveTmplButton(NULL)
{
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXHorizontalFrame *hf;

  /* description */
  m_label=new FOX16_HtmlLabel(vf, "", LAYOUT_FILL_X | FOX16_HtmlLabel::FLAGS_USE_FULL_WIDTH);

  m_transferEdit=new FF_TransferEdit(m_app, editMode, rules, vf,
                                     LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                     ((opts&OPTS_SHOWDATE)?(FF_TransferEdit::OPTS_SHOWDATE):0));

  new FXSpring(vf, LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);

  if (opts & OPTS_SHOWLOADTMPL)
    m_loadTmplButton=new THEMEBUTTON(hf, I18N("Load Template"), NULL, this, ID_BUTTON_LOAD_TMPL, BUTTON_NORMAL);
  if (opts & OPTS_SHOWSAVETMPL)
    m_saveTmplButton=new THEMEBUTTON(hf, I18N("Save Template"), NULL, this, ID_BUTTON_SAVE_TMPL, BUTTON_NORMAL);

  new FXSpring(hf, LAYOUT_FILL_X);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
                  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL, BUTTON_NORMAL);
}



FF_EditTransfer2::FF_EditTransfer2()
:FXDialogBox()
,m_app(NULL)
,m_rules(FF_TransferEdit::RULES_SINGLE_TRANSFER)
,m_label(NULL)
,m_transferEdit(NULL)
,m_loadTmplButton(NULL)
,m_saveTmplButton(NULL)
{
}



FF_EditTransfer2::~FF_EditTransfer2() {
}





void FF_EditTransfer2::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;
  FXString gname;

  gname="FF_EditTransfer2";
  switch(m_rules) {
  case FF_TransferEdit::RULES_CREATE_STO:
  case FF_TransferEdit::RULES_MODIFY_STO:
    gname+="-rec";
    break;
  default:
  case FF_TransferEdit::RULES_SINGLE_TRANSFER:
  case FF_TransferEdit::RULES_INTERNAL_TRANSFER:
  case FF_TransferEdit::RULES_CREATE_DATED_TRANSFER:
  case FF_TransferEdit::RULES_MODIFY_DATED_TRANSFER:
  case FF_TransferEdit::RULES_SEPA_TRANSFER:
    gname+="-norec";
    break;
  }
  gname+="-";
  gname+=FXStringVal(options, 16);

  db=m_app->getDialogConfig(gname.text());
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "transferEdit");
  m_transferEdit->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_EditTransfer2::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;
  FXString gname;

  gname="FF_EditTransfer2";
  switch(m_rules) {
  case FF_TransferEdit::RULES_CREATE_STO:
  case FF_TransferEdit::RULES_MODIFY_STO:
    gname+="-rec";
    break;
  default:
  case FF_TransferEdit::RULES_SINGLE_TRANSFER:
  case FF_TransferEdit::RULES_INTERNAL_TRANSFER:
  case FF_TransferEdit::RULES_CREATE_DATED_TRANSFER:
  case FF_TransferEdit::RULES_MODIFY_DATED_TRANSFER:
    gname+="-norec";
    break;
  }
  gname+="-";
  gname+=FXStringVal(options, 16);

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "transferEdit");
  m_transferEdit->fini(dbT);


  rv=m_app->setDialogConfig(gname.text(), db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



int FF_EditTransfer2::checkGui(FXbool quiet) {
  return m_transferEdit->checkGui(quiet);
}



int FF_EditTransfer2::toGui(const AE_STATEMENT *stm) {
  return m_transferEdit->toGui(stm);
}



int FF_EditTransfer2::fromGui(AE_STATEMENT *stm) {
  return m_transferEdit->fromGui(stm);
}



int FF_EditTransfer2::copyTransfer(AE_STATEMENT *st, AE_BOOK_TABLE_TYPE tt) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_STATEMENT *cpy;

    cpy=AE_Statement_dup(st);
    AE_Statement_SetDbId(cpy, 0);
    AE_Statement_SetId(cpy, 0);
    rv=AE_Book_AddStatement(b, tt, cpy, 1);
    if (rv<0) {
      DBG_ERROR(0, "Unable to add statement (%d)", rv);
    }
    else {
      app->signalDbReload(tt);
    }
    AE_Statement_free(cpy);
    return rv;
  }
  else
    return GWEN_ERROR_NOT_OPEN;
}



long FF_EditTransfer2::onCmdLoadTmpl(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_STATEMENT *stm=NULL;

  app=dynamic_cast<FF_App*>(getApp());
  if (FF_PickTransfer::pickTransfer(app, this, I18N("Select Transfer"),
                                    AE_Book_TableType_BankTransferTemplate, &stm)) {
    assert(stm);
    toGui(stm);
    AE_Statement_free(stm);
  }

  return 1;
}



long FF_EditTransfer2::onCmdSaveTmpl(FXObject*, FXSelector, void *ptr) {
  AE_STATEMENT *st;
  int rv;

  DBG_ERROR(0, "Button: Save Template");
  st=AE_Statement_new();
  rv=m_transferEdit->checkGui(false, FF_TransferEdit::EDITMODE_TEMPLATE);
  if (rv>=0) {
    m_transferEdit->fromGui(st, FF_TransferEdit::EDITMODE_TEMPLATE);
    copyTransfer(st, AE_Book_TableType_BankTransferTemplate);
  }
  AE_Statement_free(st);

  return 1;
}







long FF_EditTransfer2::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this, TRUE);
  }
  return 1;
}



long FF_EditTransfer2::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this, FALSE);
  return 1;
}



bool FF_EditTransfer2::editSingleTransfer(FF_App* app,
                                          int editMode,
                                          FXComposite *parent,
                                          const FXString &title,
                                          AE_STATEMENT *stm,
                                          FXuint opts) {
  FF_EditTransfer2 dlg(app, editMode, FF_TransferEdit::RULES_SINGLE_TRANSFER, parent, title, opts);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.m_label->setText(I18N("Transfer money from one of your bank accounts."));
  dlg.init();
  res=dlg.toGui(stm);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(stm);
  if (res==0)
    return true;
  return false;
}



bool FF_EditTransfer2::editInternalTransfer(FF_App* app,
                                            int editMode,
                                            FXComposite *parent,
                                            const FXString &title,
                                            AE_STATEMENT *stm,
                                            FXuint opts) {
  FF_EditTransfer2 dlg(app, editMode, FF_TransferEdit::RULES_INTERNAL_TRANSFER, parent, title, opts);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.m_label->setText(I18N("Transfer money between accounts at the same bank."));
  dlg.init();
  res=dlg.toGui(stm);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(stm);
  if (res==0)
    return true;
  return false;
}




bool FF_EditTransfer2::createStandingOrder(FF_App* app,
                                           int editMode,
                                           FXComposite *parent,
                                           const FXString &title,
                                           AE_STATEMENT *stm,
                                           FXuint opts) {
  FF_EditTransfer2 dlg(app, editMode, FF_TransferEdit::RULES_CREATE_STO, parent, title, opts | OPTS_SHOWDATE);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.m_label->setText(I18N("Create a recurring transfer order."));
  dlg.init();
  res=dlg.toGui(stm);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(stm);
  if (res==0)
    return true;
  return false;
}



bool FF_EditTransfer2::editStandingOrder(FF_App* app,
                                         int editMode,
                                         FXComposite *parent,
                                         const FXString &title,
                                         AE_STATEMENT *stm,
                                         FXuint opts) {
  FF_EditTransfer2 dlg(app, editMode, FF_TransferEdit::RULES_MODIFY_STO, parent, title, opts);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.m_label->setText(I18N("Modify a recurring transfer order."));
  dlg.init();
  res=dlg.toGui(stm);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(stm);
  if (res==0)
    return true;
  return false;
}



bool FF_EditTransfer2::editSepaTransfer(FF_App* app,
                                        int editMode,
                                        FXComposite *parent,
                                        const FXString &title,
                                        AE_STATEMENT *stm,
                                        FXuint opts) {
  FF_EditTransfer2 dlg(app, editMode, FF_TransferEdit::RULES_SEPA_TRANSFER, parent, title, opts);
  FXuint dlgres;
  FXint res;

  dlg.create();
  dlg.m_label->setText(I18N("Create a SEPA transfer order."));
  dlg.init();
  res=dlg.toGui(stm);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  dlg.hide();
  if (dlgres==0)
    return false;

  res=dlg.fromGui(stm);
  if (res==0)
    return true;
  return false;
}





