/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_manageitems.hpp"

#include "ff_app.hpp"
#include "ff_itemview.hpp"

#include <aqfinance/engine/book/ae_item.h>


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_ManageItems) FF_ManageItemsMap[]={
  FXMAPFUNC(SEL_INSERTED, FF_ManageItems::ID_ITEM_VIEW, FF_ManageItems::onEngineItemInserted),
  FXMAPFUNC(SEL_REPLACED, FF_ManageItems::ID_ITEM_VIEW, FF_ManageItems::onEngineItemReplaced),
  FXMAPFUNC(SEL_OPENED, FF_ManageItems::ID_ITEM_VIEW, FF_ManageItems::onEngineItemOpened),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_ManageItems::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_ManageItems::onCmdCancel)
};


FXIMPLEMENT(FF_ManageItems, FXDialogBox, FF_ManageItemsMap, ARRAYNUMBER(FF_ManageItemsMap))




FF_ManageItems::FF_ManageItems(FF_App* a,
			       FXComposite *parent,
			       const FXString& title,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h,
			       FXint pl, FXint pr, FXint pt, FXint pb,
			       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_itemView(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FF_ItemTree *tr;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
			      I18N("Manage items."),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  m_itemView=new FF_ItemView(AE_Book_TableType_ItemTemplates, vf, LAYOUT_FILL_X|LAYOUT_FILL_Y);
  m_itemView->setSelector(ID_ITEM_VIEW);
  m_itemView->setTarget(this);

  tr=m_itemView->getItemTree();
  tr->setListStyle(FOLDINGLIST_BROWSESELECT |
		   FOLDINGLIST_SHOWS_LINES |
		   FOLDINGLIST_SHOWS_BOXES |
		   FOLDINGLIST_ROOT_BOXES |
		   FRAME_SUNKEN|FRAME_THICK);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Dismiss"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_ManageItems::FF_ManageItems()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_itemView(NULL)
{

}



FF_ManageItems::~FF_ManageItems() {
}



FXint FF_ManageItems::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



long FF_ManageItems::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_ManageItems::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



void FF_ManageItems::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;

  db=m_app->getDialogConfig("FF_ManageItems");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "itemView");
  m_itemView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_ManageItems::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "itemView");
  m_itemView->fini(dbT);

  rv=m_app->setDialogConfig("FF_ManageItems", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



void FF_ManageItems::reload() {
  FF_App *app;
  AE_BOOK *b;

  m_itemView->clear();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ITEM_LIST *lst;
    int rv;

    lst=AE_Item_List_new();
    rv=AE_Book_ReadItemList(b, AE_Book_TableType_ItemTemplates, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      m_itemView->toGui(lst);
      m_itemView->sortItems();
    }
    AE_Item_List_free(lst);
  }
}



long FF_ManageItems::onEngineItemInserted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Item inserted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ITEM *item;
    int rv;

    item=(AE_ITEM*) ptr;

    rv=AE_Book_AddItem(b, AE_Book_TableType_ItemTemplates, item, 1);
    if (rv<0) {
      DBG_ERROR(0, "Return code: %d", rv);
    }
  }

  return 1;
}



long FF_ManageItems::onEngineItemReplaced(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Item replaced");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_ITEM *item;
    int rv;

    item=(AE_ITEM*) ptr;

    /* write any changes */
    rv=AE_Book_WriteItem(b, AE_Book_TableType_ItemTemplates, item, 1);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
  }

  return 1;
}



long FF_ManageItems::onEngineItemOpened(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_ITEM *item;

    DBG_ERROR(0, "Item opened");
    item=(AE_ITEM*) ptr;

    // nothing to do for now
  }

  return 1;
}



void FF_ManageItems::manageItems(FF_App* a,
				 FXComposite *parent,
				 const FXString& title,
				 FXuint opts) {
  FF_ManageItems dlg(a, parent, title, opts);
  int rv;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  dlg.init();
  dlg.reload();
  rv=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (rv==0) {
    DBG_ERROR(AE_LOGDOMAIN, "User aborted");
  }
}



