/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_managetaxes.hpp"
#include "ff_app.hpp"
#include "ff_taxview.hpp"

#include <aqfinance/engine/book/ae_tax.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ManageTaxes) FF_ManageTaxesMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_ManageTaxes::onCmdAccept),
  FXMAPFUNC(SEL_INSERTED, FF_ManageTaxes::ID_TAX_VIEW, FF_ManageTaxes::onTaxInserted),
  FXMAPFUNC(SEL_REPLACED, FF_ManageTaxes::ID_TAX_VIEW, FF_ManageTaxes::onTaxReplaced),
  FXMAPFUNC(SEL_OPENED, FF_ManageTaxes::ID_TAX_VIEW, FF_ManageTaxes::onTaxOpened),
};


FXIMPLEMENT(FF_ManageTaxes, FXDialogBox,
            FF_ManageTaxesMap, ARRAYNUMBER(FF_ManageTaxesMap))




FF_ManageTaxes::FF_ManageTaxes(FF_App* a,
			       FXComposite *parent,
			       const FXString& title,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h,
			       FXint pl, FXint pr, FXint pt, FXint pb,
			       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_taxView(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_taxView=new FF_TaxView(vf, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_taxView->setTarget(this);
  m_taxView->setSelector(ID_TAX_VIEW);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Close"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_ManageTaxes::FF_ManageTaxes()
:FXDialogBox()
,m_app(NULL)
{
}



FF_ManageTaxes::~FF_ManageTaxes() {
}



void FF_ManageTaxes::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_ManageTaxes");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  m_taxView->init(db);
  GWEN_DB_Group_free(db);
}



void FF_ManageTaxes::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  m_taxView->fini(db);

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_ManageTaxes", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



void FF_ManageTaxes::create() {
  FXDialogBox::create();
}



long FF_ManageTaxes::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  /*hide(); */
  return 1;
}



void FF_ManageTaxes::reload() {
  FF_App *app;
  AE_BOOK *b;

  m_taxView->clear();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TAX_LIST *lst;
    int rv;

    lst=AE_Tax_List_new();
    rv=AE_Book_ReadTaxList(b, AE_Book_TableType_Tax, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      m_taxView->toGui(lst);
      m_taxView->sortTaxes();
    }
    AE_Tax_List_free(lst);
  }
}



long FF_ManageTaxes::onTaxInserted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Tax inserted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TAX *tax;
    int rv;

    tax=(AE_TAX*) ptr;

    rv=AE_Book_AddTax(b, AE_Book_TableType_Tax, tax, 1);
    if (rv<0) {
      DBG_ERROR(0, "Return code: %d", rv);
    }
  }

  return 1;
}



long FF_ManageTaxes::onTaxReplaced(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Tax replaced");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_TAX *tax;
    int rv;

    tax=(AE_TAX*) ptr;

    /* write any changes */
    rv=AE_Book_WriteTax(b, AE_Book_TableType_Tax, tax, 1);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
  }

  return 1;
}



long FF_ManageTaxes::onTaxOpened(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_TAX *tax;

    DBG_ERROR(0, "Tax opened");
    tax=(AE_TAX*) ptr;

    // nothing to do for now
  }

  return 1;
}


void FF_ManageTaxes::manageTaxes(FF_App* app,
				 FXComposite *parent,
				 const FXString &title) {
  FF_ManageTaxes dlg(app, parent, title, DECOR_ALL);

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.reload();

  dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  dlg.hide();
}



