/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_managetransfers.hpp"
#include "ff_app.hpp"
#include "ff_transferview.hpp"

#include <aqfinance/engine/book/ae_statement.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ManageTransfers) FF_ManageTransfersMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_ManageTransfers::onCmdAccept),
  FXMAPFUNC(SEL_INSERTED, FF_ManageTransfers::ID_STATEMENT_VIEW, FF_ManageTransfers::onTransferInserted),
  FXMAPFUNC(SEL_REPLACED, FF_ManageTransfers::ID_STATEMENT_VIEW, FF_ManageTransfers::onTransferReplaced),
  FXMAPFUNC(SEL_DELETED, FF_ManageTransfers::ID_STATEMENT_VIEW, FF_ManageTransfers::onTransferDeleted),
  FXMAPFUNC(SEL_OPENED, FF_ManageTransfers::ID_STATEMENT_VIEW, FF_ManageTransfers::onTransferOpened),
};


FXIMPLEMENT(FF_ManageTransfers, FXDialogBox,
            FF_ManageTransfersMap, ARRAYNUMBER(FF_ManageTransfersMap))




FF_ManageTransfers::FF_ManageTransfers(FF_App* a,
                                       FXComposite *parent,
                                       const FXString& title,
                                       AE_BOOK_TABLE_TYPE tt,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h,
                                       FXint pl, FXint pr, FXint pt, FXint pb,
                                       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_transferView(NULL)
,m_type(tt)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FF_TransferTree *t;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_transferView=new FF_TransferView(vf, m_type, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_transferView->setTarget(this);
  m_transferView->setSelector(ID_STATEMENT_VIEW);

  t=m_transferView->getTransferTree();
  assert(t);
  /*t->addColumn(FF_TransferTree::COLUMN_DATE, 100);*/
  t->addColumn(FF_TransferTree::COLUMN_REMOTE_NAME, 200);
  t->addColumn(FF_TransferTree::COLUMN_PURPOSE, 200);
  /*t->addColumn(FF_TransferTree::COLUMN_TYPE, 100);*/
  t->addColumn(FF_TransferTree::COLUMN_AMOUNT, 100);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Close"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_ManageTransfers::FF_ManageTransfers()
:FXDialogBox()
,m_app(NULL)
{
}



FF_ManageTransfers::~FF_ManageTransfers() {
}



void FF_ManageTransfers::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_ManageTransfers");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  m_transferView->init(db);
  GWEN_DB_Group_free(db);
}



void FF_ManageTransfers::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  m_transferView->fini(db);

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  rv=m_app->setDialogConfig("FF_ManageTransfers", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



void FF_ManageTransfers::create() {
  FXDialogBox::create();
}



long FF_ManageTransfers::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  /*hide(); */
  return 1;
}



void FF_ManageTransfers::reload() {
  FF_App *app;
  AE_BOOK *b;

  m_transferView->clear();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_STATEMENT_LIST *lst;
    int rv;

    lst=AE_Statement_List_new();
    rv=AE_Book_ReadStatementList(b, m_type, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      m_transferView->toGui(lst);
    }
    AE_Statement_List_free(lst);
  }
}



long FF_ManageTransfers::onTransferInserted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Transfer inserted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_STATEMENT *transfer;
    int rv;

    transfer=(AE_STATEMENT*) ptr;

    rv=AE_Book_AddStatement(b, m_type, transfer, 1);
    if (rv<0) {
      DBG_ERROR(0, "Return code: %d", rv);
    }
  }

  return 1;
}



long FF_ManageTransfers::onTransferReplaced(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Transfer replaced");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_STATEMENT *transfer;
    int rv;

    transfer=(AE_STATEMENT*) ptr;

    /* write any changes */
    rv=AE_Book_WriteStatement(b, m_type, transfer, 1);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
  }

  return 1;
}



long FF_ManageTransfers::onTransferOpened(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_STATEMENT *transfer;

    DBG_ERROR(0, "Transfer opened");
    transfer=(AE_STATEMENT*) ptr;

    // nothing to do for now
  }

  return 1;
}



long FF_ManageTransfers::onTransferDeleted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Transfer deleted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_STATEMENT *transfer;
    int rv;

    transfer=(AE_STATEMENT*) ptr;

    /* write any changes */
    rv=AE_Book_DeleteStatement(b, m_type, transfer, 1);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Database Error"),
                          I18N("Could not delete transfer (%d)"),
                          rv);
    }
  }

  return 1;
}


void FF_ManageTransfers::manageTransfers(FF_App* app,
                                         FXComposite *parent,
                                         const FXString &title,
                                         AE_BOOK_TABLE_TYPE tt) {
  FF_ManageTransfers dlg(app, parent, title, tt, DECOR_ALL);

  dlg.resize(440, 400);
  dlg.create();
  dlg.init();
  dlg.reload();

  dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  dlg.hide();
}



