/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_opendb.hpp"
#include "ff_app.hpp"


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_OpenDB) FF_OpenDBMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_OpenDB::ID_COMBOBOX, FF_OpenDB::onCmdComboBox),
  FXMAPFUNC(SEL_COMMAND, FF_OpenDB::ID_PATHBUTTON, FF_OpenDB::onCmdPathButton),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_OpenDB::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_OpenDB::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_OpenDB::ID_TEXTFIELD, FF_OpenDB::onCmdTextField),
};


FXIMPLEMENT(FF_OpenDB, FXDialogBox, FF_OpenDBMap, ARRAYNUMBER(FF_OpenDBMap))




FF_OpenDB::FF_OpenDB(FF_App* a,
                     FXComposite *parent,
                     const FXString& title,
                     FXuint opts,
                     FXint x, FXint y, FXint w, FXint h,
                     FXint pl, FXint pr, FXint pt, FXint pb,
                     FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_moduleCombo(NULL)
,m_pathLabel(NULL)
,m_pathEdit(NULL)
,m_pathButton(NULL)
,m_moduleLabel(NULL)
,m_mustExist(false){
  FXVerticalFrame *vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  FXMatrix *m=new FXMatrix(vf, 2, MATRIX_BY_COLUMNS | LAYOUT_FILL_X);
  FXHorizontalFrame *hf;

  new FXLabel(m, I18N("Module"), NULL, LABEL_NORMAL);
  m_moduleCombo=new FXComboBox(m, 1, this, ID_COMBOBOX, COMBOBOX_STATIC);
  m_moduleCombo->appendItem(I18N("Folder"));
  m_pathLabel=new FXLabel(m, I18N("Path"), NULL, LABEL_NORMAL | LAYOUT_FILL_X);

  hf=new FXHorizontalFrame(m, LAYOUT_FILL_X,
                           0, 0, 0, 0,
                           0, 0, 0, 0, 0, 0);
  m_pathEdit=new FXTextField(hf, 32, this, ID_TEXTFIELD, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  m_pathButton=new THEMEBUTTON(hf, "...", NULL, this, ID_PATHBUTTON, BUTTON_NORMAL);

  m_moduleLabel=new FOX16_HtmlLabel(vf, "", LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_moduleLabel->setMinimumWidth(m->getDefaultWidth());

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_OpenDB::FF_OpenDB()
:FXDialogBox()
,m_moduleCombo(NULL)
,m_pathLabel(NULL)
,m_pathEdit(NULL)
,m_pathButton(NULL)
,m_moduleLabel(NULL)
,m_mustExist(false){

}



FF_OpenDB::~FF_OpenDB() {
}



void FF_OpenDB::create() {
  FXDialogBox::create();
  m_moduleCombo->create();
  m_pathLabel->create();
  m_pathEdit->create();
  m_pathButton->create();
  m_moduleLabel->create();
}



FXint FF_OpenDB::checkGui(FXbool quiet) {
  FXint i=m_moduleCombo->getCurrentItem();
  FXString path;
  GWEN_URL *url;

  switch(i) {
  case 0:
    break;
  default:
    if (!quiet) {
      DBG_ERROR(0, "Unknown module selected");
    }
    return GWEN_ERROR_INVALID;
  }

  path=m_pathEdit->getText().simplify();
  if (path.empty()) {
    if (!quiet) {
      DBG_ERROR(0, "Empty path");
    }
    return GWEN_ERROR_INVALID;
  }

  path.insert(0,"dir://");
  url=GWEN_Url_fromString(path.text());
  if (url==NULL) {
    if (!quiet) {
      DBG_ERROR(0, "Bad URL");
    }
    return GWEN_ERROR_INVALID;
  }
  GWEN_Url_free(url);
  return 0;
}



FXint FF_OpenDB::toGui(const FXString &urlString) {
  GWEN_URL *url;
  const char *s;
  FXString path;

  url=GWEN_Url_fromString(urlString.text());
  if (url==NULL) {
    DBG_ERROR(0, "Bad URL, assuming default");
    m_moduleCombo->setCurrentItem(0);
    m_pathEdit->setText("");
    selectModule(0);
    return 0;
  }

  /* select protocol (module) */
  s=GWEN_Url_GetProtocol(url);
  if (s) {
    if (strcasecmp(s, "dir")!=0) {
      DBG_ERROR(0, "Bad protocol [%s]", s);
      GWEN_Url_free(url);
      return GWEN_ERROR_INVALID;
    }
    m_moduleCombo->setCurrentItem(0);
  }
  else
    m_moduleCombo->setCurrentItem(0);

  s=GWEN_Url_GetServer(url);
  if (s)
    path=FXString(s);

  s=GWEN_Url_GetPath(url);
  if (s) {
    if (!path.empty())
      path+=GWEN_DIR_SEPARATOR_S;
    path+=FXString(s);
  }
  m_pathEdit->setText(path);

  return 0;
}



FXint FF_OpenDB::fromGui(FXString &urlString) {
  FXint i=m_moduleCombo->getCurrentItem();
  FXString path;
  GWEN_URL *url;

  switch(i) {
  case 0:
    break;
  default:
    DBG_ERROR(0, "Unknown module selected");
    return GWEN_ERROR_INVALID;
  }

  path=m_pathEdit->getText().simplify();
  if (path.empty()) {
    DBG_ERROR(0, "Empty path");
    return GWEN_ERROR_INVALID;
  }

  path.insert(0,"dir://");
  url=GWEN_Url_fromString(path.text());
  if (url==NULL) {
    DBG_ERROR(0, "Bad URL");
    return GWEN_ERROR_INVALID;
  }
  GWEN_Url_free(url);

  urlString=path;

  return 0;
}



void FF_OpenDB::selectModule(int i) {
  switch(i) {
  case 0:
    DBG_ERROR(0, "Entry 0 selected");
    m_pathLabel->setText(I18N("Path"));
    m_moduleLabel->setText(I18N("<p>This plugin uses a folder to store the "
                                "database.</p>"
                                "<p><b>Path</b> must point to that folder.</p>"));
    m_pathButton->enable();
    break;

  default:
    DBG_ERROR(0, "Unknown entry %d selected", i);
    m_moduleLabel->setText("");
    break;
  }
}



long FF_OpenDB::onCmdComboBox(FXObject*, FXSelector, void *ptr) {
  FXint i=m_moduleCombo->getCurrentItem();

  DBG_ERROR(0, "ComboBox");
  selectModule(i);

  return 1;
}



long FF_OpenDB::onCmdPathButton(FXObject*, FXSelector, void *ptr) {

  FXString dirName=FXFileDialog::getOpenDirectory(this,
                                                  I18N("Select Folder"),
                                                  "");
  if (!dirName.empty()) {
    DBG_ERROR(0, "Dir=[%s]", dirName.text());
    m_pathEdit->setText(dirName);
  }
  return 1;
}



long FF_OpenDB::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "CMD TextField");
  return 1;
}



long FF_OpenDB::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_OpenDB::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}




FXString FF_OpenDB::getOpenDbUrl(FF_App* app,
                                 FXComposite *parent,
                                 const FXString &title,
                                 const FXString &url) {
  FF_OpenDB dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;
  FXString result;

  dlg.m_mustExist=true;
  res=dlg.toGui(url);
  if (res)
    return FXString();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return FXString();
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(result);
      if (res==0) {
        return result;
        break;
      }
    }
  }
  /* should never reach this point */
  return FXString();
}



FXString FF_OpenDB::getCreateDbUrl(FF_App* app,
                                   FXComposite *parent,
                                   const FXString &title,
                                   const FXString &url) {
  FF_OpenDB dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;
  FXString result;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  dlg.m_mustExist=false;
  res=dlg.toGui(url);
  if (res)
    return FXString();
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return FXString();
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(result);
      if (res==0) {
        return result;
        break;
      }
    }
  }
  /* should never reach this point */
  return FXString();
}







