/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_picktransfer.hpp"
#include "ff_app.hpp"
#include "ff_transfertree.hpp"

#include <aqfinance/engine/book/ae_statement.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_PickTransfer) FF_PickTransferMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_PickTransfer::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_PickTransfer::onCmdCancel),
  FXMAPFUNC(SEL_CHANGED, FF_PickTransfer::ID_STATEMENT_TREE, FF_PickTransfer::onTreeChanged),
  FXMAPFUNC(SEL_DOUBLECLICKED, FF_PickTransfer::ID_STATEMENT_TREE, FF_PickTransfer::onTreeDoubleClicked),
};


FXIMPLEMENT(FF_PickTransfer, FXDialogBox,
            FF_PickTransferMap, ARRAYNUMBER(FF_PickTransferMap))




FF_PickTransfer::FF_PickTransfer(FF_App* a,
                                 FXComposite *parent,
                                 const FXString& title,
                                 AE_BOOK_TABLE_TYPE tt,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h,
                                 FXint pl, FXint pr, FXint pt, FXint pb,
                                 FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_transferTree(NULL)
,m_acceptButton(NULL)
,m_type(tt)
,m_selectedStatement(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_transferTree=new FF_TransferTree(vf, this, ID_STATEMENT_TREE,
				     LAYOUT_FILL_X | LAYOUT_FILL_Y);

  /*t->addColumn(FF_TransferTree::COLUMN_DATE, 100);*/
  m_transferTree->addColumn(FF_TransferTree::COLUMN_REMOTE_NAME, 200);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_PURPOSE, 200);
  /*t->addColumn(FF_TransferTree::COLUMN_TYPE, 100);*/
  m_transferTree->addColumn(FF_TransferTree::COLUMN_AMOUNT, 100);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new FXSpring(hf, LAYOUT_FILL_X);
  m_acceptButton=new THEMEBUTTON(hf, I18N("Accept"), NULL, this, ID_ACCEPT,
                                 BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
                  BUTTON_NORMAL | BUTTON_DEFAULT);
}



FF_PickTransfer::FF_PickTransfer()
:FXDialogBox()
,m_app(NULL)
,m_transferTree(NULL)
,m_acceptButton(NULL)
,m_selectedStatement(NULL)
{
}



FF_PickTransfer::~FF_PickTransfer() {
}



void FF_PickTransfer::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_PickTransfer");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_transferTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_transferTree->getHeaderSize(i));
    m_transferTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      m_transferTree->setHeaderArrowDir(i, sortUp?TRUE:FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_PickTransfer::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_transferTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_transferTree->getHeaderSize(i));
    b=m_transferTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_PickTransfer", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



void FF_PickTransfer::create() {
  FXDialogBox::create();
}



long FF_PickTransfer::onTreeChanged(FXObject*, FXSelector, void *ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
	m_acceptButton->enable();
      }
      else {
	m_acceptButton->disable();
      }
    }
    else {
      m_acceptButton->disable();
    }
  }
  return 1;
}



long FF_PickTransfer::onTreeDoubleClicked(FXObject*, FXSelector, void *ptr) {
  FXFoldingItem *item;
  FF_TransferTreeItem *grItem;

  item=(FXFoldingItem*) ptr;

  grItem=dynamic_cast<FF_TransferTreeItem*>(item);
  if (grItem) {
    AE_STATEMENT *transfer=grItem->getTransfer();
    if (transfer) {
      m_selectedStatement=transfer;
      getApp()->stopModal(this, TRUE);
    }
  }

  return 1;
}



long FF_PickTransfer::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  AE_STATEMENT *stm;

  stm=m_transferTree->getCurrentTransfer();
  if (stm) {
    m_selectedStatement=AE_Statement_dup(stm);
    getApp()->stopModal(this, TRUE);
  }
  return 1;
}



long FF_PickTransfer::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  getApp()->stopModal(this, FALSE);
  return 1;
}



void FF_PickTransfer::reload() {
  FF_App *app;
  AE_BOOK *b;

  m_transferTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_STATEMENT_LIST *lst;
    int rv;

    lst=AE_Statement_List_new();
    rv=AE_Book_ReadStatementList(b, m_type, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      AE_STATEMENT *stm;

      while( (stm=AE_Statement_List_First(lst)) ) {
	AE_Statement_List_Del(stm);
	m_transferTree->addTransfer(stm, true, FF_TransferTree::USECOLOR_NORMAL);
      }
    }
    AE_Statement_List_free(lst);
  }

  m_transferTree->sort();
  m_transferTree->expandItems();
  m_transferTree->killSelection();
}



bool FF_PickTransfer::pickTransfer(FF_App* app,
				   FXComposite *parent,
				   const FXString &title,
				   AE_BOOK_TABLE_TYPE tt,
				   AE_STATEMENT **pStm) {
  FF_PickTransfer dlg(app, parent, title, tt, DECOR_ALL);
  FXuint dlgres;

  dlg.create();
  dlg.resize(dlg.getDefaultWidth()+150, dlg.getDefaultHeight()+150);
  dlg.init();
  dlg.reload();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);

  dlg.fini();
  dlg.hide();

  if (dlgres==0) {
    return false;
  }

  if (dlg.m_selectedStatement) {
    *pStm=AE_Statement_dup(dlg.m_selectedStatement);
    return true;
  }

  return false;
}



