/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectabacc.hpp"
#include "ff_app.hpp"
#include "ff_bankaccounttree.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SelectAbAccount) FF_SelectAbAccountMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectAbAccount::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectAbAccount::onCmdCancel),

};


FXIMPLEMENT(FF_SelectAbAccount, FXDialogBox,
            FF_SelectAbAccountMap, ARRAYNUMBER(FF_SelectAbAccountMap))




FF_SelectAbAccount::FF_SelectAbAccount(FF_App* a,
                                       FXComposite *parent,
                                       const FXString& title,
                                       AQDB_ID id,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h,
                                       FXint pl, FXint pr, FXint pt, FXint pb,
                                       FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_accountTree(NULL)
,m_accountId(id) {
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
			      I18N("Please select the <b>AqBanking</b> account from the "
				   "list below"),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);
  m_accountTree=new FF_BankAccountTree(vf, this, ID_ACCOUNT_LIST,
                                       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                       LIST_SINGLESELECT);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_SelectAbAccount::FF_SelectAbAccount()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_accountTree(NULL)
,m_accountId(0) {
}



FF_SelectAbAccount::~FF_SelectAbAccount() {
}



FXint FF_SelectAbAccount::checkGui(FXbool quiet) {
  if (m_accountTree->getFirstSelectedBankAccount())
    return 0;
  return GWEN_ERROR_GENERIC;
}



FXint FF_SelectAbAccount::toGui(AQDB_ID id) {
  AB_ACCOUNT_LIST2 *al;

  al=AB_Banking_GetAccounts(m_app->getBanking());
  if (al==NULL) {
    DBG_ERROR(0, "No data");
    return GWEN_ERROR_NO_DATA;
  }
  else {
    AB_ACCOUNT_LIST2_ITERATOR *it;

    it=AB_Account_List2_First(al);
    if (it) {
      AB_ACCOUNT *aba;

      aba=AB_Account_List2Iterator_Data(it);
      while(aba) {
        AE_BANKACCOUNT *a=AE_BankAccount_new();
        const char *s;

        s=AB_Account_GetAccountName(aba);
        AE_BankAccount_SetAccountName(a, s);

        s=AB_Account_GetAccountNumber(aba);
        AE_BankAccount_SetAccountNumber(a, s);

        s=AB_Account_GetBankCode(aba);
        AE_BankAccount_SetBankCode(a, s);

        s=AB_Account_GetBIC(aba);
        AE_BankAccount_SetBic(a, s);

        s=AB_Account_GetIBAN(aba);
        AE_BankAccount_SetIban(a, s);

        AE_BankAccount_SetId(a, AB_Account_GetUniqueId(aba));
        m_accountTree->addAccount(a, NULL, NULL);

        aba=AB_Account_List2Iterator_Next(it);
      }
      AB_Account_List2Iterator_free(it);
    }
    AB_Account_List2_free(al);
  }

  m_accountTree->killSelection();
  m_accountTree->selectBankAccount(id, true);
  m_accountTree->makeBankAccountVisible(id);

  return 0;
}



FXint FF_SelectAbAccount::fromGui(AQDB_ID &id) {
  AE_BANKACCOUNT *ba;

  ba=m_accountTree->getFirstSelectedBankAccount();
  if (ba) {
    id=AE_BankAccount_GetId(ba);
    return 0;
  }

  return GWEN_ERROR_GENERIC;
}



void FF_SelectAbAccount::create() {
  FXDialogBox::create();
  m_label->create();
  m_accountTree->create();
}



long FF_SelectAbAccount::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  hide();
  return 1;
}



long FF_SelectAbAccount::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_SelectAbAccount::selectAccount(FF_App* app,
                                       FXComposite *parent,
                                       const FXString &title,
                                       AQDB_ID &id) {
  FF_SelectAbAccount dlg(app, parent, title, id, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(440, 400);
  dlg.create();
  res=dlg.toGui(id);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();
  dlg.resize(dlg.getWidth(), dlg.getDefaultHeight());

  for (;;) {
    dlgres=dlg.execute(PLACEMENT_VISIBLE);
    DBG_ERROR(0, "Result: %d", dlgres);
    if (dlgres==0) {
      return false;
    }

    res=dlg.checkGui(FALSE);
    if (res==0) {
      res=dlg.fromGui(id);
      if (res==0) {
        return true;
      }
    }
  }
  /* should never reach this point */
  return false;
}



