/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectbankacc.hpp"
#include "ff_app.hpp"
#include "ff_bankaccounttree.hpp"
#include "ff_editbankaccount.hpp"

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SelectBankAccount) FF_SelectBankAccountMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectBankAccount::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectBankAccount::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_SelectBankAccount::ID_BUTTON_CREATE, FF_SelectBankAccount::onCmdCreate)

};


FXIMPLEMENT(FF_SelectBankAccount, FXDialogBox,
            FF_SelectBankAccountMap, ARRAYNUMBER(FF_SelectBankAccountMap))




FF_SelectBankAccount::FF_SelectBankAccount(FF_App* a,
                                           FXComposite *parent,
                                           const FXString& title,
                                           FXuint opts,
                                           FXint x, FXint y, FXint w, FXint h,
                                           FXint pl, FXint pr, FXint pt, FXint pb,
                                           FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_okButton(NULL)
,m_accountTree(NULL)
,m_presetBankAccount(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FXIcon *ic;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf, "", LAYOUT_FILL_X);
  //m_label->setMinimumWidth(150);
  m_accountTree=new FF_BankAccountTree(vf, this, ID_ACCOUNT_LIST,
                                       LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                       FOLDINGLIST_SHOWS_LINES |
                                       FOLDINGLIST_SHOWS_BOXES |
                                       FOLDINGLIST_ROOT_BOXES |
                                       FRAME_SUNKEN|FRAME_THICK |
                                       FOLDINGLIST_SINGLESELECT);
  ic=a->getIcon("minifolderopen");
  if (ic)
    m_accountTree->setIconOpen(ic);
  ic=a->getIcon("minifolder");
  if (ic)
    m_accountTree->setIconClosed(ic);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);

  new THEMEBUTTON(hf, I18N("Create"), NULL, this, ID_BUTTON_CREATE,
		  BUTTON_NORMAL | BUTTON_DEFAULT);

  new FXSpring(hf, LAYOUT_FILL_X);

  m_okButton=new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
                             BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL);
}



FF_SelectBankAccount::FF_SelectBankAccount()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_okButton(NULL)
,m_accountTree(NULL)
,m_presetBankAccount(NULL)
{
}



FF_SelectBankAccount::~FF_SelectBankAccount() {
}



void FF_SelectBankAccount::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_SelectBankAccount");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_accountTree->getHeaderSize(i));
    m_accountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_accountTree->setHeaderArrowDir(i, TRUE);
      else
        m_accountTree->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_SelectBankAccount::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_accountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_accountTree->getHeaderSize(i));
    b=m_accountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_SelectBankAccount", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_SelectBankAccount::checkGui(FXbool quiet) {
  if (m_accountTree->getFirstSelectedBankAccount())
    return 0;
  return GWEN_ERROR_GENERIC;
}



FXint FF_SelectBankAccount::toGui(const AE_BANKACCOUNT *presetba, AQDB_ID id) {
  AE_BOOK *b;

  m_accountTree->clearItems();
  m_accountTree->killSelection();
  m_presetBankAccount=presetba;

  b=m_app->getBook();
  if (b) {
    FXString txt;
    AE_GROUP_TREE *gt;
    AE_BANKACCOUNT_LIST *balist;
    AE_BANKACCOUNT *ba;
    int rv;

    /* read account groups */
    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_BankAccountGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Group_Tree_free(gt);
      return rv;
    }
    else {
      DBG_ERROR(0, "Adding %d groups", AE_Group_Tree_GetCount(gt));
      m_accountTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);

    /* read account list */
    balist=AE_BankAccount_List_new();
    rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, balist);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_BankAccount_List_free(balist);
      return rv;
    }

    while( (ba=AE_BankAccount_List_First(balist)) ) {
      /* unlink from list */
      AE_BankAccount_List_Del(ba);
      if (AE_BankAccount_GetOwnerType(ba)==0 &&
          AE_BankAccount_GetOwnerId(ba)==0) {
        /* tree takes over ownership */
        m_accountTree->addAccount(ba, NULL, NULL);
      }
      else
        AE_BankAccount_free(ba);
    }
    AE_BankAccount_List_free(balist);

    if (presetba) {
      const char *s1=AE_BankAccount_GetBankName(presetba);
      const char *s2=AE_BankAccount_GetAccountName(presetba);
      const char *s3=AE_BankAccount_GetBankCode(presetba);
      const char *s4=AE_BankAccount_GetAccountNumber(presetba);
      const char *s5=AE_BankAccount_GetIban(presetba);
      const char *sn=I18N("unknown");

      txt=FXStringFormat(I18N("Please select the bank account which "
                              "matches this:<br>"
                              "<b>Bank Name</b> <i>%s,</i><b>Account Name</b> <i>%s,</i>"
                              "<b>Bank Code</b> <i>%s,</i><b>Account Number</b> <i>%s,</i>"),
                         s1?s1:sn, s2?s2:sn, s3?s3:sn, s4?s4:sn, s5?s5:sn);
      m_accountTree->sortItems();
      m_accountTree->expandItems();
      m_accountTree->unselectAll();
      if (AE_BankAccount_GetId(presetba)) {
	m_accountTree->selectBankAccount(AE_BankAccount_GetId(presetba), true);
	m_accountTree->makeBankAccountVisible(AE_BankAccount_GetId(presetba));
      }
      else {
	/* TODO: try to find the best matching account */
      }
    }
    else {
      txt=FXString(I18N("Please select the <b>AqBanking</b> account from the list below"));
      m_accountTree->sortItems();
      m_accountTree->expandItems();
      if (id)  {
	m_accountTree->selectBankAccount(id, true);
	m_accountTree->makeBankAccountVisible(id);
      }
    }

    m_label->setText(txt);
    return 0;
  }
  else {
    DBG_ERROR(0, "No book");
    return GWEN_ERROR_NOT_OPEN;
  }
}



FXint FF_SelectBankAccount::fromGui(AQDB_ID &id) {
  AE_BANKACCOUNT *ba;

  ba=m_accountTree->getFirstSelectedBankAccount();
  if (ba) {
    id=AE_BankAccount_GetId(ba);
    return 0;
  }

  return GWEN_ERROR_GENERIC;
}



void FF_SelectBankAccount::create() {
  FXDialogBox::create();
  m_label->create();
  m_accountTree->create();
}



long FF_SelectBankAccount::onCmdCreate(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_BANKACCOUNT *ba;

    if (m_presetBankAccount) {
      ba=AE_BankAccount_dup(m_presetBankAccount);
      AE_BankAccount_SetDbId(ba, 0);
    }
    else
      ba=AE_BankAccount_new();
    if (FF_EditBankAccount::editBankAccount(app, this,
					    I18N("Add Bank Account"),
					    DECOR_TITLE|DECOR_BORDER |
					    FF_EditBankAccount::FF_EDITBANKACCOUNT_OPTS_SHOWGROUP,
					    ba)) {
      int rv;

      rv=AE_Book_AddBankAccount(b, AE_Book_TableType_BankAccount, ba, 1);
      if (rv<0) {
        DBG_ERROR(0, "Unable to add bank account (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_BankAccount);
    }

    /* tree takes over ownership */
    m_accountTree->addAccount(ba, NULL, NULL);

    m_accountTree->sortItems();
    m_accountTree->expandItems();
    m_accountTree->unselectAll();
    m_accountTree->selectBankAccount(AE_BankAccount_GetId(ba), true);
    m_accountTree->makeBankAccountVisible(AE_BankAccount_GetId(ba));
    m_okButton->setFocus();
  }

  return 1;
}



long FF_SelectBankAccount::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_SelectBankAccount::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_SelectBankAccount::selectAccount(FF_App* app,
                                         FXComposite *parent,
                                         const FXString &title,
                                         const AE_BANKACCOUNT *ba,
                                         AQDB_ID &id) {
  FF_SelectBankAccount dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(400, dlg.getDefaultHeight()+100);
  dlg.create();
  dlg.init();
  res=dlg.toGui(ba, id);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);
  dlg.layout();

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();

  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(id);
  if (res!=0) {
    return false;
  }

  return true;
}



