/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selectjob.hpp"
#include "ff_app.hpp"
#include "ff_jobtree.hpp"

#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/modules/ae_openbills.h>
#include <aqfinance/engine/modules/ae_opentasks.h>

#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_SelectJob) FF_SelectJobMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectJob::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectJob::onCmdCancel)
};


FXIMPLEMENT(FF_SelectJob, FXDialogBox,
            FF_SelectJobMap, ARRAYNUMBER(FF_SelectJobMap))




void FF_SelectJob::construct(const FXString &text){
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf, text, LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  m_jobTree=new FF_JobTree(vf, this, ID_JOB_LIST,
			   FOLDINGLIST_SHOWS_LINES |
			   FOLDINGLIST_SHOWS_BOXES |
			   FOLDINGLIST_ROOT_BOXES |
			   FOLDINGLIST_SINGLESELECT |
			   LAYOUT_FILL_X | LAYOUT_FILL_Y);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new FXSpring(hf, LAYOUT_FILL_X);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL);
}



FF_SelectJob::FF_SelectJob(FF_App* a,
			   FXComposite *parent,
			   const FXString& title,
			   const FXString &text,
			   AQDB_ID id,
			   JobSelection js,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr, FXint pt, FXint pb,
			   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_jobTree(NULL)
,m_jobId(id)
,m_jobSelection(js)
{
  construct(text);
}



FF_SelectJob::FF_SelectJob(FF_App* a,
			   const FXString& title,
			   const FXString &text,
			   AQDB_ID id,
			   JobSelection js,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr, FXint pt, FXint pb,
			   FXint hs, FXint vs)
:FXDialogBox(a, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_jobTree(NULL)
,m_jobId(id)
,m_jobSelection(js)
{
  construct(text);
}



FF_SelectJob::FF_SelectJob()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_jobTree(NULL)
,m_jobId(0)
{
}



FF_SelectJob::~FF_SelectJob() {
}



void FF_SelectJob::init() {
  GWEN_DB_NODE *db;
  int w, h;
  int i;
  int sortByColumn;
  int sortUp;

  db=m_app->getDialogConfig("FF_SelectJob");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_jobTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_jobTree->getHeaderSize(i));
    m_jobTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
        m_jobTree->setHeaderArrowDir(i, TRUE);
      else
        m_jobTree->setHeaderArrowDir(i, FALSE);
    }
  }

  GWEN_DB_Group_free(db);
}



void FF_SelectJob::fini() {
  GWEN_DB_NODE *db;
  int rv;
  int i;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  for (i=0; i<m_jobTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_jobTree->getHeaderSize(i));
    b=m_jobTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }

  rv=m_app->setDialogConfig("FF_SelectJob", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



FXint FF_SelectJob::checkGui(FXbool quiet) {
  if (m_jobTree->getFirstSelectedJob())
    return 0;
  return GWEN_ERROR_GENERIC;
}



static uint32_t calcTaskFlags(const AE_TASK *tk) {
  AE_JOB *j;
  uint32_t flags=0;
  AQDB_VALUE *v;
  int booked=0;
  int toBook=0;

  v=AQDB_Value_new();

  j=AE_Job_List_First(AE_Task_GetJobs(tk));
  while(j) {
    const AQDB_VALUE *vt;

    switch(AE_Job_GetType(j)) {
    case AE_Job_TypeInvoice:
      vt=AE_Job_GetOpenSum(j);
      if (vt) {
	AQDB_Value_AddValue(v, vt);
	if (!AQDB_Value_IsZero(vt)) {
	  const GWEN_DATE *dd;

	  dd=AE_Job_GetDueDate(j);
	  if (dd) {
	    GWEN_DATE *cd;
            int diff;

	    cd=GWEN_Date_CurrentDate();
	    diff=GWEN_Date_Diff(cd, dd);
	    if (diff>=-2)
	      flags|=FF_JobTreeItem::FLAGS_URGENT;
            GWEN_Date_free(cd);
	  }
	}
      }
      // TODO: determine if the billing is complete or partial
      flags|=FF_JobTreeItem::FLAGS_BILLED_FULL;
      toBook++;
      if (AE_Job_GetTransactionId(j))
        booked++;
      break;

    case AE_Job_TypePayment:
      flags|=FF_JobTreeItem::FLAGS_PAID_PARTIAL;
      toBook++;
      if (AE_Job_GetTransactionId(j))
	booked++;
      break;

    case AE_Job_TypeReminder:
      flags|=FF_JobTreeItem::FLAGS_REMINDER;
      break;

    case AE_Job_TypeDelivery:
      // TODO: determine whether the delivery is full or partial
      flags|=FF_JobTreeItem::FLAGS_DELIVERED_PARTIAL;
      break;

    default:
      break;
    }
    j=AE_Job_List_Next(j);
  }

  if (flags & FF_JobTreeItem::FLAGS_PAID_PARTIAL) {
    if (AQDB_Value_IsZero(v)) {
      flags&=~FF_JobTreeItem::FLAGS_PAID_PARTIAL;
      flags|=FF_JobTreeItem::FLAGS_PAID_FULL;
    }
  }

  if (toBook) {
    if (booked==toBook)
      flags|=FF_JobTreeItem::FLAGS_BOOKED_FULL;
    else if (booked)
      flags|=FF_JobTreeItem::FLAGS_BOOKED_PARTIAL;
  }


  AQDB_Value_free(v);

  return flags;
}



static uint32_t calcJobFlags(const AE_JOB *j) {
  uint32_t flags=0;
  const AQDB_VALUE *vt;

  switch(AE_Job_GetType(j)) {
  case AE_Job_TypeInvoice:
    vt=AE_Job_GetOpenSum(j);
    if (vt) {
      if (AQDB_Value_IsZero(vt)) {
	flags|=FF_JobTreeItem::FLAGS_PAID_FULL;
      }
      else {
	const GWEN_DATE *dd;

	dd=AE_Job_GetDueDate(j);
	if (dd) {
	  GWEN_DATE *cd;
	  int diff;

	  cd=GWEN_Date_CurrentDate();
	  diff=GWEN_Date_Diff(cd, dd);
	  if (diff>=-2)
	    flags|=FF_JobTreeItem::FLAGS_URGENT;
	  GWEN_Date_free(cd);
	}
      }
    }

    if (AE_Job_GetTransactionId(j))
      flags|=FF_JobTreeItem::FLAGS_BOOKED_FULL;
    break;

  case AE_Job_TypePayment:
    if (AE_Job_GetTransactionId(j))
      flags|=FF_JobTreeItem::FLAGS_BOOKED_FULL;
    break;

  case AE_Job_TypeReminder:
    flags|=FF_JobTreeItem::FLAGS_REMINDER;
    break;

  case AE_Job_TypeDelivery:
    // TODO: determine whether the delivery is full or partial
    flags|=FF_JobTreeItem::FLAGS_DELIVERED_PARTIAL;
    break;

  default:
    break;
  }

  return flags;
}



void FF_SelectJob::rebuild() {
  FF_App *app;
  AE_BOOK *b;
  AE_JOB *j;
  AQDB_ID currentJobId=0;
  AQDB_ID currentTaskId=0;

  j=m_jobTree->getCurrentJob();
  if (j) {
    currentJobId=AE_Job_GetId(j);
    currentTaskId=AE_Job_GetTaskId(j);
  }
  else {
    DBG_ERROR(0, "No current job");
  }
  m_jobTree->clearItems();
  m_jobTree->reload();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TASK_LIST *tl;
    int rv;

    tl=AE_Task_List_new();
    rv=AE_Book_GetOpenTasks(b, (m_jobSelection & JobSelection_Buys)?1:0, tl);
    if (rv<0) {
      DBG_ERROR(0, "Error getting open bills (%d)", rv);
    }
    else {
      AE_TASK *tk;

      tk=AE_Task_List_First(tl);
      while(tk) {
	AE_JOB *j;
        uint32_t flags;

        flags=calcTaskFlags(tk);
	m_jobTree->addTask(tk, flags);
	while( (j=AE_Job_List_First(AE_Task_GetJobs(tk))) ) {
	  AE_Job_List_Del(j);
	  flags=calcJobFlags(j);
	  m_jobTree->addJob(j, flags);
	}

	tk=AE_Task_List_Next(tk);
      }
    }
    AE_Task_List_free(tl);

    /* sort list */
    m_jobTree->sort();
    m_jobTree->expandItems();
  }
  else {
    DBG_ERROR(0, "No book (%s)", (m_jobSelection & JobSelection_Buys)?"incoming":"outgoing");
  }

  if (currentJobId) {
    FF_JobTreeItem *item=m_jobTree->findJobItem(currentJobId);
    if (item) {
      m_jobTree->unselectAll();
      item->setSelected(true);
      m_jobTree->makeItemVisible(item);
      m_jobTree->setCurrentItem(item);
    }
  }
  else if (currentTaskId) {
    FF_JobTreeItem *item=m_jobTree->findTaskItem(currentTaskId);
    if (item) {
      m_jobTree->unselectAll();
      item->setSelected(true);
      m_jobTree->makeItemVisible(item);
      m_jobTree->setCurrentItem(item);
    }
  }
}




FXint FF_SelectJob::toGui(AQDB_ID id) {
  m_jobId=id;
  rebuild();
  return 0;
}



FXint FF_SelectJob::fromGui(AQDB_ID &id) {
  AE_JOB *j;

  j=m_jobTree->getFirstSelectedJob();
  if (j) {
    id=AE_Job_GetId(j);
    return 0;
  }

  return GWEN_ERROR_GENERIC;
}



long FF_SelectJob::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  if (checkGui(FALSE)==0) {
    getApp()->stopModal(this,TRUE);
    hide();
  }
  return 1;
}



long FF_SelectJob::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  hide();
  return 1;
}



bool FF_SelectJob::selectJob(FF_App* app,
			     FXComposite *parent,
			     const FXString &title,
			     const FXString &text,
			     AQDB_ID &id,
			     JobSelection js) {
  FF_SelectJob dlg(app, parent, title, text, id, js, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(400, dlg.getDefaultHeight()+100);
  dlg.create();
  dlg.init();
  res=dlg.toGui(id);
  if (res)
    return false;
  dlg.layout();
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(id);
  if (res!=0)
    return false;
  return true;
}



bool FF_SelectJob::selectJob(FF_App* app,
			     const FXString &title,
			     const FXString &text,
			     AQDB_ID &id,
			     JobSelection js) {
  FF_SelectJob dlg(app, title, text, id, js, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  dlg.resize(400, dlg.getDefaultHeight()+100);
  dlg.create();
  dlg.init();
  res=dlg.toGui(id);
  if (res)
    return false;
  dlg.show(PLACEMENT_VISIBLE);

  dlgres=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(id);
  if (res!=0)
    return false;
  return true;
}







