/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_selecttax.hpp"

#include "ff_app.hpp"
#include "ff_taxview.hpp"

#include <aqfinance/engine/book/ae_tax.h>


#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/url.h>

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FXDEFMAP(FF_SelectTax) FF_SelectTaxMap[]={
  FXMAPFUNC(SEL_INSERTED, FF_SelectTax::ID_TAX_VIEW, FF_SelectTax::onTaxInserted),
  FXMAPFUNC(SEL_REPLACED, FF_SelectTax::ID_TAX_VIEW, FF_SelectTax::onTaxReplaced),
  FXMAPFUNC(SEL_OPENED, FF_SelectTax::ID_TAX_VIEW, FF_SelectTax::onTaxOpened),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_SelectTax::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_SelectTax::onCmdCancel)
};


FXIMPLEMENT(FF_SelectTax, FXDialogBox, FF_SelectTaxMap, ARRAYNUMBER(FF_SelectTaxMap))




FF_SelectTax::FF_SelectTax(FF_App* a,
			   FXComposite *parent,
			   const FXString& title,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr, FXint pt, FXint pb,
			   FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_label(NULL)
,m_taxView(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FF_TaxTree *tr;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_label=new FOX16_HtmlLabel(vf,
			      I18N("Please select the tax from the "
				   "list below"),
			      LAYOUT_FILL_X);
  m_label->setMinimumWidth(150);

  m_taxView=new FF_TaxView(vf, LAYOUT_FILL_X|LAYOUT_FILL_Y);
  m_taxView->setSelector(ID_TAX_VIEW);
  m_taxView->setTarget(this);

  tr=m_taxView->getTaxTree();
  tr->setListStyle(FOLDINGLIST_BROWSESELECT |
		   FOLDINGLIST_SHOWS_LINES |
		   FOLDINGLIST_SHOWS_BOXES |
		   FOLDINGLIST_ROOT_BOXES |
		   FRAME_SUNKEN|FRAME_THICK);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  new THEMEBUTTON(hf, I18N("Cancel"), NULL, this, ID_CANCEL,
		  BUTTON_NORMAL | LAYOUT_RIGHT);
  new THEMEBUTTON(hf, I18N("Ok"), NULL, this, ID_ACCEPT,
		  BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL | LAYOUT_RIGHT);
}



FF_SelectTax::FF_SelectTax()
:FXDialogBox()
,m_app(NULL)
,m_label(NULL)
,m_taxView(NULL)
{

}



FF_SelectTax::~FF_SelectTax() {
}



FXint FF_SelectTax::getDefaultHeight() {
  return FXDialogBox::getDefaultHeight()+100;
}



long FF_SelectTax::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_SelectTax::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



void FF_SelectTax::init() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int w, h;

  db=m_app->getDialogConfig("FF_SelectTax");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);
  else
    resize(getWidth(), getDefaultHeight()+100);

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_DEFAULT, "taxView");
  m_taxView->init(dbT);

  GWEN_DB_Group_free(db);
}



void FF_SelectTax::fini() {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbT;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "height", getHeight());

  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "taxView");
  m_taxView->fini(dbT);

  rv=m_app->setDialogConfig("FF_SelectTax", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



AE_TAX *FF_SelectTax::getSelectedTax() {
  return m_taxView->getTaxTree()->getCurrentTax();
}



void FF_SelectTax::reload() {
  FF_App *app;
  AE_BOOK *b;

  m_taxView->clear();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TAX_LIST *lst;
    int rv;

    lst=AE_Tax_List_new();
    rv=AE_Book_ReadTaxList(b, AE_Book_TableType_Tax, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      m_taxView->toGui(lst);
      m_taxView->sortTaxes();
    }
    AE_Tax_List_free(lst);
  }
}



long FF_SelectTax::onTaxInserted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Tax inserted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TAX *tax;
    int rv;

    tax=(AE_TAX*) ptr;

    rv=AE_Book_AddTax(b, AE_Book_TableType_Tax, tax, 1);
    if (rv<0) {
      DBG_ERROR(0, "Return code: %d", rv);
    }
  }

  return 1;
}



long FF_SelectTax::onTaxReplaced(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Tax replaced");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_TAX *tax;
    int rv;

    tax=(AE_TAX*) ptr;

    /* write any changes */
    rv=AE_Book_WriteTax(b, AE_Book_TableType_Tax, tax, 1);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
  }

  return 1;
}



long FF_SelectTax::onTaxOpened(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_TAX *tax;

    DBG_ERROR(0, "Tax opened");
    tax=(AE_TAX*) ptr;

    // nothing to do for now
  }

  return 1;
}



AE_TAX *FF_SelectTax::selectTax(FF_App* a,
				FXComposite *parent,
				const FXString& title,
				FXuint opts) {
  FF_SelectTax dlg(a, parent, title, opts);
  int rv;
  AE_TAX *tax;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  dlg.init();
  dlg.reload();
  rv=dlg.execute(PLACEMENT_VISIBLE);
  dlg.fini();
  if (rv==0) {
    DBG_ERROR(AE_LOGDOMAIN, "User aborted");
    return NULL;
  }

  tax=dlg.getSelectedTax();
  if (tax)
    AE_Tax_Attach(tax);

  return tax;
}




