/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_jobprinter.hpp"

#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>
#include <gwen-gui-fox16/fox16_htmllabel.hpp>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/directory.h>
#include <gwenhywfar/process.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg



FXDEFMAP(FF_JobPrinter) FF_JobPrinterMap[]={
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_ACCEPT, FF_JobPrinter::onCmdAccept),
  FXMAPFUNC(SEL_COMMAND, FXDialogBox::ID_CANCEL, FF_JobPrinter::onCmdCancel),
  FXMAPFUNC(SEL_COMMAND, FF_JobPrinter::ID_VIEW_BUTTON, FF_JobPrinter::onCmdView),
  FXMAPFUNC(SEL_COMMAND, FF_JobPrinter::ID_SAVE_BUTTON, FF_JobPrinter::onCmdSave),
  FXMAPFUNC(SEL_COMMAND, FF_JobPrinter::ID_EMAIL_BUTTON, FF_JobPrinter::onCmdEmail),
};



FXIMPLEMENT(FF_JobPrinter, FXDialogBox,
            FF_JobPrinterMap, ARRAYNUMBER(FF_JobPrinterMap))



FF_JobPrinter::FF_JobPrinter(FF_App* a,
			     FXComposite *parent,
			     const FXString& title,
			     FXuint opts,
			     FXint x, FXint y, FXint w, FXint h,
			     FXint pl, FXint pr, FXint pt, FXint pb,
			     FXint hs, FXint vs)
:FXDialogBox(parent, title, opts, x, y, w, h, pl, pr, pt, pb, hs, vs)
,m_app(a)
,m_dbJob(NULL)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;

  vf=new FXVerticalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);

  m_label=new FOX16_HtmlLabel(vf,
			      I18N("The PDF file has been created. The following steps are usually taken in a "
				   "typical scenario:"
				   "<ul>"
				   "<li><b>view</b> the file (this will start the external PDF viewer selected in "
				   "the applications settings dialog under Edit/Settings)</li>"
				   "<li><b>print</b> the file from within the external PDF viewer</li>"
				   "<li>mark the file as being printed by checking the <i>Mark as Printed</i> checkbox "
				   "below</li>"
				   "<li><b>save</b> the file by clicking the <i>Save</i> button below</li>"
                                   "</ul>"),
			      LAYOUT_FILL_X);
  m_markAsPrintedCheck=new FXCheckButton(vf, I18N("Mark as Printed"), this, ID_PRINTED_CHECK, CHECKBUTTON_NORMAL);
  m_markAsMailedCheck=new FXCheckButton(vf, I18N("Mark as Mailed"), this, ID_MAILED_CHECK, CHECKBUTTON_NORMAL);

  new FXHorizontalSeparator(vf, LAYOUT_FILL_X | SEPARATOR_GROOVE);
  hf=new FXHorizontalFrame(vf, LAYOUT_FILL_X | PACK_UNIFORM_WIDTH);
  m_printButton=new THEMEBUTTON(hf, I18N("View"), NULL, this, ID_VIEW_BUTTON, BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL);
  m_emailButton=new THEMEBUTTON(hf, I18N("Email"), NULL, this, ID_EMAIL_BUTTON, BUTTON_NORMAL);
  m_saveButton=new THEMEBUTTON(hf, I18N("Save"), NULL, this, ID_SAVE_BUTTON, BUTTON_NORMAL);
  new FXSpring(hf, LAYOUT_FILL_X);
  new THEMEBUTTON(hf, I18N("Close"), NULL, this, ID_ACCEPT, BUTTON_NORMAL);
}



FF_JobPrinter::FF_JobPrinter()
:FXDialogBox()
,m_app(NULL)
,m_dbJob(NULL)
{
}



FF_JobPrinter::~FF_JobPrinter() {
  if (m_dbJob)
    GWEN_DB_Group_free(m_dbJob);
}



void FF_JobPrinter::init() {
  GWEN_DB_NODE *db;
  int w, h;

  db=m_app->getDialogConfig("FF_JobPrinter");
  if (db==NULL)
    db=GWEN_DB_Group_new("config");

  w=GWEN_DB_GetIntValue(db, "width", 0, 0);
  h=GWEN_DB_GetIntValue(db, "height", 0, 0);
  if (w>0 && h>0)
    resize(w, h);

  GWEN_DB_Group_free(db);
}



void FF_JobPrinter::fini() {
  GWEN_DB_NODE *db;
  int rv;

  db=GWEN_DB_Group_new("config");

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
                      "width", getWidth());
  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS,
		      "height", getHeight());

  rv=m_app->setDialogConfig("FF_JobPrinter", db);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  GWEN_DB_Group_free(db);
}



int FF_JobPrinter::toGui(const AE_JOB *j,
			 const AF_PRINT_TABLE_DESCR *descr,
			 const char *tmpFileName, const char *saveFileName) {
  const char *s;

  if (AE_Job_GetFlags(j) & AE_JOB_FLAGS_PRINTED)
    m_markAsPrintedCheck->setCheck(TRUE);
  else
    m_markAsPrintedCheck->setCheck(FALSE);

  if (AE_Job_GetFlags(j) & AE_JOB_FLAGS_EMAILED)
    m_markAsMailedCheck->setCheck(TRUE);
  else
    m_markAsMailedCheck->setCheck(FALSE);

  s=AE_Job_GetDocName(j);
  if (s && *s)
    m_saveButton->disable();
  else
    m_saveButton->enable();

  m_tmpFileName=FXString(tmpFileName);
  m_saveFileName=FXString(saveFileName);

  s=AF_PrintTableDescr_GetEmailBody(descr);
  if (s)
    m_emailBody=FXString(s);

  if (m_dbJob)
    GWEN_DB_Group_free(m_dbJob);
  m_dbJob=GWEN_DB_Group_new("job");
  AE_Job_toDb(j, m_dbJob);
  s=GWEN_DB_GetCharValue(m_dbJob, "addressName", 0, NULL);
  if (s && *s) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    GWEN_Text_EscapeToBuffer(s, tbuf);
    GWEN_DB_SetCharValue(m_dbJob, GWEN_DB_FLAGS_OVERWRITE_VARS, "escapedAddressName", GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }

  return 0;
}



int FF_JobPrinter::fromGui(AE_JOB *j) {
  const char *s;

  if (m_markAsPrintedCheck->getCheck()==TRUE)
    AE_Job_AddFlags(j, AE_JOB_FLAGS_PRINTED);
  else
    AE_Job_SubFlags(j, AE_JOB_FLAGS_PRINTED);

  if (m_markAsMailedCheck->getCheck()==TRUE)
    AE_Job_AddFlags(j, AE_JOB_FLAGS_EMAILED);
  else
    AE_Job_SubFlags(j, AE_JOB_FLAGS_EMAILED);

  s=AE_Job_GetDocName(j);
  if (!(s && *s)) {
    s=strrchr(m_saveFileName.text(), GWEN_DIR_SEPARATOR);
    if (s==NULL)
      s=m_saveFileName.text();
    AE_Job_SetDocName(j, s);
  }

  return 0;
}



long FF_JobPrinter::onCmdAccept(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Accept");
  getApp()->stopModal(this,TRUE);
  return 1;
}



long FF_JobPrinter::onCmdCancel(FXObject*, FXSelector, void *ptr) {
  DBG_ERROR(0, "Cancel");
  getApp()->stopModal(this,FALSE);
  return 1;
}



long FF_JobPrinter::onCmdView(FXObject*, FXSelector, void *ptr) {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbJob;
  int rv;
  const char *s;

  /* prepare DB for args */
  db=GWEN_DB_Group_new("args");
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "fileName", m_tmpFileName.text());

  dbJob=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "job");
  if (m_dbJob)
    GWEN_DB_AddGroupChildren(dbJob, m_dbJob);

  s=AE_GetUserDataFolder();
  if (s && *s)
    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "userdatadir", s);

  rv=m_app->viewPdfFile(m_tmpFileName.text(), db);
  GWEN_DB_Group_free(db);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }

  return 1;
}



long FF_JobPrinter::onCmdEmail(FXObject*, FXSelector, void *ptr) {
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbJob;
  int rv;
  const char *s;

  /* prepare DB for args */
  db=GWEN_DB_Group_new("args");
  GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "fileName", m_tmpFileName.text());

  dbJob=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "job");
  if (m_dbJob)
    GWEN_DB_AddGroupChildren(dbJob, m_dbJob);

  s=AE_GetUserDataFolder();
  if (s && *s)
    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "userdatadir", s);

  /* this should be handled last */
  if (!m_emailBody.empty()) {
    GWEN_BUFFER *bbuf;

    bbuf=GWEN_Buffer_new(0, 256, 0, 1);
    rv=AE_Utils_Replace_Vars(m_emailBody.text(), bbuf, db);
    if (rv<0) {
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Setup  Error"),
			  "%s",
			  I18N("There seems to be an error in the email body settings in the job templates."));
      GWEN_Buffer_free(bbuf);
      GWEN_DB_Group_free(db);
      return 1;
    }
    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "body", GWEN_Buffer_GetStart(bbuf));
    GWEN_Buffer_free(bbuf);
  }

  rv=m_app->emailPdfFile(m_tmpFileName.text(), db);
  GWEN_DB_Group_free(db);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }

  return 1;
}



long FF_JobPrinter::onCmdSave(FXObject*, FXSelector, void *ptr) {
  if (FXFile::copy(m_tmpFileName, m_saveFileName)) {
    m_saveButton->disable();
  }
  else {
    FXMessageBox::error(this, MBOX_OK,
			I18N("IO  Error"),
                        "%s",
			I18N("Could not copy PDF file. Maybe it already exists?"));
  }
  return 1;
}



bool FF_JobPrinter::run(FF_App* app,
			FXComposite *parent,
			const FXString &title,
			AE_JOB *job,
			const AF_PRINT_TABLE_DESCR *descr,
			const char *tmpFileName,
			const char *saveFileName) {
  FF_JobPrinter dlg(app, parent, title, DECOR_ALL);
  FXuint dlgres;
  FXint res;

  /* this is needed before layout() because otherwise is doesn't work */
  dlg.create();
  dlg.init();
  res=dlg.toGui(job, descr, tmpFileName, saveFileName);
  if (res)
    return false;

  dlgres=dlg.execute();
  dlg.fini();

  DBG_ERROR(0, "Result: %d", dlgres);
  if (dlgres==0) {
    return false;
  }

  res=dlg.fromGui(job);
  if (res==0) {
    return true;
  }

  return false;
}







