/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2012 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_modtabbook.hpp"
#include "ff_modtabpage.hpp"
#include "ff_app.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)





FXDEFMAP(FF_ModTabBook) FF_ModTabBookMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_ModTabBook::ID_TABBOOK,
	    FF_ModTabBook::onTabClicked),
  FXMAPFUNC(SEL_LEFTBUTTONPRESS, FF_ModTabBook::ID_TABITEM,
	    FF_ModTabBook::onTabLeftButtonPressed),
};


            
FXIMPLEMENT(FF_ModTabBook, FXHorizontalFrame,
            FF_ModTabBookMap, ARRAYNUMBER(FF_ModTabBookMap))





FF_ModTabBook::FF_ModTabBook(FXComposite *p,
                             FXuint opts,
                             FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_tabBook(NULL)
,m_lastEnteredTab(NULL)
,m_config(NULL)
{
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());

  m_closeTabIcon=app->getIcon("closetab");
  m_tabBook=new FXTabBook(this, this, ID_TABBOOK,
                          LAYOUT_FILL_X|LAYOUT_FILL_Y);
}




FF_ModTabBook::FF_ModTabBook()
:FXHorizontalFrame()
,m_tabBook(NULL)
,m_lastEnteredTab(NULL)
,m_config(NULL)
{
}



FF_ModTabBook::~FF_ModTabBook() {
}



void FF_ModTabBook::create() {
  FXHorizontalFrame::create();
  if (m_closeTabIcon)
    m_closeTabIcon->create();
}



void FF_ModTabBook::init(GWEN_DB_NODE *db) {
  if (m_config==NULL) {
    m_config=GWEN_DB_Group_dup(db);
    std::list<FF_ModTabPage*>::iterator it;

    for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
      (*it)->init(m_config);
    }
  }
}



void FF_ModTabBook::fini(GWEN_DB_NODE *db) {
  if (m_config) {
    std::list<FF_ModTabPage*>::iterator it;

    /* sample open pages */
    GWEN_DB_DeleteVar(m_config, "openTabs");
    for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
      GWEN_DB_SetCharValue(m_config, GWEN_DB_FLAGS_DEFAULT, "openTabs",
                           (*it)->getPageId().text());
    }

    /* deinit open pages */
    for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
      (*it)->fini(m_config);
    }

    /* store config in given db */
    GWEN_DB_ClearGroup(db, NULL);
    GWEN_DB_AddGroupChildren(db, m_config);

    GWEN_DB_Group_free(m_config);
    m_config=NULL;
  }
}



void FF_ModTabBook::dbOpened(AE_BOOK *bk) {
  std::list<FF_ModTabPage*>::iterator it;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    (*it)->dbOpened(bk);
  }
}



void FF_ModTabBook::dbClosing() {
  std::list<FF_ModTabPage*>::iterator it;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    (*it)->dbClosing();
  }
}



void FF_ModTabBook::dbClosed() {
  std::list<FF_ModTabPage*>::iterator it;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    (*it)->dbClosed();
  }
}



void FF_ModTabBook::dbReload(AE_BOOK_TABLE_TYPE tt) {
  std::list<FF_ModTabPage*>::iterator it;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    (*it)->dbReload(tt);
  }
}



void FF_ModTabBook::aboutToSendJobs() {
  std::list<FF_ModTabPage*>::iterator it;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    (*it)->aboutToSendJobs();
  }
}



void FF_ModTabBook::enter() {
  if (m_lastEnteredTab!=NULL)
    m_lastEnteredTab->enter();
}



void FF_ModTabBook::leave() {
  if (m_lastEnteredTab!=NULL)
    m_lastEnteredTab->leave();
}



void FF_ModTabBook::reset() {
  std::list<FF_ModTabPage*>::iterator it;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    (*it)->reset();
  }
}



long FF_ModTabBook::onTabClicked(FXObject*, FXSelector, void *ptr) {
  FXint idx=(FXint)(FXival)ptr;
  FF_ModTabPage *ap;

  if (m_lastEnteredTab)
    m_lastEnteredTab->leave();

  ap=findPageAt(idx);
  if (ap)
    ap->enter();

  m_lastEnteredTab=ap;

  return 1;
}



long FF_ModTabBook::onTabLeftButtonPressed(FXObject *obj, FXSelector, void *ptr) {
  FXEvent* event=(FXEvent*) ptr;
  FXTabItem *item=(FXTabItem*) obj;

  if (item->isEnabled()){
    if (m_closeTabIcon) {
      FF_ModTabPage *page;

      page=(FF_ModTabPage*)(item->getNext());
      if (page) {
        if (page->getIsCloseable()) {
          int ix;

          ix=item->getWidth()-m_closeTabIcon->getWidth();
          if (event->win_x>ix) {
            int idx;
            FF_ModTabPage *nextPage=NULL;

            DBG_DEBUG(0, "Closing page %s", page->getPageId().text());
            idx=findIndexByPage(page);
            if (page==m_lastEnteredTab) {
              page->leave();
              m_lastEnteredTab=NULL;
            }
            page->fini(m_config);
            m_tabPages.remove(page);
            if (idx>=0) {
              nextPage=findPageAt(idx);
            }
            delete page;
            delete item;
            m_tabBook->layout();

            if (nextPage) {
              DBG_DEBUG(0, "Setting next page at same index");
              m_lastEnteredTab=nextPage;
              nextPage->enter();
            }
            return 1;
          }
        }
        else {
          DBG_ERROR(0, "Page %s not closeable", page->getPageId().text());
        }
      }
      else {
        DBG_ERROR(0, "No page");
      }
    }
    else {
      DBG_ERROR(0, "no icon");
    }
  }
  else {
    DBG_ERROR(0, "Not enabled");
  }

  return 0;
}



FF_ModTabPage *FF_ModTabBook::findPageById(const FXString &id) {
  std::list<FF_ModTabPage*>::iterator it;

  DBG_DEBUG(0, "Looking for page [%s]", id.text());
  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    DBG_DEBUG(0, "  Comparing against [%s]", (*it)->getPageId().text());
    if (FX::comparecase(id, (*it)->getPageId())==0) {
      return *it;
    }
  }

  return NULL;
}



int FF_ModTabBook::findIndexById(const FXString &id) {
  std::list<FF_ModTabPage*>::iterator it;
  int idx=0;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    if (FX::comparecase(id, (*it)->getPageId())==0) {
      return idx;
    }
    idx++;
  }

  return -1;
}



int FF_ModTabBook::findIndexByPage(FF_ModTabPage *p) {
  std::list<FF_ModTabPage*>::iterator it;
  int idx=0;

  for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
    if (*it==p)
      return idx;
    idx++;
  }

  return -1;
}



FF_ModTabPage *FF_ModTabBook::findPageAt(int idx) {
  if (idx>=0) {
    std::list<FF_ModTabPage*>::iterator it;

    for (it=m_tabPages.begin(); it!=m_tabPages.end(); it++) {
      if (idx==0)
        return *it;
      idx--;
    }
  }

  return NULL;
}



FXTabItem *FF_ModTabBook::addTabHeader(const FXString &title, bool closeable) {
  FXTabItem *tab;

  /* create tab entry */
  tab=new THEMETABITEM(m_tabBook, title, closeable?m_closeTabIcon:NULL,
                       JUSTIFY_LEFT | JUSTIFY_CENTER_Y | TEXT_BEFORE_ICON | FRAME_RAISED | FRAME_THICK);
  tab->setTarget(this);
  tab->setSelector(ID_TABITEM);
  return tab;
}



void FF_ModTabBook::addTabPage(FF_ModTabPage *p) {
  m_tabPages.push_back(p);
  m_tabBook->layout();
}



void FF_ModTabBook::setCurrentPage(FF_ModTabPage *p) {
  int idx=findIndexByPage(p);
  if (idx!=-1) {
    if (m_lastEnteredTab)
      m_lastEnteredTab->leave();
    m_lastEnteredTab=NULL;

    m_tabBook->setCurrent(idx, true);
    m_lastEnteredTab=p;
    m_lastEnteredTab->enter();
  }
}








