/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_modulemanager.hpp"
#include "ff_module.hpp"
#include "ff_app.hpp"
#include "ff_listitem.hpp"

#include <gwenhywfar/debug.h>



FXDEFMAP(FF_ModuleManager) FF_ModuleManagerMap[]={
  FXMAPFUNC(SEL_SELECTED, FF_ModuleManager::ID_LISTBOX, FF_ModuleManager::onListSelected)
};


FXIMPLEMENT(FF_ModuleManager, FXComposite, FF_ModuleManagerMap, ARRAYNUMBER(FF_ModuleManagerMap))




FF_ModuleManager::FF_ModuleManager(FF_App *app,
                                   FXComposite *p,
                                   FXuint opts)
:FXComposite(p, opts)
,m_app(app)
,m_currentModule(NULL)
,m_splitter(NULL)
,m_listBox(NULL)
,m_switcher(NULL)
{
  flags|=FLAG_ENABLED | FLAG_SHOWN;
  m_splitter=new FXSplitter(this, this,
                            ID_SPLITTER,
                            LAYOUT_SIDE_TOP | LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_listBox=new FXList(m_splitter,
                       this, ID_LISTBOX,
                       LAYOUT_FILL_Y |
                       ICON_ABOVE_TEXT);
  m_switcher=new FXSwitcher(m_splitter,
                            LAYOUT_FILL_X | LAYOUT_FILL_Y);
}



FF_ModuleManager::FF_ModuleManager()
:FXComposite()
,m_app(NULL)
,m_currentModule(NULL)
,m_splitter(NULL)
,m_listBox(NULL)
,m_switcher(NULL)
{
}



FF_ModuleManager::~FF_ModuleManager() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    delete (*it);
  }
  m_modules.clear();
}



void FF_ModuleManager::create() {
  FXComposite::create();
  m_splitter->create();
  m_listBox->create();
  m_switcher->create();
}



void FF_ModuleManager::init() {
  int rv;
  int i;

  rv=GWEN_ConfigMgr_GetGroup(AE_GetConfigManager(),
                             "modules",
                             "main",
                             &m_mainConfig);
  if (rv<0) {
    DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
    return;
  }

  i=GWEN_DB_GetIntValue(m_mainConfig, "splitterColumn", 0, m_splitter->getSplit(0));
  m_splitter->setSplit(0, i);

}



void FF_ModuleManager::fini() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->fini();
  }

  if (m_mainConfig) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(AE_GetConfigManager(),
                                "modules",
                                "main");
    if (rv<0) {
      DBG_ERROR(0, "Unable to lock main configuration group (%d)", rv);
    }
    else {
      GWEN_DB_SetIntValue(m_mainConfig, GWEN_DB_FLAGS_OVERWRITE_VARS,
                          "splitterColumn", m_splitter->getSplit(0));

      rv=GWEN_ConfigMgr_SetGroup(AE_GetConfigManager(),
                                 "modules",
                                 "main",
                                 m_mainConfig);
      if (rv<0) {
        DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }

      rv=GWEN_ConfigMgr_UnlockGroup(AE_GetConfigManager(),
                                    "modules",
                                    "main");
    }

    GWEN_DB_Group_free(m_mainConfig);
    m_mainConfig=NULL;
  }

}



void FF_ModuleManager::addModule(FF_Module *m) {
  FXString text;
  FXIcon *icon;
  FF_ListItem *item;
  AE_BOOK *b;

  b=m_app->getBook();

  text=m->getLabel();
  icon=m->getIcon();

  item=new FF_ListItem(text, icon, (void*) m);
  m->init(m_switcher);
  m_modules.push_back(m);
  m_listBox->appendItem(item, FALSE);
  if (b)
    m->dbOpened(b);
  if (m_modules.size()==1) {
    m_listBox->setCurrentItem(m_modules.size()-1, false);
    m_switcher->setCurrent(m_modules.size()-1);
    enterModule(m->getId());
  }
}



bool FF_ModuleManager::enterModule(int id) {
  FF_ModulePtrList::iterator it;
  int i;

  for (it=m_modules.begin(), i=0;
       it!=m_modules.end();
       it++, i++) {
    if ((*it)->getId()==id) {
      if (m_currentModule)
        m_currentModule->leave();
      m_currentModule=(*it);
      m_switcher->setCurrent(i);
      (*it)->enter();
      return true;
    }
  }

  return false;
}




void FF_ModuleManager::signalReset() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->reset();
  }
}



void FF_ModuleManager::signalDbOpened(AE_BOOK *bk) {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->dbOpened(bk);
  }
}



void FF_ModuleManager::signalDbClosing() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->dbClosing();
  }
}



void FF_ModuleManager::signalDbClosed() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->dbClosed();
  }
}



void FF_ModuleManager::signalDbReload(AE_BOOK_TABLE_TYPE tt) {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->dbReload(tt);
  }
}



bool FF_ModuleManager::dbCheckDeleteObject(FXComposite *parent,
                                           AE_BOOK_TABLE_TYPE tt,
                                           AQDB_ID id) {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    if (!((*it)->dbCheckDeleteObject(parent, tt, id)))
      return false;
  }

  return true;
}



void FF_ModuleManager::signalJobListUpdated() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->jobListUpdated();
  }
}



void FF_ModuleManager::signalAboutToSendJobs() {
  FF_ModulePtrList::iterator it;

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->aboutToSendJobs();
  }

}



long FF_ModuleManager::onListSelected(FXObject*, FXSelector, void* ptr) {
  FXint idx;
  FF_ModulePtrList::iterator it;
  FXint i;

  idx=(FXival) ptr;
  for (it=m_modules.begin(), i=idx;
       it!=m_modules.end();
       it++, i--) {
    if (i==0) {
      if (m_currentModule)
        m_currentModule->leave();
      m_currentModule=(*it);
      m_switcher->setCurrent(idx);
      (*it)->enter();
      break;
    }
  }

  return 1;
}



void FF_ModuleManager::layout(){
  m_splitter->position(0, 0, getWidth(), getHeight());
  flags&=~FLAG_DIRTY;
}




void FF_ModuleManager::dump(int indent) {
  int i;
  FF_ModulePtrList::iterator it;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FF_ModuleManager: %d/%d %d/%d\n", getX(), getY(), getWidth(), getHeight());

  for (i=0; i<indent+2; i++) fprintf(stderr, " ");
  fprintf(stderr, "FXSplitter: %d/%d %d/%d (%d %d)\n",
          m_splitter->getX(), m_splitter->getY(),
          m_splitter->getWidth(), m_splitter->getHeight(),
          m_splitter->getDefaultWidth(), m_splitter->getDefaultHeight());
  for (i=0; i<indent+4; i++) fprintf(stderr, " ");
  fprintf(stderr, "FXList: %d/%d %d/%d (%d %d)\n",
          m_listBox->getX(), m_listBox->getY(),
          m_listBox->getWidth(), m_listBox->getHeight(),
          m_listBox->getDefaultWidth(), m_listBox->getDefaultHeight());
  for (i=0; i<indent+4; i++) fprintf(stderr, " ");
  fprintf(stderr, "FXSwitcher: %d/%d %d/%d (%d %d)\n",
          m_switcher->getX(), m_switcher->getY(),
          m_switcher->getWidth(), m_switcher->getHeight(),
          m_switcher->getDefaultWidth(), m_switcher->getDefaultHeight());

  for (it=m_modules.begin();
       it!=m_modules.end();
       it++) {
    (*it)->dump(indent+6);
  }

}




