/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_partnerpage.hpp"
#include "ff_partnerview.hpp"
#include "ff_app.hpp"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_PartnerPage) FM_PartnerPageMap[]={
  FXMAPFUNC(SEL_INSERTED, FM_PartnerPage::ID_PARTNERVIEW, FM_PartnerPage::onPartnerInserted),
  FXMAPFUNC(SEL_REPLACED, FM_PartnerPage::ID_PARTNERVIEW, FM_PartnerPage::onPartnerReplaced),
  FXMAPFUNC(SEL_OPENED, FM_PartnerPage::ID_PARTNERVIEW, FM_PartnerPage::onPartnerOpened),
  FXMAPFUNC(SEL_CHANGED, FM_PartnerPage::ID_MATCHER, FM_PartnerPage::onChgMatcher)
};


            
FXIMPLEMENT(FM_PartnerPage, FXVerticalFrame,
            FM_PartnerPageMap, ARRAYNUMBER(FM_PartnerPageMap))





FM_PartnerPage::FM_PartnerPage(FXComposite *p,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h)
:FXVerticalFrame(p, opts, x, y, w, h)
,m_partnerView(NULL)
{
  FXHorizontalFrame *hf;

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  m_partnerView=new FF_PartnerView(this, LAYOUT_FILL_X|LAYOUT_FILL_Y);
  m_partnerView->setSelector(ID_PARTNERVIEW);
  m_partnerView->setTarget(this);
}



FM_PartnerPage::FM_PartnerPage()
:FXVerticalFrame()
,m_partnerView(NULL)
{
}



FM_PartnerPage::~FM_PartnerPage() {
}



void FM_PartnerPage::init(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Init");
  if (m_partnerView)
    m_partnerView->init(db);
}



void FM_PartnerPage::fini(GWEN_DB_NODE *db) {
  DBG_INFO(0, "Fini");
  if (m_partnerView)
    m_partnerView->fini(db);
}



void FM_PartnerPage::dbOpened(AE_BOOK *bk) {
  if (m_partnerView)
    reload();
}



void FM_PartnerPage::dbClosing() {
  if (m_partnerView)
    m_partnerView->clear();
}



void FM_PartnerPage::dbClosed() {
  if (m_partnerView)
    m_partnerView->clear();
}



void FM_PartnerPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (m_partnerView) {
    if (tt==AE_Book_TableType_None ||
	tt==AE_Book_TableType_Partner ||
	tt==AE_Book_TableType_PartnerGroup) {
      reload();
    }
  }
}



void FM_PartnerPage::enter() {
}



void FM_PartnerPage::leave() {
}



void FM_PartnerPage::reset() {
  if (m_partnerView)
    m_partnerView->clear();
}



long FM_PartnerPage::onPartnerInserted(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Partner inserted");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_PARTNER *pt;
    int rv;
    uint32_t flags;

    pt=(AE_PARTNER*) ptr;

    /* assign payable and receivable accounts */
    flags=AE_Partner_GetFlags(pt);
    if (((flags & AE_PARTNER_FLAGS_CUSTOMER) && AE_Partner_GetReceivableAccountId(pt)==0) ||
	((flags & AE_PARTNER_FLAGS_VENDOR) && AE_Partner_GetPayableAccountId(pt)==0)) {
      AE_ACCOUNT_TREE *at;
      AE_ACCOUNT *acc;

      at=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	if (flags & AE_PARTNER_FLAGS_CUSTOMER) {
	  if (AE_Partner_GetReceivableAccountId(pt)==0) {
	    acc=app->getPartnerReceivableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetReceivableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}

	if (flags & AE_PARTNER_FLAGS_VENDOR) {
	  if (AE_Partner_GetPayableAccountId(pt)==0) {
	    acc=app->getPartnerPayableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetPayableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}
      }
      AE_Account_Tree_free(at);
    }

    rv=AE_Book_ApplyPartner(b, pt, NULL, 1);
    if (rv<0) {
      DBG_ERROR(0, "Return code: %d", rv);
    }

    app->signalDbReload(AE_Book_TableType_Partner);
  }

  return 1;
}



long FM_PartnerPage::onPartnerReplaced(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  DBG_ERROR(0, "Partner replaced");
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_PARTNER *pt;
    AE_PARTNER *oldpt=NULL;
    int rv;
    uint32_t flags;

    pt=(AE_PARTNER*) ptr;

    /* assign payable and receivable accounts */
    flags=AE_Partner_GetFlags(pt);
    if (((flags & AE_PARTNER_FLAGS_CUSTOMER) && AE_Partner_GetReceivableAccountId(pt)==0) ||
	((flags & AE_PARTNER_FLAGS_VENDOR) && AE_Partner_GetPayableAccountId(pt)==0)) {
      AE_ACCOUNT_TREE *at;
      AE_ACCOUNT *acc;

      at=AE_Account_Tree_new();
      rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
	if (flags & AE_PARTNER_FLAGS_CUSTOMER) {
	  if (AE_Partner_GetReceivableAccountId(pt)==0) {
	    acc=app->getPartnerReceivableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetReceivableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}

	if (flags & AE_PARTNER_FLAGS_VENDOR) {
	  if (AE_Partner_GetPayableAccountId(pt)==0) {
	    acc=app->getPartnerPayableAccount(this, at, pt, 1);
	    if (acc)
	      AE_Partner_SetPayableAccountId(pt, AE_Account_GetId(acc));
	    else {
	      DBG_INFO(0, "here (%d)", rv);
	    }
	  }
	}
      }
      AE_Account_Tree_free(at);
    }

    /* read original partner */
    rv=AE_Book_ReadPartner(b, AE_Book_TableType_Partner,
			   AE_Partner_GetId(pt), &oldpt);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      /* get saved state */
      AE_Book_ReadPartnerSubObjects(b, oldpt);

      /* apply any changes */
      rv=AE_Book_ApplyPartner(b, pt, oldpt, 1);
      AE_Partner_free(oldpt);
      if (rv<0) {
	DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      }
      app->signalDbReload(AE_Book_TableType_Partner);
    }
  }

  return 1;
}



long FM_PartnerPage::onPartnerOpened(FXObject*, FXSelector, void *ptr) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b && ptr) {
    AE_PARTNER *pt;

    DBG_ERROR(0, "Partner opened");
    pt=(AE_PARTNER*) ptr;
    AE_Book_ReadPartnerSubObjects(b, pt);
  }

  return 1;
}



long FM_PartnerPage::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  reload();
  m_partnerView->getPartnerTree()->expandItems();
  return 1;
}



void FM_PartnerPage::reload() {
  if (!m_partnerView->isEditing()) {
    FF_App *app;
    AE_BOOK *b;
    FXString str;

    str=m_matcher->getText().simplify().trim();

    m_partnerView->clear();
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      AE_PARTNER_LIST *lst;
      int rv;

      lst=AE_Partner_List_new();
      rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, lst);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      }
      else {
	AE_PARTNER *pt;

	if (!str.empty()) {
	  AE_Partner_List_SelectMatching(lst, str.text());
	  /* remove non-matching partners */
	  pt=AE_Partner_List_First(lst);
	  while(pt) {
	    AE_PARTNER *next;

	    next=AE_Partner_List_Next(pt);
	    if (!(AE_Partner_GetRuntimeFlags(pt) & AE_PARTNER_RTFLAGS_SELECTED)) {
	      AE_Partner_List_Del(pt);
	      AE_Partner_free(pt);
	    }
	    pt=next;
	  }
	}

	m_partnerView->toGui(lst);
	m_partnerView->sortPartners();
      }
      AE_Partner_List_free(lst);
    }
  }
}




