/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "fm_transferstopage.hpp"
#include "fm_transferpage.hpp"
#include "ff_transfertree.hpp"
#include "ff_transferedit.hpp"
#include "ff_edittransfergroup.hpp"
#include "ff_edittransfer.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/modules/ae_statementexport.h>
#include <aqfinance/engine/modules/ae_recontransfers.h>

#include <aqbanking/jobsingletransfer.h>
#include <aqbanking/jobinternaltransfer.h>
#include <aqbanking/jobcreatesto.h>
#include <aqbanking/jobdeletesto.h>
#include <aqbanking/jobmodifysto.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FM_TransferStoPage) FM_TransferStoPageMap[]={
  FXMAPFUNC(SEL_COMMAND, FM_TransferStoPage::ID_BUTTON_CREATE_STO,
            FM_TransferStoPage::onCmdCreateSto),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStoPage::ID_BUTTON_MODIFY_STO,
            FM_TransferStoPage::onCmdModifySto),
  FXMAPFUNC(SEL_COMMAND, FM_TransferStoPage::ID_BUTTON_DELETE_STO,
            FM_TransferStoPage::onCmdDeleteSto),

  FXMAPFUNC(SEL_CHANGED, FM_TransferStoPage::ID_TRANSFER_TREE,
            FM_TransferStoPage::onTreeChanged),
  FXMAPFUNC(SEL_RIGHTBUTTONPRESS, FM_TransferStoPage::ID_TRANSFER_TREE,
            FM_TransferStoPage::onTreeRightBtnPress),

  FXMAPFUNC(SEL_CHANGED, FM_TransferStoPage::ID_MATCHER,
            FM_TransferStoPage::onChgMatcher)
};



FXIMPLEMENT(FM_TransferStoPage, FXVerticalFrame,
            FM_TransferStoPageMap, ARRAYNUMBER(FM_TransferStoPageMap))






FM_TransferStoPage::FM_TransferStoPage(FM_TransferPage *mainPage,
                                       FXComposite *p,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h)
:FXVerticalFrame(p, opts, x, y, w, h)
,m_mainPage(mainPage)
,m_transferTree(NULL)
,m_createStoButton(NULL)
,m_modifyStoButton(NULL)
,m_deleteStoButton(NULL)
,m_needRebuild(true)
,m_isVisible(false)
{
  FXVerticalFrame *vf;
  FXHorizontalFrame *hf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X);
  new FXLabel(hf, I18N("Filter"), NULL, LABEL_NORMAL | JUSTIFY_LEFT);
  m_matcher=new FXTextField(hf, 64, this, ID_MATCHER, TEXTFIELD_NORMAL);

  hf=new FXHorizontalFrame(this, LAYOUT_FILL_X | LAYOUT_FILL_Y);
  m_transferTree=new FF_TransferTree(hf, this, ID_TRANSFER_TREE,
				     LAYOUT_SIDE_LEFT |
				     LAYOUT_FILL_X | LAYOUT_FILL_Y |
				     FOLDINGLIST_EXTENDEDSELECT |
				     FOLDINGLIST_SHOWS_LINES |
				     FOLDINGLIST_SHOWS_BOXES |
				     FOLDINGLIST_ROOT_BOXES |
                                     FRAME_SUNKEN|FRAME_THICK);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_REMOTE_NAME, 200);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_PURPOSE, 200);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_AMOUNT, 100);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_PERIOD, 100);
  m_transferTree->addColumn(FF_TransferTree::COLUMN_DATE_LASTUPDATED, 100);

  ic=app->getIcon("minifolderopen");
  if (ic)
    m_transferTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_transferTree->setIconClosed(ic);

  vf=new FXVerticalFrame(hf, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_createStoButton=new THEMEBUTTON(vf,
                                    I18N("FM_TransferStoPage|Create Order"),
                                    NULL,
                                    this, ID_BUTTON_CREATE_STO,
                                    BUTTON_NORMAL);

  m_modifyStoButton=new THEMEBUTTON(vf,
                                    I18N("FM_TransferStoPage|Modify Order"),
                                    NULL,
                                    this, ID_BUTTON_MODIFY_STO,
                                    BUTTON_NORMAL);

  m_deleteStoButton=new THEMEBUTTON(vf,
                                    I18N("FM_TransferStoPage|Delete Order"),
                                    NULL,
                                    this, ID_BUTTON_DELETE_STO,
                                    BUTTON_NORMAL);

}




FM_TransferStoPage::FM_TransferStoPage()
:FXVerticalFrame()
,m_mainPage(NULL)
,m_transferTree(NULL)
,m_createStoButton(NULL)
,m_modifyStoButton(NULL)
,m_deleteStoButton(NULL)
,m_needRebuild(false)
{
}



FM_TransferStoPage::~FM_TransferStoPage() {
}



void FM_TransferStoPage::init(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;

  DBG_INFO(0, "Init");
  dbT=GWEN_DB_GetGroup(db, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "stoPage");
  if (dbT) {
    int i;
    int sortByColumn;
    int sortUp;

    sortByColumn=GWEN_DB_GetIntValue(dbT, "sortByColumn", 0, -1);
    sortUp=GWEN_DB_GetIntValue(dbT, "sortUp", 0, 0);

    for (i=0; i<m_transferTree->getNumHeaders(); i++) {
      int j=GWEN_DB_GetIntValue(dbT, "columns", i, m_transferTree->getHeaderSize(i));
      m_transferTree->setHeaderSize(i, j);
      if (i==sortByColumn) {
        if (sortUp)
          m_transferTree->setHeaderArrowDir(i, TRUE);
        else
          m_transferTree->setHeaderArrowDir(i, FALSE);
      }
    }
  }
}



void FM_TransferStoPage::fini(GWEN_DB_NODE *db) {
  GWEN_DB_NODE *dbT;
  int i;

  DBG_INFO(0, "Fini");
  dbT=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "stoPage");
  for (i=0; i<m_transferTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_DEFAULT, "columns",
                        m_transferTree->getHeaderSize(i));
    b=m_transferTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
        GWEN_DB_SetIntValue(dbT, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FM_TransferStoPage::create(){
  FXVerticalFrame::create();
}



void FM_TransferStoPage::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FM_TransferView: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  m_transferTree->dump(indent+2);
}



int FM_TransferStoPage::createSto(void) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;
  
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    AE_STATEMENT *cpy;
    int rb;

    cpy=AE_Statement_new();
    rb=FF_EditTransfer2::createStandingOrder(app, FF_TransferEdit::EDITMODE_STRICT,
					     this,
					     I18N("Create Standing Order"),
					     cpy,
					     DECOR_ALL);
    if (rb) {
      AE_BANKACCOUNT *ba=NULL;
      AB_ACCOUNT *a;
      AB_JOB *j;
      AB_TRANSACTION *t;
      uint32_t baId=0;
      int rv;

      rv=AE_Book_ReadBankAccount(b, AE_Book_TableType_BankAccount, AE_Statement_GetBankAccountId(cpy), &ba);
      if (rv<0) {
        DBG_ERROR(0, "Error reading bank account %lu",
                  (unsigned long int) AE_Statement_GetBankAccountId(cpy));
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Database Error"),
                            I18N("Error reading bank account %lu from database (%d)"),
                            (unsigned long int) AE_Statement_GetBankAccountId(cpy),
                            rv);
        return GWEN_ERROR_NOT_FOUND;
      }
      baId=AE_BankAccount_GetBankingId(ba);
      AE_BankAccount_free(ba);

      a=AB_Banking_GetAccount(ab, baId);
      if (a==NULL) {
        DBG_ERROR(0, "Account %d not found", baId);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Banking Error"),
                            I18N("AqBanking account %lu not found"), (unsigned long int) baId);
        return GWEN_ERROR_NOT_FOUND;
      }

      j=AB_JobCreateStandingOrder_new(a);
      rv=AB_Job_CheckAvailability(j);
      if (rv<0) {
        DBG_ERROR(0, "Job not available (%d)", rv);
        FXMessageBox::error(this, MBOX_OK,
			    I18N("Banking Error"),
                            I18N("Create Standing Order job is not available (%d)"), rv);
        AB_Job_free(j);
        return rv;
      }

      /* transform to banking transaction */
      t=AB_Transaction_new();
      rv=AE_StatementExport_StatementToTransaction(cpy, t);
      if (rv<0) {
        DBG_ERROR(0, "Error transforming statement to AB_TRANSACTION (%d)", rv);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Banking Error"),
                            I18N("Unable to create a banking transaction (%d)"), rv);
        AB_Transaction_free(t);
        AB_Job_free(j);
        return rv;
      }
      AE_Statement_free(cpy);

      AB_Job_SetTransaction(j, t);
      AB_Transaction_free(t);
      app->addJobToList(j);
      app->signalJobListUpdated();

      FXMessageBox::information(this, MBOX_OK,
                                I18N("Online Banking"),
                                "%s",
				I18N("A creation request for a standing order has been added to the outbox."));
      return 0;
    }
    else {
      DBG_ERROR(0, "Aborted by user");
      return GWEN_ERROR_USER_ABORTED;
    }
  }
  else {
    DBG_ERROR(0, "Aborted by user");
    return GWEN_ERROR_USER_ABORTED;
  }
  return 0;
}



long FM_TransferStoPage::onCmdCreateSto(FXObject*, FXSelector, void*) {
  int rv;

  rv=createSto();
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  return 1;
}



int FM_TransferStoPage::modifySto(const AE_STATEMENT *st) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;
  
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    AE_STATEMENT *cpy;
    int rb;

    cpy=AE_Statement_dup(st);
    rb=FF_EditTransfer2::editStandingOrder(app, FF_TransferEdit::EDITMODE_STRICT,
                                           this,
                                           I18N("Modify Standing Order"), cpy,
                                           DECOR_ALL);
    if (rb) {
      AE_BANKACCOUNT *ba=NULL;
      AB_ACCOUNT *a;
      AB_JOB *j;
      AB_TRANSACTION *t;
      uint32_t baId=0;
      int rv;

      rv=AE_Book_ReadBankAccount(b, AE_Book_TableType_BankAccount, AE_Statement_GetBankAccountId(cpy), &ba);
      if (rv<0) {
        DBG_ERROR(0, "Error reading bank account %lu",
                  (unsigned long int) AE_Statement_GetBankAccountId(cpy));
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Database Error"),
                            I18N("Error reading bank account %lu from database (%d)"),
                            (unsigned long int) AE_Statement_GetBankAccountId(cpy),
                            rv);
        return GWEN_ERROR_NOT_FOUND;
      }
      baId=AE_BankAccount_GetBankingId(ba);
      AE_BankAccount_free(ba);

      a=AB_Banking_GetAccount(ab, baId);
      if (a==NULL) {
        DBG_ERROR(0, "Account %d not found", baId);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Banking Error"),
                            I18N("AqBanking account %lu not found"), (unsigned long int) baId);
        return GWEN_ERROR_NOT_FOUND;
      }

      j=AB_JobModifyStandingOrder_new(a);
      rv=AB_Job_CheckAvailability(j);
      if (rv<0) {
        DBG_ERROR(0, "Job not available (%d)", rv);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Banking Error"),
                            I18N("Modify Standing Order job is not available (%d)"), rv);
        AB_Job_free(j);
        return rv;
      }

      /* transform to banking transaction */
      t=AB_Transaction_new();
      rv=AE_StatementExport_StatementToTransaction(cpy, t);
      if (rv<0) {
        DBG_ERROR(0, "Error transforming statement to AB_TRANSACTION (%d)", rv);
        FXMessageBox::error(this, MBOX_OK,
                            I18N("Banking Error"),
                            I18N("Unable to create a banking transaction (%d)"), rv);
        AB_Transaction_free(t);
        AB_Job_free(j);
        return rv;
      }
      AE_Statement_free(cpy);

      AB_Job_SetTransaction(j, t);
      AB_Transaction_free(t);
      app->addJobToList(j);
      app->signalJobListUpdated();

      FXMessageBox::information(this, MBOX_OK,
                                I18N("Online Banking"),
                                "%s",
                                I18N("A modification request for a standing order has been added to the outbox."));
      return 0;
    }
    else {
      DBG_ERROR(0, "Aborted by user");
      return GWEN_ERROR_USER_ABORTED;
    }
  }
  else {
    DBG_ERROR(0, "Aborted by user");
    return GWEN_ERROR_USER_ABORTED;
  }
  return 0;
}



long FM_TransferStoPage::onCmdModifySto(FXObject*, FXSelector, void*) {
  AE_STATEMENT *st;

  st=m_transferTree->getCurrentTransfer();
  if (st) {
    int rv;

    rv=modifySto(st);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }
  return 1;
}



int FM_TransferStoPage::deleteSto(const AE_STATEMENT *st) {
  FF_App *app;
  AE_BOOK *b;
  AB_BANKING *ab;
  
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  ab=app->getBanking();
  if (b && ab) {
    AE_BANKACCOUNT *ba=NULL;
    AB_ACCOUNT *a;
    AB_JOB *j;
    AB_TRANSACTION *t;
    uint32_t baId=0;
    int rv;

    rv=AE_Book_ReadBankAccount(b, AE_Book_TableType_BankAccount, AE_Statement_GetBankAccountId(st), &ba);
    if (rv<0) {
      DBG_ERROR(0, "Error reading bank account %lu not found",
                (unsigned long int) AE_Statement_GetBankAccountId(st));
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Database Error"),
                          I18N("Error reading bank account %lu from database (%d)"),
                          (unsigned long int) AE_Statement_GetBankAccountId(st),
                          rv);
      return GWEN_ERROR_NOT_FOUND;
    }
    baId=AE_BankAccount_GetBankingId(ba);
    AE_BankAccount_free(ba);

    a=AB_Banking_GetAccount(ab, baId);
    if (a==NULL) {
      DBG_ERROR(0, "Account %d not found", AE_Statement_GetBankAccountId(st));
      FXMessageBox::error(this, MBOX_OK,
			  I18N("Banking Error"),
			  "%s",
                          I18N("AqBanking account not found"));
      return GWEN_ERROR_NOT_FOUND;
    }

    j=AB_JobDeleteStandingOrder_new(a);
    rv=AB_Job_CheckAvailability(j);
    if (rv<0) {
      DBG_ERROR(0, "Job not available (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Banking Error"),
                          I18N("Delete Standing Order job is not available (%d)"), rv);
      AB_Job_free(j);
      return rv;
    }

    /* transform to banking transaction */
    t=AB_Transaction_new();
    rv=AE_StatementExport_StatementToTransaction(st, t);
    if (rv<0) {
      DBG_ERROR(0, "Error transforming statement to AB_TRANSACTION (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
                          I18N("Banking Error"),
                          I18N("Unable to create a banking transaction (%d)"), rv);
      AB_Transaction_free(t);
      AB_Job_free(j);
      return rv;
    }
    AB_Job_SetTransaction(j, t);

    app->addJobToList(j);
    app->signalJobListUpdated();

    FXMessageBox::information(this, MBOX_OK,
                              I18N("Online Banking"),
                              "%s",
                              I18N("A request for deleting a standing order has been added to the outbox."));
    return 0;
  }
  else {
    DBG_ERROR(0, "Aborted by user");
    return GWEN_ERROR_USER_ABORTED;
  }
  return 0;
}



long FM_TransferStoPage::onCmdDeleteSto(FXObject*, FXSelector, void*) {
  AE_STATEMENT *st;

  st=m_transferTree->getCurrentTransfer();
  if (st) {
    DBG_ERROR(0, "Deleting STO");
    deleteSto(st);
  }

  return 1;
}



long FM_TransferStoPage::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_modifyStoButton->enable();
        m_deleteStoButton->enable();
      }
      else {
        m_modifyStoButton->disable();
        m_deleteStoButton->disable();
      }
    }
    else {
      m_modifyStoButton->disable();
      m_deleteStoButton->disable();
    }
  }
  return 1;
}



long FM_TransferStoPage::onChgMatcher(FXObject*, FXSelector, void* ptr) {
  rebuild();
  m_transferTree->expandItems();
  return 1;
}



long FM_TransferStoPage::onTreeRightBtnPress(FXObject*, FXSelector, void* ptr) {
  FXEvent* event=(FXEvent*)ptr;
  if (!event->moved){
    FXMenuPane pmenu(this);
    FXMenuCommand *c;
    AE_GROUP *gr;
    AE_STATEMENT *st;

    gr=m_transferTree->getCurrentGroup();
    st=m_transferTree->getCurrentTransfer();

    new FXMenuSeparator(&pmenu);
    c=new FXMenuCommand(&pmenu, I18N("Create standing order"), NULL,
                        this, ID_BUTTON_CREATE_STO);
    new FXMenuSeparator(&pmenu);

    c=new FXMenuCommand(&pmenu, I18N("Modify standing order"), NULL,
                        this, ID_BUTTON_MODIFY_STO);
    if (!st) c->disable();

    c=new FXMenuCommand(&pmenu, I18N("Delete standing order"), NULL,
                        this, ID_BUTTON_DELETE_STO);
    if (!st) c->disable();

    new FXMenuSeparator(&pmenu);
    new FXMenuCommand(&pmenu, I18N("Back"), NULL, &pmenu, ID_UNPOST);

    pmenu.create();
    pmenu.popup(NULL, event->root_x, event->root_y);
    getApp()->runModalWhileShown(&pmenu);
  }

  return 1;
}



void FM_TransferStoPage::dbOpened(AE_BOOK *b) {
  if (m_isVisible) {
    rebuild();
    m_needRebuild=false;
  }
  else {
    m_needRebuild=true;
  }
}



void FM_TransferStoPage::dbClosing() {
  m_transferTree->clearItems();
}



void FM_TransferStoPage::dbClosed() {
}



void FM_TransferStoPage::dbReload(AE_BOOK_TABLE_TYPE tt) {
  if (tt==AE_Book_TableType_BankTransfer ||
      tt==AE_Book_TableType_BankStoRecurrence ||
      tt==AE_Book_TableType_None) {
    if (m_isVisible) {
      rebuild();
      m_needRebuild=false;
    }
    else
      m_needRebuild=true;
  }
}



void FM_TransferStoPage::rebuild() {
  FF_App *app;
  AE_BOOK *b;
  AE_GROUP *gr;
  AE_STATEMENT *st;
  AQDB_ID currentStmId=0;
  AQDB_ID currentGrId=0;
  FXString str;

  str=m_matcher->getText().simplify().trim();

  gr=m_transferTree->getCurrentGroup();
  if (gr)
    currentGrId=AE_Group_GetId(gr);
  st=m_transferTree->getCurrentTransfer();
  if (st)
    currentStmId=AE_Statement_GetId(st);

  m_transferTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_STATEMENT_LIST *sl;
    AE_BANKACCOUNT_LIST *al;

    /* read bank account list */
    al=AE_BankAccount_List_new();
    rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, al);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      /* read transfers */
      sl=AE_Statement_List_new();
      rv=AE_Book_ReadStatementList(b, AE_Book_TableType_BankStandingOrder, sl);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      else {
        rv=AE_Book_ReadBankStandingOrderListSubObjects(b, sl);
        if (rv<0) {
          DBG_ERROR(0, "here (%d)", rv);
        }
        else {
          AE_STATEMENT *st;
  
          if (!str.empty()) {
            AE_Statement_List_SelectMatching(sl, str.text());
            /* remove non-matching statements */
            st=AE_Statement_List_First(sl);
            while(st) {
              AE_STATEMENT *next;
  
              next=AE_Statement_List_Next(st);
              if (!(AE_Statement_GetRuntimeFlags(st) & AE_STATEMENT_RTFLAGS_SELECTED)) {
                AE_Statement_List_Del(st);
                AE_Statement_free(st);
              }
  
              st=next;
            }
          }
        }
  
        /* add transfers */
        while ( (st=AE_Statement_List_First(sl)) ) {
          AQDB_ID aid;

          AE_Statement_List_Del(st);
          aid=AE_Statement_GetBankAccountId(st);
          if (aid) {
            FF_GroupTreeItem *grItem;
            FF_TransferTreeItem *item;

            grItem=m_transferTree->findGroupItem(aid);
            if (grItem==NULL) {
              AE_BANKACCOUNT *ba;

              ba=AE_BankAccount_List_GetById(al, aid);
              if (ba) {
                FXString name;
                const char *s;

                gr=AE_Group_new();

                /* create name: bank name */
                s=AE_BankAccount_GetBankName(ba);
                if (s==NULL)
                  s=AE_BankAccount_GetBankCode(ba);
                if (s==NULL)
                  s=AE_BankAccount_GetBic(ba);
                if (s)
                  name+=FXString(s);

                /* account name */
                s=AE_BankAccount_GetAccountName(ba);
                if (s==NULL)
                  s=AE_BankAccount_GetAccountNumber(ba);
                if (s==NULL)
                  s=AE_BankAccount_GetIban(ba);
                if (s) {
                  if (!name.empty())
                    name+="/";
                  name+=FXString(s);
                }

                AE_Group_SetId(gr, aid);
                AE_Group_SetName(gr, name.text());

                grItem=new FF_GroupTreeItem(gr,
                                            m_transferTree->getIconOpen(),
                                            m_transferTree->getIconClosed());
                m_transferTree->appendItem(NULL, grItem, false);
              }
            }

            /* add to group item */
            item=new FF_TransferTreeItem(st);
            m_transferTree->appendItem(grItem, item, false);
          }
          else {
            m_transferTree->addTransfer(st, false, FF_TransferTree::USECOLOR_NORMAL);
          }
        }
      }
      AE_Statement_List_free(sl);
  
      m_transferTree->sort();
      m_transferTree->expandItems();
    }
    AE_BankAccount_List_free(al);
  }
  else {
    DBG_ERROR(0, "No book");
  }

  if (currentStmId!=0) {
    FF_TransferTreeItem *item=m_transferTree->findTransferItem(currentStmId);
    if (item) {
      m_transferTree->unselectAll();
      item->setSelected(true);
      m_transferTree->makeItemVisible(item);
      m_transferTree->setCurrentItem(item);
    }
    else {
      DBG_ERROR(0, "Item not found");
    }
  }
  else if (currentGrId!=0) {
    FF_GroupTreeItem *item=m_transferTree->findGroupItem(currentGrId);
    if (item) {
      item->setSelected(true);
      m_transferTree->makeItemVisible(item);
      m_transferTree->setCurrentItem(item);
    }
    else {
      DBG_ERROR(0, "Item not found");
    }
  }
}


void FM_TransferStoPage::enter() {
  m_isVisible=true;

  if (m_needRebuild) {
    rebuild();
    m_needRebuild=false;
  }
}



void FM_TransferStoPage::leave() {
  m_isVisible=false;
}



void FM_TransferStoPage::reset() {
  m_transferTree->clearItems();
}









