/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_addressview.hpp"
#include "ff_addresstree.hpp"
#include "ff_app.hpp"
#include "ff_editaddress.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_AddressView) FF_AddressViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_AddressView::ID_BUTTON_ADDADDRESS,
            FF_AddressView::onCmdAddAddress),
  FXMAPFUNC(SEL_COMMAND, FF_AddressView::ID_BUTTON_EDITADDRESS,
            FF_AddressView::onCmdEditAddress),
  FXMAPFUNC(SEL_COMMAND, FF_AddressView::ID_BUTTON_DELADDRESS,
            FF_AddressView::onCmdDelAddress),

  FXMAPFUNC(SEL_CHANGED, FF_AddressView::ID_ADDRESS_TREE,
            FF_AddressView::onTreeChanged),

};



FXIMPLEMENT(FF_AddressView, FXHorizontalFrame,
            FF_AddressViewMap, ARRAYNUMBER(FF_AddressViewMap))






FF_AddressView::FF_AddressView(FXComposite *p,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_addressTree(NULL)
,m_addAddressButton(NULL)
,m_editAddressButton(NULL)
,m_delAddressButton(NULL)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_addressTree=new FF_AddressTree(this, this, ID_ADDRESS_TREE,
				   LAYOUT_SIDE_LEFT |
				   LAYOUT_FILL_X | LAYOUT_FILL_Y |
				   FOLDINGLIST_EXTENDEDSELECT |
				   FOLDINGLIST_SHOWS_LINES |
				   FOLDINGLIST_SHOWS_BOXES |
				   FOLDINGLIST_ROOT_BOXES |
				   FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_addressTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_addressTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_addAddressButton=new THEMEBUTTON(vf,
				     I18N("Add Address"),
				     NULL,
				     this, ID_BUTTON_ADDADDRESS,
				     BUTTON_NORMAL);
  m_editAddressButton=new THEMEBUTTON(vf,
				      I18N("Edit Address"),
				      NULL,
				      this, ID_BUTTON_EDITADDRESS,
				      BUTTON_NORMAL);
  m_delAddressButton=new THEMEBUTTON(vf,
				     I18N("Delete Address"),
				     NULL,
				     this, ID_BUTTON_DELADDRESS,
				     BUTTON_NORMAL);
}



FF_AddressView::FF_AddressView()
:FXHorizontalFrame()
,m_addressTree(NULL)
,m_addAddressButton(NULL)
,m_editAddressButton(NULL)
,m_delAddressButton(NULL)
{
}



FF_AddressView::~FF_AddressView() {
}



void FF_AddressView::create(){
  FXHorizontalFrame::create();
  m_addressTree->create();

  m_addAddressButton->create();
  m_editAddressButton->create();
  m_delAddressButton->create();
}



void FF_AddressView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_addressTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_addressTree->getHeaderSize(i));
    m_addressTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_addressTree->setHeaderArrowDir(i, TRUE);
      else
	m_addressTree->setHeaderArrowDir(i, FALSE);
    }
  }
}



void FF_AddressView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_addressTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_addressTree->getHeaderSize(i));
    b=m_addressTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_AddressView::toGui(const AE_ADDRESS_LIST *addresses) {
  const AE_ADDRESS *addr;

  addr=AE_Address_List_First(addresses);
  while(addr) {
    m_addressTree->addAddress(AE_Address_dup(addr));
    addr=AE_Address_List_Next(addr);
  }
}



void FF_AddressView::fromGui(AE_ADDRESS_LIST *addresses) {
  FXFoldingItem *item;

  item=m_addressTree->getFirstItem();
  while (item) {
    FF_AddressTreeItem *grItem;

    grItem=dynamic_cast<FF_AddressTreeItem*>(item);
    if (grItem) {
      AE_ADDRESS *addr=grItem->getAddress();
      if (addr)
	AE_Address_List_Add(AE_Address_dup(addr), addresses);
    }
    item=item->getBelow();
  }
}



long FF_AddressView::onCmdAddAddress(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ADDRESS *addr;

    addr=AE_Address_new();
    if (FF_EditAddress::editAddress(app, this, I18N("Add Address"), addr)) {
      FF_AddressTreeItem *item;

      m_addressTree->addAddress(addr);
      item=m_addressTree->findAddressItem(addr);
      assert(item);
      m_addressTree->makeItemVisible(item);
      item->setSelected(TRUE);
    }
    else {
      AE_Address_free(addr);
    }
  }

  return 1;
}



long FF_AddressView::onCmdEditAddress(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    FXFoldingItem *item=m_addressTree->getCurrentItem();
    if (item) {
      FF_AddressTreeItem *addrItem=dynamic_cast<FF_AddressTreeItem*>(item);
      if (addrItem) {
	AE_ADDRESS *addr;

	addr=addrItem->getAddress();
	if (addr) {
	  if (FF_EditAddress::editAddress(app, this, I18N("Edit Address"), addr)) {
	    DBG_ERROR(0, "Modified address");
	    addrItem->rebuild();
	    m_addressTree->updateItem(addrItem);
	  }
	}
      }
    }
  }

  return 1;
}



long FF_AddressView::onCmdDelAddress(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_AddressView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editAddressButton->enable();
        m_delAddressButton->enable();
      }
      else {
        m_editAddressButton->disable();
        m_delAddressButton->disable();
      }
    }
    else {
      m_editAddressButton->disable();
      m_delAddressButton->disable();
    }
  }
  return 1;
}



void FF_AddressView::clear() {
  m_addressTree->clearItems();
}



void FF_AddressView::sortAddresss() {
  m_addressTree->sortItems();
}




