/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_bankaccountview.hpp"
#include "ff_bankaccounttree.hpp"
#include "ff_app.hpp"
#include "ff_editbankaccgroup.hpp"
#include "ff_editbankaccount.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_BankAccountView) FF_BankAccountViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_BankAccountView::ID_BUTTON_ADDACCOUNT,
            FF_BankAccountView::onCmdAddAccount),
  FXMAPFUNC(SEL_COMMAND, FF_BankAccountView::ID_BUTTON_EDITACCOUNT,
            FF_BankAccountView::onCmdEditAccount),
  FXMAPFUNC(SEL_COMMAND, FF_BankAccountView::ID_BUTTON_DELACCOUNT,
            FF_BankAccountView::onCmdDelAccount),

  FXMAPFUNC(SEL_CHANGED, FF_BankAccountView::ID_ACCOUNT_TREE,
            FF_BankAccountView::onTreeChanged),

};



FXIMPLEMENT(FF_BankAccountView, FXHorizontalFrame,
            FF_BankAccountViewMap, ARRAYNUMBER(FF_BankAccountViewMap))






FF_BankAccountView::FF_BankAccountView(FXComposite *p,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_bankAccountTree(NULL)
,m_addAccountButton(NULL)
,m_editAccountButton(NULL)
,m_delAccountButton(NULL)
,m_isEditing(false)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_bankAccountTree=new FF_BankAccountTree(this, this, ID_ACCOUNT_TREE,
                                           LAYOUT_SIDE_LEFT |
                                           LAYOUT_FILL_X | LAYOUT_FILL_Y |
                                           FOLDINGLIST_EXTENDEDSELECT |
                                           FOLDINGLIST_SHOWS_LINES |
                                           FOLDINGLIST_SHOWS_BOXES |
                                           FOLDINGLIST_ROOT_BOXES |
                                           FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_bankAccountTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_bankAccountTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_addAccountButton=new THEMEBUTTON(vf,
				     I18N("Add Account"),
				     NULL,
				     this, ID_BUTTON_ADDACCOUNT,
				     BUTTON_NORMAL);
  m_editAccountButton=new THEMEBUTTON(vf,
				      I18N("Edit Account"),
				      NULL,
				      this, ID_BUTTON_EDITACCOUNT,
				      BUTTON_NORMAL);
  m_delAccountButton=new THEMEBUTTON(vf,
				     I18N("Delete Account"),
				     NULL,
				     this, ID_BUTTON_DELACCOUNT,
				     BUTTON_NORMAL);
}



FF_BankAccountView::FF_BankAccountView()
:FXHorizontalFrame()
,m_bankAccountTree(NULL)
,m_addAccountButton(NULL)
,m_editAccountButton(NULL)
,m_delAccountButton(NULL)
,m_isEditing(false)
{
}



FF_BankAccountView::~FF_BankAccountView() {
}



void FF_BankAccountView::create(){
  FXHorizontalFrame::create();
  m_bankAccountTree->create();

  m_addAccountButton->create();
  m_editAccountButton->create();
  m_delAccountButton->create();
}



void FF_BankAccountView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_bankAccountTree->getHeaderSize(i));
    m_bankAccountTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_bankAccountTree->setHeaderArrowDir(i, TRUE);
      else
	m_bankAccountTree->setHeaderArrowDir(i, FALSE);
    }
  }
}



void FF_BankAccountView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_bankAccountTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_bankAccountTree->getHeaderSize(i));
    b=m_bankAccountTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_BankAccountView::toGui(const AE_BANKACCOUNT_LIST *bankAccounts) {
  const AE_BANKACCOUNT *ba;

  ba=AE_BankAccount_List_First(bankAccounts);
  while(ba) {
    m_bankAccountTree->addAccount(AE_BankAccount_dup(ba), NULL, NULL);
    ba=AE_BankAccount_List_Next(ba);
  }
}



void FF_BankAccountView::fromGui(AE_BANKACCOUNT_LIST *bankAccounts) {
  FXFoldingItem *item;

  item=m_bankAccountTree->getFirstItem();
  while (item) {
    FF_BankAccountTreeItem *grItem;

    grItem=dynamic_cast<FF_BankAccountTreeItem*>(item);
    if (grItem) {
      AE_BANKACCOUNT *ba=grItem->getBankAccount();
      if (ba)
	AE_BankAccount_List_Add(AE_BankAccount_dup(ba), bankAccounts);
    }
    item=item->getBelow();
  }
}



long FF_BankAccountView::onCmdAddAccount(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_BANKACCOUNT *ba;
    FXuint opts=DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE;

    ba=AE_BankAccount_new();
    AE_BankAccount_SetGroupId(ba, 0);
    if (options & FF_BANKACCOUNTVIEW_OPTS_SHOWGROUP)
      opts|=FF_EditBankAccount::FF_EDITBANKACCOUNT_OPTS_SHOWGROUP;
    m_isEditing=true;
    if (FF_EditBankAccount::editBankAccount(app, this,
					    I18N("Add Bank Account"),
                                            opts,
					    ba)) {
      FF_BankAccountTreeItem *item;

      if (target)
	target->tryHandle(this, FXSEL(SEL_INSERTED, message), (void*) ba);
      m_bankAccountTree->addAccount(ba, NULL, NULL);
      item=m_bankAccountTree->findBankAccountItem(ba);
      assert(item);
      m_bankAccountTree->makeItemVisible(item);
      item->setSelected(TRUE);
    }
    else {
      AE_BankAccount_free(ba);
    }
    m_isEditing=false;
  }

  return 1;
}



long FF_BankAccountView::onCmdEditAccount(FXObject*, FXSelector, void*) {
  AE_BANKACCOUNT *ba;

  ba=m_bankAccountTree->getCurrentBankAccount();
  if (ba) {
    FF_App *app;
    AE_BOOK *b;
    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      FXuint opts=DECOR_TITLE|DECOR_BORDER|DECOR_RESIZE;

      m_isEditing=true;
      if (options & FF_BANKACCOUNTVIEW_OPTS_SHOWGROUP)
	opts|=FF_EditBankAccount::FF_EDITBANKACCOUNT_OPTS_SHOWGROUP;
      if (FF_EditBankAccount::editBankAccount(app, this,
					      I18N("Edit Bank Account"),
                                              opts,
					      ba)) {
	if (target)
	  target->tryHandle(this, FXSEL(SEL_REPLACED, message), (void*) ba);
	DBG_ERROR(0, "Modified bank account");
      }
      m_isEditing=false;
    }
  }
  return 1;
}



long FF_BankAccountView::onCmdDelAccount(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_BankAccountView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editAccountButton->enable();
        m_delAccountButton->enable();
      }
      else {
        m_editAccountButton->disable();
        m_delAccountButton->disable();
      }
    }
    else {
      m_editAccountButton->disable();
      m_delAccountButton->disable();
    }
  }
  return 1;
}



void FF_BankAccountView::clear() {
  m_bankAccountTree->clearItems();
}



void FF_BankAccountView::sortAccounts() {
  m_bankAccountTree->sort();
}



AE_BANKACCOUNT *FF_BankAccountView::getCurrentBankAccount() {
  return m_bankAccountTree->getCurrentBankAccount();
}



AE_BANKACCOUNT *FF_BankAccountView::getFirstSelectedBankAccount() {
  return m_bankAccountTree->getFirstSelectedBankAccount();
}



void FF_BankAccountView::getSelectedBankAccounts(AE_BANKACCOUNT_LIST2 *al) {
  m_bankAccountTree->getSelectedBankAccounts(al);
}


void FF_BankAccountView::jobListUpdated() {
  // TODO: update flags of account entries
}



