/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_groupview.hpp"
#include "ff_grouptree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_GroupView) FF_GroupViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_GroupView::ID_BUTTON_ADDGROUP,
            FF_GroupView::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FF_GroupView::ID_BUTTON_EDITGROUP,
            FF_GroupView::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FF_GroupView::ID_BUTTON_DELGROUP,
            FF_GroupView::onCmdDelGroup),

  FXMAPFUNC(SEL_CHANGED, FF_GroupView::ID_GROUP_TREE,
            FF_GroupView::onTreeChanged),

};



FXIMPLEMENT(FF_GroupView, FXHorizontalFrame,
            FF_GroupViewMap, ARRAYNUMBER(FF_GroupViewMap))






FF_GroupView::FF_GroupView(AE_BOOK_TABLE_TYPE tt,
			   FXComposite *p,
			   FXuint opts,
			   FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_groupTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_type(tt)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_groupTree=new FF_GroupTree(this, this, ID_GROUP_TREE,
			       LAYOUT_SIDE_LEFT |
			       LAYOUT_FILL_X | LAYOUT_FILL_Y |
			       FOLDINGLIST_EXTENDEDSELECT |
			       FOLDINGLIST_SHOWS_LINES |
			       FOLDINGLIST_SHOWS_BOXES |
			       FOLDINGLIST_ROOT_BOXES |
			       FRAME_SUNKEN|FRAME_THICK);
  m_groupTree->appendHeader(I18N("Group Name"));
  m_groupTree->setHeaderSize(0, 100);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_groupTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_groupTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_addGroupButton=new THEMEBUTTON(vf,
				   I18N("Add Group"),
				   NULL,
				   this, ID_BUTTON_ADDGROUP,
				   BUTTON_NORMAL);
  m_editGroupButton=new THEMEBUTTON(vf,
				    I18N("Edit Group"),
				    NULL,
				    this, ID_BUTTON_EDITGROUP,
				    BUTTON_NORMAL);
  m_delGroupButton=new THEMEBUTTON(vf,
				   I18N("Delete Group"),
				   NULL,
				   this, ID_BUTTON_DELGROUP,
				   BUTTON_NORMAL);
}



FF_GroupView::FF_GroupView()
:FXHorizontalFrame()
,m_groupTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_type(AE_Book_TableType_None)
{
}



FF_GroupView::~FF_GroupView() {
}



void FF_GroupView::create(){
  FXHorizontalFrame::create();
  m_groupTree->create();

  m_addGroupButton->create();
  m_editGroupButton->create();
  m_delGroupButton->create();
}



void FF_GroupView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_groupTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_groupTree->getHeaderSize(i));
    m_groupTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_groupTree->setHeaderArrowDir(i, TRUE);
      else
	m_groupTree->setHeaderArrowDir(i, FALSE);
    }
  }
}



void FF_GroupView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_groupTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_groupTree->getHeaderSize(i));
    b=m_groupTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_GroupView::reload() {
  FF_App *app;
  AE_BOOK *b;
  AE_GROUP *gr;
  AQDB_ID currentGrId=0;

  gr=m_groupTree->getCurrentGroup();
  if (gr)
    currentGrId=AE_Group_GetId(gr);

  m_groupTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP_TREE *gt;
    int rv;
    int i;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, m_type, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      m_groupTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);

    for (i=0; i<m_groupTree->getNumHeaders(); i++) {
      FXbool b;

      b=m_groupTree->getHeaderArrowDir(i);
      if (b!=MAYBE) {
	if (b==TRUE)
	  m_groupTree->sortByColumn(i, true);
	else
	  m_groupTree->sortByColumn(i, false);
	break;
      }
    }

    m_groupTree->expandItems();

    if (currentGrId!=0) {
      FF_GroupTreeItem *item=m_groupTree->findGroupItem(currentGrId);
      if (item) {
	item->setSelected(true);
	m_groupTree->makeItemVisible(item);
	m_groupTree->setCurrentItem(item);
      }
      else {
	DBG_ERROR(0, "Item not found");
      }
    }
  }
}



long FF_GroupView::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;

    gr=AE_Group_new();
    AE_Group_SetId(gr, 0);
    if (FF_EditGroup::editGroup(app, this,
				I18N("Add Partner Group"),
				m_type,
				gr)) {
      int rv;

      rv=AE_Book_AddGroup(b, m_type, gr, 1);
      if (rv<0) {
	DBG_ERROR(0, "Unable to add group (%d)", rv);
      }
      app->signalDbReload(m_type);
      reload();
    }
    else {
      AE_Group_free(gr);
    }
  }

  return 1;
}



long FF_GroupView::onCmdEditGroup(FXObject*, FXSelector, void*) {
  AE_GROUP *gr;

  gr=m_groupTree->getCurrentGroup();
  if (gr) {
    FF_App *app;
    AE_BOOK *b;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();
    if (b) {
      if (FF_EditGroup::editGroup(app, this,
				  I18N("Edit Partner Group"),
                                  m_type,
				  gr)) {
	int rv;

	rv=AE_Book_WriteGroup(b, m_type, gr, 1);
	if (rv<0) {
	  DBG_ERROR(0, "Unable to edit group (%d)", rv);
	}
	app->signalDbReload(m_type);
        reload();
      }
    }
  }
  return 1;
}



long FF_GroupView::onCmdDelGroup(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_GroupView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editGroupButton->enable();
        m_delGroupButton->enable();
      }
      else {
        m_editGroupButton->disable();
        m_delGroupButton->disable();
      }
    }
    else {
      m_editGroupButton->disable();
      m_delGroupButton->disable();
    }
  }
  return 1;
}



void FF_GroupView::selectGroup(AQDB_ID id, bool b) {
  if (m_groupTree)
    m_groupTree->selectGroup(id, b);
}



void FF_GroupView::makeGroupVisible(AQDB_ID id) {
  if (m_groupTree)
    m_groupTree->makeGroupVisible(id);
}



AE_GROUP *FF_GroupView::getCurrentGroup() {
  if (m_groupTree)
    return m_groupTree->getCurrentGroup();
  else
    return NULL;
}



AE_GROUP *FF_GroupView::getFirstSelectedGroup() {
  if (m_groupTree)
    return m_groupTree->getFirstSelectedGroup();
  else
    return NULL;
}









