/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_printdocview.hpp"
#include "ff_printdoctree.hpp"
#include "ff_app.hpp"
#include "ff_edit_printdoc.hpp"

#include <aqfinance/print/af_print.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/directory.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_PrintDocView) FF_PrintDocViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_PrintDocView::ID_BUTTON_ADDDOC,
            FF_PrintDocView::onCmdAddDoc),
  FXMAPFUNC(SEL_COMMAND, FF_PrintDocView::ID_BUTTON_EDITDOC,
            FF_PrintDocView::onCmdEditDoc),
  FXMAPFUNC(SEL_COMMAND, FF_PrintDocView::ID_BUTTON_DELDOC,
            FF_PrintDocView::onCmdDelDoc),

  FXMAPFUNC(SEL_CHANGED, FF_PrintDocView::ID_DOC_TREE,
            FF_PrintDocView::onTreeChanged),

};



FXIMPLEMENT(FF_PrintDocView, FXHorizontalFrame,
            FF_PrintDocViewMap, ARRAYNUMBER(FF_PrintDocViewMap))






FF_PrintDocView::FF_PrintDocView(FXComposite *p,
				 FXuint opts,
				 FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_printDocTree(NULL)
,m_addPrintDocButton(NULL)
,m_editPrintDocButton(NULL)
,m_delPrintDocButton(NULL)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_printDocTree=new FF_PrintDocTree(this, this, ID_DOC_TREE,
				     LAYOUT_SIDE_LEFT |
				     LAYOUT_FILL_X | LAYOUT_FILL_Y |
				     FOLDINGLIST_BROWSESELECT |
				     FOLDINGLIST_SHOWS_LINES |
				     FOLDINGLIST_SHOWS_BOXES |
				     FOLDINGLIST_ROOT_BOXES |
				     FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_printDocTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_printDocTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);
  m_addPrintDocButton=new THEMEBUTTON(vf,
				      I18N("Add"),
				      NULL,
				      this, ID_BUTTON_ADDDOC,
				      BUTTON_NORMAL);
  m_editPrintDocButton=new THEMEBUTTON(vf,
				       I18N("Edit"),
				       NULL,
				       this, ID_BUTTON_EDITDOC,
				       BUTTON_NORMAL);
  m_delPrintDocButton=new THEMEBUTTON(vf,
				      I18N("Delete"),
				      NULL,
				      this, ID_BUTTON_DELDOC,
				      BUTTON_NORMAL);
}



FF_PrintDocView::FF_PrintDocView()
:FXHorizontalFrame()
,m_printDocTree(NULL)
,m_addPrintDocButton(NULL)
,m_editPrintDocButton(NULL)
,m_delPrintDocButton(NULL)
{
}



FF_PrintDocView::~FF_PrintDocView() {
}



void FF_PrintDocView::create(){
  FXHorizontalFrame::create();
  m_printDocTree->create();

  m_addPrintDocButton->create();
  m_editPrintDocButton->create();
  m_delPrintDocButton->create();
}



void FF_PrintDocView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_printDocTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_printDocTree->getHeaderSize(i));
    m_printDocTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      if (sortUp)
	m_printDocTree->setHeaderArrowDir(i, TRUE);
      else
	m_printDocTree->setHeaderArrowDir(i, FALSE);
    }
  }
}



void FF_PrintDocView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_printDocTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_printDocTree->getHeaderSize(i));
    b=m_printDocTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_PrintDocView::toGui(const AF_PRINT_DOC_LIST *docs) {
  const AF_PRINT_DOC *doc;

  doc=AF_PrintDoc_List_First(docs);
  while(doc) {
    m_printDocTree->addDoc(AF_PrintDoc_dup(doc));
    doc=AF_PrintDoc_List_Next(doc);
  }
}



void FF_PrintDocView::fromGui(AF_PRINT_DOC_LIST *docs) {
  FXFoldingItem *item;

  item=m_printDocTree->getFirstItem();
  while (item) {
    FF_PrintDocTreeItem *grItem;

    grItem=dynamic_cast<FF_PrintDocTreeItem*>(item);
    if (grItem) {
      AF_PRINT_DOC *doc=grItem->getDoc();
      if (doc)
	AF_PrintDoc_List_Add(AF_PrintDoc_dup(doc), docs);
    }
    item=item->getBelow();
  }
}



long FF_PrintDocView::onCmdAddDoc(FXObject*, FXSelector, void*) {
  FF_App *app;
  AF_PRINT_DOC *doc;

  app=dynamic_cast<FF_App*>(getApp());
  if (app==NULL) {
    DBG_ERROR(0, "No FF_App???");
    abort();
  }

  doc=AF_Print_GenerateTemplate_A4(300.0);
  if (FF_EditPrintDoc::edit(app, this, I18N("Add a Document Template"), doc)) {
    FF_PrintDocTreeItem *item;
    const char *s;
    GWEN_BUFFER *tbuf;
    GWEN_DB_NODE *db;
    int rv;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    s=AE_GetUserDataFolder();
    GWEN_Buffer_AppendString(tbuf, s);
    GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S "templates" GWEN_DIR_SEPARATOR_S "printdocs");
    rv=GWEN_Directory_GetPath(GWEN_Buffer_GetStart(tbuf), GWEN_PATH_FLAGS_CHECKROOT);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("IO Error"),
			  I18N("Error accessing path [%s]"),
			  GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
      AF_PrintDoc_free(doc);
      return 1;
    }

    GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);
    s=AF_PrintDoc_GetName(doc);
    if (s && *s) {
      GWEN_Text_EscapeToBuffer(s, tbuf);
    }
    else {
      // TODO: error message
      DBG_ERROR(AE_LOGDOMAIN, "No name");
    }
    GWEN_Buffer_AppendString(tbuf, ".db");

    s=GWEN_Buffer_GetStart(tbuf);
    db=GWEN_DB_Group_new("doc");
    AF_PrintDoc_toDb(doc, db);
    rv=GWEN_DB_WriteFile(db, s, GWEN_DB_FLAGS_DEFAULT);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      FXMessageBox::error(this, MBOX_OK,
			  I18N("IO Error"),
			  I18N("Error writing document template to file [%s]: %d"),
			  s, rv);
      GWEN_DB_Group_free(db);
      GWEN_Buffer_free(tbuf);
      AF_PrintDoc_free(doc);
      return 1;
    }
    GWEN_DB_Group_free(db);

    AF_PrintDoc_SetFileName(doc, s);
    GWEN_Buffer_free(tbuf);

    m_printDocTree->addDoc(doc);
    item=m_printDocTree->findDocItem(AF_PrintDoc_GetCategory(doc), AF_PrintDoc_GetName(doc));
    assert(item);
    m_printDocTree->makeItemVisible(item);
    m_printDocTree->unselectAll();
    m_printDocTree->setCurrentItem(item);
    item->setSelected(TRUE);
  }
  else {
    AF_PrintDoc_free(doc);
  }
  return 1;
}



long FF_PrintDocView::onCmdEditDoc(FXObject*, FXSelector, void*) {
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());
  FXFoldingItem *item=m_printDocTree->getCurrentItem();
  if (item) {
    FF_PrintDocTreeItem *docItem=dynamic_cast<FF_PrintDocTreeItem*>(item);
    if (docItem) {
      AF_PRINT_DOC *doc;

      doc=docItem->getDoc();
      if (doc) {
	if (FF_EditPrintDoc::edit(app, this, I18N("Edit Existing Document Template"), doc)) {
	  const char *s;

	  DBG_ERROR(0, "Modified doc");
	  s=AF_PrintDoc_GetFileName(doc);
	  if (s && *s) {
	    GWEN_DB_NODE *db;
            int rv;

	    db=GWEN_DB_Group_new("doc");
	    AF_PrintDoc_toDb(doc, db);
	    rv=GWEN_DB_WriteFile(db, s, GWEN_DB_FLAGS_DEFAULT);
	    if (rv<0) {
	      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
	      FXMessageBox::error(this, MBOX_OK,
				  I18N("IO Error"),
				  I18N("Error writing document template to file [%s]: %d"),
				  s, rv);
	    }
	    GWEN_DB_Group_free(db);
	  }
	  else {
            DBG_ERROR(AE_LOGDOMAIN, "No filename");
	  }

	  docItem->rebuild();
	  m_printDocTree->updateItem(docItem);
	}
      }
    }
  }
  return 1;
}



long FF_PrintDocView::onCmdDelDoc(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_PrintDocView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editPrintDocButton->enable();
        m_delPrintDocButton->enable();
      }
      else {
        m_editPrintDocButton->disable();
        m_delPrintDocButton->disable();
      }
    }
    else {
      m_editPrintDocButton->disable();
      m_delPrintDocButton->disable();
    }
  }
  return 1;
}



void FF_PrintDocView::clear() {
  m_printDocTree->clearItems();
}



void FF_PrintDocView::addDoc(AF_PRINT_DOC *doc, FXbool notify) {
  m_printDocTree->addDoc(doc, notify);
}



void FF_PrintDocView::sortDocs() {
  m_printDocTree->sortItems();
}




