/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_taxview.hpp"
#include "ff_taxtree.hpp"
#include "ff_app.hpp"
#include "ff_editgroup.hpp"
#include "ff_edittax.hpp"

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_TaxView) FF_TaxViewMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_TaxView::ID_BUTTON_ADDTAX,
            FF_TaxView::onCmdAddTax),
  FXMAPFUNC(SEL_COMMAND, FF_TaxView::ID_BUTTON_EDITTAX,
            FF_TaxView::onCmdEditTax),
  FXMAPFUNC(SEL_COMMAND, FF_TaxView::ID_BUTTON_DELTAX,
            FF_TaxView::onCmdDelTax),

  FXMAPFUNC(SEL_COMMAND, FF_TaxView::ID_BUTTON_ADDGROUP,
            FF_TaxView::onCmdAddGroup),
  FXMAPFUNC(SEL_COMMAND, FF_TaxView::ID_BUTTON_EDITGROUP,
            FF_TaxView::onCmdEditGroup),
  FXMAPFUNC(SEL_COMMAND, FF_TaxView::ID_BUTTON_DELGROUP,
            FF_TaxView::onCmdDelGroup),

  FXMAPFUNC(SEL_CHANGED, FF_TaxView::ID_TAX_TREE,
            FF_TaxView::onTreeChanged),

};



FXIMPLEMENT(FF_TaxView, FXHorizontalFrame,
            FF_TaxViewMap, ARRAYNUMBER(FF_TaxViewMap))






FF_TaxView::FF_TaxView(FXComposite *p,
		       FXuint opts,
		       FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h)
,m_taxTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addTaxButton(NULL)
,m_editTaxButton(NULL)
,m_delTaxButton(NULL)
{
  FXVerticalFrame *vf;
  FXIcon *ic;
  FF_App *app;

  flags|=FLAG_ENABLED|FLAG_SHOWN;
  app=dynamic_cast<FF_App*>(getApp());

  m_taxTree=new FF_TaxTree(this, this, ID_TAX_TREE,
				   LAYOUT_SIDE_LEFT |
				   LAYOUT_FILL_X | LAYOUT_FILL_Y |
				   FOLDINGLIST_EXTENDEDSELECT |
				   FOLDINGLIST_SHOWS_LINES |
				   FOLDINGLIST_SHOWS_BOXES |
				   FOLDINGLIST_ROOT_BOXES |
				   FRAME_SUNKEN|FRAME_THICK);
  ic=app->getIcon("minifolderopen");
  if (ic)
    m_taxTree->setIconOpen(ic);
  ic=app->getIcon("minifolder");
  if (ic)
    m_taxTree->setIconClosed(ic);

  vf=new FXVerticalFrame(this, LAYOUT_FILL_Y | PACK_UNIFORM_WIDTH);

  m_addGroupButton=new THEMEBUTTON(vf,
				   I18N("Add Group"),
				   NULL,
				   this, ID_BUTTON_ADDGROUP,
				   BUTTON_NORMAL);
  m_editGroupButton=new THEMEBUTTON(vf,
				    I18N("Edit Group"),
				    NULL,
				    this, ID_BUTTON_EDITGROUP,
				    BUTTON_NORMAL);
  m_delGroupButton=new THEMEBUTTON(vf,
				   I18N("Delete Group"),
				   NULL,
				   this, ID_BUTTON_DELGROUP,
				   BUTTON_NORMAL);

  new FXLabel(vf, "");
  new FXHorizontalSeparator(vf, SEPARATOR_GROOVE|LAYOUT_FILL_X);
  new FXLabel(vf, "");

  m_addTaxButton=new THEMEBUTTON(vf,
				 I18N("Add Tax"),
				 NULL,
				 this, ID_BUTTON_ADDTAX,
				 BUTTON_NORMAL);
  m_editTaxButton=new THEMEBUTTON(vf,
				  I18N("Edit Tax"),
				  NULL,
				  this, ID_BUTTON_EDITTAX,
				  BUTTON_NORMAL);
  m_delTaxButton=new THEMEBUTTON(vf,
				 I18N("Delete Tax"),
				 NULL,
				 this, ID_BUTTON_DELTAX,
				 BUTTON_NORMAL);
}



FF_TaxView::FF_TaxView()
:FXHorizontalFrame()
,m_taxTree(NULL)
,m_addGroupButton(NULL)
,m_editGroupButton(NULL)
,m_delGroupButton(NULL)
,m_addTaxButton(NULL)
,m_editTaxButton(NULL)
,m_delTaxButton(NULL)
{
}



FF_TaxView::~FF_TaxView() {
}



void FF_TaxView::create(){
  FXHorizontalFrame::create();
  m_taxTree->create();

  m_addTaxButton->create();
  m_editTaxButton->create();
  m_delTaxButton->create();
}



void FF_TaxView::init(GWEN_DB_NODE *db) {
  int i;
  int sortByColumn;
  int sortUp;

  sortByColumn=GWEN_DB_GetIntValue(db, "sortByColumn", 0, -1);
  sortUp=GWEN_DB_GetIntValue(db, "sortUp", 0, 0);

  for (i=0; i<m_taxTree->getNumHeaders(); i++) {
    int j=GWEN_DB_GetIntValue(db, "columns", i, m_taxTree->getHeaderSize(i));
    m_taxTree->setHeaderSize(i, j);
    if (i==sortByColumn) {
      m_taxTree->setHeaderArrowDir(i, sortUp?TRUE:FALSE);
    }
  }
}



void FF_TaxView::fini(GWEN_DB_NODE *db) {
  int i;

  for (i=0; i<m_taxTree->getNumHeaders(); i++) {
    FXbool b;

    GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_DEFAULT, "columns",
			m_taxTree->getHeaderSize(i));
    b=m_taxTree->getHeaderArrowDir(i);
    if (b!=MAYBE) {
      GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortByColumn", i);
      if (b==TRUE)
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 1);
      else
	GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "sortUp", 0);
    }
  }
}



void FF_TaxView::toGui(const AE_TAX_LIST *taxes) {
  FF_App *app;
  AE_BOOK *b;

  m_taxTree->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    const AE_TAX *tax;
    AE_GROUP_TREE *gt;
    int rv;
    FXFoldingItem *firstItem;

    gt=AE_Group_Tree_new();
    rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_TaxGroup, gt);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
    else {
      m_taxTree->addGroups(gt);
    }
    AE_Group_Tree_free(gt);

    tax=AE_Tax_List_First(taxes);
    while(tax) {
      m_taxTree->addTax(AE_Tax_dup(tax));
      tax=AE_Tax_List_Next(tax);
    }
    m_taxTree->expandItems();

    firstItem=m_taxTree->getFirstItem();
    onTreeChanged(NULL, FXSEL(SEL_CHANGED, ID_TAX_TREE), firstItem);
  }
}



void FF_TaxView::fromGui(AE_TAX_LIST *taxes) {
  FXFoldingItem *item;

  item=m_taxTree->getFirstItem();
  while (item) {
    FF_TaxTreeItem *grItem;

    grItem=dynamic_cast<FF_TaxTreeItem*>(item);
    if (grItem) {
      AE_TAX *tax=grItem->getTax();
      if (tax)
	AE_Tax_List_Add(AE_Tax_dup(tax), taxes);
    }
    item=item->getBelow();
  }
}



long FF_TaxView::onCmdAddTax(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_TAX *tax;

    tax=AE_Tax_new();
    AE_Tax_SetGroupId(tax, 0);
    if (FF_EditTax::editTax(app, this, I18N("Add Tax"), tax)) {
      FF_TaxTreeItem *item;

      if (target)
	target->tryHandle(this, FXSEL(SEL_INSERTED, message), (void*) tax);
      m_taxTree->addTax(tax);
      item=m_taxTree->findTaxItem(tax);
      assert(item);
      m_taxTree->makeItemVisible(item);
      m_taxTree->unselectAll();
      item->setSelected(TRUE);
      m_taxTree->setCurrentItem(item, TRUE);
      m_taxTree->sort();
   }
    else {
      AE_Tax_free(tax);
    }
  }

  return 1;
}



long FF_TaxView::onCmdEditTax(FXObject*, FXSelector, void*) {
  FXFoldingItem *item;

  item=m_taxTree->getCurrentItem();
  if (item) {
    FF_TaxTreeItem *ptitem;

    ptitem=dynamic_cast<FF_TaxTreeItem*>(item);
    if (ptitem) {
      AE_TAX *tax;

      tax=ptitem->getTax();
      if (tax) {
	FF_App *app;
	AE_BOOK *b;

	app=dynamic_cast<FF_App*>(getApp());
	b=app->getBook();
	if (b) {
	  AQDB_ID oldGroupId;

	  if (target)
	    target->tryHandle(this, FXSEL(SEL_OPENED, message), (void*) tax);

	  oldGroupId=AE_Tax_GetGroupId(tax);

	  if (FF_EditTax::editTax(app, this, I18N("Edit Tax"), tax)) {
	    if (target)
	      target->tryHandle(this, FXSEL(SEL_REPLACED, message), (void*) tax);
	    DBG_ERROR(0, "Modified tax");
	    ptitem->rebuild();
	    m_taxTree->updateItem(ptitem);
	    if (oldGroupId!=AE_Tax_GetGroupId(tax)) {
	      FF_GroupTreeItem *gitem=NULL;

	      /* re-group */
	      gitem=m_taxTree->findGroupItem(AE_Tax_GetGroupId(tax));
	      m_taxTree->updateItem(ptitem);
	      m_taxTree->moveItem(NULL, gitem, ptitem);
	      m_taxTree->sort();
	      m_taxTree->makeItemVisible(ptitem);
	      m_taxTree->selectTax(tax, true);
	    }
	  }
	}
      }
    }
  }
  return 1;
}



long FF_TaxView::onCmdDelTax(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_TaxView::onCmdAddGroup(FXObject*, FXSelector, void*) {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_GROUP *gr;
    AE_GROUP *parent;

    gr=AE_Group_new();
    parent=m_taxTree->getCurrentGroup();
    if (parent)
      AE_Group_SetParentId(gr, AE_Group_GetId(parent));
    else
      AE_Group_SetParentId(gr, 0);

    if (FF_EditGroup::editGroup(app,
				this,
				I18N("Add Tax Group"),
				AE_Book_TableType_TaxGroup,
				gr)) {
      int rv;
      FF_GroupTreeItem *item;

      rv=AE_Book_AddGroup(b, AE_Book_TableType_TaxGroup, gr, 1);
      if (rv<0) {
	DBG_ERROR(0, "Unable to add group (%d)", rv);
	FXMessageBox::error(this, MBOX_OK,
			    I18N("Database Error"),
			    I18N("Unable to write object to database: %d"), rv);
      }
      m_taxTree->addGroup(gr);
      item=m_taxTree->findGroupItem(gr);
      assert(item);
      m_taxTree->makeItemVisible(item);
      m_taxTree->unselectAll();
      item->setSelected(TRUE);
      m_taxTree->setCurrentItem(item, TRUE);
      app->signalDbReload(AE_Book_TableType_TaxGroup);
    }
    else
      AE_Group_free(gr);
  }

  return 1;
}



long FF_TaxView::onCmdEditGroup(FXObject*, FXSelector, void*) {
  FF_GroupTreeItem *grItem;

  grItem=m_taxTree->getCurrentGroupItem();
  if (grItem) {
    AE_GROUP *gr;

    gr=grItem->getGroup();
    if (gr) {
      FF_App *app;
      AE_BOOK *b;

      app=dynamic_cast<FF_App*>(getApp());
      b=app->getBook();
      if (b) {
	if (FF_EditGroup::editGroup(app,
				    this,
				    I18N("Edit Tax Group"),
				    AE_Book_TableType_TaxGroup,
				    gr)) {
	  int rv;
	  FXString t;

	  rv=AE_Book_WriteGroup(b, AE_Book_TableType_TaxGroup, gr, 1);
	  if (rv<0) {
	    DBG_ERROR(0, "Unable to write group (%d)", rv);
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Database Error"),
				I18N("Unable to write object to database: %d"), rv);
	  }

	  t=FXString(AE_Group_GetName(gr));
	  grItem->setText(t);
	  m_taxTree->updateItem(grItem);
	  app->signalDbReload(AE_Book_TableType_TaxGroup);
	}
      }
    }
  }
  return 1;
}



long FF_TaxView::onCmdDelGroup(FXObject*, FXSelector, void*) {
  return 1;
}



long FF_TaxView::onTreeChanged(FXObject*, FXSelector, void* ptr) {
  if (ptr) {
    FF_GroupTreeItem *item=dynamic_cast<FF_GroupTreeItem*>((FXFoldingItem*) ptr);
    if (item) {
      if (item->isDataItem()) {
        m_editTaxButton->enable();
        m_delTaxButton->enable();

	m_editGroupButton->disable();
        m_delGroupButton->disable();
      }
      else {
        m_editTaxButton->disable();
        m_delTaxButton->disable();

        m_editGroupButton->enable();
        m_delGroupButton->enable();
      }
    }
    else {
      m_editTaxButton->disable();
      m_delTaxButton->disable();

      m_editGroupButton->disable();
      m_delGroupButton->disable();
    }
  }
  return 1;
}



void FF_TaxView::clear() {
  m_taxTree->clearItems();
}



void FF_TaxView::sortTaxes() {
  m_taxTree->sort();
}




