/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_accountedit.hpp"
#include "ff_app.hpp"
#include "ff_selectaccount.hpp"

#include <aqfinance/engine/book/ae_account.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_AccountEdit) FF_AccountEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_AccountEdit::ID_COMBOEDIT, FF_AccountEdit::onCmdCombo),
  FXMAPFUNC(SEL_CHANGED, FF_AccountEdit::ID_COMBOEDIT, FF_AccountEdit::onChgCombo),
  FXMAPFUNC(SEL_COMMAND, FF_AccountEdit::ID_BUTTON, FF_AccountEdit::onCmdButton),
  FXMAPFUNC(SEL_DESELECTED, FF_AccountEdit::ID_COMBOEDIT, FF_AccountEdit::onCmdCancel),
  FXMAPFUNC(SEL_FOCUS_SELF, 0, FF_AccountEdit::onFocusSelf),
  FXMAPFUNC(SEL_FOCUSIN, 0, FF_AccountEdit::onFocusIn),
};


// Object implementation
FXIMPLEMENT(FF_AccountEdit, FXHorizontalFrame, FF_AccountEditMap, ARRAYNUMBER(FF_AccountEditMap))



FF_AccountEdit::FF_AccountEdit(FXComposite *p,
                               FXObject* tgt,
                               FXSelector sel,
                               FXuint opts,
                               FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h, 0l, 0, 0, 0)
,m_accountTree(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
  FF_App *app;
  target=tgt;
  message=sel;

  app=dynamic_cast<FF_App*>(getApp());

  m_comboEdit=new FXXComboEdit(this, 1,
                               this, ID_COMBOEDIT,
                               LAYOUT_FILL_X);
  m_comboEdit->setSortFunc(FXList::ascendingCase);
  m_iconButton=app->getIcon("minifolderopen");
  if (m_iconButton) {
    m_button=new THEMEBUTTON(this, "", m_iconButton, this, ID_BUTTON);
  }
  else
    m_button=new THEMEBUTTON(this, "...", NULL, this, ID_BUTTON);
}



FF_AccountEdit::FF_AccountEdit()
:FXHorizontalFrame()
,m_accountTree(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
}



FF_AccountEdit::~FF_AccountEdit() {
  AE_Account_Tree_free(m_accountTree);
}



void FF_AccountEdit::create() {
  FXHorizontalFrame::create();
  if (m_iconButton)
    m_iconButton->create();
  m_comboEdit->create();
  m_button->create();
}



void FF_AccountEdit::enable() {
  FXHorizontalFrame::enable();
  m_comboEdit->enable();
  m_button->enable();
}



void FF_AccountEdit::disable() {
  FXHorizontalFrame::disable();
  m_comboEdit->disable();
  m_button->disable();
}



void FF_AccountEdit::setTipText(const FXString &text) {
  m_comboEdit->setTipText(text);
  m_button->setTipText(text);
}



void FF_AccountEdit::setFont(FXFont* fnt) {
  m_comboEdit->setFont(fnt);
  m_button->setFont(fnt);
}



void FF_AccountEdit::setBackColor(FXColor clr) {
  FXHorizontalFrame::setBackColor(clr);
  m_comboEdit->setBackColor(clr);
  m_button->setBackColor(clr);
}



void FF_AccountEdit::setTextColor(FXColor clr) {
  m_comboEdit->setTextColor(clr);
  m_button->setTextColor(clr);
}



void FF_AccountEdit::setSelBackColor(FXColor clr) {
  m_comboEdit->setSelBackColor(clr);
}



void FF_AccountEdit::setSelTextColor(FXColor clr) {
  m_comboEdit->setSelTextColor(clr);
}



void FF_AccountEdit::setText(const FXString& text) {
  m_comboEdit->setText(text);
}



FXString FF_AccountEdit::getText() const {
  return m_comboEdit->getText();
}



void FF_AccountEdit::selectAll() {
  m_comboEdit->selectAll();
}



void FF_AccountEdit::setAccountId(AQDB_ID id) {
  AE_ACCOUNT *acc;

  acc=AE_Account_Tree_GetById(m_accountTree, id);
  if (acc) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AE_Account_PathToBuffer(acc, tbuf);
    m_comboEdit->setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }
}



AQDB_ID FF_AccountEdit::getAccountId() {
  FXString str;

  str=m_comboEdit->getText();
  if (!str.empty()) {
    AE_ACCOUNT *acc;

    acc=AE_Account_Tree_GetByPath(m_accountTree, NULL, str.text());
    if (acc)
      return AE_Account_GetId(acc);
  }

  return 0;
}



long FF_AccountEdit::onCmdCombo(FXObject*, FXSelector, void*) {
  if (target && target->handle(this, FXSEL(SEL_COMMAND, message), NULL))
    return 1;
  return 1;
}



long FF_AccountEdit::onChgCombo(FXObject*, FXSelector, void*) {
  if (target && target->handle(this, FXSEL(SEL_CHANGED, message), NULL))
    return 1;
  return 1;
}



long FF_AccountEdit::onCmdButton(FXObject*, FXSelector, void*) {
  AQDB_ID id;

  /* TODO: get currently selected account */
  id=0;

  if (FF_SelectAccount::selectAccount(dynamic_cast<FF_App*>(getApp()),
                                      I18N("Select an Account"),
                                      id)) {
    AE_ACCOUNT *acc;

    reload();

    acc=AE_Account_Tree_GetFirst(m_accountTree);
    while(acc) {
      if (AE_Account_GetId(acc)==id) {
        GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
        AE_Account_PathToBuffer(acc, tbuf);
        setText(GWEN_Buffer_GetStart(tbuf));
        GWEN_Buffer_free(tbuf);
        if (target)
          target->handle(this, FXSEL(SEL_COMMAND, message), NULL);
        break;
      }
      acc=AE_Account_Tree_GetBelow(acc);
    }
  }

  return 1;
}



long FF_AccountEdit::onFocusSelf(FXObject *sender, FXSelector sel, void *ptr){
  return m_comboEdit->handle(sender,FXSEL(SEL_FOCUS_SELF,0),ptr);
}



long FF_AccountEdit::onFocusIn(FXObject *sender, FXSelector sel, void *ptr){
  FXHorizontalFrame::onFocusIn(sender, sel, ptr);
  m_comboEdit->setFocus();
  return 1;
}



long FF_AccountEdit::onCmdCancel(FXObject *sender, FXSelector sel, void *ptr){
  if (target && target->handle(this, FXSEL(SEL_DESELECTED, message), NULL))
    return 1;

  return 0;
}



void FF_AccountEdit::reload() {
  FF_App *app;
  int rv;

  app=dynamic_cast<FF_App*>(getApp());
  AE_Account_Tree_free(m_accountTree);
  m_accountTree=AE_Account_Tree_new();
  rv=AE_Book_ReadAccountTree(app->getBook(), AE_Book_TableType_Account, m_accountTree);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else {
    AE_ACCOUNT *acc;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    acc=AE_Account_Tree_GetFirst(m_accountTree);
    while(acc) {
      AE_Account_PathToBuffer(acc, tbuf);
      m_comboEdit->addEntry(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
      acc=AE_Account_Tree_GetBelow(acc);
    }
    GWEN_Buffer_free(tbuf);
    m_comboEdit->sortItems();
    m_comboEdit->reloadList();
  }
}






