/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_accounttree.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



#define SIDE_SPACING        4   // Spacing between side and item
#define ICON_SPACING        4   // Spacing between parent and child in x direction



FF_AccountTreeItem::FF_AccountTreeItem(FF_AccountTree *tree, AE_ACCOUNT *acc, void* ptr)
:FXXCheckTreeItem("", NULL, NULL, ptr)
,m_tree(tree)
,m_account(acc) {
  rebuild();
}



void FF_AccountTreeItem::rebuild() {
  FXString t;
  const char *s;
  AE_ACCOUNTBALANCE *bal;

  s=AE_Account_GetName(m_account);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  s=AE_Account_GetCode(m_account);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  s=AE_Account_GetDescription(m_account);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  bal=AE_Account_GetBranchBalance(m_account);
  if (bal) {
    const AQDB_VALUE *v;
    AQDB_VALUE *vcpy=NULL;

    v=AE_AccountBalance_GetValue(bal);
    if (v) {
      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AE_COMMODITY_LIST *cl=m_tree->getCommodities();
      int rv;

      if (AE_AccountType_NeedReverseSign(AE_Account_GetType(m_account))) {
	vcpy=AQDB_Value_dup(v);
	AQDB_Value_Negate(vcpy);
	v=vcpy;
      }

      rv=AE_Utils_AmountToString(v, AE_AccountBalance_GetCommodityId(bal), cl, tbuf);
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      t+=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);

      AQDB_Value_free(vcpy);
    }
  }

  setText(t);
}



FF_AccountTreeItem::~FF_AccountTreeItem() {
  if (m_account) {
    AE_Account_free(m_account);
    m_account=(AE_ACCOUNT*) -1;
  }
}



void FF_AccountTreeItem::drawColumn(const FXXCheckTree* list, FXDC &dc, FXint col,
				    FXint xx, FXint y, FXint w, FXint h,
				    const FXchar* string, FXuint length) const {
  if (col!=3)
    FXXCheckTreeItem::drawColumn(list, dc, col, xx, y, w, h, string, length);
  else {
    register FXFont *font=list->getFont();
    register FXint tw=font->getTextWidth(string, length);
    register FXint th=font->getFontHeight();
    register FXint yt=y+(h-th-4)/2;

    if (!isEnabled())
      dc.setForeground(makeShadowColor(list->getBackColor()));
    else if(isSelected())
      dc.setForeground(list->getSelTextColor());
    else {
      if (string[0]=='-')
	dc.setForeground(FXRGB(255, 0, 0));
      else
	dc.setForeground(list->getTextColor());
    }

    if(tw>w-4){
      FXint dw=font->getTextWidth("...",3);

      while((tw=font->getTextWidth(string, length))+dw>w-4 && length>1)
	length--;
      dc.setClipRectangle(xx, y, w, h);
      dc.drawText(xx+(w-tw)-2, yt+font->getFontAscent()+2, string, length);
      dc.drawText(xx+tw+(w-tw)-2, yt+font->getFontAscent()+2, "...", 3);
      dc.clearClipRectangle();
    }
    else
      dc.drawText(xx+(w-tw)-2, yt+font->getFontAscent()+2, string, length);
  }
}



FXDEFMAP(FF_AccountTree) FF_AccountTreeMap[]={
  FXMAPFUNC(SEL_COMMAND, FXFoldingList::ID_HEADER_CHANGE,
            FF_AccountTree::onHeaderClicked)
};


            
FXIMPLEMENT(FF_AccountTree, FXFoldingList,
            FF_AccountTreeMap, ARRAYNUMBER(FF_AccountTreeMap))




FF_AccountTree::FF_AccountTree(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
                               FXint x, FXint y, FXint w, FXint h)
:FXXCheckTree(p, tgt, sel, opts, x, y, w, h)
,m_commodities(NULL)
{
  int idx=0;
  flags|=FLAG_ENABLED | FLAG_SHOWN;


  appendHeader(I18N("Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Code"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Description"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Balance"));
  setHeaderSize(idx++, 100);
}



FF_AccountTree::FF_AccountTree()
:FXXCheckTree()
{
}



FF_AccountTree::~FF_AccountTree() {
  if (m_commodities)
    AE_Commodity_List_free(m_commodities);
}



void FF_AccountTree::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FF_AccountTree: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  if (1) {
    FXHeader *h;

    h=getHeader();
    if (h) {
      int j;

      for (i=0; i<indent+2; i++) fprintf(stderr, " ");
      fprintf(stderr, "Header: %d/%d %d/%d (%d %d)\n",
              h->getX(),
              h->getY(),
              h->getWidth(),
              h->getHeight(),
              h->getDefaultWidth(),
              h->getDefaultHeight());
      for (j=0; j<h->getNumItems(); j++) {
        for (i=0; i<indent+4; i++) fprintf(stderr, " ");
        fprintf(stderr, "%d %d: %s\n",
                h->getItemOffset(j), h->getItemSize(j), h->getItemText(j).text());
      }
    }
  }
}



FF_AccountTreeItem *FF_AccountTree::findAccountItem(FF_AccountTreeItem *accItem, AQDB_ID id) {
  AE_ACCOUNT *acc=accItem->getAccount();
  if (acc && AE_Account_GetId(acc)==id)
    return accItem;
  else {
    FXFoldingItem *item;

    item=accItem->getFirst();
    while(item) {
      accItem=dynamic_cast<FF_AccountTreeItem*>(item);
      if (accItem) {
	FF_AccountTreeItem *res=findAccountItem(accItem, id);
	if (res!=NULL)
	  return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_AccountTreeItem *FF_AccountTree::findAccountItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_AccountTreeItem *accItem;

    accItem=dynamic_cast<FF_AccountTreeItem*>(item);
    if (accItem) {
      AE_ACCOUNT *acc=accItem->getAccount();
      if (acc && AE_Account_GetId(acc)==id)
        return accItem;
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_AccountTree::addAccountWithChildren(FF_AccountTreeItem *itemFather,
                                            const AE_ACCOUNT *acc,
                                            AQDB_ID exceptId) {
  AE_ACCOUNT *accChild;
  FF_AccountTreeItem *item;

  if (exceptId==0 || AE_Account_GetId(acc)!=exceptId) {
    /* add this account */
    item=new FF_AccountTreeItem(this, AE_Account_dup(acc));
    appendItem(itemFather, item, false);

    /* add children of this account */
    accChild=AE_Account_Tree_GetFirstChild(acc);
    while(accChild) {
      addAccountWithChildren(item, accChild, exceptId);
      accChild=AE_Account_Tree_GetNext(accChild);
    }
  }
}



void FF_AccountTree::addAccounts(AE_ACCOUNT_TREE *gt, AQDB_ID exceptId) {
  AE_ACCOUNT *acc;

  acc=AE_Account_Tree_GetFirst(gt);
  while(acc) {
    addAccountWithChildren(NULL, acc, exceptId);
    acc=AE_Account_Tree_GetNext(acc);
  }
}



void FF_AccountTree::addSelectedAccountWithChildren(FF_AccountTreeItem *itemFather,
						    const AE_ACCOUNT *acc,
						    AQDB_ID exceptId) {
  const AE_ACCOUNT *accChild;
  FF_AccountTreeItem *item;

  if (exceptId==0 || AE_Account_GetId(acc)!=exceptId) {
    /* add this account */
    item=new FF_AccountTreeItem(this, AE_Account_dup(acc));
    appendItem(itemFather, item, false);

    /* add children of this account */
    accChild=AE_Account_Tree_GetFirstChild(acc);
    while(accChild) {
      if (AE_Account_GetRuntimeFlags(accChild) & AE_ACCOUNT_RTFLAGS_SELECTED)
	addSelectedAccountWithChildren(item, accChild, exceptId);
      accChild=AE_Account_Tree_GetNext(accChild);
    }
  }
}



void FF_AccountTree::addSelectedAccounts(const AE_ACCOUNT_TREE *gt, AQDB_ID exceptId) {
  const AE_ACCOUNT *acc;

  acc=AE_Account_Tree_GetFirst(gt);
  while(acc) {
    if (AE_Account_GetRuntimeFlags(acc) & AE_ACCOUNT_RTFLAGS_SELECTED)
      addSelectedAccountWithChildren(NULL, acc, exceptId);
    acc=AE_Account_Tree_GetNext(acc);
  }
}



AE_ACCOUNT *FF_AccountTree::getCurrentAccount() {
  FXFoldingItem *item;

  item=getCurrentItem();

  while (item) {
    FF_AccountTreeItem *accItem=dynamic_cast<FF_AccountTreeItem*>(item);
    if (accItem)
      return accItem->getAccount();
    item=item->getParent();
  }

  return NULL;
}



void FF_AccountTree::setCurrentAccount(AQDB_ID id, bool b) {
  FF_AccountTreeItem *item;

  item=findAccountItem(id);
  if (item)
    setCurrentItem(item, b);
}



AE_ACCOUNT *FF_AccountTree::getFirstSelectedAccount() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_AccountTreeItem *accItem=dynamic_cast<FF_AccountTreeItem*>(item);
      if (accItem)
        return accItem->getAccount();
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_AccountTree::selectAccount(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_AccountTreeItem *accItem=dynamic_cast<FF_AccountTreeItem*>(item);
    if (accItem) {
      AE_ACCOUNT *acc=accItem->getAccount();
      if (AE_Account_GetId(acc)==id) {
        item->setSelected(b);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_AccountTree::makeAccountVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_AccountTreeItem *accItem=dynamic_cast<FF_AccountTreeItem*>(item);
    if (accItem) {
      AE_ACCOUNT *acc=accItem->getAccount();
      if (AE_Account_GetId(acc)==id) {
        makeItemVisible(item);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_AccountTree::expandItems() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    expandTree(item, false);
    item=item->getBelow();
  }
}



void FF_AccountTree::create() {
  FXFoldingList::create();
}



static FXint sortByColumnNameUp(const FXFoldingItem *a, const FXFoldingItem *b) {
  const FF_AccountTreeItem *aa=dynamic_cast<const FF_AccountTreeItem*>(a);
  const FF_AccountTreeItem *bb=dynamic_cast<const FF_AccountTreeItem*>(b);
  const AE_ACCOUNT *acc1=aa->getAccount();
  const AE_ACCOUNT *acc2=bb->getAccount();
  const char *s1=AE_Account_GetName(acc1);
  const char *s2=AE_Account_GetName(acc2);

  if (s1 && s2)
    return strcasecmp(s1, s2);
  else if (s1) {
    return 1;
  }
  else if (s2) {
    return -1;
  }

  return 0;
}



static FXint sortByColumnNameDown(const FXFoldingItem *a, const FXFoldingItem *b) {
  const FF_AccountTreeItem *aa=dynamic_cast<const FF_AccountTreeItem*>(a);
  const FF_AccountTreeItem *bb=dynamic_cast<const FF_AccountTreeItem*>(b);
  const AE_ACCOUNT *acc1=aa->getAccount();
  const AE_ACCOUNT *acc2=bb->getAccount();
  const char *s1=AE_Account_GetName(acc1);
  const char *s2=AE_Account_GetName(acc2);

  if (s1 && s2)
    return strcasecmp(s2, s1);
  else if (s1) {
    return -1;
  }
  else if (s2) {
    return 1;
  }

  return 0;
}



static FXint sortByColumnCodeUp(const FXFoldingItem *a, const FXFoldingItem *b) {
  const FF_AccountTreeItem *aa=dynamic_cast<const FF_AccountTreeItem*>(a);
  const FF_AccountTreeItem *bb=dynamic_cast<const FF_AccountTreeItem*>(b);
  const AE_ACCOUNT *acc1=aa->getAccount();
  const AE_ACCOUNT *acc2=bb->getAccount();
  const char *s1=AE_Account_GetCode(acc1);
  const char *s2=AE_Account_GetCode(acc2);

  if (s1 && s2)
    return strcasecmp(s1, s2);
  else if (s1) {
    return 1;
  }
  else if (s2) {
    return -1;
  }

  return 0;
}



static FXint sortByColumnCodeDown(const FXFoldingItem *a, const FXFoldingItem *b) {
  const FF_AccountTreeItem *aa=dynamic_cast<const FF_AccountTreeItem*>(a);
  const FF_AccountTreeItem *bb=dynamic_cast<const FF_AccountTreeItem*>(b);
  const AE_ACCOUNT *acc1=aa->getAccount();
  const AE_ACCOUNT *acc2=bb->getAccount();
  const char *s1=AE_Account_GetCode(acc1);
  const char *s2=AE_Account_GetCode(acc2);

  if (s1 && s2)
    return strcasecmp(s2, s1);
  else if (s1) {
    return -1;
  }
  else if (s2) {
    return 1;
  }

  return 0;
}



static FXint sortByColumnDescriptionUp(const FXFoldingItem *a, const FXFoldingItem *b) {
  const FF_AccountTreeItem *aa=dynamic_cast<const FF_AccountTreeItem*>(a);
  const FF_AccountTreeItem *bb=dynamic_cast<const FF_AccountTreeItem*>(b);
  const AE_ACCOUNT *acc1=aa->getAccount();
  const AE_ACCOUNT *acc2=bb->getAccount();
  const char *s1=AE_Account_GetDescription(acc1);
  const char *s2=AE_Account_GetDescription(acc2);

  if (s1 && s2)
    return strcasecmp(s1, s2);
  else if (s1) {
    return 1;
  }
  else if (s2) {
    return -1;
  }

  return 0;
}



static FXint sortByColumnDescriptionDown(const FXFoldingItem *a, const FXFoldingItem *b) {
  const FF_AccountTreeItem *aa=dynamic_cast<const FF_AccountTreeItem*>(a);
  const FF_AccountTreeItem *bb=dynamic_cast<const FF_AccountTreeItem*>(b);
  const AE_ACCOUNT *acc1=aa->getAccount();
  const AE_ACCOUNT *acc2=bb->getAccount();
  const char *s1=AE_Account_GetDescription(acc1);
  const char *s2=AE_Account_GetDescription(acc2);

  if (s1 && s2)
    return strcasecmp(s2, s1);
  else if (s1) {
    return -1;
  }
  else if (s2) {
    return 1;
  }

  return 0;
}



void FF_AccountTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    if (up)
      setSortFunc(sortByColumnNameUp);
    else
      setSortFunc(sortByColumnNameDown);
    break;

  case 1:
    if (up)
      setSortFunc(sortByColumnCodeUp);
    else
      setSortFunc(sortByColumnCodeDown);
    break;

  case 2:
    if (up)
      setSortFunc(sortByColumnDescriptionUp);
    else
      setSortFunc(sortByColumnDescriptionDown);
    break;

  default:
    return;
  }

  sortItems();
}



void FF_AccountTree::setHeaderArrowDir(int c, FXbool b) {
  header->setArrowDir(c, b);
}



FXbool FF_AccountTree::getHeaderArrowDir(int c) const {
  return header->getArrowDir(c);
}



long FF_AccountTree::onHeaderClicked(FXObject*, FXSelector, void* ptr) {
  FXint idx=(FXint)(FXival)ptr;
  FXbool b;
  int i;

  b=header->getArrowDir(idx);
  for (i=0; i<header->getNumItems(); i++)
    header->setArrowDir(i, MAYBE);

  if (b==TRUE) {
    /* was up, becomes down */
    header->setArrowDir(idx, FALSE);
    sortByColumn(idx, false);
  }
  else if (b==FALSE) {
    /* was down, becomes MAYBE */
    header->setArrowDir(idx, MAYBE);
  }
  else {
    /* was MAYBE, becomes up */
    header->setArrowDir(idx, TRUE);
    sortByColumn(idx, true);
  }
  return 1;
}



void FF_AccountTree::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;

    if (m_commodities)
      AE_Commodity_List_Clear(m_commodities);
    else
      m_commodities=AE_Commodity_List_new();

    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, m_commodities);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }
  }
}



