/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_addresstree.hpp"

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/text.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FF_AddressTreeItem::FF_AddressTreeItem(AE_ADDRESS *addr,
				       FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_address(addr) {
  rebuild();
}



void FF_AddressTreeItem::rebuild() {
  FXString t;
  const char *s;

  /* name */
  s=AE_Address_GetName(m_address);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  /* street */
  s=AE_Address_GetStreetNum(m_address);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  /* city */
  s=AE_Address_GetCity(m_address);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  /* country */
  s=AE_Address_GetCountry(m_address);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  setText(t);
}



FF_AddressTreeItem::~FF_AddressTreeItem() {
  if (m_isDataItem && m_address) {
    AE_Address_free(m_address);
    m_address=(AE_ADDRESS*) -1;
  }
}



FF_AddressTree::FF_AddressTree(FXComposite *p,
			       FXObject* tgt, FXSelector sel,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Street"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("City"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Country"));
  setHeaderSize(idx++, 100);
}



FF_AddressTree::~FF_AddressTree() {
}



FF_AddressTreeItem *FF_AddressTree::findAddressItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_AddressTreeItem *res=findAddressItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_AddressTreeItem *FF_AddressTree::findAddressItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_AddressTreeItem *jItem=dynamic_cast<FF_AddressTreeItem*>(grItem);
    if (jItem) {
      AE_ADDRESS *addr=jItem->getAddress();
      if (AE_Address_GetId(addr)==id)
        return jItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_AddressTreeItem *res=findAddressItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_AddressTreeItem *FF_AddressTree::findAddressItem(const AE_ADDRESS *addr) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_AddressTreeItem *aitem=dynamic_cast<FF_AddressTreeItem*>(item);
    if (aitem) {
      if (aitem->getAddress()==addr)
	return aitem;
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_AddressTree::addAddress(AE_ADDRESS *addr, AQDB_ID gid) {
  FF_GroupTreeItem *itemFather=NULL;
  FF_AddressTreeItem *item;

  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for bank account not found, reassigning to root");
    }
  }

  item=new FF_AddressTreeItem(addr,
			       /*m_iconOpen, m_iconClosed*/
			       NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



FF_AddressTree::FF_AddressTree(): FF_GroupTree() {
}



void FF_AddressTree::create() {
  FF_GroupTree::create();
}



AE_ADDRESS *FF_AddressTree::getCurrentAddress() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_AddressTreeItem *jItem;

      jItem=dynamic_cast<FF_AddressTreeItem*>(grItem);
      if (jItem)
        return jItem->getAddress();
    }
  }

  return NULL;
}



AE_ADDRESS *FF_AddressTree::getFirstSelectedAddress() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_AddressTreeItem *jItem;

        jItem=dynamic_cast<FF_AddressTreeItem*>(grItem);
        if (jItem)
          return jItem->getAddress();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_AddressTree::getSelectedCurrencies(AE_ADDRESS_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_AddressTreeItem *jItem=dynamic_cast<FF_AddressTreeItem*>(grItem);
        if (jItem)
          AE_Address_List2_PushBack(al, jItem->getAddress());
      }
    }
    item=item->getBelow();
  }
}



void FF_AddressTree::selectAddress(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_AddressTreeItem *jItem;

      jItem=dynamic_cast<FF_AddressTreeItem*>(grItem);
      if (jItem) {
        AE_ADDRESS *addr;

        addr=jItem->getAddress();
        if (AE_Address_GetId(addr)==id) {
          jItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_AddressTree::makeAddressVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_AddressTreeItem *jItem;

      jItem=dynamic_cast<FF_AddressTreeItem*>(grItem);
      if (jItem) {
        AE_ADDRESS *addr;

        addr=jItem->getAddress();
        if (AE_Address_GetId(addr)==id) {
          makeItemVisible(jItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_AddressTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                   \
                                                 const FXFoldingItem *it2) {                 \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_AddressTreeItem *tti1=dynamic_cast<const FF_AddressTreeItem*>(it1);         \
      const FF_AddressTreeItem *tti2=dynamic_cast<const FF_AddressTreeItem*>(it2);         \
      const AE_ADDRESS *addr1=tti1->getAddress();                                           \
      const AE_ADDRESS *addr2=tti2->getAddress();                                           \
      const char *s1=AE_Address_Get##macro_name(addr1);                                      \
      const char *s2=AE_Address_Get##macro_name(addr2);                                      \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_AddressTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,                 \
                                                   const FXFoldingItem *it2){                \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, Name)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, StreetNum)
DEF_SORT_FN_DOWN(1)
DEF_SORT_FN_UP_TXT(2, City)
DEF_SORT_FN_DOWN(2)
DEF_SORT_FN_UP_TXT(3, Country)
DEF_SORT_FN_DOWN(3)

#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



void FF_AddressTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  case 2:
    setSortFunc(up?sortFunctionUp2:sortFunctionDown2);
    sortItems();
    break;
  case 3:
    setSortFunc(up?sortFunctionUp3:sortFunctionDown3);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}



