/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_amountedit.hpp"
#include "ff_app.hpp"

#include <aqdatabase/aqdb_value.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_AmountEdit) FF_AmountEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_AmountEdit::ID_TEXTFIELD, FF_AmountEdit::onCmdTextField),
  FXMAPFUNC(SEL_FOCUSIN, 0, FF_AmountEdit::onFocusIn),
};


// Object implementation
FXIMPLEMENT(FF_AmountEdit, FXHorizontalFrame, FF_AmountEditMap, ARRAYNUMBER(FF_AmountEditMap))



FF_AmountEdit::FF_AmountEdit(FXComposite *p,
			     FXObject* tgt,
			     FXSelector sel,
			     FXuint opts,
			     FXint x, FXint y, FXint w, FXint h,
			     FXint pl,
			     FXint pr,
			     FXint pt,
			     FXint pb)
:FXHorizontalFrame(p, opts, x, y, w, h, pl, pr, pt, pb)
,m_commodities(AE_Commodity_List_new())
,m_textField(NULL)
,m_comboBox(NULL){
  target=tgt;
  message=sel;

  m_textField=new FXTextField(this, 12,
			      this, ID_TEXTFIELD,
			      LAYOUT_FILL_X | TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY);
  m_comboBox=new FXComboBox(this, 3,
			    this, ID_COMBOBOX,
                            COMBOBOX_NORMAL | COMBOBOX_STATIC);
}



FF_AmountEdit::FF_AmountEdit()
:FXHorizontalFrame()
,m_commodities(NULL)
,m_textField(NULL)
,m_comboBox(NULL){
}



FF_AmountEdit::~FF_AmountEdit() {
  AE_Commodity_List_free(m_commodities);
}



void FF_AmountEdit::create() {
  FXHorizontalFrame::create();
}



void FF_AmountEdit::setFont(FXFont* fnt) {
  m_textField->setFont(fnt);
  m_comboBox->setFont(fnt);
}



void FF_AmountEdit::setBackColor(FXColor clr) {
  FXHorizontalFrame::setBackColor(clr);
  m_textField->setBackColor(clr);
  m_comboBox->setBackColor(clr);
}



void FF_AmountEdit::setTextColor(FXColor clr) {
  m_textField->setTextColor(clr);
  m_comboBox->setTextColor(clr);
}



void FF_AmountEdit::setSelBackColor(FXColor clr) {
  m_textField->setSelBackColor(clr);
  m_comboBox->setSelBackColor(clr);
}



void FF_AmountEdit::setSelTextColor(FXColor clr) {
  m_textField->setSelTextColor(clr);
  m_comboBox->setSelTextColor(clr);
}



long FF_AmountEdit::onFocusIn(FXObject *sender, FXSelector sel, void *ptr){
  FXHorizontalFrame::onFocusIn(sender, sel, ptr);
  m_textField->setFocus();
  return 1;
}



long FF_AmountEdit::onCmdTextField(FXObject*, FXSelector, void *ptr) {
  if (target && target->tryHandle(this,
				  FXSEL(SEL_COMMAND, message),
				  ptr))
    return 1;
  return 0;
}



void FF_AmountEdit::addCurrency(AE_COMMODITY *co) {
  const char *s;

  s=AE_Commodity_GetCode(co);
  if (!s || !*s)
    s=AE_Commodity_GetSymbol(co);
  if (!s || !*s)
    s=AE_Commodity_GetName(co);
  if (s && *s) {
    AE_Commodity_List_Add(co, m_commodities);
    m_comboBox->appendItem(FX::fromAscii(s), co);
  }
}



void FF_AmountEdit::setValue(const AQDB_VALUE *v) {
  if (v) {
    GWEN_BUFFER *vbuf;

    vbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AQDB_Value_toHumanReadableString(v, vbuf, 2);
    m_textField->setText(GWEN_Buffer_GetStart(vbuf));
    GWEN_Buffer_free(vbuf);
  }
  else
    m_textField->setText("");
}



FXString FF_AmountEdit::getText() {
  return m_textField->getText();
}



AQDB_VALUE *FF_AmountEdit::getValue() {
  FXString s;

  s=m_textField->getText().trim();
  if (s.empty())
    return NULL;
  else
    return AQDB_Value_fromString(s.text());
}



AE_COMMODITY *FF_AmountEdit::getCurrency() {
  AE_COMMODITY *co=NULL;
  int i;

  i=m_comboBox->getCurrentItem();
  if (i>=0)
    co=(AE_COMMODITY*) (m_comboBox->getItemData(i));

  return co;
}



void FF_AmountEdit::setCurrencyById(AQDB_ID id) {
  int i;

  for (i=0; i<m_comboBox->getNumItems(); i++) {
    AE_COMMODITY *co;

    co=(AE_COMMODITY*) (m_comboBox->getItemData(i));
    if (co && AE_Commodity_GetId(co)==id) {
      m_comboBox->setCurrentItem(i);
      break;
    }
  }
}



AQDB_ID FF_AmountEdit::getCurrencyId() {
  int i;

  i=m_comboBox->getCurrentItem();
  if (i>=0) {
    AE_COMMODITY *co;

    co=(AE_COMMODITY*) (m_comboBox->getItemData(i));
    if (co)
      return AE_Commodity_GetId(co);
  }
  return 0;
}



void FF_AmountEdit::selectAll() {
  m_textField->selectAll();
}



void FF_AmountEdit::reload() {
  FF_App *app;
  AE_BOOK *b;
  AQDB_ID cid;

  cid=getCurrencyId();
  m_comboBox->clearItems();
  AE_Commodity_List_Clear(m_commodities);

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    int rv;
    AE_COMMODITY_LIST *cl;
    AE_COMMODITY *co;

    cl=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
    }

    co=AE_Commodity_List_First(cl);
    while( (co=AE_Commodity_List_First(cl)) ) {
      AE_Commodity_List_Del(co);
      addCurrency(co);
    }
    AE_Commodity_List_free(cl);
  }

  m_comboBox->sortItems();
  if (cid)
    setCurrencyById(cid);
}





