/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_bankaccounttree.hpp"
#include "ff_app.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_bankaccount.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING 4
#define ICON_SPACING 4



FF_BankAccountTreeItem::FF_BankAccountTreeItem(AE_BANKACCOUNT *ba,
                                               AE_BANKBALANCE *booked,
                                               AE_BANKBALANCE *noted,
                                               FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_bankAccount(ba)
,m_notedBalance(noted)
,m_bookedBalance(booked)
,m_flags(0)
{
  FXString t;
  const char *s;

  s=AE_BankAccount_GetBankName(ba);
  if (s)
    t+=s;
  t+="\t";

  s=AE_BankAccount_GetAccountName(ba);
  if (s)
    t+=s;
  t+="\t";

  s=AE_BankAccount_GetBankCode(ba);
  if (s)
    t+=s;
  t+="\t";

  s=AE_BankAccount_GetAccountNumber(ba);
  if (s)
    t+=s;
  t+="\t";

  s=AE_BankAccount_GetBic(ba);
  if (s)
    t+=s;
  t+="\t";

  s=AE_BankAccount_GetIban(ba);
  if (s)
    t+=s;
  t+="\t";

  if (booked) {
    const AQDB_VALUE *v;

    v=AE_BankBalance_GetValue(booked);
    if (v) {
      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      t+=GWEN_Buffer_GetStart(tbuf);
      GWEN_Buffer_free(tbuf);
      s=AE_BankBalance_GetCurrency(booked);
      if (s) {
        t+=" ";
        t+=s;
      }
    }
  }
  t+="\t";

  if (noted) {
    const AQDB_VALUE *v;

    v=AE_BankBalance_GetValue(noted);
    if (v) {
      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      t+=GWEN_Buffer_GetStart(tbuf);
      GWEN_Buffer_free(tbuf);
      s=AE_BankBalance_GetCurrency(noted);
      if (s) {
        t+=" ";
        t+=s;
      }
    }
  }
  t+="\t";

  /* flags */
  t+="test\t";

  setText(t);
}



FF_BankAccountTreeItem::~FF_BankAccountTreeItem() {
  if (m_isDataItem && m_bankAccount) {
    AE_BankAccount_free(m_bankAccount);
    AE_BankBalance_free(m_notedBalance);
    AE_BankBalance_free(m_bookedBalance);
  }
}



void FF_BankAccountTreeItem::drawColumn(const FXXCheckTree* list, FXDC &dc, FXint col,
					FXint xx, FXint y, FXint w, FXint h,
					const FXchar* string, FXuint length) const {
  switch(col) {
  case 6:
  case 7:
    {
      register FXFont *font=list->getFont();
      register FXint tw=font->getTextWidth(string, length);
      register FXint th=font->getFontHeight();
      register FXint yt=y+(h-th-4)/2;
  
      if (!isEnabled())
        dc.setForeground(makeShadowColor(list->getBackColor()));
      else if(isSelected())
        dc.setForeground(list->getSelTextColor());
      else {
        if (string[0]=='-')
          dc.setForeground(FXRGB(255, 0, 0));
        else
          dc.setForeground(list->getTextColor());
      }
  
      if(tw>w-4){
        FXint dw=font->getTextWidth("...",3);
  
        while((tw=font->getTextWidth(string, length))+dw>w-4 && length>1)
          length--;
        dc.setClipRectangle(xx, y, w, h);
        dc.drawText(xx+(w-tw)-2, yt+font->getFontAscent()+2, string, length);
        dc.drawText(xx+tw+(w-tw)-2, yt+font->getFontAscent()+2, "...", 3);
        dc.clearClipRectangle();
      }
      else
        dc.drawText(xx+(w-tw)-2, yt+font->getFontAscent()+2, string, length);
      break;
    }

  case 8:
    {
      FXIcon *ic;
      int ix, iw, ih;
      const FF_BankAccountTree *bat;
      register FXHeader *header=list->getHeader();
  
      bat=dynamic_cast<const FF_BankAccountTree*>(list);
      assert(bat);

      /* flags, draw icons */
      dc.setClipRectangle(header->getItemOffset(col), y, header->getItemSize(col), h);
      ix=xx;
  
      /* payments enqueued */
      if (m_flags & FLAGS_PAYMENTS_ENQUEUED)
        ic=bat->m_icon_paymnt;
      else
        ic=bat->m_icon_paymnt_not;
      if (ic) {
        iw=ic->getWidth();
        ih=ic->getHeight();
        dc.drawIcon(ic, ix, y+(h-ih)/2);
        ix+=iw+ICON_SPACING;
      }

      if (m_flags & FLAGS_RETRIEVAL_ENQUEUED)
        ic=bat->m_icon_retr;
      else
        ic=bat->m_icon_retr_not;
      if (ic) {
        iw=ic->getWidth();
        ih=ic->getHeight();
        dc.drawIcon(ic, ix, y+(h-ih)/2);
        ix+=iw+ICON_SPACING;
      }

      dc.clearClipRectangle();

      break;
    }

  default:
    FF_GroupTreeItem::drawColumn(list, dc, col, xx, y, w, h, string, length);
  }
}








FF_BankAccountTree::FF_BankAccountTree(FXComposite *p,
                                       FXObject* tgt, FXSelector sel,
                                       FXuint opts,
                                       FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_icon_paymnt(NULL)
,m_icon_paymnt_not(NULL)
,m_icon_retr(NULL)
,m_icon_retr_not(NULL)
{
  FF_App *app;
  int idx=0;
  
  app=dynamic_cast<FF_App*>(getApp());

  appendHeader(I18N("Bank Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Account Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Bank Code"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Account Number"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("SWIFT BIC"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("IBAN"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Booked Balance"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Noted Balance"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Enqueued"));
  setHeaderSize(idx++, 100);

  m_icon_paymnt=app->getIcon("paid_full");
  m_icon_paymnt_not=app->getIcon("empty16x16");

  m_icon_retr=app->getIcon("retrieve");
  m_icon_retr_not=app->getIcon("empty16x16");
}



FF_BankAccountTree::~FF_BankAccountTree() {
}



void FF_BankAccountTree::create() {
  FF_GroupTree::create();

  if (m_icon_paymnt)
    m_icon_paymnt->create();
  if (m_icon_paymnt_not)
    m_icon_paymnt_not->create();

  if (m_icon_retr)
    m_icon_retr->create();
  if (m_icon_retr_not)
    m_icon_retr_not->create();
}



FF_BankAccountTreeItem *FF_BankAccountTree::findBankAccountItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_BankAccountTreeItem *res=findBankAccountItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_BankAccountTreeItem *FF_BankAccountTree::findBankAccountItem(FF_GroupTreeItem *grItem,
                                                                AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
    if (baItem) {
      AE_BANKACCOUNT *ba=baItem->getBankAccount();
      if (AE_BankAccount_GetId(ba)==id)
        return baItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_BankAccountTreeItem *res=findBankAccountItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_BankAccountTreeItem *FF_BankAccountTree::findBankAccountItem(const AE_BANKACCOUNT *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankAccountTreeItem *baItem;

      baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
      if (baItem) {
        AE_BANKACCOUNT *ba;

	ba=baItem->getBankAccount();
	if (ba==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_BankAccountTree::addAccount(AE_BANKACCOUNT *ba,
                                    AE_BANKBALANCE *booked,
                                    AE_BANKBALANCE *noted) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_BankAccountTreeItem *item;

  gid=AE_BankAccount_GetGroupId(ba);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for bank account not found, reassigning to root");
    }
  }
  else {
    DBG_ERROR(0, "Account has no gid");
  }

  item=new FF_BankAccountTreeItem(ba,
                                  booked,
                                  noted,
                                  /*m_iconOpen, m_iconClosed*/
                                  NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



void FF_BankAccountTree::addAccount(const AE_BANKACCOUNT *ba,
                                    const AE_BANKBALANCE *booked,
                                    const AE_BANKBALANCE *noted) {
  AE_BANKBALANCE *bb=NULL;
  AE_BANKBALANCE *bn=NULL;

  if (booked)
    bb=AE_BankBalance_dup(booked);
  if (noted)
    bn=AE_BankBalance_dup(noted);

  addAccount(AE_BankAccount_dup(ba), bb, bn);
}



FF_BankAccountTree::FF_BankAccountTree(): FF_GroupTree() {
}



AE_BANKACCOUNT *FF_BankAccountTree::getCurrentBankAccount() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankAccountTreeItem *baItem;

      baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
      if (baItem)
        return baItem->getBankAccount();
    }
  }

  return NULL;
}



AE_BANKACCOUNT *FF_BankAccountTree::getFirstSelectedBankAccount() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_BankAccountTreeItem *baItem;

        baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
        if (baItem)
          return baItem->getBankAccount();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_BankAccountTree::getSelectedBankAccounts(AE_BANKACCOUNT_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
        if (baItem)
          AE_BankAccount_List2_PushBack(al, baItem->getBankAccount());
      }
    }
    item=item->getBelow();
  }
}



void FF_BankAccountTree::selectBankAccount(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankAccountTreeItem *baItem;

      baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
      if (baItem) {
        AE_BANKACCOUNT *ba;

        ba=baItem->getBankAccount();
        if (AE_BankAccount_GetId(ba)==id) {
          baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_BankAccountTree::getBankAccounts(AE_BANKACCOUNT_LIST2 *al, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_NONE:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
	if (baItem)
	  AE_BankAccount_List2_PushBack(al, baItem->getBankAccount());
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
	if (baItem) {
	  AE_BANKACCOUNT *ba;

	  ba=baItem->getBankAccount();
	  assert(ba);
	  AE_BankAccount_List2_PushBack(al, AE_BankAccount_dup(ba));
	}
      }
      item=item->getBelow();
    }
    break;

  default:
  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_BankAccountTreeItem *baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
	if (baItem) {
	  AE_BANKACCOUNT *ba;

	  ba=baItem->getBankAccount();
	  assert(ba);
	  AE_BankAccount_Attach(ba);
	  AE_BankAccount_List2_PushBack(al, ba);
	}
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_BankAccountTree::makeBankAccountVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankAccountTreeItem *baItem;

      baItem=dynamic_cast<FF_BankAccountTreeItem*>(grItem);
      if (baItem) {
        AE_BANKACCOUNT *ba;

        ba=baItem->getBankAccount();
        if (AE_BankAccount_GetId(ba)==id) {
          makeItemVisible(baItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_BankAccountTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                \
                                                    const FXFoldingItem *it2) {              \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_BankAccountTreeItem *tti1=dynamic_cast<const FF_BankAccountTreeItem*>(it1);   \
      const FF_BankAccountTreeItem *tti2=dynamic_cast<const FF_BankAccountTreeItem*>(it2);   \
      const AE_BANKACCOUNT *ba1=tti1->getBankAccount();                                      \
      const AE_BANKACCOUNT *ba2=tti2->getBankAccount();                                      \
      const char *s1=AE_BankAccount_Get##macro_name(ba1);                                    \
      const char *s2=AE_BankAccount_Get##macro_name(ba2);                                    \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_BankAccountTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,              \
                                                      const FXFoldingItem *it2){             \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, BankName)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, AccountName)
DEF_SORT_FN_DOWN(1)
DEF_SORT_FN_UP_TXT(2, BankCode)
DEF_SORT_FN_DOWN(2)
DEF_SORT_FN_UP_TXT(3, AccountNumber)
DEF_SORT_FN_DOWN(3)
DEF_SORT_FN_UP_TXT(4, Bic)
DEF_SORT_FN_DOWN(4)
DEF_SORT_FN_UP_TXT(5, Iban)
DEF_SORT_FN_DOWN(5)




FXint FF_BankAccountTree::sortFunctionUp6(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_BankAccountTreeItem *tti1=dynamic_cast<const FF_BankAccountTreeItem*>(it1);
      const FF_BankAccountTreeItem *tti2=dynamic_cast<const FF_BankAccountTreeItem*>(it2);
      const AE_BANKBALANCE *bal1=NULL;
      const AE_BANKBALANCE *bal2=NULL;
      const AQDB_VALUE *v1=NULL;
      const AQDB_VALUE *v2=NULL;

      bal1=tti1->getBookedBalance();
      if (bal1)
	v1=AE_BankBalance_GetValue(bal1);

      bal2=tti2->getBookedBalance();
      if (bal2)
	v2=AE_BankBalance_GetValue(bal2);

      /* both are data items */
      if (v1 && v2)
        return AQDB_Value_Compare(v1, v2);
      else if (v1)
        return 1;
      else if (v2)
        return -1;
    }
  }

  return 0;
}



FXint FF_BankAccountTree::sortFunctionDown6(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUp6(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}




FXint FF_BankAccountTree::sortFunctionUp7(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_BankAccountTreeItem *tti1=dynamic_cast<const FF_BankAccountTreeItem*>(it1);
      const FF_BankAccountTreeItem *tti2=dynamic_cast<const FF_BankAccountTreeItem*>(it2);
      const AE_BANKBALANCE *bal1=NULL;
      const AE_BANKBALANCE *bal2=NULL;
      const AQDB_VALUE *v1=NULL;
      const AQDB_VALUE *v2=NULL;

      bal1=tti1->getNotedBalance();
      if (bal1)
	v1=AE_BankBalance_GetValue(bal1);

      bal2=tti2->getNotedBalance();
      if (bal2)
	v2=AE_BankBalance_GetValue(bal2);

      /* both are data items */
      if (v1 && v2)
        return AQDB_Value_Compare(v1, v2);
      else if (v1)
        return 1;
      else if (v2)
        return -1;
    }
  }

  return 0;
}



FXint FF_BankAccountTree::sortFunctionDown7(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUp6(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}


#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



void FF_BankAccountTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  case 2:
    setSortFunc(up?sortFunctionUp2:sortFunctionDown2);
    sortItems();
    break;
  case 3:
    setSortFunc(up?sortFunctionUp3:sortFunctionDown3);
    sortItems();
    break;
  case 4:
    setSortFunc(up?sortFunctionUp4:sortFunctionDown4);
    sortItems();
    break;
  case 5:
    setSortFunc(up?sortFunctionUp5:sortFunctionDown5);
    sortItems();
    break;
  case 6:
    setSortFunc(up?sortFunctionUp6:sortFunctionDown6);
    sortItems();
    break;
  case 7:
    setSortFunc(up?sortFunctionUp7:sortFunctionDown7);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}




