/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_bankingjobtree.hpp"

#include <fx.h>

#include <aqbanking/jobgetbalance.h>
#include <aqbanking/jobgettransactions.h>
#include <aqbanking/jobgetstandingorders.h>
#include <aqbanking/jobsingletransfer.h>
#include <aqbanking/jobsingledebitnote.h>
#include <aqbanking/jobeutransfer.h>
#include <aqbanking/jobgetdatedtransfers.h>
#include <aqbanking/jobloadcellphone.h>
#include <aqbanking/jobsepatransfer.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/text.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FF_BankingJobTreeItem::FF_BankingJobTreeItem(AB_JOB *j,
                                             FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_bankingJob(j) {
  FXString t;
  const char *s;
  char numbuf[32];
  AB_ACCOUNT *a;

  AB_Job_Attach(j);
  a=AB_Job_GetAccount(j);

  /* job number */
  snprintf(numbuf, sizeof(numbuf)-1, "%d", AB_Job_GetJobId(j));
  numbuf[sizeof(numbuf)-1]=0;
  t+=numbuf;
  t+="\t";

  /* job type */
  s=AB_Job_Type2LocalChar(AB_Job_GetType(j));
  if (s)
    t+=s;
  t+="\t";

  /* details */
  t+=createJobDetails(j);
  t+="\t";

  /* bank name */
  if (a) {
    s=AB_Account_GetBankName(a);
    if (s==NULL)
      s=AB_Account_GetBankCode(a);
    if (s==NULL)
      s=AB_Account_GetBIC(a);
    if (s)
      t+=FXString(s);
  }
  t+="\t";

  /* account name */
  if (a) {
    s=AB_Account_GetAccountName(a);
    if (s==NULL)
      s=AB_Account_GetAccountNumber(a);
    if (s==NULL)
      s=AB_Account_GetIBAN(a);
    if (s)
      t+=FXString(s);
  }
  t+="\t";


  /* status */
  switch(AB_Job_GetStatus(j)) {
    case AB_Job_StatusNew:      s=I18N("JobStatus|new"); break;
    case AB_Job_StatusUpdated:  s=I18N("JobStatus|updated"); break;
    case AB_Job_StatusEnqueued: s=I18N("JobStatus|enqueued"); break;
    case AB_Job_StatusSent:     s=I18N("JobStatus|sent"); break;
    case AB_Job_StatusPending:  s=I18N("JobStatus|pending"); break;
    case AB_Job_StatusFinished: s=I18N("JobStatus|finished"); break;
    case AB_Job_StatusError:    s=I18N("JobStatus|error"); break;
    default:                    s=I18N("JobStatus|unknown"); break;
  }
  if (s)
    t+=s;
  t+="\t";

  setText(t);
}



FF_BankingJobTreeItem::~FF_BankingJobTreeItem() {
  if (m_isDataItem && m_bankingJob) {
    AB_Job_free(m_bankingJob);
    m_bankingJob=(AB_JOB*) -1;
  }
}



FXString FF_BankingJobTreeItem::createJobDetails(AB_JOB *j) {
  FXString result;

  switch(AB_Job_GetType(j)) {
  case AB_Job_TypeGetBalance:
    break;
  case AB_Job_TypeGetTransactions: {
    const GWEN_TIME *ti1=AB_JobGetTransactions_GetFromTime(j);
    const GWEN_TIME *ti2=AB_JobGetTransactions_GetToTime(j);

    if (ti1 && ti2) {
      GWEN_BUFFER *tbuf;
      FXString fromDate;
      FXString toDate;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Time_toString(ti1, I18N("YYYY/MM/DD"), tbuf);
      fromDate=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);

      GWEN_Time_toString(ti2, I18N("YYYY/MM/DD"), tbuf);
      toDate=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);

      result=FXStringFormat("Get statements from %s to %s",
			    fromDate.text(), toDate.text());
    }
    else if (ti1) {
      GWEN_BUFFER *tbuf;
      FXString fromDate;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Time_toString(ti1, I18N("YYYY/MM/DD"), tbuf);
      fromDate=FXString(GWEN_Buffer_GetStart(tbuf));

      result=FXStringFormat("Get statements from %s on",
			    fromDate.text());
      GWEN_Buffer_free(tbuf);
    }
    else if (ti2) {
      GWEN_BUFFER *tbuf;
      FXString toDate;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      GWEN_Time_toString(ti2, I18N("YYYY/MM/DD"), tbuf);
      toDate=FXString(GWEN_Buffer_GetStart(tbuf));

      result=FXStringFormat("Get statements up until %s",
			    toDate.text());
      GWEN_Buffer_free(tbuf);
    }
    else {
      result=FXString(I18N("Get latest statements"));
    }
    break;
  }

  case AB_Job_TypeTransfer: {
    const AB_TRANSACTION *t;
    const GWEN_STRINGLIST *sl;
    const char *s;
    FXString recipient;
    FXString amount;
    const AB_VALUE *v;

    t=AB_Job_GetTransaction(j);
    sl=AB_Transaction_GetRemoteName(t);
    if (sl) {
      s=GWEN_StringList_FirstString(sl);
      if (s)
        recipient=FXString(s);
    }

    if (recipient.empty()) {
      const char *s1;
      const char *s2;

      s1=AB_Transaction_GetRemoteBankCode(t);
      s2=AB_Transaction_GetRemoteAccountNumber(t);
      if (s1 && s2)
        recipient=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        recipient=FXString(s1);
      else if (s2)
        recipient=FXString(s2);
      else
        recipient=FXString(I18N("Unknown recipient!"));
    }

    v=AB_Transaction_GetValue(t);
    if (v==NULL) {
      amount=FXString(I18N("Unknown amount!"));
    }
    else {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AB_Value_toHumanReadableString(v, tbuf, 2);
      amount=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    result=FXStringFormat(I18N("Transfer %s to %s"), amount.text(), recipient.text());
    break;
  }

  case AB_Job_TypeSepaTransfer: {
    const AB_TRANSACTION *t;
    const GWEN_STRINGLIST *sl;
    const char *s;
    FXString recipient;
    FXString amount;
    const AB_VALUE *v;

    t=AB_Job_GetTransaction(j);
    sl=AB_Transaction_GetRemoteName(t);
    if (sl) {
      s=GWEN_StringList_FirstString(sl);
      if (s)
        recipient=FXString(s);
    }

    if (recipient.empty()) {
      const char *s1;
      const char *s2;

      s1=AB_Transaction_GetRemoteBic(t);
      s2=AB_Transaction_GetRemoteIban(t);
      if (s1 && s2)
        recipient=FXString(s1)+"/"+FXString(s2);
      else if (s1)
        recipient=FXString(s1);
      else if (s2)
        recipient=FXString(s2);
      else
        recipient=FXString(I18N("Unknown recipient!"));
    }

    v=AB_Transaction_GetValue(t);
    if (v==NULL) {
      amount=FXString(I18N("Unknown amount!"));
    }
    else {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AB_Value_toHumanReadableString(v, tbuf, 2);
      amount=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    result=FXStringFormat(I18N("Transfer %s to %s (SEPA)"), amount.text(), recipient.text());
    break;
  }

  case AB_Job_TypeLoadCellPhone: {
    const char *sPhoneNumber=NULL;
    const char *sProductName=NULL;
    FXString recipient;
    FXString amount;
    const AB_VALUE *v;
    const AB_CELLPHONE_PRODUCT *p;

    sPhoneNumber=AB_JobLoadCellPhone_GetPhoneNumber(j);
    if (sPhoneNumber && *sPhoneNumber==0)
      sPhoneNumber=NULL;
    p=AB_JobLoadCellPhone_GetCellPhoneProduct(j);
    if (p)
      sProductName=AB_CellPhoneProduct_GetProductName(p);
    if (sProductName && *sProductName==0)
      sProductName=NULL;

    if (sPhoneNumber && sProductName)
      recipient=FXString(sPhoneNumber)+" ("+FXString(sProductName)+")";
    else if (sPhoneNumber)
      recipient=FXString(sPhoneNumber);
    else if (sProductName)
      recipient=FXString(sProductName);
    else
      recipient=FXString(I18N("Unknown recipient!"));

    v=AB_JobLoadCellPhone_GetValue(j);
    if (v==NULL) {
      amount=FXString(I18N("Unknown amount!"));
    }
    else {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 32, 0, 1);
      AB_Value_toHumanReadableString(v, tbuf, 2);
      amount=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    result=FXStringFormat(I18N("Load cell phone %s with %s"), recipient.text(), amount.text());
    break;
  }

  case AB_Job_TypeDebitNote:
  case AB_Job_TypeEuTransfer:
  case AB_Job_TypeGetStandingOrders:
  case AB_Job_TypeGetDatedTransfers:
  case AB_Job_TypeCreateStandingOrder:
  case AB_Job_TypeModifyStandingOrder:
  case AB_Job_TypeDeleteStandingOrder:
  case AB_Job_TypeCreateDatedTransfer:
  case AB_Job_TypeModifyDatedTransfer:
  case AB_Job_TypeDeleteDatedTransfer:
  case AB_Job_TypeInternalTransfer:
  case AB_Job_TypeSepaDebitNote:
  default:
    break;
  }

  return result;
}




FF_BankingJobTree::FF_BankingJobTree(FXComposite *p,
				     FXObject* tgt, FXSelector sel,
				     FXuint opts,
				     FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Job Id"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Job Type"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Details"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Bank"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Account"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Status"));
  setHeaderSize(idx++, 100);
}



FF_BankingJobTree::~FF_BankingJobTree() {
}



FF_BankingJobTreeItem *FF_BankingJobTree::findBankingJobItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_BankingJobTreeItem *res=findBankingJobItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_BankingJobTreeItem *FF_BankingJobTree::findBankingJobItem(FF_GroupTreeItem *grItem,
                                                                AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_BankingJobTreeItem *jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
    if (jItem) {
      AB_JOB *j=jItem->getBankingJob();
      if (AB_Job_GetJobId(j)==id)
        return jItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_BankingJobTreeItem *res=findBankingJobItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



void FF_BankingJobTree::addJob(AB_JOB *j) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_BankingJobTreeItem *item;

  gid=AB_Account_GetUniqueId(AB_Job_GetAccount(j));
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for bank account not found, reassigning to root");
    }
  }

  item=new FF_BankingJobTreeItem(j,
				 /*m_iconOpen, m_iconClosed*/
				 NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



FF_BankingJobTree::FF_BankingJobTree(): FF_GroupTree() {
}



void FF_BankingJobTree::create() {
  FF_GroupTree::create();
}



AB_JOB *FF_BankingJobTree::getCurrentBankingJob() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankingJobTreeItem *jItem;

      jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
      if (jItem)
        return jItem->getBankingJob();
    }
  }

  return NULL;
}



AB_JOB *FF_BankingJobTree::getFirstSelectedBankingJob() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_BankingJobTreeItem *jItem;

        jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
        if (jItem)
          return jItem->getBankingJob();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_BankingJobTree::getSelectedBankingJobs(AB_JOB_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_BankingJobTreeItem *jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
        if (jItem)
          AB_Job_List2_PushBack(al, jItem->getBankingJob());
      }
    }
    item=item->getBelow();
  }
}



void FF_BankingJobTree::selectBankingJob(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankingJobTreeItem *jItem;

      jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
      if (jItem) {
        AB_JOB *j;

        j=jItem->getBankingJob();
        if (AB_Job_GetJobId(j)==id) {
          jItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_BankingJobTree::makeBankingJobVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_BankingJobTreeItem *jItem;

      jItem=dynamic_cast<FF_BankingJobTreeItem*>(grItem);
      if (jItem) {
        AB_JOB *j;

        j=jItem->getBankingJob();
        if (AB_Job_GetJobId(j)==id) {
          makeItemVisible(jItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}






