/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_currencytree.hpp"

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/text.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FF_CurrencyTreeItem::FF_CurrencyTreeItem(AE_COMMODITY *co,
                                         FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_currency(co) {
  FXString t;
  const char *s;
  const AQDB_VALUE *v;

  /* code */
  s=AE_Commodity_GetCode(co);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  /* name */
  s=AE_Commodity_GetName(co);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  /* symbol */
  s=AE_Commodity_GetSymbol(co);
  if (s && *s)
    t+=FXString(s);
  t+="\t";

  /* fractions */
  v=AE_Commodity_GetSmallestUnit(co);
  if (v) {
    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 32, 0, 1);
    AQDB_Value_toHumanReadableString(v, tbuf, 2);
    t+=GWEN_Buffer_GetStart(tbuf);
    GWEN_Buffer_free(tbuf);
  }
  t+="\t";

  setText(t);
}



FF_CurrencyTreeItem::~FF_CurrencyTreeItem() {
  if (m_isDataItem && m_currency) {
    AE_Commodity_free(m_currency);
    m_currency=(AE_COMMODITY*) -1;
  }
}



FF_CurrencyTree::FF_CurrencyTree(FXComposite *p,
				 FXObject* tgt, FXSelector sel,
				 FXuint opts,
				 FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("FF_CurrencyTree|Code"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_CurrencyTree|Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_CurrencyTree|Symbol"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_CurrencyTree|Fraction"));
  setHeaderSize(idx++, 100);
}



FF_CurrencyTree::~FF_CurrencyTree() {
}



FF_CurrencyTreeItem *FF_CurrencyTree::findCurrencyItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_CurrencyTreeItem *res=findCurrencyItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_CurrencyTreeItem *FF_CurrencyTree::findCurrencyItem(FF_GroupTreeItem *grItem,
						       AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_CurrencyTreeItem *jItem=dynamic_cast<FF_CurrencyTreeItem*>(grItem);
    if (jItem) {
      AE_COMMODITY *co=jItem->getCurrency();
      if (AE_Commodity_GetId(co)==id)
        return jItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_CurrencyTreeItem *res=findCurrencyItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



void FF_CurrencyTree::addCurrency(AE_COMMODITY *co, AQDB_ID gid) {
  FF_GroupTreeItem *itemFather=NULL;
  FF_CurrencyTreeItem *item;

  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for bank account not found, reassigning to root");
    }
  }

  item=new FF_CurrencyTreeItem(co,
			       /*m_iconOpen, m_iconClosed*/
			       NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



FF_CurrencyTree::FF_CurrencyTree(): FF_GroupTree() {
}



void FF_CurrencyTree::create() {
  FF_GroupTree::create();
}



AE_COMMODITY *FF_CurrencyTree::getCurrentCurrency() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CurrencyTreeItem *jItem;

      jItem=dynamic_cast<FF_CurrencyTreeItem*>(grItem);
      if (jItem)
        return jItem->getCurrency();
    }
  }

  return NULL;
}



AE_COMMODITY *FF_CurrencyTree::getFirstSelectedCurrency() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_CurrencyTreeItem *jItem;

        jItem=dynamic_cast<FF_CurrencyTreeItem*>(grItem);
        if (jItem)
          return jItem->getCurrency();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_CurrencyTree::getSelectedCurrencies(AE_COMMODITY_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_CurrencyTreeItem *jItem=dynamic_cast<FF_CurrencyTreeItem*>(grItem);
        if (jItem)
          AE_Commodity_List2_PushBack(al, jItem->getCurrency());
      }
    }
    item=item->getBelow();
  }
}



void FF_CurrencyTree::selectCurrency(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CurrencyTreeItem *jItem;

      jItem=dynamic_cast<FF_CurrencyTreeItem*>(grItem);
      if (jItem) {
        AE_COMMODITY *co;

        co=jItem->getCurrency();
        if (AE_Commodity_GetId(co)==id) {
          jItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_CurrencyTree::makeCurrencyVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_CurrencyTreeItem *jItem;

      jItem=dynamic_cast<FF_CurrencyTreeItem*>(grItem);
      if (jItem) {
        AE_COMMODITY *co;

        co=jItem->getCurrency();
        if (AE_Commodity_GetId(co)==id) {
          makeItemVisible(jItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_CurrencyTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                   \
                                                 const FXFoldingItem *it2) {                 \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_CurrencyTreeItem *tti1=dynamic_cast<const FF_CurrencyTreeItem*>(it1);         \
      const FF_CurrencyTreeItem *tti2=dynamic_cast<const FF_CurrencyTreeItem*>(it2);         \
      const AE_COMMODITY *co1=tti1->getCurrency();                                           \
      const AE_COMMODITY *co2=tti2->getCurrency();                                           \
      const char *s1=AE_Commodity_Get##macro_name(co1);                                      \
      const char *s2=AE_Commodity_Get##macro_name(co2);                                      \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_CurrencyTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,                 \
                                                   const FXFoldingItem *it2){                \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, Code)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, Name)
DEF_SORT_FN_DOWN(1)
DEF_SORT_FN_UP_TXT(2, Symbol)
DEF_SORT_FN_DOWN(2)

#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



FXint FF_CurrencyTree::sortFunctionUp3(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_CurrencyTreeItem *tti1=dynamic_cast<const FF_CurrencyTreeItem*>(it1);
      const FF_CurrencyTreeItem *tti2=dynamic_cast<const FF_CurrencyTreeItem*>(it2);
      const AE_COMMODITY *co1=NULL;
      const AE_COMMODITY *co2=NULL;
      const AQDB_VALUE *v1=NULL;
      const AQDB_VALUE *v2=NULL;

      co1=tti1->getCurrency();
      if (co1)
        v1=AE_Commodity_GetSmallestUnit(co1);

      co2=tti2->getCurrency();
      if (co2)
        v2=AE_Commodity_GetSmallestUnit(co2);

      /* both are data items */
      if (v1 && v2)
        return AQDB_Value_Compare(v1, v2);
      else if (v1)
        return 1;
      else if (v2)
        return -1;
    }
  }

  return 0;
}



FXint FF_CurrencyTree::sortFunctionDown3(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUp3(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



void FF_CurrencyTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  case 2:
    setSortFunc(up?sortFunctionUp2:sortFunctionDown2);
    sortItems();
    break;
  case 3:
    setSortFunc(up?sortFunctionUp3:sortFunctionDown3);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}



