/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_dateedit.hpp"
#include <fxkeys.h>

#include <gwenhywfar/gwentime.h>
#include <gwenhywfar/debug.h>



FXDEFMAP(FF_DateEdit) FF_DateEditMap[]={
  FXMAPFUNC(SEL_VERIFY,0,FF_DateEdit::onVerify),
  FXMAPFUNC(SEL_KEYPRESS,0,FF_DateEdit::onKeyPress),
};


FXIMPLEMENT(FF_DateEdit, FXTextField,
            FF_DateEditMap, ARRAYNUMBER(FF_DateEditMap))




FXbool FF_DateEdit::dateToString(const FXDate &date,
                                 const FXString &fmt,
                                 FXString &str) {
  GWEN_DATE *ti;
  GWEN_BUFFER *buf;

  ti=GWEN_Date_fromGregorian(date.year(),
			     date.month(),
			     date.day());
  if (!ti) {
    DBG_ERROR(0, "Bad date");
    return FALSE;
  }

  buf=GWEN_Buffer_new(0, 64, 0, 1);
  if (GWEN_Date_toStringWithTemplate(ti, fmt.text(), buf)) {
    DBG_ERROR(0, "Bad date");
    GWEN_Buffer_free(buf);
    GWEN_Date_free(ti);
    return FALSE;
  }

  str.assign(GWEN_Buffer_GetStart(buf), GWEN_Buffer_GetUsedBytes(buf));
  GWEN_Buffer_free(buf);
  GWEN_Date_free(ti);
  return TRUE;
}



FXbool FF_DateEdit::dateToString(const GWEN_TIME *ti,
                                 const FXString &fmt,
                                 FXString &str) {
  GWEN_BUFFER *buf;

  buf=GWEN_Buffer_new(0, 64, 0, 1);
  if (GWEN_Time_toString(ti, fmt.text(), buf)) {
    DBG_ERROR(0, "Bad date");
    GWEN_Buffer_free(buf);
    return FALSE;
  }

  str.assign(GWEN_Buffer_GetStart(buf), GWEN_Buffer_GetUsedBytes(buf));
  GWEN_Buffer_free(buf);
  return TRUE;
}



FXbool FF_DateEdit::dateToString(const GWEN_DATE *ti,
                                 const FXString &fmt,
                                 FXString &str) {
  GWEN_BUFFER *buf;

  buf=GWEN_Buffer_new(0, 64, 0, 1);
  if (GWEN_Date_toStringWithTemplate(ti, fmt.text(), buf)) {
    DBG_ERROR(0, "Bad date");
    GWEN_Buffer_free(buf);
    return FALSE;
  }

  str.assign(GWEN_Buffer_GetStart(buf), GWEN_Buffer_GetUsedBytes(buf));
  GWEN_Buffer_free(buf);
  return TRUE;
}



FXbool FF_DateEdit::dateFromString(const FXString &str,
                                   const FXString &fmt,
                                   FXDate &date) {
  GWEN_DATE *ti;

  ti=GWEN_Date_fromStringWithTemplate(str.text(), fmt.text());
  if (ti==0) {
    DBG_ERROR(0, "Bad date");
    return FALSE;
  }

  date.setDate(GWEN_Date_GetYear(ti),
	       GWEN_Date_GetMonth(ti),
               GWEN_Date_GetDay(ti));
  GWEN_Date_free(ti);

  return TRUE;
}



FXbool FF_DateEdit::dateFromString(const FXString &str,
                                   const FXString &fmt,
                                   GWEN_TIME **pTi) {
  GWEN_TIME *ti;
  FXString s;
  FXString f;

  s=str+"-12:00";
  f=fmt+"-hh:mm";

  ti=GWEN_Time_fromString(s.text(), f.text());
  if (ti==0) {
    DBG_ERROR(0, "Bad date");
    return FALSE;
  }

  *pTi=ti;

  return TRUE;
}



FXbool FF_DateEdit::dateFromString(const FXString &str,
                                   const FXString &fmt,
                                   GWEN_DATE **pTi) {
  GWEN_DATE *ti;

  ti=GWEN_Date_fromStringWithTemplate(str.text(), fmt.text());
  if (ti==0) {
    DBG_ERROR(0, "Bad date");
    return FALSE;
  }

  *pTi=ti;

  return TRUE;
}



FXbool FF_DateEdit::convertDate(const FXDate d, GWEN_TIME **pTi) {
  GWEN_TIME *ti;

  ti=GWEN_Time_new(d.year(),
                   d.month()-1,
                   d.day(),
                   0, 0, 0, 0);
  if (ti==0) {
    DBG_ERROR(0, "Bad date");
    return FALSE;
  }

  *pTi=ti;

  return TRUE;
}



FXbool FF_DateEdit::convertDate(const FXDate d, GWEN_DATE **pTi) {
  GWEN_DATE *ti;

  ti=GWEN_Date_fromGregorian(d.year(), d.month(), d.day());
  if (ti==0) {
    DBG_ERROR(0, "Bad date");
    return FALSE;
  }

  *pTi=ti;

  return TRUE;
}



FXbool FF_DateEdit::convertDate(const GWEN_TIME *ti, FXDate &res) {
  int year, month, day;

  GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year);
  res=FXDate(year, month+1, day);
  return TRUE;
}



FXbool FF_DateEdit::convertDate(const GWEN_DATE *ti, FXDate &res) {
  res=FXDate(GWEN_Date_GetYear(ti),
	     GWEN_Date_GetMonth(ti),
	     GWEN_Date_GetDay(ti));
  return TRUE;
}




FF_DateEdit::FF_DateEdit(FXComposite *p,
                         FXObject* tgt,
                         FXSelector sel,
                         const FXString &fmtStr,
                         FXuint opts,
                         FXint x, FXint y, FXint w, FXint h)
:FXTextField(p, 12, tgt, sel, opts, x, y, w, h)
,formatString(fmtStr){

  if (!setDate(FXDate::localDate(), FALSE)) {
    fxerror("%s::FF_DateEdit: could not set local date\n", getClassName());
    DBG_ERROR(0, "Could not set localDate");
  }
}



FF_DateEdit::~FF_DateEdit() {
}



FXbool FF_DateEdit::getDate(FXDate &rdate){
  return dateFromString(getText().simplify().trim(), formatString, rdate);
}



FXbool FF_DateEdit::setDate(const FXDate &date, FXbool notify) {
  FXString s;

  if (!dateToString(date, formatString, s)) {
    DBG_ERROR(0, "Bad argument");
    return FALSE;
  }
  flags|=FLAG_CHANGED;
  setText(s, notify);
  return TRUE;
}



FXbool FF_DateEdit::getDate(GWEN_TIME **pTi) {
  return dateFromString(getText().simplify().trim(), formatString, pTi);
}



FXbool FF_DateEdit::getDate(GWEN_DATE **pTi) {
  return dateFromString(getText().simplify().trim(), formatString, pTi);
}



FXbool FF_DateEdit::setDate(const GWEN_TIME *ti, FXbool notify) {
  if (ti==0) {
    DBG_ERROR(0, "NULL pointer");
    return FALSE;
  }
  else {
    FXString s;

    if (!dateToString(ti, formatString, s)) {
      DBG_ERROR(0, "Bad argument");
      return FALSE;
    }
    flags|=FLAG_CHANGED;
    setText(s, notify);
    return TRUE;
  }
}



FXbool FF_DateEdit::setDate(const GWEN_DATE *ti, FXbool notify) {
  if (ti==0) {
    DBG_ERROR(0, "NULL pointer");
    return FALSE;
  }
  else {
    FXString s;

    if (!dateToString(ti, formatString, s)) {
      DBG_ERROR(0, "Bad argument");
      return FALSE;
    }
    flags|=FLAG_CHANGED;
    setText(s, notify);
    return TRUE;
  }
}



long FF_DateEdit::onKeyPress(FXObject* sender,FXSelector sel,void* ptr){
  FXDate date;
  GWEN_DATE *dt;
  FXEvent* event=(FXEvent*)ptr;

  FXTRACE((200, "%s::onKeyPress: %p (%04x, %04x)\n",
           getClassName(),this, FXSELID(sel), FXSELTYPE(sel)));

  switch (event->code) {
  case KEY_plus:
    if (getDate(date)) {
      setDate(date+1,TRUE);
      return 1;
    }
    break;
  case KEY_minus:
    if (getDate(date)) {
      setDate(date-1,TRUE);
      return 1;
    }
    break;
  case KEY_w:
    if (getDate(date)) {
      setDate(date+7,TRUE);
      return 1;
    }
    break;
  case KEY_m:
    if (getDate(&dt)) {
      int y, m, j;
      GWEN_DATE *nd;
  
      j=GWEN_Date_GetJulian(dt);
      y=GWEN_Date_GetYear(dt);
      m=GWEN_Date_GetMonth(dt);
      switch(m) {
      case 1:
      case 3:
      case 5:
      case 7:
      case 8:
      case 10:
      case 12: /* months with 31 days */
        j+=31;
        break;
      case 2:
        if (GWEN_Date_IsLeapYear(y))
          j+=29;
        else
          j+=28;
        break;
      default:
        j+=30;
        break;
      }
      nd=GWEN_Date_fromJulian(j);
      setDate(nd, TRUE);
      GWEN_Date_free(nd);
      GWEN_Date_free(dt);
      return 1;
    }
    break;

  case KEY_t:
    if (!setDate(FXDate::localDate(), TRUE)) {
      DBG_ERROR(0, "Could not set localDate");
    }
    break;

  default:
    /* let spinner handle this message */
    return FXTextField::onKeyPress(sender, sel, ptr);
  }

  /* bad date, let spinner handle this message */
  return FXTextField::onKeyPress(sender, sel, ptr);
}



long FF_DateEdit::onVerify(FXObject* sender,FXSelector sel,void* ptr){
  return 0;
}



FXDate FF_DateEdit::today() {
  GWEN_TIME *ti;
  FXDate today;

  ti=GWEN_CurrentTime();
  convertDate(ti, today);
  GWEN_Time_free(ti);
  return today;
}




