/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_grouptree.hpp"

#include <gwenhywfar/debug.h>


#define SIDE_SPACING        4   // Spacing between side and item
#define ICON_SPACING        4   // Spacing between parent and child in x direction



FF_GroupTreeItem::FF_GroupTreeItem(AE_GROUP *gr, FXIcon* oi, FXIcon* ci, void* ptr)
:FXXHtmlTreeItem("", oi, ci, ptr)
,m_isDataItem(false)
,m_group(gr) {
  FXString t;

  t=FXString(AE_Group_GetName(gr));
  setText(t);
}



FF_GroupTreeItem::FF_GroupTreeItem(FXIcon* oi, FXIcon* ci, void* ptr)
:FXXHtmlTreeItem("", oi, ci, ptr)
,m_isDataItem(true)
,m_group(NULL) {
}



FF_GroupTreeItem::~FF_GroupTreeItem() {
  if (!m_isDataItem && m_group) {
    AE_Group_free(m_group);
    m_group=(AE_GROUP*) -1;
  }
}



void FF_GroupTreeItem::draw(const FXFoldingList* list,
                            FXDC& dc,
                            FXint x, FXint y, FXint w, FXint h) const {
  if (m_isDataItem) {
    FXXHtmlTreeItem::draw(list, dc, x, y, w, h);
  }
  else {
    register FXHeader *header=list->getHeader();
    register FXIcon *icon=(state&OPENED)?openIcon:closedIcon;
    register FXFont *font=list->getFont();
    register FXint th=0, tw=0, ih=0, iw=0, yt, xb, beg, drw, space, used, dw, xx;

    if (header->getNumItems()==0)
      return;

    xx=x+SIDE_SPACING/2;
    if (icon){
      iw=icon->getWidth();
      ih=icon->getHeight();
      dc.setClipRectangle(header->getItemOffset(0), y, header->getItemSize(0), h);
      dc.drawIcon(icon, xx,y+(h-ih)/2);
      dc.clearClipRectangle();
      xx+=ICON_SPACING+iw;
    }
    if (!label.empty()){
      th=font->getFontHeight();
      dw=font->getTextWidth("...", 3);
      xb=header->getItemOffset(0)+header->getItemSize(0);
      if(xb>xx)
        xb=xx;
      yt=y+(h-th-4)/2;
      if (isSelected()){
        dc.setForeground(list->getSelBackColor());
        dc.fillRectangle(xb, y, header->getTotalSize()-xb, h);
      }
      if (hasFocus()){
        dc.drawFocusRectangle(xb+1, y+1, header->getTotalSize()-xb-2, h-2);
      }
      if (!isEnabled())
        dc.setForeground(makeShadowColor(list->getBackColor()));
      else if (isSelected())
        dc.setForeground(list->getSelTextColor());
      else
        dc.setForeground(list->getTextColor());

      used=xx-header->getItemOffset(0);
      space=header->getTotalSize()-used;

      tw=font->getTextWidth(label);
      if (tw>space-4){
        beg=0;
        drw=label.length();
        while ((tw=font->getTextWidth(&label[beg], drw))+dw>space-4 && drw>1)
          drw=label.dec(drw);
        dc.setClipRectangle(xx, y, space, h);
        dc.drawText(xx+2, yt+font->getFontAscent()+2, &label[beg], drw);
        dc.drawText(xx+tw+2, yt+font->getFontAscent()+2, "...", 3);
        dc.clearClipRectangle();
      }
      else{
        dc.drawText(xx+2, yt+font->getFontAscent()+2, label);
      }
    }
  }
}



FXDEFMAP(FF_GroupTree) FF_GroupTreeMap[]={
};


            
FXIMPLEMENT(FF_GroupTree, FXXCheckTree,
            FF_GroupTreeMap, ARRAYNUMBER(FF_GroupTreeMap))




FF_GroupTree::FF_GroupTree(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
			   FXint x, FXint y, FXint w, FXint h)
:FXXCheckTree(p, tgt, sel, opts, x, y, w, h)
,m_iconOpen(NULL)
,m_iconClosed(NULL)
{
  flags|=FLAG_ENABLED | FLAG_SHOWN;
}



FF_GroupTree::FF_GroupTree()
:FXXCheckTree()
,m_iconOpen(NULL)
,m_iconClosed(NULL)
{
}



FF_GroupTree::~FF_GroupTree() {
}



void FF_GroupTree::dump(int indent) {
  int i;

  for (i=0; i<indent; i++) fprintf(stderr, " ");
  fprintf(stderr, "FF_GroupTree: %d/%d %d/%d (%d %d)\n",
          getX(),
          getY(),
          getWidth(),
          getHeight(),
          getDefaultWidth(),
          getDefaultHeight());

  if (1) {
    FXHeader *h;

    h=getHeader();
    if (h) {
      int j;

      for (i=0; i<indent+2; i++) fprintf(stderr, " ");
      fprintf(stderr, "Header: %d/%d %d/%d (%d %d)\n",
              h->getX(),
              h->getY(),
              h->getWidth(),
              h->getHeight(),
              h->getDefaultWidth(),
              h->getDefaultHeight());
      for (j=0; j<h->getNumItems(); j++) {
        for (i=0; i<indent+4; i++) fprintf(stderr, " ");
        fprintf(stderr, "%d %d: %s\n",
                h->getItemOffset(j), h->getItemSize(j), h->getItemText(j).text());
      }
    }
  }
}



FF_GroupTreeItem *FF_GroupTree::findGroupItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (!(grItem->isDataItem())) {
    AE_GROUP *gr=grItem->getGroup();
    if (gr && AE_Group_GetId(gr)==id)
      return grItem;
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
	FF_GroupTreeItem *res=findGroupItem(grItem, id);
	if (res!=NULL)
	  return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_GroupTreeItem *FF_GroupTree::findGroupItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      if (!(grItem->isDataItem())) {
        AE_GROUP *gr=grItem->getGroup();
        if (gr && AE_Group_GetId(gr)==id)
          return grItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



FF_GroupTreeItem *FF_GroupTree::findGroupItem(const AE_GROUP *group) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      if (!(grItem->isDataItem())) {
	AE_GROUP *gr=grItem->getGroup();
	if (gr && gr==group)
	  return grItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



FF_GroupTreeItem *FF_GroupTree::findGroupItemByName(FF_GroupTreeItem *grItem, const char *name) {
  if (grItem->isDataItem()) {
    /* data item, so it is not a group item, ergo it can't match */
    return NULL;
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
	FF_GroupTreeItem *res=findGroupItemByName(grItem, name);
	if (res!=NULL)
	  return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_GroupTreeItem *FF_GroupTree::findGroupItemByName(const char *name) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      if (!(grItem->isDataItem())) {
        AE_GROUP *gr=grItem->getGroup();
	if (gr) {
	  const char *s;

	  s=AE_Group_GetName(gr);
          if (s && strcasecmp(s, name)==0)
	    return grItem;
	}
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_GroupTree::addGroupWithChildren(FF_GroupTreeItem *itemFather,
                                        const AE_GROUP *gr,
                                        AQDB_ID exceptId) {
  AE_GROUP *grChild;
  FF_GroupTreeItem *item;

  if (exceptId==0 || AE_Group_GetId(gr)!=exceptId) {
    /* add this group */
    item=new FF_GroupTreeItem(AE_Group_dup(gr), m_iconOpen, m_iconClosed);
    appendItem(itemFather, item, false);

    /* add children of this group */
    grChild=AE_Group_Tree_GetFirstChild(gr);
    while(grChild) {
      addGroupWithChildren(item, grChild, exceptId);
      grChild=AE_Group_Tree_GetNext(grChild);
    }
  }
}



void FF_GroupTree::addGroups(AE_GROUP_TREE *gt, AQDB_ID exceptId) {
  AE_GROUP *gr;

  gr=AE_Group_Tree_GetFirst(gt);
  while(gr) {
    addGroupWithChildren(NULL, gr, exceptId);
    gr=AE_Group_Tree_GetNext(gr);
  }
}



void FF_GroupTree::addGroup(AE_GROUP *gr) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_GroupTreeItem *item;

  gid=AE_Group_GetParentId(gr);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Parent for group not found, reassigning to root");
    }
  }
  else {
    DBG_ERROR(0, "Group has no parent gid");
  }

  item=new FF_GroupTreeItem(gr, m_iconOpen, m_iconClosed);
  appendItem(itemFather, item, false);
}



AE_GROUP *FF_GroupTree::getCurrentGroup() {
  FXFoldingItem *item;

  item=getCurrentItem();

  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && !(grItem->isDataItem())) {
      return grItem->getGroup();
    }
    item=item->getParent();
  }

  return NULL;
}



FF_GroupTreeItem *FF_GroupTree::getCurrentGroupItem() {
  FXFoldingItem *item;

  item=getCurrentItem();

  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && !(grItem->isDataItem())) {
      return grItem;
    }
    item=item->getParent();
  }

  return NULL;
}



AE_GROUP *FF_GroupTree::getFirstSelectedGroup() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && !(grItem->isDataItem())) {
        return grItem->getGroup();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_GroupTree::selectGroup(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && !(grItem->isDataItem())) {
      AE_GROUP *gr=grItem->getGroup();
      if (AE_Group_GetId(gr)==id) {
        item->setSelected(b);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_GroupTree::unselectAll() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    item->setSelected(false);
    item=item->getBelow();
  }
}



void FF_GroupTree::makeGroupVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && !(grItem->isDataItem())) {
      AE_GROUP *gr=grItem->getGroup();
      if (AE_Group_GetId(gr)==id) {
        makeItemVisible(item);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_GroupTree::expandItems() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    expandTree(item, false);
    item=item->getBelow();
  }
}



void FF_GroupTree::create() {
  FXFoldingList::create();
  if (m_iconOpen)
    m_iconOpen->create();
  if (m_iconClosed)
    m_iconClosed->create();
}



FXint FF_GroupTree::sortFunctionUp(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem()) {
      const AE_GROUP *g1;
      const AE_GROUP *g2;

      g1=grItem1->getGroup();
      g2=grItem2->getGroup();
      if (g1 && g2) {
        const char *s1;
        const char *s2;

        s1=AE_Group_GetName(g1);
        s2=AE_Group_GetName(g2);
        if (s1 && s2) {
          return strcmp(s1, s2);
        }
        else if (s1 && s2==NULL)
          return 1;
        else if (s1==NULL && s2)
          return -1;
      }
    }
  }
  return 0;
}



FXint FF_GroupTree::sortFunctionDown(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUp(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



void FF_GroupTree::sortByColumn(int c, bool up) {
  setSortFunc(up?sortFunctionUp:sortFunctionDown);
  sortItems();
}



void FF_GroupTree::setHeaderArrowDir(int c, FXbool b) {
  header->setArrowDir(c, b);
}



FXbool FF_GroupTree::getHeaderArrowDir(int c) const {
  return header->getArrowDir(c);
}



