/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_itemtable.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_accountedit.hpp"
#include "ff_amountedit.hpp"
#include "ff_taxedit.hpp"
#include "ff_itemedit.hpp"
#include "ff_valueedit.hpp"

#include <aqfinance/engine/book/ae_utils.h>
#include <aqfinance/engine/book/ae_item.h>


#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#include <assert.h>


#define ITEM_BUNCH 256

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_ItemTable) FF_ItemTableMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_ItemTable::ID_AMOUNT_EDIT, FF_ItemTable::onCmdAmount),
  FXMAPFUNC(SEL_DESELECTED, FF_ItemTable::ID_AMOUNT_EDIT, FF_ItemTable::onCancelAmount),
  FXMAPFUNC(SEL_COMMAND, FF_ItemTable::ID_TAX_EDIT, FF_ItemTable::onCmdTax),
  FXMAPFUNC(SEL_DESELECTED, FF_ItemTable::ID_TAX_EDIT, FF_ItemTable::onCancelTax),
  FXMAPFUNC(SEL_COMMAND, FF_ItemTable::ID_CODE_EDIT, FF_ItemTable::onCmdCode),
  FXMAPFUNC(SEL_DESELECTED, FF_ItemTable::ID_CODE_EDIT, FF_ItemTable::onCancelCode),
  FXMAPFUNC(SEL_COMMAND, FF_ItemTable::ID_TEXTFIELD, FF_ItemTable::onCmdTextField),
  FXMAPFUNC(SEL_DESELECTED, FF_ItemTable::ID_TEXTFIELD, FF_ItemTable::onCancelTextField),
};


FXIMPLEMENT(FF_ItemTable, FXXTable, FF_ItemTableMap, ARRAYNUMBER(FF_ItemTableMap))




FF_ItemTable::ItemContext::ItemContext()
:item(NULL)
{

}



FF_ItemTable::ItemContext::~ItemContext() {
  AE_Item_free(item);
}








FF_ItemTable::FF_ItemTable(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
			   FXint x, FXint y, FXint w, FXint h,
			   FXint pl, FXint pr,
			   FXint pt, FXint pb)
:FXXTable(p, tgt, sel, opts, x, y, w, h, pl, pr, pt, pb)
,m_itemArray(NULL)
,m_itemArraySize(0)
,m_itemCount(0)
,m_markBeyondId(0)
,m_commodities(NULL)
,m_taxes(NULL)
,m_useTemplateEditor(TRUE)
{
  int i;

  m_useTemplateEditor=(opts & ITEMTABLE_USE_ITEM_TEMPLATES)?true:false;

  setMarkColor(0, FF_APP_YELLOW1, 0, 0);
  setMarkColor(0, FF_APP_YELLOW1, 0, 1);
  setMarkColor(0, FF_APP_YELLOW2,  1, 0);
  setMarkColor(0, FF_APP_YELLOW2,  1, 1);

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=COLUMN_TYPE_NONE;
}



FF_ItemTable::FF_ItemTable()
:FXXTable()
,m_itemArray(NULL)
,m_itemArraySize(0)
,m_itemCount(0)
,m_markBeyondId(0)
,m_commodities(NULL)
,m_taxes(NULL)
,m_useTemplateEditor(TRUE)
{
  int i;

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=COLUMN_TYPE_NONE;
}



FF_ItemTable::~FF_ItemTable() {
  if (m_itemArray) {
    for (int i=0; i<m_itemCount; i++)
      if (m_itemArray[i])
        delete m_itemArray[i];
    free(m_itemArray);
    m_itemArray=NULL;
  }

  AE_Commodity_List_free(m_commodities);
  m_commodities=NULL;

  AE_Tax_List_free(m_taxes);
  m_taxes=NULL;
}



void FF_ItemTable::setNumberOfRows(int i) {
  /* reserve space needed for i rows */
  if (m_itemArray==NULL)
    addSpaceForItems(i);
  if (i>m_itemArraySize)
    addSpaceForItems(i-m_itemArraySize);
  FXXTable::setNumberOfRows(i);

  if (i<m_itemCount) {
    int j;

    /* shrinking, free all statements behind i */
    for (j=i; j<m_itemCount; j++) {
      if (m_itemArray[j])
        delete m_itemArray[j];
      m_itemArray[j]=NULL;
    }
  }
  m_itemCount=i;
}



void FF_ItemTable::addColumn(COLUMN_TYPE coltype, int size) {
  if ((m_ncolumns+1)<MAX_COLUMNS) {
    switch(coltype) {
    case COLUMN_TYPE_CODE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Code"), size);
      break;
    case COLUMN_TYPE_NAME:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Name"), size);
      break;
    case COLUMN_TYPE_MEMO:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Memo"), size);
      break;
    case COLUMN_TYPE_AMOUNT:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Amount"), size);
      break;

    case COLUMN_TYPE_BUY_TAX:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Tax"), size);
      break;
    case COLUMN_TYPE_BUY_NETSUM:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Net Sum"), size);
      break;
    case COLUMN_TYPE_BUY_TAXSUM:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Tax Sum"), size);
      break;
    case COLUMN_TYPE_BUY_TOTALSUM:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Total"), size);
      break;

    case COLUMN_TYPE_SELL_TAX:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Tax"), size);
      break;
    case COLUMN_TYPE_SELL_NETSUM:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Net Sum"), size);
      break;
    case COLUMN_TYPE_SELL_TAXSUM:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Tax Sum"), size);
      break;
    case COLUMN_TYPE_SELL_TOTALSUM:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_ItemTable|Total"), size);
      break;

    case COLUMN_TYPE_NONE:
      break;
    }
  }
}



FXXTable::ColumnInfo *FF_ItemTable::getColumnInfo(int r, int c) {
  if (r<m_itemCount && c<MAX_COLUMNS) {
    FXString txt;
    ItemContext *sctx;
    FXColor bg=getCellBackColor(r, c);

    sctx=getItemContextByRow(r);
    if (sctx==NULL) {
      DBG_ERROR(0, "No item found for row %d (col=%d)", r, c);
      return NULL;
    }

    if (m_markBeyondId!=0 && m_markBeyondId<AE_Item_GetId(sctx->item)) {
      if (r & 1)
	bg=FXColor(FF_APP_YELLOW2);
      else
	bg=FXColor(FF_APP_YELLOW1);
    }


    switch(m_columnTypes[c]) {

    case COLUMN_TYPE_CODE:
      if (sctx->item) {
        const char *s;

        s=AE_Item_GetCode(sctx->item);
	if (s)
	  txt+=s;
      }
      break;

    case COLUMN_TYPE_NAME:
      if (sctx->item) {
        const char *s;

        s=AE_Item_GetName(sctx->item);
	if (s)
	  txt+=s;
      }
      break;

    case COLUMN_TYPE_MEMO:
      if (sctx->item) {
        const char *s;

        s=AE_Item_GetMemo(sctx->item);
	if (s)
	  txt+=s;
      }
      break;

    case COLUMN_TYPE_AMOUNT:
      if (sctx->item) {
	if (sctx->sAmount.empty()) {
	  const AQDB_VALUE *v;

	  v=AE_Item_GetValue(sctx->item);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    const char *units;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);

	    units=AE_Utils_ItemUnitsToString(AE_Item_GetUnits(sctx->item));
	    if (units) {
	      str+=" ";
	      str+=units;
	    }

	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sAmount=str;
	  }
	}
	txt+=sctx->sAmount;
      }
      break;

    case COLUMN_TYPE_BUY_TAX:
      if (sctx->item) {
	if (sctx->sBuyTax.empty()) {
	  AQDB_ID tid;

	  tid=AE_Item_GetBuyTaxId(sctx->item);
	  if (tid && m_taxes) {
	    AE_TAX *tax;

	    tax=AE_Tax_List_GetById(m_taxes, tid);
	    if (tax) {
	      GWEN_BUFFER *tbuf;

	      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      AE_Tax_PathToBuffer(tax, tbuf);
	      sctx->sBuyTax=FXString(GWEN_Buffer_GetStart(tbuf));
              GWEN_Buffer_free(tbuf);
	    }
	  }
	}
	txt+=sctx->sBuyTax;
      }
      break;

    case COLUMN_TYPE_SELL_TAX:
      if (sctx->item) {
	if (sctx->sSellTax.empty()) {
	  AQDB_ID tid;

	  tid=AE_Item_GetSellTaxId(sctx->item);
	  if (tid && m_taxes) {
	    AE_TAX *tax;

	    tax=AE_Tax_List_GetById(m_taxes, tid);
	    if (tax) {
	      GWEN_BUFFER *tbuf;

	      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      AE_Tax_PathToBuffer(tax, tbuf);
	      sctx->sSellTax=FXString(GWEN_Buffer_GetStart(tbuf));
              GWEN_Buffer_free(tbuf);
	    }
	  }
	}
	txt+=sctx->sSellTax;
      }
      break;

    case COLUMN_TYPE_BUY_NETSUM:
      if (sctx->item) {
	if (sctx->sBuyNetSum.empty()) {
	  const AQDB_VALUE *v;

	  v=AE_Item_GetBuyPriceValue(sctx->item);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    FXString scom;
            AE_COMMODITY *co;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);
	    /* create commodity string */
	    co=AE_Commodity_List_GetById(m_commodities,
					 AE_Item_GetCommodityId(sctx->item));
	    if (co) {
	      const char *s;
	    
	      s=AE_Commodity_GetCode(co);
	      if (!s || !*s)
		s=AE_Commodity_GetSymbol(co);
	      if (!s || !*s)
		s=AE_Commodity_GetName(co);
	      if (s && *s)
		scom=FXString(s);
	    }
	    if (!scom.empty()) {
	      str+=" ";
	      str+=scom;
	    }
	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sBuyNetSum=str;
	  }
	}
	txt+=sctx->sBuyNetSum;
      }
      break;

    case COLUMN_TYPE_SELL_NETSUM:
      if (sctx->item) {
	if (sctx->sSellNetSum.empty()) {
	  const AQDB_VALUE *v;

	  v=AE_Item_GetSellPriceValue(sctx->item);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    FXString scom;
            AE_COMMODITY *co;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);
	    /* create commodity string */
	    co=AE_Commodity_List_GetById(m_commodities,
					 AE_Item_GetCommodityId(sctx->item));
	    if (co) {
	      const char *s;
	    
	      s=AE_Commodity_GetCode(co);
	      if (!s || !*s)
		s=AE_Commodity_GetSymbol(co);
	      if (!s || !*s)
		s=AE_Commodity_GetName(co);
	      if (s && *s)
		scom=FXString(s);
	    }
	    if (!scom.empty()) {
	      str+=" ";
	      str+=scom;
	    }
	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sSellNetSum=str;
	  }
	}
	txt+=sctx->sSellNetSum;
      }
      break;

    case COLUMN_TYPE_BUY_TAXSUM:
      if (sctx->item) {
	if (sctx->sBuyTaxSum.empty()) {
	  const AQDB_VALUE *v;

	  v=AE_Item_GetBuyTaxSumValue(sctx->item);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    FXString scom;
            AE_COMMODITY *co;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);
	    /* create commodity string */
	    co=AE_Commodity_List_GetById(m_commodities,
					 AE_Item_GetCommodityId(sctx->item));
	    if (co) {
	      const char *s;
	    
	      s=AE_Commodity_GetCode(co);
	      if (!s || !*s)
		s=AE_Commodity_GetSymbol(co);
	      if (!s || !*s)
		s=AE_Commodity_GetName(co);
	      if (s && *s)
		scom=FXString(s);
	    }
	    if (!scom.empty()) {
	      str+=" ";
	      str+=scom;
	    }
	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sBuyTaxSum=str;
	  }
	}
	txt+=sctx->sBuyTaxSum;
      }
      break;

    case COLUMN_TYPE_SELL_TAXSUM:
      if (sctx->item) {
	if (sctx->sSellTaxSum.empty()) {
	  const AQDB_VALUE *v;

	  v=AE_Item_GetSellTaxSumValue(sctx->item);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    FXString scom;
            AE_COMMODITY *co;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);
	    /* create commodity string */
	    co=AE_Commodity_List_GetById(m_commodities,
					 AE_Item_GetCommodityId(sctx->item));
	    if (co) {
	      const char *s;
	    
	      s=AE_Commodity_GetCode(co);
	      if (!s || !*s)
		s=AE_Commodity_GetSymbol(co);
	      if (!s || !*s)
		s=AE_Commodity_GetName(co);
	      if (s && *s)
		scom=FXString(s);
	    }
	    if (!scom.empty()) {
	      str+=" ";
	      str+=scom;
	    }
	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sSellTaxSum=str;
	  }
	}
	txt+=sctx->sSellTaxSum;
      }
      break;

    case COLUMN_TYPE_BUY_TOTALSUM:
      if (sctx->item) {
	if (sctx->sBuyTotal.empty()) {
          AQDB_VALUE *v;
	  const AQDB_VALUE *vt;

	  v=AQDB_Value_new();
	  vt=AE_Item_GetBuyPriceValue(sctx->item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetBuyTaxSumValue(sctx->item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    FXString scom;
            AE_COMMODITY *co;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);
	    /* create commodity string */
	    co=AE_Commodity_List_GetById(m_commodities,
					 AE_Item_GetCommodityId(sctx->item));
	    if (co) {
	      const char *s;
	    
	      s=AE_Commodity_GetCode(co);
	      if (!s || !*s)
		s=AE_Commodity_GetSymbol(co);
	      if (!s || !*s)
		s=AE_Commodity_GetName(co);
	      if (s && *s)
		scom=FXString(s);
	    }
	    if (!scom.empty()) {
	      str+=" ";
	      str+=scom;
	    }
	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sBuyTotal=str;

            AQDB_Value_free(v);
	  }
	}
	txt+=sctx->sBuyTotal;
      }
      break;

    case COLUMN_TYPE_SELL_TOTALSUM:
      if (sctx->item) {
	if (sctx->sSellTotal.empty()) {
          AQDB_VALUE *v;
	  const AQDB_VALUE *vt;

	  v=AQDB_Value_new();
	  vt=AE_Item_GetSellPriceValue(sctx->item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetSellTaxSumValue(sctx->item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  if (v) {
	    FXString str;
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    FXString scom;
            AE_COMMODITY *co;

	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    str="<right>";
	    str+=GWEN_Buffer_GetStart(tbuf);
	    /* create commodity string */
	    co=AE_Commodity_List_GetById(m_commodities,
					 AE_Item_GetCommodityId(sctx->item));
	    if (co) {
	      const char *s;
	    
	      s=AE_Commodity_GetCode(co);
	      if (!s || !*s)
		s=AE_Commodity_GetSymbol(co);
	      if (!s || !*s)
		s=AE_Commodity_GetName(co);
	      if (s && *s)
		scom=FXString(s);
	    }
	    if (!scom.empty()) {
	      str+=" ";
	      str+=scom;
	    }
	    str+="</right>";
	    GWEN_Buffer_free(tbuf);
	    sctx->sSellTotal=str;

            AQDB_Value_free(v);
	  }
	}
	txt+=sctx->sSellTotal;
      }
      break;

    case COLUMN_TYPE_NONE:
      DBG_ERROR(0, "Unknown column type %d", m_columnTypes[c]);
      break;
    }

    return new ColumnInfoText(this, r, c, txt, bg);
  }
  else {
    DBG_ERROR(0, "Out of range (r=%d, c=%d)", r, c);
    return NULL;
  }
}



void FF_ItemTable::releaseColumnInfo(FXXTable::ColumnInfo *ci) {
  delete ci;
}




FXWindow *FF_ItemTable::getEditor(int r, int c) {
  if (r<m_itemCount && c<MAX_COLUMNS) {
    FXString txt;
    ItemContext *sctx;

    sctx=getItemContextByRow(r);
    if (sctx==NULL) {
      DBG_ERROR(0, "No item found for row %d (col=%d)", r, c);
      return NULL;
    }

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_CODE: {
      if (m_useTemplateEditor) {
	FF_ItemEdit *ed=new FF_ItemEdit(AE_Book_TableType_ItemTemplates,
					this, this, ID_CODE_EDIT,
					0,
					0, 0, 0, 0,
					0, 0, 0, 0);
	ed->create();
        ed->reload();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);
	if (sctx->item) {
	  const char *s;
  
	  s=AE_Item_GetCode(sctx->item);
	  if (s) {
	    ed->setText(s);
	    ed->selectAll();
	  }
	}
        DBG_ERROR(0, "Using code editor");
	return ed;
      }
      else {
	FXTextField *ed=new FXTextField(this, 1, this, ID_TEXTFIELD,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
	ed->create();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);
	if (sctx->item) {
	  const char *s;
  
	  s=AE_Item_GetCode(sctx->item);
	  if (s) {
	    ed->setText(s);
	    ed->selectAll();
	  }
	}
	return ed;
      }
    }

    case COLUMN_TYPE_NAME: {
      FXTextField *ed=new FXTextField(this, 1, this, ID_TEXTFIELD,
                                      TEXTFIELD_ENTER_ONLY,
                                      0, 0, 0, 0,
                                      0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      if (sctx->item) {
        const char *s;

	s=AE_Item_GetName(sctx->item);
        if (s) {
          ed->setText(s);
          ed->selectAll();
        }
      }
      return ed;
    }

    case COLUMN_TYPE_MEMO: {
      FXTextField *ed=new FXTextField(this, 1, this, ID_TEXTFIELD,
                                      TEXTFIELD_ENTER_ONLY,
                                      0, 0, 0, 0,
                                      0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      if (sctx->item) {
        const char *s;

	s=AE_Item_GetMemo(sctx->item);
        if (s) {
          ed->setText(s);
          ed->selectAll();
        }
      }
      return ed;
    }

    case COLUMN_TYPE_BUY_TAX: {
      FF_TaxEdit *ed=new FF_TaxEdit(this, this, ID_TAX_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();
      if (sctx->item) {
        ed->setTaxId(AE_Item_GetBuyTaxId(sctx->item));
	ed->selectAll();
      }
      return ed;
    }

    case COLUMN_TYPE_SELL_TAX: {
      FF_TaxEdit *ed=new FF_TaxEdit(this, this, ID_TAX_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();
      if (sctx->item) {
        ed->setTaxId(AE_Item_GetSellTaxId(sctx->item));
	ed->selectAll();
      }
      return ed;
    }

    case COLUMN_TYPE_AMOUNT: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      if (sctx->item) {
	const AQDB_VALUE *v;

	v=AE_Item_GetValue(sctx->item);
	if (v) {
	  const char *units;

	  units=AE_Utils_ItemUnitsToString(AE_Item_GetUnits(sctx->item));
	  ed->setValueAndUnits(v, units);
	  ed->selectAll();
	}
      }
      return ed;
      break;
    }

    case COLUMN_TYPE_BUY_NETSUM: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      if (sctx->item) {
	const AQDB_VALUE *v;
        AQDB_ID cid;

	v=AE_Item_GetBuyPriceValue(sctx->item);
	cid=AE_Item_GetCommodityId(sctx->item);
	if (v) {
	  int rv;
	  AQDB_VALUE *v2;
	  GWEN_BUFFER *tbuf;
  
	  v2=AQDB_Value_dup(v);
	  AQDB_Value_Negate(v2);
	  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	  rv=AE_Utils_AmountToString(v2,
                                     cid,
				     m_commodities,
				     tbuf);
	  if (rv<0) {
	  }
	  else {
	    ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	    ed->selectAll();
	  }
	  GWEN_Buffer_free(tbuf);
	  AQDB_Value_free(v2);
	}
      }

      return ed;
    }

    case COLUMN_TYPE_SELL_NETSUM: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      if (sctx->item) {
	const AQDB_VALUE *v;
	AQDB_ID cid;

	v=AE_Item_GetSellPriceValue(sctx->item);
        cid=AE_Item_GetCommodityId(sctx->item);
	if (v) {
	  int rv;
	  AQDB_VALUE *v2;
	  GWEN_BUFFER *tbuf;
  
	  v2=AQDB_Value_dup(v);
	  AQDB_Value_Negate(v2);
	  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	  rv=AE_Utils_AmountToString(v2,
				     cid,
				     m_commodities,
				     tbuf);
	  if (rv<0) {
	  }
	  else {
	    ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	    ed->selectAll();
	  }
	  GWEN_Buffer_free(tbuf);
	  AQDB_Value_free(v2);
	}
      }

      return ed;
    }

    case COLUMN_TYPE_BUY_TAXSUM: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      if (sctx->item) {
	const AQDB_VALUE *v;
	AQDB_ID cid;

	v=AE_Item_GetBuyTaxSumValue(sctx->item);
        if (v)
	  ed->setValue(v);
	cid=AE_Item_GetCommodityId(sctx->item);
	if (v) {
	  int rv;
	  AQDB_VALUE *v2;
	  GWEN_BUFFER *tbuf;
  
	  v2=AQDB_Value_dup(v);
	  AQDB_Value_Negate(v2);
	  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	  rv=AE_Utils_AmountToString(v2,
                                     cid,
				     m_commodities,
				     tbuf);
	  if (rv<0) {
	  }
	  else {
	    ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	    ed->selectAll();
	  }
	  GWEN_Buffer_free(tbuf);
	  AQDB_Value_free(v2);
	}
      }

      return ed;
    }

    case COLUMN_TYPE_SELL_TAXSUM: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      if (sctx->item) {
	const AQDB_VALUE *v;
	AQDB_ID cid;

	v=AE_Item_GetSellTaxSumValue(sctx->item);
	if (v)
          ed->setValue(v);
	cid=AE_Item_GetCommodityId(sctx->item);
	if (v) {
	  int rv;
	  AQDB_VALUE *v2;
	  GWEN_BUFFER *tbuf;
  
	  v2=AQDB_Value_dup(v);
	  AQDB_Value_Negate(v2);
	  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	  rv=AE_Utils_AmountToString(v2,
                                     cid,
				     m_commodities,
				     tbuf);
	  if (rv<0) {
	  }
	  else {
	    ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	    ed->selectAll();
	  }
	  GWEN_Buffer_free(tbuf);
	  AQDB_Value_free(v2);
	}
      }

      return ed;
    }

    case COLUMN_TYPE_BUY_TOTALSUM: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      if (sctx->item) {
	AQDB_VALUE *v;
	const AQDB_VALUE *vt;

	v=AQDB_Value_new();
	vt=AE_Item_GetBuyPriceValue(sctx->item);
	if (vt)
	  AQDB_Value_AddValue(v, vt);
	vt=AE_Item_GetBuyTaxSumValue(sctx->item);
	if (vt)
	  AQDB_Value_AddValue(v, vt);
	if (v) {
	  int rv;
	  GWEN_BUFFER *tbuf;
  
	  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	  rv=AE_Utils_AmountToString(v,
				     AE_Item_GetCommodityId(sctx->item),
				     m_commodities,
				     tbuf);
	  if (rv<0) {
	  }
	  else {
	    ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	    ed->selectAll();
	  }
	  GWEN_Buffer_free(tbuf);
	}
	AQDB_Value_free(v);
      }

      return ed;
    }

    case COLUMN_TYPE_SELL_TOTALSUM: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      if (sctx->item) {
	AQDB_VALUE *v;
	const AQDB_VALUE *vt;

	v=AQDB_Value_new();
	vt=AE_Item_GetSellPriceValue(sctx->item);
	if (vt)
	  AQDB_Value_AddValue(v, vt);
	vt=AE_Item_GetSellTaxSumValue(sctx->item);
	if (vt)
	  AQDB_Value_AddValue(v, vt);
	if (v) {
	  int rv;
	  GWEN_BUFFER *tbuf;
  
	  tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	  rv=AE_Utils_AmountToString(v,
				     AE_Item_GetCommodityId(sctx->item),
				     m_commodities,
				     tbuf);
	  if (rv<0) {
	  }
	  else {
	    ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	    ed->selectAll();
	  }
	  GWEN_Buffer_free(tbuf);
	}
	AQDB_Value_free(v);
      }

      return ed;
    }

    case COLUMN_TYPE_NONE:
      DBG_ERROR(0, "Unknown column type %d", m_columnTypes[c]);
      break;
    }
  }
  else {
    DBG_ERROR(0, "Out of range (r=%d, c=%d)", r, c);
    return NULL;
  }

  return NULL;
}



void FF_ItemTable::setFromEditor(FXXTable::ColumnInfo *ci, FXWindow *editor) {
  int r;
  int c;

  r=ci->getRow();
  c=ci->getColumn();

  if (r<m_itemCount && c<MAX_COLUMNS) {
    FF_App *app;
    AE_BOOK *b;
    ItemContext *sctx;

    app=dynamic_cast<FF_App*>(getApp());
    b=app->getBook();

    sctx=getItemContextByRow(r);
    if (sctx==NULL) {
      DBG_ERROR(0, "No item found for row %d (col=%d)", r, c);
      return;
    }

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_CODE:
      if (sctx->item) {
	if (m_useTemplateEditor) {
	  FF_ItemEdit *ed=dynamic_cast<FF_ItemEdit*>(editor);
	  if (ed) {
	    FXString txt=ed->getText();
	    if (txt.empty())
	      AE_Item_SetCode(sctx->item, NULL);
	    else {
	      AQDB_ID iid;

	      // TODO: set from item template
	      iid=ed->getItemId();
	      if (iid) {
		AE_ITEM *tmpl=NULL;
		int rv;

		rv=AE_Book_ReadItem(b, AE_Book_TableType_ItemTemplates, iid, &tmpl);
		if (rv<0) {
		  DBG_ERROR(0, "Could not read item %08x: %d", iid, rv);
		  return;
		}

		sctx->sAmount="";
		sctx->sBuyTax="";
		sctx->sBuyNetSum="";
		sctx->sBuyTaxSum="";
		sctx->sBuyTotal="";
		sctx->sSellTax="";
		sctx->sSellNetSum="";
		sctx->sSellTaxSum="";
		sctx->sSellTotal="";

		/* copy data from template */
                AE_Item_SetCode(sctx->item, AE_Item_GetCode(tmpl));
		AE_Item_SetName(sctx->item, AE_Item_GetName(tmpl));
		AE_Item_SetGroupId(sctx->item, AE_Item_GetGroupId(tmpl));
		AE_Item_SetCommodityId(sctx->item, AE_Item_GetCommodityId(tmpl));

		AE_Item_SetUnits(sctx->item, AE_Item_GetUnits(tmpl));
		AE_Item_SetValue(sctx->item, AE_Item_GetValue(tmpl));

		AE_Item_SetBuyPriceValue(sctx->item, AE_Item_GetBuyPriceValue(tmpl));
		AE_Item_SetBuyTaxId(sctx->item, AE_Item_GetBuyTaxId(tmpl));
		AE_Item_SetBuyAccountId(sctx->item, AE_Item_GetBuyAccountId(tmpl));

		AE_Item_SetSellPriceValue(sctx->item, AE_Item_GetSellPriceValue(tmpl));
		AE_Item_SetSellTaxId(sctx->item, AE_Item_GetSellTaxId(tmpl));
		AE_Item_SetSellAccountId(sctx->item, AE_Item_GetSellAccountId(tmpl));

		AE_Item_SetCostOfSalesAccountId(sctx->item, AE_Item_GetCostOfSalesAccountId(tmpl));

		AE_Item_SetMemo(sctx->item, AE_Item_GetMemo(tmpl));

		AE_Item_free(tmpl);

		recalcValues(sctx);
	      }
              else
		AE_Item_SetCode(sctx->item, txt.text());
	    }
	    AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
            updateRow(r);
	  }
	}
	else {
	  FXTextField *ed=dynamic_cast<FXTextField*>(editor);
	  if (ed) {
	    FXString txt=ed->getText();
	    if (txt.empty())
	      AE_Item_SetCode(sctx->item, NULL);
	    else
	      AE_Item_SetCode(sctx->item, txt.text());
	    AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	    updateRow(r);
	  }
	}
      }
      break;

    case COLUMN_TYPE_NAME:
      if (sctx->item) {
        FXTextField *ed=dynamic_cast<FXTextField*>(editor);
	if (ed) {
          FXString txt=ed->getText();
          if (txt.empty())
	    AE_Item_SetName(sctx->item, NULL);
          else
	    AE_Item_SetName(sctx->item, txt.text());
	  AE_Item_SetRuntimeFlags(sctx->item,
				  AE_Item_GetRuntimeFlags(sctx->item) |
				  AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_MEMO:
      if (sctx->item) {
        FXTextField *ed=dynamic_cast<FXTextField*>(editor);
	if (ed) {
          FXString txt=ed->getText();
          if (txt.empty())
	    AE_Item_SetMemo(sctx->item, NULL);
          else
	    AE_Item_SetMemo(sctx->item, txt.text());
	  AE_Item_SetRuntimeFlags(sctx->item,
				  AE_Item_GetRuntimeFlags(sctx->item) |
				  AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_AMOUNT:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty())
	    AE_Item_SetValue(sctx->item, NULL);
	  else {
	    AQDB_VALUE *v=NULL;
	    int units;
            int rv;

	    rv=AE_Utils_ValueAndItemUnitsFromString(txt.text(), &v, &units);
	    if (rv<0) {
	    }
	    else {
	      AE_Item_SetValue(sctx->item, v);
	      AE_Item_SetUnits(sctx->item, units);
	      AQDB_Value_free(v);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	}
      }
      break;

    case COLUMN_TYPE_BUY_NETSUM:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty())
	    AE_Item_SetBuyPriceValue(sctx->item, NULL);
	  else {
	    AQDB_VALUE *v=NULL;
            AE_COMMODITY *co=NULL;
	    int rv;

	    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
	    if (rv<0) {
	    }
	    else {
	      AQDB_VALUE *vNetSum;
	      AQDB_VALUE *vTaxSum;
	      AQDB_VALUE *vTotal;
	      const AQDB_VALUE *vt;
	      AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);

	      if (co)
		coId=AE_Commodity_GetId(co);
	      vNetSum=v;
	      vt=AE_Item_GetBuyTaxSumValue(sctx->item);
	      if (vt)
		vTaxSum=AQDB_Value_dup(vt);
	      else
		vTaxSum=AQDB_Value_new();
	      vTotal=AQDB_Value_new();
  
	      calculateAmountsByNetSum(AE_Item_GetBuyTaxId(sctx->item),
				       coId,
				       vNetSum,
				       vTaxSum,
				       vTotal);
	      sctx->sBuyNetSum="";
	      sctx->sBuyTaxSum="";
	      sctx->sBuyTotal="";
	      AE_Item_SetBuyPriceValue(sctx->item, vNetSum);
	      AE_Item_SetBuyTaxSumValue(sctx->item, vTaxSum);
  
	      AQDB_Value_free(vTotal);
	      AQDB_Value_free(vTaxSum);
	      AQDB_Value_free(vNetSum);
	      AE_Item_SetCommodityId(sctx->item, coId);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_SELL_NETSUM:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty())
	    AE_Item_SetSellPriceValue(sctx->item, NULL);
	  else {
	    AQDB_VALUE *v=NULL;
            AE_COMMODITY *co=NULL;
	    int rv;

	    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
	    if (rv<0) {
	    }
	    else {
	      AQDB_VALUE *vNetSum;
	      AQDB_VALUE *vTaxSum;
	      AQDB_VALUE *vTotal;
	      const AQDB_VALUE *vt;
	      AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);

	      if (co)
		coId=AE_Commodity_GetId(co);
	      vNetSum=v;
	      vt=AE_Item_GetSellTaxSumValue(sctx->item);
	      if (vt)
		vTaxSum=AQDB_Value_dup(vt);
	      else
		vTaxSum=AQDB_Value_new();
	      vTotal=AQDB_Value_new();
  
	      calculateAmountsByNetSum(AE_Item_GetSellTaxId(sctx->item),
				       coId,
				       vNetSum,
				       vTaxSum,
				       vTotal);
	      sctx->sSellNetSum="";
	      sctx->sSellTaxSum="";
	      sctx->sSellTotal="";
	      AE_Item_SetSellPriceValue(sctx->item, vNetSum);
	      AE_Item_SetSellTaxSumValue(sctx->item, vTaxSum);
  
	      AQDB_Value_free(vTotal);
	      AQDB_Value_free(vTaxSum);
	      AQDB_Value_free(vNetSum);
	      AE_Item_SetCommodityId(sctx->item, coId);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_BUY_TAXSUM:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty())
	    AE_Item_SetBuyTaxSumValue(sctx->item, NULL);
	  else {
	    AQDB_VALUE *v=NULL;
            AE_COMMODITY *co=NULL;
	    int rv;

	    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
	    if (rv<0) {
	    }
	    else {
	      AQDB_VALUE *vNetSum;
	      AQDB_VALUE *vTaxSum;
	      AQDB_VALUE *vTotal;
	      const AQDB_VALUE *vt;
	      AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);

	      if (co)
		coId=AE_Commodity_GetId(co);

	      vTaxSum=v;
	      vt=AE_Item_GetBuyPriceValue(sctx->item);
	      if (vt)
		vNetSum=AQDB_Value_dup(vt);
	      else
		vNetSum=AQDB_Value_new();
	      vTotal=AQDB_Value_new();
  
	      calculateAmountsByTaxSum(AE_Item_GetBuyTaxId(sctx->item),
                                       coId,
				       vNetSum,
				       vTaxSum,
				       vTotal);
	      sctx->sBuyNetSum="";
	      sctx->sBuyTaxSum="";
	      sctx->sBuyTotal="";
	      AE_Item_SetBuyPriceValue(sctx->item, vNetSum);
	      AE_Item_SetBuyTaxSumValue(sctx->item, vTaxSum);
  
	      AQDB_Value_free(vTotal);
	      AQDB_Value_free(vTaxSum);
	      AQDB_Value_free(vNetSum);
	      AE_Item_SetCommodityId(sctx->item, coId);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_SELL_TAXSUM:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty())
	    AE_Item_SetSellTaxSumValue(sctx->item, NULL);
	  else {
	    AQDB_VALUE *v=NULL;
            AE_COMMODITY *co=NULL;
	    int rv;

	    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
	    if (rv<0) {
	    }
	    else {
	      AQDB_VALUE *vNetSum;
	      AQDB_VALUE *vTaxSum;
	      AQDB_VALUE *vTotal;
	      const AQDB_VALUE *vt;
	      AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);

	      if (co)
		coId=AE_Commodity_GetId(co);

	      vTaxSum=v;
	      vt=AE_Item_GetSellPriceValue(sctx->item);
	      if (vt)
		vNetSum=AQDB_Value_dup(vt);
	      else
		vNetSum=AQDB_Value_new();
	      vTotal=AQDB_Value_new();
  
	      calculateAmountsByTaxSum(AE_Item_GetSellTaxId(sctx->item),
                                       coId,
				       vNetSum,
				       vTaxSum,
				       vTotal);
	      sctx->sSellNetSum="";
	      sctx->sSellTaxSum="";
	      sctx->sSellTotal="";
	      AE_Item_SetSellPriceValue(sctx->item, vNetSum);
	      AE_Item_SetSellTaxSumValue(sctx->item, vTaxSum);
  
	      AQDB_Value_free(vTotal);
	      AQDB_Value_free(vTaxSum);
	      AQDB_Value_free(vNetSum);
	      AE_Item_SetCommodityId(sctx->item, coId);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_BUY_TOTALSUM:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty()) {
	    AE_Item_SetBuyPriceValue(sctx->item, NULL);
            AE_Item_SetBuyTaxSumValue(sctx->item, NULL);
	  }
	  else {
	    AQDB_VALUE *v=NULL;
            AE_COMMODITY *co=NULL;
	    int rv;

	    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
	    if (rv<0) {
	    }
	    else {
	      AQDB_VALUE *vNetSum;
	      AQDB_VALUE *vTaxSum;
	      AQDB_VALUE *vTotal;
	      const AQDB_VALUE *vt;
	      AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);

	      if (co)
		coId=AE_Commodity_GetId(co);

	      vTotal=v;
	      vt=AE_Item_GetBuyPriceValue(sctx->item);
	      if (vt)
		vNetSum=AQDB_Value_dup(vt);
	      else
		vNetSum=AQDB_Value_new();
	      vt=AE_Item_GetBuyTaxSumValue(sctx->item);
	      if (vt)
		vTaxSum=AQDB_Value_dup(vt);
	      else
		vTaxSum=AQDB_Value_new();
  
	      calculateAmountsByTotal(AE_Item_GetBuyTaxId(sctx->item),
				      coId,
				      vNetSum,
				      vTaxSum,
				      vTotal);
	      sctx->sBuyNetSum="";
	      sctx->sBuyTaxSum="";
	      sctx->sBuyTotal="";
	      AE_Item_SetBuyPriceValue(sctx->item, vNetSum);
	      AE_Item_SetBuyTaxSumValue(sctx->item, vTaxSum);
  
	      AQDB_Value_free(vTotal);
	      AQDB_Value_free(vTaxSum);
	      AQDB_Value_free(vNetSum);
	      AE_Item_SetCommodityId(sctx->item, coId);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_SELL_TOTALSUM:
      if (sctx->item) {
	FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
	if (ed) {
	  FXString txt=ed->getText();
	  sctx->sAmount="";
	  if (txt.empty()) {
	    AE_Item_SetSellPriceValue(sctx->item, NULL);
            AE_Item_SetSellTaxSumValue(sctx->item, NULL);
	  }
	  else {
	    AQDB_VALUE *v=NULL;
            AE_COMMODITY *co=NULL;
	    int rv;

	    rv=AE_Utils_AmountFromString(txt.text(), m_commodities, &v, &co);
	    if (rv<0) {
	    }
	    else {
	      AQDB_VALUE *vNetSum;
	      AQDB_VALUE *vTaxSum;
	      AQDB_VALUE *vTotal;
	      const AQDB_VALUE *vt;
	      AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);

	      if (co)
		coId=AE_Commodity_GetId(co);

	      vTotal=v;
	      vt=AE_Item_GetSellPriceValue(sctx->item);
	      if (vt)
		vNetSum=AQDB_Value_dup(vt);
	      else
		vNetSum=AQDB_Value_new();
	      vt=AE_Item_GetSellTaxSumValue(sctx->item);
	      if (vt)
		vTaxSum=AQDB_Value_dup(vt);
	      else
		vTaxSum=AQDB_Value_new();
  
	      calculateAmountsByTotal(AE_Item_GetSellTaxId(sctx->item),
				      coId,
				      vNetSum,
				      vTaxSum,
				      vTotal);
	      sctx->sSellNetSum="";
	      sctx->sSellTaxSum="";
	      sctx->sSellTotal="";
	      AE_Item_SetSellPriceValue(sctx->item, vNetSum);
	      AE_Item_SetSellTaxSumValue(sctx->item, vTaxSum);
  
	      AQDB_Value_free(vTotal);
	      AQDB_Value_free(vTaxSum);
	      AQDB_Value_free(vNetSum);
	      AE_Item_SetCommodityId(sctx->item, coId);
	    }
	  }
	  AE_Item_AddRuntimeFlags(sctx->item, AE_ITEM_RTFLAGS_MODIFIED);
	  updateRow(r);
	}
      }
      break;

    case COLUMN_TYPE_BUY_TAX:
      if (sctx->item) {
	FF_TaxEdit *ed=dynamic_cast<FF_TaxEdit*>(editor);
	if (ed) {
	  AQDB_ID tid;

	  sctx->sBuyTax="";
	  tid=ed->getTaxId();
          AE_Item_SetBuyTaxId(sctx->item, tid);
	  AE_Item_SetRuntimeFlags(sctx->item,
				  AE_Item_GetRuntimeFlags(sctx->item) |
				  AE_ITEM_RTFLAGS_MODIFIED);
	}
      }
      break;

    case COLUMN_TYPE_SELL_TAX:
      if (sctx->item) {
	FF_TaxEdit *ed=dynamic_cast<FF_TaxEdit*>(editor);
	if (ed) {
	  AQDB_ID tid;

	  sctx->sSellTax="";
	  tid=ed->getTaxId();
          AE_Item_SetSellTaxId(sctx->item, tid);
	  AE_Item_SetRuntimeFlags(sctx->item,
				  AE_Item_GetRuntimeFlags(sctx->item) |
				  AE_ITEM_RTFLAGS_MODIFIED);
	}
      }
      break;

    case COLUMN_TYPE_NONE:
      DBG_ERROR(0, "Unknown column type %d", m_columnTypes[c]);
      break;
    }
  }
  else {
    DBG_ERROR(0, "Invalid table coords (row %d, col=%d)", r, c);
  }
}



void FF_ItemTable::releaseEditor(int r, int c, FXWindow *ed) {
  delete ed;
}



AE_ITEM *FF_ItemTable::getItemByRow(int r) {
  if (m_itemArray && r<m_itemCount) {
    ItemContext *sctx=m_itemArray[r];

    if (sctx)
      return sctx->item;
  }

  return NULL;
}



FF_ItemTable::ItemContext *FF_ItemTable::getItemContextByRow(int r) {
  if (m_itemArray && r<m_itemCount)
    return m_itemArray[r];
  return NULL;
}



void FF_ItemTable::addSpaceForItems(int n) {
  if (m_itemArray) {
    ItemContext **p;
    int i;

    p=(ItemContext**)malloc((n+m_itemArraySize)*sizeof(ItemContext*));
    memset(p, 0, (n+m_itemArraySize)*sizeof(ItemContext*));
    for (i=0; i<m_itemCount; i++) {
      p[i]=m_itemArray[i];
    }
    free(m_itemArray);
    m_itemArray=p;
    m_itemArraySize+=n;
  }
  else {
    m_itemArray=(ItemContext**)malloc(n*sizeof(ItemContext*));
    memset(m_itemArray, 0, n*sizeof(ItemContext*));
    m_itemArraySize=n;
    m_itemCount=0;
  }
}



void FF_ItemTable::addItem(AE_ITEM *st) {
  if (m_itemArray==NULL)
    addSpaceForItems(ITEM_BUNCH);
  if ((m_itemCount+1)>=m_itemArraySize)
    addSpaceForItems(ITEM_BUNCH);
  ItemContext *sctx=new ItemContext();
  sctx->item=st;

  m_itemArray[m_itemCount++]=sctx;
  if (m_itemCount>getNumberOfRows())
    FXXTable::setNumberOfRows(m_itemCount);
}



void FF_ItemTable::delRow(int r) {
  if (r>=0 && r<m_itemCount) {
    int j;

    if (m_itemArray[r])
      delete m_itemArray[r];
    for (j=r+1; j<m_itemCount; j++)
      m_itemArray[j-1]=m_itemArray[j];
    m_itemArray[j-1]=0;
    m_itemCount--;
    setNumberOfRows(m_itemCount);
  }
}



void FF_ItemTable::addItems(AE_ITEM_LIST2 *sl, int copyMode) {
  AE_ITEM_LIST2_ITERATOR *it;

  it=AE_Item_List2_First(sl);
  if (it) {
    AE_ITEM *st;


    switch(copyMode) {
      case COPYMODE_COPY:
        st=AE_Item_List2Iterator_Data(it);
        while(st) {
          addItem(AE_Item_dup(st));
          st=AE_Item_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_ATTACH:
        st=AE_Item_List2Iterator_Data(it);
        while(st) {
          AE_Item_Attach(st);
          addItem(st);
          st=AE_Item_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_NONE:
      default:
        st=AE_Item_List2Iterator_Data(it);
        while(st) {
          addItem(st);
          st=AE_Item_List2Iterator_Next(it);
        }
    }

    AE_Item_List2Iterator_free(it);
  }
}



void FF_ItemTable::getSelectedItems(AE_ITEM_LIST2 *sl, int copyMode) {
  const SelectionRange *r=getSelection().getFirstRange();

  while(r) {
    int i;

    for (i=r->first; i<=r->last; i++) {
      AE_ITEM *st;

      st=getItemByRow(i);
      if (st) {
        switch(copyMode) {
        case COPYMODE_COPY:
          AE_Item_List2_PushBack(sl, AE_Item_dup(st));
          break;

        case COPYMODE_ATTACH:
          AE_Item_Attach(st);
          AE_Item_List2_PushBack(sl, st);
          break;

        case COPYMODE_NONE:
        default:
          AE_Item_List2_PushBack(sl, st);
          break;
        }
      }
    }

    r=r->next;
  }
}



void FF_ItemTable::clearItems() {
  FXXTable::clearItems();
  setNumberOfRows(0);
}



AE_ITEM *FF_ItemTable::getCurrentItem() {
  int r;

  r=getCurrentRow();
  if (r>=0 && r<m_itemCount) {
    AE_ITEM *st;

    st=getItemByRow(r);
    if (st==NULL)
      return NULL;
    return st;
  }

  return NULL;
}



int FF_ItemTable::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  AE_Commodity_List_free(m_commodities);
  m_commodities=AE_Commodity_List_new();

  AE_Tax_List_free(m_taxes);
  m_taxes=AE_Tax_List_new();


  if (b) {
    int rv;

    /* read commodity list */
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, m_commodities);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      return rv;
    }

    /* read tax list */
    rv=AE_Book_ReadTaxList(b, AE_Book_TableType_Tax, m_taxes);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      return rv;
    }

  }

  return 0;
}



long FF_ItemTable::onCmdAmount(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "onCmdAmount");
  acceptInput(true);
  return 1;
}



long FF_ItemTable::onCancelAmount(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



long FF_ItemTable::onCmdTax(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "onCmdTax");
  acceptInput(true);
  return 1;
}



long FF_ItemTable::onCancelTax(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



long FF_ItemTable::onCmdCode(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_ItemTable::onCancelCode(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



long FF_ItemTable::onCmdTextField(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_ItemTable::onCancelTextField(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



int FF_ItemTable::calculateAmountsByNetSum(AQDB_ID taxId,
					   AQDB_ID commodityId,
					   AQDB_VALUE *vNetSum,
					   AQDB_VALUE *vTaxSum,
					   AQDB_VALUE *vTotal) {
  AE_TAX *tax=NULL;

  if (taxId)
    tax=AE_Tax_List_GetById(m_taxes, taxId);

  if (tax==NULL) {
    AQDB_Value_SetZero(vTaxSum);
    AQDB_Value_SetZero(vTotal);
    AQDB_Value_AddValue(vTotal, vNetSum);
  }
  else {
    if (AE_Tax_GetType(tax)==AE_Tax_TypePercentualAmount) {
      const AQDB_VALUE *vPercent;

      /* add percentual amount */
      AQDB_Value_SetZero(vTotal);
      vPercent=AE_Tax_GetAmount(tax);
      if (vPercent) {
	AQDB_VALUE *v100;

	/* vTaxSum=(vNetSum x vPercent) / 100 */
	AQDB_Value_SetZero(vTaxSum);
	AQDB_Value_AddValue(vTaxSum, vNetSum);
	v100=AQDB_Value_fromDouble(100.0);
	AQDB_Value_MultValue(vTaxSum, vPercent);
	AQDB_Value_DivValue(vTaxSum, v100);
	AQDB_Value_free(v100);
      }
      AQDB_Value_AddValue(vTotal, vNetSum);
      AQDB_Value_AddValue(vTotal, vTaxSum);
    }
    else if (AE_Tax_GetType(tax)==AE_Tax_TypeFixedAmount) {
      const AQDB_VALUE *vt;

      AQDB_Value_SetZero(vTaxSum);
      AQDB_Value_SetZero(vTotal);
      AQDB_Value_AddValue(vTotal, vNetSum);

      /* add fixed amount */
      vt=AE_Tax_GetAmount(tax);
      if (vt)
	AQDB_Value_AddValue(vTaxSum, vt);
      AQDB_Value_AddValue(vTotal, vTaxSum);
    }
  }

  return 0;
}



int FF_ItemTable::calculateAmountsByTaxSum(AQDB_ID taxId,
					   AQDB_ID commodityId,
					   AQDB_VALUE *vNetSum,
					   AQDB_VALUE *vTaxSum,
					   AQDB_VALUE *vTotal) {
  AE_TAX *tax=NULL;

  if (taxId)
    tax=AE_Tax_List_GetById(m_taxes, taxId);

  if (tax==NULL) {
    /* no tax, only add netsum and taxsum */
    AQDB_Value_SetZero(vTotal);
    AQDB_Value_AddValue(vTotal, vNetSum);
    AQDB_Value_AddValue(vTotal, vTaxSum);
  }
  else {
    if (AE_Tax_GetType(tax)==AE_Tax_TypePercentualAmount) {
      const AQDB_VALUE *vPercent;

      /* add percentual amount */
      AQDB_Value_SetZero(vTotal);
      vPercent=AE_Tax_GetAmount(tax);
      if (vPercent && !AQDB_Value_IsZero(vPercent)) {
	AQDB_VALUE *v100;

	/* vNetSum=(vTaxSum x 100) / vPercent */
	AQDB_Value_SetZero(vNetSum);
	AQDB_Value_AddValue(vNetSum, vTaxSum);
	v100=AQDB_Value_fromDouble(100.0);

	AQDB_Value_MultValue(vNetSum, v100);
	AQDB_Value_DivValue(vNetSum, vPercent);
	AQDB_Value_free(v100);
      }
      AQDB_Value_AddValue(vTotal, vNetSum);
      AQDB_Value_AddValue(vTotal, vTaxSum);
    }
    else if (AE_Tax_GetType(tax)==AE_Tax_TypeFixedAmount) {
      /* fixed amount, only add netsum and taxsum */
      AQDB_Value_SetZero(vTotal);
      AQDB_Value_AddValue(vTotal, vNetSum);
      AQDB_Value_AddValue(vTotal, vTaxSum);
    }
  }

  return 0;
}



int FF_ItemTable::calculateAmountsByTotal(AQDB_ID taxId,
					  AQDB_ID commodityId,
					  AQDB_VALUE *vNetSum,
					  AQDB_VALUE *vTaxSum,
					  AQDB_VALUE *vTotal) {
  AE_TAX *tax=NULL;

  if (taxId)
    tax=AE_Tax_List_GetById(m_taxes, taxId);

  if (tax==NULL) {
    AQDB_Value_SetZero(vTaxSum);
    AQDB_Value_SetZero(vTotal);
    AQDB_Value_AddValue(vTotal, vNetSum);
  }
  else {
    if (AE_Tax_GetType(tax)==AE_Tax_TypePercentualAmount) {
      const AQDB_VALUE *vPercent;

      /* calculate percentual amounts */
      vPercent=AE_Tax_GetAmount(tax);
      if (vPercent) {
	AQDB_VALUE *v100;
        AQDB_VALUE *vt;

	vt=AQDB_Value_new();
	v100=AQDB_Value_fromDouble(100.0);

	/* vNetSum = ( vTotal / ( 100 + vPercent ) ) x 100 */
	AQDB_Value_AddValue(vt, v100);
	AQDB_Value_AddValue(vt, vPercent);

	AQDB_Value_SetZero(vNetSum);
	AQDB_Value_AddValue(vNetSum, vTotal);
	AQDB_Value_MultValue(vNetSum, v100);
	AQDB_Value_DivValue(vNetSum, vt);

	AQDB_Value_SetZero(vTaxSum);
	AQDB_Value_AddValue(vTaxSum, vTotal);
	AQDB_Value_SubValue(vTaxSum, vNetSum);

	AQDB_Value_free(v100);
	AQDB_Value_free(vt);
      }
    }
    else if (AE_Tax_GetType(tax)==AE_Tax_TypeFixedAmount) {
      const AQDB_VALUE *vPercent;

      vPercent=AE_Tax_GetAmount(tax);
      if (vPercent) {
	AQDB_Value_SetZero(vTaxSum);
	AQDB_Value_AddValue(vTaxSum, vPercent);

	AQDB_Value_SetZero(vNetSum);
	AQDB_Value_AddValue(vNetSum, vTotal);
	AQDB_Value_SubValue(vNetSum, vTaxSum);
      }
    }
  }

  return 0;

}



void FF_ItemTable::recalcValues(ItemContext *sctx) {
  AQDB_VALUE *vNetSum;
  AQDB_VALUE *vTaxSum;
  AQDB_VALUE *vTotal;
  const AQDB_VALUE *vt;
  AQDB_ID coId=AE_Item_GetCommodityId(sctx->item);
  
  /* setup buy amounts */
  vt=AE_Item_GetBuyPriceValue(sctx->item);
  if (vt)
    vNetSum=AQDB_Value_dup(vt);
  else
    vNetSum=AQDB_Value_new();
  vt=AE_Item_GetBuyTaxSumValue(sctx->item);
  if (vt)
    vTaxSum=AQDB_Value_dup(vt);
  else
    vTaxSum=AQDB_Value_new();
  vTotal=AQDB_Value_new();
  
  calculateAmountsByNetSum(AE_Item_GetBuyTaxId(sctx->item),
			   coId,
			   vNetSum,
			   vTaxSum,
			   vTotal);
  
  sctx->sBuyNetSum="";
  sctx->sBuyTaxSum="";
  sctx->sBuyTotal="";
  AE_Item_SetBuyPriceValue(sctx->item, vNetSum);
  AE_Item_SetBuyTaxSumValue(sctx->item, vTaxSum);
  
  AQDB_Value_free(vTotal);
  AQDB_Value_free(vTaxSum);
  AQDB_Value_free(vNetSum);
  
  /* setup sell amounts */
  vt=AE_Item_GetSellPriceValue(sctx->item);
  if (vt)
    vNetSum=AQDB_Value_dup(vt);
  else
    vNetSum=AQDB_Value_new();
  vt=AE_Item_GetSellTaxSumValue(sctx->item);
  if (vt)
    vTaxSum=AQDB_Value_dup(vt);
  else
    vTaxSum=AQDB_Value_new();
  vTotal=AQDB_Value_new();
  
  calculateAmountsByNetSum(AE_Item_GetSellTaxId(sctx->item),
			   coId,
			   vNetSum,
			   vTaxSum,
			   vTotal);
  
  sctx->sSellNetSum="";
  sctx->sSellTaxSum="";
  sctx->sSellTotal="";
  AE_Item_SetSellPriceValue(sctx->item, vNetSum);
  AE_Item_SetSellTaxSumValue(sctx->item, vTaxSum);
  
  AQDB_Value_free(vTotal);
  AQDB_Value_free(vTaxSum);
  AQDB_Value_free(vNetSum);
}


