/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_itemtree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_item.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FF_ItemTreeItem::FF_ItemTreeItem(AE_ITEM *itl,
				 FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_itemTemplate(itl)
{
  rebuild();
}



FF_ItemTreeItem::~FF_ItemTreeItem() {
  if (m_isDataItem && m_itemTemplate) {
    AE_Item_free(m_itemTemplate);
  }
}



void FF_ItemTreeItem::rebuild() {
  FXString t;
  const char *s;

  s=AE_Item_GetCode(m_itemTemplate);
  if (s)
    t+=s;
  t+="\t";

  s=AE_Item_GetName(m_itemTemplate);
  if (s)
    t+=s;
  t+="\t";

  s=AE_Item_GetMemo(m_itemTemplate);
  if (s)
    t+=s;
  t+="\t";

  setText(t);
}








FF_ItemTree::FF_ItemTree(FXComposite *p,
			 FXObject* tgt, FXSelector sel,
			 FXuint opts,
			 FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Code"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Name"));
  setHeaderSize(idx++, 150);
  appendHeader(I18N("Memo"));
  setHeaderSize(idx++, 150);
}



FF_ItemTree::~FF_ItemTree() {
}



FF_ItemTreeItem *FF_ItemTree::findItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_ItemTreeItem *res=findItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_ItemTreeItem *FF_ItemTree::findItem(FF_GroupTreeItem *grItem,
				       AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_ItemTreeItem *baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
    if (baItem) {
      AE_ITEM *ba=baItem->getItem();
      if (AE_Item_GetId(ba)==id)
        return baItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_ItemTreeItem *res=findItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_ItemTreeItem *FF_ItemTree::findItem(const AE_ITEM *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_ItemTreeItem *baItem;

      baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
      if (baItem) {
        AE_ITEM *ba;

	ba=baItem->getItem();
	if (ba==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_ItemTree::addEngineItem(AE_ITEM *ba) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_ItemTreeItem *item;

  gid=AE_Item_GetGroupId(ba);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for item template not found, reassigning to root");
    }
  }
  else {
    DBG_ERROR(0, "Item has no gid");
  }

  item=new FF_ItemTreeItem(ba,
				   /*m_iconOpen, m_iconClosed*/
				   NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



void FF_ItemTree::addEngineItem(const AE_ITEM *ba) {
  addEngineItem(AE_Item_dup(ba));
}



FF_ItemTree::FF_ItemTree(): FF_GroupTree() {
}



void FF_ItemTree::create() {
  FF_GroupTree::create();
}



AE_ITEM *FF_ItemTree::getCurrentEngineItem() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_ItemTreeItem *baItem;

      baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
      if (baItem)
        return baItem->getItem();
    }
  }

  return NULL;
}



AE_ITEM *FF_ItemTree::getFirstSelectedEngineItem() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_ItemTreeItem *baItem;

        baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
        if (baItem)
          return baItem->getItem();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_ItemTree::getSelectedEngineItems(AE_ITEM_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_ItemTreeItem *baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
        if (baItem)
          AE_Item_List2_PushBack(al, baItem->getItem());
      }
    }
    item=item->getBelow();
  }
}



void FF_ItemTree::selectEngineItem(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_ItemTreeItem *baItem;

      baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
      if (baItem) {
        AE_ITEM *ba;

        ba=baItem->getItem();
        if (AE_Item_GetId(ba)==id) {
          baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_ItemTree::selectEngineItem(const AE_ITEM *itl, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_ItemTreeItem *baItem;

      baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
      if (baItem) {
	const AE_ITEM *ba;

        ba=baItem->getItem();
        if (ba==itl) {
	  baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_ItemTree::getEngineItems(AE_ITEM_LIST2 *al, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_NONE:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_ItemTreeItem *baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
	if (baItem)
	  AE_Item_List2_PushBack(al, baItem->getItem());
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_ItemTreeItem *baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
	if (baItem) {
	  AE_ITEM *ba;

	  ba=baItem->getItem();
	  assert(ba);
	  AE_Item_List2_PushBack(al, AE_Item_dup(ba));
	}
      }
      item=item->getBelow();
    }
    break;

  default:
  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_ItemTreeItem *baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
	if (baItem) {
	  AE_ITEM *ba;

	  ba=baItem->getItem();
	  assert(ba);
	  AE_Item_Attach(ba);
	  AE_Item_List2_PushBack(al, ba);
	}
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_ItemTree::makeEngineItemVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_ItemTreeItem *baItem;

      baItem=dynamic_cast<FF_ItemTreeItem*>(grItem);
      if (baItem) {
        AE_ITEM *ba;

        ba=baItem->getItem();
        if (AE_Item_GetId(ba)==id) {
          makeItemVisible(baItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_ItemTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,               \
                                                    const FXFoldingItem *it2) {              \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_ItemTreeItem *tti1=dynamic_cast<const FF_ItemTreeItem*>(it1); \
      const FF_ItemTreeItem *tti2=dynamic_cast<const FF_ItemTreeItem*>(it2); \
      const AE_ITEM *ba1=tti1->getItem();                                   \
      const AE_ITEM *ba2=tti2->getItem();                                   \
      const char *s1=AE_Item_Get##macro_name(ba1);                                   \
      const char *s2=AE_Item_Get##macro_name(ba2);                                   \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_ItemTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,             \
                                                      const FXFoldingItem *it2){             \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, Code)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, Name)
DEF_SORT_FN_DOWN(1)
DEF_SORT_FN_UP_TXT(2, Memo)
DEF_SORT_FN_DOWN(2)




#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



void FF_ItemTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  case 2:
    setSortFunc(up?sortFunctionUp2:sortFunctionDown2);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}





