/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_jobtree.hpp"
#include "ff_app.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_utils.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING        4   // Spacing between side and item
#define ICON_SPACING        4   // Spacing between parent and child in x direction

#define COLUMN_FLAGS        9




FF_JobTreeItem::FF_JobTreeItem(const FXString &txt, uint32_t flags,
			       AE_JOB *j,
			       FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_job(j)
,m_flags(flags)
{
  setText(txt);
}



FF_JobTreeItem::~FF_JobTreeItem() {
  if (m_isDataItem && m_job) {
    AE_Job_free(m_job);
  }
}



FXString FF_JobTreeItem::getColumnTooltipText(FXint col) {
  if (m_job) {
    if (col==COLUMN_FLAGS) {
      FXString str;
  
      str="<html><table>";
  
      /* billed */
      str+="<tr><td>";
      str+=I18N("<b>Billing status:</b>");
      str+="</td><td>";
      if (m_flags & FLAGS_BILLED_FULL)
	str+=I18N("fully billed\n");
      else if (m_flags & FLAGS_BILLED_PARTIAL)
	str+=I18N("partially billed\n");
      else
	str+=I18N("<font color=red>not</font> billed");
      str+="</td></tr>";
  
      /* paid */
      str+="<tr><td>";
      str+=I18N("<b>Payment status:</b>");
      str+="</td><td>";
      if (m_flags & FLAGS_PAID_FULL)
	str+=I18N("fully paid\n");
      else if (m_flags & FLAGS_PAID_PARTIAL)
	str+=I18N("partially paid\n");
      else
	str+=I18N("<font color=red>not</font> paid");
      str+="</td></tr>";
    
      /* delivered */
      str+="<tr><td>";
      str+=I18N("<b>Delivery status:</b>");
      str+="</td><td>";
      if (m_flags & FLAGS_DELIVERED_FULL)
	str+=I18N("fully delivered\n");
      else if (m_flags & FLAGS_DELIVERED_PARTIAL)
	str+=I18N("partially delivered\n");
      else
	str+=I18N("<font color=red>not</font> delivered");
      str+="</td></tr>";
    
      /* booked */
      str+="<tr><td>";
      str+=I18N("<b>Booking status:</b>");
      str+="</td><td>";
      if (m_flags & FLAGS_BOOKED_FULL)
	str+=I18N("fully booked\n");
      else if (m_flags & FLAGS_BOOKED_PARTIAL)
	str+=I18N("partially booked\n");
      else
	str+=I18N("<font color=red>not</font> booked");
      str+="</td></tr>";
      str+="</table>";
      str+="</html>";
      return str;
    }
    else {
      if (!(m_preparedToolTip.empty()))
	return m_preparedToolTip;
      else
	return FF_GroupTreeItem::getColumnTooltipText(col);
    }
  }
  else
    return FF_GroupTreeItem::getColumnTooltipText(col);

}



void FF_JobTreeItem::drawColumn(const FXXCheckTree* list,
				FXDC &dc,
				FXint col,
				FXint x, FXint y, FXint w, FXint h,
				const FXchar* string, FXuint length) const {
  if (col==COLUMN_FLAGS) {
    FXIcon *ic;
    int ix, iw, ih;
    const FF_JobTree *jt;
    register FXHeader *header=list->getHeader();

    jt=dynamic_cast<const FF_JobTree*>(list);
    assert(jt);

    /* flags, draw icons */
    dc.setClipRectangle(header->getItemOffset(col), y, header->getItemSize(col), h);
    ix=x;

    /* urgent */
    if (m_flags & FLAGS_URGENT)
      ic=jt->m_icon_urgent;
    else
      ic=jt->m_icon_urgent_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }
  
    /* reminded */
    if (m_flags & FLAGS_REMINDER)
      ic=jt->m_icon_reminded;
    else
      ic=jt->m_icon_reminded_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }
  
    /* billed */
    if (m_flags & FLAGS_BILLED_FULL)
      ic=jt->m_icon_billed_full;
    else if (m_flags & FLAGS_BILLED_PARTIAL)
      ic=jt->m_icon_billed_part;
    else
      ic=jt->m_icon_billed_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }

    /* printed */
    if (m_flags & FLAGS_PRINTED)
      ic=jt->m_icon_printed;
    else
      ic=jt->m_icon_printed_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }

    /* saved */
    if (m_flags & FLAGS_SAVED)
      ic=jt->m_icon_saved;
    else
      ic=jt->m_icon_saved_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }

    /* mailed */
    if (m_flags & FLAGS_MAILED)
      ic=jt->m_icon_mailed;
    else
      ic=jt->m_icon_mailed_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }

    /* paid */
    if (m_flags & FLAGS_PAID_FULL)
      ic=jt->m_icon_paid_full;
    else if (m_flags & FLAGS_PAID_PARTIAL)
      ic=jt->m_icon_paid_part;
    else
      ic=jt->m_icon_paid_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }
  
    /* delivered */
    if (m_flags & FLAGS_DELIVERED_FULL)
      ic=jt->m_icon_delivered_full;
    else if (m_flags & FLAGS_DELIVERED_PARTIAL)
      ic=jt->m_icon_delivered_part;
    else
      ic=jt->m_icon_delivered_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }
  
    /* booked */
    if (m_flags & FLAGS_BOOKED_FULL)
      ic=jt->m_icon_booked_full;
    else if (m_flags & FLAGS_BOOKED_PARTIAL)
      ic=jt->m_icon_booked_part;
    else
      ic=jt->m_icon_booked_not;
    if (ic) {
      iw=ic->getWidth();
      ih=ic->getHeight();
      dc.drawIcon(ic, ix, y+(h-ih)/2);
      ix+=iw+ICON_SPACING;
    }
  }
  else
    FF_GroupTreeItem::drawColumn(list, dc, col, x, y, w, h, string, length);
}











FF_JobTree::FF_JobTree(FXComposite *p,
		       FXObject* tgt, FXSelector sel,
		       FXuint opts,
		       FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_partners(NULL)
,m_commodities(NULL)
,m_icon_billed_full(NULL)
,m_icon_billed_part(NULL)
,m_icon_billed_not(NULL)
,m_icon_delivered_full(NULL)
,m_icon_delivered_part(NULL)
,m_icon_delivered_not(NULL)
,m_icon_paid_full(NULL)
,m_icon_paid_part(NULL)
,m_icon_paid_not(NULL)
,m_icon_reminded(NULL)
,m_icon_reminded_not(NULL)
,m_icon_urgent(NULL)
,m_icon_urgent_not(NULL)
,m_icon_doc(NULL)
,m_icon_van(NULL)
,m_icon_coin(NULL)
,m_icon_bill(NULL)
,m_icon_payment(NULL)
,m_icon_booked_full(NULL)
,m_icon_booked_part(NULL)
,m_icon_booked_not(NULL)
,m_icon_mailed(NULL)
,m_icon_mailed_not(NULL)
,m_icon_printed(NULL)
,m_icon_printed_not(NULL)
,m_icon_saved(NULL)
,m_icon_saved_not(NULL)
{
  int idx=0;
  FF_App *app;
  
  app=dynamic_cast<FF_App*>(getApp());

  appendHeader(I18N("FF_JobTree|Type"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Enter Date"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Due Date"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Code"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Priority"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Partner"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Memo"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Total Amount"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Open Amount"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("FF_JobTree|Flags"));
  setHeaderSize(idx++, 100);

  m_icon_billed_full=app->getIcon("billed_full");
  m_icon_billed_part=app->getIcon("billed_partial");
  m_icon_billed_not=app->getIcon("empty16x16");

  m_icon_delivered_full=app->getIcon("delivered_full");
  m_icon_delivered_part=app->getIcon("delivered_part");
  m_icon_delivered_not=app->getIcon("empty16x16");

  m_icon_paid_full=app->getIcon("paid_full");
  m_icon_paid_part=app->getIcon("paid_partial");
  m_icon_paid_not=app->getIcon("empty16x16");

  m_icon_reminded=app->getIcon("reminded");
  m_icon_reminded_not=app->getIcon("empty16x16");

  m_icon_urgent=app->getIcon("urgent");
  m_icon_urgent_not=app->getIcon("empty16x16");

  m_icon_doc=app->getIcon("doc24x24");
  m_icon_van=app->getIcon("van24x24");
  m_icon_coin=app->getIcon("coin24x24");
  m_icon_bill=app->getIcon("bill24x24");
  m_icon_payment=app->getIcon("payment24x24");

  m_icon_booked_full=app->getIcon("booked_full");
  m_icon_booked_part=app->getIcon("booked_partial");
  m_icon_booked_not=app->getIcon("empty16x16");

  m_icon_mailed=app->getIcon("mailappt");
  m_icon_mailed_not=app->getIcon("empty16x16");
  m_icon_saved=app->getIcon("pdf");
  m_icon_saved_not=app->getIcon("empty16x16");
  m_icon_printed=app->getIcon("fileprint");
  m_icon_printed_not=app->getIcon("empty16x16");

}



FF_JobTree::~FF_JobTree() {
  AE_Partner_List_free(m_partners);
  AE_Commodity_List_free(m_commodities);
}



void FF_JobTree::create() {
  FF_GroupTree::create();

  if (m_icon_billed_full)
    m_icon_billed_full->create();
  if (m_icon_billed_part)
    m_icon_billed_part->create();
  if (m_icon_billed_not)
    m_icon_billed_not->create();

  if (m_icon_delivered_full)
    m_icon_delivered_full->create();
  if (m_icon_delivered_part)
    m_icon_delivered_part->create();
  if (m_icon_delivered_not)
    m_icon_delivered_not->create();

  if (m_icon_paid_full)
    m_icon_paid_full->create();
  if (m_icon_paid_part)
    m_icon_paid_part->create();
  if (m_icon_paid_not)
    m_icon_paid_not->create();

  if (m_icon_reminded)
    m_icon_reminded->create();
  if (m_icon_reminded_not)
    m_icon_reminded_not->create();

  if (m_icon_urgent)
    m_icon_urgent->create();
  if (m_icon_urgent_not)
    m_icon_urgent_not->create();

  if (m_icon_doc)
    m_icon_doc->create();
  if (m_icon_van)
    m_icon_van->create();
  if (m_icon_coin)
    m_icon_coin->create();
  if (m_icon_bill)
    m_icon_bill->create();
  if (m_icon_payment)
    m_icon_payment->create();

  if (m_icon_booked_full)
    m_icon_booked_full->create();
  if (m_icon_booked_part)
    m_icon_booked_part->create();
  if (m_icon_booked_not)
    m_icon_booked_not->create();

  if (m_icon_mailed)
    m_icon_mailed->create();
  if (m_icon_mailed_not)
    m_icon_mailed_not->create();
  if (m_icon_printed)
    m_icon_printed->create();
  if (m_icon_printed_not)
    m_icon_printed_not->create();
  if (m_icon_saved)
    m_icon_saved->create();
  if (m_icon_saved_not)
    m_icon_saved_not->create();
}



FF_JobTreeItem *FF_JobTree::findJobItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_JobTreeItem *jitem=dynamic_cast<FF_JobTreeItem*>(item);

    if (jitem) {
      AE_JOB *j=jitem->getJob();

      if (j && AE_Job_GetId(j)==id)
        return jitem;
    }
    item=item->getBelow();
  }

  return NULL;
}



FF_JobTreeItem *FF_JobTree::findTaskItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_JobTreeItem *jItem=dynamic_cast<FF_JobTreeItem*>(item);
    if (jItem) {
      AE_JOB *j=jItem->getJob();
      if (AE_Job_GetTaskId(j)==id)
	return jItem;
    }

    //item=item->getBelow();
    item=item->getNext();
  }

  return NULL;
}



FF_JobTreeItem *FF_JobTree::findJobItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_JobTreeItem *jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
    if (jItem) {
      AE_JOB *j=jItem->getJob();
      if (AE_Job_GetId(j)==id)
        return jItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_JobTreeItem *res=findJobItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_JobTreeItem *FF_JobTree::findJobItem(const AE_JOB *qj) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
        AE_JOB *j;

	j=jItem->getJob();
	if (j==qj)
          return jItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_JobTree::addTask(const AE_TASK *tk, uint32_t flags) {
  FXString txt;
  FXString tt;
  GWEN_BUFFER *tbuf;
  const GWEN_DATE *dt;
  const char *s;
  AE_JOB *dummy;
  AQDB_ID id;
  FF_JobTreeItem *nitem;
  int rv;

  tbuf=GWEN_Buffer_new(0, 64, 0, 1);

  /* type */
  txt+=I18N("FF_JobTree|Task");
  txt+="\t";

  /* enter date */
  dt=AE_Task_GetEnterDate(tk);
  if (dt) {
    /* translators: the following is a template for the local date */
    GWEN_Date_toStringWithTemplate(dt, I18N("YYYY/MM/DD"), tbuf);
    txt+=FXString(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_Reset(tbuf);
  }
  txt+="\t";

  /* due date (none for now) */
  txt+="\t";

  /* code (none) */
  txt+="\t";

  /* priority (none for now) */
  txt+="\t";

  /* partner */
  id=AE_Task_GetPartnerId(tk);
  if (id>0) {
    AE_PARTNER *pt;

    pt=AE_Partner_List_GetById(m_partners, id);
    if (pt) {
      AE_Partner_PathToBuffer(pt, tbuf);
      txt+=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
    }
  }
  txt+="\t";

  /* memo */
  s=AE_Task_GetMemo(tk);
  if (s && *s)
    txt+=FXString(s);
  else {
    AE_JOB_LIST *jl;

    jl=AE_Task_GetJobs(tk);
    if (jl) {
      AE_JOB *j;

      j=AE_Job_List_First(jl);
      while(j) {
	s=AE_Job_GetMemo(j);
	if (s && *s)
	  break;
	j=AE_Job_List_Next(j);
      }
      if (j && s && *s)
	txt+=FXString(s);
    }
  }
  txt+="\t";


  /* amount */
  txt+="<right>";
  rv=AE_Utils_AmountToString(AE_Task_GetTotalSum(tk),
			     AE_Task_GetCommodityId(tk),
			     m_commodities,
			     tbuf);
  if (rv==0)
    txt+=FXString(GWEN_Buffer_GetStart(tbuf));
  GWEN_Buffer_Reset(tbuf);
  txt+="</right>";
  txt+="\t";

  /* open amount */
  txt+="<right>";
  rv=AE_Utils_AmountToString(AE_Task_GetOpenSum(tk),
			     AE_Task_GetCommodityId(tk),
			     m_commodities,
			     tbuf);
  if (rv==0)
    txt+=FXString(GWEN_Buffer_GetStart(tbuf));
  GWEN_Buffer_Reset(tbuf);
  txt+="</right>";
  txt+="\t";

  /* flags */
  txt+="test\t";

  /* done */
  GWEN_Buffer_free(tbuf);

  /* preapre tool tip */
  if (1) {
    AE_ITEM *item;

    tt="<html>";
    tt+="<h2>";
    tt+=I18N("FF_JobTree|Task");
    tt+="</h2>";

    /* open orders */
    tt+="<h3>";
    tt+=I18N("Open Orders");
    tt+="</h3>";
    item=AE_Item_List_First(AE_Task_GetOpenOrderedItems(tk));
    if (item) {
      bool headerWritten=false;

      while(item) {
	const AQDB_VALUE *v;

	v=AE_Item_GetValue(item);
	if (v && !AQDB_Value_IsNegative(v)) {
	  s=AE_Item_GetMemo(item);
	  if (s==NULL || *s==0)
	    s=AE_Item_GetName(item);
	  if (s==NULL || *s==0)
	    s=AE_Item_GetCode(item);
	  if (s && *s) {
	    GWEN_BUFFER *tbuf;
	    int u;
  
	    if (!headerWritten) {
	      tt+="<table>";
	      headerWritten=true;
	    }
  
	    tt+="<tr>";
	    tt+="<td>";
	    tt+=s;
	    tt+="</td>";
    
	    tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    tt+="<td>";
	    tt+=FXString(GWEN_Buffer_GetStart(tbuf));
	    GWEN_Buffer_free(tbuf);

	    u=AE_Item_GetUnits(item);
	    if (u>AE_Item_UnitsNone) {
	      tt+=AE_Utils_ItemUnitsToString(u);
	    }
	    tt+="</td>";

	    tt+="</tr>";
	  }
	}
	item=AE_Item_List_Next(item);
      }
      if (headerWritten)
	tt+="</table>";
      else
	tt+=I18N("<i>no undelivered orders</i>");
    }
    else
      tt+=I18N("<i>no undelivered orders</i>");

    /* open deliveries */
    tt+="<h3>";
    tt+=I18N("Open Deliveries");
    tt+="</h3>";
    item=AE_Item_List_First(AE_Task_GetOpenDeliveredItems(tk));
    if (item) {
      bool headerWritten=false;

      while(item) {
	const AQDB_VALUE *v;

	v=AE_Item_GetValue(item);
	if (v && !AQDB_Value_IsNegative(v)) {
	  s=AE_Item_GetMemo(item);
	  if (s==NULL || *s==0)
	    s=AE_Item_GetName(item);
	  if (s==NULL || *s==0)
	    s=AE_Item_GetCode(item);
	  if (s && *s) {
	    GWEN_BUFFER *tbuf;
	    int u;

	    if (!headerWritten) {
	      tt+="<table>";
	      headerWritten=true;
	    }
  
	    tt+="<tr>";
	    tt+="<td>";
	    tt+=s;
	    tt+="</td>";
    
	    tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    tt+="<td>";
	    tt+=FXString(GWEN_Buffer_GetStart(tbuf));
	    GWEN_Buffer_free(tbuf);

	    u=AE_Item_GetUnits(item);
	    if (u>AE_Item_UnitsNone) {
	      tt+=AE_Utils_ItemUnitsToString(u);
	    }
	    tt+="</td>";

	    tt+="</tr>";
	  }
	}
	item=AE_Item_List_Next(item);
      }
      if (headerWritten)
	tt+="</table>";
      else
	tt+=I18N("<i>no unbilled deliveries</i>");
    }
    else
      tt+=I18N("<i>no unbilled deliveries</i>");

    /* open bills */
    tt+="<h3>";
    tt+=I18N("Open Bills");
    tt+="</h3>";
    item=AE_Item_List_First(AE_Task_GetOpenBilledItems(tk));
    if (item) {
      bool headerWritten=false;

      while(item) {
	const AQDB_VALUE *v;

	v=AE_Item_GetValue(item);
	if (v && !AQDB_Value_IsNegative(v)) {
	  s=AE_Item_GetMemo(item);
	  if (s==NULL || *s==0)
	    s=AE_Item_GetName(item);
	  if (s==NULL || *s==0)
	    s=AE_Item_GetCode(item);
	  if (s && *s) {
	    GWEN_BUFFER *tbuf;
	    int u;

	    if (!headerWritten) {
	      tt+="<table>";
	      headerWritten=true;
	    }
  
	    tt+="<tr>";
	    tt+="<td>";
	    tt+=s;
	    tt+="</td>";
    
	    tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	    AQDB_Value_toHumanReadableString(v, tbuf, 2);
	    tt+="<td>";
	    tt+=FXString(GWEN_Buffer_GetStart(tbuf));
	    GWEN_Buffer_free(tbuf);

	    u=AE_Item_GetUnits(item);
	    if (u>AE_Item_UnitsNone) {
	      tt+=AE_Utils_ItemUnitsToString(u);
	    }
	    tt+="</td>";

	    tt+="</tr>";
	  }
	}
	item=AE_Item_List_Next(item);
      }
      if (headerWritten)
	tt+="</table>";
      else
	tt+=I18N("<i>no unpaid bills</i>");
    }
    else
      tt+=I18N("<i>no unpaid bills</i>");

    tt+="</html>";
  }


  dummy=AE_Job_new();
  AE_Job_SetTaskId(dummy, AE_Task_GetId(tk));
  AE_Job_SetEnterDate(dummy, AE_Task_GetEnterDate(tk));

  nitem=new FF_JobTreeItem(txt,
			   flags,
			   dummy,
			   m_icon_doc,
			   m_icon_doc);
  nitem->m_preparedToolTip=tt;
  appendItem(NULL, nitem, false);

}



void FF_JobTree::addJob(AE_JOB *j, uint32_t flags) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID id;
  FF_JobTreeItem *item;
  FXString txt;
  GWEN_BUFFER *tbuf;
  int rv;
  const GWEN_DATE *dt;
  const char *s;
  FXIcon *ic=NULL;
  int jt;

  jt=AE_Job_GetType(j);

  id=AE_Job_GetReferenceId(j);
  if (id) {
    itemFather=findJobItem(id);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Reference job not found, reassigning to task");
      id=AE_Job_GetTaskId(j);
      if (id) {
	itemFather=findTaskItem(id);
	if (itemFather==NULL) {
	  DBG_ERROR(AE_LOGDOMAIN, "Task not found, reassigning to root");
	}
      }
    }
  }
  else {
    id=AE_Job_GetTaskId(j);
    if (id) {
      itemFather=findTaskItem(id);
      if (itemFather==NULL) {
	DBG_ERROR(AE_LOGDOMAIN, "Task not found, reassigning to root");
      }
    }
  }

  tbuf=GWEN_Buffer_new(0, 64, 0, 1);

  switch(jt) {
  case AE_Job_TypeRequest:      txt+=I18N("Request"); break;
  case AE_Job_TypeOffer:        txt+=I18N("Offer"); break;
  case AE_Job_TypeOrder:        txt+=I18N("Order"); break;
  case AE_Job_TypeConfirmation: txt+=I18N("Confirmation"); break;
  case AE_Job_TypeRelease:      txt+=I18N("Release"); break;
  case AE_Job_TypeDelivery:     txt+=I18N("Delivery"); break;
  case AE_Job_TypeInvoice:      txt+=I18N("Invoice"); break;
  case AE_Job_TypePayment:      txt+=I18N("Payment"); break;
  case AE_Job_TypeReminder:     txt+=I18N("Reminder"); break;
  case AE_Job_TypeCancellation: txt+=I18N("Cancellation"); break;

  case AE_Job_TypeUnknown:
  case AE_Job_TypeNone:
    break;
  }
  txt+="\t";

  /* enter date */
  dt=AE_Job_GetEnterDate(j);
  if (dt) {
    /* translators: the following is a template for the local date */
    GWEN_Date_toStringWithTemplate(dt, I18N("YYYY/MM/DD"), tbuf);
    txt+=FXString(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_Reset(tbuf);
  }
  txt+="\t";

  /* due date */
  if (jt==AE_Job_TypeOrder ||
      jt==AE_Job_TypeInvoice) {
    dt=AE_Job_GetDueDate(j);
    if (dt) {
      /* translators: the following is a template for the local date */
      GWEN_Date_toStringWithTemplate(dt, I18N("YYYY/MM/DD"), tbuf);
      txt+=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
    }
  }
  txt+="\t";

  /* code */
  s=AE_Job_GetCode(j);
  if (s && *s)
    txt+=FXString(s);
  txt+="\t";

  /* priority */
  switch(AE_Job_GetPriority(j)) {
  case AE_Job_PriorityLow:     txt+=I18N("Priority|low"); break;
  case AE_Job_PriorityMedium:  txt+=I18N("Priority|medium"); break;
  case AE_Job_PriorityHigh:    txt+=I18N("Priority|high"); break;
  case AE_Job_PriorityUrgent:  txt+=I18N("Priority|urgent"); break;
  case AE_Job_PriorityUnknown:
  case AE_Job_PriorityNone:
    break;
  }
  txt+="\t";

  /* partner */
  id=AE_Job_GetPartnerId(j);
  if (id>0) {
    AE_PARTNER *pt;

    pt=AE_Partner_List_GetById(m_partners, id);
    if (pt) {
      AE_Partner_PathToBuffer(pt, tbuf);
      txt+=FXString(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
    }
  }
  txt+="\t";

  s=AE_Job_GetMemo(j);
  if (s && *s)
    txt+=FXString(s);

#if 0
  if (jt==AE_Job_TypeOrder ||
      jt==AE_Job_TypeDelivery ||
      jt==AE_Job_TypeInvoice) {
    AE_ITEM *item;

    txt+="<font size=8><table>";
    item=AE_Item_List_First(AE_Job_GetItems(j));
    while(item) {
      s=AE_Item_GetMemo(item);
      if (s==NULL || *s==0)
	s=AE_Item_GetName(item);
      if (s==NULL || *s==0)
	s=AE_Item_GetCode(item);
      if (s && *s) {
	const AQDB_VALUE *v;

	txt+="<tr>";
	txt+="<td>";
	txt+=s;
	txt+="</td>";
	v=AE_Item_GetValue(item);
	if (v) {
	  GWEN_BUFFER *tbuf;
          int u;

	  tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	  AQDB_Value_toHumanReadableString(v, tbuf, 2);
	  txt+="<td>";
	  txt+=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);

	  u=AE_Item_GetUnits(item);
	  if (u>AE_Item_UnitsNone) {
	    txt+=AE_Utils_ItemUnitsToString(u);
	  }
	  txt+="</td>";
	}
	txt+="</tr>";
      }
      item=AE_Item_List_Next(item);
    }
    txt+="</table></font>";
  }
#endif
  txt+="\t";

  /* amount */
  if (jt!=AE_Job_TypeDelivery &&
      jt!=AE_Job_TypeRelease) {
    rv=AE_Utils_AmountToString(AE_Job_GetTotalSum(j),
			       AE_Job_GetCommodityId(j),
			       m_commodities,
			       tbuf);
    if (rv==0) {
      txt+="<right>";
      txt+=FXString(GWEN_Buffer_GetStart(tbuf));
      txt+="</right>";
    }
    GWEN_Buffer_Reset(tbuf);
  }
  txt+="\t";

  /* open amount */
  if (jt==AE_Job_TypeInvoice) {
    rv=AE_Utils_AmountToString(AE_Job_GetOpenSum(j),
			       AE_Job_GetCommodityId(j),
			       m_commodities,
			       tbuf);
    if (rv==0) {
      txt+="<right>";
      txt+=FXString(GWEN_Buffer_GetStart(tbuf));
      txt+="</right>";
    }
    GWEN_Buffer_Reset(tbuf);
  }
  txt+="\t";

  /* flags */
  txt+="test\t";

  /* get icon */
  switch(jt) {
  case AE_Job_TypeRequest:      ic=m_icon_doc; break;
  case AE_Job_TypeOffer:        ic=m_icon_doc; break;
  case AE_Job_TypeOrder:        ic=m_icon_doc; break;
  case AE_Job_TypeConfirmation: ic=m_icon_doc; break;
  case AE_Job_TypeRelease:      ic=m_icon_doc; break;
  case AE_Job_TypeDelivery:     ic=m_icon_van; break;
  case AE_Job_TypeInvoice:      ic=m_icon_bill; break;
  case AE_Job_TypePayment:      ic=m_icon_payment; break;
  case AE_Job_TypeReminder:     ic=m_icon_doc; break;
  case AE_Job_TypeCancellation: ic=m_icon_doc; break;

  case AE_Job_TypeUnknown:
  case AE_Job_TypeNone:
    break;
  }

  /* done */
  GWEN_Buffer_free(tbuf);
  item=new FF_JobTreeItem(txt,
			  flags,
			  j,
                          ic, ic);
  appendItem(itemFather, item, false);
}



void FF_JobTree::addJob(const AE_JOB *j, uint32_t flags) {
  addJob(AE_Job_dup(j), flags);
}



FF_JobTree::FF_JobTree(): FF_GroupTree() {
}



AE_JOB *FF_JobTree::getCurrentJob() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem)
	return jItem->getJob();
      else {
        DBG_ERROR(0, "Item is no job");
      }
    }
    else {
      DBG_ERROR(0, "Item is no group");
    }
  }
  else {
    DBG_ERROR(0, "No current item");
  }

  return NULL;
}



AE_JOB *FF_JobTree::getFirstSelectedJob() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_JobTreeItem *jItem;

        jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
        if (jItem)
          return jItem->getJob();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_JobTree::getSelectedJobs(AE_JOB_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_JobTreeItem *jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
        if (jItem)
          AE_Job_List2_PushBack(al, jItem->getJob());
      }
    }
    item=item->getBelow();
  }
}



void FF_JobTree::selectJob(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
        AE_JOB *j;

        j=jItem->getJob();
        if (AE_Job_GetId(j)==id) {
          jItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_JobTree::selectJob(const AE_JOB *j, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
	const AE_JOB *ij;

        ij=jItem->getJob();
        if (ij==j) {
	  jItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_JobTree::setCurrentJob(const AE_JOB *j, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
	const AE_JOB *ij;

        ij=jItem->getJob();
        if (ij==j) {
	  setCurrentItem(jItem, b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_JobTree::setCurrentJob(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
	const AE_JOB *j;

        j=jItem->getJob();
	if (AE_Job_GetId(j)==id) {
	  setCurrentItem(jItem, b);
	  break;
	}
      }
    }
    item=item->getBelow();
  }
}



void FF_JobTree::getJobs(AE_JOB_LIST2 *al, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_NONE:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_JobTreeItem *jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
	if (jItem)
	  AE_Job_List2_PushBack(al, jItem->getJob());
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_JobTreeItem *jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
	if (jItem) {
	  AE_JOB *j;

	  j=jItem->getJob();
	  assert(j);
	  AE_Job_List2_PushBack(al, AE_Job_dup(j));
	}
      }
      item=item->getBelow();
    }
    break;

  default:
  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_JobTreeItem *jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
	if (jItem) {
	  AE_JOB *j;

	  j=jItem->getJob();
	  assert(j);
	  AE_Job_Attach(j);
	  AE_Job_List2_PushBack(al, j);
	}
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_JobTree::makeJobVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
        AE_JOB *j;

        j=jItem->getJob();
	if (AE_Job_GetId(j)==id) {
	  FXFoldingItem *parent;

	  expandTree(jItem, TRUE);
	  parent=jItem->getParent();
	  while(parent) {
	    expandTree(parent, TRUE);
            parent=parent->getParent();
	  }

	  makeItemVisible(jItem);
	  break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_JobTree::openTaskItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_JobTreeItem *jItem;

      jItem=dynamic_cast<FF_JobTreeItem*>(grItem);
      if (jItem) {
	AE_JOB *j;

	j=jItem->getJob();
	if (AE_Job_GetTaskId(j)==id) {
	  expandTree(jItem, TRUE);
	  break;
	}
      }
    }
    item=item->getBelow();
  }
}






#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_JobTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                    \
                                                const FXFoldingItem *it2) {                  \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_JobTreeItem *tti1=dynamic_cast<const FF_JobTreeItem*>(it1);           \
      const FF_JobTreeItem *tti2=dynamic_cast<const FF_JobTreeItem*>(it2);           \
      const AE_JOB *ba1=tti1->getJob();                                                      \
      const AE_JOB *ba2=tti2->getJob();                                                      \
      const char *s1=AE_Job_Get##macro_name(ba1);                                            \
      const char *s2=AE_Job_Get##macro_name(ba2);                                            \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define DEF_SORT_FN_UP_INT(macro_num, macro_name)                                            \
FXint FF_JobTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                    \
                                                const FXFoldingItem *it2) {                  \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_JobTreeItem *tti1=dynamic_cast<const FF_JobTreeItem*>(it1);           \
      const FF_JobTreeItem *tti2=dynamic_cast<const FF_JobTreeItem*>(it2);           \
      const AE_JOB *ba1=tti1->getJob();                                                      \
      const AE_JOB *ba2=tti2->getJob();                                                      \
      int s1=AE_Job_Get##macro_name(ba1);                                                    \
      int s2=AE_Job_Get##macro_name(ba2);                                                    \
                                                                                             \
      /* both are data items */                                                              \
      if (s1==s2)                                                                            \
        return 0;                                                                            \
      else if (s1>s2)                                                                        \
        return 1;                                                                            \
      else                                                                                   \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_UP_DATE(macro_num, macro_name)                                           \
FXint FF_JobTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                        \
                                            const FXFoldingItem *it2) {                      \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_JobTreeItem *tti1=dynamic_cast<const FF_JobTreeItem*>(it1);                   \
      const FF_JobTreeItem *tti2=dynamic_cast<const FF_JobTreeItem*>(it2);                   \
      const AE_JOB *ba1=tti1->getJob();                                                      \
      const AE_JOB *ba2=tti2->getJob();                                                      \
      const GWEN_DATE *s1=AE_Job_Get##macro_name(ba1);                                       \
      const GWEN_DATE *s2=AE_Job_Get##macro_name(ba2);                                       \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return GWEN_Date_Compare(s1, s2);                                                    \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}



#define DEF_SORT_FN_UP_VALUE(macro_num, macro_name)                                          \
FXint FF_JobTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                    \
                                                const FXFoldingItem *it2) {                  \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_JobTreeItem *tti1=dynamic_cast<const FF_JobTreeItem*>(it1);           \
      const FF_JobTreeItem *tti2=dynamic_cast<const FF_JobTreeItem*>(it2);           \
      const AE_JOB *ba1=tti1->getJob();                                                      \
      const AE_JOB *ba2=tti2->getJob();                                                      \
      const AQDB_VALUE *s1=AE_Job_Get##macro_name(ba1);                                      \
      const AQDB_VALUE *s2=AE_Job_Get##macro_name(ba2);                                      \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return AQDB_Value_Compare(s1, s2);                                                   \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_JobTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,                  \
                                                  const FXFoldingItem *it2){                 \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}






DEF_SORT_FN_UP_DATE(1, EnterDate)
DEF_SORT_FN_DOWN(1)
DEF_SORT_FN_UP_DATE(2, DueDate)
DEF_SORT_FN_DOWN(2)
DEF_SORT_FN_UP_TXT(3, Code)
DEF_SORT_FN_DOWN(3)
DEF_SORT_FN_UP_INT(4, Priority)
DEF_SORT_FN_DOWN(4)
DEF_SORT_FN_UP_TXT(6, Memo)
DEF_SORT_FN_DOWN(6)
DEF_SORT_FN_UP_VALUE(7, TotalSum)
DEF_SORT_FN_DOWN(7)
DEF_SORT_FN_UP_VALUE(8, OpenSum)
DEF_SORT_FN_DOWN(8)




#undef DEF_SORT_FN_UP_VALUE
#undef DEF_SORT_FN_UP_DATE
#undef DEF_SORT_FN_UP_INT
#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN




void FF_JobTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  case 2:
    setSortFunc(up?sortFunctionUp2:sortFunctionDown2);
    sortItems();
    break;
  case 3:
    setSortFunc(up?sortFunctionUp3:sortFunctionDown3);
    sortItems();
    break;
  case 4:
    setSortFunc(up?sortFunctionUp4:sortFunctionDown4);
    sortItems();
    break;
  case 6:
    setSortFunc(up?sortFunctionUp6:sortFunctionDown6);
    sortItems();
    break;
  case 7:
    setSortFunc(up?sortFunctionUp7:sortFunctionDown7);
    sortItems();
    break;
  case 8:
    setSortFunc(up?sortFunctionUp7:sortFunctionDown7);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}



void FF_JobTree::reload() {
  FF_App *app;
  AE_BOOK *b;
  
  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_PARTNER_LIST *pList;
    AE_COMMODITY_LIST *cList;
    int rv;

    /* read parters */
    pList=AE_Partner_List_new();
    rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, pList);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_List_free(pList);
    }
    else {
      AE_Partner_List_free(m_partners);
      m_partners=pList;
    }

    /* read commodities */
    cList=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cList);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Commodity_List_free(cList);
    }
    else {
      AE_Commodity_List_free(m_commodities);
      m_commodities=cList;
    }
  }
}



