/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_partnertree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_partner.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FF_PartnerTreeItem::FF_PartnerTreeItem(AE_PARTNER *pt,
				       FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_partner(pt)
{
  rebuild();
}



FF_PartnerTreeItem::~FF_PartnerTreeItem() {
  if (m_isDataItem && m_partner) {
    AE_Partner_free(m_partner);
  }
}



void FF_PartnerTreeItem::rebuild() {
  FXString t;
  const char *s;

  s=AE_Partner_GetLastName(m_partner);
  if (s)
    t+=s;
  t+="\t";

  s=AE_Partner_GetFirstName(m_partner);
  if (s)
    t+=s;
  t+="\t";

  setText(t);
}








FF_PartnerTree::FF_PartnerTree(FXComposite *p,
			       FXObject* tgt, FXSelector sel,
			       FXuint opts,
			       FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Last Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("First Name"));
  setHeaderSize(idx++, 100);
}



FF_PartnerTree::~FF_PartnerTree() {
}



FF_PartnerTreeItem *FF_PartnerTree::findPartnerItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_PartnerTreeItem *res=findPartnerItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_PartnerTreeItem *FF_PartnerTree::findPartnerItem(FF_GroupTreeItem *grItem,
						    AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_PartnerTreeItem *baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
    if (baItem) {
      AE_PARTNER *ba=baItem->getPartner();
      if (AE_Partner_GetId(ba)==id)
        return baItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_PartnerTreeItem *res=findPartnerItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_PartnerTreeItem *FF_PartnerTree::findPartnerItem(const AE_PARTNER *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_PartnerTreeItem *baItem;

      baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
      if (baItem) {
        AE_PARTNER *ba;

	ba=baItem->getPartner();
	if (ba==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_PartnerTree::addPartner(AE_PARTNER *ba) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_PartnerTreeItem *item;

  gid=AE_Partner_GetGroupId(ba);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for partner not found, reassigning to root");
    }
  }
  else {
    DBG_DEBUG(AE_LOGDOMAIN, "Partner has no gid");
  }

  item=new FF_PartnerTreeItem(ba,
			      /*m_iconOpen, m_iconClosed*/
			      NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



void FF_PartnerTree::addPartner(const AE_PARTNER *ba) {
  addPartner(AE_Partner_dup(ba));
}



FF_PartnerTree::FF_PartnerTree(): FF_GroupTree() {
}



void FF_PartnerTree::create() {
  FF_GroupTree::create();
}



AE_PARTNER *FF_PartnerTree::getCurrentPartner() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_PartnerTreeItem *baItem;

      baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
      if (baItem)
        return baItem->getPartner();
    }
  }

  return NULL;
}



AE_PARTNER *FF_PartnerTree::getFirstSelectedPartner() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_PartnerTreeItem *baItem;

        baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
        if (baItem)
          return baItem->getPartner();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_PartnerTree::getSelectedPartners(AE_PARTNER_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_PartnerTreeItem *baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
        if (baItem)
          AE_Partner_List2_PushBack(al, baItem->getPartner());
      }
    }
    item=item->getBelow();
  }
}



void FF_PartnerTree::selectPartner(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_PartnerTreeItem *baItem;

      baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
      if (baItem) {
        AE_PARTNER *ba;

        ba=baItem->getPartner();
        if (AE_Partner_GetId(ba)==id) {
          baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_PartnerTree::selectPartner(const AE_PARTNER *pt, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_PartnerTreeItem *baItem;

      baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
      if (baItem) {
	const AE_PARTNER *ba;

        ba=baItem->getPartner();
        if (ba==pt) {
	  baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_PartnerTree::getPartners(AE_PARTNER_LIST2 *al, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_NONE:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_PartnerTreeItem *baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
	if (baItem)
	  AE_Partner_List2_PushBack(al, baItem->getPartner());
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_PartnerTreeItem *baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
	if (baItem) {
	  AE_PARTNER *ba;

	  ba=baItem->getPartner();
	  assert(ba);
	  AE_Partner_List2_PushBack(al, AE_Partner_dup(ba));
	}
      }
      item=item->getBelow();
    }
    break;

  default:
  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_PartnerTreeItem *baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
	if (baItem) {
	  AE_PARTNER *ba;

	  ba=baItem->getPartner();
	  assert(ba);
	  AE_Partner_Attach(ba);
	  AE_Partner_List2_PushBack(al, ba);
	}
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_PartnerTree::makePartnerVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_PartnerTreeItem *baItem;

      baItem=dynamic_cast<FF_PartnerTreeItem*>(grItem);
      if (baItem) {
        AE_PARTNER *ba;

        ba=baItem->getPartner();
        if (AE_Partner_GetId(ba)==id) {
          makeItemVisible(baItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_PartnerTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                \
                                                    const FXFoldingItem *it2) {              \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_PartnerTreeItem *tti1=dynamic_cast<const FF_PartnerTreeItem*>(it1);   \
      const FF_PartnerTreeItem *tti2=dynamic_cast<const FF_PartnerTreeItem*>(it2);   \
      const AE_PARTNER *ba1=tti1->getPartner();                                      \
      const AE_PARTNER *ba2=tti2->getPartner();                                      \
      const char *s1=AE_Partner_Get##macro_name(ba1);                                    \
      const char *s2=AE_Partner_Get##macro_name(ba2);                                    \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_PartnerTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,              \
                                                      const FXFoldingItem *it2){             \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, LastName)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, FirstName)
DEF_SORT_FN_DOWN(1)




#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



void FF_PartnerTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}





