/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_refuelingtree.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING 4
#define ICON_SPACING 4



FF_RefuelingTreeItem::FF_RefuelingTreeItem(AE_REFUELING *st, void* ptr)
:FF_GroupTreeItem(NULL, NULL, ptr)
,m_refueling(st)
{
}



FF_RefuelingTreeItem::~FF_RefuelingTreeItem() {
  if (m_isDataItem && m_refueling) {
    AE_Refueling_free(m_refueling);
    m_refueling=(AE_REFUELING*) -1;
  }
}



FXString FF_RefuelingTreeItem::getItemText(const FXXCheckTree* list) const {
  const FF_RefuelingTree *tree;
  FXString t;
  uint32_t flags;

  tree=dynamic_cast<const FF_RefuelingTree*>(list);
  if (tree) {
    int i;

    for (i=0; i<tree->m_columns; i++) {
      const char *s;

      switch(tree->m_columnTypes[i]) {

      case FF_RefuelingTree::COLUMN_PETROLSTATION:
        s=AE_Refueling_GetPetrolStation(m_refueling);
        if (s)
          t+=FXString(s);
        break;

      case FF_RefuelingTree::COLUMN_FUELAMOUNT:
        {
          const AQDB_VALUE *v;
          int units;
          GWEN_BUFFER *tbuf;

          units=AE_Refueling_GetFuelAmountUnits(m_refueling);
          v=AE_Refueling_GetFuelAmount(m_refueling);
          tbuf=GWEN_Buffer_new(0, 256, 0, 1);

          t+="<right>";

          AE_Utils_ValueAndUnitsToString(v, units, tbuf, 2);
          t+=GWEN_Buffer_GetStart(tbuf);
          GWEN_Buffer_free(tbuf);

          t+="</right>";
        }
        break;

      case FF_RefuelingTree::COLUMN_FUELPRICE:
        {
          const AQDB_VALUE *v;
          AQDB_ID coId;

          coId=AE_Refueling_GetCommodityId(m_refueling);
          v=AE_Refueling_GetFuelPrice(m_refueling);
          if (v) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

            t+="<right>";

            AQDB_Value_toHumanReadableString(v, tbuf, 2);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
            if (coId) {
              if (tree->m_commodityList) {
                const AE_COMMODITY *co;

                co=AE_Commodity_List_GetById(tree->m_commodityList, coId);
                if (co) {
                  const char *s;

                  s=AE_Commodity_GetSymbol(co);
                  if (!(s && *s))
                    s=AE_Commodity_GetCode(co);
                  if (!(s && *s))
                    s=AE_Commodity_GetName(co);
                  if (s && *s) {
                    t+=" ";
                    t+=s;
                  }
                }
              }
            }
            t+="</right>";
          }
        }
        break;

      case FF_RefuelingTree::COLUMN_MILEAGE:
        {
          const AQDB_VALUE *v;
          int units;
          GWEN_BUFFER *tbuf;

          units=AE_Refueling_GetMileageUnits(m_refueling);
          v=AE_Refueling_GetMileage(m_refueling);
          tbuf=GWEN_Buffer_new(0, 256, 0, 1);

          t+="<right>";

          AE_Utils_ValueAndUnitsToString(v, units, tbuf, 2);
          t+=GWEN_Buffer_GetStart(tbuf);
          GWEN_Buffer_free(tbuf);

          t+="</right>";
        }
        break;

      case FF_RefuelingTree::COLUMN_DATE:
        {
          const GWEN_DATE *ti;

          ti=AE_Refueling_GetDate(m_refueling);
          if (ti) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
          }
        }
        break;

      case FF_RefuelingTree::COLUMN_MEMO:
        s=AE_Refueling_GetMemo(m_refueling);
        if (s)
          t+=FXString(s);
        break;

      case FF_RefuelingTree::COLUMN_FLAGS:
        flags=AE_Refueling_GetFlags(m_refueling);
        if (flags & AE_REFUELING_FLAGS_FULL)
          t+=FXString("F");
        break;


      default:
        break;
      }
      t+="\t";
    } /* for */
  }

  return t;
}









FF_RefuelingTree::FF_RefuelingTree(FXComposite *p,
                                   FXObject* tgt, FXSelector sel,
                                   FXuint opts,
                                   FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_columns(0)
,m_commodityList(NULL)
{
}



void FF_RefuelingTree::addColumn(int c, int w) {
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_PETROLSTATION:
      appendHeader(I18N("AE_Refueling|Petrol Station"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_FUELAMOUNT:
      appendHeader(I18N("AE_Refueling|Fuel Amount"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_FUELPRICE:
      appendHeader(I18N("AE_Refueling|Fuel Price"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_MILEAGE:
      appendHeader(I18N("AE_Refueling|Mileage"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_DATE:
      appendHeader(I18N("AE_Refueling|Date"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_MEMO:
      appendHeader(I18N("AE_Refueling|Memo"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_FLAGS:
      appendHeader(I18N("AE_Refueling|Flags"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    default:
      break;
    }
  }
}



FF_RefuelingTree::~FF_RefuelingTree() {
  AE_Commodity_List_free(m_commodityList);
}



FF_RefuelingTreeItem *FF_RefuelingTree::findRefuelingItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_RefuelingTreeItem *res=findRefuelingItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_RefuelingTreeItem *FF_RefuelingTree::findRefuelingItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_RefuelingTreeItem *stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
    if (stItem) {
      AE_REFUELING *st=stItem->getRefueling();
      if (AE_Refueling_GetId(st)==id)
        return stItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_RefuelingTreeItem *res=findRefuelingItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_RefuelingTreeItem *FF_RefuelingTree::findRefuelingItem(const AE_REFUELING *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_RefuelingTreeItem *baItem;

      baItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
      if (baItem) {
        AE_REFUELING *sta;

	sta=baItem->getRefueling();
	if (sta==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_RefuelingTree::addRefueling(AE_REFUELING *st, bool direct, int useColor) {
  FF_RefuelingTreeItem *item;

  item=new FF_RefuelingTreeItem(st);
  /* maybe colorize */
  switch(useColor) {
  case FF_RefuelingTree::USECOLOR_NOTICE:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(247, 248, 96));
    item->setTextColor(FXRGB(0, 0, 0));
    break;
  case FF_RefuelingTree::USECOLOR_WARNING:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(255, 127, 0));
    item->setTextColor(FXRGB(255, 255, 255));
    break;

  case FF_RefuelingTree::USECOLOR_NORMAL:
  default:
    item->setUseOwnColors(false);
    break;
  }

  appendItem(NULL, item, false);
}



FF_RefuelingTree::FF_RefuelingTree(): FF_GroupTree() {
}



void FF_RefuelingTree::create() {
  FF_GroupTree::create();
}



AE_REFUELING *FF_RefuelingTree::getCurrentRefueling() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_RefuelingTreeItem *stItem;

      stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
      if (stItem)
        return stItem->getRefueling();
    }
  }

  return NULL;
}



AE_REFUELING *FF_RefuelingTree::getFirstSelectedRefueling() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_RefuelingTreeItem *stItem;

        stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
        if (stItem)
          return stItem->getRefueling();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_RefuelingTree::addRefuelings(AE_REFUELING_LIST2 *sl, int copyMode, bool direct) {
  AE_REFUELING_LIST2_ITERATOR *it;

  it=AE_Refueling_List2_First(sl);
  if (it) {
    AE_REFUELING *st;

    st=AE_Refueling_List2Iterator_Data(it);
    switch(copyMode) {
    case COPYMODE_COPY:
      while(st) {
        addRefueling(AE_Refueling_dup(st), direct, USECOLOR_NORMAL);
        st=AE_Refueling_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_ATTACH:
      while(st) {
        AE_Refueling_Attach(st);
        addRefueling(st, direct, USECOLOR_NORMAL);
        st=AE_Refueling_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_NONE:
    default:
      while(st) {
        addRefueling(st, direct, USECOLOR_NORMAL);
        st=AE_Refueling_List2Iterator_Next(it);
      }
      break;
    }
    AE_Refueling_List2Iterator_free(it);
  }
}



void FF_RefuelingTree::getSelectedRefuelings(AE_REFUELING_LIST2 *sl, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_RefuelingTreeItem *stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
          if (stItem)
            AE_Refueling_List2_PushBack(sl, AE_Refueling_dup(stItem->getRefueling()));
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_RefuelingTreeItem *stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
	  if (stItem) {
	    AE_REFUELING *st=stItem->getRefueling();
            AE_Refueling_Attach(st);
            AE_Refueling_List2_PushBack(sl, st);
          }
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_NONE:
  default:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_RefuelingTreeItem *stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
          if (stItem)
            AE_Refueling_List2_PushBack(sl, stItem->getRefueling());
        }
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_RefuelingTree::selectRefueling(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_RefuelingTreeItem *stItem;

      stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
      if (stItem) {
        AE_REFUELING *st;

        st=stItem->getRefueling();
        if (AE_Refueling_GetId(st)==id) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_RefuelingTree::selectRefueling(const AE_REFUELING *fst, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_RefuelingTreeItem *stItem;

      stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
      if (stItem) {
        if (stItem->getRefueling()==fst) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_RefuelingTree::makeRefuelingVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_RefuelingTreeItem *stItem;

      stItem=dynamic_cast<FF_RefuelingTreeItem*>(grItem);
      if (stItem) {
        AE_REFUELING *st;

        st=stItem->getRefueling();
        if (AE_Refueling_GetId(st)==id) {
          makeItemVisible(stItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




GROUPTREE_SORT_FN_UP_TXT(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, PetrolStation)
GROUPTREE_SORT_FN_DOWN(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, PetrolStation)

GROUPTREE_SORT_FN_UP_TXT(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, Memo)
GROUPTREE_SORT_FN_DOWN(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, Memo)

GROUPTREE_SORT_FN_UP_VALUE(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, FuelAmount)
GROUPTREE_SORT_FN_DOWN(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, FuelAmount)

GROUPTREE_SORT_FN_UP_VALUE(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, FuelPrice)
GROUPTREE_SORT_FN_DOWN(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, FuelPrice)

GROUPTREE_SORT_FN_UP_VALUE(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, Mileage)
GROUPTREE_SORT_FN_DOWN(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, Mileage)

GROUPTREE_SORT_FN_UP_DATE(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, Date)
GROUPTREE_SORT_FN_DOWN(FF_RefuelingTree, FF_RefuelingTreeItem, getRefueling, AE_REFUELING, AE_Refueling, Date)






void FF_RefuelingTree::sortByColumn(int c, bool up) {
  if (c<m_columns) {
    switch(m_columnTypes[c]) {
    case COLUMN_PETROLSTATION:
      setSortFunc(up?sortFunctionUpPetrolStation:sortFunctionDownPetrolStation);
      sortItems();
      break;
    case COLUMN_FUELAMOUNT:
      setSortFunc(up?sortFunctionUpFuelAmount:sortFunctionDownFuelAmount);
      sortItems();
      break;
    case COLUMN_FUELPRICE:
      setSortFunc(up?sortFunctionUpFuelPrice:sortFunctionDownFuelPrice);
      sortItems();
      break;
    case COLUMN_MILEAGE:
      setSortFunc(up?sortFunctionUpMileage:sortFunctionDownMileage);
      sortItems();
      break;
    case COLUMN_DATE:
      setSortFunc(up?sortFunctionUpDate:sortFunctionDownDate);
      sortItems();
      break;
    case COLUMN_MEMO:
      setSortFunc(up?sortFunctionUpMemo:sortFunctionDownMemo);
      sortItems();
      break;

    default:
      DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
      break;
    }
  }
}



void FF_RefuelingTree::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_COMMODITY_LIST *col;
    int rv;

    col=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, col);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Commodity_List_free(col);
    }
    else {
      AE_Commodity_List_free(m_commodityList);
      m_commodityList=col;
    }
  }
}



