/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_report2settingstree.hpp"

#include <fx.h>


#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FF_Report2SettingsTreeItem::FF_Report2SettingsTreeItem(AF_REPORT2_SETTINGS *settings,
                                                       FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_settings(settings)
{
  rebuild();
}



FF_Report2SettingsTreeItem::~FF_Report2SettingsTreeItem() {
  if (m_isDataItem && m_settings) {
    AF_Report2_Settings_free(m_settings);
  }
}



void FF_Report2SettingsTreeItem::rebuild() {
  FXString t;
  const char *s;

  s=AF_Report2_Settings_GetName(m_settings);
  if (s)
    t+=s;
  t+="\t";

  s=AF_Report2_Settings_GetDescription(m_settings);
  if (s)
    t+=s;
  t+="\t";

  setText(t);
}








FF_Report2SettingsTree::FF_Report2SettingsTree(FXComposite *p,
                                               FXObject* tgt, FXSelector sel,
                                               FXuint opts,
                                               FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Name"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Description"));
  setHeaderSize(idx++, 100);
}



FF_Report2SettingsTree::~FF_Report2SettingsTree() {
}



FF_Report2SettingsTreeItem *FF_Report2SettingsTree::findSettingsItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_Report2SettingsTreeItem *res=findSettingsItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_Report2SettingsTreeItem *FF_Report2SettingsTree::findSettingsItem(const char *id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && grItem->isDataItem()) {
      FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
      if (baItem) {
        AF_REPORT2_SETTINGS *ba;
        const char *s;

        ba=baItem->getSettings();
        s=AF_Report2_Settings_GetName(ba);
        if (s && id && strcasecmp(s, id)==0)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



FF_Report2SettingsTreeItem *FF_Report2SettingsTree::findSettingsItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
    if (baItem) {
      AF_REPORT2_SETTINGS *ba=baItem->getSettings();
      if (AF_Report2_Settings_GetId(ba)==id)
        return baItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_Report2SettingsTreeItem *res=findSettingsItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_Report2SettingsTreeItem *FF_Report2SettingsTree::findSettingsItem(const AF_REPORT2_SETTINGS *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_Report2SettingsTreeItem *baItem;

      baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
      if (baItem) {
        AF_REPORT2_SETTINGS *ba;

	ba=baItem->getSettings();
	if (ba==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_Report2SettingsTree::addSettings(AF_REPORT2_SETTINGS *ba) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_Report2SettingsTreeItem *item;

  gid=AF_Report2_Settings_GetGroupId(ba);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for partner not found, reassigning to root");
    }
  }
  else {
    DBG_DEBUG(AE_LOGDOMAIN, "Settings has no gid");
  }

  item=new FF_Report2SettingsTreeItem(ba,
			      /*m_iconOpen, m_iconClosed*/
			      NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



void FF_Report2SettingsTree::addSettings(const AF_REPORT2_SETTINGS *ba) {
  addSettings(AF_Report2_Settings_dup(ba));
}



FF_Report2SettingsTree::FF_Report2SettingsTree(): FF_GroupTree() {
}



void FF_Report2SettingsTree::create() {
  FF_GroupTree::create();
}



AF_REPORT2_SETTINGS *FF_Report2SettingsTree::getCurrentSettings() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_Report2SettingsTreeItem *baItem;

      baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
      if (baItem)
        return baItem->getSettings();
    }
  }

  return NULL;
}



AF_REPORT2_SETTINGS *FF_Report2SettingsTree::getFirstSelectedSettings() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_Report2SettingsTreeItem *baItem;

        baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
        if (baItem)
          return baItem->getSettings();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_Report2SettingsTree::getSelectedSettings(AF_REPORT2_SETTINGS_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
        if (baItem)
          AF_Report2_Settings_List2_PushBack(al, baItem->getSettings());
      }
    }
    item=item->getBelow();
  }
}



void FF_Report2SettingsTree::selectSettings(const char *id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_Report2SettingsTreeItem *baItem;

      baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
      if (baItem) {
        AF_REPORT2_SETTINGS *ba;
        const char *s;

        ba=baItem->getSettings();
        s=AF_Report2_Settings_GetName(ba);
        if (s && id && strcasecmp(id, s)==0) {
          baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_Report2SettingsTree::selectSettings(const AF_REPORT2_SETTINGS *settings, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_Report2SettingsTreeItem *baItem;

      baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
      if (baItem) {
	const AF_REPORT2_SETTINGS *ba;

        ba=baItem->getSettings();
        if (ba==settings) {
	  baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_Report2SettingsTree::getSettings(AF_REPORT2_SETTINGS_LIST2 *al, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_NONE:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
	if (baItem)
	  AF_Report2_Settings_List2_PushBack(al, baItem->getSettings());
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
	if (baItem) {
	  AF_REPORT2_SETTINGS *ba;

	  ba=baItem->getSettings();
	  assert(ba);
	  AF_Report2_Settings_List2_PushBack(al, AF_Report2_Settings_dup(ba));
	}
      }
      item=item->getBelow();
    }
    break;

  default:
  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_Report2SettingsTreeItem *baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
	if (baItem) {
	  AF_REPORT2_SETTINGS *ba;

	  ba=baItem->getSettings();
	  assert(ba);
	  AF_Report2_Settings_Attach(ba);
	  AF_Report2_Settings_List2_PushBack(al, ba);
	}
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_Report2SettingsTree::makeSettingsVisible(const char *id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_Report2SettingsTreeItem *baItem;

      baItem=dynamic_cast<FF_Report2SettingsTreeItem*>(grItem);
      if (baItem) {
        AF_REPORT2_SETTINGS *ba;
        const char *s;

        ba=baItem->getSettings();
        s=AF_Report2_Settings_GetName(ba);
        if (s && id && strcasecmp(id, s)==0) {
          makeItemVisible(baItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_Report2SettingsTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                \
                                                    const FXFoldingItem *it2) {              \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_Report2SettingsTreeItem *tti1=dynamic_cast<const FF_Report2SettingsTreeItem*>(it1);   \
      const FF_Report2SettingsTreeItem *tti2=dynamic_cast<const FF_Report2SettingsTreeItem*>(it2);   \
      const AF_REPORT2_SETTINGS *ba1=tti1->getSettings();                                             \
      const AF_REPORT2_SETTINGS *ba2=tti2->getSettings();                                             \
      const char *s1=AF_Report2_Settings_Get##macro_name(ba1);                               \
      const char *s2=AF_Report2_Settings_Get##macro_name(ba2);                               \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
  FXint FF_Report2SettingsTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,        \
                                                            const FXFoldingItem *it2){       \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, Name)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, Description)
DEF_SORT_FN_DOWN(1)




#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



void FF_Report2SettingsTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}





