/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ff_ruletree.hpp"

#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FF_RuleTreeItem::FF_RuleTreeItem(AE_RULE *r)
:FXFoldingItem("", NULL, NULL, NULL)
,m_rule(r) {
  updateRule();
}



FF_RuleTreeItem::~FF_RuleTreeItem() {
  AE_Rule_free(m_rule);
}



void FF_RuleTreeItem::updateRule() {
  const char *s;
  FXString t;
  FXString mode;

  switch(AE_Rule_GetCombineMode(m_rule)) {
  case AE_RuleCombineMode_Or:       mode="||"; break;
  case AE_RuleCombineMode_And:      mode="&&"; break;
  case AE_RuleCombineMode_Unknown:  mode=","; break;
  }

  s=AE_Rule_GetExpression(m_rule);
  if (s && *s) {
    while(*s) {
      if (*s=='\t')
        t+=' ';
      else if (*s=='\n') {
	if (s[1])
	  t+=mode+" ";
      }
      else
        t+=*s;
      s++;
    }
  }
  setText(t);
}




FXDEFMAP(FF_RuleTree) FF_RuleTreeMap[]={
};


            
FXIMPLEMENT(FF_RuleTree, FXFoldingList, FF_RuleTreeMap, ARRAYNUMBER(FF_RuleTreeMap))




FF_RuleTree::FF_RuleTree(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
                         FXint x, FXint y, FXint w, FXint h)
:FXFoldingList(p, tgt, sel, opts, x, y, w, h)
{
  int idx=0;

  flags|=FLAG_ENABLED | FLAG_SHOWN;

  appendHeader(I18N("Rule"));
  setHeaderSize(idx++, 100);

}



FF_RuleTree::FF_RuleTree()
:FXFoldingList()
{
}



FF_RuleTree::~FF_RuleTree() {
}



FF_RuleTreeItem *FF_RuleTree::findRuleItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_RuleTreeItem *grItem;

    grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      AE_RULE *r=grItem->getRule();
      if (r && AE_Rule_GetId(r)==id)
        return grItem;
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_RuleTree::addRule(const AE_RULE *r) {
  FXFoldingItem *item=new FF_RuleTreeItem(AE_Rule_dup(r));
  appendItem(NULL, item, false);
}



void FF_RuleTree::addRules(const AE_RULE_LIST *rl) {
  AE_RULE *r;

  r=AE_Rule_List_First(rl);
  while(r) {
    addRule(r);
    r=AE_Rule_List_Next(r);
  }
}



AE_RULE *FF_RuleTree::getCurrentRule() {
  FXFoldingItem *item;

  item=getCurrentItem();

  while (item) {
    FF_RuleTreeItem *grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      return grItem->getRule();
    }
    item=item->getParent();
  }

  return NULL;
}



AE_RULE *FF_RuleTree::getFirstSelectedRule() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_RuleTreeItem *grItem=dynamic_cast<FF_RuleTreeItem*>(item);
      if (grItem) {
        return grItem->getRule();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_RuleTree::selectRule(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_RuleTreeItem *grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      AE_RULE *r=grItem->getRule();
      if (AE_Rule_GetId(r)==id) {
        item->setSelected(b);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_RuleTree::makeRuleVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_RuleTreeItem *grItem=dynamic_cast<FF_RuleTreeItem*>(item);
    if (grItem) {
      AE_RULE *r=grItem->getRule();
      if (AE_Rule_GetId(r)==id) {
        makeItemVisible(item);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_RuleTree::expandItems() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    expandTree(item, false);
    item=item->getBelow();
  }
}



void FF_RuleTree::create() {
  FXFoldingList::create();
}





