/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_sessionlist.hpp"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




class FF_SessionListItem: public FXFoldingItem {
  FF_SessionListItem():FXFoldingItem() {};
public:
  FF_SessionListItem(AQDB_SESSION_DESCR *sd,
                     FXIcon* icon=NULL);
  ~FF_SessionListItem();

  AQDB_SESSION_DESCR *getSession() const;

};





FF_SessionListItem::FF_SessionListItem(AQDB_SESSION_DESCR *sd, FXIcon* icon)
:FXFoldingItem("", icon) {
  FXString t;
  const char *s;
  AQDB_ID id;
  char buf[32];
  const GWEN_TIME *ti;

  id=AQDB_SessionDescr_GetSessionId(sd);
  snprintf(buf, sizeof(buf)-1, "%lu", (unsigned long int)id);
  buf[sizeof(buf)-1]=0;
  t=buf;
  t+="\t";

  ti=AQDB_SessionDescr_GetDateTime(sd);
  if (ti) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 32, 0, 1);
    GWEN_Time_toString(ti, I18N("YYYY/MM/DD-hh:mm:ss"), tbuf);
    t+=GWEN_Buffer_GetStart(tbuf);
    GWEN_Buffer_free(tbuf);
  }
  t+="\t";

  s=AQDB_SessionDescr_GetUserName(sd);
  if (s)
    t+=s;
  t+="\t";

  snprintf(buf, sizeof(buf)-1, "%d", (int)AQDB_SessionDescr_GetAdditions(sd));
  t+=buf;
  t+="\t";

  snprintf(buf, sizeof(buf)-1, "%d", (int)AQDB_SessionDescr_GetDeletions(sd));
  t+=buf;
  t+="\t";

  snprintf(buf, sizeof(buf)-1, "%d", (int)AQDB_SessionDescr_GetModifications(sd));
  t+=buf;
  t+="\t";

  setText(t);
  setData((void*) sd);
}



FF_SessionListItem::~FF_SessionListItem() {
  AQDB_SESSION_DESCR *sd;

  sd=(AQDB_SESSION_DESCR*) getData();
  if (sd) {
    AQDB_SessionDescr_free(sd);
    setData(NULL);
  }
}



AQDB_SESSION_DESCR *FF_SessionListItem::getSession() const {
  return (AQDB_SESSION_DESCR*) getData();
}






FF_SessionList::FF_SessionList()
:FXFoldingList() {

}



FF_SessionList::FF_SessionList(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
                               FXint x, FXint y, FXint w, FXint h)
:FXFoldingList(p, tgt, sel, opts, x, y, w, h) {
  appendHeader(I18N("Id"), NULL, 100);
  appendHeader(I18N("Timestamp"), NULL, 100);
  appendHeader(I18N("User"), NULL, 100);
  appendHeader(I18N("Additions"), NULL, 100);
  appendHeader(I18N("Deletions"), NULL, 100);
  appendHeader(I18N("Modifications"), NULL, 100);
}



FF_SessionList::~FF_SessionList() {
}



void FF_SessionList::addSession(AQDB_SESSION_DESCR *sd) {
  FF_SessionListItem *item;

  item=new FF_SessionListItem(sd, NULL);
  appendItem(NULL, item, false);
}



void FF_SessionList::removeSession(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_SessionListItem *sItem=dynamic_cast<FF_SessionListItem*>(item);
    if (sItem) {
      AQDB_SESSION_DESCR *sd=sItem->getSession();
      if (AQDB_SessionDescr_GetSessionId(sd)==id) {
        removeItem(sItem);
        break;
      }
    }
    item=item->getBelow();
  }
}



AQDB_SESSION_DESCR *FF_SessionList::getCurrentSession() {
  FXFoldingItem *item=getCurrentItem();
  if (item) {
    FF_SessionListItem *sItem=dynamic_cast<FF_SessionListItem*>(item);
    if (sItem)
      return sItem->getSession();
  }

  return NULL;
}



AQDB_SESSION_DESCR *FF_SessionList::getFirstSelectedSession() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_SessionListItem *sItem=dynamic_cast<FF_SessionListItem*>(item);
    if (sItem && sItem->isSelected())
      return sItem->getSession();
    item=item->getBelow();
  }

  return NULL;
}



void FF_SessionList::selectSession(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_SessionListItem *sItem=dynamic_cast<FF_SessionListItem*>(item);
    if (sItem) {
      AQDB_SESSION_DESCR *sd=sItem->getSession();

      if (AQDB_SessionDescr_GetSessionId(sd)==id) {
        selectItem(sItem, false);
        break;
      }
    }
    item=item->getBelow();
  }
}



void FF_SessionList::deselectSession(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_SessionListItem *sItem=dynamic_cast<FF_SessionListItem*>(item);
    if (sItem) {
      AQDB_SESSION_DESCR *sd=sItem->getSession();

      if (AQDB_SessionDescr_GetSessionId(sd)==id) {
        deselectItem(sItem, false);
        break;
      }
    }
    item=item->getBelow();
  }
}



FXint FF_SessionList::sortFunctionUpUserName(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_SessionListItem *sItem1=dynamic_cast<const FF_SessionListItem*>(it1);
  const FF_SessionListItem *sItem2=dynamic_cast<const FF_SessionListItem*>(it2);

  if (sItem1 && sItem2) {
    const AQDB_SESSION_DESCR *session1;
    const AQDB_SESSION_DESCR *session2;

    session1=sItem1->getSession();
    session2=sItem2->getSession();

    if (session1 && session2) {
      const char *s1;
      const char *s2;

      s1=AQDB_SessionDescr_GetUserName(session1);
      s2=AQDB_SessionDescr_GetUserName(session2);
      if (s1 && s2) {
        return strcmp(s1, s2);
      }
      else if (s1 && s2==NULL)
        return 1;
      else if (s1==NULL && s2)
        return -1;
    }
  }
  return 0;
}



FXint FF_SessionList::sortFunctionDownUserName(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpUserName(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_SessionList::sortFunctionUpSessionId(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_SessionListItem *sItem1=dynamic_cast<const FF_SessionListItem*>(it1);
  const FF_SessionListItem *sItem2=dynamic_cast<const FF_SessionListItem*>(it2);

  if (sItem1 && sItem2) {
    const AQDB_SESSION_DESCR *session1;
    const AQDB_SESSION_DESCR *session2;

    session1=sItem1->getSession();
    session2=sItem2->getSession();

    if (session1 && session2) {
      AQDB_ID id1;
      AQDB_ID id2;

      id1=AQDB_SessionDescr_GetSessionId(session1);
      id2=AQDB_SessionDescr_GetSessionId(session2);
      if (id1<id2)
        return 1;
      else if (id2>id1)
        return -1;
    }
  }
  return 0;
}



FXint FF_SessionList::sortFunctionDownSessionId(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpSessionId(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_SessionList::sortFunctionUpDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_SessionListItem *sItem1=dynamic_cast<const FF_SessionListItem*>(it1);
  const FF_SessionListItem *sItem2=dynamic_cast<const FF_SessionListItem*>(it2);

  if (sItem1 && sItem2) {
    const AQDB_SESSION_DESCR *session1;
    const AQDB_SESSION_DESCR *session2;

    session1=sItem1->getSession();
    session2=sItem2->getSession();

    if (session1 && session2) {
      const GWEN_TIME *ti1;
      const GWEN_TIME *ti2;

      ti1=AQDB_SessionDescr_GetDateTime(session1);
      ti2=AQDB_SessionDescr_GetDateTime(session2);
      if (ti1 && ti2) {
        return GWEN_Time_Compare(ti1, ti2);
      }
      else if (ti1 && ti2==NULL)
        return 1;
      else if (ti1==NULL && ti2)
        return -1;
    }
  }
  return 0;
}



FXint FF_SessionList::sortFunctionDownDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpDate(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



void FF_SessionList::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUpSessionId:sortFunctionDownSessionId);
    break;
  case 1:
    setSortFunc(up?sortFunctionUpDate:sortFunctionDownDate);
    break;

  case 2:
    setSortFunc(up?sortFunctionUpUserName:sortFunctionDownUserName);
    break;
  default:
    return;
  }
  sortItems();

}



void FF_SessionList::setHeaderArrowDir(int c, FXbool b) {
  header->setArrowDir(c, b);
}



FXbool FF_SessionList::getHeaderArrowDir(int c) const {
  return header->getArrowDir(c);
}



long FF_SessionList::onHeaderClicked(FXObject*, FXSelector, void* ptr) {
  FXint idx=(FXint)(FXival)ptr;
  FXbool b;
  int i;

  b=header->getArrowDir(idx);
  for (i=0; i<header->getNumItems(); i++)
    header->setArrowDir(i, MAYBE);

  if (b==TRUE) {
    /* was up, becomes down */
    header->setArrowDir(idx, FALSE);
    sortByColumn(idx, false);
  }
  else if (b==FALSE) {
    /* was down, becomes MAYBE */
    header->setArrowDir(idx, MAYBE);
  }
  else {
    /* was MAYBE, becomes up */
    header->setArrowDir(idx, TRUE);
    sortByColumn(idx, true);
  }
  return 1;
}



void FF_SessionList::sort() {
  int i;

  for (i=0; i<getNumHeaders(); i++) {
    FXbool b;

    b=getHeaderArrowDir(i);
    if (b!=MAYBE) {
      sortByColumn(i, b);
      break;
    }
  }
}



