/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_splittable.hpp"
#include "ff_app.hpp"
#include "ff_accountedit.hpp"
#include "ff_valueedit.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#include <assert.h>


#define SPLIT_BUNCH 256

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


FXDEFMAP(FF_SplitTable) FF_SplitTableMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_SplitTable::ID_ACCOUNT_EDIT, FF_SplitTable::onCmdAccount),
  FXMAPFUNC(SEL_DESELECTED, FF_SplitTable::ID_ACCOUNT_EDIT, FF_SplitTable::onCancelAccount),
  FXMAPFUNC(SEL_COMMAND, FF_SplitTable::ID_AMOUNT_EDIT, FF_SplitTable::onCmdAmount),
  FXMAPFUNC(SEL_DESELECTED, FF_SplitTable::ID_AMOUNT_EDIT, FF_SplitTable::onCancelAmount),
  FXMAPFUNC(SEL_COMMAND, FF_SplitTable::ID_TEXTFIELD, FF_SplitTable::onCmdTextField),
  FXMAPFUNC(SEL_DESELECTED, FF_SplitTable::ID_TEXTFIELD, FF_SplitTable::onCancelTextField),
};


FXIMPLEMENT(FF_SplitTable, FXXTable, FF_SplitTableMap, ARRAYNUMBER(FF_SplitTableMap))



FF_SplitTable::FF_SplitTable(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
			     FXint x, FXint y, FXint w, FXint h,
			     FXint pl, FXint pr,
			     FXint pt, FXint pb)
:FXXTable(p, tgt, sel, opts, x, y, w, h, pl, pr, pt, pb)
,m_splitArray(NULL)
,m_splitArraySize(0)
,m_splitCount(0)
,m_markBeyondId(0)
,m_colorizeMatches(false)
,m_accountTree(NULL)
,m_commodityList(NULL)
{
  int i;
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());

  setMarkColor(0, app->getMarkColor1());
  /* red'ish */
  setMarkColor(1, FXColor(FXRGB(255, 209,209)));
  /* yellow */
  setMarkColor(2, FXColor(FXRGB(236, 229, 14)));
  /* green */
  setMarkColor(3, FXColor(FXRGB(177, 242, 65)));

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=0;
}



FF_SplitTable::FF_SplitTable()
:FXXTable()
,m_splitArray(NULL)
,m_splitArraySize(0)
,m_splitCount(0)
,m_colorizeMatches(false)
,m_accountTree(NULL)
,m_commodityList(NULL)
{
  int i;

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=COLUMN_TYPE_NONE;
}



FF_SplitTable::~FF_SplitTable() {
  if (m_splitArray) {
    for (int i=0; i<m_splitCount; i++)
      AE_Split_free(m_splitArray[i]);
    free(m_splitArray);
    m_splitArray=NULL;
  }

  AE_Account_Tree_free(m_accountTree);
  m_accountTree=NULL;

  AE_Commodity_List_free(m_commodityList);
  m_commodityList=NULL;
}



void FF_SplitTable::setNumberOfRows(int i) {
  /* reserve space needed for i rows */
  if (m_splitArray==NULL)
    addSpaceForSplits(i);
  if (i>m_splitArraySize)
    addSpaceForSplits(i-m_splitArraySize);
  FXXTable::setNumberOfRows(i);

  if (i<m_splitCount) {
    int j;

    /* shrinking, free all splits behind i */
    for (j=i; j<m_splitCount; j++) {
      AE_Split_free(m_splitArray[j]);
      m_splitArray[j]=NULL;
    }
  }
  m_splitCount=i;
}



void FF_SplitTable::setColorizeMatches(bool b) {
  if (b!=m_colorizeMatches) {
    m_colorizeMatches=b;
    rebuild();
  }
}



void FF_SplitTable::addColumn(int coltype, int size) {
  if ((m_ncolumns+1)<MAX_COLUMNS) {
    switch(coltype) {
    case COLUMN_TYPE_MEMO:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Memo"), size);
      break;
    case COLUMN_TYPE_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Amount"), size);
      break;
    case COLUMN_TYPE_CREDIT_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Credit Amount"), size);
      break;
    case COLUMN_TYPE_DEBIT_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Debit Amount"), size);
      break;
    case COLUMN_TYPE_ACCOUNT:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Account"), size);
      break;
    default:
      break;
    }
  }
}



FXXTable::ColumnInfo *FF_SplitTable::getColumnInfo(int r, int c) {
  if (r<m_splitCount && c<MAX_COLUMNS) {
    AE_SPLIT *st;
    FXString txt;
    FXColor bg=getCellBackColor(r, c);

    st=getSplitByRow(r);
    if (st==NULL)
      return NULL;

    if (m_accountTree==NULL)
      reload();

    if (m_colorizeMatches) {
      switch(AE_Split_GetRuntimeFlags(st) & AE_SPLIT_RTFLAGS_MATCHMASK) {
      case AE_SPLIT_RTFLAGS_MATCHAUTO1:  bg=FXColor(FXRGB(236, 229, 14)); break;
      case AE_SPLIT_RTFLAGS_MATCHAUTO2:
      case AE_SPLIT_RTFLAGS_MATCHMANUAL: bg=FXColor(FXRGB(177, 242, 65)); break;
      default:
      case AE_SPLIT_RTFLAGS_MATCHNONE:   bg=FXColor(FXRGB(255, 209,209)); break;
      }
    }

    if (m_markBeyondId!=0 && m_markBeyondId<AE_Split_GetId(st)) {
      if (r & 1)
        bg=FXColor(FF_APP_YELLOW2);
      else
        bg=FXColor(FF_APP_YELLOW1);
    }


    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_MEMO: {
      const char *s;

      s=AE_Split_GetMemo(st);
      if (s && *s) {
	txt+="<b>";
	txt+=s;
	txt+="</b><br>";
      }
      break;
    }

    case COLUMN_TYPE_VALUE: {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(st);
      if (v) {
	GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	AE_Utils_AmountToString(v,
				AE_Split_GetCommodityId(st),
				m_commodityList,
				tbuf);

	txt+="<right>";
	if (AQDB_Value_IsNegative(v))
	  txt+="<font color=red>";

	txt+=FXString(GWEN_Buffer_GetStart(tbuf));
	GWEN_Buffer_free(tbuf);

	if (AQDB_Value_IsNegative(v))
	  txt+="</font>";
	txt+="</right>";
      }
      break;
    }

    case COLUMN_TYPE_CREDIT_VALUE: {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(st);
      if (v) {
	if (AQDB_Value_IsNegative(v)) {
	  AQDB_VALUE *v2;
	  GWEN_BUFFER *tbuf;

	  v2=AQDB_Value_dup(v);
	  AQDB_Value_Negate(v2);

	  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	  AE_Utils_AmountToString(v2,
				  AE_Split_GetCommodityId(st),
				  m_commodityList,
				  tbuf);
          AQDB_Value_free(v2);

	  txt+="<right>";
	  txt+=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	  txt+="</right>";
	}
      }
      break;
    }

    case COLUMN_TYPE_DEBIT_VALUE: {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(st);
      if (v) {
	if (!AQDB_Value_IsNegative(v)) {
	  GWEN_BUFFER *tbuf;

	  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	  AE_Utils_AmountToString(v,
				  AE_Split_GetCommodityId(st),
				  m_commodityList,
				  tbuf);
	  txt+="<right>";
	  txt+=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	  txt+="</right>";
	}
      }
      break;
    }

    case COLUMN_TYPE_ACCOUNT: {
      uint32_t id;

      id=AE_Split_GetAccountId(st);
      if (id && m_accountTree) {
	AE_ACCOUNT *acc;

        acc=AE_Account_Tree_GetById(m_accountTree, id);
	if (acc) {
	  GWEN_BUFFER *tbuf;

	  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	  AE_Account_PathToBuffer(acc, tbuf);
	  txt+=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	}
      }
      break;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }

    return new ColumnInfoText(this, r, c, txt, bg);
  }
  else
    return NULL;
}



void FF_SplitTable::releaseColumnInfo(FXXTable::ColumnInfo *ci) {
  delete ci;
}




FXWindow *FF_SplitTable::getEditor(int r, int c) {
  if (r<m_splitCount && c<MAX_COLUMNS) {
    AE_SPLIT *st;
    FXString txt;

    st=getSplitByRow(r);
    if (st==NULL)
      return NULL;

    if (m_accountTree==NULL)
      reload();

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_MEMO: {
      const char *s;
      FXTextField *ed=new FXTextField(this, 1, this, ID_TEXTFIELD,
                                      TEXTFIELD_ENTER_ONLY,
                                      0, 0, 0, 0,
                                      0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      s=AE_Split_GetMemo(st);
      if (s && *s) {
	ed->setText(s);
	ed->selectAll();
      }
      return ed;
    }

    case COLUMN_TYPE_VALUE: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      const AQDB_VALUE *v;

      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      v=AE_Split_GetAmount(st);
      if (v) {
	int rv;
        GWEN_BUFFER *tbuf;

        tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	rv=AE_Utils_AmountToString(v, AE_Split_GetCommodityId(st),
				   m_commodityList,
				   tbuf);
	if (rv<0) {
	}
	else {
	  ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	  ed->selectAll();
	}
	GWEN_Buffer_free(tbuf);
      }
      return ed;
    }

    case COLUMN_TYPE_CREDIT_VALUE: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      const AQDB_VALUE *v;

      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      v=AE_Split_GetAmount(st);
      if (v && AQDB_Value_IsNegative(v)) {
	int rv;
        AQDB_VALUE *v2;
        GWEN_BUFFER *tbuf;

	v2=AQDB_Value_dup(v);
        AQDB_Value_Negate(v2);
        tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	rv=AE_Utils_AmountToString(v2, AE_Split_GetCommodityId(st),
				   m_commodityList,
				   tbuf);
	if (rv<0) {
	}
	else {
	  ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	  ed->selectAll();
	}
	GWEN_Buffer_free(tbuf);
        AQDB_Value_free(v2);
      }
      return ed;
    }

      case COLUMN_TYPE_DEBIT_VALUE: {
      FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_AMOUNT_EDIT,
					TEXTFIELD_ENTER_ONLY,
					0, 0, 0, 0,
					0, 0, 0, 0);
      const AQDB_VALUE *v;

      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);

      v=AE_Split_GetAmount(st);
      if (v && !AQDB_Value_IsNegative(v)) {
	int rv;
        GWEN_BUFFER *tbuf;

        tbuf=GWEN_Buffer_new(0, 64, 0, 1);
	rv=AE_Utils_AmountToString(v, AE_Split_GetCommodityId(st),
				   m_commodityList,
				   tbuf);
	if (rv<0) {
	}
	else {
	  ed->setText(FXString(GWEN_Buffer_GetStart(tbuf)));
	  ed->selectAll();
	}
	GWEN_Buffer_free(tbuf);
      }
      return ed;
    }

    case COLUMN_TYPE_ACCOUNT: {
      FF_AccountEdit *ed;

      ed=new FF_AccountEdit(this, this, ID_ACCOUNT_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();
      ed->setAccountId(AE_Split_GetAccountId(st));
      ed->selectAll();
      return ed;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }
  return NULL;
}



void FF_SplitTable::setFromEditor(FXXTable::ColumnInfo *ci, FXWindow *editor) {
  int r;
  int c;

  r=ci->getRow();
  c=ci->getColumn();

  if (r<m_splitCount && c<MAX_COLUMNS) {
    AE_SPLIT *st;

    st=getSplitByRow(r);
    if (st==NULL)
      return;

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_MEMO: {
      FXTextField *ed=dynamic_cast<FXTextField*>(editor);
      if (ed) {
	FXString txt=ed->getText();
	if (txt.empty())
	  AE_Split_SetMemo(st, NULL);
	else
          AE_Split_SetMemo(st, txt.text());
	AE_Split_AddRuntimeFlags(st, AE_SPLIT_RTFLAGS_MODIFIED);

        modifiedRowInfo(r);
        updateRow(r);
      }
      break;
    }

    case COLUMN_TYPE_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	FXString txt=ed->getText();
	if (!txt.empty()) {
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;
	  int rv;

	  rv=AE_Utils_AmountFromString(txt.text(), m_commodityList, &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad strng [%s] (no amount)", txt.text());
	  }
	  else {
            AE_Split_SetAmount(st, v);
            if (co==NULL) {
              AE_ACCOUNT *a;
              AQDB_ID coId=0;
      
              a=AE_Account_Tree_GetById(m_accountTree, AE_Split_GetAccountId(st));
              if (a) {
                coId=AE_Account_GetPrimaryCommodityId(a);
                if (coId==0)
                  coId=AE_Account_GetSecondaryCommodityId(a);
              }
              if (coId)
                co=AE_Commodity_List_GetById(m_commodityList, coId);
            }
            if (co)
	      AE_Split_SetCommodityId(st, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
	  }
	}
	AE_Split_AddRuntimeFlags(st, AE_SPLIT_RTFLAGS_MODIFIED);

	modifiedRowInfo(r);
	updateRow(r);
      }

      break;
    }

    case COLUMN_TYPE_CREDIT_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	FXString txt=ed->getText();
	if (!txt.empty()) {
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;
	  int rv;

	  rv=AE_Utils_AmountFromString(txt.text(), m_commodityList, &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad strng [%s] (no amount)", txt.text());
	  }
	  else {
            AQDB_Value_Negate(v);
	    AE_Split_SetAmount(st, v);
            if (co)
	      AE_Split_SetCommodityId(st, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
	  }
	}
	AE_Split_AddRuntimeFlags(st, AE_SPLIT_RTFLAGS_MODIFIED);

	modifiedRowInfo(r);
	updateRow(r);
      }

      break;
    }

    case COLUMN_TYPE_DEBIT_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	FXString txt=ed->getText();
	if (!txt.empty()) {
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;
	  int rv;

	  rv=AE_Utils_AmountFromString(txt.text(), m_commodityList, &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad strng [%s] (no amount)", txt.text());
	  }
	  else {
	    AE_Split_SetAmount(st, v);
            if (co)
	      AE_Split_SetCommodityId(st, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
	  }
	}
	AE_Split_AddRuntimeFlags(st, AE_SPLIT_RTFLAGS_MODIFIED);

	modifiedRowInfo(r);
	updateRow(r);
      }

      break;
    }

    case COLUMN_TYPE_ACCOUNT: {
      FF_AccountEdit *ed=dynamic_cast<FF_AccountEdit*>(editor);
      if (ed) {
	FXString txt;
	AQDB_ID aid;

	aid=ed->getAccountId();
	if (aid==0) {
	  FXMessageBox::error(this, MBOX_OK,
			      I18N("Input Error"),
			      "%s",
			      I18N("The input does not reference a known account."));
	}
	else {
	  AE_Split_SetAccountId(st, aid);
	  AE_Split_AddRuntimeFlags(st, AE_SPLIT_RTFLAGS_MODIFIED);

	  modifiedRowInfo(r);
	  updateRow(r);
	}
      }
      break;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }
}



void FF_SplitTable::releaseEditor(int r, int c, FXWindow *ed) {
  reload();
  delete ed;
}



AE_SPLIT *FF_SplitTable::getSplitByRow(int r) {
  if (m_splitArray && r<m_splitCount)
    return m_splitArray[r];

  return NULL;
}



void FF_SplitTable::addSpaceForSplits(int n) {
  if (m_splitArray) {
    AE_SPLIT **p;
    int i;

    p=(AE_SPLIT**)malloc((n+m_splitArraySize)*sizeof(AE_SPLIT*));
    memset(p, 0, (n+m_splitArraySize)*sizeof(AE_SPLIT*));
    for (i=0; i<m_splitCount; i++) {
      p[i]=m_splitArray[i];
    }
    free(m_splitArray);
    m_splitArray=p;
    m_splitArraySize+=n;
  }
  else {
    m_splitArray=(AE_SPLIT**)malloc(n*sizeof(AE_SPLIT*));
    memset(m_splitArray, 0, n*sizeof(AE_SPLIT*));
    m_splitArraySize=n;
    m_splitCount=0;
  }
}



void FF_SplitTable::addSplit(AE_SPLIT *st) {
  if (m_splitArray==NULL)
    addSpaceForSplits(SPLIT_BUNCH);
  if ((m_splitCount+1)>=m_splitArraySize)
    addSpaceForSplits(SPLIT_BUNCH);
  m_splitArray[m_splitCount++]=st;
  if (m_splitCount>getNumberOfRows())
    FXXTable::setNumberOfRows(m_splitCount);
}



void FF_SplitTable::delRow(int r) {
  if (r>=0 && r<m_splitCount) {
    int j;

    AE_Split_free(m_splitArray[r]);
    for (j=r+1; j<m_splitCount; j++)
      m_splitArray[j-1]=m_splitArray[j];
    m_splitArray[j-1]=0;
    m_splitCount--;
    setNumberOfRows(m_splitCount);
  }
}



void FF_SplitTable::addSplits(AE_SPLIT_LIST2 *sl, int copyMode) {
  AE_SPLIT_LIST2_ITERATOR *it;

  it=AE_Split_List2_First(sl);
  if (it) {
    AE_SPLIT *st;


    switch(copyMode) {
      case COPYMODE_COPY:
        st=AE_Split_List2Iterator_Data(it);
        while(st) {
          addSplit(AE_Split_dup(st));
          st=AE_Split_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_ATTACH:
        st=AE_Split_List2Iterator_Data(it);
        while(st) {
          AE_Split_Attach(st);
          addSplit(st);
          st=AE_Split_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_NONE:
      default:
        st=AE_Split_List2Iterator_Data(it);
        while(st) {
          addSplit(st);
          st=AE_Split_List2Iterator_Next(it);
        }
    }

    AE_Split_List2Iterator_free(it);
  }
}



void FF_SplitTable::getSelectedSplits(AE_SPLIT_LIST2 *sl, int copyMode) {
  const SelectionRange *r=getSelection().getFirstRange();

  while(r) {
    int i;

    for (i=r->first; i<=r->last; i++) {
      AE_SPLIT *st;

      st=getSplitByRow(i);
      if (st) {
        switch(copyMode) {
        case COPYMODE_COPY:
          AE_Split_List2_PushBack(sl, AE_Split_dup(st));
          break;

        case COPYMODE_ATTACH:
          AE_Split_Attach(st);
          AE_Split_List2_PushBack(sl, st);
          break;

        case COPYMODE_NONE:
        default:
          AE_Split_List2_PushBack(sl, st);
          break;
        }
      }
    }

    r=r->next;
  }
}



void FF_SplitTable::clearItems() {
  FXXTable::clearItems();
  setNumberOfRows(0);
}



void FF_SplitTable::sortByColumn(int c, bool up) {
  if (c<MAX_COLUMNS) {
    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_MEMO:
      break;

    case COLUMN_TYPE_ACCOUNT:
      break;

    case COLUMN_TYPE_VALUE:
      break;

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }

  releaseAllRowInfos();
  m_firstVisibleRow=0;
  m_alignTop=true;
  m_needLayout=true;
  update();
}



AE_SPLIT *FF_SplitTable::getCurrentSplit() {
  int r;

  r=getCurrentRow();
  if (r>=0 && r<m_splitCount) {
    AE_SPLIT *st;

    st=getSplitByRow(r);
    if (st==NULL)
      return NULL;
    return st;
  }

  return NULL;
}



void FF_SplitTable::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_ACCOUNT_TREE *at;
    AE_COMMODITY_LIST *col;
    int rv;

    at=AE_Account_Tree_new();
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Account_Tree_free(at);
    }
    else {
      AE_Account_Tree_free(m_accountTree);
      m_accountTree=at;
    }

    col=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, col);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      AE_Commodity_List_free(col);
    }
    else {
      AE_Commodity_List_free(m_commodityList);
      m_commodityList=col;
    }
  }
}



long FF_SplitTable::onCmdAccount(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_SplitTable::onCancelAccount(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



long FF_SplitTable::onCmdAmount(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_SplitTable::onCancelAmount(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



long FF_SplitTable::onCmdTextField(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_SplitTable::onCancelTextField(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



