/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_statementtable.hpp"
#include "ff_app.hpp"
#include "ff_dateedit.hpp"
#include "ff_partneredit.hpp"


#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#include <assert.h>


#define STATEMENT_BUNCH 256

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


FXDEFMAP(FF_StatementTable) FF_StatementTableMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_StatementTable::ID_PARTNER_EDIT, FF_StatementTable::onCmdPartner),
  FXMAPFUNC(SEL_DESELECTED, FF_StatementTable::ID_PARTNER_EDIT, FF_StatementTable::onCancelPartner),

  FXMAPFUNC(SEL_COMMAND, FF_StatementTable::ID_TEXTFIELD, FF_StatementTable::onCmdTextField),
  FXMAPFUNC(SEL_DESELECTED, FF_StatementTable::ID_TEXTFIELD, FF_StatementTable::onCancelTextField),
  FXMAPFUNC(SEL_KEYPRESS, FF_StatementTable::ID_TEXTFIELD, FXXTable::onEditorKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE, FF_StatementTable::ID_TEXTFIELD, FXXTable::onEditorKeyRelease),
};


FXIMPLEMENT(FF_StatementTable, FXXTable, FF_StatementTableMap, ARRAYNUMBER(FF_StatementTableMap))



FF_StatementTable::FF_StatementTable(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
                                     FXint x, FXint y, FXint w, FXint h,
                                     FXint pl, FXint pr,
                                     FXint pt, FXint pb)
:FXXTable(p, tgt, sel, opts, x, y, w, h, pl, pr, pt, pb)
,m_stmArray(NULL)
,m_stmArraySize(0)
,m_stmCount(0)
,m_partners(NULL)
,m_markBeyondId(0)
,m_colorizeMatches(false)
{
  int i;

  setMarkColor(0, FF_APP_YELLOW1, 0, 0);
  setMarkColor(0, FF_APP_YELLOW1, 0, 1);
  setMarkColor(0, FF_APP_YELLOW2,  1, 0);
  setMarkColor(0, FF_APP_YELLOW2,  1, 1);

  /* red'ish */
  setMarkColor(1, FXColor(FXRGB(255, 209,209)));
  /* yellow */
  setMarkColor(2, FXColor(FXRGB(236, 229, 14)));
  /* green */
  setMarkColor(3, FXColor(FXRGB(177, 242, 65)));

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=0;
}



FF_StatementTable::FF_StatementTable()
:FXXTable()
,m_stmArray(NULL)
,m_stmArraySize(0)
,m_stmCount(0)
,m_partners(NULL)
,m_colorizeMatches(false)
{
  int i;

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=COLUMN_TYPE_NONE;
}



FF_StatementTable::~FF_StatementTable() {
  if (m_stmArray) {
    for (int i=0; i<m_stmCount; i++)
      AE_Statement_free(m_stmArray[i]);
    free(m_stmArray);
    m_stmArray=NULL;
  }
  AE_Partner_List_free(m_partners);
}



void FF_StatementTable::setNumberOfRows(int i) {
  /* reserve space needed for i rows */
  if (m_stmArray==NULL)
    addSpaceForStatements(i);
  if (i>m_stmArraySize)
    addSpaceForStatements(i-m_stmArraySize);
  FXXTable::setNumberOfRows(i);

  if (i<m_stmCount) {
    int j;

    /* shrinking, free all statements behind i */
    for (j=i; j<m_stmCount; j++) {
      AE_Statement_free(m_stmArray[j]);
      m_stmArray[j]=NULL;
    }
  }
  m_stmCount=i;
}



void FF_StatementTable::setColorizeMatches(bool b) {
  if (b!=m_colorizeMatches) {
    m_colorizeMatches=b;
    rebuild();
  }
}



void FF_StatementTable::addColumn(int coltype, int size) {
  if ((m_ncolumns+1)<MAX_COLUMNS) {
    switch(coltype) {
    case COLUMN_TYPE_DATE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Date"), size);
      break;
    case COLUMN_TYPE_MEMO:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Memo"), size);
      break;
    case COLUMN_TYPE_TYPE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Type"), size);
      break;
    case COLUMN_TYPE_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Amount"), size);
      break;
    case COLUMN_TYPE_PARTNER:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Partner"), size);
      break;
    default:
      break;
    }
  }
}


// TODO: change to the way it is in TransactionTable
FXXTable::ColumnInfo *FF_StatementTable::getColumnInfo(int r, int c) {
  if (r<m_stmCount && c<MAX_COLUMNS) {
    AE_STATEMENT *st;
    FXString txt;
    FXColor bg=getCellBackColor(r, c);

    st=getStatementByRow(r);
    if (st==NULL)
      return NULL;

    if (m_partners==NULL)
      reload();

    if (m_colorizeMatches) {
      switch(AE_Statement_GetRuntimeFlags(st) & AE_STATEMENT_RTFLAGS_MATCHMASK) {
      case AE_STATEMENT_RTFLAGS_MATCHAUTO1:  bg=FXColor(FXRGB(236, 229, 14)); break;
      case AE_STATEMENT_RTFLAGS_MATCHAUTO2:
      case AE_STATEMENT_RTFLAGS_MATCHMANUAL: bg=FXColor(FXRGB(177, 242, 65)); break;
      default:
      case AE_STATEMENT_RTFLAGS_MATCHNONE:   bg=FXColor(FXRGB(255, 209,209)); break;
      }
    }

    if (m_markBeyondId!=0 && m_markBeyondId<AE_Statement_GetId(st)) {
      if (r & 1)
        bg=FXColor(FF_APP_YELLOW2);
      else
        bg=FXColor(FF_APP_YELLOW1);
    }


#if 0
    if (c==0) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AE_Statement_toHashString(st, tbuf);
      fprintf(stderr, "HASH: (%3d: [%s])\n", r, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
#endif

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE: {
      const GWEN_DATE *ti;

      ti=AE_Statement_GetDate(st);
      if (ti) {
        GWEN_BUFFER *tbuf;

	tbuf=GWEN_Buffer_new(0, 32, 0, 1);
        GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
        txt=FXString(GWEN_Buffer_GetStart(tbuf));
        GWEN_Buffer_free(tbuf);
      }
      break;
    }

    case COLUMN_TYPE_MEMO: {
      const char *s;

      s=AE_Statement_GetMemo(st);
      if (s && *s) {
	txt+="<b>";
	txt+=s;
	txt+="</b><br>";
      }

      s=AE_Statement_GetRemoteName(st);
      if (s && *s) {
        txt+="<i>";
	txt+=s;
	txt+="</i><br>";
      }
      s=AE_Statement_GetPurpose(st);
      if (s && *s) {
	txt+=s;
	txt+="<br>";
      }
      break;
    }

    case COLUMN_TYPE_TYPE: {
      const char *s;

      s=AE_Statement_GetInfoText(st);
      if (s)
        txt+=s;
      else {
        int k=AE_Statement_GetTextKey(st);
        if (k!=0) {
          char numbuf[32];

          snprintf(numbuf, sizeof(numbuf)-1, "%d", k);
          numbuf[sizeof(numbuf)-1]=0;
          txt+=numbuf;
        }
      }
      break;
    }

    case COLUMN_TYPE_VALUE: {
      const AQDB_VALUE *v;

      v=AE_Statement_GetValue(st);
      if (v) {
        const char *s;
        GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

        AQDB_Value_toHumanReadableString(v, tbuf, 2);
        txt+="<right>";
        if (AQDB_Value_IsNegative(v)) {
          txt+="<font color=red>";
          txt+=GWEN_Buffer_GetStart(tbuf);
          s=AE_Statement_GetCurrency(st);
          if (s) {
            txt+=" ";
            txt+=s;
          }
          txt+="</font>";
        }
        else {
          txt+=GWEN_Buffer_GetStart(tbuf);
          s=AE_Statement_GetCurrency(st);
          if (s) {
            txt+=" ";
            txt+=s;
          }
        }
        txt+="</right>";
        GWEN_Buffer_free(tbuf);
      }
      break;
    }

    case COLUMN_TYPE_PARTNER: {
      uint32_t id;

      id=AE_Statement_GetPartnerId(st);
      if (id && m_partners) {
	AE_PARTNER *pt;

	pt=AE_Partner_List_GetById(m_partners, id);
	if (pt) {
	  GWEN_BUFFER *tbuf;

	  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	  AE_Partner_PathToBuffer(pt, tbuf);
	  txt+=FXString(GWEN_Buffer_GetStart(tbuf));
          GWEN_Buffer_free(tbuf);
	}
      }
      break;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }

    return new ColumnInfoText(this, r, c, txt, bg);
  }
  else
    return NULL;
}



void FF_StatementTable::releaseColumnInfo(FXXTable::ColumnInfo *ci) {
  delete ci;
}




FXWindow *FF_StatementTable::getEditor(int r, int c) {
  if (r<m_stmCount && c<MAX_COLUMNS) {
    AE_STATEMENT *st;
    FXString txt;

    st=getStatementByRow(r);
    if (st==NULL)
      return NULL;

    if (m_partners==NULL)
      reload();

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE: {
      FF_DateEdit *ed=new FF_DateEdit(this, this, ID_DATE_EDIT, I18N("YYYY/MM/DD"));
      const GWEN_DATE *ti;

      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ti=AE_Statement_GetDate(st);
      if (ti)
	ed->setDate(ti);
      return ed;
    }

    case COLUMN_TYPE_MEMO: {
      const char *s;
      FXTextField *ed=new FXTextField(this, 1, this, ID_TEXTFIELD,
                                      TEXTFIELD_ENTER_ONLY,
                                      0, 0, 0, 0,
                                      0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      s=AE_Statement_GetMemo(st);
      if (s && *s) {
	ed->setText(s);
	ed->selectAll();
      }
      return ed;
    }

    case COLUMN_TYPE_TYPE:
      return NULL;

    case COLUMN_TYPE_VALUE:
      return NULL;

    case COLUMN_TYPE_PARTNER: {
      FF_PartnerEdit *ed;

      ed=new FF_PartnerEdit(this, this, ID_PARTNER_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();
      ed->setPartnerId(AE_Statement_GetPartnerId(st));
      ed->selectAll();
      return ed;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }
  return NULL;
}



void FF_StatementTable::setFromEditor(FXXTable::ColumnInfo *ci, FXWindow *editor) {
  int r;
  int c;

  r=ci->getRow();
  c=ci->getColumn();

  if (r<m_stmCount && c<MAX_COLUMNS) {
    AE_STATEMENT *st;

    st=getStatementByRow(r);
    if (st==NULL)
      return;

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE: {
      FF_DateEdit *ed=dynamic_cast<FF_DateEdit*>(editor);
      if (ed) {
      }
      break;
    }

    case COLUMN_TYPE_MEMO: {
      FXTextField *ed=dynamic_cast<FXTextField*>(editor);
      if (ed) {
	FXString txt=ed->getText();
	if (txt.empty())
	  AE_Statement_SetMemo(st, NULL);
	else
          AE_Statement_SetMemo(st, txt.text());
	AE_Statement_AddRuntimeFlags(st, AE_STATEMENT_RTFLAGS_MODIFIED);

        modifiedRowInfo(r);
        updateRow(r);
      }
      break;
    }

    case COLUMN_TYPE_TYPE:
      break;

    case COLUMN_TYPE_VALUE:
      break;

    case COLUMN_TYPE_PARTNER: {
      FF_PartnerEdit *ed=dynamic_cast<FF_PartnerEdit*>(editor);
      if (ed) {
	FXString txt;
	AQDB_ID pid;
        bool doModify=false;

	txt=ed->getText().simplify();
	if (txt.empty()) {
	  pid=0;
	  doModify=true;
	}
	else {
	  pid=ed->getPartnerId();
	  if (pid==0) {
	    FXMessageBox::error(this, MBOX_OK,
				I18N("Input Error"),
				"%s",
				I18N("The input does not reference a known partner."));
	  }
	  else
	    doModify=true;
	}
	if (doModify) {
	  AE_Statement_SetPartnerId(st, pid);
	  AE_Statement_AddRuntimeFlags(st, AE_STATEMENT_RTFLAGS_MODIFIED);
	}

	modifiedRowInfo(r);
	updateRow(r);
      }
      break;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }
}



void FF_StatementTable::releaseEditor(int r, int c, FXWindow *ed) {
  reload();
  delete ed;
}



AE_STATEMENT *FF_StatementTable::getStatementByRow(int r) {
  if (m_stmArray && r<m_stmCount)
    return m_stmArray[r];

  return NULL;
}



void FF_StatementTable::addSpaceForStatements(int n) {
  if (m_stmArray) {
    AE_STATEMENT **p;
    int i;

    p=(AE_STATEMENT**)malloc((n+m_stmArraySize)*sizeof(AE_STATEMENT*));
    memset(p, 0, (n+m_stmArraySize)*sizeof(AE_STATEMENT*));
    for (i=0; i<m_stmCount; i++) {
      p[i]=m_stmArray[i];
    }
    free(m_stmArray);
    m_stmArray=p;
    m_stmArraySize+=n;
  }
  else {
    m_stmArray=(AE_STATEMENT**)malloc(n*sizeof(AE_STATEMENT*));
    memset(m_stmArray, 0, n*sizeof(AE_STATEMENT*));
    m_stmArraySize=n;
    m_stmCount=0;
  }
}



void FF_StatementTable::addStatement(AE_STATEMENT *st) {
  if (m_stmArray==NULL)
    addSpaceForStatements(STATEMENT_BUNCH);
  if ((m_stmCount+1)>=m_stmArraySize)
    addSpaceForStatements(STATEMENT_BUNCH);
  m_stmArray[m_stmCount++]=st;
  if (m_stmCount>getNumberOfRows())
    FXXTable::setNumberOfRows(m_stmCount);
}



void FF_StatementTable::delRow(int r) {
  if (r>=0 && r<m_stmCount) {
    int j;

    AE_Statement_free(m_stmArray[r]);
    for (j=r+1; j<m_stmCount; j++)
      m_stmArray[j-1]=m_stmArray[j];
    m_stmArray[j-1]=0;
    m_stmCount--;
    setNumberOfRows(m_stmCount);
  }
}



void FF_StatementTable::addStatements(AE_STATEMENT_LIST2 *sl, int copyMode) {
  AE_STATEMENT_LIST2_ITERATOR *it;

  it=AE_Statement_List2_First(sl);
  if (it) {
    AE_STATEMENT *st;


    switch(copyMode) {
      case COPYMODE_COPY:
        st=AE_Statement_List2Iterator_Data(it);
        while(st) {
          addStatement(AE_Statement_dup(st));
          st=AE_Statement_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_ATTACH:
        st=AE_Statement_List2Iterator_Data(it);
        while(st) {
          AE_Statement_Attach(st);
          addStatement(st);
          st=AE_Statement_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_NONE:
      default:
        st=AE_Statement_List2Iterator_Data(it);
        while(st) {
          addStatement(st);
          st=AE_Statement_List2Iterator_Next(it);
        }
    }

    AE_Statement_List2Iterator_free(it);
  }
}



void FF_StatementTable::getSelectedStatements(AE_STATEMENT_LIST2 *sl, int copyMode) {
  const SelectionRange *r=getSelection().getFirstRange();

  while(r) {
    int i;

    for (i=r->first; i<=r->last; i++) {
      AE_STATEMENT *st;

      st=getStatementByRow(i);
      if (st) {
        switch(copyMode) {
        case COPYMODE_COPY:
          AE_Statement_List2_PushBack(sl, AE_Statement_dup(st));
          break;

        case COPYMODE_ATTACH:
          AE_Statement_Attach(st);
          AE_Statement_List2_PushBack(sl, st);
          break;

        case COPYMODE_NONE:
        default:
          AE_Statement_List2_PushBack(sl, st);
          break;
        }
      }
    }

    r=r->next;
  }
}



void FF_StatementTable::clearItems() {
  FXXTable::clearItems();
  setNumberOfRows(0);
}



int FF_StatementTable::compareDateUp(const void *a, const void *b) {
  const AE_STATEMENT *st1=*((const AE_STATEMENT**) a);
  const AE_STATEMENT *st2=*((const AE_STATEMENT**) b);

  const GWEN_DATE *t1=AE_Statement_GetDate(st1);
  const GWEN_DATE *t2=AE_Statement_GetDate(st2);

  /* both are data items */
  if (t1 && t2) {
    int d=GWEN_Date_Diff(t1, t2);
    if (d<0)
      return -1;
    else if (d>0)
      return 1;
  }
  else if (t1)
    return 1;
  else if (t2)
    return -1;

  return 0;
}



int FF_StatementTable::compareDateDown(const void *a, const void *b) {
  const AE_STATEMENT *st1=*((const AE_STATEMENT**) a);
  const AE_STATEMENT *st2=*((const AE_STATEMENT**) b);

  /* reversed */
  const GWEN_DATE *t2=AE_Statement_GetDate(st1);
  const GWEN_DATE *t1=AE_Statement_GetDate(st2);

  /* both are data items */
  if (t1 && t2) {
    int d=GWEN_Date_Diff(t1, t2);
    if (d<0)
      return -1;
    else if (d>0)
      return 1;
  }
  else if (t1)
    return -1;
  else if (t2)
    return 1;

  return 0;
}



void FF_StatementTable::sortByColumn(int c, bool up) {
  if (c<MAX_COLUMNS) {
    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE:
      if (up)
        qsort(m_stmArray, m_stmCount, sizeof(AE_STATEMENT*), compareDateUp);
      else
        qsort(m_stmArray, m_stmCount, sizeof(AE_STATEMENT*), compareDateDown);
      break;

    case COLUMN_TYPE_MEMO:
      break;

    case COLUMN_TYPE_TYPE:
      break;

    case COLUMN_TYPE_VALUE:
      break;

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }

  releaseAllRowInfos();
  m_firstVisibleRow=0;
  m_alignTop=true;
  m_needLayout=true;
  update();
}



AE_STATEMENT *FF_StatementTable::getCurrentStatement() {
  int r;

  r=getCurrentRow();
  if (r>=0 && r<m_stmCount) {
    AE_STATEMENT *st;

    st=getStatementByRow(r);
    if (st==NULL)
      return NULL;
    return st;
  }

  return NULL;
}



void FF_StatementTable::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();
  if (b) {
    AE_PARTNER_LIST *lst;
    int rv;

    lst=AE_Partner_List_new();
    rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, lst);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_List_free(lst);
    }
    else {
      AE_Partner_List_free(m_partners);
      m_partners=lst;
    }
  }
}



long FF_StatementTable::onCmdPartner(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_StatementTable::onCancelPartner(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



long FF_StatementTable::onCmdTextField(FXObject*, FXSelector, void* ptr) {
  acceptInput(true);
  return 1;
}



long FF_StatementTable::onCancelTextField(FXObject*, FXSelector, void* ptr) {
  cancelInput(true);
  return 1;
}



