/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_taxedit.hpp"
#include "ff_app.hpp"
#include "ff_selecttax.hpp"

#include <aqfinance/engine/book/ae_tax.h>

#include <gwen-gui-fox16/theme.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



FXDEFMAP(FF_TaxEdit) FF_TaxEditMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_TaxEdit::ID_COMBOEDIT, FF_TaxEdit::onCmdCombo),
  FXMAPFUNC(SEL_COMMAND, FF_TaxEdit::ID_BUTTON, FF_TaxEdit::onCmdButton),
  FXMAPFUNC(SEL_DESELECTED, FF_TaxEdit::ID_COMBOEDIT, FF_TaxEdit::onCmdCancel),
  FXMAPFUNC(SEL_FOCUSIN, 0, FF_TaxEdit::onFocusIn),
};


// Object implementation
FXIMPLEMENT(FF_TaxEdit, FXHorizontalFrame, FF_TaxEditMap, ARRAYNUMBER(FF_TaxEditMap))



FF_TaxEdit::FF_TaxEdit(FXComposite *p,
                               FXObject* tgt,
                               FXSelector sel,
                               FXuint opts,
                               FXint x, FXint y, FXint w, FXint h)
:FXHorizontalFrame(p, opts, x, y, w, h, 0, 0, 0, 0)
,m_taxList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
  FF_App *app;
  target=tgt;
  message=sel;

  app=dynamic_cast<FF_App*>(getApp());

  m_comboEdit=new FXXComboEdit(this, 1,
                               this, ID_COMBOEDIT,
                               LAYOUT_FILL_X);
  m_comboEdit->setSortFunc(FXList::ascendingCase);
  m_iconButton=app->getIcon("minifolderopen");
  if (m_iconButton) {
    m_button=new THEMEBUTTON(this, "", m_iconButton, this, ID_BUTTON);
  }
  else
    m_button=new THEMEBUTTON(this, "...", NULL, this, ID_BUTTON);
}



FF_TaxEdit::FF_TaxEdit()
:FXHorizontalFrame()
,m_taxList(NULL)
,m_comboEdit(NULL)
,m_button(NULL)
,m_iconButton(NULL) {
}



FF_TaxEdit::~FF_TaxEdit() {
  AE_Tax_List_free(m_taxList);
}



void FF_TaxEdit::create() {
  FXHorizontalFrame::create();
  if (m_iconButton)
    m_iconButton->create();
  m_comboEdit->create();
  m_button->create();
}



void FF_TaxEdit::setFont(FXFont* fnt) {
  m_comboEdit->setFont(fnt);
  m_button->setFont(fnt);
}



void FF_TaxEdit::setBackColor(FXColor clr) {
  FXHorizontalFrame::setBackColor(clr);
  m_comboEdit->setBackColor(clr);
  m_button->setBackColor(clr);
}



void FF_TaxEdit::setTextColor(FXColor clr) {
  m_comboEdit->setTextColor(clr);
  m_button->setTextColor(clr);
}



void FF_TaxEdit::setSelBackColor(FXColor clr) {
  m_comboEdit->setSelBackColor(clr);
}



void FF_TaxEdit::setSelTextColor(FXColor clr) {
  m_comboEdit->setSelTextColor(clr);
}



void FF_TaxEdit::setText(const FXString& text) {
  m_comboEdit->setText(text);
}



FXString FF_TaxEdit::getText() const {
  return m_comboEdit->getText();
}



void FF_TaxEdit::selectAll() {
  m_comboEdit->selectAll();
}



void FF_TaxEdit::setTaxId(AQDB_ID id) {
  AE_TAX *tax;

  tax=AE_Tax_List_GetById(m_taxList, id);
  if (tax) {
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AE_Tax_PathToBuffer(tax, tbuf);
    m_comboEdit->setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }
}



AQDB_ID FF_TaxEdit::getTaxId() {
  FXString str;

  str=m_comboEdit->getText();
  if (!str.empty()) {
    AE_TAX *tax;

    tax=AE_Tax_List_GetByPath(m_taxList, str.text());
    if (tax)
      return AE_Tax_GetId(tax);
  }

  return 0;
}



long FF_TaxEdit::onCmdCombo(FXObject*, FXSelector, void*) {
  if (target && target->handle(this, FXSEL(SEL_COMMAND, message), NULL))
    return 1;
  return 1;
}



long FF_TaxEdit::onCmdButton(FXObject*, FXSelector, void*) {
  AQDB_ID id;
  AE_TAX *tax;

  /* TODO: get currently selected tax */
  id=0;

  tax=FF_SelectTax::selectTax(dynamic_cast<FF_App*>(getApp()),
			      this,
			      I18N("Select a Tax"));
  reload();
  if (tax) {
    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

    AE_Tax_PathToBuffer(tax, tbuf);
    setText(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
    if (target)
      target->handle(this, FXSEL(SEL_COMMAND, message), NULL);
  }

  return 1;
}



long FF_TaxEdit::onFocusIn(FXObject *sender, FXSelector sel, void *ptr){
  FXHorizontalFrame::onFocusIn(sender, sel, ptr);
  m_comboEdit->setFocus();
  return 1;
}



long FF_TaxEdit::onCmdCancel(FXObject *sender, FXSelector sel, void *ptr){
  if (target && target->handle(this, FXSEL(SEL_DESELECTED, message), NULL))
    return 1;

  return 0;
}



void FF_TaxEdit::reload() {
  FF_App *app;
  int rv;

  m_comboEdit->clearItems();
  app=dynamic_cast<FF_App*>(getApp());
  AE_Tax_List_free(m_taxList);
  m_taxList=AE_Tax_List_new();
  rv=AE_Book_ReadTaxList(app->getBook(), AE_Book_TableType_Tax, m_taxList);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
  }
  else {
    AE_TAX *tax;
    GWEN_BUFFER *tbuf;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    tax=AE_Tax_List_First(m_taxList);
    while(tax) {
      AE_Tax_PathToBuffer(tax, tbuf);
      m_comboEdit->addEntry(GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
      tax=AE_Tax_List_Next(tax);
    }
    GWEN_Buffer_free(tbuf);
    m_comboEdit->sortItems();
    m_comboEdit->reloadList();
  }
}






