/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_taxtree.hpp"

#include <fx.h>

#include <aqfinance/engine/book/ae_tax.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)




FF_TaxTreeItem::FF_TaxTreeItem(AE_TAX *pt,
			       FXIcon* oi, FXIcon* ci, void* ptr)
:FF_GroupTreeItem(oi, ci, ptr)
,m_tax(pt)
{
  rebuild();
}



FF_TaxTreeItem::~FF_TaxTreeItem() {
  if (m_isDataItem && m_tax) {
    AE_Tax_free(m_tax);
  }
}



void FF_TaxTreeItem::rebuild() {
  FXString t;
  const char *s;

  s=AE_Tax_GetCode(m_tax);
  if (s)
    t+=s;
  t+="\t";

  s=AE_Tax_GetName(m_tax);
  if (s)
    t+=s;
  t+="\t";

  setText(t);
}








FF_TaxTree::FF_TaxTree(FXComposite *p,
		       FXObject* tgt, FXSelector sel,
		       FXuint opts,
		       FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h) {
  int idx=0;

  appendHeader(I18N("Code"));
  setHeaderSize(idx++, 100);
  appendHeader(I18N("Name"));
  setHeaderSize(idx++, 100);
}



FF_TaxTree::~FF_TaxTree() {
}



FF_TaxTreeItem *FF_TaxTree::findTaxItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_TaxTreeItem *res=findTaxItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_TaxTreeItem *FF_TaxTree::findTaxItem(FF_GroupTreeItem *grItem, AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_TaxTreeItem *baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
    if (baItem) {
      AE_TAX *tax=baItem->getTax();
      if (AE_Tax_GetId(tax)==id)
        return baItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_TaxTreeItem *res=findTaxItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_TaxTreeItem *FF_TaxTree::findTaxItem(const AE_TAX *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TaxTreeItem *baItem;

      baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
      if (baItem) {
        AE_TAX *tax;

	tax=baItem->getTax();
	if (tax==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_TaxTree::addTax(AE_TAX *tax) {
  FF_GroupTreeItem *itemFather=NULL;
  AQDB_ID gid;
  FF_TaxTreeItem *item;

  gid=AE_Tax_GetGroupId(tax);
  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Group for tax not found, reassigning to root");
    }
  }
  else {
    DBG_ERROR(0, "Tax has no gid");
  }

  item=new FF_TaxTreeItem(tax,
			  /*m_iconOpen, m_iconClosed*/
			  NULL, NULL); /* no icons here */
  appendItem(itemFather, item, false);
}



void FF_TaxTree::addTax(const AE_TAX *tax) {
  addTax(AE_Tax_dup(tax));
}



FF_TaxTree::FF_TaxTree(): FF_GroupTree() {
}



void FF_TaxTree::create() {
  FF_GroupTree::create();
}



AE_TAX *FF_TaxTree::getCurrentTax() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TaxTreeItem *baItem;

      baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
      if (baItem)
        return baItem->getTax();
    }
  }

  return NULL;
}



AE_TAX *FF_TaxTree::getFirstSelectedTax() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_TaxTreeItem *baItem;

        baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
        if (baItem)
          return baItem->getTax();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_TaxTree::getSelectedTaxes(AE_TAX_LIST2 *al) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_TaxTreeItem *baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
        if (baItem)
          AE_Tax_List2_PushBack(al, baItem->getTax());
      }
    }
    item=item->getBelow();
  }
}



void FF_TaxTree::selectTax(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TaxTreeItem *baItem;

      baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
      if (baItem) {
        AE_TAX *tax;

        tax=baItem->getTax();
        if (AE_Tax_GetId(tax)==id) {
          baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_TaxTree::selectTax(const AE_TAX *pt, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TaxTreeItem *baItem;

      baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
      if (baItem) {
	const AE_TAX *tax;

        tax=baItem->getTax();
        if (tax==pt) {
	  baItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_TaxTree::getTaxes(AE_TAX_LIST2 *al, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_NONE:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_TaxTreeItem *baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
	if (baItem)
	  AE_Tax_List2_PushBack(al, baItem->getTax());
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_TaxTreeItem *baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
	if (baItem) {
	  AE_TAX *tax;

	  tax=baItem->getTax();
	  assert(tax);
	  AE_Tax_List2_PushBack(al, AE_Tax_dup(tax));
	}
      }
      item=item->getBelow();
    }
    break;

  default:
  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
	FF_TaxTreeItem *baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
	if (baItem) {
	  AE_TAX *tax;

	  tax=baItem->getTax();
	  assert(tax);
	  AE_Tax_Attach(tax);
	  AE_Tax_List2_PushBack(al, tax);
	}
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_TaxTree::makeTaxVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TaxTreeItem *baItem;

      baItem=dynamic_cast<FF_TaxTreeItem*>(grItem);
      if (baItem) {
        AE_TAX *tax;

        tax=baItem->getTax();
        if (AE_Tax_GetId(tax)==id) {
          makeItemVisible(baItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




#define DEF_SORT_FN_UP_TXT(macro_num, macro_name)                                            \
FXint FF_TaxTree::sortFunctionUp##macro_num(const FXFoldingItem *it1,                        \
                                                    const FXFoldingItem *it2) {              \
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);                \
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);                \
                                                                                             \
  if (grItem1 && grItem2) {                                                                  \
    if (!grItem1->isDataItem() && !grItem2->isDataItem())                                    \
      return FF_GroupTree::sortFunctionUp(it1, it2);                                         \
    else if (!grItem1->isDataItem())                                                         \
      return 1;                                                                              \
    else if (!grItem2->isDataItem())                                                         \
      return -1;                                                                             \
    else {                                                                                   \
      const FF_TaxTreeItem *tti1=dynamic_cast<const FF_TaxTreeItem*>(it1);                   \
      const FF_TaxTreeItem *tti2=dynamic_cast<const FF_TaxTreeItem*>(it2);                   \
      const AE_TAX *ba1=tti1->getTax();                                                      \
      const AE_TAX *ba2=tti2->getTax();                                                      \
      const char *s1=AE_Tax_Get##macro_name(ba1);                                            \
      const char *s2=AE_Tax_Get##macro_name(ba2);                                            \
                                                                                             \
      /* both are data items */                                                              \
      if (s1 && s2)                                                                          \
        return strcasecmp(s1, s2);                                                           \
      else if (s1)                                                                           \
        return 1;                                                                            \
      else if (s2)                                                                           \
        return -1;                                                                           \
    }                                                                                        \
  }                                                                                          \
                                                                                             \
  return 0;                                                                                  \
}


#define DEF_SORT_FN_DOWN(macro_num)                                                          \
FXint FF_TaxTree::sortFunctionDown##macro_num(const FXFoldingItem *it1,                      \
                                              const FXFoldingItem *it2){                     \
  FXint rv;                                                                                  \
                                                                                             \
  rv=sortFunctionUp##macro_num(it1, it2);                                                    \
  if (rv<0)                                                                                  \
    return 1;                                                                                \
  else if (rv>0)                                                                             \
    return -1;                                                                               \
  return 0;                                                                                  \
}



DEF_SORT_FN_UP_TXT(0, Code)
DEF_SORT_FN_DOWN(0)
DEF_SORT_FN_UP_TXT(1, Name)
DEF_SORT_FN_DOWN(1)




#undef DEF_SORT_FN_UP_TXT
#undef DEF_SORT_FN_DOWN



void FF_TaxTree::sortByColumn(int c, bool up) {
  switch(c) {
  case 0:
    setSortFunc(up?sortFunctionUp0:sortFunctionDown0);
    sortItems();
    break;
  case 1:
    setSortFunc(up?sortFunctionUp1:sortFunctionDown1);
    sortItems();
    break;
  default:
    DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
    break;
  }
}





