/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ff_transactiontable.hpp"
#include "ff_accountedit.hpp"
#include "ff_partneredit.hpp"
#include "ff_dateedit.hpp"
#include "ff_valueedit.hpp"
#include "ff_app.hpp"

#include <aqfinance/engine/book/ae_utils.h>


#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>

#include <fxkeys.h>

#include <assert.h>


#define TRANSACTION_BUNCH 256

#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



#define MAX_COMMODITIES_PER_TRANSACTION 16


FXDEFMAP(FF_TransactionTable) FF_TransactionTableMap[]={
  FXMAPFUNC(SEL_COMMAND, FF_TransactionTable::ID_ACCOUNT_EDIT, FF_TransactionTable::onCmdAccount),
  FXMAPFUNC(SEL_DESELECTED, FF_TransactionTable::ID_ACCOUNT_EDIT, FF_TransactionTable::onCancelAccount),
  FXMAPFUNC(SEL_COMMAND, FF_TransactionTable::ID_PARTNER_EDIT, FF_TransactionTable::onCmdPartner),
  FXMAPFUNC(SEL_DESELECTED, FF_TransactionTable::ID_PARTNER_EDIT, FF_TransactionTable::onCancelPartner),

  FXMAPFUNC(SEL_COMMAND, FF_TransactionTable::ID_VALUE_EDIT, FF_TransactionTable::onCmdValue),
  FXMAPFUNC(SEL_DESELECTED, FF_TransactionTable::ID_VALUE_EDIT, FF_TransactionTable::onCancelValue),
  FXMAPFUNC(SEL_KEYPRESS, FF_TransactionTable::ID_VALUE_EDIT, FXXTable::onEditorKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE, FF_TransactionTable::ID_VALUE_EDIT, FXXTable::onEditorKeyRelease),

  FXMAPFUNC(SEL_COMMAND, FF_TransactionTable::ID_DATE_EDIT, FF_TransactionTable::onCmdDate),
  FXMAPFUNC(SEL_DESELECTED, FF_TransactionTable::ID_DATE_EDIT, FF_TransactionTable::onCancelDate),
  FXMAPFUNC(SEL_KEYPRESS, FF_TransactionTable::ID_DATE_EDIT, FXXTable::onEditorKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE, FF_TransactionTable::ID_DATE_EDIT, FXXTable::onEditorKeyRelease),

  FXMAPFUNC(SEL_COMMAND, FF_TransactionTable::ID_TEXT_EDIT, FF_TransactionTable::onCmdTextField),
  FXMAPFUNC(SEL_DESELECTED, FF_TransactionTable::ID_TEXT_EDIT, FF_TransactionTable::onCancelTextField),
  FXMAPFUNC(SEL_KEYPRESS, FF_TransactionTable::ID_TEXT_EDIT, FXXTable::onEditorKeyPress),
  FXMAPFUNC(SEL_KEYRELEASE, FF_TransactionTable::ID_TEXT_EDIT, FXXTable::onEditorKeyRelease),
};


FXIMPLEMENT(FF_TransactionTable, FXXTable, FF_TransactionTableMap, ARRAYNUMBER(FF_TransactionTableMap))





FF_TransactionTable::FF_TransactionTable(FXComposite *p, FXObject* tgt, FXSelector sel, FXuint opts,
					 FXint x, FXint y, FXint w, FXint h,
					 FXint pl, FXint pr,
					 FXint pt, FXint pb)
:FXXTable(p, tgt, sel, opts, x, y, w, h, pl, pr, pt, pb)
,m_stmArray(NULL)
,m_stmArraySize(0)
,m_stmCount(0)
,m_markBeyondId(0)
,m_commodities(NULL)
,m_accountTree(NULL)
,m_partners(NULL)
,m_iconMultiSplitOn(NULL)
,m_iconMultiSplitOff(NULL)
,m_iconMultiCurrencyOn(NULL)
,m_iconMultiCurrencyOff(NULL)
,m_iconNewOn(NULL)
,m_iconNewOff(NULL)
,m_iconInWorkOn(NULL)
,m_iconInWorkOff(NULL)
,m_iconAccept(NULL)
,m_iconReject(NULL)
,m_colorizeMatches(false)
,m_colorizeCounterAccounts(false)
,m_accountId(0)
{
  int i;
  FF_App *app;

  app=dynamic_cast<FF_App*>(getApp());

  setMarkColor(0, app->getMarkColor1());
  /* red'ish */
  setMarkColor(1, FXColor(FXRGB(255, 209,209)));
  /* yellow */
  setMarkColor(2, FXColor(FXRGB(236, 229, 14)));
  /* green */
  setMarkColor(3, FXColor(FXRGB(177, 242, 65)));

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=0;

  m_iconMultiSplitOn=app->getIcon("multisplit");
  m_iconMultiSplitOff=app->getIcon("!multisplit");
  m_iconMultiCurrencyOn=app->getIcon("multicurrency");
  m_iconMultiCurrencyOff=app->getIcon("!multicurrency");
  m_iconNewOn=app->getIcon("new");
  m_iconNewOff=app->getIcon("!new");
  m_iconInWorkOn=app->getIcon("inwork");
  m_iconInWorkOff=app->getIcon("!inwork");

  m_iconAccept=app->getIcon("accept");
  m_iconReject=app->getIcon("reject");
}



FF_TransactionTable::FF_TransactionTable()
:FXXTable()
,m_stmArray(NULL)
,m_stmArraySize(0)
,m_stmCount(0)
,m_markBeyondId(0)
,m_commodities(NULL)
,m_accountTree(NULL)
,m_partners(NULL)
,m_iconMultiSplitOn(NULL)
,m_iconMultiSplitOff(NULL)
,m_iconMultiCurrencyOn(NULL)
,m_iconMultiCurrencyOff(NULL)
,m_iconNewOn(NULL)
,m_iconNewOff(NULL)
,m_iconInWorkOn(NULL)
,m_iconInWorkOff(NULL)
,m_iconAccept(NULL)
,m_iconReject(NULL)
,m_colorizeMatches(false)
,m_colorizeCounterAccounts(false)
,m_accountId(0)
{
  int i;

  for (i=0; i<MAX_COLUMNS; i++)
    m_columnTypes[i]=COLUMN_TYPE_NONE;
}



FF_TransactionTable::~FF_TransactionTable() {
  if (m_stmArray) {
    for (int i=0; i<m_stmCount; i++)
      AE_Transaction_free(m_stmArray[i]);
    free(m_stmArray);
    m_stmArray=NULL;
  }

  AE_Commodity_List_free(m_commodities);
  AE_Account_Tree_free(m_accountTree);
  AE_Partner_List_free(m_partners);
}



void FF_TransactionTable::create() {
  FXXTable::create();

  if (m_iconMultiSplitOn)
    m_iconMultiSplitOn->create();
  if (m_iconMultiSplitOff)
    m_iconMultiSplitOff->create();
  if (m_iconMultiCurrencyOn)
    m_iconMultiCurrencyOn->create();
  if (m_iconMultiCurrencyOff)
    m_iconMultiCurrencyOff->create();
  if (m_iconNewOn)
    m_iconNewOn->create();
  if (m_iconNewOff)
    m_iconNewOff->create();
  if (m_iconInWorkOn)
    m_iconInWorkOn->create();
  if (m_iconInWorkOff)
    m_iconInWorkOff->create();

  if (m_iconAccept)
    m_iconAccept->create();
  if (m_iconReject)
    m_iconReject->create();
}



void FF_TransactionTable::setColorizeMatches(bool b) {
  if (b!=m_colorizeMatches) {
    m_colorizeMatches=b;
    rebuild();
  }
}



void FF_TransactionTable::setColorizeCounterAccounts(bool b) {
  if (b!=m_colorizeCounterAccounts) {
    m_colorizeCounterAccounts=b;
    rebuild();
  }
}



void FF_TransactionTable::setNumberOfRows(int i) {
  /* reserve space needed for i rows */
  if (m_stmArray==NULL)
    addSpaceForTransactions(i);
  if (i>m_stmArraySize)
    addSpaceForTransactions(i-m_stmArraySize);
  FXXTable::setNumberOfRows(i);

  if (i<m_stmCount) {
    int j;

    /* shrinking, free all transactions behind i */
    for (j=i; j<m_stmCount; j++) {
      AE_Transaction_free(m_stmArray[j]);
      m_stmArray[j]=NULL;
    }
  }
  m_stmCount=i;
}



void FF_TransactionTable::addColumn(int coltype, int size) {
  if ((m_ncolumns+1)<MAX_COLUMNS) {
    switch(coltype) {
    case COLUMN_TYPE_DATE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Date"), size);
      break;
    case COLUMN_TYPE_MEMO:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Memo"), size);
      break;
    case COLUMN_TYPE_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Amount"), size);
      break;
    case COLUMN_TYPE_CREDIT_ACCOUNT:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Credit Account"), size);
      break;
    case COLUMN_TYPE_DEBIT_ACCOUNT:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Debit Account"), size);
      break;
    case COLUMN_TYPE_COUNTER_ACCOUNT:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Counter Account"), size);
      break;
    case COLUMN_TYPE_CREDIT_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_TransactionTable|Credit"), size);
      break;
    case COLUMN_TYPE_DEBIT_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_TransactionTable|Debit"), size);
      break;

    case COLUMN_TYPE_DEPOSIT_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_TransactionTable|Deposit"), size);
      break;
    case COLUMN_TYPE_WITHDRAWAL_VALUE:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("FF_TransactionTable|Withdrawal"), size);
      break;
    case COLUMN_TYPE_FLAGS:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Flags"), size);
      break;
    case COLUMN_TYPE_IMPORT:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Import"), size);
      break;
    case COLUMN_TYPE_PARTNER:
      m_columnTypes[m_ncolumns]=coltype;
      FXXTable::addColumn(I18N("Partner"), size);
      break;

    default:
      break;
    }
  }
}



#if 0
static FXString createAmountString(const AE_TRANSACTION *t, const AE_COMMODITY_LIST *cl) {
  FXString result;
  AQDB_ID ids[MAX_COMMODITIES_PER_TRANSACTION];
  int cnt;
  int i;
  GWEN_BUFFER *tbuf;

  for (i=0; i<MAX_COMMODITIES_PER_TRANSACTION; i++)
    ids[i]=0;

  /* count number of different commodities */
  cnt=AE_Transaction_SampleCommodities(t, ids, MAX_COMMODITIES_PER_TRANSACTION);

  /* for every commodity: calculate amount */
  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  for (i=0; i<cnt; i++) {
    AQDB_VALUE *v;

    v=AE_Transaction_CalculateAmount(t, ids[i]);
    if (v) {
      const char *s;
      const AE_COMMODITY *co;

      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      if (!(result.empty()))
	result+=", ";
      result+=GWEN_Buffer_GetStart(tbuf);
      co=AE_Commodity_List_GetById(cl, ids[i]);
      if (co) {
	s=AE_Commodity_GetSymbol(co);
	if (s==NULL || *s==0)
	  s=AE_Commodity_GetCode(co);
	if (s && *s) {
	  result+=" ";
	  result+=s;
	}
      }
      else {
        DBG_ERROR(0, "Commodity %08x not found", ids[i]);
      }
      GWEN_Buffer_Reset(tbuf);
      AQDB_Value_free(v);
    }
    else {
      DBG_ERROR(0, "No value");
    }
  }
  GWEN_Buffer_free(tbuf);

  return result;
}
#endif



/* SOLL (positive) */
static AQDB_ID getDebitAccountId(const AE_TRANSACTION *t) {
  AE_SPLIT *spmax=NULL;

  spmax=AE_Transaction_GetMaxDebitSplit(t);
  if (spmax)
    return AE_Split_GetAccountId(spmax);

  return 0;
}



/* HABEN (negative) */
static AQDB_ID getCreditAccountId(const AE_TRANSACTION *t) {
  AE_SPLIT *spmax=NULL;

  spmax=AE_Transaction_GetMaxCreditSplit(t);
  if (spmax)
    return AE_Split_GetAccountId(spmax);

  return 0;
}



FXXTable::ColumnInfo *FF_TransactionTable::getColumnInfo(int r, int c) {
  if (r<m_stmCount && c<MAX_COLUMNS) {
    AE_TRANSACTION *t;
    FXString txt;
    FXColor bg=getCellBackColor(r, c);
    FF_App *app;

    app=dynamic_cast<FF_App*>(getApp());

    t=getTransactionByRow(r);
    if (t==NULL) {
      DBG_ERROR(0, "Transaction %d not found", r);
      return NULL;
    }

    if (AE_Transaction_GetRuntimeFlags(t) & AE_TRANSACTION_RTFLAGS_SPARE)
      return new ColumnInfoText(this, r, c, "", 0);

    if (m_colorizeMatches) {
      uint32_t lowestMatch=AE_SPLIT_RTFLAGS_MATCHMANUAL;
      AE_SPLIT *sp;

      /* determine lowest match to colorize the transaction */
      sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
      while(sp) {
        uint32_t rf;

        rf=(AE_Split_GetRuntimeFlags(sp) & AE_SPLIT_RTFLAGS_MATCHMASK);
        if (rf<lowestMatch)
          lowestMatch=rf;
        sp=AE_Split_List_Next(sp);
      }

      switch(lowestMatch) {
        case AE_SPLIT_RTFLAGS_MATCHAUTO1:  bg=FXColor(FXRGB(236, 229, 14)); break;
        case AE_SPLIT_RTFLAGS_MATCHAUTO2:
        case AE_SPLIT_RTFLAGS_MATCHMANUAL: bg=FXColor(FXRGB(177, 242, 65)); break;
        default:
        case AE_SPLIT_RTFLAGS_MATCHNONE:   bg=FXColor(FXRGB(255, 209,209)); break;
      }
    }
    else if (m_colorizeCounterAccounts) {
      AE_SPLIT *sp;

      /* if m_accountId==0, then the counter split to the fixed one will be returned */
      sp=AE_Transaction_GetCounterSplit(t, m_accountId);
      if (sp) {
        AQDB_ID aid;

        aid=AE_Split_GetAccountId(sp);
        if (aid>0) {
          AE_ACCOUNT *acc;

          acc=AE_Account_Tree_GetById(m_accountTree, aid);
          if (acc)
            bg=app->getAccountColor(AE_Account_GetType(acc));
        }
      }
    }

    if (m_markBeyondId!=0 && m_markBeyondId<AE_Transaction_GetId(t)) {
      if (r & 1)
        bg=FXColor(FF_APP_YELLOW2);
      else
        bg=FXColor(FF_APP_YELLOW1);
    }

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE: {
      if (AE_Transaction_GetFlags(t) & AE_TRANSACTION_FLAGS_INITIAL) {
        txt="";
      }
      else {
	const GWEN_DATE *ti;

	ti=AE_Transaction_GetDate(t);
	if (ti) {
	  GWEN_BUFFER *tbuf;

	  tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	  GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
	  txt=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	}
      }
      break;
    }

    case COLUMN_TYPE_MEMO: {
      const char *s;

      s=AE_Transaction_GetMemo(t);
      if (s)
	txt+=s;
      break;
    }

    case COLUMN_TYPE_VALUE: {
      AE_SPLIT *sp;

      /* first look for flagged split (this should be fast) */
      if (m_accountId) {
	sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
	    break;
	  sp=AE_Split_List_Next(sp);
	}
      }
      else {
	sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
	while(sp) {
	  if (AE_Split_GetRuntimeFlags(sp) & AE_SPLIT_RTFLAGS_FIX)
	    break;
	  sp=AE_Split_List_Next(sp);
	}
      }

      if (sp) {
	const AQDB_VALUE *v=AE_Split_GetAmount(sp);
	if (v) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	  AQDB_ID coid;

	  coid=AE_Split_GetCommodityId(sp);
	  AQDB_Value_toHumanReadableString(v, tbuf, 2);
	  txt="<right>";
	  if (AQDB_Value_IsNegative(v))
	    txt+="<font color=red>";
	  txt+=GWEN_Buffer_GetStart(tbuf);
	  if (coid) {
	    AE_COMMODITY *co=AE_Commodity_List_GetById(m_commodities, coid);
	    if (co) {
	      const char *s;

	      s=AE_Commodity_GetSymbol(co);
	      if (s==NULL || *s==0)
		s=AE_Commodity_GetCode(co);
	      if (s && *s) {
		txt+=" ";
		txt+=s;
	      }
	    }
	  }
	  else {
	    DBG_ERROR(0, "Commodity %08x not found", coid);
	  }
          GWEN_Buffer_free(tbuf);
	  if (AQDB_Value_IsNegative(v))
	    txt+="</font>";
	  txt+="</right>";
	}
      }

      break;
    }

    case COLUMN_TYPE_CREDIT_ACCOUNT: {
      AE_SPLIT *spmax=NULL;
      AQDB_ID aid=0;
    
      spmax=AE_Transaction_GetMaxCreditSplit(t);
      if (spmax) {
        aid=AE_Split_GetAccountId(spmax);
        if (AE_Split_GetRuntimeFlags(spmax) & AE_SPLIT_RTFLAGS_FIX)
          /* done use special color */
          bg=getCellBackColor(r, c);
      }

      if (aid>0) {
	AE_ACCOUNT *acc;

	acc=AE_Account_Tree_GetById(m_accountTree, aid);
	if (acc) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	  AE_Account_PathToBuffer(acc, tbuf);
	  txt=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	}
      }
      break;
    }

    case COLUMN_TYPE_DEBIT_ACCOUNT: {
      AE_SPLIT *spmax=NULL;
      AQDB_ID aid=0;
    
      spmax=AE_Transaction_GetMaxDebitSplit(t);
      if (spmax) {
        aid=AE_Split_GetAccountId(spmax);
        if (AE_Split_GetRuntimeFlags(spmax) & AE_SPLIT_RTFLAGS_FIX)
          /* done use special color */
          bg=getCellBackColor(r, c);
      }

      if (aid>0) {
	AE_ACCOUNT *acc;

	acc=AE_Account_Tree_GetById(m_accountTree, aid);
	if (acc) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	  AE_Account_PathToBuffer(acc, tbuf);
	  txt=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	}
      }
      break;
    }

    case COLUMN_TYPE_DEBIT_VALUE:
      if (m_accountId) {
	AE_SPLIT *sp;

	/* first look for flagged split (this should be fast) */
	sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
	    break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *v=AE_Split_GetAmount(sp);
	  if (v) {
	    if (AQDB_Value_IsPositive(v)) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	      AQDB_ID coid;

	      coid=AE_Split_GetCommodityId(sp);
	      AQDB_Value_toHumanReadableString(v, tbuf, 2);
	      txt="<right>";
	      txt+=GWEN_Buffer_GetStart(tbuf);
	      if (coid) {
		AE_COMMODITY *co=AE_Commodity_List_GetById(m_commodities, coid);
		if (co) {
		  const char *s;

		  s=AE_Commodity_GetSymbol(co);
		  if (s==NULL || *s==0)
		    s=AE_Commodity_GetCode(co);
		  if (s && *s) {
		    txt+=" ";
		    txt+=s;
		  }
		}
	      }
	      else {
		DBG_ERROR(0, "Commodity %08x not found", coid);
	      }

	      txt+="</right>";
	    }
	  }
	}
      }
      break;

    case COLUMN_TYPE_CREDIT_VALUE:
      if (m_accountId) {
	AE_SPLIT *sp;

	/* first look for flagged split (this should be fast) */
	sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
	    break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *v=AE_Split_GetAmount(sp);
	  if (v) {
	    if (AQDB_Value_IsNegative(v)) {
	      AQDB_VALUE *v2=AQDB_Value_dup(v);
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	      AQDB_ID coid;

	      coid=AE_Split_GetCommodityId(sp);
	      AQDB_Value_Negate(v2);
	      AQDB_Value_toHumanReadableString(v2, tbuf, 2);
	      AQDB_Value_free(v2);
	      txt="<right>";
	      txt+=GWEN_Buffer_GetStart(tbuf);
	      if (coid) {
		AE_COMMODITY *co=AE_Commodity_List_GetById(m_commodities, coid);
		if (co) {
		  const char *s;

		  s=AE_Commodity_GetSymbol(co);
		  if (s==NULL || *s==0)
		    s=AE_Commodity_GetCode(co);
		  if (s && *s) {
		    txt+=" ";
		    txt+=s;
		  }
		}
	      }
	      else {
		DBG_ERROR(0, "Commodity %08x not found", coid);
	      }

	      txt+="</right>";
	    }
	  }
	}
      }
      break;

    case COLUMN_TYPE_DEPOSIT_VALUE:
      if (m_accountId) {
	AE_SPLIT *sp;

	/* first look for flagged split (this should be fast) */
	sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
	    break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *v=AE_Split_GetAmount(sp);
	  if (v) {
	    AE_ACCOUNT *acc;
	    bool isNegative=false;

	    isNegative=AQDB_Value_IsNegative(v);
	    acc=AE_Account_Tree_GetById(m_accountTree, m_accountId);
	    if (acc) {
	      if (AE_AccountType_NeedReverseSign(AE_Account_GetType(acc))) {
		if (isNegative)
		  isNegative=false;
		else
		  isNegative=true;
	      }
	    }

	    if (!isNegative) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	      AQDB_ID coid;

	      coid=AE_Split_GetCommodityId(sp);

	      if (AQDB_Value_IsNegative(v)) {
		AQDB_VALUE *v2;

		v2=AQDB_Value_dup(v);
                AQDB_Value_Negate(v2);
		AQDB_Value_toHumanReadableString(v2, tbuf, 2);
		AQDB_Value_free(v2);
	      }
	      else {
		AQDB_Value_toHumanReadableString(v, tbuf, 2);
	      }

	      txt="<right>";
	      txt+=GWEN_Buffer_GetStart(tbuf);
	      if (coid) {
		AE_COMMODITY *co=AE_Commodity_List_GetById(m_commodities, coid);
		if (co) {
		  const char *s;

		  s=AE_Commodity_GetSymbol(co);
		  if (s==NULL || *s==0)
		    s=AE_Commodity_GetCode(co);
		  if (s && *s) {
		    txt+=" ";
		    txt+=s;
		  }
		}
	      }
	      else {
		DBG_ERROR(0, "Commodity %08x not found", coid);
	      }

	      txt+="</right>";
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}
      }
      break;

    case COLUMN_TYPE_WITHDRAWAL_VALUE:
      if (m_accountId) {
	AE_SPLIT *sp;

	/* first look for flagged split (this should be fast) */
	sp=AE_Split_List_First(AE_Transaction_GetSplitList(t));
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
	    break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *v=AE_Split_GetAmount(sp);
	  if (v) {
	    AE_ACCOUNT *acc;
	    bool isNegative=false;

	    isNegative=AQDB_Value_IsNegative(v);
	    acc=AE_Account_Tree_GetById(m_accountTree, m_accountId);
	    if (acc) {
	      if (AE_AccountType_NeedReverseSign(AE_Account_GetType(acc))) {
		if (isNegative)
		  isNegative=false;
		else
		  isNegative=true;
	      }
	    }

	    if (isNegative) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 32, 0, 1);
	      AQDB_ID coid;

	      coid=AE_Split_GetCommodityId(sp);

	      if (AQDB_Value_IsNegative(v)) {
		AQDB_VALUE *v2;

		v2=AQDB_Value_dup(v);
                AQDB_Value_Negate(v2);
		AQDB_Value_toHumanReadableString(v2, tbuf, 2);
		AQDB_Value_free(v2);
	      }
	      else {
		AQDB_Value_toHumanReadableString(v, tbuf, 2);
	      }

	      txt="<right>";
	      txt+=GWEN_Buffer_GetStart(tbuf);
	      if (coid) {
		AE_COMMODITY *co=AE_Commodity_List_GetById(m_commodities, coid);
		if (co) {
		  const char *s;

		  s=AE_Commodity_GetSymbol(co);
		  if (s==NULL || *s==0)
		    s=AE_Commodity_GetCode(co);
		  if (s && *s) {
		    txt+=" ";
		    txt+=s;
		  }
		}
	      }
	      else {
		DBG_ERROR(0, "Commodity %08x not found", coid);
	      }

              txt+="</right>";
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}
      }
      break;


    case COLUMN_TYPE_COUNTER_ACCOUNT: {
      AE_SPLIT *sp;

      /* if m_accountId==0, then the counter split to the fixed one will be returned */
      sp=AE_Transaction_GetCounterSplit(t, m_accountId);
      if (sp) {
	AQDB_ID aid;

	aid=AE_Split_GetAccountId(sp);
	if (aid>0) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_Tree_GetById(m_accountTree, aid);
	  if (acc) {
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	    AE_Account_PathToBuffer(acc, tbuf);
	    txt=FXString(GWEN_Buffer_GetStart(tbuf));
	    GWEN_Buffer_free(tbuf);
	  }
	}
      }
      break;
    }

    case COLUMN_TYPE_IMPORT:
      if (AE_Transaction_GetRuntimeFlags(t) & AE_TRANSACTION_RTFLAGS_IMPORT) {
	/* mark as to-import */
        if (m_iconAccept) {
	  ColumnInfoIcons *cic=new ColumnInfoIcons(this, r, c, bg);
          cic->addIcon(m_iconAccept);
	  return cic;
	}
      }
      else {
	if (m_iconReject) {
	  ColumnInfoIcons *cic=new ColumnInfoIcons(this, r, c, bg);
	  cic->addIcon(m_iconReject);
	  return cic;
	}
      }
      break;

    case COLUMN_TYPE_FLAGS: {
      ColumnInfoIcons *cic=NULL;
      FXIcon *ic;
      uint32_t flags;

      cic=new ColumnInfoIcons(this, r, c, bg);

      flags=AE_Transaction_GetRuntimeFlags(t);
      if (flags & AE_TRANSACTION_RTFLAGS_MULTISPLIT)
        ic=m_iconMultiSplitOn;
      else
        ic=m_iconMultiSplitOff;
      if (ic)
        cic->addIcon(ic);

      if (flags & AE_TRANSACTION_RTFLAGS_MULTICURRENCY)
        ic=m_iconMultiCurrencyOn;
      else
        ic=m_iconMultiCurrencyOff;
      if (ic)
        cic->addIcon(ic);

      if (flags & (AE_TRANSACTION_RTFLAGS_NEW |
                   AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED |
                   AE_TRANSACTION_RTFLAGS_MODIFIED))
        ic=m_iconNewOn;
      else
        ic=m_iconNewOff;
      if (ic)
        cic->addIcon(ic);

      /* TODO: more flags to handle */

      return cic;
    }

    case COLUMN_TYPE_PARTNER: {
      AQDB_ID pid;

      pid=AE_Transaction_GetPartnerId(t);
      if (pid>0) {
	AE_PARTNER *pt;

	pt=AE_Partner_List_GetById(m_partners, pid);
	if (pt) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	  AE_Partner_PathToBuffer(pt, tbuf);
	  txt=FXString(GWEN_Buffer_GetStart(tbuf));
	  GWEN_Buffer_free(tbuf);
	}
      }
      break;
    }

    case COLUMN_TYPE_NONE:
    default:
      break;
    }

    return new ColumnInfoText(this, r, c, txt, bg);
  }
  else
    return NULL;
}



void FF_TransactionTable::releaseColumnInfo(FXXTable::ColumnInfo *ci) {
  FXTRACE((150,"%s::releaseColumnInfo %p\n",getClassName(),this));
  delete ci;
}




FXWindow *FF_TransactionTable::getEditor(int r, int c) {
  FXTRACE((150,"%s::getEditor %p\n",getClassName(),this));
  if (r<m_stmCount && c<MAX_COLUMNS) {
    AE_TRANSACTION *t;

    t=getTransactionByRow(r);
    if (t==NULL) {
      DBG_ERROR(0, "Transaction %d not found", r);
      return NULL;
    }

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE: {
      if (AE_Transaction_GetFlags(t) & AE_TRANSACTION_FLAGS_INITIAL) {
	DBG_INFO(AE_LOGDOMAIN, "Transaction for opening balance, not allowing editing of date");
        return NULL;
      }
      else {
	FF_DateEdit *ed=new FF_DateEdit(this, this, ID_DATE_EDIT, I18N("YYYY/MM/DD"));
	const GWEN_DATE *ti;

	ed->create();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);
	ti=AE_Transaction_GetDate(t);
	if (ti)
	  ed->setDate(ti);
	return ed;
      }
    }

    case COLUMN_TYPE_MEMO: {
      const char *s;
      FXTextField *ed=new FXTextField(this, 1, this, ID_TEXT_EDIT,
                                      TEXTFIELD_ENTER_ONLY,
                                      0, 0, 0, 0,
                                      0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      s=AE_Transaction_GetMemo(t);
      if (s) {
        ed->setText(s);
        ed->selectAll();
      }
      return ed;
    }

    case COLUMN_TYPE_VALUE:
      return NULL;

    case COLUMN_TYPE_CREDIT_ACCOUNT: {
      AQDB_ID aid=getCreditAccountId(t);
      FF_AccountEdit *ed=new FF_AccountEdit(this, this, ID_ACCOUNT_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();

      if (aid>0) {
        AE_ACCOUNT *acc;

        acc=AE_Account_Tree_GetById(m_accountTree, aid);
        if (acc) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	  AE_Account_PathToBuffer(acc, tbuf);
          ed->setText(GWEN_Buffer_GetStart(tbuf));
          ed->selectAll();
          GWEN_Buffer_free(tbuf);
        }
      }

      return ed;
    }

    case COLUMN_TYPE_DEBIT_ACCOUNT: {
      AQDB_ID aid=getDebitAccountId(t);
      FF_AccountEdit *ed=new FF_AccountEdit(this, this, ID_ACCOUNT_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();

      if (aid>0) {
        AE_ACCOUNT *acc;

	acc=AE_Account_Tree_GetById(m_accountTree, aid);
	if (acc) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	  AE_Account_PathToBuffer(acc, tbuf);
          ed->setText(GWEN_Buffer_GetStart(tbuf));
          ed->selectAll();
          GWEN_Buffer_free(tbuf);
        }
      }

      return ed;
    }

    case COLUMN_TYPE_COUNTER_ACCOUNT: {
      AE_SPLIT *sp;
      FF_AccountEdit *ed=new FF_AccountEdit(this, this, ID_ACCOUNT_EDIT, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();

      /* if m_accountId==0, then the counter split to the fixed one will be returned */
      sp=AE_Transaction_GetCounterSplit(t, m_accountId);
      if (sp) {
	AQDB_ID aid;
	aid=AE_Split_GetAccountId(sp);

	if (aid>0) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_Tree_GetById(m_accountTree, aid);
	  if (acc) {
	    GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	    AE_Account_PathToBuffer(acc, tbuf);
	    ed->setText(GWEN_Buffer_GetStart(tbuf));
            ed->selectAll();
	    GWEN_Buffer_free(tbuf);
	  }
	}
      }
      return ed;
    }

    case COLUMN_TYPE_CREDIT_VALUE: {
      if (m_accountId) {
	FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_VALUE_EDIT, 0);
	AE_SPLIT *sp;

	ed->create();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);

	sp=AE_Transaction_GetFirstSplit(t);
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
            break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *vt;
	  vt=AE_Split_GetAmount(sp);
	  if (vt) {
	    AQDB_VALUE *v;
	    GWEN_BUFFER *tbuf;

	    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    v=AQDB_Value_dup(vt);
	    AQDB_Value_Negate(v);
	    AE_Utils_AmountToString(v, AE_Split_GetCommodityId(sp), m_commodities, tbuf);
	    AQDB_Value_free(v);
	    ed->setText(GWEN_Buffer_GetStart(tbuf));
	    ed->selectAll();
	    GWEN_Buffer_free(tbuf);
	  }
	}

        return ed;
      }
      return NULL;
    }

    case COLUMN_TYPE_DEBIT_VALUE: {
      if (m_accountId) {
	FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_VALUE_EDIT, 0);
	AE_SPLIT *sp;

	ed->create();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);

	sp=AE_Transaction_GetFirstSplit(t);
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
            break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *vt;
	  vt=AE_Split_GetAmount(sp);
	  if (vt) {
	    GWEN_BUFFER *tbuf;

	    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    AE_Utils_AmountToString(vt, AE_Split_GetCommodityId(sp), m_commodities, tbuf);
	    ed->setText(GWEN_Buffer_GetStart(tbuf));
	    ed->selectAll();
	    GWEN_Buffer_free(tbuf);
	  }
	}

        return ed;
      }
      return NULL;
    }

    case COLUMN_TYPE_DEPOSIT_VALUE: {
      if (m_accountId) {
	FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_VALUE_EDIT, 0);
	AE_SPLIT *sp;

	ed->create();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);

	sp=AE_Transaction_GetFirstSplit(t);
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
            break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *v;

	  v=AE_Split_GetAmount(sp);
	  if (v) {
	    AE_ACCOUNT *acc;
	    bool isNegative=false;

	    isNegative=AQDB_Value_IsNegative(v);
	    acc=AE_Account_Tree_GetById(m_accountTree, m_accountId);
	    if (acc) {
	      if (AE_AccountType_NeedReverseSign(AE_Account_GetType(acc))) {
		if (isNegative)
		  isNegative=false;
		else
		  isNegative=true;
	      }
	    }

	    if (!isNegative) {
	      GWEN_BUFFER *tbuf;

	      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      if (AQDB_Value_IsNegative(v)) {
		AQDB_VALUE *v2;

		v2=AQDB_Value_dup(v);
		AQDB_Value_Negate(v2);
		AE_Utils_AmountToString(v2, AE_Split_GetCommodityId(sp), m_commodities, tbuf);
		AQDB_Value_free(v2);
	      }
	      else {
		AE_Utils_AmountToString(v, AE_Split_GetCommodityId(sp), m_commodities, tbuf);
	      }
	      ed->setText(GWEN_Buffer_GetStart(tbuf));
	      ed->selectAll();
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}

        return ed;
      }
      return NULL;
    }

    case COLUMN_TYPE_WITHDRAWAL_VALUE: {
      if (m_accountId) {
	FF_ValueEdit *ed=new FF_ValueEdit(this, 1, this, ID_VALUE_EDIT, 0);
	AE_SPLIT *sp;

	ed->create();
	ed->setFont(m_font);
	ed->setBackColor(getCellBackColor(r, c));
	ed->setTextColor(m_textColor);
	ed->setSelBackColor(m_selbackColor);
	ed->setSelTextColor(m_seltextColor);

	sp=AE_Transaction_GetFirstSplit(t);
	while(sp) {
	  if (AE_Split_GetAccountId(sp)==m_accountId)
            break;
	  sp=AE_Split_List_Next(sp);
	}

	if (sp) {
	  const AQDB_VALUE *v;

	  v=AE_Split_GetAmount(sp);
	  if (v) {
	    AE_ACCOUNT *acc;
	    bool isNegative=false;

	    isNegative=AQDB_Value_IsNegative(v);
	    acc=AE_Account_Tree_GetById(m_accountTree, m_accountId);
	    if (acc) {
	      if (AE_AccountType_NeedReverseSign(AE_Account_GetType(acc))) {
		if (isNegative)
		  isNegative=false;
		else
		  isNegative=true;
	      }
	    }

	    if (isNegative) {
	      GWEN_BUFFER *tbuf;

	      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      if (AQDB_Value_IsNegative(v)) {
		AQDB_VALUE *v2;

		v2=AQDB_Value_dup(v);
		AQDB_Value_Negate(v2);
		AE_Utils_AmountToString(v2, AE_Split_GetCommodityId(sp), m_commodities, tbuf);
		AQDB_Value_free(v2);
	      }
	      else {
		AE_Utils_AmountToString(v, AE_Split_GetCommodityId(sp), m_commodities, tbuf);
	      }
	      ed->setText(GWEN_Buffer_GetStart(tbuf));
	      ed->selectAll();
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}

        return ed;
      }
      return NULL;
    }

    case COLUMN_TYPE_IMPORT: {
      return NULL;
    }

    case COLUMN_TYPE_FLAGS: {
      return NULL;
    }

    case COLUMN_TYPE_PARTNER: {
      AQDB_ID pid=AE_Transaction_GetPartnerId(t);
      FF_PartnerEdit *ed=new FF_PartnerEdit(this, this, ID_PARTNER_EDIT,
					    0, /* opts */
					    0, 0, 0, 0,
					    0, 0, 0, 0);
      ed->create();
      ed->setFont(m_font);
      ed->setBackColor(getCellBackColor(r, c));
      ed->setTextColor(m_textColor);
      ed->setSelBackColor(m_selbackColor);
      ed->setSelTextColor(m_seltextColor);
      ed->reload();

      if (pid>0) {
        AE_PARTNER *pt;

	pt=AE_Partner_List_GetById(m_partners, pid);
	if (pt) {
	  GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

	  AE_Partner_PathToBuffer(pt, tbuf);
	  ed->setText(GWEN_Buffer_GetStart(tbuf));
	  ed->selectAll();
	  GWEN_Buffer_free(tbuf);
	}
      }

      return ed;
    }

    case COLUMN_TYPE_NONE:
    default:
      return NULL;
    }
  }
  else
    return NULL;
}



void FF_TransactionTable::setFromEditor(FXXTable::ColumnInfo *ci, FXWindow *editor) {
  int r;
  int c;

  FXTRACE((150,"%s::setFromEditor %p\n",getClassName(),this));
  r=ci->getRow();
  c=ci->getColumn();
  DBG_ERROR(0, "SetFromEditor(%d, %d, %p)", r, c, editor);

  if (r<m_stmCount && c<MAX_COLUMNS) {
    AE_TRANSACTION *t;

    t=getTransactionByRow(r);
    if (t==NULL) {
      DBG_ERROR(0, "Transaction %d not found", r);
      return;
    }

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE: {
      DBG_ERROR(0, "setFromEditor: date");
      FF_DateEdit *ed=dynamic_cast<FF_DateEdit*>(editor);
      if (ed) {
	GWEN_DATE *d=NULL;

	if (ed->getDate(&d)) {
	  AE_Transaction_SetDate(t, d);
          GWEN_Date_free(d);
	  AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	  rebuild();
	}
      }
      break;
    }

    case COLUMN_TYPE_MEMO: {
      FXTextField *ed=dynamic_cast<FXTextField*>(editor);
      if (ed) {
        FXString txt=ed->getText();
        reload();
        if (txt.empty())
          AE_Transaction_SetMemo(t, NULL);
        else
          AE_Transaction_SetMemo(t, txt.text());
        AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	rebuild();
      }

      break;
    }

    case COLUMN_TYPE_VALUE:
      break;

    case COLUMN_TYPE_CREDIT_ACCOUNT: {
      FF_AccountEdit *ed=dynamic_cast<FF_AccountEdit*>(editor);
      if (ed) {
        FXString txt=ed->getText();
        reload();
        if (!txt.empty()) {
          AE_ACCOUNT *acc=AE_Account_Tree_GetByPath(m_accountTree, NULL, txt.text());
          if (acc) {
            AE_SPLIT *sp;

            sp=AE_Transaction_GetMaxCreditSplit(t);
            if (sp) {
	      AE_Split_SetAccountId(sp, AE_Account_GetId(acc));
              AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MODIFIED);
              AE_Split_SubRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MATCHMASK);
              AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MATCHMANUAL);
              AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
            }
          }
          else {
            DBG_ERROR(0, "Account [%s] not found", txt.text());
          }
        }
        rebuild();
      }
      break;
    }

    case COLUMN_TYPE_DEBIT_ACCOUNT: {
      FF_AccountEdit *ed=dynamic_cast<FF_AccountEdit*>(editor);
      if (ed) {
        FXString txt=ed->getText();
        reload();
        if (!txt.empty()) {
          AE_ACCOUNT *acc=AE_Account_Tree_GetByPath(m_accountTree, NULL, txt.text());
          if (acc) {
            AE_SPLIT *sp;

            sp=AE_Transaction_GetMaxDebitSplit(t);
            if (sp) {
              AE_Split_SetAccountId(sp, AE_Account_GetId(acc));
              AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MODIFIED);
              AE_Split_SubRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MATCHMASK);
              AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MATCHMANUAL);
              AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
            }
          }
          else {
            DBG_ERROR(0, "Account [%s] not found", txt.text());
          }
        }
        rebuild();
      }
      break;
    }

    case COLUMN_TYPE_COUNTER_ACCOUNT: {
      FF_AccountEdit *ed=dynamic_cast<FF_AccountEdit*>(editor);
      if (ed) {
        AQDB_ID aid=ed->getAccountId();
        DBG_ERROR(0, "Setting counter account");
        reload();
        if (aid) {
          AE_SPLIT *sp;

	  /* if m_accountId==0, then the counter split to the fixed one will be returned */
	  sp=AE_Transaction_GetCounterSplit(t, m_accountId);
	  if (sp) {
	    AE_Split_SetAccountId(sp, aid);
	    AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MODIFIED);
            AE_Split_SubRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MATCHMASK);
            AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_MATCHMANUAL);
            AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
          }

          rebuild();
        }
        else {
          DBG_ERROR(0, "No account id");
	}
      }
      break;
    }

    case COLUMN_TYPE_PARTNER: {
      FF_PartnerEdit *ed=dynamic_cast<FF_PartnerEdit*>(editor);
      if (ed) {
	AE_Transaction_SetPartnerId(t, ed->getPartnerId());
	AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	reload();
	rebuild();
      }
      break;
    }

    case COLUMN_TYPE_CREDIT_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	AE_SPLIT *sp1=NULL;
	AE_SPLIT *sp2=NULL;

	/* get split for this account */
	if (m_accountId) {
	  sp1=AE_Transaction_GetFirstSplit(t);
	  while(sp1) {
	    if (AE_Split_GetAccountId(sp1)==m_accountId)
	      break;
	    sp1=AE_Split_List_Next(sp1);
	  }

	  sp2=AE_Transaction_GetCounterSplit(t, m_accountId);
	}

	if (sp1 && sp2) {
	  int rv;
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;

	  rv=AE_Utils_AmountFromString(ed->getText().simplify().text(),
				       m_commodities,
				       &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad amount");
	  }
	  else {
	    AE_Split_SetAmount(sp2, v);
	    if (co)
              AE_Split_SetCommodityId(sp2, AE_Commodity_GetId(co));
	    AQDB_Value_Negate(v);
	    AE_Split_SetAmount(sp1, v);
	    if (co)
	      AE_Split_SetCommodityId(sp1, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
            AE_Split_AddRuntimeFlags(sp1, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Split_AddRuntimeFlags(sp2, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
	  }
	}
	rebuild();
      }
      break;
    }

    case COLUMN_TYPE_DEBIT_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	AE_SPLIT *sp1=NULL;
	AE_SPLIT *sp2=NULL;

	/* get split for this account */
	if (m_accountId) {
	  sp1=AE_Transaction_GetFirstSplit(t);
	  while(sp1) {
	    if (AE_Split_GetAccountId(sp1)==m_accountId)
	      break;
	    sp1=AE_Split_List_Next(sp1);
	  }

	  sp2=AE_Transaction_GetCounterSplit(t, m_accountId);
	}

	if (sp1 && sp2) {
	  int rv;
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;

	  rv=AE_Utils_AmountFromString(ed->getText().simplify().text(),
				       m_commodities,
				       &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad amount");
	  }
	  else {
	    AE_Split_SetAmount(sp1, v);
	    if (co)
	      AE_Split_SetCommodityId(sp1, AE_Commodity_GetId(co));
	    AQDB_Value_Negate(v);
	    AE_Split_SetAmount(sp2, v);
	    if (co)
	      AE_Split_SetCommodityId(sp2, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
            AE_Split_AddRuntimeFlags(sp1, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Split_AddRuntimeFlags(sp2, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
	  }
	}
	rebuild();
      }
      break;
    }

    case COLUMN_TYPE_DEPOSIT_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	AE_SPLIT *sp1=NULL;
	AE_SPLIT *sp2=NULL;
        bool isNegative=false;

	/* get split for this account */
	if (m_accountId) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_Tree_GetById(m_accountTree, m_accountId);
	  if (acc)
	    isNegative=AE_AccountType_NeedReverseSign(AE_Account_GetType(acc));

	  sp1=AE_Transaction_GetFirstSplit(t);
	  while(sp1) {
	    if (AE_Split_GetAccountId(sp1)==m_accountId)
	      break;
	    sp1=AE_Split_List_Next(sp1);
	  }

	  sp2=AE_Transaction_GetCounterSplit(t, m_accountId);
	}

	if (sp1 && sp2) {
	  int rv;
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;

	  rv=AE_Utils_AmountFromString(ed->getText().simplify().text(),
				       m_commodities,
				       &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad amount");
	  }
	  else {
	    if (isNegative)
              AQDB_Value_Negate(v);
	    AE_Split_SetAmount(sp1, v);
	    if (co)
	      AE_Split_SetCommodityId(sp1, AE_Commodity_GetId(co));
	    AQDB_Value_Negate(v);
	    AE_Split_SetAmount(sp2, v);
	    if (co)
	      AE_Split_SetCommodityId(sp2, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
            AE_Split_AddRuntimeFlags(sp1, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Split_AddRuntimeFlags(sp2, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
	  }
	}
	rebuild();
      }
      break;
    }

    case COLUMN_TYPE_WITHDRAWAL_VALUE: {
      FF_ValueEdit *ed=dynamic_cast<FF_ValueEdit*>(editor);
      if (ed) {
	AE_SPLIT *sp1=NULL;
	AE_SPLIT *sp2=NULL;
        bool isNegative=false;

	/* get split for this account */
	if (m_accountId) {
	  AE_ACCOUNT *acc;

	  acc=AE_Account_Tree_GetById(m_accountTree, m_accountId);
	  if (acc)
	    isNegative=AE_AccountType_NeedReverseSign(AE_Account_GetType(acc));

	  sp1=AE_Transaction_GetFirstSplit(t);
	  while(sp1) {
	    if (AE_Split_GetAccountId(sp1)==m_accountId)
	      break;
	    sp1=AE_Split_List_Next(sp1);
	  }

	  sp2=AE_Transaction_GetCounterSplit(t, m_accountId);
	}

	if (sp1 && sp2) {
	  int rv;
	  AQDB_VALUE *v=NULL;
          AE_COMMODITY *co=NULL;

	  rv=AE_Utils_AmountFromString(ed->getText().simplify().text(),
				       m_commodities,
				       &v, &co);
	  if (rv<0) {
            DBG_ERROR(0, "Bad amount");
	  }
	  else {
	    if (!isNegative)
	      AQDB_Value_Negate(v);
	    AE_Split_SetAmount(sp1, v);
	    if (co)
	      AE_Split_SetCommodityId(sp1, AE_Commodity_GetId(co));
	    AQDB_Value_Negate(v);
	    AE_Split_SetAmount(sp2, v);
	    if (co)
	      AE_Split_SetCommodityId(sp2, AE_Commodity_GetId(co));
	    AQDB_Value_free(v);
            AE_Split_AddRuntimeFlags(sp1, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Split_AddRuntimeFlags(sp2, AE_SPLIT_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MODIFIED);
	    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SPLITS_MODIFIED);
	  }
	}
	rebuild();
      }
      break;
    }



    case COLUMN_TYPE_IMPORT:
    case COLUMN_TYPE_FLAGS:
    case COLUMN_TYPE_NONE:
    default:
      break;
    }

    rebuildTransactionflags(t);
  }
}



void FF_TransactionTable::releaseEditor(int r, int c, FXWindow *ed) {
  FXTRACE((150,"%s::releaseEditor %p\n",getClassName(),this));
  DBG_ERROR(0, "ReleaseEditor(%d, %d, %p)", r, c, ed);
  reload();
  delete ed;
}



AE_TRANSACTION *FF_TransactionTable::getTransactionByRow(int r) {
  if (m_stmArray && r<m_stmCount)
    return m_stmArray[r];

  return NULL;
}



void FF_TransactionTable::addSpaceForTransactions(int n) {
  if (m_stmArray) {
    AE_TRANSACTION **p;
    int i;

    p=(AE_TRANSACTION**)malloc((n+m_stmArraySize)*sizeof(AE_TRANSACTION*));
    memset(p, 0, (n+m_stmArraySize)*sizeof(AE_TRANSACTION*));
    for (i=0; i<m_stmCount; i++) {
      p[i]=m_stmArray[i];
    }
    free(m_stmArray);
    m_stmArray=p;
    m_stmArraySize+=n;
  }
  else {
    m_stmArray=(AE_TRANSACTION**)malloc(n*sizeof(AE_TRANSACTION*));
    memset(m_stmArray, 0, n*sizeof(AE_TRANSACTION*));
    m_stmArraySize=n;
    m_stmCount=0;
  }
}



void FF_TransactionTable::addTransaction(AE_TRANSACTION *t) {
  rebuildTransactionflags(t);

  if (m_stmArray==NULL)
    addSpaceForTransactions(TRANSACTION_BUNCH);
  if ((m_stmCount+1)>=m_stmArraySize)
    addSpaceForTransactions(TRANSACTION_BUNCH);

  m_stmArray[m_stmCount++]=t;
  if (m_stmCount>getNumberOfRows())
    FXXTable::setNumberOfRows(m_stmCount);
}



void FF_TransactionTable::delRow(int r) {
  if (r>=0 && r<m_stmCount) {
    int j;

    AE_Transaction_free(m_stmArray[r]);
    for (j=r+1; j<m_stmCount; j++)
      m_stmArray[j-1]=m_stmArray[j];
    m_stmArray[j-1]=0;
    m_stmCount--;
    setNumberOfRows(m_stmCount);
  }
}



void FF_TransactionTable::addTransactions(AE_TRANSACTION_LIST2 *sl, int copyMode) {
  AE_TRANSACTION_LIST2_ITERATOR *it;

  it=AE_Transaction_List2_First(sl);
  if (it) {
    AE_TRANSACTION *st;


    switch(copyMode) {
      case COPYMODE_COPY:
        st=AE_Transaction_List2Iterator_Data(it);
        while(st) {
          addTransaction(AE_Transaction_dup(st));
          st=AE_Transaction_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_ATTACH:
        st=AE_Transaction_List2Iterator_Data(it);
        while(st) {
          AE_Transaction_Attach(st);
          addTransaction(st);
          st=AE_Transaction_List2Iterator_Next(it);
        }
        break;
  
      case COPYMODE_NONE:
      default:
        st=AE_Transaction_List2Iterator_Data(it);
        while(st) {
          addTransaction(st);
          st=AE_Transaction_List2Iterator_Next(it);
        }
    }

    AE_Transaction_List2Iterator_free(it);
  }
}



void FF_TransactionTable::getSelectedTransactions(AE_TRANSACTION_LIST2 *sl, int copyMode) {
  const SelectionRange *r=getSelection().getFirstRange();

  while(r) {
    int i;

    for (i=r->first; i<=r->last; i++) {
      AE_TRANSACTION *st;

      st=getTransactionByRow(i);
      if (st) {
        switch(copyMode) {
        case COPYMODE_COPY:
          AE_Transaction_List2_PushBack(sl, AE_Transaction_dup(st));
          break;

        case COPYMODE_ATTACH:
          AE_Transaction_Attach(st);
          AE_Transaction_List2_PushBack(sl, st);
          break;

        case COPYMODE_NONE:
        default:
          AE_Transaction_List2_PushBack(sl, st);
          break;
        }
      }
    }

    r=r->next;
  }
}



void FF_TransactionTable::clearItems() {
  FXXTable::clearItems();
  setNumberOfRows(0);
}



int FF_TransactionTable::compareDateUp(const void *a, const void *b) {
  const AE_TRANSACTION *st1=*((const AE_TRANSACTION**) a);
  const AE_TRANSACTION *st2=*((const AE_TRANSACTION**) b);

  const GWEN_DATE *t1=AE_Transaction_GetDate(st1);
  const GWEN_DATE *t2=AE_Transaction_GetDate(st2);

  if (AE_Transaction_GetRuntimeFlags(st1) & AE_TRANSACTION_RTFLAGS_SPARE)
    return 1;
  else if (AE_Transaction_GetRuntimeFlags(st2) & AE_TRANSACTION_RTFLAGS_SPARE)
    return -1;

  /* both are data items */
  if (t1 && t2)
    return GWEN_Date_Compare(t1, t2);
  else if (t1)
    return 1;
  else if (t2)
    return -1;

  return 0;
}



int FF_TransactionTable::compareDateDown(const void *a, const void *b) {
  const AE_TRANSACTION *st1=*((const AE_TRANSACTION**) a);
  const AE_TRANSACTION *st2=*((const AE_TRANSACTION**) b);

  /* reversed */
  const GWEN_DATE *t2=AE_Transaction_GetDate(st1);
  const GWEN_DATE *t1=AE_Transaction_GetDate(st2);

  if (AE_Transaction_GetRuntimeFlags(st1) & AE_TRANSACTION_RTFLAGS_SPARE)
    return -1;
  else if (AE_Transaction_GetRuntimeFlags(st2) & AE_TRANSACTION_RTFLAGS_SPARE)
    return 1;

  /* both are data items */
  if (t1 && t2)
    return GWEN_Date_Compare(t1, t2);
  else if (t1)
    return 1;
  else if (t2)
    return -1;

  return 0;
}



void FF_TransactionTable::sortByColumn(int c, bool up) {
  if (c<MAX_COLUMNS) {

    switch(m_columnTypes[c]) {
    case COLUMN_TYPE_DATE:
      if (up)
        qsort(m_stmArray, m_stmCount, sizeof(AE_TRANSACTION*), compareDateUp);
      else
        qsort(m_stmArray, m_stmCount, sizeof(AE_TRANSACTION*), compareDateDown);
      break;

    case COLUMN_TYPE_MEMO:
      break;

    case COLUMN_TYPE_VALUE:
      break;

    case COLUMN_TYPE_CREDIT_ACCOUNT:
      break;

    case COLUMN_TYPE_DEBIT_ACCOUNT:
      break;

    case COLUMN_TYPE_NONE:
    default:
      break;
    }
  }

  releaseAllRowInfos();
  m_firstVisibleRow=0;
  m_alignTop=true;
  m_needLayout=true;
  update();
}



AE_TRANSACTION *FF_TransactionTable::getCurrentTransaction() {
  int r;

  r=getCurrentRow();
  if (r>=0 && r<m_stmCount) {
    AE_TRANSACTION *st;

    st=getTransactionByRow(r);
    if (st==NULL)
      return NULL;
    return st;
  }

  return NULL;
}



int FF_TransactionTable::reload() {
  FF_App *app;
  AE_BOOK *b;

  app=dynamic_cast<FF_App*>(getApp());
  b=app->getBook();

  AE_Account_Tree_free(m_accountTree);
  m_accountTree=AE_Account_Tree_new();

  AE_Commodity_List_free(m_commodities);
  m_commodities=AE_Commodity_List_new();

  AE_Partner_List_free(m_partners);
  m_partners=AE_Partner_List_new();

  if (b) {
    int rv;

    /* read account tree */
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, m_accountTree);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      return rv;
    }

    /* read commodity list */
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, m_commodities);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      return rv;
    }

    /* read partner list */
    rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, m_partners);
    if (rv<0) {
      DBG_ERROR(0, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



void FF_TransactionTable::rebuildTransactionflags(AE_TRANSACTION *t) {
  AE_SPLIT_LIST *spl;

  spl=AE_Transaction_GetSplitList(t);
  if (spl) {
    AQDB_ID idList[MAX_COMMODITIES_PER_TRANSACTION];
    int rv;

    /* handle multiSplit flag */
    if (AE_Split_List_GetCount(spl)>2)
      AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTISPLIT);
    else
      AE_Transaction_SubRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTISPLIT);

    /* handle multi-currency flag */
    rv=AE_Transaction_SampleCommodities(t, idList, MAX_COMMODITIES_PER_TRANSACTION);
    if (rv>1)
      AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTICURRENCY);
    else
      AE_Transaction_SubRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTICURRENCY);
  }
}



long FF_TransactionTable::onCmdAccount(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "onCmdAccount");
  acceptInput(true);
  return 1;
}



long FF_TransactionTable::onCancelAccount(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  cancelInput(true);
  return 1;
}



long FF_TransactionTable::onCmdPartner(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  acceptInput(true);
  return 1;
}



long FF_TransactionTable::onCancelPartner(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  cancelInput(true);
  return 1;
}



long FF_TransactionTable::onCmdValue(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  acceptInput(true);
  return 1;
}



long FF_TransactionTable::onCancelValue(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  cancelInput(true);
  return 1;
}



long FF_TransactionTable::onCmdDate(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  acceptInput(true);
  return 1;
}



long FF_TransactionTable::onCancelDate(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  cancelInput(true);
  return 1;
}



long FF_TransactionTable::onCmdTextField(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  acceptInput(true);
  return 1;
}



long FF_TransactionTable::onCancelTextField(FXObject*, FXSelector, void* ptr) {
  DBG_ERROR(0, "Mark");
  cancelInput(true);
  return 1;
}



