/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_transfertree.hpp"

#include <fx.h>

#include <aqbanking/jobgetbalance.h>
#include <aqbanking/jobgettransactions.h>
#include <aqbanking/jobgetstandingorders.h>
#include <aqbanking/jobsingletransfer.h>
#include <aqbanking/jobsingledebitnote.h>
#include <aqbanking/jobeutransfer.h>
#include <aqbanking/jobgetdatedtransfers.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING 4
#define ICON_SPACING 4



FF_TransferTreeItem::FF_TransferTreeItem(AE_STATEMENT *st, void* ptr)
:FF_GroupTreeItem(NULL, NULL, ptr)
,m_transfer(st)
{
}



FF_TransferTreeItem::~FF_TransferTreeItem() {
  if (m_isDataItem && m_transfer) {
    AE_Statement_free(m_transfer);
    m_transfer=(AE_STATEMENT*) -1;
  }
}



FXString FF_TransferTreeItem::getItemText(const FXXCheckTree* list) const {
  const FF_TransferTree *tree;
  FXString t;

  tree=dynamic_cast<const FF_TransferTree*>(list);
  if (tree) {
    int i;

    for (i=0; i<tree->m_columns; i++) {
      const char *s;

      switch(tree->m_columnTypes[i]) {
      case FF_TransferTree::COLUMN_DATE:
        {
          const GWEN_DATE *ti;

          ti=AE_Statement_GetDate(m_transfer);
          if (ti) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
          }
        }
        break;

      case FF_TransferTree::COLUMN_REMOTE_NAME:
        s=AE_Statement_GetRemoteName(m_transfer);
        if (s)
          t+=FXString(s);
        break;

      case FF_TransferTree::COLUMN_PURPOSE:
        s=AE_Statement_GetPurpose(m_transfer);
        if (s) {
          FXString pt;
      
          if (strchr(s, '\n')) {
            /* multi-line purpose, handle appropriately */
            while(*s) {
              const char *ts;
      
              ts=strchr(s, '\n');
              if (ts) {
                int l;
                char *cpy;
      
                l=ts-s;
                cpy=(char*) malloc(l+1);
                assert(cpy);
                memmove(cpy, s, l);
                cpy[l]=0;
                if (!pt.empty())
                  pt+="-";
                pt+=FXString(cpy);
                free(cpy);
                s=ts;
                s++;
              }
              else {
                if (!pt.empty())
                  pt+="-";
                pt+=FXString(s);
                s+=strlen(s);
              }
            }
            t+=pt;
          }
          else
            /* single-line purpose, simply add */
            t+=FXString(s);
        }
        break;

      case FF_TransferTree::COLUMN_TYPE:
        {
          switch((AE_Statement_GetFlags(m_transfer) & AE_STATEMENT_FLAGS_XFER_TYPE_MASK)) {
          case AE_STATEMENT_FLAGS_XFER_TYPE_STD_XFER:
            t+=FXString(I18N("FF_TransferTreeItem|Transfer"));
            break;
          case AE_STATEMENT_FLAGS_XFER_TYPE_INT_XFER:
            t+=FXString(I18N("FF_TransferTreeItem|Internal Transfer"));
            break;
          case AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_XFER:
            t+=FXString(I18N("FF_TransferTreeItem|SEPA Transfer"));
            break;
          case AE_STATEMENT_FLAGS_XFER_TYPE_SEPA_DEBIT:
            t+=FXString(I18N("FF_TransferTreeItem|SEPA Direct Debit"));
            break;
          case AE_STATEMENT_FLAGS_XFER_TYPE_DATED_XFER:
            t+=FXString(I18N("FF_TransferTreeItem|Dated Transfer"));
            break;
          case AE_STATEMENT_FLAGS_XFER_TYPE_STO_XFER:
            t+=FXString(I18N("FF_TransferTreeItem|Standing Orderr"));
            break;
          }
          break;
        }

      case FF_TransferTree::COLUMN_AMOUNT:
        {
          const AQDB_VALUE *v;

          v=AE_Statement_GetValue(m_transfer);
          if (v) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

            t+="<right>";

            AQDB_Value_toHumanReadableString(v, tbuf, 2);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
            s=AE_Statement_GetCurrency(m_transfer);
            if (s) {
              t+=" ";
              t+=s;
            }
            t+="</right>";
          }
        }
        break;


      case FF_TransferTree::COLUMN_STATUS:
        switch(AE_Statement_GetStatus(m_transfer)) {
        case AE_Statement_StatusNone:               s=""; break;
        case AE_Statement_StatusEnqueued:           s=I18N("StatementStatus|enqueued"); break;
        case AE_Statement_StatusSending:            s=I18N("StatementStatus|sending"); break;
        case AE_Statement_StatusAccepted:           s=I18N("StatementStatus|accepted"); break;
        case AE_Statement_StatusRejected:           s=I18N("StatementStatus|rejected"); break;
        case AE_Statement_StatusPending:            s=I18N("StatementStatus|pending"); break;
        case AE_Statement_StatusAutoReconciled:     s=I18N("StatementStatus|reconciled (auto)"); break;
        case AE_Statement_StatusManuallyReconciled: s=I18N("StatementStatus|reconciled (man)"); break;
        case AE_Statement_StatusRevoked:            s=I18N("StatementStatus|revoked"); break;
        case AE_Statement_StatusAborted:            s=I18N("StatementStatus|aborted"); break;
        case AE_Statement_StatusUnknown:            
        default:                                    s=""; break;
        }
        if (s)
          t+=FXString(s);
        break;

      case FF_TransferTree::COLUMN_DATE_BEGIN:
        {
	  const AE_RECURRENCE *rec;

	  rec=AE_Statement_GetRecurrence(m_transfer);
	  if (rec) {
	    const GWEN_DATE *ti;

	    ti=AE_Recurrence_GetBeginDate(rec);
	    if (ti) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
	      t+=GWEN_Buffer_GetStart(tbuf);
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}
	break;

      case FF_TransferTree::COLUMN_DATE_END:
        {
	  const AE_RECURRENCE *rec;

	  rec=AE_Statement_GetRecurrence(m_transfer);
	  if (rec) {
	    const GWEN_DATE *ti;

	    ti=AE_Recurrence_GetEndDate(rec);
	    if (ti) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
	      t+=GWEN_Buffer_GetStart(tbuf);
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}
	break;

      case FF_TransferTree::COLUMN_DATE_LAST:
        {
	  const AE_RECURRENCE *rec;

	  rec=AE_Statement_GetRecurrence(m_transfer);
	  if (rec) {
	    const GWEN_DATE *ti;

	    ti=AE_Recurrence_GetLastExecuted(rec);
	    if (ti) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
	      t+=GWEN_Buffer_GetStart(tbuf);
	      GWEN_Buffer_free(tbuf);
	    }
	  }
	}
	break;

      case FF_TransferTree::COLUMN_DATE_LASTUPDATED:
        {
	  const AE_RECURRENCE *rec;

	  rec=AE_Statement_GetRecurrence(m_transfer);
	  if (rec) {
	    const GWEN_DATE *ti;

	    ti=AE_Recurrence_GetLastUpdated(rec);
	    if (ti) {
	      GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
	      t+=GWEN_Buffer_GetStart(tbuf);
	      GWEN_Buffer_free(tbuf);
	    }
          }
          else {
            DBG_ERROR(0, "No recurrence??");
          }
	}
	break;

      case FF_TransferTree::COLUMN_PERIOD:
        {
	  const AE_RECURRENCE *rec;

	  rec=AE_Statement_GetRecurrence(m_transfer);
	  if (rec) {
	    char numbuf[256];

	    switch(AE_Recurrence_GetPeriod(rec)) {
	      case AE_Recurrence_PeriodDays:
		snprintf(numbuf, sizeof(numbuf)-1, I18N("Every %d day(s)"),
			 AE_Recurrence_GetEvery(rec));
		numbuf[sizeof(numbuf)-1]=0;
		t+=numbuf;
		break;
  
	      case AE_Recurrence_PeriodWeeks:
		{
		  const char *wd;
  
		  switch(AE_Recurrence_GetOn(rec)) {
		    case 1:  wd=I18N("Monday");    break;
		    case 2:  wd=I18N("Tuesday");   break;
		    case 3:  wd=I18N("Wednesday"); break;
		    case 4:  wd=I18N("Thursday");  break;
		    case 5:  wd=I18N("Friday");    break;
		    case 6:  wd=I18N("Saturday");  break;
		    case 7:  wd=I18N("Sunday");    break;
		    default: wd=I18N("unknown");   break;
		  }
		  snprintf(numbuf, sizeof(numbuf)-1, I18N("Every %d week(s) on %s"),
			   AE_Recurrence_GetEvery(rec), wd);
		  numbuf[sizeof(numbuf)-1]=0;
		  t+=numbuf;
		}
		break;
  
	      case AE_Recurrence_PeriodMonths:
		snprintf(numbuf, sizeof(numbuf)-1, I18N("Every %d month(s) on %d"),
			 AE_Recurrence_GetEvery(rec), AE_Recurrence_GetOn(rec));
		numbuf[sizeof(numbuf)-1]=0;
		t+=numbuf;
		break;
  
	      case AE_Recurrence_PeriodYears:
		snprintf(numbuf, sizeof(numbuf)-1, I18N("Every %d year(s)"),
			 AE_Recurrence_GetEvery(rec));
		numbuf[sizeof(numbuf)-1]=0;
		t+=numbuf;
		break;
  
	      default:
		break;
	    }
	  }
	}
	break;

      default:
        break;
      }
      t+="\t";
    } /* for */
  }

  return t;
}



FXString FF_TransferTreeItem::getColumnTooltipText(FXint col) {
  FXString result;

  if (m_transfer) {
    const char *s;
    const char *s1;
    const char *s2;
    const AQDB_VALUE *v;

    result="<html>";
    if (AE_Statement_GetFlags(m_transfer) & AE_STATEMENT_FLAGS_INTERNAL_XFER)
      result+=I18N("<h3>Internal Transfer</h3>\n");
    else
      result+=I18N("<h3>Standard Transfer</h3>\n");

    result+="<table>";

    s=AE_Statement_GetLocalName(m_transfer);
    if (s && *s) {
      result+="<tr><td><b>";
      result+=I18N("From:");
      result+="</b></td><td>";
      result+=s;
      result+="</td></tr>";
    }

    s1=AE_Statement_GetLocalBankCode(m_transfer);
    s2=AE_Statement_GetLocalAccountNumber(m_transfer);
    if ((s1 && *s1 && s2 && *s2)) {
      result+="<tr><td><b>";
      result+=I18N("From Account:");
      result+="</b></td><td>";
      result+=I18N("Bank Code: ");
      result+=s1;
      result+=" / ";
      result+=I18N("Account Number: ");
      result+=s2;
      result+="</td></tr>";
    }
    else {
      s1=AE_Statement_GetLocalBic(m_transfer);
      s2=AE_Statement_GetLocalIban(m_transfer);
      if ((s1 && *s1 && s2 && *s2)) {
        result+="<tr><td><b>";
        result+=I18N("From Account:");
        result+="</b></td><td>";
        result+=I18N("SWIFT Code: ");
        result+=s1;
        result+=" / ";
        result+=I18N("IBAN: ");
        result+=s2;
        result+="</td></tr>";
      }
    }

    s=AE_Statement_GetRemoteName(m_transfer);
    if (s && *s) {
      result+="<tr><td><b>";
      result+=I18N("To:");
      result+="</b></td><td>";
      result+="<font color=\"dark green\">";
      result+=s;
      result+="</font>";
      result+="</td></tr>";
    }

    s1=AE_Statement_GetRemoteBankCode(m_transfer);
    s2=AE_Statement_GetRemoteAccountNumber(m_transfer);
    if ((s1 && *s1 && s2 && *s2)) {
      result+="<tr><td><b>";
      result+=I18N("To Account:");
      result+="</b></td><td>";
      result+=I18N("Bank Code: ");
      result+=s1;
      result+=" / ";
      result+=I18N("Account Number: ");
      result+=s2;
      result+="</td></tr>";
    }
    else {
      s1=AE_Statement_GetRemoteBic(m_transfer);
      s2=AE_Statement_GetRemoteIban(m_transfer);
      if ((s1 && *s1 && s2 && *s2)) {
        result+="<tr><td><b>";
        result+=I18N("To Account:");
        result+="</b></td><td>";
        result+=I18N("SWIFT Code: ");
        result+=s1;
        result+=" / ";
        result+=I18N("IBAN: ");
        result+=s2;
        result+="</td></tr>";
      }
    }

    v=AE_Statement_GetValue(m_transfer);
    if (v) {
      GWEN_BUFFER *tbuf;

      result+="<tr><td><b>";
      result+=I18N("Amount:");
      result+="</b></td><td>";
      result+="<font color=\"red\">";

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AQDB_Value_toHumanReadableString(v, tbuf, 2);
      result+=GWEN_Buffer_GetStart(tbuf);
      GWEN_Buffer_free(tbuf);
      s=AE_Statement_GetCurrency(m_transfer);
      if (s) {
        result+=" ";
        result+=s;
      }
      result+="</font>";
      result+="</td></tr>";
    }

    switch(AE_Statement_GetStatus(m_transfer)) {
    case AE_Statement_StatusNone:               s=""; break;
    case AE_Statement_StatusEnqueued:           s=I18N("StatementStatus|enqueued"); break;
    case AE_Statement_StatusSending:            s=I18N("StatementStatus|sending"); break;
    case AE_Statement_StatusAccepted:           s=I18N("StatementStatus|accepted"); break;
    case AE_Statement_StatusRejected:           s=I18N("StatementStatus|rejected"); break;
    case AE_Statement_StatusPending:            s=I18N("StatementStatus|pending"); break;
    case AE_Statement_StatusAutoReconciled:     s=I18N("StatementStatus|reconciled (auto)"); break;
    case AE_Statement_StatusManuallyReconciled: s=I18N("StatementStatus|reconciled (man)"); break;
    case AE_Statement_StatusRevoked:            s=I18N("StatementStatus|revoked"); break;
    case AE_Statement_StatusAborted:            s=I18N("StatementStatus|aborted"); break;
    case AE_Statement_StatusUnknown:            
    default:                                    s=NULL; break;
    }
    if (s) {
      result+="<tr><td><b>";
      result+=I18N("Status:");
      result+="</b></td><td>";
      result+=FXString(s);
      result+="</td></tr>";
    }


    s=AE_Statement_GetPurpose(m_transfer);
    if (s) {
      FXString pt;
  
      result+="<tr><td><b>";
      result+=I18N("Purpose:");
      result+="</b></td><td>";
      if (strchr(s, '\n')) {
        /* multi-line purpose, handle appropriately */
        while(*s) {
          const char *ts;
  
          ts=strchr(s, '\n');
          if (ts) {
            int l;
            char *cpy;
  
            l=ts-s;
            cpy=(char*) malloc(l+1);
            assert(cpy);
            memmove(cpy, s, l);
            cpy[l]=0;
            if (!pt.empty())
              pt+="<br>";
            pt+=FXString(cpy);
            free(cpy);
            s=ts;
            s++;
          }
          else {
            if (!pt.empty())
              pt+="<br>";
            pt+=FXString(s);
            s+=strlen(s);
          }
        }
        result+=pt;
      }
      else
        /* single-line purpose, simply add */
        result+=FXString(s);
      result+="</td></tr>";
    }


    result+="</html>";
  }

  return result;
}






FF_TransferTree::FF_TransferTree(FXComposite *p,
                                 FXObject* tgt, FXSelector sel,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_columns(0)
{
}



void FF_TransferTree::addColumn(int c, int w) {
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_DATE:
      appendHeader(I18N("TransferTree|Date"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_REMOTE_NAME:
      appendHeader(I18N("TransferTree|Name"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_PURPOSE:
      appendHeader(I18N("TransferTree|Purpose"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_TYPE:
      appendHeader(I18N("TransferTree|Type"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_AMOUNT:
      appendHeader(I18N("TransferTree|Amount"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_STATUS:
      appendHeader(I18N("TransferTree|Status"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_DATE_BEGIN:
      appendHeader(I18N("TransferTree|Start Date"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_DATE_END:
      appendHeader(I18N("TransferTree|End Date"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_DATE_LAST:
      appendHeader(I18N("TransferTree|Last"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case FF_TransferTree::COLUMN_DATE_LASTUPDATED:
      appendHeader(I18N("TransferTree|Last Update"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_PERIOD:
      appendHeader(I18N("TransferTree|Period"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    default:
      break;
    }
  }
}



FF_TransferTree::~FF_TransferTree() {
}



FF_TransferTreeItem *FF_TransferTree::findTransferItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_TransferTreeItem *res=findTransferItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_TransferTreeItem *FF_TransferTree::findTransferItem(FF_GroupTreeItem *grItem,
						       AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_TransferTreeItem *stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
    if (stItem) {
      AE_STATEMENT *st=stItem->getTransfer();
      if (AE_Statement_GetId(st)==id)
        return stItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_TransferTreeItem *res=findTransferItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_TransferTreeItem *FF_TransferTree::findTransferItem(const AE_STATEMENT *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TransferTreeItem *baItem;

      baItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
      if (baItem) {
        AE_STATEMENT *sta;

	sta=baItem->getTransfer();
	if (sta==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_TransferTree::addTransfer(AE_STATEMENT *st, bool direct, int useColor) {
  FF_GroupTreeItem *itemFather=NULL;

  if (!direct) {
    AQDB_ID gid=AE_Statement_GetGroupId(st);
    if (gid) {
      itemFather=findGroupItem(gid);
      if (itemFather==NULL) {
        DBG_INFO(AE_LOGDOMAIN, "Group for statement not found, reassigning to root");
      }
    }
  }

  addTransferToItem(st, itemFather, useColor);
}



void FF_TransferTree::addTransferToGroup(AE_STATEMENT *st, AQDB_ID gid, int useColor) {
  FF_GroupTreeItem *itemFather=NULL;

  if (gid) {
    itemFather=findGroupItem(gid);
    if (itemFather==NULL) {
      DBG_INFO(AE_LOGDOMAIN, "Group for statement not found, reassigning to root");
    }
  }

  addTransferToItem(st, itemFather, useColor);
}



void FF_TransferTree::addTransferToItem(AE_STATEMENT *st, FF_GroupTreeItem *itemFather, int useColor) {
  FF_TransferTreeItem *item;

  if (itemFather==NULL) {
    DBG_ERROR(0, "No fatherItem");
  }

  item=new FF_TransferTreeItem(st);
  /* maybe colorize */
  switch(useColor) {
  case FF_TransferTree::USECOLOR_NOTICE:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(247, 248, 96));
    item->setTextColor(FXRGB(0, 0, 0));
    break;
  case FF_TransferTree::USECOLOR_WARNING:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(255, 127, 0));
    item->setTextColor(FXRGB(255, 255, 255));
    break;

  case FF_TransferTree::USECOLOR_NORMAL:
  default:
    item->setUseOwnColors(false);
    break;
  }

  appendItem(itemFather, item, false);
}



FF_TransferTree::FF_TransferTree(): FF_GroupTree() {
}



void FF_TransferTree::create() {
  FF_GroupTree::create();
}



AE_STATEMENT *FF_TransferTree::getCurrentTransfer() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TransferTreeItem *stItem;

      stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
      if (stItem)
        return stItem->getTransfer();
    }
  }

  return NULL;
}



AE_STATEMENT *FF_TransferTree::getFirstSelectedTransfer() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_TransferTreeItem *stItem;

        stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
        if (stItem)
          return stItem->getTransfer();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_TransferTree::addTransfers(AE_STATEMENT_LIST2 *sl, int copyMode, bool direct) {
  AE_STATEMENT_LIST2_ITERATOR *it;

  it=AE_Statement_List2_First(sl);
  if (it) {
    AE_STATEMENT *st;

    st=AE_Statement_List2Iterator_Data(it);
    switch(copyMode) {
    case COPYMODE_COPY:
      while(st) {
        addTransfer(AE_Statement_dup(st), direct, USECOLOR_NORMAL);
        st=AE_Statement_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_ATTACH:
      while(st) {
        AE_Statement_Attach(st);
        addTransfer(st, direct, USECOLOR_NORMAL);
        st=AE_Statement_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_NONE:
    default:
      while(st) {
        addTransfer(st, direct, USECOLOR_NORMAL);
        st=AE_Statement_List2Iterator_Next(it);
      }
      break;
    }
    AE_Statement_List2Iterator_free(it);
  }
}



void FF_TransferTree::getSelectedTransfers(AE_STATEMENT_LIST2 *sl, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_TransferTreeItem *stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
          if (stItem)
            AE_Statement_List2_PushBack(sl, AE_Statement_dup(stItem->getTransfer()));
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_TransferTreeItem *stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
	  if (stItem) {
	    AE_STATEMENT *st=stItem->getTransfer();
            AE_Statement_Attach(st);
            AE_Statement_List2_PushBack(sl, st);
          }
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_NONE:
  default:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_TransferTreeItem *stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
          if (stItem)
            AE_Statement_List2_PushBack(sl, stItem->getTransfer());
        }
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_TransferTree::selectTransfer(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TransferTreeItem *stItem;

      stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
      if (stItem) {
        AE_STATEMENT *st;

        st=stItem->getTransfer();
        if (AE_Statement_GetId(st)==id) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_TransferTree::selectTransfer(const AE_STATEMENT *fst, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TransferTreeItem *stItem;

      stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
      if (stItem) {
        if (stItem->getTransfer()==fst) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_TransferTree::makeTransferVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_TransferTreeItem *stItem;

      stItem=dynamic_cast<FF_TransferTreeItem*>(grItem);
      if (stItem) {
        AE_STATEMENT *st;

        st=stItem->getTransfer();
        if (AE_Statement_GetId(st)==id) {
          makeItemVisible(stItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



FXint FF_TransferTree::sortFunctionUpDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const GWEN_DATE *t1=AE_Statement_GetDate(st1);
      const GWEN_DATE *t2=AE_Statement_GetDate(st2);

      /* both are data items */
      if (t1 && t2) {
        int d=GWEN_Date_Diff(t1, t2);
	if (d<0)
          return -1;
        else if (d>0)
          return 1;
      }
      else if (t1)
        return 1;
      else if (t2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpDate(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpRemoteName(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const char *s1=AE_Statement_GetRemoteName(st1);
      const char *s2=AE_Statement_GetRemoteName(st2);

      /* both are data items */
      if (s1 && s2)
        return strcasecmp(s1, s2);
      else if (s1)
        return 1;
      else if (s2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownRemoteName(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUpRemoteName(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpPurpose(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const char *s1=AE_Statement_GetPurpose(st1);
      const char *s2=AE_Statement_GetPurpose(st2);

      /* both are data items */
      if (s1 && s2)
        return strcasecmp(s1, s2);
      else if (s1)
        return 1;
      else if (s2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownPurpose(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUpPurpose(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpInfoText(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const char *s1=AE_Statement_GetInfoText(st1);
      const char *s2=AE_Statement_GetInfoText(st2);

      /* both are data items */
      if (s1 && s2)
        return strcasecmp(s1, s2);
      else if (s1)
        return 1;
      else if (s2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownInfoText(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUpInfoText(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpAmount(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const AQDB_VALUE *v1=AE_Statement_GetValue(st1);
      const AQDB_VALUE *v2=AE_Statement_GetValue(st2);

      /* both are data items */
      if (v1 && v2)
        return AQDB_Value_Compare(v1, v2);
      else if (v1)
        return 1;
      else if (v2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownAmount(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUpAmount(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpStatus(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      int s1=AE_Statement_GetStatus(st1);
      int s2=AE_Statement_GetStatus(st2);

      if (s1<s2)
        return -1;
      else if (s1>s2)
        return 1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownStatus(const FXFoldingItem *it1, const FXFoldingItem *it2){
  FXint rv;

  rv=sortFunctionUpStatus(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpBeginDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const GWEN_DATE *t1=NULL;
      const GWEN_DATE *t2=NULL;

      if (st1) {
	const AE_RECURRENCE *rec1=AE_Statement_GetRecurrence(st1);
	if (rec1)
	  t1=AE_Recurrence_GetBeginDate(rec1);
      }
      if (st2) {
	const AE_RECURRENCE *rec2=AE_Statement_GetRecurrence(st2);
	if (rec2)
	  t2=AE_Recurrence_GetBeginDate(rec2);
      }

      /* both are data items */
      if (t1 && t2) {
        int d=GWEN_Date_Diff(t1, t2);
	if (d<0)
          return -1;
        else if (d>0)
          return 1;
      }
      else if (t1)
        return 1;
      else if (t2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownBeginDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpBeginDate(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpEndDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const GWEN_DATE *t1=NULL;
      const GWEN_DATE *t2=NULL;

      if (st1) {
	const AE_RECURRENCE *rec1=AE_Statement_GetRecurrence(st1);
	if (rec1)
	  t1=AE_Recurrence_GetEndDate(rec1);
      }
      if (st2) {
	const AE_RECURRENCE *rec2=AE_Statement_GetRecurrence(st2);
	if (rec2)
	  t2=AE_Recurrence_GetEndDate(rec2);
      }

      /* both are data items */
      if (t1 && t2) {
        int d=GWEN_Date_Diff(t1, t2);
	if (d<0)
          return -1;
        else if (d>0)
          return 1;
      }
      else if (t1)
        return 1;
      else if (t2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownEndDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpEndDate(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



FXint FF_TransferTree::sortFunctionUpLastDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  const FF_GroupTreeItem *grItem1=dynamic_cast<const FF_GroupTreeItem*>(it1);
  const FF_GroupTreeItem *grItem2=dynamic_cast<const FF_GroupTreeItem*>(it2);

  if (grItem1 && grItem2) {
    if (!grItem1->isDataItem() && !grItem2->isDataItem())
      return FF_GroupTree::sortFunctionUp(it1, it2);
    else if (!grItem1->isDataItem())
      return 1;
    else if (!grItem2->isDataItem())
      return -1;
    else {
      const FF_TransferTreeItem *tti1=dynamic_cast<const FF_TransferTreeItem*>(it1);
      const FF_TransferTreeItem *tti2=dynamic_cast<const FF_TransferTreeItem*>(it2);
      const AE_STATEMENT *st1=tti1->getTransfer();
      const AE_STATEMENT *st2=tti2->getTransfer();
      const GWEN_DATE *t1=NULL;
      const GWEN_DATE *t2=NULL;

      if (st1) {
	const AE_RECURRENCE *rec1=AE_Statement_GetRecurrence(st1);
	if (rec1)
	  t1=AE_Recurrence_GetLastExecuted(rec1);
      }
      if (st2) {
	const AE_RECURRENCE *rec2=AE_Statement_GetRecurrence(st2);
	if (rec2)
	  t2=AE_Recurrence_GetLastExecuted(rec2);
      }

      /* both are data items */
      if (t1 && t2) {
        int d=GWEN_Date_Diff(t1, t2);
	if (d<0)
          return -1;
        else if (d>0)
          return 1;
      }
      else if (t1)
        return 1;
      else if (t2)
        return -1;
    }
  }

  return 0;
}



FXint FF_TransferTree::sortFunctionDownLastDate(const FXFoldingItem *it1, const FXFoldingItem *it2) {
  FXint rv;

  rv=sortFunctionUpLastDate(it1, it2);
  if (rv<0)
    return 1;
  else if (rv>0)
    return -1;
  return 0;
}



void FF_TransferTree::sortByColumn(int c, bool up) {
  if (c<m_columns) {
    switch(m_columnTypes[c]) {
    case COLUMN_DATE:
      setSortFunc(up?sortFunctionUpDate:sortFunctionDownDate);
      sortItems();
      break;
    case COLUMN_REMOTE_NAME:
      setSortFunc(up?sortFunctionUpRemoteName:sortFunctionDownRemoteName);
      sortItems();
      break;
    case COLUMN_PURPOSE:
      setSortFunc(up?sortFunctionUpPurpose:sortFunctionDownPurpose);
      sortItems();
      break;
    case COLUMN_TYPE:
      setSortFunc(up?sortFunctionUpInfoText:sortFunctionDownInfoText);
      sortItems();
      break;
    case COLUMN_AMOUNT:
      setSortFunc(up?sortFunctionUpAmount:sortFunctionDownAmount);
      sortItems();
      break;
    case COLUMN_STATUS:
      setSortFunc(up?sortFunctionUpStatus:sortFunctionDownStatus);
      sortItems();
      break;
    case COLUMN_DATE_BEGIN:
      setSortFunc(up?sortFunctionUpBeginDate:sortFunctionDownBeginDate);
      sortItems();
      break;
    case COLUMN_DATE_END:
      setSortFunc(up?sortFunctionUpEndDate:sortFunctionDownEndDate);
      sortItems();
      break;
    case COLUMN_DATE_LAST:
      setSortFunc(up?sortFunctionUpLastDate:sortFunctionDownLastDate);
      sortItems();
      break;
    case COLUMN_PERIOD:

    default:
      DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
      break;
    }
  }
}






