/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif



#include "ff_vehicletree.hpp"

#include <aqfinance/engine/book/ae_utils.h>

#include <fx.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/gwentime.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)


#define SIDE_SPACING 4
#define ICON_SPACING 4



FF_VehicleTreeItem::FF_VehicleTreeItem(AE_VEHICLE *st, void* ptr)
:FF_GroupTreeItem(NULL, NULL, ptr)
,m_vehicle(st)
{
}



FF_VehicleTreeItem::~FF_VehicleTreeItem() {
  if (m_isDataItem && m_vehicle) {
    AE_Vehicle_free(m_vehicle);
    m_vehicle=(AE_VEHICLE*) -1;
  }
}



FXString FF_VehicleTreeItem::getItemText(const FXXCheckTree* list) const {
  const FF_VehicleTree *tree;
  FXString t;

  tree=dynamic_cast<const FF_VehicleTree*>(list);
  if (tree) {
    int i;

    for (i=0; i<tree->m_columns; i++) {
      const char *s;

      switch(tree->m_columnTypes[i]) {

      case FF_VehicleTree::COLUMN_NAME:
        s=AE_Vehicle_GetName(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_MAKE:
        s=AE_Vehicle_GetMake(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_MODEL:
        s=AE_Vehicle_GetModel(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_BUILDYEAR:
	{
	  char numbuf[32];

	  snprintf(numbuf, sizeof(numbuf)-1, I18N("%d"), AE_Vehicle_GetBuildYear(m_vehicle));
	  numbuf[sizeof(numbuf)-1]=0;
	  t+=numbuf;
	}

      case FF_VehicleTree::COLUMN_BASE:
        s=AE_Vehicle_GetBase(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_LICENSEPLATE:
        s=AE_Vehicle_GetLicensePlate(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_VIN:
        s=AE_Vehicle_GetIdentificationNumber(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_MEMO:
        s=AE_Vehicle_GetMemo(m_vehicle);
        if (s)
          t+=FXString(s);
        break;

      case FF_VehicleTree::COLUMN_FUELCAPACITY:
        {
          const AQDB_VALUE *v;
          int units;

          units=AE_Vehicle_GetFuelCapacityUnits(m_vehicle);
          v=AE_Vehicle_GetFuelCapacity(m_vehicle);
          if (v) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);

            t+="<right>";

            AE_Utils_ValueAndUnitsToString(v, units, tbuf, 2);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
	    t+="</right>";
          }
        }
        break;

      case FF_VehicleTree::COLUMN_FUELTYPE:
        {
          int ft;

          ft=AE_Vehicle_GetFuelType(m_vehicle);
          s=AE_Utils_FuelType_toString(ft);
          t+=FXString(s);
        }
        break;

      case FF_VehicleTree::COLUMN_BUYDATE:
        {
          const GWEN_DATE *ti;

          ti=AE_Vehicle_GetBuyDate(m_vehicle);
          if (ti) {
            GWEN_BUFFER *tbuf=GWEN_Buffer_new(0, 256, 0, 1);
            GWEN_Date_toStringWithTemplate(ti, I18N("YYYY/MM/DD"), tbuf);
            t+=GWEN_Buffer_GetStart(tbuf);
            GWEN_Buffer_free(tbuf);
          }
        }
        break;

      default:
        break;
      }
      t+="\t";
    } /* for */
  }

  return t;
}









FF_VehicleTree::FF_VehicleTree(FXComposite *p,
                                 FXObject* tgt, FXSelector sel,
                                 FXuint opts,
                                 FXint x, FXint y, FXint w, FXint h)
:FF_GroupTree(p, tgt, sel, opts, x, y, w, h)
,m_columns(0)
{
}



void FF_VehicleTree::addColumn(int c, int w) {
  if (m_columns<COLUMN_MAX) {
    switch(c) {
    case COLUMN_NAME:
      appendHeader(I18N("AE_Vehicle|Name"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_MAKE:
      appendHeader(I18N("AE_Vehicle|Make"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_MODEL:
      appendHeader(I18N("AE_Vehicle|Model"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_BUILDYEAR:
      appendHeader(I18N("AE_Vehicle|Build Year"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_BASE:
      appendHeader(I18N("AE_Vehicle|Base"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_LICENSEPLATE:
      appendHeader(I18N("AE_Vehicle|License Plate"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_VIN:
      appendHeader(I18N("AE_Vehicle|VIN"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_MEMO:
      appendHeader(I18N("AE_Vehicle|Memo"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_FUELCAPACITY:
      appendHeader(I18N("AE_Vehicle|Fuel Capacity"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_FUELTYPE:
      appendHeader(I18N("AE_Vehicle|Fuel Type"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    case COLUMN_BUYDATE:
      appendHeader(I18N("AE_Vehicle|Buy Date"));
      setHeaderSize(m_columns, w);
      m_columnTypes[m_columns++]=c;
      break;
    default:
      break;
    }
  }
}



FF_VehicleTree::~FF_VehicleTree() {
}



FF_VehicleTreeItem *FF_VehicleTree::findVehicleItem(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem;

    grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem) {
      FF_VehicleTreeItem *res=findVehicleItem(grItem, id);
      if (res!=NULL)
        return res;
    }
    item=item->getNext();
  }

  return NULL;
}



FF_VehicleTreeItem *FF_VehicleTree::findVehicleItem(FF_GroupTreeItem *grItem,
						    AQDB_ID id) {
  if (grItem->isDataItem()) {
    FF_VehicleTreeItem *stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
    if (stItem) {
      AE_VEHICLE *st=stItem->getVehicle();
      if (AE_Vehicle_GetId(st)==id)
        return stItem;
    }
  }
  else {
    FXFoldingItem *item;

    item=grItem->getFirst();
    while(item) {
      grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem) {
        FF_VehicleTreeItem *res=findVehicleItem(grItem, id);
        if (res!=NULL)
          return res;
      }
      item=item->getNext();
    }
  }

  return NULL;
}



FF_VehicleTreeItem *FF_VehicleTree::findVehicleItem(const AE_VEHICLE *qba) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_VehicleTreeItem *baItem;

      baItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
      if (baItem) {
        AE_VEHICLE *sta;

	sta=baItem->getVehicle();
	if (sta==qba)
          return baItem;
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_VehicleTree::addVehicle(AE_VEHICLE *st, bool direct, int useColor) {
  FF_GroupTreeItem *itemFather=NULL;
  FF_VehicleTreeItem *item;

  if (!direct) {
    AQDB_ID gid=AE_Vehicle_GetGroupId(st);
    if (gid) {
      itemFather=findGroupItem(gid);
      if (itemFather==NULL) {
        DBG_INFO(AE_LOGDOMAIN, "Group for statement not found, reassigning to root");
      }
    }
  }

  item=new FF_VehicleTreeItem(st);
  /* maybe colorize */
  switch(useColor) {
  case FF_VehicleTree::USECOLOR_NOTICE:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(247, 248, 96));
    item->setTextColor(FXRGB(0, 0, 0));
    break;
  case FF_VehicleTree::USECOLOR_WARNING:
    item->setUseOwnColors(true);
    item->setBackColor(FXRGB(255, 127, 0));
    item->setTextColor(FXRGB(255, 255, 255));
    break;

  case FF_VehicleTree::USECOLOR_NORMAL:
  default:
    item->setUseOwnColors(false);
    break;
  }

  appendItem(itemFather, item, false);
}



FF_VehicleTree::FF_VehicleTree(): FF_GroupTree() {
}



void FF_VehicleTree::create() {
  FF_GroupTree::create();
}



AE_VEHICLE *FF_VehicleTree::getCurrentVehicle() {
  FXFoldingItem *item;

  item=getCurrentItem();
  if (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_VehicleTreeItem *stItem;

      stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
      if (stItem)
        return stItem->getVehicle();
    }
  }

  return NULL;
}



AE_VEHICLE *FF_VehicleTree::getFirstSelectedVehicle() {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    if (item->isSelected()) {
      FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
      if (grItem && (grItem->isDataItem())) {
        FF_VehicleTreeItem *stItem;

        stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
        if (stItem)
          return stItem->getVehicle();
      }
    }
    item=item->getBelow();
  }

  return NULL;
}



void FF_VehicleTree::addVehicles(AE_VEHICLE_LIST2 *sl, int copyMode, bool direct) {
  AE_VEHICLE_LIST2_ITERATOR *it;

  it=AE_Vehicle_List2_First(sl);
  if (it) {
    AE_VEHICLE *st;

    st=AE_Vehicle_List2Iterator_Data(it);
    switch(copyMode) {
    case COPYMODE_COPY:
      while(st) {
        addVehicle(AE_Vehicle_dup(st), direct, USECOLOR_NORMAL);
        st=AE_Vehicle_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_ATTACH:
      while(st) {
        AE_Vehicle_Attach(st);
        addVehicle(st, direct, USECOLOR_NORMAL);
        st=AE_Vehicle_List2Iterator_Next(it);
      }
      break;

    case COPYMODE_NONE:
    default:
      while(st) {
        addVehicle(st, direct, USECOLOR_NORMAL);
        st=AE_Vehicle_List2Iterator_Next(it);
      }
      break;
    }
    AE_Vehicle_List2Iterator_free(it);
  }
}



void FF_VehicleTree::getSelectedVehicles(AE_VEHICLE_LIST2 *sl, int copyMode) {
  FXFoldingItem *item;

  switch(copyMode) {
  case COPYMODE_COPY:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_VehicleTreeItem *stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
          if (stItem)
            AE_Vehicle_List2_PushBack(sl, AE_Vehicle_dup(stItem->getVehicle()));
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_ATTACH:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_VehicleTreeItem *stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
	  if (stItem) {
	    AE_VEHICLE *st=stItem->getVehicle();
            AE_Vehicle_Attach(st);
            AE_Vehicle_List2_PushBack(sl, st);
          }
        }
      }
      item=item->getBelow();
    }
    break;

  case COPYMODE_NONE:
  default:
    item=getFirstItem();
    while (item) {
      if (item->isSelected()) {
        FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
        if (grItem && (grItem->isDataItem())) {
          FF_VehicleTreeItem *stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
          if (stItem)
            AE_Vehicle_List2_PushBack(sl, stItem->getVehicle());
        }
      }
      item=item->getBelow();
    }
    break;
  }
}



void FF_VehicleTree::selectVehicle(AQDB_ID id, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_VehicleTreeItem *stItem;

      stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
      if (stItem) {
        AE_VEHICLE *st;

        st=stItem->getVehicle();
        if (AE_Vehicle_GetId(st)==id) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_VehicleTree::selectVehicle(const AE_VEHICLE *fst, bool b) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_VehicleTreeItem *stItem;

      stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
      if (stItem) {
        if (stItem->getVehicle()==fst) {
          stItem->setSelected(b);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}



void FF_VehicleTree::makeVehicleVisible(AQDB_ID id) {
  FXFoldingItem *item;

  item=getFirstItem();
  while (item) {
    FF_GroupTreeItem *grItem=dynamic_cast<FF_GroupTreeItem*>(item);
    if (grItem && (grItem->isDataItem())) {
      FF_VehicleTreeItem *stItem;

      stItem=dynamic_cast<FF_VehicleTreeItem*>(grItem);
      if (stItem) {
        AE_VEHICLE *st;

        st=stItem->getVehicle();
        if (AE_Vehicle_GetId(st)==id) {
          makeItemVisible(stItem);
          break;
        }
      }
    }
    item=item->getBelow();
  }
}




GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Name)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Name)

GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Make)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Make)

GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Model)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Model)

GROUPTREE_SORT_FN_UP_INT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, BuildYear)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, BuildYear)

GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Base)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Base)

GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, LicensePlate)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, LicensePlate)

GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, IdentificationNumber)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, IdentificationNumber)

GROUPTREE_SORT_FN_UP_TXT(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Memo)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, Memo)

GROUPTREE_SORT_FN_UP_VALUE(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, FuelCapacity)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, FuelCapacity)

GROUPTREE_SORT_FN_UP_DATE(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, BuyDate)
GROUPTREE_SORT_FN_DOWN(FF_VehicleTree, FF_VehicleTreeItem, getVehicle, AE_VEHICLE, AE_Vehicle, BuyDate)






void FF_VehicleTree::sortByColumn(int c, bool up) {
  if (c<m_columns) {
    switch(m_columnTypes[c]) {
    case COLUMN_NAME:
      setSortFunc(up?sortFunctionUpName:sortFunctionDownName);
      sortItems();
      break;

    default:
      DBG_ERROR(0, "Sort by column %d (%s) not implemented", c, up?"up":"down");
      break;
    }
  }
}



