/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_p.h"

#include <gwenhywfar/gwenhywfar.h>
#include <gwenhywfar/pathmanager.h>
#include <gwenhywfar/plugin.h>
#include <gwenhywfar/misc.h>
#include <gwenhywfar/i18n.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/directory.h>


#ifdef OS_WIN32
# define DIRSEP "\\"
#else
# define DIRSEP "/"
#endif


#define AQFINANCE_USERDATADIR ".aqfinance"



static AE_SETTINGS *ae_settings=NULL;



static AE_SETTINGS *createSettings(const char *dname) {
  AE_SETTINGS *as;
  const char *s;
  GWEN_BUFFER *buf;
  GWEN_BUFFER *cbuf;
  GWEN_CONFIGMGR *cm;
  int rv;

  GWEN_NEW_OBJECT(AE_SETTINGS, as);

  buf=GWEN_Buffer_new(0, 256, 0, 1);
  if (dname)
    GWEN_Buffer_AppendString(buf, dname);
  else {
    char home[256];

    if (GWEN_Directory_GetHomeDirectory(home, sizeof(home))) {
      DBG_ERROR(0,
		"Could not determine home directory, aborting.");
      abort();
    }

    s=getenv("AQFINANCE_HOME");
    if (s && !*s)
      s=0;
    if (s)
      GWEN_Buffer_AppendString(buf, s);
    else {
      /* use default */
      GWEN_Buffer_AppendString(buf, home);
      GWEN_Buffer_AppendString(buf, GWEN_DIR_SEPARATOR_S AQFINANCE_USERDATADIR);
    }
  }

  /* now buf contains the data path, create it if necessary */
  rv=GWEN_Directory_GetPath(GWEN_Buffer_GetStart(buf), GWEN_PATH_FLAGS_CHECKROOT);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "Unable to acces/create folder [%s]. ",
              GWEN_Buffer_GetStart(buf));
    GWEN_Buffer_free(buf);
    return NULL;
  }

  /* make path absolute */
  cbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=GWEN_Directory_GetAbsoluteFolderPath(GWEN_Buffer_GetStart(buf), cbuf);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "Unable to get the absolute folder for [%s]. ",
              GWEN_Buffer_GetStart(buf));
    GWEN_Buffer_free(cbuf);
    GWEN_Buffer_free(buf);
    return NULL;
  }
  /* exchange buffers */
  GWEN_Buffer_free(buf);
  buf=cbuf;

  as->userDataFolder=strdup(GWEN_Buffer_GetStart(buf));
  GWEN_Buffer_AppendString(buf, GWEN_DIR_SEPARATOR_S "settings");

  cbuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(cbuf, "dir://");
  GWEN_Buffer_AppendString(cbuf, GWEN_Buffer_GetStart(buf));
  GWEN_Buffer_free(buf);

  cm=GWEN_ConfigMgr_Factory(GWEN_Buffer_GetStart(cbuf));
  if (cm==NULL) {
    DBG_ERROR(0,
	      "Could not create ConfigMgr[%s]. "
	      "Maybe the gwenhywfar plugins are not installed?",
	      GWEN_Buffer_GetStart(cbuf));
    GWEN_Buffer_free(cbuf);
    return NULL;
  }
  as->configManager=cm;

  GWEN_Buffer_free(cbuf);
  return as;
}



static void releaseSettings(AE_SETTINGS *as) {
  if (as) {
    GWEN_ConfigMgr_free(as->configManager);
    AF_PrintTableDescr_free(as->psInvoice);
    AF_PrintTableDescr_free(as->psRequest);
    AF_PrintTableDescr_free(as->psOffer);
    AF_PrintTableDescr_free(as->psConfirmation);
    AF_PrintTableDescr_free(as->psDelivery);
    AF_PrintTableDescr_free(as->psRelease);
    AF_PrintTableDescr_free(as->psReminder);
    AF_PrintTableDescr_free(as->psBuyOrder);
    AF_PrintTableDescr_free(as->psSellOrder);

    free(as->userDataFolder);

    GWEN_FREE_OBJECT(as);
  }
}



int AE_Init(const char *dname) {
  int rv;
  const char *s;

  rv=GWEN_Init();
  if (rv) {
    DBG_ERROR_ERR(AE_LOGDOMAIN, rv);
    return rv;
  }
  if (!GWEN_Logger_IsOpen(AE_LOGDOMAIN)) {
    GWEN_Logger_Open(AE_LOGDOMAIN,
		     "aqfinance", 0,
		     GWEN_LoggerType_Console,
		     GWEN_LoggerFacility_User);
  }

  s=getenv("AE_LOGLEVEL");
  if (s && *s) {
    GWEN_LOGGER_LEVEL ll;

    ll=GWEN_Logger_Name2Level(s);
    GWEN_Logger_SetLevel(AE_LOGDOMAIN, ll);
  }
  else
    GWEN_Logger_SetLevel(AE_LOGDOMAIN, GWEN_LoggerLevel_Notice);

  DBG_INFO(AE_LOGDOMAIN,
	   "AqFinance v"
	   AF_VERSION_FULL_STRING
	   " (compiled at "
	   COMPILE_DATETIME
	   "): initialising");

  ae_settings=createSettings(dname);
  if (ae_settings==NULL) {
    DBG_INFO(AE_LOGDOMAIN, "here");
    return GWEN_ERROR_INTERNAL;
  }

  /* define locale paths */
  GWEN_PathManager_DefinePath(AE_PM_LIBNAME, AE_PM_LOCALEDIR);
#if defined(OS_WIN32) || defined(ENABLE_LOCAL_INSTALL)
  /* add folder relative to EXE */
  GWEN_PathManager_AddRelPath(AE_PM_LIBNAME,
			      AE_PM_LIBNAME,
			      AE_PM_LOCALEDIR,
			      LOCALEDIR,
			      GWEN_PathManager_RelModeExe);
#else
  /* add absolute folder */
  GWEN_PathManager_AddPath(AE_PM_LIBNAME,
			   AE_PM_LIBNAME,
			   AE_PM_LOCALEDIR,
			   LOCALEDIR);
#endif

  /* define data paths */
  GWEN_PathManager_DefinePath(AE_PM_LIBNAME, AE_PM_DATADIR);
#if defined(OS_WIN32) || defined(ENABLE_LOCAL_INSTALL)
  /* add folder relative to EXE */
  GWEN_PathManager_AddRelPath(AE_PM_LIBNAME,
			      AE_PM_LIBNAME,
			      AE_PM_DATADIR,
			      AQFINANCE_DATA_DIR,
			      GWEN_PathManager_RelModeExe);
#else
  /* add absolute folder */
  GWEN_PathManager_AddPath(AE_PM_LIBNAME,
			   AE_PM_LIBNAME,
			   AE_PM_DATADIR,
			   AQFINANCE_DATA_DIR);
#endif

  if (1) {
    GWEN_STRINGLIST *sl=GWEN_PathManager_GetPaths(AE_PM_LIBNAME,
						  AE_PM_LOCALEDIR);
    if (sl) {
      const char *localedir=GWEN_StringList_FirstString(sl);

      rv=GWEN_I18N_BindTextDomain_Dir(PACKAGE, localedir);
      if (rv) {
	DBG_ERROR(AE_LOGDOMAIN, "Could not bind textdomain (%d)", rv);
      }
      else {
	rv=GWEN_I18N_BindTextDomain_Codeset(PACKAGE, "UTF-8");
	if (rv) {
	  DBG_ERROR(AE_LOGDOMAIN, "Could not set codeset (%d)", rv);
	}
      }

      GWEN_StringList_free(sl);
    }
  }

  return 0;
}



int AE_Fini() {
  /* undefine our own paths */
  GWEN_PathManager_UndefinePath(AE_PM_LIBNAME, AE_PM_LOCALEDIR);
  GWEN_PathManager_UndefinePath(AE_PM_LIBNAME, AE_PM_DATADIR);
  
  /* remove AqBanking additions to all pathmanagers */
  GWEN_PathManager_RemovePaths(AE_PM_LIBNAME);
  releaseSettings(ae_settings);
  GWEN_Logger_Close(AE_LOGDOMAIN);
  GWEN_Fini();
  return 0;
}



const char *AE_GetUserDataFolder() {
  if (ae_settings)
    return ae_settings->userDataFolder;
  return NULL;
}



GWEN_CONFIGMGR *AE_GetConfigManager() {
  if (ae_settings)
    return ae_settings->configManager;
  return NULL;
}



const AF_PRINT_TABLE_DESCR *AE_GetPsInvoice() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Invoice", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psInvoice);
    ae_settings->psInvoice=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsInvoice(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Invoice");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Invoice", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Invoice");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psInvoice);
	  ae_settings->psInvoice=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsRequest() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Request", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psRequest);
    ae_settings->psRequest=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsRequest(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Request");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Request", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Request");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psRequest);
	  ae_settings->psRequest=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsOffer() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Offer", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psOffer);
    ae_settings->psOffer=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsOffer(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Offer");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Offer", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Offer");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psOffer);
	  ae_settings->psOffer=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsConfirmation() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Confirmation", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psConfirmation);
    ae_settings->psConfirmation=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsConfirmation(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Confirmation");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Confirmation", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Confirmation");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psConfirmation);
	  ae_settings->psConfirmation=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsDelivery() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Delivery", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psDelivery);
    ae_settings->psDelivery=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsDelivery(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Delivery");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Delivery", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Delivery");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psDelivery);
	  ae_settings->psDelivery=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsRelease() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Release", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psRelease);
    ae_settings->psRelease=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsRelease(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Release");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Release", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Release");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psRelease);
	  ae_settings->psRelease=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsReminder() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "Reminder", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psReminder);
    ae_settings->psReminder=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsReminder(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "Reminder");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "Reminder", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "Reminder");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psReminder);
	  ae_settings->psReminder=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsBuyOrder() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "BuyOrder", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psBuyOrder);
    ae_settings->psBuyOrder=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsBuyOrder(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "BuyOrder");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "BuyOrder", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "BuyOrder");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psBuyOrder);
	  ae_settings->psBuyOrder=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



const AF_PRINT_TABLE_DESCR *AE_GetPsSellOrder() {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;

    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "jobPrintTemplates", "SellOrder", &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    ps=AF_PrintTableDescr_fromDb(db);
    AF_PrintTableDescr_free(ae_settings->psSellOrder);
    ae_settings->psSellOrder=ps;
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_SetPsSellOrder(const AF_PRINT_TABLE_DESCR *ps) {
  if (ae_settings) {
    int rv;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "jobPrintTemplates", "SellOrder");
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "jobPrintTemplates", "SellOrder", db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "jobPrintTemplates", "SellOrder");
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	  AF_PrintTableDescr_free(ae_settings->psSellOrder);
	  ae_settings->psSellOrder=AF_PrintTableDescr_dup(ps);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



AF_PRINT_TABLE_DESCR *AE_ReadPsInvoiceForPartner(AQDB_ID id) {
  if (ae_settings) {
    int rv;
    GWEN_DB_NODE *db;
    AF_PRINT_TABLE_DESCR *ps;
    char nbuf[128];

    snprintf(nbuf, sizeof(nbuf)-1, "%08x-print-invoice", id);
    nbuf[sizeof(nbuf)-1]=0;
    rv=GWEN_ConfigMgr_GetGroup(ae_settings->configManager, "partnerSettings", nbuf, &db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return NULL;
    }
    if (GWEN_DB_Groups_Count(db)==0 && GWEN_DB_Variables_Count(db)==0) {
      const AF_PRINT_TABLE_DESCR *cps;

      cps=AE_GetPsInvoice();
      if (cps) {
	ps=AF_PrintTableDescr_dup(cps);
	GWEN_DB_Group_free(db);
	return ps;
      }
      else {
	DBG_ERROR(AE_LOGDOMAIN, "No template, will return empty one");
      }
    }

    ps=AF_PrintTableDescr_fromDb(db);
    GWEN_DB_Group_free(db);
    return ps;
  }
  return NULL;
}



void AE_WritePsInvoiceForPartner(const AF_PRINT_TABLE_DESCR *ps, AQDB_ID id) {
  if (ae_settings) {
    int rv;
    char nbuf[128];

    snprintf(nbuf, sizeof(nbuf)-1, "%08x-print-invoice", id);
    nbuf[sizeof(nbuf)-1]=0;

    rv=GWEN_ConfigMgr_LockGroup(ae_settings->configManager, "partnerSettings", nbuf);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    else {
      GWEN_DB_NODE *db;

      db=GWEN_DB_Group_new("jobPrintTemplate");
      AF_PrintTableDescr_toDb(ps, db);
      rv=GWEN_ConfigMgr_SetGroup(ae_settings->configManager, "partnerSettings", nbuf, db);
      if (rv<0) {
	DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
      }
      else {
	rv=GWEN_ConfigMgr_UnlockGroup(ae_settings->configManager, "partnerSettings", nbuf);
	if (rv<0) {
	  DBG_ERROR(0, "Error getting main configuration group (%d)", rv);
	}
      }
      GWEN_DB_Group_free(db);
    }
  }
}



int AE_Units_fromString(const char *s) {
  if (s && *s) {
    if (strcasecmp(s, "none")==0)
      return AE_Units_None;
    else if (strcasecmp(s, "liters")==0)
      return AE_Units_Liters;
    else if (strcasecmp(s, "grammes")==0)
      return AE_Units_Grammes;
    else if (strcasecmp(s, "meters")==0)
      return AE_Units_Meters;
    else if (strcasecmp(s, "squareMeters")==0)
      return AE_Units_SquareMeters;
    else if (strcasecmp(s, "cubicMeters")==0)
      return AE_Units_CubicMeters;
    else if (strcasecmp(s, "minutes")==0)
      return AE_Units_Minutes;
    else if (strcasecmp(s, "hours")==0)
      return AE_Units_Hours;
  }
  return AE_Units_Unknown;
}



const char *AE_Units_toString(int i) {
  switch(i) {
    case AE_Units_None:         return "none";
    case AE_Units_Liters:       return "liters";
    case AE_Units_Grammes:      return "grammes";
    case AE_Units_Meters:       return "meters";
    case AE_Units_SquareMeters: return "squareMeters";
    case AE_Units_CubicMeters:  return "cubicMeters";
    case AE_Units_Minutes:      return "minutes";
    case AE_Units_Hours:        return "hours";
    default:                    return "unknown";
  }
}





