/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#include <aqfinance/engine/book/ae_utils.h>

#include <ctype.h>




void AE_Account_Tree_ClearBranchBalances(AE_ACCOUNT_TREE *at) {
  AE_ACCOUNT *acc;
  AQDB_VALUE *v;

  v=AQDB_Value_new();

  acc=AE_Account_Tree_GetFirst(at);
  while(acc) {
    if (acc->branchBalance)
      AE_AccountBalance_SetValue(acc->branchBalance, v);
    acc=AE_Account_Tree_GetBelow(acc);
  }

  AQDB_Value_free(v);
}



void AE_Account_Tree_ClearBalances(AE_ACCOUNT_TREE *at) {
  AE_ACCOUNT *acc;
  AQDB_VALUE *v;

  v=AQDB_Value_new();

  acc=AE_Account_Tree_GetFirst(at);
  while(acc) {
    if (acc->balance==NULL) {
      acc->balance=AE_AccountBalance_new();
      AE_AccountBalance_SetCommodityId(acc->balance, acc->primaryCommodityId);
      AE_AccountBalance_SetAccountId(acc->balance, acc->id);
    }
    AE_AccountBalance_SetValue(acc->balance, v);
    acc=AE_Account_Tree_GetBelow(acc);
  }

  AQDB_Value_free(v);
}




void AE_Account_Tree_CalcBranchBalance(AE_ACCOUNT *acc) {
  AE_ACCOUNT *c;
  AQDB_VALUE *vDest;

  if (acc->branchBalance==NULL) {
    acc->branchBalance=AE_AccountBalance_new();
    AE_AccountBalance_SetCommodityId(acc->branchBalance, acc->primaryCommodityId);
  }
  vDest=AE_AccountBalance_GetValue(acc->branchBalance);

  /* add balance of this account */
  if (acc->balance) {
    AQDB_VALUE *vSrc;

    vSrc=AE_AccountBalance_GetValue(acc->balance);
    AQDB_Value_AddValue(vDest, vSrc);
  }

  /* let all children create their branch balance and add that */
  c=AE_Account_Tree_GetFirstChild(acc);
  while(c) {
    AE_Account_Tree_CalcBranchBalance(c);
    if (c->primaryCommodityId==acc->primaryCommodityId) {
      AQDB_VALUE *vSrc;

      vSrc=AE_AccountBalance_GetValue(c->branchBalance);
      AQDB_Value_AddValue(vDest, vSrc);
    }

    c=AE_Account_Tree_GetNext(c);
  }
}



void AE_Account_Tree_CalcBranchBalances(AE_ACCOUNT_TREE *at) {
  AE_ACCOUNT *acc;

  acc=AE_Account_Tree_GetFirst(at);
  while(acc) {
    AE_Account_Tree_CalcBranchBalance(acc);
    acc=AE_Account_Tree_GetNext(acc);
  }
}



int AE_Account_Tree_AddToBalance(AE_ACCOUNT_TREE *at,
				 AQDB_ID accountId,
				 const AQDB_VALUE *vt,
				 AQDB_ID commodityId) {
  AE_ACCOUNT *acc;

  acc=AE_Account_Tree_GetById(at, accountId);
  if (acc==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Account %08x not found", accountId);
    return GWEN_ERROR_NOT_FOUND;
  }
  else {
    AQDB_VALUE *v;

    // TODO: check for secondary commodity */
    if (acc->primaryCommodityId!=commodityId) {
      DBG_ERROR(AE_LOGDOMAIN, "Not the same commodity as in account");
      return GWEN_ERROR_INVALID;
    }

    /* add to balance */
    if (acc->balance==NULL) {
      acc->balance=AE_AccountBalance_new();
      AE_AccountBalance_SetCommodityId(acc->balance, commodityId);
      AE_AccountBalance_SetAccountId(acc->balance, acc->id);
    }
    v=AE_AccountBalance_GetValue(acc->balance);
    AQDB_Value_AddValue(v, vt);

    AE_AccountBalance_AddRuntimeFlags(acc->balance, AE_ACCOUNTBALANCE_RTFLAGS_MODIFIED);

    /* add to branch balance of this account and its parents */
    while(acc) {
      if (acc->branchBalance==NULL) {
	acc->branchBalance=AE_AccountBalance_new();
	AE_AccountBalance_SetCommodityId(acc->branchBalance, commodityId);
	AE_AccountBalance_SetAccountId(acc->branchBalance, acc->id);
      }
      v=AE_AccountBalance_GetValue(acc->branchBalance);
      AQDB_Value_AddValue(v, vt);
      AE_AccountBalance_AddRuntimeFlags(acc->branchBalance, AE_ACCOUNTBALANCE_RTFLAGS_MODIFIED);

      acc=AE_Account_Tree_GetParent(acc);
    }
  }

  return 0;
}



int AE_Account_Tree_SubFromBalance(AE_ACCOUNT_TREE *at,
				   AQDB_ID accountId,
				   const AQDB_VALUE *vt,
				   AQDB_ID commodityId) {
  AQDB_VALUE *v;
  int rv;

  v=AQDB_Value_dup(vt);
  AQDB_Value_Negate(v);
  rv=AE_Account_Tree_AddToBalance(at, accountId, v, commodityId);
  AQDB_Value_free(v);

  return rv;
}



int AE_AccountType_NeedReverseSign(AE_ACCOUNT_TYPE t) {
  return AE_AccountGroup_NeedReverseSign(AE_AccountType_GetGroup(t));
}



int AE_AccountGroup_NeedReverseSign(AE_ACCOUNT_GROUP g) {
  switch(g) {
  case AE_AccountGroup_Unknown:   return 0;
  case AE_AccountGroup_Asset:     return 0;
  case AE_AccountGroup_Liability: return 1;
  case AE_AccountGroup_Equity:    return 0;
  case AE_AccountGroup_Income:    return 1;
  case AE_AccountGroup_Expense:   return 0;
  }

  /* SNH */
  return 0;
}



void AE_Account_Tree_SetSelected(AE_ACCOUNT *acc) {
  AE_Account_AddRuntimeFlags(acc, AE_ACCOUNT_RTFLAGS_SELECTED);
  while( (acc=AE_Account_Tree_GetParent(acc)) ) {
    AE_Account_AddRuntimeFlags(acc, AE_ACCOUNT_RTFLAGS_SELECTED);
  }
}



void AE_Account_Tree_AddRuntimeFlagsUpAndDown(AE_ACCOUNT *acc, int withParents, int withChildren, uint32_t rtflags) {
  AE_Account_AddRuntimeFlags(acc, rtflags);
  if (withParents) {
    AE_ACCOUNT *p;

    p=acc;
    while( (p=AE_Account_Tree_GetParent(p)) ) {
      AE_Account_AddRuntimeFlags(p, rtflags);
    }
  }

  if (withChildren) {
    AE_ACCOUNT *c;

    c=AE_Account_Tree_GetFirstChild(acc);
    while(c) {
      /* set selected with children, don't select parents (we already did that) */
      AE_Account_Tree_AddRuntimeFlagsUpAndDown(c, 0, 1, rtflags);
      c=AE_Account_Tree_GetNext(c);
    }
  }
}



void AE_Account_Tree_SubRuntimeFlagsUpAndDown(AE_ACCOUNT *acc, int withParents, int withChildren, uint32_t rtflags) {
  AE_Account_SubRuntimeFlags(acc, rtflags);
  if (withParents) {
    AE_ACCOUNT *p;

    p=acc;
    while( (p=AE_Account_Tree_GetParent(p)) ) {
      AE_Account_SubRuntimeFlags(p, rtflags);
    }
  }

  if (withChildren) {
    AE_ACCOUNT *c;

    c=AE_Account_Tree_GetFirstChild(acc);
    while(c) {
      /* set selected with children, don't select parents (we already did that) */
      AE_Account_Tree_SubRuntimeFlagsUpAndDown(c, 0, 1, rtflags);
      c=AE_Account_Tree_GetNext(c);
    }
  }
}



void AE_Account_Tree_AddRuntimeFlags(AE_ACCOUNT_TREE *at, uint32_t rtflags) {
  AE_ACCOUNT *a;

  a=AE_Account_Tree_GetFirst(at);
  while(a) {
    AE_Account_Tree_AddRuntimeFlagsUpAndDown(a, 0, 1, rtflags);
    a=AE_Account_Tree_GetNext(a);
  }
}



void AE_Account_Tree_SubRuntimeFlags(AE_ACCOUNT_TREE *at, uint32_t rtflags) {
  AE_ACCOUNT *a;

  a=AE_Account_Tree_GetFirst(at);
  while(a) {
    AE_Account_Tree_SubRuntimeFlagsUpAndDown(a, 0, 1, rtflags);
    a=AE_Account_Tree_GetNext(a);
  }
}



void AE_Account_Tree_SelectMatching(AE_ACCOUNT_TREE *at, const char *s) {
  AE_ACCOUNT *acc;

  if (s && *s) {
    /* clear the SELECTED flags */
    acc=AE_Account_Tree_GetFirst(at);
    while(acc) {
      AE_Account_SubRuntimeFlags(acc, AE_ACCOUNT_RTFLAGS_SELECTED);
      acc=AE_Account_Tree_GetBelow(acc);
    }

    /* select matching accounts */
    acc=AE_Account_Tree_GetFirst(at);
    while(acc) {
      const char *t;
      int match=0;

      t=AE_Account_GetName(acc);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL)
	match=1;
      if (match==0) {
	t=AE_Account_GetCode(acc);
	if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL)
	  match=1;
      }
      if (match==0) {
	t=AE_Account_GetDescription(acc);
	if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL)
	  match=1;
      }
      if (match)
	AE_Account_Tree_SetSelected(acc);
      acc=AE_Account_Tree_GetBelow(acc);
    }
  }
  else {
    /* set the SELECTED flags */
    acc=AE_Account_Tree_GetFirst(at);
    while(acc) {
      AE_Account_AddRuntimeFlags(acc, AE_ACCOUNT_RTFLAGS_SELECTED);
      acc=AE_Account_Tree_GetBelow(acc);
    }
  }
}



int AE_Account_Tree_IsAncestorOf(const AE_ACCOUNT *parentInQuestion, const AE_ACCOUNT *acc) {
  const AE_ACCOUNT *pa;

  pa=AE_Account_Tree_GetParent(acc);
  while(pa) {
    if (pa==parentInQuestion)
      return 1;
    pa=AE_Account_Tree_GetParent(pa);
  }

  return 0;
}



void AE_Account_Tree_SelectAccountAndChildrenByName(AE_ACCOUNT_TREE *at, const char *path, int withChildren) {
  if (path) {
    AE_ACCOUNT *a;

    a=AE_Account_Tree_GetByPath(at, NULL, path);
    if (a)
      AE_Account_Tree_AddRuntimeFlagsUpAndDown(a, 0, 1, AE_ACCOUNT_RTFLAGS_SELECTED);
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Account [%s] not found", path);
    }
  }
}



