/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_alias_p.h"
#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>


GWEN_LIST_FUNCTIONS(AE_ALIAS, AE_Alias)




AE_ALIAS *AE_Alias_new() {
  AE_ALIAS *al;

  GWEN_NEW_OBJECT(AE_ALIAS, al);
  al->refCount=1;
  GWEN_LIST_INIT(AE_ALIAS, al);

  return al;
}



AE_ALIAS *AE_Alias_dup(const AE_ALIAS *osp) {
  AE_ALIAS *al;

  assert(osp);
  assert(osp->refCount);

  al=AE_Alias_new();
  al->id=osp->id;
  if (osp->name)
    al->name=strdup(osp->name);
  al->dbId=osp->dbId;

  return al;
}



void AE_Alias_Attach(AE_ALIAS *al) {
  assert(al);
  assert(al->refCount);
  al->refCount++;
}



void AE_Alias_free(AE_ALIAS *al) {
  if (al) {
    if (al->refCount==1) {
      GWEN_LIST_FINI(AE_ALIAS, al);
      free(al->name);
      al->refCount=0;
      GWEN_FREE_OBJECT(al);
    }
    else {
      al->refCount--;
    }
  }
}



AQDB_ID AE_Alias_GetId(const AE_ALIAS *al) {
  assert(al);
  assert(al->refCount);
  return al->id;
}



void AE_Alias_SetId(AE_ALIAS *al, AQDB_ID id) {
  assert(al);
  assert(al->refCount);
  al->id=id;
}



const char *AE_Alias_GetNameSpace(const AE_ALIAS *al) {
  assert(al);
  assert(al->refCount);
  return al->nameSpace;
}



void AE_Alias_SetNameSpace(AE_ALIAS *al, const char *s) {
  assert(al);
  assert(al->refCount);
  free(al->nameSpace);
  if (s) al->nameSpace=strdup(s);
  else al->nameSpace=NULL;
}



const char *AE_Alias_GetName(const AE_ALIAS *al) {
  assert(al);
  assert(al->refCount);
  return al->name;
}



void AE_Alias_SetName(AE_ALIAS *al, const char *s) {
  assert(al);
  assert(al->refCount);
  free(al->name);
  if (s) al->name=strdup(s);
  else al->name=NULL;
}



AQDB_ID AE_Alias_GetMainId(const AE_ALIAS *al) {
  assert(al);
  assert(al->refCount);
  return al->mainId;
}



void AE_Alias_SetMainId(AE_ALIAS *al, AQDB_ID id) {
  assert(al);
  assert(al->refCount);
  al->mainId=id;
}



AQDB_ID AE_Alias_GetDbId(const AE_ALIAS *al) {
  assert(al);
  assert(al->refCount);
  return al->dbId;
}



void AE_Alias_SetDbId(AE_ALIAS *al, AQDB_ID id) {
  assert(al);
  assert(al->refCount);
  al->dbId=id;
}



int AE_Alias_CreateTable(AQDB_DB *db, const char *tname, uint32_t flags) {
  AQDB_COLUMN_LIST *cl;
  AQDB_COLUMN *c;

  cl=AQDB_Column_List_new();

  c=AQDB_Column_new(AQDB_DataType_Int, "id", 8);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_String, "namespace", 64);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_String, "name", 64);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_Int, "mainid", 8);
  AQDB_Column_List_Add(c, cl);

  return 0;
}



int AE_Alias_fromObject(const AQDB_OBJECT *o, AE_ALIAS **pT) {
  int rv;
  int64_t v64;
  const char *s;
  AE_ALIAS *al;

  al=AE_Alias_new();
  al->dbId=AQDB_Object_GetObjectId(o);

  rv=AQDB_Object_GetFieldInt64(o, AE_ALIAS_FIELD_ID, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Alias_free(al);
    return rv;
  }
  al->id=(AQDB_ID) v64;

  rv=AQDB_Object_GetFieldString(o, AE_ALIAS_FIELD_NAMESPACE, &s);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Alias_free(al);
    return rv;
  }
  if (s && *s)
    al->nameSpace=strdup(s);

  rv=AQDB_Object_GetFieldString(o, AE_ALIAS_FIELD_NAME, &s);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Alias_free(al);
    return rv;
  }
  if (s && *s)
    al->name=strdup(s);

  rv=AQDB_Object_GetFieldInt64(o, AE_ALIAS_FIELD_MAINID, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Alias_free(al);
    return rv;
  }
  al->mainId=(AQDB_ID) v64;

  *pT=al;
  return 0;
}



int AE_Alias_toObject(const AE_ALIAS *al, AQDB_OBJECT *o) {
  int64_t v64;
  int rv;

  assert(al);
  assert(al->refCount);

  v64=(int64_t) al->id;
  rv=AQDB_Object_SetFieldInt64(o, AE_ALIAS_FIELD_ID, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AQDB_Object_SetFieldString(o, AE_ALIAS_FIELD_NAMESPACE, al->nameSpace);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AQDB_Object_SetFieldString(o, AE_ALIAS_FIELD_NAME, al->name);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  v64=(int64_t) al->mainId;
  rv=AQDB_Object_SetFieldInt64(o, AE_ALIAS_FIELD_MAINID, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_Alias_CreateObject(AQDB_DB *db, AQDB_ID tableId, AQDB_ID id, AQDB_OBJECT **pObj) {
  AQDB_OBJECT *o;

  o=AQDB_Object_new(tableId, id, AE_ALIAS_FIELD_COUNT);
  *pObj=o;
  return 0;
}











