/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_book_p.h"

#include "ae_commodity.h"
#include "ae_account.h"
#include "ae_statement.h"
#include "ae_idcounter.h"
#include "ae_bankaccount.h"
#include "ae_transaction.h"
#include "ae_split.h"
#include "ae_group.h"
#include "ae_bankbalance.h"
#include "ae_address.h"
#include "ae_partner.h"
#include "ae_recurrence.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>


GWEN_INHERIT_FUNCTIONS(AE_BOOK)



static const char *tableNames[]={
  NULL,
  AE_BOOK_TABLENAME_IDCOUNTER,
  AE_BOOK_TABLENAME_COMMODITY,
  AE_BOOK_TABLENAME_ACCOUNT,
  AE_BOOK_TABLENAME_TRANSACTION,
  AE_BOOK_TABLENAME_SPLIT,
  AE_BOOK_TABLENAME_BANKACCOUNT,
  AE_BOOK_TABLENAME_ADDRESS,
  AE_BOOK_TABLENAME_PARTNER,

  AE_BOOK_TABLENAME_PROJECT,
  AE_BOOK_TABLENAME_TASK,
  AE_BOOK_TABLENAME_JOB,

  AE_BOOK_TABLENAME_WAREHOUSE,
  AE_BOOK_TABLENAME_ITEM,
  AE_BOOK_TABLENAME_BANKSTATEMENT,
  AE_BOOK_TABLENAME_BANKTRANSFER,
  AE_BOOK_TABLENAME_BANKDEBITNOTE,

  AE_BOOK_TABLENAME_BANKACCOUNTGROUP,

  AE_BOOK_TABLENAME_BANKTRANSFERPLANNED,
  AE_BOOK_TABLENAME_BANKTRANSFERTEMPLATE,

  AE_BOOK_TABLENAME_BANKTRANSFERGROUP,
  AE_BOOK_TABLENAME_BOOKEDBALANCE,
  AE_BOOK_TABLENAME_NOTEDBALANCE,

  AE_BOOK_TABLENAME_RULE,

  AE_BOOK_TABLENAME_PARTNERGROUP,
  AE_BOOK_TABLENAME_PARTNERRULE,

  AE_BOOK_TABLENAME_TAX,
  AE_BOOK_TABLENAME_TAXGROUP,

  AE_BOOK_TABLENAME_ITEMTEMPLATES,
  AE_BOOK_TABLENAME_ITEMTEMPLATEGROUP,

  AE_BOOK_TABLENAME_ACCOUNTBALANCE,

  AE_BOOK_TABLENAME_BANKSTANDINGORDER,
  AE_BOOK_TABLENAME_BANKSTORECURRENCE,

  AE_BOOK_TABLENAME_VEHICLE,
  AE_BOOK_TABLENAME_VEHICLEGROUP,

  AE_BOOK_TABLENAME_REFUELING,

  AE_BOOK_TABLENAME_GRAPHICREPORTGROUP,
  AE_BOOK_TABLENAME_GRAPHICREPORTSETTINGS,

  AE_BOOK_TABLENAME_SHARE,

  NULL
};



AE_BOOK *AE_Book_new(AQDB_DB *db) {
  AE_BOOK *bk;

  GWEN_NEW_OBJECT(AE_BOOK, bk);
  GWEN_INHERIT_INIT(AE_BOOK, bk);
  bk->refCount=1;

  bk->dataBase=db;
  AQDB_DB_Attach(bk->dataBase);

  return bk;
}



void AE_Book_free(AE_BOOK *bk) {
  if(bk) {
    assert(bk->refCount);
    if (bk->refCount==1) {
      GWEN_INHERIT_FINI(AE_BOOK, bk);
      if (bk->dataBase) {
	if (bk->dbOpened) {
	  int i;

	  for (i=0; i<AE_Book_TableType_Last; i++) {
	    AQDB_ID id=bk->tableIds[i];

	    if (id!=0) {
	      AQDB_DB_CloseTable(bk->dataBase, id, 0);
	      bk->tableIds[i]=0;
	    }
	  }
	  AQDB_DB_Close(bk->dataBase, 0);
          bk->dbOpened=0;
	}
      }

      AQDB_DB_free(bk->dataBase);
      bk->dataBase=NULL;

      bk->refCount=0;
      GWEN_FREE_OBJECT(bk);
    }
    else {
      bk->refCount--;
    }
  }
}



void AE_Book_Attach(AE_BOOK *bk) {
  assert(bk);
  assert(bk->refCount);
  bk->refCount++;
}



int AE_Book_Open(AE_BOOK *bk, uint32_t flags) {
  int rv;
  int i;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened) {
    DBG_ERROR(AE_LOGDOMAIN, "Book already open");
    return GWEN_ERROR_OPEN;
  }

  for (i=0; i<AE_Book_TableType_Last; i++)
    bk->tableIds[i]=0;

  rv=AQDB_DB_Open(bk->dataBase, flags);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  bk->dbOpened=1;
  return 0;
}



int AE_Book_Create(AE_BOOK *bk, uint32_t flags) {
  int rv;
  int i;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened) {
    DBG_ERROR(AE_LOGDOMAIN, "Book already open");
    return GWEN_ERROR_OPEN;
  }

  for (i=0; i<AE_Book_TableType_Last; i++)
    bk->tableIds[i]=0;

  rv=AQDB_DB_Create(bk->dataBase, flags);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_Book_Close(AE_BOOK *bk, uint32_t flags) {
  int rv;
  int i;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  for (i=0; i<AE_Book_TableType_Last; i++) {
    AQDB_ID id=bk->tableIds[i];

    if (id!=0) {
      AQDB_DB_CloseTable(bk->dataBase, id, flags);
      bk->tableIds[i]=0;
    }
  }

  rv=AQDB_DB_Close(bk->dataBase, flags);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}




int AE_Book_CreateTable(AE_BOOK *bk,
			AE_BOOK_TABLE_TYPE tt,
			uint32_t flags) {
  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return GWEN_ERROR_INVALID;
  }

  switch(tt) {
  case AE_Book_TableType_Unknown:
  case AE_Book_TableType_None:
  case AE_Book_TableType_Last:
    DBG_ERROR(AE_LOGDOMAIN, "Invalid table type %d", tt);
    return GWEN_ERROR_INVALID;

  case AE_Book_TableType_IdCounter:
    return AE_IdCounter_CreateTable(bk->dataBase, tableNames[tt], flags);
  case AE_Book_TableType_Commodity:
    return AE_Commodity_CreateTable(bk->dataBase, tableNames[tt], flags);
  case AE_Book_TableType_Account:
    return AE_Account_CreateTable(bk->dataBase, tableNames[tt], flags);
  case AE_Book_TableType_BankAccount:
    return AE_BankAccount_CreateTable(bk->dataBase, tableNames[tt], flags);
  case AE_Book_TableType_BankStatement:
    return AE_Statement_CreateTable(bk->dataBase, tableNames[tt], flags);
  case AE_Book_TableType_BankTransfer:
  case AE_Book_TableType_BankDebitNote:
  case AE_Book_TableType_BankTransferPlanned:
  case AE_Book_TableType_BankTransferTemplate:
  case AE_Book_TableType_BankStandingOrder:
    return AE_Statement_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Transaction:
    return AE_Transaction_CreateTable(bk->dataBase, tableNames[tt], flags);
  case AE_Book_TableType_Split:
    return AE_Split_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_BankAccountGroup:
  case AE_Book_TableType_BankTransferGroup:
  case AE_Book_TableType_PartnerGroup:
  case AE_Book_TableType_TaxGroup:
  case AE_Book_TableType_ItemTemplateGroup:
  case AE_Book_TableType_VehicleGroup:
  case AE_Book_TableType_GraphicReportGroup:
    return AE_Group_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_BookedBalance:
  case AE_Book_TableType_NotedBalance:
    return AE_BankBalance_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Rule:
    return AE_Rule_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Address:
    return AE_Address_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Partner:
    return AE_Partner_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_PartnerRule:
    return AE_Rule_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Task:
    return AE_Task_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Job:
    return AE_Job_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Item:
  case AE_Book_TableType_ItemTemplates:
    return AE_Item_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Tax:
    return AE_Tax_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_AccountBalance:
    return AE_AccountBalance_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_BankStoRecurrence:
    return AE_Recurrence_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Vehicle:
    return AE_Vehicle_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Refueling:
    return AE_Refueling_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_GraphicReportSettings:
    return AE_Data10k_CreateTable(bk->dataBase, tableNames[tt], flags);

  case AE_Book_TableType_Share:
    return AE_Share_CreateTable(bk->dataBase, tableNames[tt], flags);


  case AE_Book_TableType_Project:
  case AE_Book_TableType_Warehouse:
    DBG_ERROR(AE_LOGDOMAIN, "Unhandled table type %d", tt);
    return GWEN_ERROR_NOT_IMPLEMENTED;

  }

  /* we should not reach this point */
  assert(0);
  return GWEN_ERROR_INTERNAL;
}



int AE_Book_OpenTable(AE_BOOK *bk,
		      AE_BOOK_TABLE_TYPE tt,
		      uint32_t flags) {
  int rv;
  AQDB_ID id;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return GWEN_ERROR_INVALID;
  }

  if (bk->tableIds[tt]!=0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table already open");
    return GWEN_ERROR_OPEN;
  }

  rv=AQDB_DB_OpenTable(bk->dataBase,
		       tableNames[tt],
		       flags,
		       &id);
  if (rv<0 && rv!=GWEN_ERROR_OPEN) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  assert(id!=0);

  bk->tableIds[tt]=id;
  return 0;
}



int AE_Book_IsTableOpen(AE_BOOK *bk,
                        AE_BOOK_TABLE_TYPE tt,
                        uint32_t flags) {
  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return GWEN_ERROR_INVALID;
  }

  if (bk->tableIds[tt]!=0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table already open");
    return GWEN_ERROR_OPEN;
  }

  return 0;
}



int AE_Book_CloseTable(AE_BOOK *bk,
		       AE_BOOK_TABLE_TYPE tt,
		       uint32_t flags) {
  int rv;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return GWEN_ERROR_INVALID;
  }

  if (bk->tableIds[tt]==0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  rv=AQDB_DB_CloseTable(bk->dataBase,
                        bk->tableIds[tt],
			flags);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  bk->tableIds[tt]=0;
  return 0;
}



AQDB_DB *AE_Book_GetDataBase(const AE_BOOK *bk) {
  assert(bk);
  assert(bk->refCount);

  return bk->dataBase;
}



AQDB_ID AE_Book_GetTableId(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt) {
  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return 0;
  }

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return 0;
  }

  return bk->tableIds[tt];
}



const char *AE_Book_GetTableName(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt) {
  assert(bk);
  assert(bk->refCount);

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return NULL;
  }

  return tableNames[tt];
}



AE_BOOK_TABLE_TYPE AE_Book_GetTableType(const AE_BOOK *bk, const char *s) {
  int i;

  assert(bk);
  assert(bk->refCount);

  for (i=AE_Book_TableType_IdCounter; i<AE_Book_TableType_Last; i++) {
    const char *t;

    t=tableNames[i];
    if (t && strcasecmp(t, s)==0)
      return i;
  }
  return AE_Book_TableType_Unknown;
}



AQDB_COLUMN_LIST *AE_Book_GetColumnList(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt) {
  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return NULL;
  }

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return NULL;
  }

  if (bk->tableIds[tt]==0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table not open");
    return NULL;
  }

  return AQDB_DB_GetColumnList(bk->dataBase, bk->tableIds[tt]);
}



int AE_Book_BeginEdit(AE_BOOK *bk, uint32_t flags) {
  int rv;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return 0;
  }

  rv=AQDB_DB_BeginEdit(bk->dataBase, flags);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_Book_EndEdit(AE_BOOK *bk, uint32_t flags) {
  int rv;

  assert(bk);
  assert(bk->refCount);

  if (bk->dbOpened<1) {
    DBG_ERROR(AE_LOGDOMAIN, "Book not open");
    return 0;
  }

  rv=AQDB_DB_EndEdit(bk->dataBase, flags);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_Book_GetFirstDbIdForQuery(AE_BOOK *bk,
				 AE_BOOK_TABLE_TYPE tt,
				 const char *qexpr,
				 AQDB_ID *pId) {
  AQDB_ID qid=0;
  int rv;
  AQDB_ID result=0;
  int len;

  assert(bk);
  assert(bk->refCount);

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return GWEN_ERROR_INVALID;
  }

  if (bk->tableIds[tt]==0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table not open");
    return GWEN_ERROR_NOT_OPEN;
  }

  /* query */
  rv=AQDB_DB_QuerySubmit(bk->dataBase,
			 bk->tableIds[tt],
			 0,
			 qexpr,
			 AQDB_QUERY_FLAGS_STOPATFIRST,
			 &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* get single first id */
  len=1;
  rv=AQDB_DB_QueryGetFirst(bk->dataBase, qid, &result, &len);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AQDB_DB_QueryClose(bk->dataBase, qid);
    return rv;
  }
  AQDB_DB_QueryClose(bk->dataBase, qid);

  if (len!=1) {
    DBG_ERROR(AE_LOGDOMAIN, "Query did not return an object id");
    return GWEN_ERROR_GENERIC;
  }

  *pId=result;
  return 0;
}



int AE_Book_GetFirstObjectForQuery(AE_BOOK *bk,
				   AE_BOOK_TABLE_TYPE tt,
				   const char *qexpr,
				   AQDB_OBJECT **pObj) {
  int rv;
  AQDB_ID oid=0;
  AQDB_OBJECT *o=NULL;

  rv=AE_Book_GetFirstDbIdForQuery(bk,
				  tt,
				  qexpr,
				  &oid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AQDB_DB_ReadObject(bk->dataBase,
			bk->tableIds[tt],
			oid,
			&o);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  *pObj=o;
  return 0;
}



int AE_Book_ForEveryQueryObject(AE_BOOK *bk,
				AE_BOOK_TABLE_TYPE tt,
                                AQDB_ID refQueryId,
				const char *qexpr,
				AE_BOOK_FOR_EVERY_QUERY_OBJECT_FN fn,
				void *data,
				void **pResult) {
  AQDB_ID qid=0;
  int rv;
  AQDB_ID oids[16];
  int len;

  assert(bk);
  assert(bk->refCount);

  if (!(tt>AE_Book_TableType_None && tt<AE_Book_TableType_Last)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad table type %d", tt);
    return GWEN_ERROR_INVALID;
  }

  if (bk->tableIds[tt]==0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table %d not open", tt);
    return GWEN_ERROR_NOT_OPEN;
  }

  /* query */
  rv=AQDB_DB_QuerySubmit(bk->dataBase,
			 bk->tableIds[tt],
			 refQueryId,
			 qexpr,
			 0, /* query flags */
			 &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* browse all objects */
  len=16;
  rv=AQDB_DB_QueryGetFirst(bk->dataBase, qid, oids, &len);
  while(rv==0) {
    int i;

    for (i=0; i<len; i++) {
      AQDB_OBJECT *o=NULL;
      void *result=NULL;

      rv=AQDB_DB_ReadObject(bk->dataBase,
			    bk->tableIds[tt],
			    oids[i],
			    &o);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
	AQDB_DB_QueryClose(bk->dataBase, qid);
	return rv;
      }

      result=fn(bk, tt, o, data);
      AQDB_Object_free(o);
      if (result!=NULL) {
	AQDB_DB_QueryClose(bk->dataBase, qid);
        if (pResult)
	  *pResult=result;
        return 0;
      }
    }

    len=16;
    rv=AQDB_DB_QueryGetNext(bk->dataBase, qid, oids, &len);
  }
  AQDB_DB_QueryClose(bk->dataBase, qid);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



int AE_Book_GetNextId(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, AQDB_ID *pId) {
  char qexpr[64];
  int rv;
  AQDB_ID result=0;
  AQDB_OBJECT *o=NULL;

  snprintf(qexpr, sizeof(qexpr)-1, "$id==%d", (int) tt);
  qexpr[sizeof(qexpr)-1]=0;

  rv=AE_Book_GetFirstObjectForQuery(bk,
				    AE_Book_TableType_IdCounter,
				    qexpr,
				    &o);
  if (rv<0) { /* no object found, create first one */
    AE_IDCOUNTER *ic;

    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }

    /* create new id counter */
    ic=AE_IdCounter_new();
    AE_IdCounter_SetId(ic, (int)tt);
    AE_IdCounter_SetName(ic, tableNames[tt]);
    AE_IdCounter_SetLastId(ic, 1);

    /* create empty object */
    rv=AE_IdCounter_CreateObject(bk->dataBase,
				 bk->tableIds[AE_Book_TableType_IdCounter],
				 (AQDB_ID)tt,
				 &o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_IdCounter_free(ic);
      return rv;
    }

    /* fill empty object with new id counter */
    rv=AE_IdCounter_toObject(ic, o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_Object_free(o);
      return rv;
    }
    AE_IdCounter_free(ic);

    /* add object to DB */
    rv=AQDB_DB_AddObject(bk->dataBase, bk->tableIds[AE_Book_TableType_IdCounter], o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_Object_free(o);
      return rv;
    }

    AQDB_Object_free(o);
    result=1;
  }
  else {
    AE_IDCOUNTER *ic=NULL;

    rv=AE_IdCounter_fromObject(o, &ic);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_Object_free(o);
      return rv;
    }
    AE_IdCounter_SetDbId(ic, AQDB_Object_GetObjectId(o));

    result=AE_IdCounter_GetLastId(ic)+1;
    AE_IdCounter_SetLastId(ic, result);

    rv=AE_IdCounter_toObject(ic, o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_Object_free(o);
      return rv;
    }
    AE_IdCounter_free(ic);

    rv=AQDB_DB_WriteObject(bk->dataBase, o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_Object_free(o);
      return rv;
    }
    AQDB_Object_free(o);
  }

  *pId=result;
  return 0;
}



int AE_Book_GetLastId(AE_BOOK *bk, AE_BOOK_TABLE_TYPE tt, AQDB_ID *pId) {
  char qexpr[64];
  int rv;
  AQDB_ID result=0;
  AQDB_OBJECT *o=NULL;

  snprintf(qexpr, sizeof(qexpr)-1, "$id==%d", (int) tt);
  qexpr[sizeof(qexpr)-1]=0;

  rv=AE_Book_GetFirstObjectForQuery(bk,
				    AE_Book_TableType_IdCounter,
				    qexpr,
				    &o);
  if (rv<0) { /* no object found, return 0 */
    result=0;
  }
  else {
    AE_IDCOUNTER *ic=NULL;

    rv=AE_IdCounter_fromObject(o, &ic);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_Object_free(o);
      return rv;
    }
    AE_IdCounter_SetDbId(ic, AQDB_Object_GetObjectId(o));

    result=AE_IdCounter_GetLastId(ic);
    AE_IdCounter_free(ic);
    AQDB_Object_free(o);
  }

  *pId=result;
  return 0;
}




#include "ae_book_read.c"
#include "ae_book_write.c"
#include "ae_book_apply.c"
#include "ae_book_conflict_account.c"
#include "ae_book_conflict_transaction.c"
#include "ae_book_conflict_bankstatement.c"
#include "ae_book_conflict_bankaccount.c"
#include "ae_book_conflict_job.c"
#include "ae_book_conflict_task.c"
#include "ae_book_conflict_partner.c"
#include "ae_book_conflict_ptgroup.c"
#include "ae_book_conflict.c"
#include "ae_book_report.c"
#include "ae_book_report2.c"


