/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


/* this file is included from ae_book.c */



static int AE_Book_SortGroupsIntoTree(AE_GROUP_TREE *gtree,
				      AE_GROUP *parent,
				      AE_GROUP_LIST *glist) {
  AQDB_ID pid=0;

  if (parent)
    pid=AE_Group_GetId(parent);

  while(AE_Group_List_GetCount(glist)) {
    AE_GROUP *gr;

    gr=AE_Group_List_First(glist);
    while(gr) {
      AQDB_ID parentId;

      parentId=AE_Group_GetParentId(gr);
      if (parentId==pid) {
	int rv;

	AE_Group_List_Del(gr);
	if (parent)
	  AE_Group_Tree_AddChild(parent, gr);
	else
	  AE_Group_Tree_Add(gtree, gr);
	rv=AE_Book_SortGroupsIntoTree(gtree, gr, glist);
	if (rv<0)
          return rv;
	break;
      }
      gr=AE_Group_List_Next(gr);
    }
    if (gr==NULL)
      /* nothing more to do */
      return 0;
  }

  return 0;
}



int AE_Book_ReadGroupTree(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt, AE_GROUP_TREE *gtree) {
  AE_GROUP_LIST *glist;
  int rv;

  /* read all groups sequentially (we will sort them later) */
  glist=AE_Group_List_new();
  rv=AE_Book_ReadGroupList(b, tt, glist);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Group_List_free(glist);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
  }

  /* now glist contains all groups read, sort them into the tree */
  rv=AE_Book_SortGroupsIntoTree(gtree, NULL, glist);
  if (AE_Group_List_GetCount(glist)) {
    AE_GROUP *gr;

    /* still some group left, add them to root */
    DBG_WARN(AE_LOGDOMAIN,
	     "The parents of some groups could not be found, assigning to root");
    while( (gr=AE_Group_List_First(glist)) ) {
      /* reassign to root */
      AE_Group_List_Del(gr);
      AE_Group_SetParentId(gr, 0);
      AE_Group_Tree_Add(gtree, gr);
    }
  }

  AE_Group_List_free(glist);

  return 0;
}



int AE_Book_SortAccountsIntoTree(AE_ACCOUNT_TREE *atree,
				 AE_ACCOUNT *parent,
				 AE_ACCOUNT_LIST *alist) {
  AQDB_ID pid=0;

  if (parent)
    pid=AE_Account_GetId(parent);

  while(AE_Account_List_GetCount(alist)) {
    AE_ACCOUNT *a;

    a=AE_Account_List_First(alist);
    while(a) {
      AQDB_ID parentId;

      parentId=AE_Account_GetParentId(a);
      if (parentId==pid) {
	int rv;

	AE_Account_List_Del(a);
	if (parent)
	  AE_Account_Tree_AddChild(parent, a);
	else
	  AE_Account_Tree_Add(atree, a);
	rv=AE_Book_SortAccountsIntoTree(atree, a, alist);
	if (rv<0)
	  return rv;
	break;
      }
      a=AE_Account_List_Next(a);
    }
    if (a==NULL)
      /* nothing more to do */
      return 0;
  }

  return 0;
}



int AE_Book_ReadAccountTree(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt, AE_ACCOUNT_TREE *atree) {
  AE_ACCOUNT_LIST *alist;
  int rv;

  /* read all groups sequentially (we will sort them later) */
  alist=AE_Account_List_new();
  rv=AE_Book_ReadAccountList(b, tt, alist);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Account_List_free(alist);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
  }

  /* now alist contains all accounts read, sort them into the tree */
  rv=AE_Book_SortAccountsIntoTree(atree, NULL, alist);
  if (AE_Account_List_GetCount(alist)) {
    AE_ACCOUNT *a;

    /* still some accounts left, add them to root */
    DBG_WARN(AE_LOGDOMAIN,
	     "The parents of some accounts could not be found, assigning to root");
    while( (a=AE_Account_List_First(alist)) ) {
      /* reassign to root */
      AE_Account_List_Del(a);
      AE_Account_SetParentId(a, 0);
      AE_Account_Tree_Add(atree, a);
    }
  }

  AE_Account_List_free(alist);

  return 0;
}



int AE_Book_ReadAccountTreeXml(AE_BOOK *b, AE_ACCOUNT_TREE *at, GWEN_XMLNODE *node) {
  GWEN_XMLNODE *n;
  AE_ACCOUNT_LIST *al;

  al=AE_Account_List_new();

  n=GWEN_XMLNode_FindFirstTag(node, "account", NULL, NULL);
  while(n) {
    AE_ACCOUNT *acc;

    acc=AE_Account_fromXml(n);
    if (acc==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "Error in account element");
    }
    else {
      AE_Account_List_Add(acc, al);
    }

    n=GWEN_XMLNode_FindNextTag(n, "account", NULL, NULL);
  }


  if (AE_Account_List_GetCount(al)) {
    int rv;

    rv=AE_Book_SortAccountsIntoTree(at, NULL, al);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Account_List_free(al);
      return rv;
    }
  }
  AE_Account_List_free(al);

  return 0;
}




#define READ_TYPE(macro_prefix, macro_type)                                      \
void GWENHYWFAR_CB  *AE_Book_Read##macro_prefix##_Callback(AE_BOOK *bk,          \
                                            AE_BOOK_TABLE_TYPE tt,               \
                                            AQDB_OBJECT *o,                      \
                                            void *data) {                        \
  AE_##macro_type##_LIST *stlist;                                                \
  AE_##macro_type *ba=NULL;                                                      \
  int rv;                                                                        \
                                                                                 \
  stlist=(AE_##macro_type##_LIST*) data;                                         \
                                                                                 \
  rv=AE_##macro_prefix##_fromObject(o, &ba);                                     \
  if (rv<0) {                                                                    \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                     \
  }                                                                              \
  else {                                                                         \
    AE_##macro_prefix##_SetDbId(ba, AQDB_Object_GetObjectId(o));                 \
    AE_##macro_prefix##_List_Add(ba, stlist);                                     \
  }                                                                              \
                                                                                 \
  return NULL;                                                                   \
}                                                                                \
                                                                                 \
                                                                                 \
                                                                                 \
int AE_Book_Read##macro_prefix##List(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,          \
                                     AE_##macro_type##_LIST *stlist) {           \
  int rv;                                                                        \
  void *dummy=NULL;                                                              \
                                                                                 \
  /* read all groups sequentially (we will sort them later) */                   \
  rv=AE_Book_ForEveryQueryObject(b,                                              \
                                 tt,                                             \
                                 0,                                              \
                                 NULL, /* empty query string */                  \
                                 AE_Book_Read##macro_prefix##_Callback,          \
                                 (void*) stlist,                                 \
                                 &dummy);                                        \
  if (rv<0) {                                                                    \
    if (rv!=GWEN_ERROR_NOT_FOUND) {                                              \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                   \
      return rv;                                                                 \
    }                                                                            \
    DBG_INFO(AE_LOGDOMAIN, "No entries found");                                  \
  }                                                                              \
                                                                                 \
  return 0;                                                                      \
}                                                                                \
                                                                                 \
                                                                                 \
                                                                                 \
int AE_Book_Query##macro_prefix##List(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt,         \
                                      AQDB_ID refQueryId,                        \
                                      const char *query,                         \
                                      AE_##macro_type##_LIST *stlist) {          \
  int rv;                                                                        \
  void *dummy=NULL;                                                              \
                                                                                 \
  /* read all groups sequentially (we will sort them later) */                   \
  rv=AE_Book_ForEveryQueryObject(b,                                              \
                                 tt,                                             \
                                 refQueryId,                                     \
                                 query,                                          \
                                 AE_Book_Read##macro_prefix##_Callback,          \
                                 (void*) stlist,                                 \
                                 &dummy);                                        \
  if (rv<0) {                                                                    \
    if (rv!=GWEN_ERROR_NOT_FOUND) {                                              \
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                   \
      return rv;                                                                 \
    }                                                                            \
    DBG_INFO(AE_LOGDOMAIN, "No entries found");                                  \
  }                                                                              \
                                                                                 \
  return 0;                                                                      \
}                                                                                \
                                                                                 \
                                                                                 \
                                                                                 \
int AE_Book_Read##macro_prefix(AE_BOOK *b,                                       \
                               AE_BOOK_TABLE_TYPE tt,                            \
                               AQDB_ID stmId,                                    \
                               AE_##macro_type **pst) {                          \
  AQDB_OBJECT *o=NULL;                                                           \
  char qexpr[64];                                                                \
  int rv;                                                                        \
                                                                                 \
  snprintf(qexpr, sizeof(qexpr)-1, "$id==%u", stmId);                            \
  qexpr[sizeof(qexpr)-1]=0;                                                      \
                                                                                 \
  rv=AE_Book_GetFirstObjectForQuery(b, tt, qexpr, &o);                           \
  if (rv<0) {                                                                    \
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);                                     \
    return rv;                                                                   \
  }                                                                              \
  else {                                                                         \
    AE_##macro_type *st=NULL;                                                    \
                                                                                 \
    rv=AE_##macro_prefix##_fromObject(o, &st);                                   \
    if (rv<0) {                                                                  \
      DBG_INFO(0, "here (%d)", rv);                                              \
      AQDB_Object_free(o);                                                       \
      return rv;                                                                 \
    }                                                                            \
                                                                                 \
    AE_##macro_prefix##_SetDbId(st, AQDB_Object_GetObjectId(o));                 \
    AQDB_Object_free(o);                                                         \
                                                                                 \
    *pst=st;                                                                     \
    return 0;                                                                    \
  }                                                                              \
}


READ_TYPE(Group, GROUP)
READ_TYPE(Statement, STATEMENT)
READ_TYPE(BankBalance, BANKBALANCE)
READ_TYPE(BankAccount, BANKACCOUNT)
READ_TYPE(Commodity, COMMODITY)
READ_TYPE(Transaction, TRANSACTION)
READ_TYPE(Split, SPLIT)
READ_TYPE(Account, ACCOUNT)
READ_TYPE(AccountBalance, ACCOUNTBALANCE)
READ_TYPE(Rule, RULE)
READ_TYPE(Address, ADDRESS)
READ_TYPE(Partner, PARTNER)
READ_TYPE(Task, TASK)
READ_TYPE(Job, JOB)
READ_TYPE(Item, ITEM)
READ_TYPE(Tax, TAX)
READ_TYPE(LoadCellPhone, LOAD_CELLPHONE)
READ_TYPE(Recurrence, RECURRENCE)
READ_TYPE(Vehicle, VEHICLE)
READ_TYPE(Refueling, REFUELING)
READ_TYPE(Data10k, DATA_10K)
READ_TYPE(Share, SHARE)



int AE_Book_ReadTransactionSplits(AE_BOOK *b,
                                  AE_TRANSACTION *t,
                                  AE_BOOK_TABLE_TYPE tt) {
  AE_SPLIT_LIST *sl;
  int rv;
  char qstr[256];

  sl=AE_Transaction_GetSplitList(t);
  if (sl==NULL) {
    sl=AE_Split_List_new();
    AE_Transaction_SetSplitList(t, sl);
  }
  else
    AE_Split_List_Clear(sl);

  snprintf(qstr, sizeof(qstr)-1, "$transactionId==%d", AE_Transaction_GetId(t));
  qstr[sizeof(qstr)-1]=0;

  rv=AE_Book_QuerySplitList(b, tt, 0, qstr, sl);
  if (rv<0) {
    DBG_ERROR(0, "here (%d)", rv);
    return rv;
  }

  if (AE_Split_List_GetCount(sl)>2)
    AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTISPLIT);
  else
    AE_Transaction_SubRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_MULTISPLIT);

  return 0;
}



int AE_Book_ReadTransactionSubObjects(AE_BOOK *b, AE_TRANSACTION *t) {
  AQDB_ID tid;
  AQDB_ID stId;
  char qstring[128];
  int rv;

  /* read splits */
  tid=AE_Transaction_GetId(t);
  if (tid) {
    AE_SPLIT_LIST *sl;

    sl=AE_Split_List_new();
    snprintf(qstring, sizeof(qstring)-1, "$transactionId==%d", tid);
    qstring[sizeof(qstring)-1]=0;

    rv=AE_Book_QuerySplitList(b, AE_Book_TableType_Split, 0, qstring, sl);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Split_List_free(sl);
      return rv;
    }
    AE_Transaction_SetSplitList(t, sl);
  }

  stId=AE_Transaction_GetStatementId(t);
  if (stId) {
    AE_STATEMENT *st=NULL;

    rv=AE_Book_ReadStatement(b, AE_Book_TableType_BankStatement, stId, &st);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    AE_Transaction_SetStatement(t, st);
  }

  /* done */
  return 0;
}



int AE_Book_ReadPartnerSubObjects(AE_BOOK *b, AE_PARTNER *pt) {
  AE_ADDRESS_LIST *al;
  AE_BANKACCOUNT_LIST *bl;
  AE_RULE_LIST *rl;
  char qstring[128];
  int rv;

  /* read addresses */
  al=AE_Address_List_new();
  snprintf(qstring, sizeof(qstring)-1, "($ownertype==%d) && ($ownerid==%d)",
	   AE_Book_TableType_Partner,
	   AE_Partner_GetId(pt));
  qstring[sizeof(qstring)-1]=0;

  rv=AE_Book_QueryAddressList(b, AE_Book_TableType_Address, 0, qstring, al);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Address_List_free(al);
    return rv;
  }
  AE_Partner_SetAddressList(pt, al);

  /* read bank accounts */
  bl=AE_BankAccount_List_new();
  snprintf(qstring, sizeof(qstring)-1, "($ownertype==%d) && ($ownerid==%d)",
	   AE_Book_TableType_Partner,
	   AE_Partner_GetId(pt));
  qstring[sizeof(qstring)-1]=0;

  rv=AE_Book_QueryBankAccountList(b, AE_Book_TableType_BankAccount, 0, qstring, bl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_BankAccount_List_free(bl);
    return rv;
  }
  AE_Partner_SetBankAccountList(pt, bl);

  /* read rules */
  rl=AE_Rule_List_new();
  snprintf(qstring, sizeof(qstring)-1, "($ownertype==%d) && ($ownerid==%d)",
	   AE_Book_TableType_Partner,
	   AE_Partner_GetId(pt));
  qstring[sizeof(qstring)-1]=0;

  rv=AE_Book_QueryRuleList(b, AE_Book_TableType_PartnerRule, 0, qstring, rl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Rule_List_free(rl);
    return rv;
  }
  AE_Partner_SetRules(pt, rl);

  /* done */
  return 0;
}



int AE_Book_ReadJobSubObjects(AE_BOOK *b, AE_JOB *j, int buy) {
  AE_ITEM_LIST *il;
  AE_ITEM *item;
  char qstring[128];
  int rv;
  AQDB_VALUE *v;

  /* read addresses */
  il=AE_Item_List_new();
  snprintf(qstring, sizeof(qstring)-1, "$jobid==%d",
	   AE_Job_GetId(j));
  qstring[sizeof(qstring)-1]=0;

  rv=AE_Book_QueryItemList(b, AE_Book_TableType_Item, 0, qstring, il);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Item_List_free(il);
    return rv;
  }
  AE_Job_SetItems(j, il);

  /* calculate total sum */
  v=AQDB_Value_new();
  item=AE_Item_List_First(il);
  while(item) {
    const AQDB_VALUE *vt;

    if (buy) {
      vt=AE_Item_GetBuyPriceValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);
      vt=AE_Item_GetBuyTaxSumValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);
    }
    else {
      vt=AE_Item_GetSellPriceValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);
      vt=AE_Item_GetSellTaxSumValue(item);
      if (vt)
	AQDB_Value_AddValue(v, vt);
    }
    item=AE_Item_List_Next(item);
  }
  AE_Job_SetTotalSum(j, v);

  /* done */
  return 0;
}



int AE_Book_ReadTaskSubObjects(AE_BOOK *b, AE_TASK *tk, int buy) {
  AE_JOB_LIST *jl;
  AE_JOB *j;
  char qstring[128];
  int rv;

  /* read addresses */
  jl=AE_Job_List_new();
  snprintf(qstring, sizeof(qstring)-1, "$taskid==%d",
	   AE_Task_GetId(tk));
  qstring[sizeof(qstring)-1]=0;

  rv=AE_Book_QueryJobList(b, AE_Book_TableType_Job, 0, qstring, jl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Job_List_free(jl);
    return rv;
  }
  AE_Task_SetJobs(tk, jl);

  /* read every job */
  j=AE_Job_List_First(jl);
  while(j) {
    rv=AE_Book_ReadJobSubObjects(b, j, buy);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }

    j=AE_Job_List_Next(j);
  }

  /* done */
  return 0;
}



int AE_Book_ReadAccountTreeBalances(AE_BOOK *b, AE_ACCOUNT_TREE *at) {
  int rv;
  AE_ACCOUNTBALANCE_LIST *abl;
  AE_ACCOUNTBALANCE *bal;

  abl=AE_AccountBalance_List_new();
  rv=AE_Book_ReadAccountBalanceList(b, AE_Book_TableType_AccountBalance, abl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_AccountBalance_List_free(abl);
    return rv;
  }

  bal=AE_AccountBalance_List_First(abl);
  while(bal) {
    AQDB_ID aid;

    aid=AE_AccountBalance_GetAccountId(bal);
    if (aid>0) {
      AE_ACCOUNT *acc;

      acc=AE_Account_Tree_GetById(at, aid);
      if (acc)
	AE_Account_SetBalance(acc, AE_AccountBalance_dup(bal));
    }
    bal=AE_AccountBalance_List_Next(bal);
  }
  AE_AccountBalance_List_free(abl);

  /* done */
  return 0;
}



static void GWENHYWFAR_CB *AE_Book__CalcAccountTreeBalances_Callback(AE_BOOK *bk,
								     AE_BOOK_TABLE_TYPE tt,
								     AQDB_OBJECT *o,
								     void *data) {
  AE_ACCOUNT_TREE *at;
  AE_SPLIT *sp=NULL;
  int rv;

  at=(AE_ACCOUNT_TREE*) data;

  rv=AE_Split_fromObject(o, &sp);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AQDB_ID aid;

    aid=AE_Split_GetAccountId(sp);
    if (aid>0) {
      AE_ACCOUNT *acc;

      acc=AE_Account_Tree_GetById(at, aid);
      if (acc) {
	if (AE_Account_GetPrimaryCommodityId(acc)==AE_Split_GetCommodityId(sp)) {
	  AE_ACCOUNTBALANCE *bal;
	  AQDB_VALUE *vDest;
	  const AQDB_VALUE *vSrc;

	  bal=AE_Account_GetBalance(acc);
	  if (bal==NULL) {
	    bal=AE_AccountBalance_new();
	    AE_AccountBalance_SetCommodityId(bal, AE_Account_GetPrimaryCommodityId(acc));
	    AE_AccountBalance_SetAccountId(bal, AE_Account_GetId(acc));
	    AE_Account_SetBalance(acc, bal);
	  }

	  vDest=AE_AccountBalance_GetValue(bal);
	  vSrc=AE_Split_GetAmount(sp);
	  AQDB_Value_AddValue(vDest, vSrc);
	}
	else {
	  DBG_ERROR(AE_LOGDOMAIN, "Split is not in primary commodity of the account");
	}
      }
      else {
        DBG_ERROR(0, "Account not found");
      }
    }
    else {
      DBG_ERROR(0, "Account id in split is 0");
    }
    AE_Split_free(sp);
  }

  return NULL;
}



int AE_Book_CalcAccountTreeBalances(AE_BOOK *b, AE_ACCOUNT_TREE *at) {
  int rv;
  void *dummy=NULL;

  /* read all splits */
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Split,
				 0,
                                 NULL, /* empty query string */
				 AE_Book__CalcAccountTreeBalances_Callback,
				 (void*) at,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
  }

  return 0;
}



static void GWENHYWFAR_CB *AE_Book__findLatestStatement_Callback(AE_BOOK *bk,
								 AE_BOOK_TABLE_TYPE tt,
								 AQDB_OBJECT *o,
								 void *data) {
  AE_STATEMENT **pStatement;
  AE_STATEMENT *st=NULL;
  int rv;

  pStatement=(AE_STATEMENT**) data;

  rv=AE_Statement_fromObject(o, &st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    if (*pStatement==NULL)
      *pStatement=st;
    else {
      const GWEN_DATE *d1;
      const GWEN_DATE *d2;

      d1=AE_Statement_GetDate(*pStatement);
      d2=AE_Statement_GetDate(st);
      if (d1 && d2 && GWEN_Date_Compare(d1, d2)<0) {
	/* new max */
	AE_Statement_free(*pStatement);
	*pStatement=st;
      }
      else
        AE_Statement_free(st);
    }
  }

  return NULL;
}



int AE_Book_FindLatestStatement(AE_BOOK *b, const char *queryString, AE_STATEMENT **pStatement) {
  int rv;
  void *dummy=NULL;
  AE_STATEMENT *st=NULL;

  /* read all splits */
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_BankStatement,
				 0,
				 queryString,
                                 AE_Book__findLatestStatement_Callback,
				 (void*) &st,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
  }

  *pStatement=st;

  return 0;
}






int AE_Book_ReadTransactionsToListAndMap(AE_BOOK *b,
                                         AQDB_ID refQueryId,
                                         const char *qString,
                                         AE_TRANSACTION_LIST *tList,
                                         AE_TRANSACTION_IDMAP *tMap) {
  AQDB_ID qid=0;
  AQDB_DB *db;
  AQDB_ID tid;
  int rv;
  AQDB_ID oids[16];
  int len;

  db=AE_Book_GetDataBase(b);
  tid=AE_Book_GetTableId(b, AE_Book_TableType_Transaction);

  /* query every transaction */
  rv=AQDB_DB_QuerySubmit(db, tid, refQueryId, qString, 0, &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* browse all objects */
  len=16;
  rv=AQDB_DB_QueryGetFirst(db, qid, oids, &len);
  while(rv==0) {
    int i;

    for (i=0; i<len; i++) {
      AQDB_OBJECT *o=NULL;
      AE_TRANSACTION *t=NULL;

      rv=AQDB_DB_ReadObject(db, tid, oids[i], &o);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_DB_QueryClose(db, qid);
        return rv;
      }

      rv=AE_Transaction_fromObject(o, &t);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_Object_free(o);
        AQDB_DB_QueryClose(db, qid);
        return rv;
      }
      AE_Transaction_SetDbId(t, AQDB_Object_GetObjectId(o));

      AE_Transaction_List_Add(t, tList);
      AE_Transaction_IdMap_Insert(tMap, AE_Transaction_GetId(t), t);
      AQDB_Object_free(o);
    }

    len=16;
    rv=AQDB_DB_QueryGetNext(db, qid, oids, &len);
  }
  AQDB_DB_QueryClose(db, qid);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}




int AE_Book_ReadSplitListSubObjects(AE_BOOK *b, AE_SPLIT_LIST *sl) {
  AE_ACCOUNT_TREE *at;
  AE_TRANSACTION_LIST *tList;
  AE_TRANSACTION_IDMAP *tMap;
  AE_COMMODITY_LIST *cl;
  AE_SPLIT *sp;
  GWEN_BUFFER *tbuf;
  int rv;

  at=AE_Account_Tree_new();
  rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Account_Tree_free(at);
    return rv;
  }

  cl=AE_Commodity_List_new();
  rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Commodity_List_free(cl);
    AE_Account_Tree_free(at);
    return rv;
  }

  tList=AE_Transaction_List_new();
  tMap=AE_Transaction_IdMap_new(GWEN_IdMapAlgo_Hex4);
  rv=AE_Book_ReadTransactionsToListAndMap(b, 0, NULL, tList, tMap);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Transaction_IdMap_free(tMap);
    AE_Transaction_List_free(tList);
    AE_Commodity_List_free(cl);
    AE_Account_Tree_free(at);
    return rv;
  }

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);

  sp=AE_Split_List_First(sl);
  while(sp) {
    AQDB_ID id;

    /* set transaction */
    id=AE_Split_GetTransactionId(sp);
    if (id) {
      AE_TRANSACTION *t;

      t=AE_Transaction_IdMap_Find(tMap, id);
      if (t) {
        AE_Transaction_Attach(t);
        AE_Split_SetTmpTransaction(sp, t);
      }
    }

    /* set commodity */
    id=AE_Split_GetCommodityId(sp);
    if (id) {
      AE_COMMODITY *c;

      c=AE_Commodity_List_GetById(cl, id);
      if (c) {
        AE_Commodity_Attach(c);
        AE_Split_SetTmpCommodity(sp, c);
      }
    }

    /* set account */
    id=AE_Split_GetAccountId(sp);
    if (id) {
      AE_ACCOUNT *a;

      a=AE_Account_Tree_GetById(at, id);
      if (a) {
        AE_Account_PathToBuffer(a, tbuf);
        AE_Split_SetTmpAccountPath(sp, GWEN_Buffer_GetStart(tbuf));
        GWEN_Buffer_Reset(tbuf);
      }
    }

    sp=AE_Split_List_Next(sp);
  }
  GWEN_Buffer_free(tbuf);

  AE_Transaction_IdMap_free(tMap);
  AE_Transaction_List_free(tList);
  AE_Commodity_List_free(cl);
  AE_Account_Tree_free(at);

  return 0;
}



int AE_Book_ReadSplitListByDateAndCommodity(AE_BOOK *b,
                                            const GWEN_DATE *fromDate,
                                            const GWEN_DATE *toDate,
                                            const char *commodity,
                                            AE_SPLIT_LIST *splitList) {
  char qBuf[256];
  AE_TRANSACTION_LIST *tList;
  AE_TRANSACTION_IDMAP *tMap;
  AE_SPLIT_LIST *sl;
  int rv;
  AQDB_ID qid=0;
  AQDB_DB *db;
  AQDB_ID tid;
  AQDB_ID oids[16];
  int len;

  if (fromDate && toDate) {
    snprintf(qBuf, sizeof(qBuf)-1, "($date>=\"%s\") && ($date<=\"%s\")",
	     GWEN_Date_GetString(fromDate),
	     GWEN_Date_GetString(toDate));
    qBuf[sizeof(qBuf)-1]=0;
  }
  else if (fromDate) {
    snprintf(qBuf, sizeof(qBuf)-1, "$date>=\"%s\"",
	     GWEN_Date_GetString(fromDate));
    qBuf[sizeof(qBuf)-1]=0;
  }
  else if (toDate) {
    snprintf(qBuf, sizeof(qBuf)-1, "$date<=\"%s\"",
	     GWEN_Date_GetString(toDate));
    qBuf[sizeof(qBuf)-1]=0;
  }
  else {
    qBuf[0]=0;
  }

  /* first read matching transactions */
  tList=AE_Transaction_List_new();
  tMap=AE_Transaction_IdMap_new(GWEN_IdMapAlgo_Hex4);
  rv=AE_Book_ReadTransactionsToListAndMap(b, 0, (qBuf[0])?qBuf:NULL, tList, tMap);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Transaction_IdMap_free(tMap);
    AE_Transaction_List_free(tList);
    return rv;
  }

  /* then read splits and keep those for which we have a transaction */
  db=AE_Book_GetDataBase(b);
  tid=AE_Book_GetTableId(b, AE_Book_TableType_Split);

  /* query every transaction */
  rv=AQDB_DB_QuerySubmit(db, tid, 0, NULL, 0, &qid);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Transaction_IdMap_free(tMap);
    AE_Transaction_List_free(tList);
    return rv;
  }

  /* browse all splits (we need this to keep the AE_TRANSACTIONs complete) */
  sl=AE_Split_List_new();
  len=16;
  rv=AQDB_DB_QueryGetFirst(db, qid, oids, &len);
  while(rv==0) {
    int i;

    for (i=0; i<len; i++) {
      AQDB_OBJECT *o=NULL;
      AE_SPLIT *sp=NULL;
      AE_TRANSACTION *t;

      rv=AQDB_DB_ReadObject(db, tid, oids[i], &o);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_DB_QueryClose(db, qid);
        AE_Split_List_free(sl);
        AE_Transaction_IdMap_free(tMap);
        AE_Transaction_List_free(tList);
        return rv;
      }

      rv=AE_Split_fromObject(o, &sp);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "Error reading object %d (%d)", oids[i], rv);
        AQDB_Object_free(o);
        AQDB_DB_QueryClose(db, qid);
        AE_Split_List_free(sl);
        AE_Transaction_IdMap_free(tMap);
        AE_Transaction_List_free(tList);
        return rv;
      }
      AE_Split_SetDbId(sp, AQDB_Object_GetObjectId(o));
      AQDB_Object_free(o);

      /* get transaction */
      t=AE_Transaction_IdMap_Find(tMap, AE_Split_GetTransactionId(sp));
      if (t) {
        AE_Transaction_Attach(t);
        AE_Split_SetTmpTransaction(sp, t);
        AE_Split_List_Add(sp, sl);
      }
      else
        AE_Split_free(sp);
    }

    len=16;
    rv=AQDB_DB_QueryGetNext(db, qid, oids, &len);
  }
  AQDB_DB_QueryClose(db, qid);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Split_List_free(sl);
      AE_Transaction_IdMap_free(tMap);
      AE_Transaction_List_free(tList);
      return rv;
    }
  }

  AE_Transaction_IdMap_free(tMap);
  AE_Transaction_List_free(tList);

  /* now filter out all non-matching splits and set their sub-objects */
  if (AE_Split_List_GetCount(sl)) {
    AE_ACCOUNT_TREE *at;
    AE_COMMODITY_LIST *cl;
    AE_SPLIT *sp;
    GWEN_BUFFER *tbuf;
    AE_COMMODITY *c;
    AQDB_ID coId=0;

    at=AE_Account_Tree_new();
    rv=AE_Book_ReadAccountTree(b, AE_Book_TableType_Account, at);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Account_Tree_free(at);
      AE_Split_List_free(sl);
      return rv;
    }
  
    cl=AE_Commodity_List_new();
    rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Commodity_List_free(cl);
      AE_Account_Tree_free(at);
      AE_Split_List_free(sl);
      return rv;
    }

    c=AE_Commodity_List_GetByCode(cl, commodity);
    if (c)
      coId=AE_Commodity_GetId(c);

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    while( (sp=AE_Split_List_First(sl)) ) {
      AQDB_ID id;

      AE_Split_List_Del(sp);

      /* set account */
      id=AE_Split_GetAccountId(sp);
      if (id) {
        AE_ACCOUNT *a;
  
        a=AE_Account_Tree_GetById(at, id);
        if (a) {
          AE_Account_PathToBuffer(a, tbuf);
          AE_Split_SetTmpAccountPath(sp, GWEN_Buffer_GetStart(tbuf));
	  AE_Split_SetTmpAccountType(sp, AE_Account_GetType(a));
          GWEN_Buffer_Reset(tbuf);
          if (AE_AccountType_NeedReverseSign(AE_Account_GetType(a)))
            AE_Split_AddRuntimeFlags(sp, AE_SPLIT_RTFLAGS_REVERSE);
        }
      }

      if ((coId==0) || (coId==AE_Split_GetCommodityId(sp))) {
        /* set commodity */
        id=AE_Split_GetCommodityId(sp);
        if (id) {
          c=AE_Commodity_List_GetById(cl, id);
          if (c) {
            AE_Commodity_Attach(c);
            AE_Split_SetTmpCommodity(sp, c);
          }
        }
        AE_Split_List_Add(sp, splitList);
      }
      else
        AE_Split_free(sp);
    }
    GWEN_Buffer_free(tbuf);

    AE_Commodity_List_free(cl);
    AE_Account_Tree_free(at);
  }

  AE_Split_List_free(sl);
  return 0;
}



int AE_Book_ReadSplitListByDateAndAccountId(AE_BOOK *b,
                                            const GWEN_DATE *fromDate,
                                            const GWEN_DATE *toDate,
                                            AQDB_ID accountId,
                                            const char *commodity,
                                            AE_SPLIT_LIST *splitList) {
  AE_SPLIT_LIST *spl;
  int rv;
  AE_SPLIT *sp;

  spl=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndCommodity(b, fromDate, toDate, commodity, spl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Split_List_free(spl);
    return rv;
  }

  /* only add splits to the return list whose account id matches the given one */
  while( (sp=AE_Split_List_First(spl)) ) {
    AE_Split_List_Del(sp);
    if ((accountId==0) || (AE_Split_GetAccountId(sp)==accountId))
      AE_Split_List_Add(sp, splitList);
    else
      AE_Split_free(sp);
  }

  AE_Split_List_free(spl);
  return 0;
}



int AE_Book_ReadSplitListByDateAndAccount(AE_BOOK *b,
                                          const GWEN_DATE *fromDate,
                                          const GWEN_DATE *toDate,
                                          const char *accountPattern,
                                          const char *commodity,
                                          AE_SPLIT_LIST *splitList) {
  AE_SPLIT_LIST *spl;
  int rv;
  AE_SPLIT *sp;

  spl=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndCommodity(b, fromDate, toDate, commodity, spl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Split_List_free(spl);
    return rv;
  }

  /* only add splits to the return list whose accounts match the given pattern */
  while( (sp=AE_Split_List_First(spl)) ) {
    const char *s;

    AE_Split_List_Del(sp);

    s=AE_Split_GetTmpAccountPath(sp);
    if ((accountPattern==NULL) || (s && accountPattern && -1!=GWEN_Text_ComparePattern(s, accountPattern, 0))) {
      AE_Split_List_Add(sp, splitList);
    }
    else
      AE_Split_free(sp);
  }

  AE_Split_List_free(spl);
  return 0;
}



int AE_Book_ReadSplitListForSelectedAccounts(AE_BOOK *b,
                                             const AE_ACCOUNT_TREE *at,
                                             const GWEN_DATE *fromDate,
                                             const GWEN_DATE *toDate,
                                             const char *commodity,
                                             AE_SPLIT_LIST *splitList) {
  AE_SPLIT_LIST *spl;
  int rv;
  AE_SPLIT *sp;

  spl=AE_Split_List_new();
  rv=AE_Book_ReadSplitListByDateAndCommodity(b, fromDate, toDate, commodity, spl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Split_List_free(spl);
    return rv;
  }

  /* only add splits to the return list whose account id matches the given one */
  while( (sp=AE_Split_List_First(spl)) ) {
    AQDB_ID aid;
    const AE_ACCOUNT *a=NULL;

    AE_Split_List_Del(sp);
    aid=AE_Split_GetAccountId(sp);
    if (aid)
      a=AE_Account_Tree_GetById(at, aid);
    if (a && (AE_Account_GetRuntimeFlags(a) & AE_ACCOUNT_RTFLAGS_SELECTED)) {
      AE_Split_List_Add(sp, splitList);
    }
    else
      AE_Split_free(sp);
  }

  AE_Split_List_free(spl);
  return 0;
}






int AE_Book_ReadBankStandingOrderSubObjects(AE_BOOK *b, AE_STATEMENT *st) {
  AE_RECURRENCE_LIST *rl;
  char qstring[128];
  int rv;

  /* read recurrence */
  rl=AE_Recurrence_List_new();
  snprintf(qstring, sizeof(qstring)-1, "($ownertype==%d) && ($ownerid==%d)",
           AE_Book_TableType_BankStandingOrder,
           AE_Statement_GetId(st));
  qstring[sizeof(qstring)-1]=0;

  rv=AE_Book_QueryRecurrenceList(b, AE_Book_TableType_BankStoRecurrence, 0, qstring, rl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Recurrence_List_free(rl);
    return rv;
  }
  if (AE_Recurrence_List_GetCount(rl)>0) {
    AE_RECURRENCE *rec;

    rec=AE_Recurrence_List_First(rl);
    assert(rec);
    AE_Recurrence_List_Del(rec);
    AE_Statement_SetRecurrence(st, rec);
  }

  AE_Recurrence_List_free(rl);

  /* done */
  return 0;
}




int AE_Book_ReadBankStandingOrderListSubObjects(AE_BOOK *b, AE_STATEMENT_LIST *sl) {
  AE_RECURRENCE_LIST *rList;
  AE_STATEMENT_IDMAP *sMap;
  AE_STATEMENT *st;
  AE_RECURRENCE *rec;
  int rv;

  /* sort statements into idmap */
  sMap=AE_Statement_IdMap_new(GWEN_IdMapAlgo_Hex4);
  st=AE_Statement_List_First(sl);
  while(st) {
    AE_Statement_IdMap_Insert(sMap, AE_Statement_GetId(st), st);
    st=AE_Statement_List_Next(st);
  }

  rList=AE_Recurrence_List_new();
  rv=AE_Book_ReadRecurrenceList(b, AE_Book_TableType_BankStoRecurrence, rList);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Statement_IdMap_free(sMap);
    AE_Recurrence_List_free(rList);
    return rv;
  }

  DBG_ERROR(0, "Number of recurrences: %d", AE_Recurrence_List_GetCount(rList));

  /* assign recurrences to statements */
  while( (rec=AE_Recurrence_List_First(rList)) ) {
    AQDB_ID id;

    id=AE_Recurrence_GetOwnerId(rec);
    if (id>0) {
      AE_Recurrence_List_Del(rec);
      st=AE_Statement_IdMap_Find(sMap, id);
      if (st)
        AE_Statement_SetRecurrence(st, rec);
      else
        AE_Recurrence_free(rec);
    }
    else {
      DBG_ERROR(0, "Statement with id %lu not found", (long unsigned int) id);
      AE_Recurrence_free(rec);
    }
  }

  /* cleanup */
  AE_Statement_IdMap_free(sMap);
  AE_Recurrence_List_free(rList);

  return 0;
}


