/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2009 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_group_p.h"
#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>


GWEN_LIST_FUNCTIONS(AE_GROUP, AE_Group)
GWEN_TREE_FUNCTIONS(AE_GROUP, AE_Group)




AE_GROUP *AE_Group_new() {
  AE_GROUP *gr;

  GWEN_NEW_OBJECT(AE_GROUP, gr);
  gr->refCount=1;
  GWEN_LIST_INIT(AE_GROUP, gr);
  GWEN_TREE_INIT(AE_GROUP, gr);

  return gr;
}



AE_GROUP *AE_Group_dup(const AE_GROUP *osp) {
  AE_GROUP *gr;

  assert(osp);
  assert(osp->refCount);

  gr=AE_Group_new();
  gr->id=osp->id;
  if (osp->name)
    gr->name=strdup(osp->name);
  if (osp->description)
    gr->description=strdup(osp->description);

  gr->dbId=osp->dbId;

  return gr;
}



void AE_Group_Attach(AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  gr->refCount++;
}



void AE_Group_free(AE_GROUP *gr) {
  if (gr) {
    if (gr->refCount==1) {
      GWEN_TREE_FINI(AE_GROUP, gr);
      GWEN_LIST_FINI(AE_GROUP, gr);
      free(gr->description);
      free(gr->name);
      gr->refCount=0;
      GWEN_FREE_OBJECT(gr);
    }
    else {
      gr->refCount--;
    }
  }
}



AQDB_ID AE_Group_GetId(const AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  return gr->id;
}



void AE_Group_SetId(AE_GROUP *gr, AQDB_ID id) {
  assert(gr);
  assert(gr->refCount);
  gr->id=id;
}



const char *AE_Group_GetName(const AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  return gr->name;
}



void AE_Group_SetName(AE_GROUP *gr, const char *s) {
  assert(gr);
  assert(gr->refCount);
  free(gr->name);
  if (s) gr->name=strdup(s);
  else gr->name=NULL;
}



const char *AE_Group_GetDescription(const AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  return gr->description;
}



void AE_Group_SetDescription(AE_GROUP *gr, const char *s) {
  assert(gr);
  assert(gr->refCount);
  free(gr->description);
  if (s) gr->description=strdup(s);
  else gr->description=NULL;
}



AQDB_ID AE_Group_GetParentId(const AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  return gr->parentId;
}



void AE_Group_SetParentId(AE_GROUP *gr, AQDB_ID id) {
  assert(gr);
  assert(gr->refCount);
  gr->parentId=id;
}



int AE_Group_GetType(const AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  return gr->type;
}



void AE_Group_SetType(AE_GROUP *gr, int i) {
  assert(gr);
  assert(gr->refCount);
  gr->type=i;
}



AQDB_ID AE_Group_GetDbId(const AE_GROUP *gr) {
  assert(gr);
  assert(gr->refCount);
  return gr->dbId;
}



void AE_Group_SetDbId(AE_GROUP *gr, AQDB_ID id) {
  assert(gr);
  assert(gr->refCount);
  gr->dbId=id;
}



int AE_Group_CreateTable(AQDB_DB *db, const char *tname, uint32_t flags) {
  AQDB_COLUMN_LIST *cl;
  AQDB_COLUMN *c;
  int rv;

  cl=AQDB_Column_List_new();

  c=AQDB_Column_new(AQDB_DataType_Int, "id", 8);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_String, "name", 64);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_Int, "parentid", 8);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_Int, "type", 8);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_String, "description", 128);
  AQDB_Column_List_Add(c, cl);

  rv=AQDB_DB_CreateTable(db, tname, flags, cl);
  AQDB_Column_List_free(cl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_Group_fromObject(const AQDB_OBJECT *o, AE_GROUP **pT) {
  int rv;
  int64_t v64;
  const char *s;
  AE_GROUP *gr;

  gr=AE_Group_new();
  gr->dbId=AQDB_Object_GetObjectId(o);

  rv=AQDB_Object_GetFieldInt64(o, AE_GROUP_FIELD_ID, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Group_free(gr);
    return rv;
  }
  gr->id=(AQDB_ID) v64;

  rv=AQDB_Object_GetFieldString(o, AE_GROUP_FIELD_NAME, &s);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Group_free(gr);
    return rv;
  }
  if (s && *s)
    gr->name=strdup(s);

  rv=AQDB_Object_GetFieldInt64(o, AE_GROUP_FIELD_PARENTID, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Group_free(gr);
    return rv;
  }
  gr->parentId=(AQDB_ID) v64;

  rv=AQDB_Object_GetFieldInt64(o, AE_GROUP_FIELD_TYPE, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Group_free(gr);
    return rv;
  }
  gr->type=(int) v64;

  rv=AQDB_Object_GetFieldString(o, AE_GROUP_FIELD_DESCRIPTION, &s);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Group_free(gr);
    return rv;
  }
  if (s && *s)
    gr->description=strdup(s);

  *pT=gr;
  return 0;
}



int AE_Group_toObject(const AE_GROUP *gr, AQDB_OBJECT *o) {
  int64_t v64;
  int rv;

  assert(gr);
  assert(gr->refCount);

  v64=(int64_t) gr->id;
  rv=AQDB_Object_SetFieldInt64(o, AE_GROUP_FIELD_ID, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AQDB_Object_SetFieldString(o, AE_GROUP_FIELD_NAME, gr->name);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  v64=(int64_t) gr->parentId;
  rv=AQDB_Object_SetFieldInt64(o, AE_GROUP_FIELD_PARENTID, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  v64=(int64_t) gr->type;
  rv=AQDB_Object_SetFieldInt64(o, AE_GROUP_FIELD_TYPE, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AQDB_Object_SetFieldString(o, AE_GROUP_FIELD_DESCRIPTION, gr->description);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_Group_CreateObject(AQDB_DB *db, AQDB_ID tableId, AQDB_ID id, AQDB_OBJECT **pObj) {
  AQDB_OBJECT *o;

  o=AQDB_Object_new(tableId, id, AE_GROUP_FIELD_COUNT);
  *pObj=o;
  return 0;
}











