/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_idcounter_p.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>
#include <gwenhywfar/buffer.h>

#include <assert.h>


GWEN_LIST_FUNCTIONS(AE_IDCOUNTER, AE_IdCounter)




AE_IDCOUNTER *AE_IdCounter_new() {
  AE_IDCOUNTER *ic;

  GWEN_NEW_OBJECT(AE_IDCOUNTER, ic);
  ic->refCount=1;
  GWEN_LIST_INIT(AE_IDCOUNTER, ic);

  return ic;
}



void AE_IdCounter_Attach(AE_IDCOUNTER *ic) {
  assert(ic);
  assert(ic->refCount);
  ic->refCount++;
}



AE_IDCOUNTER *AE_IdCounter_dup(const AE_IDCOUNTER *oic) {
  AE_IDCOUNTER *ic;

  assert(oic);
  assert(oic->refCount);

  ic=AE_IdCounter_new();
  ic->id=oic->id;

  if (oic->name)
    ic->name=strdup(oic->name);

  ic->lastId=oic->lastId;
  ic->dbId=oic->dbId;

  return ic;
}



void AE_IdCounter_free(AE_IDCOUNTER *ic) {
  if (ic) {
    assert(ic->refCount);
    if (ic->refCount==1) {
      GWEN_LIST_FINI(AE_IDCOUNTER, ic);
      ic->refCount=0;
      free(ic->name);
      GWEN_FREE_OBJECT(ic);
    }
    else {
      ic->refCount--;
    }
  }
}



AQDB_ID AE_IdCounter_GetId(const AE_IDCOUNTER *ic) {
  assert(ic);
  assert(ic->refCount);
  return ic->id;
}



void AE_IdCounter_SetId(AE_IDCOUNTER *ic, AQDB_ID id) {
  assert(ic);
  assert(ic->refCount);
  ic->id=id;
}



const char *AE_IdCounter_GetName(const AE_IDCOUNTER *ic) {
  assert(ic);
  assert(ic->refCount);
  return ic->name;
}



void AE_IdCounter_SetName(AE_IDCOUNTER *ic, const char *s) {
  assert(ic);
  assert(ic->refCount);
  free(ic->name);
  if (s) ic->name=strdup(s);
  else ic->name=NULL;
}



AQDB_ID AE_IdCounter_GetLastId(const AE_IDCOUNTER *ic) {
  assert(ic);
  assert(ic->refCount);
  return ic->lastId;
}



void AE_IdCounter_SetLastId(AE_IDCOUNTER *ic, AQDB_ID id) {
  assert(ic);
  assert(ic->refCount);
  ic->lastId=id;
}



AQDB_ID AE_IdCounter_GetDbId(const AE_IDCOUNTER *ic) {
  assert(ic);
  assert(ic->refCount);
  return ic->dbId;
}



void AE_IdCounter_SetDbId(AE_IDCOUNTER *ic, AQDB_ID id) {
  assert(ic);
  assert(ic->refCount);
  ic->dbId=id;
}



int AE_IdCounter_CreateTable(AQDB_DB *db,
			     const char *tname,
			     uint32_t flags) {
  AQDB_COLUMN_LIST *cl;
  AQDB_COLUMN *c;
  int rv;

  cl=AQDB_Column_List_new();

  c=AQDB_Column_new(AQDB_DataType_Int, "id", 8);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_String, "name", 64);
  AQDB_Column_List_Add(c, cl);

  c=AQDB_Column_new(AQDB_DataType_Int, "lastid", 8);
  AQDB_Column_List_Add(c, cl);

  rv=AQDB_DB_CreateTable(db, tname, flags, cl);
  AQDB_Column_List_free(cl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_IdCounter_CreateObject(AQDB_DB *db, AQDB_ID tableId, AQDB_ID id, AQDB_OBJECT **pObj) {
  AQDB_OBJECT *o;

  o=AQDB_Object_new(tableId, id, AE_IDCOUNTER_FIELD_COUNT);
  *pObj=o;
  return 0;
}



int AE_IdCounter_fromObject(const AQDB_OBJECT *o,
			    AE_IDCOUNTER **pIc) {
  int rv;
  int64_t v64;
  const char *s;
  AE_IDCOUNTER *ic;

  ic=AE_IdCounter_new();
  ic->dbId=AQDB_Object_GetObjectId(o);

  rv=AQDB_Object_GetFieldInt64(o, AE_IDCOUNTER_FIELD_ID, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_IdCounter_free(ic);
    return rv;
  }
  ic->id=(AQDB_ID) v64;

  rv=AQDB_Object_GetFieldString(o, AE_IDCOUNTER_FIELD_NAME, &s);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_IdCounter_free(ic);
    return rv;
  }
  if (s)
    ic->name=strdup(s);

  rv=AQDB_Object_GetFieldInt64(o, AE_IDCOUNTER_FIELD_LASTID, &v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_IdCounter_free(ic);
    return rv;
  }
  ic->lastId=(AQDB_ID) v64;

  *pIc=ic;
  return 0;
}



int AE_IdCounter_toObject(const AE_IDCOUNTER *ic, AQDB_OBJECT *o) {
  int64_t v64;
  int rv;

  v64=(int64_t) ic->id;
  rv=AQDB_Object_SetFieldInt64(o, AE_IDCOUNTER_FIELD_ID, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AQDB_Object_SetFieldString(o, AE_IDCOUNTER_FIELD_NAME, ic->name);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  v64=(int64_t) ic->lastId;
  rv=AQDB_Object_SetFieldInt64(o, AE_IDCOUNTER_FIELD_LASTID, v64);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}


