/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#include <aqfinance/engine/book/ae_utils.h>

#include <ctype.h>



int AE_Statement_Matches(const AE_STATEMENT *st, const char *s) {
  if (s && *s) {
    const char *t;
    int match=0;

    t=AE_Statement_GetPurpose(st);
    if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
      match=1;
    }
    if (match==0) {
      t=AE_Statement_GetMemo(st);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    if (match==0) {
      t=AE_Statement_GetRemoteName(st);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    if (match==0) {
      t=AE_Statement_GetRemoteBankCode(st);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    if (match==0) {
      t=AE_Statement_GetRemoteAccountNumber(st);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    if (match==0) {
      t=AE_Statement_GetInfoText(st);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    return match;
  }
  else {
    return 1;
  }
}




void AE_Statement_List_SelectMatching(AE_STATEMENT_LIST *sl, const char *s) {
  AE_STATEMENT *st;

  if (s && *s) {
    /* select matching statements */
    st=AE_Statement_List_First(sl);
    while(st) {
      if (AE_Statement_Matches(st, s))
	AE_Statement_AddRuntimeFlags(st, AE_STATEMENT_RTFLAGS_SELECTED);
      else
	AE_Statement_SubRuntimeFlags(st, AE_STATEMENT_RTFLAGS_SELECTED);

      st=AE_Statement_List_Next(st);
    }
  }
  else {
    /* set the SELECTED flags */
    st=AE_Statement_List_First(sl);
    while(st) {
      AE_Statement_AddRuntimeFlags(st, AE_STATEMENT_RTFLAGS_SELECTED);
      st=AE_Statement_List_Next(st);
    }
  }
}



void AE_Statement_PrepareCompare(AE_STATEMENT *st) {
  const char *s;

  s=AE_Statement_GetNormalizedPurpose(st);
  if (!(s && *s)) {
    s=AE_Statement_GetPurpose(st);
    if (s && *s) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AE_Utils_NormalizeUtf8Umlauts(s, -1, tbuf);
      AE_Statement_SetNormalizedPurpose(st, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
  }

  s=AE_Statement_GetNormalizedRemoteName(st);
  if (!(s && *s)) {
    s=AE_Statement_GetRemoteName(st);
    if (s && *s) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      AE_Utils_NormalizeUtf8Umlauts(s, -1, tbuf);
      GWEN_Text_CondenseBuffer(tbuf);
      AE_Statement_SetNormalizedRemoteName(st, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }
  }
}



int AE_Statement_Compare(const AE_STATEMENT *st1, const AE_STATEMENT *st2) {
  const char *s1;
  const char *s2;
  const GWEN_DATE *d1;
  const GWEN_DATE *d2;
  const AQDB_VALUE *v1;
  const AQDB_VALUE *v2;
  AQDB_ID i1, i2;

#define MATCH_STR(macro_mname)                           \
  s1=AE_Statement_Get##macro_mname(st1);                 \
  s2=AE_Statement_Get##macro_mname(st2);                 \
  if (s1 && *s1 && s2 && *s2 && strcasecmp(s1, s2)!=0)   \
    return 0;

#define MATCH_NSTR(macro_mname)                          \
  s1=AE_Statement_Get##macro_mname(st1);                 \
  s2=AE_Statement_Get##macro_mname(st2);                 \
  if (s1) {                                              \
    while(*s1=='0')                                      \
      s1++;                                              \
  }                                                      \
  if (s2) {                                              \
    while(*s2=='0')                                      \
      s2++;                                              \
  }                                                      \
  if (s1 && *s1 && s2 && *s2 && strcasecmp(s1, s2)!=0)   \
    return 0;

  i1=AE_Statement_GetBankAccountId(st1);
  i2=AE_Statement_GetBankAccountId(st2);
  if (i1 && i2 && i1!=i2)
    return 0;

  MATCH_STR(FiId);

  v1=AE_Statement_GetValue(st1);
  v2=AE_Statement_GetValue(st2);
  if (v1 && v2 && AQDB_Value_Compare(v1, v2)!=0)
    return 0;

  d1=AE_Statement_GetDate(st1);
  d2=AE_Statement_GetDate(st2);
  if (d1 && d2 && GWEN_Date_Compare(d1, d2)!=0)
    return 0;

  d1=AE_Statement_GetValutaDate(st1);
  d2=AE_Statement_GetValutaDate(st2);
  if (d1 && d2 && GWEN_Date_Compare(d1, d2)!=0)
    return 0;


  MATCH_STR(LocalCountry);
  MATCH_NSTR(LocalBankCode);
  MATCH_NSTR(LocalAccountNumber);
  MATCH_STR(LocalIban);
  MATCH_STR(LocalBic);

  MATCH_STR(RemoteCountry);
  MATCH_NSTR(RemoteBankCode);
  MATCH_NSTR(RemoteAccountNumber);
  MATCH_STR(RemoteIban);
  MATCH_STR(RemoteBic);

  MATCH_STR(Currency);
  MATCH_STR(NormalizedPurpose);
  MATCH_STR(NormalizedRemoteName);


#undef MATCH_NSTR
#undef MATCH_STR
  return 1;
}




int AE_Statement_toReportDb(const AE_STATEMENT *st, GWEN_DB_NODE *db) {
  int rv;

  rv=AE_Statement_toDb(st, db);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* nothing to do right now */

  return 0;
}





AF_REPORT_CFG_COLUMN_LIST *AE_Statement_CreateReportColumnList() {
  AF_REPORT_CFG_COLUMN_LIST *cl;
  AF_REPORT_CFG_COLUMN *c;

  cl=AF_Report_Cfg_Column_List_new();

  /* member "localCountry" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Local Country"));
  AF_Report_Cfg_Column_SetVarName(c, "localCountry");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "localBankCode" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Local Bank Code"));
  AF_Report_Cfg_Column_SetVarName(c, "localBankCode");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "localAccountNumber" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Local Account Number"));
  AF_Report_Cfg_Column_SetVarName(c, "localAccountNumber");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "localIban" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Local IBAN"));
  AF_Report_Cfg_Column_SetVarName(c, "localIban");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "localBic" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Local BIC"));
  AF_Report_Cfg_Column_SetVarName(c, "localBic");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "localName" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Local Name"));
  AF_Report_Cfg_Column_SetVarName(c, "localName");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "remoteCountry" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Remote Country"));
  AF_Report_Cfg_Column_SetVarName(c, "remoteCountry");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "remoteBankCode" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Remote Bank Code"));
  AF_Report_Cfg_Column_SetVarName(c, "remoteBankCode");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "remoteAccountNumber" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Remote Account Number"));
  AF_Report_Cfg_Column_SetVarName(c, "remoteAccountNumber");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "remoteIban" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Remote IBAN"));
  AF_Report_Cfg_Column_SetVarName(c, "remoteIban");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "remoteBic" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Remote BIC"));
  AF_Report_Cfg_Column_SetVarName(c, "remoteBic");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "remoteName" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Remote Name"));
  AF_Report_Cfg_Column_SetVarName(c, "remoteName");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "date" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Date);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Date"));
  AF_Report_Cfg_Column_SetVarName(c, "date");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "valutaDate" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Date);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Valuta Date"));
  AF_Report_Cfg_Column_SetVarName(c, "valutaDate");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "value" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Amount);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Amount"));
  AF_Report_Cfg_Column_SetVarName(c, "value");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "currency" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Currency"));
  AF_Report_Cfg_Column_SetVarName(c, "currency");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "fiId" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Financial Institute Id"));
  AF_Report_Cfg_Column_SetVarName(c, "fiId");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "textKey" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Int);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Text Key"));
  AF_Report_Cfg_Column_SetVarName(c, "textKey");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "textKeyExt" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Int);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Text Key Extension"));
  AF_Report_Cfg_Column_SetVarName(c, "textKeyExt");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "infoText" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Info Text"));
  AF_Report_Cfg_Column_SetVarName(c, "infoText");
  AF_Report_Cfg_Column_List_Add(c, cl);


  /* member "customerRef" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Customer Reference"));
  AF_Report_Cfg_Column_SetVarName(c, "customerRef");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "bankRef" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Bank Reference"));
  AF_Report_Cfg_Column_SetVarName(c, "bankRef");
  AF_Report_Cfg_Column_List_Add(c, cl);


  /* member "purpose" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Purpose"));
  AF_Report_Cfg_Column_SetVarName(c, "purpose");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "status" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Int);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Status"));
  AF_Report_Cfg_Column_SetVarName(c, "status");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "groupId" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_Int);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Group Id"));
  AF_Report_Cfg_Column_SetVarName(c, "groupId");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "category" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Category"));
  AF_Report_Cfg_Column_SetVarName(c, "category");
  AF_Report_Cfg_Column_List_Add(c, cl);

  /* member "memo" */
  c=AF_Report_Cfg_Column_new();
  AF_Report_Cfg_Column_SetDataType(c, AQDB_DataType_String);
  AF_Report_Cfg_Column_SetHeader(c, I18N("AE_STATEMENT|Memo"));
  AF_Report_Cfg_Column_SetVarName(c, "memo");
  AF_Report_Cfg_Column_List_Add(c, cl);

  return cl;
}







