/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2007-2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#include <aqfinance/engine/book/ae_utils.h>

#include <ctype.h>


void AE_Transaction_AddSplit(AE_TRANSACTION *t, AE_SPLIT *sp) {
  assert(t);
  assert(t->_refCount);

  if (t->splitList==NULL)
    t->splitList=AE_Split_List_new();
  AE_Split_List_Add(sp, t->splitList);
}



AQDB_VALUE *AE_Transaction_CalculateAmount(const AE_TRANSACTION *t, AQDB_ID commodityId) {
  assert(t);
  assert(t->_refCount);

  if (t->splitList) {
    const AE_SPLIT *sp;
    AQDB_VALUE *vpos;
    AQDB_VALUE *vneg;
    AQDB_VALUE *v2;

    vpos=AQDB_Value_new();
    vneg=AQDB_Value_new();

    /* add all positive amounts of the given commodity,
     * sub all negative amounts */
    sp=AE_Split_List_First(t->splitList);
    while(sp) {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(sp);
      if (v && AE_Split_GetCommodityId(sp)==commodityId) {
	if (AQDB_Value_IsNegative(v))
	  AQDB_Value_AddValue(vneg, v);
	else
	  AQDB_Value_AddValue(vpos, v);
      }
      sp=AE_Split_List_Next(sp);
    }

    /* vpos=vpos-(vneg), i.e. the result is positive and holds the double amount moved */
    AQDB_Value_SubValue(vpos, vneg);
    AQDB_Value_free(vneg);

    /* vpos=vpos/2 */
    v2=AQDB_Value_fromDouble(2.0);
    AQDB_Value_DivValue(vpos, v2);

    AQDB_Value_free(v2);

    return vpos;
  }
  else
    /* no splits, no amount */
    return NULL;
}



AQDB_VALUE *AE_Transaction_CalculateSum(const AE_TRANSACTION *t, AQDB_ID commodityId) {
  assert(t);
  assert(t->_refCount);

  if (t->splitList) {
    const AE_SPLIT *sp;
    AQDB_VALUE *vsum;

    vsum=AQDB_Value_new();

    /* add all positive amounts of the given commodity,
     * sub all negative amounts */
    sp=AE_Split_List_First(t->splitList);
    while(sp) {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(sp);
      if (v && AE_Split_GetCommodityId(sp)==commodityId)
	AQDB_Value_AddValue(vsum, v);
      sp=AE_Split_List_Next(sp);
    }

    return vsum;
  }
  else
    /* no splits, no amount */
    return NULL;
}



int AE_Transaction_SampleCommodities(const AE_TRANSACTION *t, AQDB_ID *idBuffer, int size) {
  assert(t);
  assert(t->_refCount);

  if (t->splitList) {
    const AE_SPLIT *sp;
    int cnt=0;

    sp=AE_Split_List_First(t->splitList);
    while(sp) {
      AQDB_ID id;

      id=AE_Split_GetCommodityId(sp);
      if (id) {
	int i;

	for (i=0; i<cnt;i++)
	  if (idBuffer[i]==id)
	    break;
	if (i>=cnt) {
	  /* add commodity */
	  if (cnt>=size) {
	    DBG_ERROR(0, "Too many different commodities in this transaction");
	    break;
	  }
	  else
	    idBuffer[cnt++]=id;
	}
      }
      else {
	DBG_ERROR(0, "No commodity id in amount");
      }
      sp=AE_Split_List_Next(sp);
    }

    return cnt;
  }
  else {
    /* no splits, no commodities */
    DBG_ERROR(0, "No splits");
    return 0;
  }
}



/* SOLL (positive) */
AE_SPLIT *AE_Transaction_GetMaxDebitSplit(const AE_TRANSACTION *t) {
  AE_SPLIT *spmax=NULL;

  assert(t);
  assert(t->_refCount);

  if (t->splitList) {
    const AQDB_VALUE *vmax=NULL;
    AE_SPLIT *sp;

    /* get split with most positive amount */
    sp=AE_Split_List_First(t->splitList);
    while(sp) {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(sp);
      if (v && AQDB_Value_IsPositive(v)) {
	if (vmax==NULL) {
	  vmax=v;
	  spmax=sp;
	}
	else {
	  if (AQDB_Value_Compare(vmax, v)<0) {
	    vmax=v;
	    spmax=sp;
	  }
	}
      }

      sp=AE_Split_List_Next(sp);
    }
  }

  return spmax;
}



/* HABEN (negative) */
AE_SPLIT *AE_Transaction_GetMaxCreditSplit(const AE_TRANSACTION *t) {
  AE_SPLIT *spmax=NULL;

  assert(t);
  assert(t->_refCount);

  if (t->splitList) {
    const AQDB_VALUE *vmax=NULL;
    AE_SPLIT *sp;

    /* get split with most positive amount */
    sp=AE_Split_List_First(t->splitList);
    while(sp) {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(sp);
      if (v && AQDB_Value_IsNegative(v)) {
	if (vmax==NULL) {
	  vmax=v;
	  spmax=sp;
	}
	else {
	  if (AQDB_Value_Compare(v, vmax)<0) {
	    vmax=v;
	    spmax=sp;
	  }
	}
      }

      sp=AE_Split_List_Next(sp);
    }
  }

  return spmax;
}



AE_SPLIT *AE_Transaction_GetCounterSplit(const AE_TRANSACTION *t, AQDB_ID aid) {
  AE_SPLIT *sp=NULL;

  assert(t);
  assert(t->_refCount);

  if (t->splitList) {
    if (aid>0) {
      /* get split for the given account */
      sp=AE_Split_List_First(t->splitList);
      while(sp) {
	if (AE_Split_GetAccountId(sp)==aid)
	  break;
	sp=AE_Split_List_Next(sp);
      }
    }
    else {
      sp=AE_Split_List_First(t->splitList);
      while(sp) {
	if (AE_Split_GetRuntimeFlags(sp) & AE_SPLIT_RTFLAGS_FIX)
	  break;
	sp=AE_Split_List_Next(sp);
      }
    }

    /* now get the counter split with the biggest amount */
    if (sp) {
      const AQDB_VALUE *v;

      v=AE_Split_GetAmount(sp);
      if (v) {
	if (AQDB_Value_IsZero(v) && aid>0) {
	  /* special case: value is zero */
	  sp=AE_Split_List_First(t->splitList);
	  while(sp) {
	    if (AE_Split_GetAccountId(sp)!=aid)
	      break;
	    sp=AE_Split_List_Next(sp);
	  }
	}
	else {
	  if (AQDB_Value_IsNegative(v))
	    sp=AE_Transaction_GetMaxDebitSplit(t);
	  else
	    sp=AE_Transaction_GetMaxCreditSplit(t);
	}
      }
    }

    return sp;
  }
  return NULL;
}



int AE_Transaction_Matches(const AE_TRANSACTION *tr, const char *s) {
  if (s && *s) {
    const char *t;
    int match=0;

    if (match==0) {
      t=AE_Transaction_GetMemo(tr);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    if (match==0) {
      t=AE_Transaction_GetCheckNumber(tr);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    if (match==0) {
      t=AE_Transaction_GetReceipt(tr);
      if (t && *t && AE_Utils_StrCaseStr(t, s)!=NULL) {
	match=1;
      }
    }

    return match;
  }
  else {
    return 1;
  }
}



void AE_Transaction_List_SelectMatching(AE_TRANSACTION_LIST *tl, const char *s) {
  AE_TRANSACTION *t;

  if (s && *s) {
    /* select matching statements */
    t=AE_Transaction_List_First(tl);
    while(t) {
      if (AE_Transaction_Matches(t, s))
	AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SELECTED);
      else
	AE_Transaction_SubRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SELECTED);

      t=AE_Transaction_List_Next(t);
    }
  }
  else {
    /* set the SELECTED flags */
    t=AE_Transaction_List_First(tl);
    while(t) {
      AE_Transaction_AddRuntimeFlags(t, AE_TRANSACTION_RTFLAGS_SELECTED);
      t=AE_Transaction_List_Next(t);
    }
  }
}






