/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_utils.h"
#include <aqfinance/engine/book/ae_job.h>
#include <aqfinance/engine/book/ae_vehicle.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/debug.h>

#include <ctype.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)
#define I18S(msg) msg



static const AE_FIELD_DESCRIPTION _itemFields[]={
{ "orderNumber",                I18S("Item Order Number"), I18S("Table|Order Number") },
{ "eanCode",                    I18S("Item EAN Code"),     I18S("Table|EAN Code") },
{ "code",                       I18S("Item Code"),         I18S("Table|Code") },
{ "name",                       I18S("Item Name"),         I18S("Table|Name") },
{ "memo",                       I18S("Item Memo"),         I18S("Table|Memo") },
{ "value",                      I18S("Item Amount"),       I18S("Table|Amount") },
{ "buyPriceValue",              I18S("Buy: Price"),        I18S("Table|Price") },
{ "buyTaxSumValue",             I18S("Buy: Tax Sum"),      I18S("Table|Tax Sum") },
{ "buyTaxName",                 I18S("Buy: Tax Name"),     I18S("Table|Tax") },
{ "sellPriceValue",             I18S("Sell: Price"),       I18S("Table|Price") },
{ "sellTaxSumValue",            I18S("Sell: Tax Value"),   I18S("Table|Tax Sum") },
{ "sellTaxName",                I18S("Sell: Tax Name"),    I18S("Table|Tax") },
{ NULL, NULL, NULL }
};



static const AE_FIELD_DESCRIPTION _jobFields[]={
{ "priority",          I18S("Priority")                   I18S("Table|Priority") },
{ "memo",              I18S("Memo"),                      I18S("Table|Memo") },
{ "enterDate",         I18S("Enter Date"),                I18S("Table|Enter Date") },
{ "dueDate",           I18S("Due Date"),                  I18S("Table|Due Date") },
{ "code",              I18S("Code"),                      I18S("Table|Code") },
{ "addressName",       I18S("Address: Name"),             I18S("Table|Name") },
{ "addressDept",       I18S("Address: Department"),       I18S("Table|Department") },
{ "addressStreetNum",  I18S("Address: Street/Number"),    I18S("Table|Street/Number") },
{ "addressZipCode",    I18S("Address: Zip Code"),         I18S("Table|Zip Code") },
{ "addressLocation",   I18S("Address: City"),             I18S("Table|City") },
{ "addressCountry",    I18S("Address: Country"),          I18S("Table|Country") },
{ "accountBankCode",   I18S("Account: Routing Number"),   I18S("Table|Routing Number") },
{ "accountBankName",   I18S("Account: Bank Name"),        I18S("Table|Bank Name") },
{ "accountNumber",     I18S("Account: Account Number"),   I18S("Table|Account Number") },
{ "accountName",       I18S("Account: Account Name"),     I18S("Table|Account Name") },
{ "accountBic",        I18S("Account: BIC"),              I18S("Table|BIC") },
{ "accountIban",       I18S("Account: IBAN"),             I18S("Table|IBAN") },
{ "purpose",           I18S("Purpose"),                   I18S("Table|Purpose") },
{ NULL, NULL, NULL }
};



const AE_FIELD_DESCRIPTION *AE_Utils_GetItemFieldDescriptions() {
  return _itemFields;
}



const AE_FIELD_DESCRIPTION *AE_Utils_GetJobFieldDescriptions() {
  return _jobFields;
}



const AE_FIELD_DESCRIPTION *AE_Utils_GetItemFieldDescriptionByIndex(const AE_FIELD_DESCRIPTION *fd, int idx) {
  while(fd->name && idx) {
    fd++;
  }
  if (idx==0 && fd->name)
    return fd;

  return NULL;
}



const AE_FIELD_DESCRIPTION *AE_Utils_GetItemFieldDescriptionByName(const AE_FIELD_DESCRIPTION *fd, const char *name) {
  while(fd->name) {
    if (fd->name && strcasecmp(fd->name, name)==0)
      return fd;
    fd++;
  }

  return NULL;
}



int AE_Utils_ParseValueAndUnits(const char *text,
				GWEN_BUFFER *valueBuf,
				GWEN_BUFFER *unitsBuf) {
  const char *s;

  s=text;

  /* skip blanks */
  while(*s && isblank(*s))
    s++;

  /* get units part and value part */
  if (isdigit(*s) || *s=='-' || *s=='+') {
    const char *sStart;

    /* starts with a digit, so we assume that the text start with the value */
    sStart=s;
    /* read value */
    while(isdigit(*s) || *s=='.' || *s==',' || *s=='-' || *s=='+')
      s++;
    if (s!=sStart) {
      GWEN_Buffer_AppendBytes(valueBuf, sStart, s-sStart);
      GWEN_Buffer_AppendByte(valueBuf, 0);
    }

    /* skip blanks */
    while(*s && isblank(*s))
      s++;

    /* add units */
    if (*s) {
      GWEN_Buffer_AppendString(unitsBuf, s);
    }

    GWEN_Text_CondenseBuffer(valueBuf);
    GWEN_Text_CondenseBuffer(unitsBuf);
    return 0;
  }
  else {
    const char *sStart;

    /* does not start with a digit, so we assume it starts with the units */
    sStart=s;
    while(*s && !isdigit(*s) && !isblank(*s))
      s++;

    if (s!=sStart) {
      GWEN_Buffer_AppendBytes(unitsBuf, sStart, s-sStart);
      GWEN_Buffer_AppendByte(unitsBuf, 0);
    }

    /* skip blanks */
    while(*s && isblank(*s))
      s++;

    /* read value */
    sStart=s;
    while(isdigit(*s) || *s=='.' || *s==',' || *s=='-' || *s=='+')
      s++;
    if (s!=sStart) {
      GWEN_Buffer_AppendBytes(valueBuf, sStart, s-sStart);
      GWEN_Buffer_AppendByte(valueBuf, 0);
    }

    GWEN_Text_CondenseBuffer(valueBuf);
    GWEN_Text_CondenseBuffer(unitsBuf);
    return 0;
  }
}



int AE_Utils_AmountFromString(const char *text, 
			      const AE_COMMODITY_LIST *cl,
			      AQDB_VALUE **pValue,
			      AE_COMMODITY **pCommodity) {
  GWEN_BUFFER *valueBuf;
  GWEN_BUFFER *unitsBuf;
  int rv;
  AQDB_VALUE *v;
  AE_COMMODITY *co=NULL;
  const char *s;

  valueBuf=GWEN_Buffer_new(0, 32, 0, 1);
  unitsBuf=GWEN_Buffer_new(0, 32, 0, 1);

  rv=AE_Utils_ParseValueAndUnits(text, valueBuf, unitsBuf);
  if (rv<0) {
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return rv;
  }

  v=AQDB_Value_fromString(GWEN_Buffer_GetStart(valueBuf));
  if (v==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Invalid value [%s]", GWEN_Buffer_GetStart(valueBuf));
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return GWEN_ERROR_BAD_DATA;
  }

  /* find commodity */
  s=GWEN_Buffer_GetStart(unitsBuf);
  if (s && *s) {
    co=AE_Commodity_List_First(cl);
    while(co) {
      const char *t;
      int match=0;

      if (!match) {
        t=AE_Commodity_GetSymbol(co);
        if (t && *t)
          match=(strcasecmp(s, t)==0)?1:0;
      }

      if (!match) {
        t=AE_Commodity_GetCode(co);
        if (t && *t)
          match=(strcasecmp(s, t)==0)?1:0;
      }

      if (!match) {
        t=AE_Commodity_GetName(co);
        if (t && *t)
          match=(strcasecmp(s, t)==0)?1:0;
      }

      if (match)
        break;

      co=AE_Commodity_List_Next(co);
    } /* while */
    if (co==NULL) {
      DBG_ERROR(0, "Commodity [%s] not found", s);
    }
  }

  if (co) {
    *pValue=v;
    *pCommodity=co;
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return 0;
  }
  else {
    *pValue=v;
    *pCommodity=NULL;
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return 0;
  }
}



int AE_Utils_AmountToString(const AQDB_VALUE *value,
			    AQDB_ID commodityId,
			    const AE_COMMODITY_LIST *cl,
			    GWEN_BUFFER *tbuf) {
  const AE_COMMODITY *co;

  AQDB_Value_toHumanReadableString(value, tbuf, 2);
  co=AE_Commodity_List_GetById(cl, commodityId);
  if (co) {
    const char *s;

    s=AE_Commodity_GetSymbol(co);
    if (!s || *s==0)
      s=AE_Commodity_GetCode(co);
    if (!s || *s==0)
      s=AE_Commodity_GetName(co);

    if (s && *s) {
      GWEN_Buffer_AppendString(tbuf, " ");
      GWEN_Buffer_AppendString(tbuf, s);
    }
  }

  return 0;
}



int AE_Utils_AmountToStringNoSymbol(const AQDB_VALUE *value,
				    AQDB_ID commodityId,
				    const AE_COMMODITY_LIST *cl,
				    GWEN_BUFFER *tbuf) {
  const AE_COMMODITY *co;

  AQDB_Value_toHumanReadableString(value, tbuf, 2);
  co=AE_Commodity_List_GetById(cl, commodityId);
  if (co) {
    const char *s;

    s=AE_Commodity_GetCode(co);
    if (!s || *s==0)
      s=AE_Commodity_GetName(co);

    if (s && *s) {
      GWEN_Buffer_AppendString(tbuf, " ");
      GWEN_Buffer_AppendString(tbuf, s);
    }
  }

  return 0;
}



int AE_Utils_ValueAndItemUnitsFromString(const char *text,
					 AQDB_VALUE **pValue,
					 int *pUnits) {
  GWEN_BUFFER *valueBuf;
  GWEN_BUFFER *unitsBuf;
  int rv;
  AQDB_VALUE *v;
  const char *s;
  int units;

  valueBuf=GWEN_Buffer_new(0, 32, 0, 1);
  unitsBuf=GWEN_Buffer_new(0, 32, 0, 1);

  rv=AE_Utils_ParseValueAndUnits(text, valueBuf, unitsBuf);
  if (rv<0) {
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return rv;
  }

  v=AQDB_Value_fromString(GWEN_Buffer_GetStart(valueBuf));
  if (v==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Invalid value [%s]", GWEN_Buffer_GetStart(valueBuf));
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return GWEN_ERROR_BAD_DATA;
  }

  /* determine units */
  s=GWEN_Buffer_GetStart(unitsBuf);
  if (*s) {
    if (strcasecmp(s, "L")==0)
      units=AE_Item_UnitsLiters;
    else if (strcasecmp(s, "G")==0 ||
	     strcasecmp(s, I18N("units|grammes"))==0 ||
	     strcasecmp(s, I18N("units|gramm"))==0)
      units=AE_Item_UnitsGrammes;
    else if (strcasecmp(s, "M")==0 ||
	     strcasecmp(s, I18N("units|meters"))==0)
      units=AE_Item_UnitsMeters;
    else if (strcasecmp(s, "M2")==0 ||
	     strcasecmp(s, "sqm")==0 ||
	     strcasecmp(s, "qm")==0 ||
	     strcasecmp(s, I18N("units|square meters"))==0)
      units=AE_Item_UnitsSquareMeters;
    else if (strcasecmp(s, "M3")==0 ||
	     strcasecmp(s, I18N("units|cubic meters"))==0)
      units=AE_Item_UnitsCubicMeters;
    else if (strcasecmp(s, "min")==0 ||
	     strcasecmp(s, I18N("units|minutes"))==0)
      units=AE_Item_UnitsMinutes;
    else if (strcasecmp(s, "H")==0 ||
	     strcasecmp(s, I18N("units|hours"))==0)
      units=AE_Item_UnitsHours;
    else
      units=AE_Item_UnitsUnknown;
  }
  else
    units=AE_Item_UnitsNone;

  GWEN_Buffer_free(unitsBuf);
  GWEN_Buffer_free(valueBuf);

  *pValue=v;
  *pUnits=units;
  return 0;
}



int AE_Utils_ValueAndUnitsFromString(const char *text,
                                     AQDB_VALUE **pValue,
                                     int *pUnits) {
  GWEN_BUFFER *valueBuf;
  GWEN_BUFFER *unitsBuf;
  int rv;
  AQDB_VALUE *v;
  const char *s;
  int units;

  valueBuf=GWEN_Buffer_new(0, 32, 0, 1);
  unitsBuf=GWEN_Buffer_new(0, 32, 0, 1);

  rv=AE_Utils_ParseValueAndUnits(text, valueBuf, unitsBuf);
  if (rv<0) {
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return rv;
  }

  v=AQDB_Value_fromString(GWEN_Buffer_GetStart(valueBuf));
  if (v==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Invalid value [%s]", GWEN_Buffer_GetStart(valueBuf));
    GWEN_Buffer_free(unitsBuf);
    GWEN_Buffer_free(valueBuf);
    return GWEN_ERROR_BAD_DATA;
  }

  /* determine units */
  s=GWEN_Buffer_GetStart(unitsBuf);
  if (*s) {
    if (strcasecmp(s, "L")==0)
      units=AE_Units_Liters;
    else if (strcasecmp(s, "G")==0 ||
             strcasecmp(s, "grammes")==0 ||
             strcasecmp(s, "gramm")==0 ||
             strcasecmp(s, I18N("units|grammes"))==0 ||
             strcasecmp(s, I18N("units|gramm"))==0)
      units=AE_Units_Grammes;
    else if (strcmp(s, "m")==0 ||
             strcasecmp(s, "meters")==0 ||
             strcasecmp(s, I18N("units|meters"))==0)
      units=AE_Units_Meters;
    else if (strcmp(s, "M")==0 ||
             strcasecmp(s, "miles")==0 ||
             strcasecmp(s, I18N("units|miles"))==0)
      units=AE_Units_Miles;
    else if (strcasecmp(s, "M2")==0 ||
	     strcasecmp(s, "sqm")==0 ||
             strcasecmp(s, "qm")==0 ||
             strcasecmp(s, "square meters")==0 ||
             strcasecmp(s, I18N("units|square meters"))==0)
      units=AE_Units_SquareMeters;
    else if (strcasecmp(s, "M3")==0 ||
             strcasecmp(s, "cubic meters")==0 ||
	     strcasecmp(s, I18N("units|cubic meters"))==0)
      units=AE_Units_CubicMeters;
    else if (strcasecmp(s, "min")==0 ||
             strcasecmp(s, "minutes")==0 ||
             strcasecmp(s, I18N("units|minutes"))==0)
      units=AE_Units_Minutes;
    else if (strcasecmp(s, "H")==0 ||
             strcasecmp(s, "hours")==0 ||
	     strcasecmp(s, I18N("units|hours"))==0)
      units=AE_Units_Hours;
    else if (strcasecmp(s, "km")==0)
      units=AE_Units_KiloMeters;
    else
      units=AE_Units_Unknown;
  }
  else
    units=AE_Units_None;

  GWEN_Buffer_free(unitsBuf);
  GWEN_Buffer_free(valueBuf);

  *pValue=v;
  *pUnits=units;
  return 0;
}



void AE_Utils_ValueAndUnitsToString(const AQDB_VALUE *v, int units, GWEN_BUFFER *buf, int precision) {
  if (v)
    AQDB_Value_toHumanReadableString(v, buf, precision);
  else {
    AQDB_VALUE *val;

    val=AQDB_Value_new();
    AQDB_Value_toHumanReadableString(val, buf, precision);
    AQDB_Value_free(val);
  }

  if (units>AE_Units_None) {
    const char *s;

    s=AE_Utils_UnitsToString(units);
    if (s && *s) {
      GWEN_Buffer_AppendString(buf, " ");
      GWEN_Buffer_AppendString(buf, s);
    }
  }
}



const char *AE_Utils_UnitsToString(int u) {
  const char *units="";

  switch(u) {
  case AE_Units_None:          units=""; break;
  case AE_Units_Liters:        units="L"; break;
  case AE_Units_Grammes:       units="g"; break;
  case AE_Units_Meters:        units="m"; break;
  case AE_Units_SquareMeters:  units=I18N("units|square meters"); break;
  case AE_Units_CubicMeters:   units=I18N("units|cubic meters"); break;
  case AE_Units_Minutes:       units=I18N("units|min"); break;
  case AE_Units_Hours:         units="h"; break;
  case AE_Units_Miles:         units=I18N("units|miles"); break;
  case AE_Units_KiloMeters:    units="km"; break;
  case AE_Units_Unknown:       units=""; break;
  }
  return units;
}



const char *AE_Utils_FuelType_toString(AE_VEHICLE_FUELTYPE i) {
  switch(i) {
    case AE_Vehicle_FuelType_Octane87:      return I18N("AE_Vehicle|Regular (87 octane)");
    case AE_Vehicle_FuelType_Octane88:      return I18N("AE_Vehicle|Regular (88 octane)");
    case AE_Vehicle_FuelType_Octane89:      return I18N("AE_Vehicle|Regular (89 octane)");
    case AE_Vehicle_FuelType_Octane90:      return I18N("AE_Vehicle|Regular (90 octane)");
    case AE_Vehicle_FuelType_Octane91:      return I18N("AE_Vehicle|Regular (91 octane)");
    case AE_Vehicle_FuelType_Octane95:      return I18N("AE_Vehicle|Super (95 octane)");
    case AE_Vehicle_FuelType_Octane95e10:   return I18N("AE_Vehicle|Super E10 (95 octane)");
    case AE_Vehicle_FuelType_OtherGasoline: return I18N("AE_Vehicle|Other Gasoline");
    case AE_Vehicle_FuelType_Diesel:        return I18N("AE_Vehicle|Diesel");
    case AE_Vehicle_FuelType_Gas:           return I18N("AE_Vehicle|Diesel");
    default:                                return I18N("AE_Vehicle|unknown");
  }
}




const char *AE_Utils_ItemUnitsToString(int u) {
  const char *units="";

  switch(u) {
  case AE_Item_UnitsNone:          units=""; break;
  case AE_Item_UnitsLiters:        units="l"; break;
  case AE_Item_UnitsGrammes:       units="g"; break;
  case AE_Item_UnitsMeters:        units="m"; break;
  case AE_Item_UnitsSquareMeters:  units=I18N("units|square meters"); break;
  case AE_Item_UnitsCubicMeters:   units=I18N("units|cubic meters"); break;
  case AE_Item_UnitsMinutes:       units=I18N("units|min"); break;
  case AE_Item_UnitsHours:         units="h"; break;
  case AE_Item_UnitsUnknown:       units=""; break;
  }
  return units;
}



const char *AE_Utils_StrCaseStr(const char *haystack, const char *needle) {
  while(*haystack) {
    while(*haystack && tolower(*haystack)!=tolower(*needle))
      haystack++;
    if (*haystack) {
      const char *t;
      const char *s;

      /* now haystack points to an area which begins with *needle */
      t=haystack;
      s=needle;
      t++;
      s++;
      while(*t && *s && (tolower(*t)==tolower(*s))) {
	t++;
	s++;
      }
      if (*s==0)
	return haystack;
    }
    else
      /* not found */
      break;
    /* no match here, advance haystack */
    haystack++;
  }

  /* not found */
  return NULL;
}



void AE_Utils_NormalizeUtf8Umlauts(const char *p, int size, GWEN_BUFFER *buf) {
  while(*p) {
    unsigned int c;

    if (!size)
      break;

    c=(unsigned char)(*(p++));
    if (c==0xc3) {
      if (size!=-1)
        size--;
      if (!size) {
        DBG_ERROR(AE_LOGDOMAIN, "Incomplete UTF-8 sequence");
        break;
      }
      c=(unsigned char)(*(p++));
      switch(c) {
      case 0x84:
      case 0xa4:
	GWEN_Buffer_AppendString(buf, "AE");
        break;
      case 0x96:
      case 0xb6:
	GWEN_Buffer_AppendString(buf, "OE");
        break;
      case 0x9c:
      case 0xbc:
	GWEN_Buffer_AppendString(buf, "UE");
	break;
      case 0x9f:
	GWEN_Buffer_AppendString(buf, "SS");
	break;
      default:
        GWEN_Buffer_AppendString(buf, " ");
	break;
      } /* switch */
    }
    else {
      c=toupper(c);
      if (!(isdigit(c) ||
	    (c>='A' && c<='Z') ||
	    (c>='a' && c<='z') ||
	    (strchr(" .,&-+*%/$", c))))
        c=' ';
      GWEN_Buffer_AppendByte(buf, c);
    }
    if (size!=-1)
      size--;
  } /* while */
}



int AE_Utils_Replace_Vars(const char *s, GWEN_BUFFER *dbuf, GWEN_DB_NODE *dbValues) {
  while(*s) {
    const char *t;

    t=strchr(s, '$');
    if (t) {
      if (t>s) {
	/* "$" found, append string so far */
	GWEN_Buffer_AppendBytes(dbuf, s, (t-s));
	s=t;
      }

      t++;
      if (*t=='(') {
	const char *tend;

        t++;
	tend=strchr(t, ')');
	if (tend) {
	  char vname[256];
          int len;

	  /* var name is now between t and tend */
	  len=tend-t;
	  if (len<sizeof(vname)) {
	    char *defValue;
	    char *params;   /* e.g. "DYYYY/MM/DD", "TYYYY" */
            const char *r;

	    memmove(vname, t, len);
	    vname[len]=0;
	    defValue=strchr(vname, '?');
	    if (defValue)
	      *(defValue++)=0;
	    params=strchr(defValue?defValue:vname, ':');
	    if (params)
	      *(params++)=0;

            //DBG_ERROR(0, "Got var [%s]", vname);
	    r=GWEN_DB_GetCharValue(dbValues, vname, 0, defValue);
	    if (r) {
	      if (params) {
		if (*params=='D') {
		  GWEN_DATE *date=NULL;

                  /* this is a date */
		  if (strcasecmp(r, "today")==0)
		    date=GWEN_Date_CurrentDate();
		  else {
		    date=GWEN_Date_fromString(r);
		    if (date==NULL) {
		      DBG_ERROR(AE_LOGDOMAIN, "Bad date: [%s]", r);
                      return GWEN_ERROR_BAD_DATA;
		    }
		  }
		  GWEN_Date_toStringWithTemplate(date, params+1, dbuf);
                  GWEN_Date_free(date);
		}
		else
		  GWEN_Buffer_AppendString(dbuf, r);
	      }
	      else
		GWEN_Buffer_AppendString(dbuf, r);
	      s=tend+1;
	    }
	    else {
	      GWEN_Buffer_AppendByte(dbuf, '$');
	      s++;
	    }
	  }
	  else {
	    GWEN_Buffer_AppendByte(dbuf, '$');
	    s++;
	  }
	}
	else {
	  GWEN_Buffer_AppendByte(dbuf, '$');
          s++;
	}
      }
      else {
	GWEN_Buffer_AppendByte(dbuf, '$');
	s++;
      }
    }
    else {
      GWEN_Buffer_AppendString(dbuf, s);
      break;
    }
  }

  return 0;
}






