/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) 2010-2011 by Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_import_qbm_p.h"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/gui.h>
#include <gwenhywfar/directory.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>




/* TODO: read categories, read transactions
 * - "income:/category"
 * - "expense:/category"
 * AB_Transaction_ClearCategory(t);
 * AB_Transaction_AddCategory(t, category, 0);
 */



int AE_Book_ImportQbm_ReadCategory(GWEN_DB_NODE *dbCat,
				   QBM_CATEGORY_IDMAP *catMap,
				   QBM_CATEGORY_TREE *cats,
				   QBM_CATEGORY *parent) {
  QBM_CATEGORY *cat;

  cat=QBM_Category_fromDb(dbCat);
  if (cat) {
    GWEN_DB_NODE *dbCats;
    uint32_t catId;
    const char *s1;
    const char *s2;

    catId=QBM_Category_GetId(cat);

    /* add to tree */
    if (parent)
      QBM_Category_Tree_AddChild(parent, cat);
    else
      QBM_Category_Tree_Add(cats, cat);

    /* insert into idmap */
    QBM_Category_IdMap_Insert(catMap, catId, cat);

    /* generate path */
    if (parent)
      s1=QBM_Category_GetPath(parent);
    else
      s1=NULL;
    s2=QBM_Category_GetName(cat);
    if (s2 && *s2) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      if (s1) {
	GWEN_Buffer_AppendString(tbuf, s1);
	GWEN_Buffer_AppendString(tbuf, "/");
      }
      GWEN_Buffer_AppendString(tbuf, s2);

      QBM_Category_SetPath(cat, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    /* read sub categories */
    dbCats=GWEN_DB_GetGroup(dbCat, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "categories");
    if (dbCats) {
      GWEN_DB_NODE *dbC;

      dbC=GWEN_DB_FindFirstGroup(dbCats, "category");
      while(dbC) {
	AE_Book_ImportQbm_ReadCategory(dbC, catMap, cats, cat);
	dbC=GWEN_DB_FindNextGroup(dbC, "category");
      }
    }
  }

  return 0;
}



int AE_Book_ImportQbm_ReadCategories(const char *abpath,
				     QBM_CATEGORY_IDMAP *catMap,
				     QBM_CATEGORY_TREE *cats) {
  GWEN_BUFFER *fbuf;
  int rv;
  GWEN_DB_NODE *dbCats;
  GWEN_DB_NODE *dbC;

  fbuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(fbuf, abpath);
  GWEN_Buffer_AppendString(fbuf,
			   GWEN_DIR_SEPARATOR_S
			   "apps"
			   GWEN_DIR_SEPARATOR_S
                           "qbankmanager/data/categories.db");

  dbCats=GWEN_DB_Group_new("categories");
  rv=GWEN_DB_ReadFile(dbCats, GWEN_Buffer_GetStart(fbuf), GWEN_PATH_FLAGS_CREATE_GROUP);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ShowError(I18N("IO Error"), I18N("Error reading QBankManager category file (%d)"), rv);
    GWEN_DB_Group_free(dbCats);
    GWEN_Buffer_free(fbuf);
    return rv;
  }

  dbC=GWEN_DB_FindFirstGroup(dbCats, "categories");
  if (dbC)
    dbC=GWEN_DB_FindFirstGroup(dbC, "category");
  while(dbC) {
    AE_Book_ImportQbm_ReadCategory(dbC, catMap, cats, NULL);
    dbC=GWEN_DB_FindNextGroup(dbC, "category");
  }
  GWEN_DB_Group_free(dbCats);
  GWEN_Buffer_free(fbuf);

  return 0;
}



int AE_Book_ImportQbm_ReadTransactions(const char *abpath,
				       AB_IMEXPORTER_CONTEXT *ctx) {
  GWEN_BUFFER *fbuf;
  int rv;
  GWEN_STRINGLIST *flist;

  fbuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(fbuf, abpath);
  GWEN_Buffer_AppendString(fbuf,
			   GWEN_DIR_SEPARATOR_S
			   "apps"
			   GWEN_DIR_SEPARATOR_S
			   "qbankmanager/data/transactions");

  flist=GWEN_StringList_new();
  rv=GWEN_Directory_GetMatchingFilesRecursively(GWEN_Buffer_GetStart(fbuf), flist, "*.trans");
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ShowError(I18N("IO Error"), I18N("Error reading QBankManager transaction list (%d)"), rv);
    GWEN_StringList_free(flist);
    GWEN_Buffer_free(fbuf);
    return rv;
  }

  if (GWEN_StringList_Count(flist)) {
    QBM_CATEGORY_IDMAP *catMap;
    QBM_CATEGORY_TREE *cats;
    GWEN_STRINGLISTENTRY *se;
    uint32_t pid;

    catMap=QBM_Category_IdMap_new(GWEN_IdMapAlgo_Hex4);
    cats=QBM_Category_Tree_new();

    /* read categories */
    rv=AE_Book_ImportQbm_ReadCategories(abpath, catMap, cats);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      QBM_Category_IdMap_free(catMap);
      QBM_Category_Tree_free(cats);
      GWEN_StringList_free(flist);
      GWEN_Buffer_free(fbuf);
      return rv;
    }

    // create progress
    pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_SHOW_PROGRESS |
                               GWEN_GUI_PROGRESS_SHOW_ABORT |
                               GWEN_GUI_PROGRESS_ALLOW_EMBED,
                               I18N("Reading Transactions"),
                               I18N("Reading bank statement files"),
                               GWEN_StringList_Count(flist),
                               0);

    /* read transactions */
    se=GWEN_StringList_FirstEntry(flist);
    while(se) {
      const char *s;

      s=GWEN_StringListEntry_Data(se);
      if (s) {
	GWEN_DB_NODE *dbDay;

	dbDay=GWEN_DB_Group_new("day");
	rv=GWEN_DB_ReadFile(dbDay, s, GWEN_PATH_FLAGS_CREATE_GROUP);
	if (rv<0) {
	  /* TODO: log error */
	}
	else {
          GWEN_DB_NODE *dbT;

	  /* read transactions */
	  dbT=GWEN_DB_FindFirstGroup(dbDay, "transaction");
	  while(dbT) {
	    AB_TRANSACTION *t;
	    GWEN_DB_NODE *dbD;
	    int x;
  
	    t=AB_Transaction_new();
  
	    x=GWEN_DB_GetIntValue(dbT, "transactionId", 0, 0);
	    AB_Transaction_SetUniqueId(t, x);
  
	    s=GWEN_DB_GetCharValue(dbT, "date", 0, NULL);
	    if (s && *s) {
	      GWEN_BUFFER *xbuf;
	      GWEN_TIME *ti;
  
	      xbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      GWEN_Buffer_AppendString(xbuf, s);
	      GWEN_Buffer_AppendString(xbuf, "-12:00");
  
	      ti=GWEN_Time_fromUtcString(GWEN_Buffer_GetStart(xbuf), "YYYYMMDD-hh:mm");
	      if (ti) {
		AB_Transaction_SetDate(t, ti);
		GWEN_Time_free(ti);
	      }
	      GWEN_Buffer_free(xbuf);
	    }
  
	    s=GWEN_DB_GetCharValue(dbT, "valutaDate", 0, NULL);
	    if (s && *s) {
	      GWEN_BUFFER *xbuf;
	      GWEN_TIME *ti;
  
	      xbuf=GWEN_Buffer_new(0, 256, 0, 1);
	      GWEN_Buffer_AppendString(xbuf, s);
	      GWEN_Buffer_AppendString(xbuf, "-12:00");
  
	      ti=GWEN_Time_fromUtcString(GWEN_Buffer_GetStart(xbuf), "YYYYMMDD-hh:mm");
	      if (ti) {
		AB_Transaction_SetValutaDate(t, ti);
		GWEN_Time_free(ti);
	      }
	      GWEN_Buffer_free(xbuf);
	    }
  
	    dbD=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "value");
	    if (dbD) {
	      AB_VALUE *v;
  
	      v=AB_Value_fromDb(dbD);
	      if (v) {
		AB_Transaction_SetValue(t, v);
		AB_Value_free(v);
	      }
	    }
  
	    x=GWEN_DB_GetIntValue(dbT, "textKey", 0, 51);
	    AB_Transaction_SetTextKey(t, x);
  
	    s=GWEN_DB_GetCharValue(dbT, "primanota", 0, NULL);
	    AB_Transaction_SetPrimanota(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "fiId", 0, NULL);
	    AB_Transaction_SetFiId(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "bankRef", 0, NULL);
	    AB_Transaction_SetBankReference(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "custRef", 0, NULL);
	    AB_Transaction_SetCustomerReference(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "localBankCode", 0, NULL);
	    AB_Transaction_SetLocalBankCode(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "localAccountNumber", 0, NULL);
	    AB_Transaction_SetLocalAccountNumber(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "localIban", 0, NULL);
	    AB_Transaction_SetLocalIban(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "localBic", 0, NULL);
	    AB_Transaction_SetLocalBic(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "remoteBankCode", 0, NULL);
	    AB_Transaction_SetRemoteBankCode(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "remoteAccountNumber", 0, NULL);
	    AB_Transaction_SetRemoteAccountNumber(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "remoteIban", 0, NULL);
	    AB_Transaction_SetRemoteIban(t, s);
  
	    s=GWEN_DB_GetCharValue(dbT, "remoteBic", 0, NULL);
	    AB_Transaction_SetRemoteBic(t, s);
  
	    /* remote name */
	    for (x=0; x<99; x++) {
	      s=GWEN_DB_GetCharValue(dbT, "remoteName", x, NULL);
	      if (!s || !*s)
		break;
	      AB_Transaction_AddRemoteName(t, s, 0);
	    }
  
	    /* purpose */
	    for (x=0; x<99; x++) {
	      s=GWEN_DB_GetCharValue(dbT, "purpose", x, NULL);
	      if (!s || !*s)
		break;
	      AB_Transaction_AddPurpose(t, s, 0);
	    }

	    /* handle category */
            AB_Transaction_ClearCategory(t);
	    x=GWEN_DB_GetIntValue(dbT, "category", 0, 0);
	    if (x>0) {
	      QBM_CATEGORY *cat;

	      cat=QBM_Category_IdMap_Find(catMap, x);
	      if (cat) {
		const char *cpath;

		cpath=QBM_Category_GetPath(cat);
		if (cpath) {
		  GWEN_BUFFER *cbuf;

		  cbuf=GWEN_Buffer_new(0, 256, 0, 1);
		  if (QBM_Category_GetIsIncome(cat))
		    GWEN_Buffer_AppendString(cbuf, "income:");
		  else
		    GWEN_Buffer_AppendString(cbuf, "expense:");
		  GWEN_Buffer_AppendString(cbuf, cpath);
		  AB_Transaction_AddCategory(t, GWEN_Buffer_GetStart(cbuf), 1);
		  GWEN_Buffer_free(cbuf);
		}
	      }
	    }

	    /* add transaction */
            AB_ImExporterContext_AddTransaction(ctx, t);
  
	    dbT=GWEN_DB_FindNextGroup(dbT, "transaction");
	  } /* while t */
        }

	GWEN_DB_Group_free(dbDay);
      }

      rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
      if (rv==GWEN_ERROR_USER_ABORTED) {
        DBG_ERROR(AE_LOGDOMAIN, "User aborted");
        QBM_Category_IdMap_free(catMap);
        QBM_Category_Tree_free(cats);
        GWEN_Buffer_free(fbuf);
        GWEN_Gui_ProgressEnd(pid);
        return rv;
      }

      se=GWEN_StringListEntry_Next(se);
    }
    GWEN_Gui_ProgressEnd(pid);

    /* done */
    QBM_Category_IdMap_free(catMap);
    QBM_Category_Tree_free(cats);
  }
  else {
    DBG_ERROR(0, "No transactions");
  }

  GWEN_Buffer_free(fbuf);
  return 0;
}




int AE_Book_ImportQbm_ReadSettings(const char *abpath, AB_IMEXPORTER_CONTEXT *ctx) {
  GWEN_BUFFER *fbuf;
  int rv;
  GWEN_DB_NODE *dbSettings;
  GWEN_DB_NODE *dbAccounts;

  fbuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(fbuf, abpath);
  GWEN_Buffer_AppendString(fbuf,
			   GWEN_DIR_SEPARATOR_S
			   "apps"
			   GWEN_DIR_SEPARATOR_S
			   "qbankmanager/settings.conf");

  dbSettings=GWEN_DB_Group_new("settings");
  rv=GWEN_DB_ReadFile(dbSettings, GWEN_Buffer_GetStart(fbuf), GWEN_PATH_FLAGS_CREATE_GROUP);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ShowError(I18N("IO Error"), I18N("Error reading QBankManager settings (%d)"), rv);
    GWEN_DB_Group_free(dbSettings);
    GWEN_Buffer_free(fbuf);
    return rv;
  }

  dbAccounts=GWEN_DB_GetGroup(dbSettings, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "prg/accounts");
  if (dbAccounts) {
    GWEN_DB_NODE *dbAcc;

    dbAcc=GWEN_DB_FindFirstGroup(dbAccounts, "account");
    while(dbAcc) {
      const char *bankCode;
      const char *accountId;
      AB_IMEXPORTER_ACCOUNTINFO *ai;
      GWEN_DB_NODE *dbTransfers;
      const char *s;

      bankCode=GWEN_DB_GetCharValue(dbAcc, "bankCode", 0, "");
      accountId=GWEN_DB_GetCharValue(dbAcc, "accountNumber", 0, "");
      ai=AB_ImExporterContext_GetAccountInfo(ctx, bankCode, accountId);

      s=GWEN_DB_GetCharValue(dbAcc, "iban", 0, NULL);
      if (s && *s)
        AB_ImExporterAccountInfo_SetIban(ai, s);

      s=GWEN_DB_GetCharValue(dbAcc, "bic", 0, NULL);
      if (s && *s)
        AB_ImExporterAccountInfo_SetBic(ai, s);

      s=GWEN_DB_GetCharValue(dbAcc, "ownerName", 0, NULL);
      if (s && *s)
        AB_ImExporterAccountInfo_SetOwner(ai, s);

      s=GWEN_DB_GetCharValue(dbAcc, "accountName", 0, NULL);
      if (s && *s)
        AB_ImExporterAccountInfo_SetAccountName(ai, s);

      s=GWEN_DB_GetCharValue(dbAcc, "bankName", 0, NULL);
      if (s && *s)
        AB_ImExporterAccountInfo_SetBankName(ai, s);

      dbTransfers=GWEN_DB_GetGroup(dbAcc, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "transfers");
      if (dbTransfers) {
	GWEN_DB_NODE *dbT;

	dbT=GWEN_DB_FindFirstGroup(dbTransfers, "transfer");
	while(dbT) {
	  AB_TRANSACTION *t;
	  GWEN_DB_NODE *dbD;
	  int x;

	  t=AB_Transaction_new();

          x=GWEN_DB_GetIntValue(dbT, "transactionId", 0, 0);
          AB_Transaction_SetUniqueId(t, x);

	  s=GWEN_DB_GetCharValue(dbT, "date", 0, NULL);
	  if (s && *s) {
	    GWEN_BUFFER *xbuf;
	    GWEN_TIME *ti;

	    xbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    GWEN_Buffer_AppendString(xbuf, s);
	    GWEN_Buffer_AppendString(xbuf, "-12:00");

	    ti=GWEN_Time_fromUtcString(GWEN_Buffer_GetStart(xbuf), "YYYYMMDD-hh:mm");
	    if (ti) {
	      AB_Transaction_SetDate(t, ti);
	      GWEN_Time_free(ti);
	    }
	    GWEN_Buffer_free(xbuf);
	  }

	  s=GWEN_DB_GetCharValue(dbT, "valutaDate", 0, NULL);
	  if (s && *s) {
	    GWEN_BUFFER *xbuf;
	    GWEN_TIME *ti;

	    xbuf=GWEN_Buffer_new(0, 256, 0, 1);
	    GWEN_Buffer_AppendString(xbuf, s);
	    GWEN_Buffer_AppendString(xbuf, "-12:00");

	    ti=GWEN_Time_fromUtcString(GWEN_Buffer_GetStart(xbuf), "YYYYMMDD-hh:mm");
	    if (ti) {
	      AB_Transaction_SetValutaDate(t, ti);
	      GWEN_Time_free(ti);
	    }
	    GWEN_Buffer_free(xbuf);
	  }

	  dbD=GWEN_DB_GetGroup(dbT, GWEN_PATH_FLAGS_NAMEMUSTEXIST, "value");
	  if (dbD) {
	    AB_VALUE *v;

	    v=AB_Value_fromDb(dbD);
	    if (v) {
	      AB_Transaction_SetValue(t, v);
	      AB_Value_free(v);
	    }
	  }

	  x=GWEN_DB_GetIntValue(dbT, "textKey", 0, 51);
	  AB_Transaction_SetTextKey(t, x);

          s=GWEN_DB_GetCharValue(dbT, "primanota", 0, NULL);
	  AB_Transaction_SetPrimanota(t, s);

          s=GWEN_DB_GetCharValue(dbT, "fiId", 0, NULL);
          AB_Transaction_SetFiId(t, s);

          s=GWEN_DB_GetCharValue(dbT, "bankRef", 0, NULL);
          AB_Transaction_SetBankReference(t, s);

          s=GWEN_DB_GetCharValue(dbT, "custRef", 0, NULL);
          AB_Transaction_SetCustomerReference(t, s);

	  s=GWEN_DB_GetCharValue(dbT, "localBankCode", 0, NULL);
	  AB_Transaction_SetLocalBankCode(t, s);

	  s=GWEN_DB_GetCharValue(dbT, "localAccountNumber", 0, NULL);
	  AB_Transaction_SetLocalAccountNumber(t, s);

          s=GWEN_DB_GetCharValue(dbT, "localIban", 0, NULL);
	  AB_Transaction_SetLocalIban(t, s);

          s=GWEN_DB_GetCharValue(dbT, "localBic", 0, NULL);
	  AB_Transaction_SetLocalBic(t, s);

	  s=GWEN_DB_GetCharValue(dbT, "remoteBankCode", 0, NULL);
	  AB_Transaction_SetRemoteBankCode(t, s);

	  s=GWEN_DB_GetCharValue(dbT, "remoteAccountNumber", 0, NULL);
	  AB_Transaction_SetRemoteAccountNumber(t, s);

          s=GWEN_DB_GetCharValue(dbT, "remoteIban", 0, NULL);
	  AB_Transaction_SetRemoteIban(t, s);

          s=GWEN_DB_GetCharValue(dbT, "remoteBic", 0, NULL);
	  AB_Transaction_SetRemoteBic(t, s);

	  /* remote name */
	  for (x=0; x<99; x++) {
	    s=GWEN_DB_GetCharValue(dbT, "remoteName", x, NULL);
	    if (!s || !*s)
	      break;
            AB_Transaction_AddRemoteName(t, s, 0);
          }

	  /* purpose */
	  for (x=0; x<99; x++) {
	    s=GWEN_DB_GetCharValue(dbT, "purpose", x, NULL);
	    if (!s || !*s)
	      break;
	    AB_Transaction_AddPurpose(t, s, 0);
	  }

	  /* add transaction */
          AB_ImExporterAccountInfo_AddTransfer(ai, t);

	  dbT=GWEN_DB_FindNextGroup(dbT, "transfer");
	}
      }

      dbAcc=GWEN_DB_FindNextGroup(dbAcc, "account");
    }
  }
  GWEN_DB_Group_free(dbSettings);

  return 0;
}



int AE_Book_ImportQbm(AE_BOOK *b, const char *abpath, AB_IMEXPORTER_CONTEXT *ctx) {
  int rv;

  /* import accounts and transfers */
  rv=AE_Book_ImportQbm_ReadSettings(abpath, ctx);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* import transactions */
  rv=AE_Book_ImportQbm_ReadTransactions(abpath, ctx);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* DEBUG */
  if (1) {
    GWEN_DB_NODE *dbTemp;

    dbTemp=GWEN_DB_Group_new("context");
    AB_ImExporterContext_toDb(ctx, dbTemp);
    GWEN_DB_WriteFile(dbTemp, "/tmp/qbm.ctx", GWEN_DB_FLAGS_DEFAULT);
    GWEN_DB_Group_free(dbTemp);
  }

  return 0;
}


