/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_openbills_p.h"

#include <aqfinance/engine/book/ae_task.h>

#include <gwenhywfar/idmap.h>



/* Get task id and store it in the given IDMAP.
 * Please note that we kinda abuse the IDMAP, since we don't actually store
 * pointers here but rather insert "-1" for every id encountered.
 * We don't need the tasks themselves but rather want to get a list of all
 * open task and store this list in a way of very fast access.
 */
static void GWENHYWFAR_CB *AE_Book__GetOpenTasks_Callback(AE_BOOK *bk,
							  AE_BOOK_TABLE_TYPE tt,
							  AQDB_OBJECT *o,
							  void *data) {
  GWEN_IDMAP *taskMap;
  AE_TASK *tk=NULL;
  int rv;

  taskMap=(GWEN_IDMAP*) data;

  DBG_ERROR(0, "Got a task");

  rv=AE_Task_fromObject(o, &tk);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AQDB_ID tid;

    tid=AE_Task_GetId(tk);
    GWEN_IdMap_Insert(taskMap, tid, (void*) (-1));

    AE_Task_free(tk);
  }

  return NULL;
}



/* for every job: add it if it is a payment or invoice belonging to one of the
 * open tasks.
 */
static void GWENHYWFAR_CB *AE_Book__GetJobsForOpenTasks_Callback(AE_BOOK *bk,
								 AE_BOOK_TABLE_TYPE tt,
								 AQDB_OBJECT *o,
								 void *data) {
  struct GETOPENBILLS_STRUCT *obStruct;
  AE_JOB *j=NULL;
  int rv;

  obStruct=(struct GETOPENBILLS_STRUCT*) data;

  rv=AE_Job_fromObject(o, &j);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    int jt;

    jt=AE_Job_GetType(j);
    if (jt==AE_Job_TypeInvoice || jt==AE_Job_TypePayment) {
      AQDB_ID tid;

      tid=AE_Job_GetTaskId(j);
      if (GWEN_IdMap_Find(obStruct->taskMap, tid)!=NULL) {
	AE_Job_List_Add(j, obStruct->jobList);
	AE_Job_IdMap_Insert(obStruct->jobMap, AE_Job_GetId(j), j);
      }
      else
        AE_Job_free(j);
    }
  }

  return NULL;
}



/* for every item: add it's value if it belongs to a job in our list.
 */
static void GWENHYWFAR_CB *AE_Book__HandleItemsForOpenTasks_Callback(AE_BOOK *bk,
								     AE_BOOK_TABLE_TYPE tt,
								     AQDB_OBJECT *o,
								     void *data) {
  struct GETOPENBILLS_STRUCT *obStruct;
  AE_ITEM *item=NULL;
  int rv;

  obStruct=(struct GETOPENBILLS_STRUCT*) data;

  rv=AE_Item_fromObject(o, &item);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AQDB_ID jid;

    jid=AE_Item_GetJobId(item);
    if (jid>0) {
      AE_JOB *j;

      j=AE_Job_IdMap_Find(obStruct->jobMap, jid);
      if (j) {
	AQDB_VALUE *v;
	const AQDB_VALUE *vt;

        DBG_ERROR(0, "Found job");
	/* add total sum */
	v=AE_Job_GetTotalSum(j);
	if (j==NULL) {
	  v=AQDB_Value_new();
          AE_Job_SetTotalSum(j, v);
	}

	if (obStruct->isBuy) {
	  vt=AE_Item_GetBuyPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetBuyTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);

	  /* add open sum */
	  v=AE_Job_GetOpenSum(j);
	  if (j==NULL) {
	    v=AQDB_Value_new();
	    AE_Job_SetOpenSum(j, v);
	  }
	  vt=AE_Item_GetBuyPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetBuyTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	}
	else {
	  vt=AE_Item_GetSellPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetSellTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);

	  /* add open sum */
	  v=AE_Job_GetOpenSum(j);
	  if (j==NULL) {
	    v=AQDB_Value_new();
	    AE_Job_SetOpenSum(j, v);
	  }
	  vt=AE_Item_GetSellPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetSellTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	}

	/* for payments: sub item from open value */
	if (AE_Job_GetType(j)==AE_Job_TypePayment) {
	  AQDB_ID refId;

	  refId=AE_Job_GetReferenceId(j);
	  if (refId) {
	    AE_JOB *rj;

	    rj=AE_Job_IdMap_Find(obStruct->jobMap, refId);
	    if (rj) {
	      AQDB_VALUE *v;
	      const AQDB_VALUE *vt;

	      DBG_ERROR(0, "Found reference job");
	      v=AE_Job_GetOpenSum(j);
	      if (v==NULL) {
		v=AQDB_Value_new();
		AE_Job_SetOpenSum(rj, v);
	      }

	      if (obStruct->isBuy) {
		vt=AE_Item_GetBuyPriceValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
		vt=AE_Item_GetBuyTaxSumValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
	      }
	      else {
		vt=AE_Item_GetSellPriceValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
		vt=AE_Item_GetSellTaxSumValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
	      }
	    }
	  }
	}
      }
      AE_Item_free(item);
    }
    else {
      DBG_ERROR(0, "Item's job id is 0");
    }
  }

  return NULL;
}



int AE_Book_GetOpenBills(AE_BOOK *b, int isBuy, AE_JOB_LIST *jobList) {
  int rv;
  void *dummy=NULL;
  char qexpr[64];
  struct GETOPENBILLS_STRUCT obStruct;
  AE_JOB *j;

  obStruct.isBuy=isBuy;

  /* we use the idmap only to store the fact that an id exists, we don't actually use
   * the void* pointer.
   */
  obStruct.taskMap=GWEN_IdMap_new(GWEN_IdMapAlgo_Hex4);

  /* read ids of all open tasks */
  /* only load tasks which are active and whose  flag matches */
  if (isBuy)
    snprintf(qexpr, sizeof(qexpr)-1,
	     "$flags ?areset %d",
	     AE_TASK_FLAGS_ACTIVE | AE_TASK_FLAGS_INCOMING);
  else
    snprintf(qexpr, sizeof(qexpr)-1,
	     "($flags ?isset %d) && ($flags ?isclear %d)",
	     AE_TASK_FLAGS_ACTIVE, AE_TASK_FLAGS_INCOMING);
  qexpr[sizeof(qexpr)-1]=0;
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Task,
				 0,
				 qexpr,
				 AE_Book__GetOpenTasks_Callback,
				 (void*) (obStruct.taskMap),
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
    GWEN_IdMap_free(obStruct.taskMap);
    return 0;
  }

  /* now read all invoices and payments for the active tasks */
  obStruct.jobList=AE_Job_List_new();
  obStruct.jobMap=AE_Job_IdMap_new(GWEN_IdMapAlgo_Hex4);

  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Job,
				 0,
				 NULL, /* no query expression */
				 AE_Book__GetJobsForOpenTasks_Callback,
				 (void*) &obStruct,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
    AE_Job_List_free(obStruct.jobList);
    AE_Job_IdMap_free(obStruct.jobMap);
    GWEN_IdMap_free(obStruct.taskMap);
    return 0;
  }

  /* now calculate the totalSum and openSum for all invoices */
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Item,
				 0,
				 NULL, /* no query expression */
                                 AE_Book__HandleItemsForOpenTasks_Callback,
				 (void*) &obStruct,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
    AE_Job_List_free(obStruct.jobList);
    AE_Job_IdMap_free(obStruct.jobMap);
    GWEN_IdMap_free(obStruct.taskMap);
    return 0;
  }

  /* now filter out all jobs which are invoices paid in full and add them to the caller's list */
  while( (j=AE_Job_List_First(obStruct.jobList)) ) {
    /* remove from list no matter what */
    AE_Job_List_Del(j);
    /* delete payments */
    if (AE_Job_GetType(j)!=AE_Job_TypeInvoice)
      AE_Job_free(j);
    else {
      const AQDB_VALUE *v;

      /* check open sum of invoices */
      v=AE_Job_GetOpenSum(j);
      if (AQDB_Value_IsZero(v))
	/* nothing open, free */
	AE_Job_free(j);
      else
        /* add to caller's list */
	AE_Job_List_Add(j, jobList);
    }
  }

  /* cleanup */
  AE_Job_List_free(obStruct.jobList);
  AE_Job_IdMap_free(obStruct.jobMap);
  GWEN_IdMap_free(obStruct.taskMap);

  /* done */
  return 0;
}




