/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_opentasks_p.h"

#include <aqfinance/engine/book/ae_task.h>

#include <gwenhywfar/idmap.h>



static void GWENHYWFAR_CB *AE_Book__GetOpenTasks_Callback(AE_BOOK *bk,
							  AE_BOOK_TABLE_TYPE tt,
							  AQDB_OBJECT *o,
							  void *data) {
  struct GETOPENTASKS_STRUCT *ots;
  AE_TASK *tk=NULL;
  int rv;

  ots=(struct GETOPENTASKS_STRUCT*) data;

  rv=AE_Task_fromObject(o, &tk);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AQDB_ID tid;

    AE_Task_SetDbId(tk, AQDB_Object_GetObjectId(o));
    tid=AE_Task_GetId(tk);
    AE_Task_IdMap_Insert(ots->taskMap, tid, tk);
    AE_Task_List_Add(tk, ots->taskList);
  }

  return NULL;
}



/* for every job: add it if it is a payment or invoice belonging to one of the
 * open tasks.
 */
static void GWENHYWFAR_CB *AE_Book__GetJobsForOpenTasks_Callback(AE_BOOK *bk,
								 AE_BOOK_TABLE_TYPE tt,
								 AQDB_OBJECT *o,
								 void *data) {
  struct GETOPENTASKS_STRUCT *otStruct;
  AE_JOB *j=NULL;
  int rv;

  otStruct=(struct GETOPENTASKS_STRUCT*) data;

  rv=AE_Job_fromObject(o, &j);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AQDB_ID tid;
    AE_TASK *tk;

    AE_Job_SetDbId(j, AQDB_Object_GetObjectId(o));
    tid=AE_Job_GetTaskId(j);

    tk=AE_Task_IdMap_Find(otStruct->taskMap, tid);
    if (tk) {
      AE_Job_List_Add(j, AE_Task_GetJobs(tk));
      AE_Job_IdMap_Insert(otStruct->jobMap, AE_Job_GetId(j), j);
    }
    else
      /* ignore job */
      AE_Job_free(j);
  }

  return NULL;
}



/* for every item: add it's value if it belongs to a job in our list.
 */
static void GWENHYWFAR_CB *AE_Book__HandleItemsForOpenTasks_Callback(AE_BOOK *bk,
								     AE_BOOK_TABLE_TYPE tt,
								     AQDB_OBJECT *o,
								     void *data) {
  struct GETOPENTASKS_STRUCT *otStruct;
  AE_ITEM *item=NULL;
  int rv;

  otStruct=(struct GETOPENTASKS_STRUCT*) data;

  rv=AE_Item_fromObject(o, &item);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
  }
  else {
    AQDB_ID jid;

    AE_Item_SetDbId(item, AQDB_Object_GetObjectId(o));
    jid=AE_Item_GetJobId(item);
    if (jid>0) {
      AE_JOB *j;

      j=AE_Job_IdMap_Find(otStruct->jobMap, jid);
      if (j) {
	int jt;
	AQDB_VALUE *v;
	const AQDB_VALUE *vt;

	jt=AE_Job_GetType(j);
	/* add total sum */
	v=AE_Job_GetTotalSum(j);
	if (j==NULL) {
	  v=AQDB_Value_new();
          AE_Job_SetTotalSum(j, v);
	}
	if (otStruct->isBuy) {
	  vt=AE_Item_GetBuyPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetBuyTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);

	  /* add open sum */
	  v=AE_Job_GetOpenSum(j);
	  if (j==NULL) {
	    v=AQDB_Value_new();
	    AE_Job_SetOpenSum(j, v);
	  }
	  vt=AE_Item_GetBuyPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetBuyTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	}
	else {
	  vt=AE_Item_GetSellPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetSellTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);

	  /* add open sum */
	  v=AE_Job_GetOpenSum(j);
	  if (j==NULL) {
	    v=AQDB_Value_new();
	    AE_Job_SetOpenSum(j, v);
	  }
	  vt=AE_Item_GetSellPriceValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	  vt=AE_Item_GetSellTaxSumValue(item);
	  if (vt)
	    AQDB_Value_AddValue(v, vt);
	}

	/* for payments: subtract item from open value */
	if (jt==AE_Job_TypePayment) {
	  AQDB_ID refId;

	  refId=AE_Job_GetReferenceId(j);
	  if (refId) {
	    AE_JOB *rj;

	    rj=AE_Job_IdMap_Find(otStruct->jobMap, refId);
	    if (rj) {
	      AQDB_VALUE *v;
	      const AQDB_VALUE *vt;

	      v=AE_Job_GetOpenSum(rj);
	      if (v==NULL) {
		v=AQDB_Value_new();
		AE_Job_SetOpenSum(rj, v);
	      }
	      if (otStruct->isBuy) {
		vt=AE_Item_GetBuyPriceValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
		vt=AE_Item_GetBuyTaxSumValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
	      }
	      else {
		vt=AE_Item_GetSellPriceValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
		vt=AE_Item_GetSellTaxSumValue(item);
		if (vt)
		  AQDB_Value_SubValue(v, vt);
	      }
	    }
	  }
	}
        AE_Item_List_Add(item, AE_Job_GetItems(j));
      }
      else
	AE_Item_free(item);
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Item's job id is 0");
      AE_Item_free(item);
    }
  }

  return NULL;
}



static void AE_Book__CalcTaskSums(AE_TASK_LIST *tl) {
  AE_TASK *tk;

  tk=AE_Task_List_First(tl);
  while(tk) {
    AE_JOB *j;
    AQDB_VALUE *vTotal;
    AQDB_VALUE *vOpen;

    vTotal=AE_Task_GetTotalSum(tk);
    if (vTotal==NULL) {
      vTotal=AQDB_Value_new();
      AE_Task_SetTotalSum(tk, vTotal);
    }

    vOpen=AE_Task_GetOpenSum(tk);
    if (vOpen==NULL) {
      vOpen=AQDB_Value_new();
      AE_Task_SetOpenSum(tk, vOpen);
    }

    j=AE_Job_List_First(AE_Task_GetJobs(tk));
    while(j) {
      const AQDB_VALUE *vj;

      if (AE_Task_GetCommodityId(tk)==0)
        AE_Task_SetCommodityId(tk, AE_Job_GetCommodityId(j));
      switch(AE_Job_GetType(j)) {
      case AE_Job_TypeInvoice:
	vj=AE_Job_GetTotalSum(j);
	if (vj) {
	  AQDB_Value_AddValue(vTotal, vj);
	  AQDB_Value_AddValue(vOpen, vj);
	}
	break;

      case AE_Job_TypePayment:
	vj=AE_Job_GetTotalSum(j);
	if (vj)
	  AQDB_Value_SubValue(vOpen, vj);
	break;
      }

      j=AE_Job_List_Next(j);
    }

    AE_Task_CalcOpenOrderedItems(tk);
    AE_Task_CalcOpenDeliveredItems(tk);

    tk=AE_Task_List_Next(tk);
  }
}



int AE_Book_GetOpenTasks(AE_BOOK *b, int buy, AE_TASK_LIST *taskList) {
  int rv;
  void *dummy=NULL;
  char qexpr[64];
  struct GETOPENTASKS_STRUCT otStruct;
  AE_TASK *tk;

  otStruct.taskMap=AE_Task_IdMap_new(GWEN_IdMapAlgo_Hex4);
  otStruct.taskList=AE_Task_List_new();
  otStruct.jobMap=NULL;
  otStruct.isBuy=buy;

  /* only load tasks which are active and whose INCOMING flag matches */
  if (buy)
    snprintf(qexpr, sizeof(qexpr)-1,
	     "$flags ?areset %d",
	     AE_TASK_FLAGS_ACTIVE | AE_TASK_FLAGS_INCOMING);
  else
    snprintf(qexpr, sizeof(qexpr)-1,
	     "($flags ?isset %d) && ($flags ?isclear %d)",
	     AE_TASK_FLAGS_ACTIVE, AE_TASK_FLAGS_INCOMING);
  qexpr[sizeof(qexpr)-1]=0;
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Task,
				 0,
				 qexpr,
				 AE_Book__GetOpenTasks_Callback,
				 (void*) &otStruct,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
    AE_Task_IdMap_free(otStruct.taskMap);
    AE_Task_List_free(otStruct.taskList);
    return 0;
  }

  /* now read all jobs for the active tasks */
  otStruct.jobMap=AE_Job_IdMap_new(GWEN_IdMapAlgo_Hex4);

  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Job,
				 0,
				 NULL, /* no query expression */
				 AE_Book__GetJobsForOpenTasks_Callback,
				 (void*) &otStruct,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
    AE_Job_IdMap_free(otStruct.jobMap);
    GWEN_IdMap_free(otStruct.taskMap);
    return 0;
  }

  /* now calculate the totalSum and openSum for all invoices */
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_Item,
				 0,
				 NULL, /* no query expression */
                                 AE_Book__HandleItemsForOpenTasks_Callback,
				 (void*) &otStruct,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    DBG_INFO(AE_LOGDOMAIN, "No entries found");
    AE_Job_IdMap_free(otStruct.jobMap);
    GWEN_IdMap_free(otStruct.taskMap);
    return 0;
  }

  AE_Book__CalcTaskSums(otStruct.taskList);

  /* move all jobs to the caller's list */
  while( (tk=AE_Task_List_First(otStruct.taskList)) ) {
    /* remove from list no matter what */
    AE_Task_List_Del(tk);
    /* add to caller's list */
    AE_Task_List_Add(tk, taskList);
  }

  /* cleanup */
  AE_Job_IdMap_free(otStruct.jobMap);
  AE_Task_IdMap_free(otStruct.taskMap);
  AE_Task_List_free(otStruct.taskList);

  /* done */
  return 0;
}




