/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_recontransfers_p.h"

#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>




void GWENHYWFAR_CB *AE_ReconTransfers_FindReconStatementSingle(AE_BOOK *bk,
							       AE_BOOK_TABLE_TYPE tt,
							       AQDB_OBJECT *o,
							       void *data) {
  AE_STATEMENT *t;
  AE_STATEMENT *st=NULL;
  int rv;

  t=(AE_STATEMENT*) data;
  rv=AE_Statement_fromObject(o, &st);
  if (rv==0) {
    const AQDB_VALUE *stV;
    const AQDB_VALUE *tV;
    AQDB_VALUE *ntV;
    const char *stS;
    const char *tS;
    const GWEN_DATE *stTi;
    const GWEN_DATE *tTi;

    AE_Statement_SetDbId(st, AQDB_Object_GetObjectId(o));

    if (AE_Statement_GetTransferId(st)!=0) {
      /* already assigned, dismiss */
      AE_Statement_free(st);
      return NULL;
    }

    /* compare statement */
    stV=AE_Statement_GetValue(st);
    tV=AE_Statement_GetValue(t);
    ntV=AQDB_Value_dup(tV);
    AQDB_Value_Negate(ntV);

    if (AQDB_Value_Compare(stV, ntV)!=0) {
      /* different value, no match */
      AQDB_Value_free(ntV);
      AE_Statement_free(st);
      DBG_DEBUG(AE_LOGDOMAIN, "here");
      return NULL;
    }

    stS=AE_Statement_GetCurrency(st);
    tS=AE_Statement_GetCurrency(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stTi=AE_Statement_GetDate(st);
    if (stTi==NULL)
      stTi=AE_Statement_GetValutaDate(st);
    tTi=AE_Statement_GetDate(t);
    if (tTi==NULL)
      tTi=AE_Statement_GetValutaDate(t);
    if (stTi && tTi) {
      int d=GWEN_Date_Diff(stTi, tTi);
      /* statement time must be after the actual transfer, but not later than 14d */
      if (d<0 || d>14) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetRemoteName(st);
    tS=AE_Statement_GetRemoteName(t);
    if (stS && *stS && tS && *tS) {
      if (GWEN_Text_CheckSimilarity(stS, tS, 1)<75.0) {
	/* different remote name, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalAccountNumber(st);
    tS=AE_Statement_GetLocalAccountNumber(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different local account number, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalBankCode(st);
    tS=AE_Statement_GetLocalBankCode(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different local bank code, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalIban(st);
    tS=AE_Statement_GetLocalIban(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalBic(st);
    tS=AE_Statement_GetLocalBic(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetRemoteAccountNumber(st);
    tS=AE_Statement_GetRemoteAccountNumber(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different remote account number, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetRemoteBankCode(st);
    tS=AE_Statement_GetRemoteBankCode(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different remote account number, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetRemoteIban(st);
    tS=AE_Statement_GetRemoteIban(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetRemoteBic(st);
    tS=AE_Statement_GetRemoteBic(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    DBG_ERROR(0, "Reconciling");
    return (void*) st;
  }
  else {
    AE_Statement_free(st);
  }

  return NULL;
}



int AE_ReconTransfers_ReconSingleTransfer(AE_BOOK *b, AE_STATEMENT *t, int doLock) {
  AE_STATEMENT *st=NULL;
  int rv;
  const GWEN_DATE *ti;
  char qbuf[64];

  ti=AE_Statement_GetDate(t);
  if (ti) {
    snprintf(qbuf, sizeof(qbuf)-1,
             "$date>=\"%s\"",
             GWEN_Date_GetString(ti));
    qbuf[sizeof(qbuf)-1]=0;
  
    rv=AE_Book_ForEveryQueryObject(b,
                                   AE_Book_TableType_BankStatement,
                                   0,
                                   qbuf,
                                   AE_ReconTransfers_FindReconStatementSingle,
                                   (void*) t,
                                   (void*) &st);
    if (rv<0) {
      if (rv!=GWEN_ERROR_NOT_FOUND) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        return rv;
      }
    }
  
    if (st) {
      AQDB_OBJECT *o1=NULL;
      AQDB_OBJECT *o2=NULL;
      GWEN_DATE *ti;
  
      /* apply changes */
      DBG_DEBUG(AE_LOGDOMAIN, "Reconciling transfer");
      AE_Statement_SetTransferId(st, AE_Statement_GetId(t));
      AE_Statement_SetStatus(t, AE_Statement_StatusAutoReconciled);
      /* autorecon time is current time (save it in otherwise unused valutaDate of the transfer) */
      ti=GWEN_Date_CurrentDate();
      AE_Statement_SetValutaDate(t, ti);
      GWEN_Date_free(ti);
  
      /* prepare modified statement */
      rv=AE_Statement_CreateObject(AE_Book_GetDataBase(b),
                                   AE_Book_GetTableId(b, AE_Book_TableType_BankStatement),
                                   AE_Statement_GetDbId(st),
                                   &o1);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Statement_free(st);
        return rv;
      }
      rv=AE_Statement_toObject(st, o1);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Statement_free(st);
        return rv;
      }
  
      /* prepare modified transfer */
      rv=AE_Statement_CreateObject(AE_Book_GetDataBase(b),
                                   AE_Book_GetTableId(b, AE_Book_TableType_BankTransfer),
                                   AE_Statement_GetDbId(t),
                                   &o2);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AQDB_Object_free(o1);
        AE_Statement_free(st);
        return rv;
      }
  
      rv=AE_Statement_toObject(t, o2);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AQDB_Object_free(o2);
        AQDB_Object_free(o1);
        AE_Statement_free(st);
        return rv;
      }
  
      /* lock database */
      if (doLock) {
        rv=AE_Book_BeginEdit(b, 0);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AQDB_Object_free(o2);
          AQDB_Object_free(o1);
          AE_Statement_free(st);
          return rv;
        }
      }
  
      /* actually write DB object for the statement */
      rv=AQDB_DB_WriteObject(AE_Book_GetDataBase(b), o1);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AQDB_Object_free(o2);
        AQDB_Object_free(o1);
        if (doLock)
          AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
        AE_Statement_free(st);
        return rv;
      }
  
      /* actually write DB object for the transfer */
      rv=AQDB_DB_WriteObject(AE_Book_GetDataBase(b), o2);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AQDB_Object_free(o2);
        AQDB_Object_free(o1);
        if (doLock)
          AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
        AE_Statement_free(st);
        return rv;
      }
  
      AQDB_Object_free(o2);
      AQDB_Object_free(o1);
  
      /* unlock database */
      if (doLock) {
        rv=AE_Book_EndEdit(b, 0);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
          AE_Statement_free(st);
          return rv;
        }
      }
  
      AE_Statement_free(st);
    }
  }
  else {
    DBG_WARN(AE_LOGDOMAIN, "Transfer %llu has no date",
             (unsigned long long int) AE_Statement_GetId(t));
  }
  return 0;
}



void GWENHYWFAR_CB *AE_ReconTransfers_ReconSingleTransfersFn(AE_BOOK *b,
							     AE_BOOK_TABLE_TYPE tt,
							     AQDB_OBJECT *o,
							     void *data) {
  RECONSINGLETRANSFERS_STRUCT *sk;
  AE_STATEMENT *t;
  int rv;

  sk=(RECONSINGLETRANSFERS_STRUCT*) data;

  rv=AE_Statement_fromObject(o, &t);
  if (rv==0) {
    AE_Statement_SetDbId(t, AQDB_Object_GetObjectId(o));
    rv=AE_ReconTransfers_ReconSingleTransfer(b, t, sk->doLock);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    }
    AE_Statement_free(t);
  }

  /* always return NULL to handle all transfers */
  return NULL;
}



int AE_ReconTransfers_ReconSingleTransfers(AE_BOOK *b, int doLock){
  int rv;
  const char *qbuf1="$transferId==0";
  char qbuf2[64];
  RECONSINGLETRANSFERS_STRUCT sk;
  AQDB_ID stQueryId=0;
  void *dummy;

  sk.stQueryId=0;
  sk.doLock=doLock;
  /* first query statements which have not been assigned to a transfer */
  rv=AQDB_DB_QuerySubmit(AE_Book_GetDataBase(b),
			 AE_Book_GetTableId(b, AE_Book_TableType_BankStatement),
			 0,
			 qbuf1,
			 0,
			 &(sk.stQueryId));
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  /* query transfers which are still pending */
  qbuf2[sizeof(qbuf2)-1]=0;
  snprintf(qbuf2, sizeof(qbuf2)-1,
	   "($status==%d) || ($status==%d)",
	   AE_Statement_StatusPending,
	   AE_Statement_StatusAccepted);
  qbuf2[sizeof(qbuf2)-1]=0;

  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_BankTransfer,
				 0,
				 qbuf2,
				 AE_ReconTransfers_ReconSingleTransfersFn,
				 (void*) &sk,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AQDB_DB_QueryClose(AE_Book_GetDataBase(b), stQueryId);
      return rv;
    }
  }

  AQDB_DB_QueryClose(AE_Book_GetDataBase(b), stQueryId);
  return 0;
}



void GWENHYWFAR_CB *AE_ReconTransfers_FindReconStatementMulti(AE_BOOK *bk,
							      AE_BOOK_TABLE_TYPE tt,
							      AQDB_OBJECT *o,
							      void *data) {
  AE_STATEMENT *t;
  AE_STATEMENT *st=NULL;
  int rv;

  t=(AE_STATEMENT*) data;
  rv=AE_Statement_fromObject(o, &st);
  if (rv==0) {
    const AQDB_VALUE *stV;
    const AQDB_VALUE *tV;
    AQDB_VALUE *ntV;
    const char *stS;
    const char *tS;
    const GWEN_DATE *stTi;
    const GWEN_DATE *tTi;

    AE_Statement_SetDbId(st, AQDB_Object_GetObjectId(o));

    if (AE_Statement_GetTransferId(st)!=0) {
      /* already assigned, dismiss */
      AE_Statement_free(st);
      return NULL;
    }

    /* compare statement */
    stV=AE_Statement_GetValue(st);
    tV=AE_Statement_GetValue(t);
    ntV=AQDB_Value_dup(tV);
    AQDB_Value_Negate(ntV);

    if (AQDB_Value_Compare(stV, ntV)!=0) {
      /* different value, no match */
      AQDB_Value_free(ntV);
      AE_Statement_free(st);
      DBG_DEBUG(AE_LOGDOMAIN, "here");
      return NULL;
    }

    stS=AE_Statement_GetCurrency(st);
    tS=AE_Statement_GetCurrency(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stTi=AE_Statement_GetDate(st);
    if (stTi==NULL)
      stTi=AE_Statement_GetValutaDate(st);
    tTi=AE_Statement_GetDate(t);
    if (tTi==NULL)
      tTi=AE_Statement_GetValutaDate(t);
    if (stTi && tTi) {
      int d=GWEN_Date_Diff(stTi, tTi);
      /* statement time must be after the actual transfer, but not later than 14d */
      if (d<0 || d>14) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalAccountNumber(st);
    tS=AE_Statement_GetLocalAccountNumber(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different local account number, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalBankCode(st);
    tS=AE_Statement_GetLocalBankCode(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different local bank code, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalIban(st);
    tS=AE_Statement_GetLocalIban(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    stS=AE_Statement_GetLocalBic(st);
    tS=AE_Statement_GetLocalBic(t);
    if (stS && *stS && tS && *tS) {
      while(*stS=='0')
	stS++;
      while(*tS=='0')
	tS++;
      if (strcasecmp(stS, tS)!=0) {
	/* different, dismiss */
	AE_Statement_free(st);
	DBG_DEBUG(AE_LOGDOMAIN, "here");
	return NULL;
      }
    }

    DBG_DEBUG(AE_LOGDOMAIN, "Statement matches");
    return (void*) st;
  }
  else {
    AE_Statement_free(st);
  }

  return NULL;
}



void GWENHYWFAR_CB *AE_ReconTransfers_ReconGroupTransfersCb(AE_BOOK *b,
							    AE_BOOK_TABLE_TYPE tt,
							    AQDB_OBJECT *o,
							    void *data) {
  AE_STATEMENT *t=NULL;
  int rv;

  rv=AE_Statement_fromObject(o, &t);
  if (rv==0) {
    GWEN_DATE *ti;
  
    /* apply changes */
    DBG_DEBUG(AE_LOGDOMAIN, "Reconciling transfer");
    AE_Statement_SetStatus(t, AE_Statement_StatusAutoReconciled);
  
    /* autorecon time is current time (save it in otherwise unused valutaDate of the transfer) */
    ti=GWEN_Date_CurrentDate();
    AE_Statement_SetValutaDate(t, ti);
    GWEN_Date_free(ti);
  
    rv=AE_Statement_toObject(t, o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Statement_free(t);
      return NULL;
    }
  
    /* actually write DB object for the transfer */
    rv=AQDB_DB_WriteObject(AE_Book_GetDataBase(b), o);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Statement_free(t);
      return NULL;
    }

    AE_Statement_free(t);
  }

  return NULL;
}



int AE_ReconTransfers_ReconMultiTransfer(AE_BOOK *b, AE_STATEMENT *t, int doLock) {
  AE_STATEMENT *st=NULL;
  int rv;
  const GWEN_DATE *ti;
  char qbuf[64];

  /* find statement which matches the given transfer group */
  ti=AE_Statement_GetDate(t);
  if (ti) {
    snprintf(qbuf, sizeof(qbuf)-1, "$date>=\"%s\"", GWEN_Date_GetString(ti));
    qbuf[sizeof(qbuf)-1]=0;
  
    rv=AE_Book_ForEveryQueryObject(b,
                                   AE_Book_TableType_BankStatement,
                                   0,
                                   qbuf,
                                   AE_ReconTransfers_FindReconStatementMulti,
                                   (void*) t,
                                   (void*) &st);
    if (rv<0) {
      if (rv!=GWEN_ERROR_NOT_FOUND) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        return rv;
      }
    }
  
    /* statement found, now reconcile all transfers of the transfer group */
    if (st) {
      AQDB_OBJECT *o1=NULL;
      void *dummy=NULL;
  
      /* apply changes */
      DBG_DEBUG(AE_LOGDOMAIN, "Reconciling transfer");
      AE_Statement_SetTransferId(st, AE_Statement_GetId(t));
  
      /* prepare modified statement */
      rv=AE_Statement_CreateObject(AE_Book_GetDataBase(b),
                                   AE_Book_GetTableId(b, AE_Book_TableType_BankStatement),
                                   AE_Statement_GetDbId(st),
                                   &o1);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Statement_free(st);
        return rv;
      }
      rv=AE_Statement_toObject(st, o1);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_Statement_free(st);
        return rv;
      }
  
      /* lock database */
      if (doLock) {
        rv=AE_Book_BeginEdit(b, 0);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AQDB_Object_free(o1);
          AE_Statement_free(st);
          return rv;
        }
      }
  
      /* actually write DB object for the statement */
      rv=AQDB_DB_WriteObject(AE_Book_GetDataBase(b), o1);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AQDB_Object_free(o1);
        if (doLock)
          AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
        AE_Statement_free(st);
        return rv;
      }
      AQDB_Object_free(o1);
  
      /* recon group */
      snprintf(qbuf, sizeof(qbuf)-1, "$groupid==\"%d\"", AE_Statement_GetGroupId(t));
      qbuf[sizeof(qbuf)-1]=0;
  
      rv=AE_Book_ForEveryQueryObject(b,
                                     AE_Book_TableType_BankTransfer,
                                     0,
                                     qbuf,
                                     AE_ReconTransfers_ReconGroupTransfersCb,
                                     NULL,
                                     (void*) &dummy);
      if (rv<0) {
        if (rv!=GWEN_ERROR_NOT_FOUND) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          if (doLock)
            AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
          AE_Statement_free(st);
          return rv;
        }
      }
  
      /* unlock database */
      if (doLock) {
        rv=AE_Book_EndEdit(b, 0);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
          AE_Statement_free(st);
          return rv;
        }
      }
  
      AE_Statement_free(st);
    }
  }
  else {
    DBG_WARN(AE_LOGDOMAIN, "Transfer %llu has no date",
             (unsigned long long int) AE_Statement_GetId(t));
  }
  return 0;
}



void GWENHYWFAR_CB *AE_ReconTransfers_SampleGroupsCb(AE_BOOK *bk,
						     AE_BOOK_TABLE_TYPE tt,
						     AQDB_OBJECT *o,
						     void *data) {
  AE_STATEMENT_LIST *groups;
  AE_STATEMENT *st=NULL;
  int rv;

  groups=(AE_STATEMENT_LIST*) data;

  rv=AE_Statement_fromObject(o, &st);
  if (rv==0) {
    AE_STATEMENT *stloop=NULL;
    AQDB_ID groupId;
    const AQDB_VALUE *v1;

    groupId=AE_Statement_GetGroupId(st);
    v1=AE_Statement_GetValue(st);
    if (v1) {
      /* find group in the list (or add the new entry) */
      stloop=AE_Statement_List_First(groups);
      while(stloop) {
	if (AE_Statement_GetGroupId(stloop)==groupId)
	  break;
	stloop=AE_Statement_List_Next(stloop);
      }
      if (stloop) {
	const AQDB_VALUE *v2;

	/* group already exists, just add value */
	v2=AE_Statement_GetValue(stloop);
	if (v2) {
	  AQDB_VALUE *v3;

	  v3=AQDB_Value_dup(v1);
	  AQDB_Value_AddValue(v3, v2);
	  AE_Statement_SetValue(stloop, v3);
          AQDB_Value_free(v3);
	}
      }
      else {
	/* group not in list, add as first member of the group */
	AE_Statement_SetDbId(st, AQDB_Object_GetObjectId(o));
	AE_Statement_List_Add(st, groups);
      }
    } /* if v1 */
  }

  return NULL;
}



int AE_ReconTransfers_ReconMultiTransfers(AE_BOOK *b, int doLock) {
  int rv;
  void *dummy=NULL;
  AE_STATEMENT_LIST *groups;
  AE_STATEMENT *st;
  char qstring[64];

  groups=AE_Statement_List_new();

  /* only count transfers with groupId!=0 and with status "pending" */
  snprintf(qstring, sizeof(qstring)-1,
	   "($groupid!=0) && ($status==%d)",
	   AE_Statement_StatusPending);
  qstring[sizeof(qstring)-1]=0;

  /* add first transfer of every group, add the values of all other transfers of each
   * group to the value of the first one */
  rv=AE_Book_ForEveryQueryObject(b,
				 AE_Book_TableType_BankTransfer,
				 0,
                                 qstring,
				 AE_ReconTransfers_SampleGroupsCb,
				 (void*) groups,
				 (void*) &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Statement_List_free(groups);
      return rv;
    }
  }

  /* recon transfer groups */
  st=AE_Statement_List_First(groups);
  while(st) {
    rv=AE_ReconTransfers_ReconMultiTransfer(b, st, doLock);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Statement_List_free(groups);
      return rv;
    }
    st=AE_Statement_List_Next(st);
  }

#if 0
  DBG_ERROR(0, "Statements:");
  st=AE_Statement_List_First(groups);
  while(st) {
    //AE_Statement_Dump(st, stderr, 2);
    st=AE_Statement_List_Next(st);
  }
#endif

  AE_Statement_List_free(groups);

  return 0;
}



int AE_ReconTransfers_ReconTransfers(AE_BOOK *b, int doLock) {
  int rv;

  rv=AE_ReconTransfers_ReconSingleTransfers(b, doLock);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  rv=AE_ReconTransfers_ReconMultiTransfers(b, doLock);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



