/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2008 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_statementimport_p.h"

#include <aqbanking/transaction.h>

#include <gwenhywfar/text.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/gui.h>
#include <gwenhywfar/i18n.h>


#define I18N(msg) GWEN_I18N_Translate(PACKAGE, msg)



GWEN_LIST_FUNCTIONS(AE_STATEMENT_DAY, AE_StatementDay)




AE_STATEMENT_DAY *AE_StatementDay_new(int day, int month, int year) {
  AE_STATEMENT_DAY *sd;

  GWEN_NEW_OBJECT(AE_STATEMENT_DAY, sd);
  GWEN_LIST_INIT(AE_STATEMENT_DAY, sd);
  sd->day=day;
  sd->month=month;
  sd->year=year;

  sd->savedList=AE_Statement_List_new();
  sd->importedList=AE_Statement_List_new();

  return sd;
}



void AE_StatementDay_free(AE_STATEMENT_DAY *sd) {
  if (sd) {
    AE_Statement_List_free(sd->savedList);
    AE_Statement_List_free(sd->importedList);
    GWEN_FREE_OBJECT(sd);
  }
}



void AE_StatementDay_AddSavedStatement(AE_STATEMENT_DAY *sd, AE_STATEMENT *st) {
  assert(sd);
  AE_Statement_List_Add(st, sd->savedList);
}



void AE_StatementDay_AddImportedStatement(AE_STATEMENT_DAY *sd, AE_STATEMENT *st) {
  assert(sd);
  AE_Statement_List_Add(st, sd->importedList);
}



int AE_StatementDay_CountSavedMatches(const AE_STATEMENT_DAY *sd, const AE_STATEMENT *mst) {
  AE_STATEMENT *st;
  int cnt=0;

  assert(sd);

  st=AE_Statement_List_First(sd->savedList);
  while(st) {
    AE_Statement_PrepareCompare(st);
    if (AE_Statement_Compare(mst, st))
      cnt++;
    st=AE_Statement_List_Next(st);
  }

  return cnt;
}



int AE_StatementDay_CountImportedMatches(const AE_STATEMENT_DAY *sd, const AE_STATEMENT *mst) {
  AE_STATEMENT *st;
  int cnt=0;

  assert(sd);

  st=AE_Statement_List_First(sd->importedList);
  while(st) {
    AE_Statement_PrepareCompare(st);
    if (AE_Statement_Compare(mst, st))
      cnt++;
    st=AE_Statement_List_Next(st);
  }

  return cnt;
}



AE_STATEMENT_DAY *AE_StatementImport_GetDay(AE_STATEMENT_DAY_LIST *dl,
					    const GWEN_DATE *dt) {
  AE_STATEMENT_DAY *sd;

  sd=AE_StatementDay_List_First(dl);
  while(sd) {
    if (sd->year==GWEN_Date_GetYear(dt) &&
	sd->month==GWEN_Date_GetMonth(dt) &&
	sd->day==GWEN_Date_GetDay(dt))
      return sd;
    sd=AE_StatementDay_List_Next(sd);
  }

  sd=AE_StatementDay_new(GWEN_Date_GetDay(dt), GWEN_Date_GetMonth(dt), GWEN_Date_GetYear(dt));
  AE_StatementDay_List_Add(sd, dl);

  return sd;
}





int AE_StatementImport_ImportStatement(AE_STATEMENT_DAY_LIST *dl,
				       AE_STATEMENT *st) {
  AE_STATEMENT_DAY *sd;
  const GWEN_DATE *dt;

  dt=AE_Statement_GetDate(st);
  if (dt==NULL) {
    dt=AE_Statement_GetValutaDate(st);
    AE_Statement_SetDate(st, dt);
  }
  sd=AE_StatementImport_GetDay(dl, dt);
  if (sd) {
    AE_StatementDay_AddImportedStatement(sd, st);
    return 0;
  }

  return GWEN_ERROR_INVALID;
}


int AE_StatementImport_TransactionStatusToStatementStatus(AB_TRANSACTION_STATUS status) {
  switch(status) {
  case AB_Transaction_StatusUnknown:
    return AE_Statement_StatusUnknown;
    break;
  case AB_Transaction_StatusNone:
    return AE_Statement_StatusNone;
    break;
  case AB_Transaction_StatusAccepted:
    return AE_Statement_StatusAccepted;
    break;
  case AB_Transaction_StatusRejected:
    return AE_Statement_StatusRejected;
    break;
  case AB_Transaction_StatusPending:
    return AE_Statement_StatusPending;
    break;
  case AB_Transaction_StatusSending:
    return AE_Statement_StatusSending;
    break;
  case AB_Transaction_StatusAutoReconciled:
    return AE_Statement_StatusAutoReconciled;
    break;
  case AB_Transaction_StatusManuallyReconciled:
    return AE_Statement_StatusManuallyReconciled;
    break;
  case AB_Transaction_StatusRevoked:
    return AE_Statement_StatusRevoked;
    break;
  case AB_Transaction_StatusAborted:
    return AE_Statement_StatusAborted;
    break;
  }

  return AE_Statement_StatusUnknown;
}



int AE_StatementImport_TransactionToStatement(const AB_TRANSACTION *t,
					      AE_STATEMENT *st) {
  const char *s;
  const GWEN_STRINGLIST *sl;
  const GWEN_TIME *ti;
  const AB_VALUE *v;
  int i;
  int rv;
  AQDB_ID id;
  AB_TRANSACTION_PERIOD period;

  s=AB_Transaction_GetLocalCountry(t);
  AE_Statement_SetLocalCountry(st, s);

  s=AB_Transaction_GetLocalBankCode(t);
  AE_Statement_SetLocalBankCode(st, s);

  s=AB_Transaction_GetLocalAccountNumber(t);
  AE_Statement_SetLocalAccountNumber(st, s);

  s=AB_Transaction_GetLocalIban(t);
  AE_Statement_SetLocalIban(st, s);

  s=AB_Transaction_GetLocalBic(t);
  AE_Statement_SetLocalBic(st, s);

  s=AB_Transaction_GetLocalName(t);
  AE_Statement_SetLocalName(st, s);

  s=AB_Transaction_GetRemoteCountry(t);
  AE_Statement_SetRemoteCountry(st, s);

  s=AB_Transaction_GetRemoteBankCode(t);
  AE_Statement_SetRemoteBankCode(st, s);

  s=AB_Transaction_GetRemoteAccountNumber(t);
  AE_Statement_SetRemoteAccountNumber(st, s);

  s=AB_Transaction_GetRemoteIban(t);
  AE_Statement_SetRemoteIban(st, s);

  s=AB_Transaction_GetRemoteBic(t);
  AE_Statement_SetRemoteBic(st, s);

  sl=AB_Transaction_GetRemoteName(t);
  if (sl) {
    s=GWEN_StringList_FirstString(sl);
    AE_Statement_SetRemoteName(st, s);
  }

  ti=AB_Transaction_GetDate(t);
  if (ti) {
    int year, month, day;
    GWEN_DATE *dt;

    GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year);
    dt=GWEN_Date_fromGregorian(year, month+1, day);
    AE_Statement_SetDate(st, dt);
    GWEN_Date_free(dt);
  }

  ti=AB_Transaction_GetValutaDate(t);
  if (ti) {
    int year, month, day;
    GWEN_DATE *dt;

    GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year);
    dt=GWEN_Date_fromGregorian(year, month+1, day);
    AE_Statement_SetValutaDate(st, dt);
    GWEN_Date_free(dt);
  }

  v=AB_Transaction_GetValue(t);
  if (v) {
    GWEN_BUFFER *tbuf;
    AB_VALUE *v2;
    AQDB_VALUE *nv;

    /* value */
    v2=AB_Value_dup(v);
    AB_Value_SetCurrency(v2, NULL);
    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    AB_Value_toString(v2, tbuf);
    AB_Value_free(v2);
    nv=AQDB_Value_fromString(GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
    AE_Statement_SetValue(st, nv);
    AQDB_Value_free(nv);

    /* currency */
    s=AB_Value_GetCurrency(v);
    AE_Statement_SetCurrency(st, s);
  }

  s=AB_Transaction_GetFiId(t);
  AE_Statement_SetFiId(st, s);

  i=AB_Transaction_GetTextKey(t);
  AE_Statement_SetTextKey(st, i);

  i=AB_Transaction_GetTextKeyExt(t);
  AE_Statement_SetTextKeyExt(st, i);

  s=AB_Transaction_GetTransactionText(t);
  AE_Statement_SetInfoText(st, s);

  s=AB_Transaction_GetCustomerReference(t);
  AE_Statement_SetCustomerRef(st, s);

  s=AB_Transaction_GetBankReference(t);
  AE_Statement_SetBankRef(st, s);

  sl=AB_Transaction_GetPurpose(t);
  if (sl) {
    GWEN_BUFFER *tbuf;
    GWEN_STRINGLISTENTRY *se;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);

    /* sample purpose lines */
    se=GWEN_StringList_FirstEntry(sl);
    while(se) {
      s=GWEN_StringListEntry_Data(se);
      if (GWEN_Buffer_GetUsedBytes(tbuf))
	GWEN_Buffer_AppendByte(tbuf, ' ');
      GWEN_Buffer_AppendString(tbuf, s);
      se=GWEN_StringListEntry_Next(se);
    }

    /* post process buffer */
    GWEN_Text_CondenseBuffer(tbuf);
    if (GWEN_Buffer_GetUsedBytes(tbuf)>128)
      GWEN_Buffer_Crop(tbuf, 0, 128);

    /* store purpose */
    AE_Statement_SetPurpose(st, GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_free(tbuf);
  }

  rv=AE_StatementImport_TransactionStatusToStatementStatus(AB_Transaction_GetStatus(t));
  AE_Statement_SetStatus(st, rv);

  id=AB_Transaction_GetGroupId(t);
  AE_Statement_SetGroupId(st, id);

  sl=AB_Transaction_GetCategory(t);
  if (sl) {
    s=GWEN_StringList_FirstString(sl);
    AE_Statement_SetCategory(st, s);
  }

  period=AB_Transaction_GetPeriod(t);
  if (period!=AB_Transaction_PeriodUnknown &&
      period!=AB_Transaction_PeriodNone) {
    AE_RECURRENCE *rec;

    rec=AE_Recurrence_new();
    switch(AB_Transaction_GetPeriod(t)) {
    case AB_Transaction_PeriodMonthly:
      AE_Recurrence_SetPeriod(rec, AE_Recurrence_PeriodMonths);
      AE_Recurrence_SetEvery(rec, AB_Transaction_GetCycle(t));
      AE_Recurrence_SetOn(rec, AB_Transaction_GetExecutionDay(t));
      break;

    case AB_Transaction_PeriodWeekly:
      AE_Recurrence_SetPeriod(rec, AE_Recurrence_PeriodWeeks);
      AE_Recurrence_SetEvery(rec, AB_Transaction_GetCycle(t));
      AE_Recurrence_SetOn(rec, AB_Transaction_GetExecutionDay(t));
      break;

    default:
      AE_Recurrence_SetPeriod(rec, AE_Recurrence_PeriodUnknown);
      break;
    }

    ti=AB_Transaction_GetFirstExecutionDate(t);
    if (ti) {
      int year, month, day;
      GWEN_DATE *dt;
  
      GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year);
      dt=GWEN_Date_fromGregorian(year, month+1, day);
      AE_Recurrence_SetBeginDate(rec, dt);
      GWEN_Date_free(dt);
    }

    ti=AB_Transaction_GetNextExecutionDate(t);
    if (ti) {
      int year, month, day;
      GWEN_DATE *dt;
  
      GWEN_Time_GetBrokenDownDate(ti, &day, &month, &year);
      dt=GWEN_Date_fromGregorian(year, month+1, day);
      AE_Recurrence_SetNextExecutionDate(rec, dt);
      GWEN_Date_free(dt);
    }


    AE_Statement_SetRecurrence(st, rec);
  }

  rv=AE_Statement_GenerateHash(st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AE_StatementImport_ImportTransaction(AE_STATEMENT_DAY_LIST *dl,
                                         AB_TRANSACTION *t,
                                         AQDB_ID aid) {
  AE_STATEMENT *st;
  int rv;

  st=AE_Statement_new();
  rv=AE_StatementImport_TransactionToStatement(t, st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Statement_free(st);
    return rv;
  }
  AE_Statement_SetBankAccountId(st, aid);

  rv=AE_StatementImport_ImportStatement(dl, st);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    /* only free statement in case of an error, otherwise it will be added
     * to the list */
    AE_Statement_free(st);
    return rv;
  }

  return 0;
}



void GWENHYWFAR_CB *AE_StatementImport_AddSavedStatement(AE_BOOK *bk,
							 AE_BOOK_TABLE_TYPE tt,
							 AQDB_OBJECT *o,
							 void *data) {
  AE_STATEMENT_DAY *sd;
  AE_STATEMENT *st=NULL;
  int rv;

  sd=(AE_STATEMENT_DAY*) data;
  rv=AE_Statement_fromObject(o, &st);
  if (rv==0) {
    AE_Statement_SetDbId(st, AQDB_Object_GetObjectId(o));
    AE_StatementDay_AddSavedStatement(sd, st);
  }
  else {
    AE_Statement_free(st);
  }

  return NULL;
}



int AE_StatementImport_FillSavedDay(AE_BOOK *b,
				    AE_STATEMENT_DAY *sd,
				    AQDB_ID refQueryId,
				    AE_BOOK_TABLE_TYPE tt) {
  char qbuf[64];
  void *dummy=NULL;
  int rv;

  snprintf(qbuf, sizeof(qbuf)-1,
	   "$date==\"%04d%02d%02d\"",
	   sd->year, sd->month, sd->day);
  rv=AE_Book_ForEveryQueryObject(b,
				 tt,
				 refQueryId,
				 qbuf,
				 AE_StatementImport_AddSavedStatement,
				 (void*) sd,
				 &dummy);
  if (rv<0) {
    if (rv!=GWEN_ERROR_NOT_FOUND) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}



AB_ACCOUNT_STATUS *AE_StatementImport_GetBestBalance(AB_IMEXPORTER_ACCOUNTINFO *ai,
                                                     int noted) {
  AB_ACCOUNT_STATUS *ast;
  AB_ACCOUNT_STATUS *best=0;

  ast=AB_ImExporterAccountInfo_GetFirstAccountStatus(ai);
  while(ast) {
    if ((noted && AB_AccountStatus_GetNotedBalance(ast)) ||
        (!noted && AB_AccountStatus_GetBookedBalance(ast))) {
      if (!best)
        best=ast;
      else {
        const GWEN_TIME *tiBest;
        const GWEN_TIME *ti;

        tiBest=AB_AccountStatus_GetTime(best);
        ti=AB_AccountStatus_GetTime(ast);

        if (!tiBest) {
          best=ast;
        }
        else {
          if (ti) {
            double d;

            /* we have two times, compare them */
            d=GWEN_Time_Diff(ti, tiBest);
            if (d>0)
              /* newer */
              best=ast;
          }
        }
      }
    } /* if noted balance */
    ast=AB_ImExporterAccountInfo_GetNextAccountStatus(ai);
  } /* while */

  return best;
}



int AE_StatementImport_UpdateTransfers(AE_BOOK *b,
                                       AB_IMEXPORTER_ACCOUNTINFO *iea,
                                       AE_BOOK_TABLE_TYPE tt,
                                       AQDB_ID bankAccountId,
                                       uint32_t guiid) {
  const AB_TRANSACTION *t;

  t=AB_ImExporterAccountInfo_GetFirstTransfer(iea);
  while(t) {
    AQDB_ID stmId;

    stmId=AB_Transaction_GetIdForApplication(t);
    if (stmId) {
      AE_STATEMENT *st=NULL;
      int rv;

      DBG_ERROR(0, "Updating transfer %d", stmId);
      rv=AE_Book_ReadStatement(b, AE_Book_TableType_BankTransfer, stmId, &st);
      if (rv<0) {
        DBG_WARN(AE_LOGDOMAIN, "Transfer not found (%d)", rv);
      }
      else {
	AQDB_ID groupId;

	rv=AE_StatementImport_TransactionStatusToStatementStatus(AB_Transaction_GetStatus(t));
	AE_Statement_SetStatus(st, rv);

        /* adjust group id */
	groupId=(AQDB_ID)AB_Transaction_GetGroupId(t);
        if (groupId!=(AQDB_ID)0)
	  AE_Statement_SetGroupId(st, groupId);

        rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, st, 0);
        if (rv<0) {
          DBG_WARN(AE_LOGDOMAIN, "Unable to write new transfer status (%d)", rv);
        }
	/*DBG_ERROR(0, "Updated statement:");
	AE_Statement_Dump(st, stderr, 2);*/
        AE_Statement_free(st);
      }
    } /* if stmId */

    t=AB_ImExporterAccountInfo_GetNextTransfer(iea);
  } /* while t */

  return 0;
}



int AE_StatementImport_ImportStandingOrders(AE_BOOK *b,
                                            AB_IMEXPORTER_ACCOUNTINFO *iea,
                                            AE_BOOK_TABLE_TYPE tt,
                                            AQDB_ID bankAccountId,
                                            uint32_t guiid) {
  const AB_TRANSACTION *t;

  t=AB_ImExporterAccountInfo_GetFirstStandingOrder(iea);
  while(t) {
    const char *fiid;

    fiid=AB_Transaction_GetFiId(t);
    if (fiid && *fiid) {
      char qstring[256];
      AQDB_ID stoId;
      int rv;

      DBG_ERROR(0, "Handling standing order [%s]", fiid);
      snprintf(qstring, sizeof(qstring)-1,
               "($bankAccountId==\"%lu\") && ($fiId==\"%s\")",
               (long unsigned int) bankAccountId, fiid);
      qstring[sizeof(qstring)-1]=0;
      rv=AE_Book_GetFirstDbIdForQuery(b, tt, qstring, &stoId);
      if (rv<0) {
        AE_STATEMENT *st;
        AE_RECURRENCE *rec;

        /* new, add it */
        st=AE_Statement_new();
        rv=AE_StatementImport_TransactionToStatement(t, st);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AE_Statement_free(st);
          return rv;
        }
        AE_Statement_SetBankAccountId(st, bankAccountId);
        DBG_ERROR(0, "Adding standing order for bank account id %lu", (long unsigned int) bankAccountId);
        rv=AE_Book_AddStatement(b, tt, st, 0);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          AE_Statement_free(st);
          return rv;
        }

        rec=AE_Statement_GetRecurrence(st);
        if (rec) {
	  GWEN_DATE *dt;

	  dt=GWEN_Date_CurrentDate();
	  AE_Recurrence_SetLastUpdated(rec, dt);
	  GWEN_Date_free(dt);
          AE_Recurrence_SetOwnerType(rec, tt);
          AE_Recurrence_SetOwnerId(rec, AE_Statement_GetId(st));
          DBG_ERROR(0, "Adding recurrence to statement %lu", (unsigned long int) AE_Statement_GetId(st));
          rv=AE_Book_AddRecurrence(b, AE_Book_TableType_BankStoRecurrence, rec, 0);
          if (rv<0) {
            DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
            AE_Statement_free(st);
            return rv;
          }
        }
        else {
          DBG_ERROR(0, "No recurrence information, not adding one");
        }

        AE_Statement_free(st);
      }
      else {
        AQDB_OBJECT *o=NULL;

        /* update existing */
        DBG_ERROR(0, "Updating standing order");
        rv=AQDB_DB_ReadObject(AE_Book_GetDataBase(b),
                              AE_Book_GetTableId(b, tt),
                              stoId,
                              &o);
        if (rv<0) {
          DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
          return rv;
        }
        else {
          AE_STATEMENT *st=NULL;
          AE_RECURRENCE *rec;

          /* read old standing order */
          rv=AE_Statement_fromObject(o, &st);
          if (rv<0) {
            DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
            return rv;
          }
	  AE_Statement_SetDbId(st, stoId);

          /* read old subobjects */
          rv=AE_Book_ReadBankStandingOrderSubObjects(b, st);
          if (rv<0) {
            DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
            AE_Statement_free(st);
            return rv;
          }

          rec=AE_Statement_GetRecurrence(st);
	  if (rec) {
	    AE_STATEMENT *stTmp;
	    AE_RECURRENCE *recTmp;
	    GWEN_DATE *dt;

	    /* read new standing order from AB_TRANSACTION */
            stTmp=AE_Statement_new();
            rv=AE_StatementImport_TransactionToStatement(t, stTmp);
            if (rv<0) {
              DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
              AE_Statement_free(stTmp);
              AE_Statement_free(st);
              return rv;
            }
            AE_Statement_SetBankAccountId(stTmp, bankAccountId);
            AE_Statement_SetId(stTmp, AE_Statement_GetId(st));
            AE_Statement_SetDbId(stTmp, AE_Statement_GetDbId(st));

	    recTmp=AE_Statement_GetRecurrence(stTmp);
	    if (recTmp) {
	      AE_Recurrence_SetId(recTmp, AE_Recurrence_GetId(rec));
	      AE_Recurrence_SetDbId(recTmp, AE_Recurrence_GetDbId(rec));
	      dt=GWEN_Date_CurrentDate();
	      AE_Recurrence_SetLastUpdated(recTmp, dt);
              AE_Recurrence_SetOwnerType(recTmp, tt);
              AE_Recurrence_SetOwnerId(recTmp, AE_Statement_GetId(stTmp));
	      GWEN_Date_free(dt);
	    }

            /* write back standing order (don't lock) */
            rv=AE_Book_WriteStatement(b, tt, stTmp, 0);
            if (rv<0) {
              DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
              AE_Statement_free(stTmp);
              AE_Statement_free(st);
              return rv;
            }

	    if (recTmp) {
	      /* write back recurrence (again, don't lock) */
	      rv=AE_Book_WriteRecurrence(b, AE_Book_TableType_BankStoRecurrence, recTmp, 0);
	      if (rv<0) {
		DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
		AE_Statement_free(stTmp);
		AE_Statement_free(st);
		return rv;
	      }
	    }
	    AE_Statement_free(stTmp);
          } /* if rec */

          AE_Statement_free(st);
        }

      }

#if 0
      stmId=AB_Transaction_GetIdForApplication(t);
      if (stmId) {
        AE_STATEMENT *st=NULL;
        int rv;

        DBG_ERROR(0, "Updating transfer %d", stmId);
        rv=AE_Book_ReadStatement(b, AE_Book_TableType_BankTransfer, stmId, &st);
        if (rv<0) {
          DBG_WARN(AE_LOGDOMAIN, "Transfer not found (%d)", rv);
        }
        else {
          AQDB_ID groupId;

          rv=AE_StatementImport_TransactionStatusToStatementStatus(AB_Transaction_GetStatus(t));
          AE_Statement_SetStatus(st, rv);

          /* adjust group id */
          groupId=(AQDB_ID)AB_Transaction_GetGroupId(t);
          if (groupId!=(AQDB_ID)0)
            AE_Statement_SetGroupId(st, groupId);

          rv=AE_Book_WriteStatement(b, AE_Book_TableType_BankTransfer, st, 0);
          if (rv<0) {
            DBG_WARN(AE_LOGDOMAIN, "Unable to write new transfer status (%d)", rv);
          }
          /*DBG_ERROR(0, "Updated statement:");
           AE_Statement_Dump(st, stderr, 2);*/
          AE_Statement_free(st);
        }
      } /* if stmId */
#endif
    }

    t=AB_ImExporterAccountInfo_GetNextStandingOrder(iea);
  } /* while t */

  return 0;
}



int AE_StatementImport_ImportTransactions(AE_BOOK *b,
                                          AB_IMEXPORTER_ACCOUNTINFO *iea,
                                          AE_BOOK_TABLE_TYPE tt,
                                          AQDB_ID bankAccountId,
                                          uint32_t guiid) {
  AB_TRANSACTION *t;
  AE_STATEMENT_DAY_LIST *dl;
  AE_STATEMENT_DAY *sd;
  int errors=0;
  AQDB_ID refQueryId=0;
  AQDB_DB *db;
  AQDB_ID tableId;
  uint32_t pid;
  int rv;

  db=AE_Book_GetDataBase(b);
  assert(db);

  tableId=AE_Book_GetTableId(b, tt);
  if (tableId==0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table type %d not open", (int)tt);
    return GWEN_ERROR_NOT_OPEN;
  }

  /* transform transactions to statements */
  dl=AE_StatementDay_List_new();

  switch(tt) {
    case AE_Book_TableType_BankTransfer:
      t=AB_ImExporterAccountInfo_GetFirstTransfer(iea);
      while(t) {
        if (AB_Transaction_GetIdForApplication(t)==0) {
          /* only add transfers with no application id, others need to be
           * updated instead */
          rv=AE_StatementImport_ImportTransaction(dl, t, bankAccountId);
          if (rv<0) {
            DBG_WARN(AE_LOGDOMAIN, "Could not import transaction (%d)", rv);
            errors++;
          }
        }
        t=AB_ImExporterAccountInfo_GetNextTransfer(iea);
      }
      break;
  
    case AE_Book_TableType_BankStatement:
    default:
      t=AB_ImExporterAccountInfo_GetFirstTransaction(iea);
      pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_SHOW_PROGRESS |
				 GWEN_GUI_PROGRESS_SHOW_ABORT |
				 GWEN_GUI_PROGRESS_ALLOW_EMBED,
				 I18N("Transforming Transactions"),
				 I18N("Transforming statements"),
				 AB_ImExporterAccountInfo_GetTransactionCount(iea),
				 guiid);
      while(t) {
        rv=AE_StatementImport_ImportTransaction(dl, t, bankAccountId);
        if (rv<0) {
          DBG_WARN(AE_LOGDOMAIN, "Could not import transaction (%d)", rv);
          errors++;
	}
	rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
	if (rv==GWEN_ERROR_USER_ABORTED) {
	  DBG_ERROR(AE_LOGDOMAIN, "User aborted");
	  AE_StatementDay_List_free(dl);
	  GWEN_Gui_ProgressEnd(pid);
	  return rv;
	}

        t=AB_ImExporterAccountInfo_GetNextTransaction(iea);
      }
      GWEN_Gui_ProgressEnd(pid);
      break;
  }

  if (errors) {
    DBG_INFO(AE_LOGDOMAIN, "Some transactions could not be imported");
    AE_StatementDay_List_free(dl);
    return GWEN_ERROR_GENERIC;
  }

  /* load saved statements for comparison, detect dupes */
  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_SHOW_PROGRESS |
                             GWEN_GUI_PROGRESS_SHOW_ABORT |
                             GWEN_GUI_PROGRESS_ALLOW_EMBED,
                             I18N("Importing days"),
                             I18N("Importing daily statements"),
                             AE_StatementDay_List_GetCount(dl),
                             guiid);
  sd=AE_StatementDay_List_First(dl);
  while(sd) {
    AE_STATEMENT *st;

    rv=AE_StatementImport_FillSavedDay(b, sd, refQueryId, tt);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_StatementDay_List_free(dl);
      GWEN_Gui_ProgressEnd(pid);
      return rv;
    }

    /* remove dupes */
    st=AE_Statement_List_First(sd->importedList);
    while(st) {
      AE_STATEMENT *next;
      int cntSaved;
      int cntImported;

      next=AE_Statement_List_Next(st);
      AE_Statement_PrepareCompare(st);

      /* check for dupes */
      cntSaved=AE_StatementDay_CountSavedMatches(sd, st);
      cntImported=AE_StatementDay_CountImportedMatches(sd, st);
      DBG_DEBUG(AE_LOGDOMAIN, "Saved=%d, imported=%d", cntSaved, cntImported);
      if (cntImported<=cntSaved) {
	AE_Statement_List_Del(st);
	AE_Statement_free(st);
	st=NULL;
      }
      st=next;
    }

    /* add remaining statements to database */
    st=AE_Statement_List_First(sd->importedList);
    while(st) {
      AE_Statement_SetBankAccountId(st, bankAccountId);
      rv=AE_Book_AddStatement(b, tt, st, 0);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        AE_StatementDay_List_free(dl);
        GWEN_Gui_ProgressEnd(pid);
        return rv;
      }

      st=AE_Statement_List_Next(st);
    }

    /* next day */
    rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
    if (rv==GWEN_ERROR_USER_ABORTED) {
      DBG_ERROR(AE_LOGDOMAIN, "User aborted");
      AE_StatementDay_List_free(dl);
      GWEN_Gui_ProgressEnd(pid);
      return rv;
    }
    sd=AE_StatementDay_List_Next(sd);
  }
  AE_StatementDay_List_free(dl);

  GWEN_Gui_ProgressEnd(pid);

  /* done */
  return 0;
}



int AE_StatementImport_ImportAccountInfo(AE_BOOK *b,
					 AB_IMEXPORTER_ACCOUNTINFO *iea,
                                         AQDB_ID bankAccountId,
                                         uint32_t guiid) {
  int rv;
  int rv1;
  int rv2;
  int rv3;
  uint32_t pid;

  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_SHOW_PROGRESS |
                             GWEN_GUI_PROGRESS_SHOW_ABORT |
                             GWEN_GUI_PROGRESS_ALLOW_EMBED,
                             I18N("Importing Account Info"),
                             I18N("Importing account info"),
                             3,
                             guiid);

  rv1=AE_StatementImport_ImportTransactions(b, iea,
                                            AE_Book_TableType_BankStatement,
                                            bankAccountId, pid);
  if (rv1==GWEN_ERROR_USER_ABORTED) {
    DBG_ERROR(0, "User aborted");
    GWEN_Gui_ProgressEnd(pid);
    return rv1;
  }
  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
  if (rv==GWEN_ERROR_USER_ABORTED) {
    DBG_ERROR(0, "User aborted");
    GWEN_Gui_ProgressEnd(pid);
    return rv;
  }


  rv2=AE_StatementImport_UpdateTransfers(b, iea,
                                         AE_Book_TableType_BankTransfer,
                                         bankAccountId, pid);
  if (rv2==GWEN_ERROR_USER_ABORTED) {
    DBG_ERROR(0, "User aborted");
    GWEN_Gui_ProgressEnd(pid);
    return rv2;
  }
  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
  if (rv==GWEN_ERROR_USER_ABORTED) {
    DBG_ERROR(0, "User aborted");
    GWEN_Gui_ProgressEnd(pid);
    return rv;
  }

  rv3=AE_StatementImport_ImportTransactions(b, iea,
                                            AE_Book_TableType_BankTransfer,
                                            bankAccountId, pid);
  if (rv3==GWEN_ERROR_USER_ABORTED) {
    DBG_ERROR(0, "User aborted");
    GWEN_Gui_ProgressEnd(pid);
    return rv3;
  }
  rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
  if (rv==GWEN_ERROR_USER_ABORTED) {
    DBG_ERROR(0, "User aborted");
    GWEN_Gui_ProgressEnd(pid);
    return rv;
  }

  GWEN_Gui_ProgressEnd(pid);

  if (rv1)
    return rv1;
  if (rv2)
    return rv2;
  if (rv3)
    return rv3;

  return 0;
}



int AE_StatementImport_ImportContext(AE_BOOK *b,
				     AB_IMEXPORTER_CONTEXT *iec,
                                     uint32_t guiid) {
  int rv;
  AB_IMEXPORTER_ACCOUNTINFO *iea;
  AE_BANKACCOUNT_LIST *baList;

  /* read bank account list for later reference */
  baList=AE_BankAccount_List_new();
  rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, baList);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_BankAccount_List_free(baList);
    return rv;
  }

  /* lock database */
  rv=AE_Book_BeginEdit(b, 0);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_BankAccount_List_free(baList);
    return rv;
  }

  iea=AB_ImExporterContext_GetFirstAccountInfo(iec);
  while(iea) {
    AQDB_ID bankAccountId=0;
    AQDB_ID bankingId;

    /* determine bank account id; we do that by checking the banking id of
     * every bank account in the database */
    bankingId=AB_ImExporterAccountInfo_GetAccountId(iea);
    if (bankingId) {
      AE_BANKACCOUNT *ba;

      ba=AE_BankAccount_List_First(baList);
      while(ba) {
        if (AE_BankAccount_GetBankingId(ba)==bankingId)
          break;
        ba=AE_BankAccount_List_Next(ba);
      }
      if (ba)
        bankAccountId=AE_BankAccount_GetId(ba);
    }

    /* import account info */
    rv=AE_StatementImport_ImportAccountInfo(b, iea,
                                            bankAccountId,
                                            guiid);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      AE_BankAccount_List_free(baList);
      return rv;
    }
    iea=AB_ImExporterContext_GetNextAccountInfo(iec);
  }

  /* unlock database */
  rv=AE_Book_EndEdit(b, 0);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    AE_BankAccount_List_free(baList);
    return rv;
  }

  AE_BankAccount_List_free(baList);

  return 0;
}






int AE_StatementImport_ExtractNewStatements(AE_BOOK *b,
                                            AB_IMEXPORTER_ACCOUNTINFO *iea,
                                            AE_BOOK_TABLE_TYPE tt,
                                            AQDB_ID bankAccountId,
                                            AE_STATEMENT_LIST *newStatements,
                                            uint32_t guiid) {
  AB_TRANSACTION *t;
  AE_STATEMENT_DAY_LIST *dl;
  AE_STATEMENT_DAY *sd;
  int errors=0;
  AQDB_ID refQueryId=0;
  AQDB_DB *db;
  AQDB_ID tableId;
  uint32_t pid;
  int rv;

  db=AE_Book_GetDataBase(b);
  assert(db);

  tableId=AE_Book_GetTableId(b, tt);
  if (tableId==0) {
    DBG_ERROR(AE_LOGDOMAIN, "Table type %d not open", (int)tt);
    return GWEN_ERROR_NOT_OPEN;
  }

  /* transform transactions to statements */
  dl=AE_StatementDay_List_new();

  switch(tt) {
    case AE_Book_TableType_BankTransfer:
      t=AB_ImExporterAccountInfo_GetFirstTransfer(iea);
      while(t) {
        if (AB_Transaction_GetIdForApplication(t)==0) {
          /* only add transfers with no application id, others need to be
           * updated instead */
          rv=AE_StatementImport_ImportTransaction(dl, t, bankAccountId);
          if (rv<0) {
            DBG_WARN(AE_LOGDOMAIN, "Could not import transaction (%d)", rv);
            errors++;
          }
        }
        t=AB_ImExporterAccountInfo_GetNextTransfer(iea);
      }
      break;
  
    case AE_Book_TableType_BankStatement:
    default:
      t=AB_ImExporterAccountInfo_GetFirstTransaction(iea);
      pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_SHOW_PROGRESS |
				 GWEN_GUI_PROGRESS_SHOW_ABORT |
				 GWEN_GUI_PROGRESS_ALLOW_EMBED,
				 I18N("Transforming Transactions"),
				 I18N("Transforming statements"),
				 AB_ImExporterAccountInfo_GetTransactionCount(iea),
				 guiid);
      while(t) {
        rv=AE_StatementImport_ImportTransaction(dl, t, bankAccountId);
        if (rv<0) {
          DBG_WARN(AE_LOGDOMAIN, "Could not import transaction (%d)", rv);
          errors++;
        }
	rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
	if (rv==GWEN_ERROR_USER_ABORTED) {
	  DBG_ERROR(AE_LOGDOMAIN, "User aborted");
	  AE_StatementDay_List_free(dl);
	  GWEN_Gui_ProgressEnd(pid);
	  return rv;
	}
        t=AB_ImExporterAccountInfo_GetNextTransaction(iea);
      }
      GWEN_Gui_ProgressEnd(pid);
      break;
  }

  if (errors) {
    DBG_INFO(AE_LOGDOMAIN, "Some transactions could not be imported");
    AE_StatementDay_List_free(dl);
    return GWEN_ERROR_GENERIC;
  }

  /* load saved statements for comparison, detect dupes */
  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_SHOW_PROGRESS |
                             GWEN_GUI_PROGRESS_SHOW_ABORT |
                             GWEN_GUI_PROGRESS_ALLOW_EMBED,
                             I18N("Importing days"),
                             I18N("Importing daily statements"),
                             AE_StatementDay_List_GetCount(dl),
                             guiid);
  sd=AE_StatementDay_List_First(dl);
  while(sd) {
    AE_STATEMENT *st;

    rv=AE_StatementImport_FillSavedDay(b, sd, refQueryId, tt);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_StatementDay_List_free(dl);
      GWEN_Gui_ProgressEnd(pid);
      return rv;
    }

    /* remove dupes */
    st=AE_Statement_List_First(sd->importedList);
    while(st) {
      AE_STATEMENT *next;
      int cntSaved;
      int cntImported;

      next=AE_Statement_List_Next(st);
      AE_Statement_PrepareCompare(st);

      /* check for dupes */
      cntSaved=AE_StatementDay_CountSavedMatches(sd, st);
      cntImported=AE_StatementDay_CountImportedMatches(sd, st);
      DBG_DEBUG(AE_LOGDOMAIN, "Saved=%d, imported=%d", cntSaved, cntImported);
      if (cntImported<=cntSaved) {
	AE_Statement_List_Del(st);
	AE_Statement_free(st);
	st=NULL;
      }
      st=next;
    }

    /* add remaining statements to list */
    while( (st=AE_Statement_List_First(sd->importedList)) ) {
      /* unlink from importedList, add to newStatements */
      AE_Statement_List_Del(st);
      AE_Statement_List_Add(st, newStatements);
      AE_Statement_SetBankAccountId(st, bankAccountId);
    }

    /* next day */
    rv=GWEN_Gui_ProgressAdvance(pid, GWEN_GUI_PROGRESS_ONE);
    if (rv==GWEN_ERROR_USER_ABORTED) {
      DBG_ERROR(AE_LOGDOMAIN, "User aborted");
      AE_StatementDay_List_free(dl);
      GWEN_Gui_ProgressEnd(pid);
      return rv;
    }
    sd=AE_StatementDay_List_Next(sd);
  }
  AE_StatementDay_List_free(dl);

  GWEN_Gui_ProgressEnd(pid);

  /* done */
  return 0;
}



static void* AE_StatementImport__handleAccountPath(const char *entry,
						   void *data,
						   int idx,
						   uint32_t flags) {
  struct AE_HANDLE_ACCOUNT_DATA *ad;
  AE_ACCOUNT *acc;
  char *cpy=NULL;
  const char *accName=NULL;
  AE_ACCOUNT_TYPE at=AE_AccountType_Expense;
  int rv;

  ad=(struct AE_HANDLE_ACCOUNT_DATA*)data;
  if (strchr(entry, ':')==NULL)
    accName=entry;
  else {
    char *t;

    cpy=strdup(entry);
    t=strchr(cpy, ':');
    assert(t);
    *t=0;
    t++;
    accName=t;
    at=AE_AccountType_fromString(cpy);
    if (at==AE_AccountType_Unknown) {
      DBG_WARN(AE_LOGDOMAIN, "Invalid account type [%s], using \"expense\" instead", cpy);
      at=AE_AccountType_Expense;
    }
  }

  /* check whether we are allowed to simply create the node */
  if (
      ((flags & GWEN_PATH_FLAGS_LAST) &&
       (((flags & GWEN_PATH_FLAGS_VARIABLE) &&
         (flags & GWEN_PATH_FLAGS_CREATE_VAR)) ||
        (!(flags & GWEN_PATH_FLAGS_VARIABLE) &&
         (flags & GWEN_PATH_FLAGS_CREATE_GROUP)))
      ) ||
      (
       !(flags & GWEN_PATH_FLAGS_LAST) &&
       (flags & GWEN_PATH_FLAGS_PATHCREATE))
     ) {
    /* simply create the new variable/group */
    if (idx!=0) {
      DBG_INFO(AE_LOGDOMAIN, "Index is not 0, not creating %s[%d]",
	       entry, idx);
      if (cpy)
	free(cpy);
      return 0;
    }
    DBG_VERBOUS(AE_LOGDOMAIN,
		"Unconditionally creating account \"%s\"", entry);
    acc=AE_Account_new();
    AE_Account_SetName(acc, accName);
    if (ad->account) {
      AE_Account_SetType(acc, AE_Account_GetType(ad->account));
      AE_Account_SetPrimaryCommodityId(acc, AE_Account_GetPrimaryCommodityId(ad->account));
      AE_Account_SetParentId(acc, AE_Account_GetId(ad->account));
    }
    else {
      AE_Account_SetType(acc, at);
      AE_Account_SetPrimaryCommodityId(acc, ad->commodityId);
    }

    rv=AE_Book_AddAccount(ad->book, AE_Book_TableType_Account, acc, ad->doLock);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Account_free(acc);
      if (cpy)
	free(cpy);
      return 0;
    }

    if (ad->account)
      AE_Account_Tree_AddChild(ad->account, acc);
    else
      AE_Account_Tree_Add(ad->accountTree, acc);
    ad->account=acc;

    if (cpy)
      free(cpy);
    return ad;
  }

  /* find the node */
  if (ad->account)
    acc=AE_Account_Tree_GetFirstChild(ad->account);
  else
    acc=AE_Account_Tree_GetFirst(ad->accountTree);
  while(acc) {
    const char *s;

    s=AE_Account_GetName(acc);
    if (s && *s && strcasecmp(accName, s)==0)
      break;
    acc=AE_Account_Tree_GetNext(acc);
  }
  if (!acc) {
    /* node not found, check, if we are allowed to create it */
    if (
        (!(flags & GWEN_PATH_FLAGS_LAST) &&
         (flags & GWEN_PATH_FLAGS_PATHMUSTEXIST)) ||
        (flags & GWEN_PATH_FLAGS_NAMEMUSTEXIST)
       ) {
      DBG_VERBOUS(AE_LOGDOMAIN,
		  "Account \"%s\" does not exist", entry);
      if (cpy)
	free(cpy);
      return 0;
    }
    /* create the new variable/group */
    if (idx!=0) {
      DBG_INFO(AE_LOGDOMAIN, "Index is not 0, not creating %s[%d]",
	       entry, idx);
      if (cpy)
	free(cpy);
      return 0;
    }
    DBG_VERBOUS(AE_LOGDOMAIN,
		"Account \"%s\" not found, creating", entry);
    acc=AE_Account_new();
    AE_Account_SetName(acc, accName);
    if (ad->account) {
      AE_Account_SetType(acc, AE_Account_GetType(ad->account));
      AE_Account_SetPrimaryCommodityId(acc, AE_Account_GetPrimaryCommodityId(ad->account));
      AE_Account_SetParentId(acc, AE_Account_GetId(ad->account));
    }
    else {
      AE_Account_SetType(acc, at);
      AE_Account_SetPrimaryCommodityId(acc, ad->commodityId);
    }

    rv=AE_Book_AddAccount(ad->book, AE_Book_TableType_Account, acc, ad->doLock);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Account_free(acc);
      if (cpy)
	free(cpy);
      return 0;
    }

    if (ad->account)
      AE_Account_Tree_AddChild(ad->account, acc);
    else
      AE_Account_Tree_Add(ad->accountTree, acc);
    ad->account=acc;
  } /* if node not found */
  else {
    /* node does exist, check whether this is ok */
    if (
        ((flags & GWEN_PATH_FLAGS_LAST) &&
         (flags & GWEN_PATH_FLAGS_NAMEMUSTNOTEXIST)) ||
        (!(flags & GWEN_PATH_FLAGS_LAST) &&
         (flags & GWEN_PATH_FLAGS_PATHMUSTNOTEXIST))
       ) {
      DBG_VERBOUS(AE_LOGDOMAIN, "Entry \"%s\" already exists", entry);
      if (cpy)
	free(cpy);
      return 0;
    }
    ad->account=acc;
  }

  if (cpy)
    free(cpy);
  return ad;
}



int AE_StatementImport_GetAccountForCategory(AE_BOOK *b, const char *cat,
					     AE_ACCOUNT_TREE *at,
					     AQDB_ID commodityId,
					     AE_ACCOUNT **pAcc, int doLock) {
  struct AE_HANDLE_ACCOUNT_DATA ad;

  ad.book=b;
  ad.accountTree=at;
  ad.account=NULL;
  ad.commodityId=commodityId;
  ad.doLock=doLock;

  if (NULL==GWEN_Path_HandleWithIdx(cat,
				    (void*) &ad,
				    GWEN_PATH_FLAGS_NO_IDX,
				    AE_StatementImport__handleAccountPath)) {
    DBG_INFO(AE_LOGDOMAIN, "here");
    return GWEN_ERROR_GENERIC;
  }
  if (ad.account==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Account not found");
    return GWEN_ERROR_GENERIC;
  }

  *pAcc=ad.account;
  return 0;
}



int AE_StatementImport_FindPartnerForStatement(const AE_STATEMENT *st,
					       const AE_PARTNER_LIST *pl,
					       AE_PARTNER **pPartner) {
  const char *bankCode;
  const char *accountNumber;
  const char *iban;
  const char *name;
  AE_PARTNER *pt;

  /* first search by account/bank code combination */
  bankCode=AE_Statement_GetRemoteBankCode(st);
  if (bankCode==NULL)
    bankCode="";
  accountNumber=AE_Statement_GetRemoteAccountNumber(st);
  if (accountNumber==NULL)
    accountNumber="";

  /* skip leading zeroes, if any */
  while(*bankCode=='0')
    bankCode++;

  while(*accountNumber=='0')
    accountNumber++;

  pt=AE_Partner_List_First(pl);
  while(pt) {
    AE_BANKACCOUNT *ba;

    ba=AE_BankAccount_List_First(AE_Partner_GetBankAccountList(pt));
    while(ba) {
      const char *baB;
      const char *baA;

      baB=AE_BankAccount_GetBankCode(ba);
      baA=AE_BankAccount_GetAccountNumber(ba);
      if (baA && *baA && baB && *baB) {
	while(*baB=='0')
	  baB++;

	while(*baA=='0')
	  baA++;

	if (strcasecmp(bankCode, baB)==0 &&
	    strcasecmp(accountNumber, baA)==0) {
	  *pPartner=pt;
	  return AE_StatementImport_FindPartner_ResultByAccount;
	}
      }

      ba=AE_BankAccount_List_Next(ba);
    }

    pt=AE_Partner_List_Next(pt);
  }

  /* search by iban */
  iban=AE_Statement_GetRemoteIban(st);
  if (iban && *iban) {
    pt=AE_Partner_List_First(pl);
    while(pt) {
      AE_BANKACCOUNT *ba;

      ba=AE_BankAccount_List_First(AE_Partner_GetBankAccountList(pt));
      while(ba) {
	const char *baI;

	baI=AE_BankAccount_GetIban(ba);
	if (baI && *baI) {
	  if (strcasecmp(iban, baI)==0) {
	    *pPartner=pt;
	    return AE_StatementImport_FindPartner_ResultByIban;
	  }
	}

	ba=AE_BankAccount_List_Next(ba);
      }

      pt=AE_Partner_List_Next(pt);
    }
  }

  /* search by name */
  name=AE_Statement_GetRemoteName(st);
  if (name && *name) {
    pt=AE_Partner_List_First(pl);
    while(pt) {
      const char *s;

      s=AE_Partner_GetLastName(pt);
      if (s && *s && strcasecmp(name, s)==0) {
	*pPartner=pt;
	return AE_StatementImport_FindPartner_ResultByName;
      }
      pt=AE_Partner_List_Next(pt);
    }
  }

  return GWEN_ERROR_NOT_FOUND;
}



int AE_StatementImport_GetPartnerForStatementNoCreate(AE_BOOK *b,
						      AB_BANKING *ab,
						      const AE_STATEMENT *st,
						      AE_COMMODITY_LIST *cl,
						      AE_GROUP_TREE *gt,
						      AE_PARTNER_LIST *pl,
						      AE_PARTNER **pPartner,
						      int doLock) {
  AE_PARTNER *pt=NULL;
  AE_BANKACCOUNT *ba;
  AE_GROUP *g;
  const char *s;
  int rv;
  int needAddGroup=0;
  const char *sBankCode;
  const char *sAccountNumber;
  const char *sIban;

  rv=AE_StatementImport_FindPartnerForStatement(st, pl, &pt);
  if (rv>=0) {
    /* possibly add bank account */
    if (rv!=AE_StatementImport_FindPartner_ResultByAccount &&
	rv!=AE_StatementImport_FindPartner_ResultByIban) {
      sBankCode=AE_Statement_GetRemoteBankCode(st);
      sAccountNumber=AE_Statement_GetRemoteAccountNumber(st);
      sIban=AE_Statement_GetRemoteIban(st);

      /* only add account if we have at least any piece of information */
      if ((sBankCode && *sBankCode) ||
	  (sAccountNumber && *sAccountNumber) ||
	  (sIban && *sIban)) {
	ba=AE_BankAccount_new();
	AE_BankAccount_SetBankCode(ba, sBankCode);
	AE_BankAccount_SetAccountNumber(ba, sAccountNumber);
	s=AE_Statement_GetRemoteBic(st);
	AE_BankAccount_SetBic(ba, s);
	AE_BankAccount_SetIban(ba, sIban);

        if (ab && (sBankCode && *sBankCode)) {
          AB_BANKINFO *bi;
	  const char *sCountry;

	  sCountry=AE_Statement_GetLocalCountry(st);
	  if (!sCountry || !*sCountry)
	    sCountry="de";
	  bi=AB_Banking_GetBankInfo(ab, sCountry, NULL, sBankCode);
	  if (bi) {
	    const char *s;

	    s=AB_BankInfo_GetBic(bi);
            if (s && *s)
	      AE_BankAccount_SetBic(ba, s);

	    s=AB_BankInfo_GetBankName(bi);
	    if (s && *s)
	      AE_BankAccount_SetBankName(ba, s);
	    AB_BankInfo_free(bi);
	  }
	}


	AE_BankAccount_SetOwnerType(ba, AE_Book_TableType_Partner);
	AE_BankAccount_SetOwnerId(ba, AE_Partner_GetId(pt));
	AE_BankAccount_AddRuntimeFlags(ba, AE_BANKACCOUNT_RTFLAGS_MODIFIED);
	AE_BankAccount_List_Add(ba, AE_Partner_GetBankAccountList(pt));

	s=AE_Statement_GetCurrency(st);
	if (!(s && *s))
	  s="EUR";
	if (s && *s)
	    AE_BankAccount_SetCurrency(ba, s);

        AE_Partner_AddRuntimeFlags(pt, AE_PARTNER_RTFLAGS_MODIFIED);
      }
    }

    if (AE_Partner_GetCommodityId(pt)==0) {
      /* set commodity */
      s=AE_Statement_GetCurrency(st);
      if (!(s && *s))
	s="EUR";
      if (s && *s) {
	AE_COMMODITY *co;
  
	co=AE_Commodity_List_GetByCode(cl, s);
	if (co) {
	  AE_Partner_SetCommodityId(pt, AE_Commodity_GetId(co));
	  AE_Partner_AddRuntimeFlags(pt, AE_PARTNER_RTFLAGS_MODIFIED);
	}
      }
    }

    *pPartner=pt;
    return rv;
  }
  if (rv!=GWEN_ERROR_NOT_FOUND) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  s=AE_Statement_GetRemoteName(st);
  if (s && *s) {
    DBG_ERROR(0, "Adding partner [%s]", s);
    pt=AE_Partner_new();
    AE_Partner_SetLastName(pt, s);
  
    sBankCode=AE_Statement_GetRemoteBankCode(st);
    sAccountNumber=AE_Statement_GetRemoteAccountNumber(st);
    sIban=AE_Statement_GetRemoteIban(st);
  
    /* find auto-new group */
    g=AE_Group_Tree_GetFirst(gt);
    while(g) {
      if (AE_Group_GetType(g)==AE_Partner_Group_TypeNew)
	break;
      g=AE_Group_Tree_GetBelow(g);
    }
    if (g==NULL) {
      g=AE_Group_new();
      AE_Group_SetType(g, AE_Partner_Group_TypeNew);
      AE_Group_SetName(g, I18N("PartnerGroup|auto"));
      AE_Group_SetDescription(g, I18N("PartnerGroup|automatically generated partners"));
      needAddGroup=1;
    }
  
    if (needAddGroup) {
      rv=AE_Book_AddGroup(b, AE_Book_TableType_PartnerGroup, g, doLock);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AE_Group_free(g);
	AE_Partner_free(pt);
	return rv;
      }
    }

    AE_Partner_SetGroupId(pt, AE_Group_GetId(g));

    /* only add account if we have at least any piece of information */
    if ((sBankCode && *sBankCode) ||
	(sAccountNumber && *sAccountNumber) ||
	(sIban && *sIban)) {
      ba=AE_BankAccount_new();
  
      AE_BankAccount_SetBankCode(ba, sBankCode);
      AE_BankAccount_SetAccountNumber(ba, sAccountNumber);
      s=AE_Statement_GetRemoteBic(st);
      AE_BankAccount_SetBic(ba, s);
      AE_BankAccount_SetIban(ba, sIban);
      AE_BankAccount_AddRuntimeFlags(ba, AE_BANKACCOUNT_RTFLAGS_MODIFIED);

      s=AE_Statement_GetCurrency(st);
      if (!(s && *s))
	s="EUR";
      if (s && *s)
	AE_BankAccount_SetCurrency(ba, s);

      if (ab && (sBankCode && *sBankCode)) {
        AB_BANKINFO *bi;
        const char *sCountry;

        sCountry=AE_Statement_GetLocalCountry(st);
        if (!sCountry || !*sCountry)
          sCountry="de";
        bi=AB_Banking_GetBankInfo(ab, sCountry, NULL, sBankCode);
        if (bi) {
          const char *s;

          s=AB_BankInfo_GetBic(bi);
          if (s && *s)
            AE_BankAccount_SetBic(ba, s);

          s=AB_BankInfo_GetBankName(bi);
          if (s && *s)
            AE_BankAccount_SetBankName(ba, s);
          AB_BankInfo_free(bi);
        }
      }

      AE_BankAccount_List_Add(ba, AE_Partner_GetBankAccountList(pt));
      AE_Partner_AddRuntimeFlags(pt, AE_PARTNER_RTFLAGS_MODIFIED);
    }

    /* set commodity */
    s=AE_Statement_GetCurrency(st);
    if (!(s && *s))
      s="EUR";
    if (s && *s) {
      AE_COMMODITY *co;

      co=AE_Commodity_List_GetByCode(cl, s);
      if (co) {
	AE_Partner_SetCommodityId(pt, AE_Commodity_GetId(co));
	AE_Partner_AddRuntimeFlags(pt, AE_PARTNER_RTFLAGS_MODIFIED);
      }
    }

    if (needAddGroup)
      AE_Group_Tree_Add(gt, g);
    AE_Partner_List_Add(pt, pl);
  
    *pPartner=pt;
  }
  else
    *pPartner=NULL;

  /* done */
  return 0;
}



int AE_StatementImport_GetPartnerForStatement(AE_BOOK *b,
					      AB_BANKING *ab,
					      const AE_STATEMENT *st,
					      AE_COMMODITY_LIST *cl,
					      AE_GROUP_TREE *gt,
					      AE_PARTNER_LIST *pl,
					      AE_PARTNER **pPartner,
					      int doLock) {
  AE_PARTNER *pt=NULL;
  int rv;

  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  rv=AE_StatementImport_GetPartnerForStatementNoCreate(b, ab, st, cl, gt, pl, &pt, 0);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    return rv;
  }
  if (pt==NULL) {
    DBG_INFO(AE_LOGDOMAIN, "No partner found and none created");
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    return GWEN_ERROR_NO_DATA;
  }

  if (AE_Partner_GetId(pt)==0) {
    /* create partner in DB */
    rv=AE_Book_ApplyPartner(b, pt, NULL, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_free(pt);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }
  }
  else if (AE_Partner_GetRuntimeFlags(pt) & AE_PARTNER_RTFLAGS_MODIFIED) {
    AE_PARTNER *ptOrig=NULL;

    /* existing partner modified, apply to database */
    rv=AE_Book_ReadPartner(b,
			   AE_Book_TableType_Partner,
			   AE_Partner_GetId(pt),
			   &ptOrig);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_free(pt);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }

    rv=AE_Book_ReadPartnerSubObjects(b, ptOrig);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_free(ptOrig);
      AE_Partner_free(pt);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      return rv;
    }

    rv=AE_Book_ApplyPartner(b, pt, ptOrig, 0);
    if (rv<0) {
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_free(ptOrig);
      AE_Partner_free(pt);
      return rv;
    }
    AE_Partner_free(ptOrig);
  }

  /* unlock database */
  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_free(pt);
      return rv;
    }
  }

  /* done */
  *pPartner=pt;
  return 0;
}



int AE_StatementImport_AssignPartnersToStatements(AE_BOOK *b,
						  AB_BANKING *ab,
						  AE_BOOK_TABLE_TYPE tt,
						  AE_STATEMENT_LIST *sl,
						  int doLock) {
  AE_GROUP_TREE *gt;
  AE_PARTNER_LIST *pl;
  AE_COMMODITY_LIST *cl;
  AE_PARTNER *pt;
  int rv;
  AE_STATEMENT *st;

  if (doLock) {
    rv=AE_Book_BeginEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  /* read group tree */
  gt=AE_Group_Tree_new();
  rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_PartnerGroup, gt);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Group_Tree_free(gt);
    return rv;
  }

  /* read partner list */
  pl=AE_Partner_List_new();
  rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, pl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Partner_List_free(pl);
    AE_Group_Tree_free(gt);
    return rv;
  }

  /* read sub-objects of partners (maybe we need to optimize this later) */
  pt=AE_Partner_List_First(pl);
  while(pt) {
    rv=AE_Book_ReadPartnerSubObjects(b, pt);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      AE_Partner_List_free(pl);
      AE_Group_Tree_free(gt);
      return rv;
    }

    pt=AE_Partner_List_Next(pt);
  }

  cl=AE_Commodity_List_new();
  rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    if (doLock)
      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
    AE_Commodity_List_free(cl);
    AE_Partner_List_free(pl);
    AE_Group_Tree_free(gt);
    return rv;
  }

  st=AE_Statement_List_First(sl);
  while(st) {
    if (AE_Statement_GetPartnerId(st)==0) {
      pt=NULL;
      rv=AE_StatementImport_GetPartnerForStatement(b, ab, st, cl, gt, pl, &pt, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	if (doLock)
	  AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_Commodity_List_free(cl);
	AE_Partner_List_free(pl);
	AE_Group_Tree_free(gt);
	return rv;
      }
      if (pt) {
	AE_Statement_SetPartnerId(st, AE_Partner_GetId(pt));
	AE_Statement_AddRuntimeFlags(st, AE_STATEMENT_RTFLAGS_MODIFIED);

	/* write back statement (if it has an id) */
	if (AE_Statement_GetId(st)) {
	  rv=AE_Book_WriteStatement(b, tt, st, 0);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    if (doLock)
	      AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	    AE_Commodity_List_free(cl);
	    AE_Partner_List_free(pl);
	    AE_Group_Tree_free(gt);
	    return rv;
	  }
	}
      }

    }
    st=AE_Statement_List_Next(st);
  }

  if (doLock) {
    rv=AE_Book_EndEdit(b, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      if (doLock)
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
      AE_Commodity_List_free(cl);
      AE_Partner_List_free(pl);
      AE_Group_Tree_free(gt);
      return rv;
    }
  }

  AE_Commodity_List_free(cl);
  AE_Partner_List_free(pl);
  AE_Group_Tree_free(gt);
  return 0;
}



int AE_StatementImport_GetPartnerForStatementSimple(AE_BOOK *b,
                                                    AB_BANKING *ab,
						    AE_BOOK_TABLE_TYPE tt,
						    const AE_STATEMENT *st,
						    AE_PARTNER **pPartner,
						    int doLock) {
  AE_GROUP_TREE *gt;
  AE_PARTNER_LIST *pl;
  AE_COMMODITY_LIST *cl;
  AE_PARTNER *pt=NULL;
  int rv;

  /* read group tree */
  gt=AE_Group_Tree_new();
  rv=AE_Book_ReadGroupTree(b, AE_Book_TableType_PartnerGroup, gt);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Group_Tree_free(gt);
    return rv;
  }

  /* read partner list */
  pl=AE_Partner_List_new();
  rv=AE_Book_ReadPartnerList(b, AE_Book_TableType_Partner, pl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Partner_List_free(pl);
    AE_Group_Tree_free(gt);
    return rv;
  }

  /* read sub-objects of partners (maybe we need to optimize this later) */
  pt=AE_Partner_List_First(pl);
  while(pt) {
    rv=AE_Book_ReadPartnerSubObjects(b, pt);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_Partner_List_free(pl);
      AE_Group_Tree_free(gt);
      return rv;
    }

    pt=AE_Partner_List_Next(pt);
  }

  cl=AE_Commodity_List_new();
  rv=AE_Book_ReadCommodityList(b, AE_Book_TableType_Commodity, cl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Commodity_List_free(cl);
    AE_Partner_List_free(pl);
    AE_Group_Tree_free(gt);
    return rv;
  }

  rv=AE_StatementImport_GetPartnerForStatementNoCreate(b, ab, st, cl, gt, pl, &pt, doLock);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    AE_Commodity_List_free(cl);
    AE_Partner_List_free(pl);
    AE_Group_Tree_free(gt);
    return rv;
  }

  if (pt==NULL) {
    DBG_INFO(AE_LOGDOMAIN, "No partner created");
    AE_Commodity_List_free(cl);
    AE_Partner_List_free(pl);
    AE_Group_Tree_free(gt);
    return GWEN_ERROR_NO_DATA;
  }
  AE_Partner_List_Del(pt);

  AE_Commodity_List_free(cl);
  AE_Partner_List_free(pl);
  AE_Group_Tree_free(gt);

  *pPartner=pt;
  return 0;
}








