/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2013 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "ae_update_bankaccounts.h"

#include <aqfinance/engine/book/ae_book.h>

#include <aqbanking/banking.h>
#include <aqbanking/account.h>




int AE_Book_UpdateBankAccounts(AE_BOOK *b, AB_BANKING *ab) {
  int accountsUpdated=0;

  if (ab && b) {
    AB_ACCOUNT_LIST2 *al;

    al=AB_Banking_GetAccounts(ab);
    if (al) {
      AB_ACCOUNT_LIST2_ITERATOR *it;
      AE_BANKACCOUNT_LIST *baList;
      int rv;

      rv=AE_Book_BeginEdit(b, 0);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	AB_Account_List2_free(al);
	return 1;
      }

      /* read known bank accounts */
      baList=AE_BankAccount_List_new();
      rv=AE_Book_ReadBankAccountList(b, AE_Book_TableType_BankAccount, baList);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
        AE_BankAccount_List_free(baList);
        AB_Account_List2_free(al);
	return rv;
      }

      it=AB_Account_List2_First(al);
      if (it) {
	AB_ACCOUNT *a;

	a=AB_Account_List2Iterator_Data(it);
        while(a) {
          AE_BANKACCOUNT *ba;

          /* look for bank account in list */
	  DBG_INFO(AE_LOGDOMAIN, "Checking AqBanking account %d", (int) AB_Account_GetUniqueId(a));
          ba=AE_BankAccount_List_First(baList);
          while(ba) {
	    if (AE_BankAccount_GetOwnerType(ba)==0 &&
		AE_BankAccount_GetOwnerId(ba)==0 &&
		AE_BankAccount_GetBankingId(ba)==AB_Account_GetUniqueId(a))
	      break;
	    ba=AE_BankAccount_List_Next(ba);
	  }

          if (ba==NULL) {
	    DBG_INFO(AE_LOGDOMAIN, "AqBanking account %d not assigned, looking further", (int) AB_Account_GetUniqueId(a));
	    /* then match by bank code and account number */
            ba=AE_BankAccount_List_First(baList);
            while(ba) {
              const char *s1;
              const char *s2;
	      int mightMatch=1;

	      if (AE_BankAccount_GetOwnerType(ba)!=0 ||
		  AE_BankAccount_GetOwnerId(ba)!=0)
		mightMatch=0;

              /* compare bank code */
	      if (mightMatch) {
		s1=AB_Account_GetBankCode(a);
		s2=AE_BankAccount_GetBankCode(ba);
		if (s1 && s2)
		  mightMatch=(strcasecmp(s1, s2)!=0)?0:1;
	      }

              /* compare account number */
              if (mightMatch) {
                s1=AB_Account_GetAccountNumber(a);
                s2=AE_BankAccount_GetAccountNumber(ba);
                if (s1 && s2) {
                  /* skip blanks and leading zeroes */
                  while(*s1 && ((uint8_t)(*s1))<33)
                    s1++;
                  while(*s1=='0')
                    s1++;

                  /* skip blanks and leading zeroes */
                  while(*s2 && ((uint8_t)(*s2))<33)
                    s2++;
                  while(*s2=='0')
                    s2++;

                  /* compare */
		  mightMatch=(strcasecmp(s1, s2)!=0)?0:1;
                }
              }

              /* compare iban */
              if (mightMatch) {
                s1=AB_Account_GetIBAN(a);
                s2=AE_BankAccount_GetIban(ba);
                if (s1 && s2)
		  mightMatch=(strcasecmp(s1, s2)!=0)?0:1;
              }

	      if (mightMatch) {
		DBG_INFO(AE_LOGDOMAIN, "Account %d matches AqBanking account %d",
			 (int)AE_BankAccount_GetId(ba), (int)AB_Account_GetUniqueId(a));
		break;
	      }

              ba=AE_BankAccount_List_Next(ba);
            }
	  }

	  if (ba) {
	    const char *s;

	    s=AB_Account_GetAccountNumber(a);
	    AE_BankAccount_SetAccountNumber(ba, s);

	    s=AB_Account_GetBankName(a);
	    AE_BankAccount_SetBankName(ba, s);

	    s=AB_Account_GetBankCode(a);
	    AE_BankAccount_SetBankCode(ba, s);

	    s=AB_Account_GetBIC(a);
	    AE_BankAccount_SetBic(ba, s);

	    s=AB_Account_GetIBAN(a);
	    AE_BankAccount_SetIban(ba, s);

	    AE_BankAccount_SetBankingId(ba, AB_Account_GetUniqueId(a));

	    rv=AE_Book_WriteBankAccount(b, AE_Book_TableType_BankAccount, ba, 0);
	    if (rv<0) {
	      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    }
	    else
	      accountsUpdated++;
	  }

	  a=AB_Account_List2Iterator_Next(it);
	}
	AB_Account_List2Iterator_free(it);
      }

      rv=AE_Book_EndEdit(b, 0);
      if (rv<0) {
	DBG_ERROR(0, "here (%d)", rv);
	AE_Book_EndEdit(b, AQDB_ACTION_FLAGS_ABORT);
	AE_BankAccount_List_free(baList);
	AB_Account_List2_free(al);
	return rv;
      }

      /* cleanup */
      AE_BankAccount_List_free(baList);
      AB_Account_List2_free(al);
    }
  }

  return accountsUpdated;
}




