/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "ae_writexml_p.h"

#include <aqfinance/engine/book/ae_book.h>

#include <gwenhywfar/text.h>

#include <stdio.h>
#include <string.h>
#include <errno.h>



static AE_BOOK_TABLE_TYPE _tableList[]={
  AE_Book_TableType_IdCounter,
  AE_Book_TableType_Commodity,
  AE_Book_TableType_Account,
  AE_Book_TableType_Transaction,
  AE_Book_TableType_Split,
  AE_Book_TableType_BankAccount,
  AE_Book_TableType_Address,
  AE_Book_TableType_Partner,

//  AE_Book_TableType_Project,
  AE_Book_TableType_Task,
  AE_Book_TableType_Job,

//  AE_Book_TableType_Warehouse,
  AE_Book_TableType_Item,
  AE_Book_TableType_BankStatement,
  AE_Book_TableType_BankTransfer,
  AE_Book_TableType_BankDebitNote,

  AE_Book_TableType_BankAccountGroup,

  AE_Book_TableType_BankTransferPlanned,
  AE_Book_TableType_BankTransferTemplate,
  AE_Book_TableType_BankTransferGroup,
  AE_Book_TableType_BookedBalance,
  AE_Book_TableType_NotedBalance,

  AE_Book_TableType_Rule,

  AE_Book_TableType_PartnerGroup,
  AE_Book_TableType_PartnerRule,

  AE_Book_TableType_Tax,
  AE_Book_TableType_TaxGroup,

  AE_Book_TableType_ItemTemplates,
  AE_Book_TableType_ItemTemplateGroup,

  AE_Book_TableType_AccountBalance,

  AE_Book_TableType_BankStandingOrder,
  AE_Book_TableType_BankStoRecurrence,

  AE_Book_TableType_Vehicle,
  AE_Book_TableType_VehicleGroup,
  AE_Book_TableType_Refueling,

  AE_Book_TableType_GraphicReportGroup,
  AE_Book_TableType_GraphicReportSettings,

  AE_Book_TableType_Last
};



static int AE_Book__WriteXml_WriteColumnInt(AQDB_COLUMN *co,
					    int idx,
					    AQDB_OBJECT *o,
					    FILE *f) {
  GWEN_BUFFER *tbuf;
  int rv;
  int64_t val;
  char buf[64];

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);

  rv=AQDB_Object_GetFieldInt64(o, idx, &val);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  snprintf(buf, sizeof(buf)-1, "%lld", (long long int)val);
  buf[sizeof(buf)-1]=0;
  GWEN_Text_EscapeXmlToBuffer(buf, tbuf);
  fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  GWEN_Buffer_free(tbuf);

  return 0;
}



static int AE_Book__WriteXml_WriteColumnString(AQDB_COLUMN *co,
					       int idx,
					       AQDB_OBJECT *o,
					       FILE *f) {
  GWEN_BUFFER *tbuf;
  int rv;
  const char *val;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AQDB_Object_GetFieldString(o, idx, &val);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }
  if (val && *val) {
    GWEN_Text_EscapeXmlToBuffer(val, tbuf);
    fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  }
  GWEN_Buffer_free(tbuf);

  return 0;
}



static int AE_Book__WriteXml_WriteColumnBin(AQDB_COLUMN *co,
					    int idx,
					    AQDB_OBJECT *o,
					    FILE *f) {
  GWEN_BUFFER *tbuf;
  int rv;
  uint8_t *val;
  uint32_t len;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AQDB_Object_GetField(o, idx, &val, &len);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }
  if (val && *val && len) {
    GWEN_Text_ToHexBuffer((const char*) val, len, tbuf, 2, ' ', 0);
    fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  }
  GWEN_Buffer_free(tbuf);

  return 0;
}



static int AE_Book__WriteXml_WriteColumnDate(AQDB_COLUMN *co,
					     int idx,
					     AQDB_OBJECT *o,
					     FILE *f) {
  GWEN_BUFFER *tbuf;
  int rv;
  const GWEN_DATE *val=NULL;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AQDB_Object_GetFieldDate(o, idx, &val);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  if (val) {
    GWEN_Date_toStringWithTemplate(val, "YYYYMMDD", tbuf);
    fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  }
  GWEN_Buffer_free(tbuf);
  return 0;
}



static int AE_Book__WriteXml_WriteColumnTimestamp(AQDB_COLUMN *co,
						  int idx,
						  AQDB_OBJECT *o,
						  FILE *f) {
  GWEN_BUFFER *tbuf;
  int rv;
  const GWEN_TIME *val;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AQDB_Object_GetFieldTimestamp(o, idx, &val);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  if (val) {
    GWEN_Time_toString(val, "YYYYMMDD-hhmmss", tbuf);
    fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  }
  GWEN_Buffer_free(tbuf);

  return 0;
}



static int AE_Book__WriteXml_WriteColumnAmount(AQDB_COLUMN *co,
					       int idx,
					       AQDB_OBJECT *o,
					       FILE *f) {
  GWEN_BUFFER *tbuf;
  int rv;
  const AQDB_VALUE *val;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AQDB_Object_GetFieldValue(o, idx, &val);
  if (rv<0) {
    DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  if (val) {
    AQDB_Value_toString(val, tbuf);
    fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
  }
  GWEN_Buffer_free(tbuf);

  return 0;
}



static void GWENHYWFAR_CB *AE_Book__WriteXml_Callback(AE_BOOK *b,
						      AE_BOOK_TABLE_TYPE tt,
						      AQDB_OBJECT *o,
						      void *data) {
  struct WRITEXML_STRUCT *wxs;
  AQDB_COLUMN *col;
  int idx=0;
  GWEN_BUFFER *tbuf;

  wxs=(struct WRITEXML_STRUCT*) data;
  tbuf=GWEN_Buffer_new(0, 256, 0, 1);

  fprintf(wxs->file, "  <object>\n");

  col=AQDB_Column_List_First(wxs->columns);
  while(col) {
    const char *s;

    s=AQDB_Column_GetName(col);
    if (s && *s) {
      int rv;

      fprintf(wxs->file, "    <column name=\"");
      GWEN_Text_EscapeXmlToBuffer(s, tbuf);
      fprintf(wxs->file, "%s", GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_Reset(tbuf);
      fprintf(wxs->file, "\">");

      switch(AQDB_Column_GetType(col)) {
      case AQDB_DataType_Int:
        rv=AE_Book__WriteXml_WriteColumnInt(col, idx, o, wxs->file);
        break;
      case AQDB_DataType_String:
	rv=AE_Book__WriteXml_WriteColumnString(col, idx, o, wxs->file);
        break;
      case AQDB_DataType_Bin:
	rv=AE_Book__WriteXml_WriteColumnBin(col, idx, o, wxs->file);
        break;
      case AQDB_DataType_Date:
	rv=AE_Book__WriteXml_WriteColumnDate(col, idx, o, wxs->file);
        break;
      case AQDB_DataType_Timestamp:
	rv=AE_Book__WriteXml_WriteColumnTimestamp(col, idx, o, wxs->file);
	break;
      case AQDB_DataType_Amount:
	rv=AE_Book__WriteXml_WriteColumnAmount(col, idx, o, wxs->file);
	break;
      case AQDB_DataType_Unknown:
      case AQDB_DataType_None:
	DBG_ERROR(AE_LOGDOMAIN, "Invalid column type %d", AQDB_Column_GetType(col));
	return data;
      }
      if (rv<0) {
        DBG_ERROR(0, "here (%d)", rv);
      }
      fprintf(wxs->file, "</column>\n");
    }
    else {
      DBG_ERROR(AE_LOGDOMAIN, "Column %d has no name", idx);
      GWEN_Buffer_free(tbuf);
      return data;
    }

    idx++;
    col=AQDB_Column_List_Next(col);
  }
  fprintf(wxs->file, "  </object>\n");
  GWEN_Buffer_free(tbuf);

  return NULL;
}



static int AE_Book_WriteXml_WriteTable(AE_BOOK *b, AE_BOOK_TABLE_TYPE tt, FILE *f, int openAndCloseTables) {
  int rv;
  struct WRITEXML_STRUCT wxs;
  void *dummy=NULL;
  GWEN_BUFFER *tbuf;
  const char *s;

  /* preset */
  wxs.book=b;
  wxs.dataBase=AE_Book_GetDataBase(b);
  wxs.columns=NULL;
  wxs.file=f;

  /* open table */
  if (openAndCloseTables) {
    rv=AE_Book_OpenTable(b, tt, AQDB_ACTION_FLAGS_READ);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  /* get column list */
  wxs.columns=AQDB_DB_GetColumnList(wxs.dataBase, AE_Book_GetTableId(b, tt));
  if (wxs.columns==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "No column list");
    if (openAndCloseTables)
      AE_Book_CloseTable(b, tt, 0);
    return GWEN_ERROR_INTERNAL;
  }

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);

  s=AE_Book_GetTableName(b, tt);
  if (s && *s) {
    fprintf(f, "<table name=\"");
    GWEN_Text_EscapeXmlToBuffer(s, tbuf);
    fprintf(f, "%s", GWEN_Buffer_GetStart(tbuf));
    GWEN_Buffer_Reset(tbuf);
    fprintf(f, "\">\n");

    /* dump every object */
    rv=AE_Book_ForEveryQueryObject(b,
				   tt,
				   0,
				   NULL, /* no query expression */
				   AE_Book__WriteXml_Callback,
				   (void*) &wxs,
				   &dummy);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_Buffer_free(tbuf);
      AQDB_Column_List_free(wxs.columns);
      if (openAndCloseTables)
	AE_Book_CloseTable(b, tt, 0);
      return rv;
    }
    fprintf(f, "</table>\n");
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "No database name");
    GWEN_Buffer_free(tbuf);
    AE_Book_CloseTable(b, tt, 0);
    return GWEN_ERROR_INTERNAL;
  }

  GWEN_Buffer_free(tbuf);
  AQDB_Column_List_free(wxs.columns);
  if (openAndCloseTables)
    AE_Book_CloseTable(b, tt, 0);

  return 0;
}



int AE_Book_WriteXml(AE_BOOK *b, const char *fname, int openAndCloseTables) {
  FILE *f;
  AE_BOOK_TABLE_TYPE tt;
  int i;
  int rv;

  f=fopen(fname, "w");
  if (f==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Error on fopen(%s): %s (%d)",
              fname, strerror(errno), errno);
    return GWEN_ERROR_IO;
  }

  fprintf(f, "<?xml ?>\n");
  fprintf(f, "<aqfinance>\n");
  for (i=0; ; i++) {
    tt=_tableList[i];
    if (tt==AE_Book_TableType_Last)
      break;
    rv=AE_Book_WriteXml_WriteTable(b, tt, f, openAndCloseTables);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      fclose(f);
      return rv;
    }
  }
  fprintf(f, "</aqfinance>\n");
  if (fclose(f)) {
    DBG_ERROR(AE_LOGDOMAIN, "Error on fclose(%s): %s (%d)",
	      fname, strerror(errno), errno);
    return GWEN_ERROR_IO;
  }

  return 0;
}




