/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/



#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "dbxmlctx_p.h"
#include "g_document_l.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/text.h>



GWEN_INHERIT(GWEN_XML_CONTEXT, AE_DBXMLCTX)




GWEN_XML_CONTEXT *AE_DbXmlCtx_new(uint32_t flags,
				  AE_BOOK *b,
				  int doOpenAndCloseTables) {
  GWEN_XML_CONTEXT *ctx;
  AE_DBXMLCTX *xctx;
  AE_DBGROUP *g;

  /* create base object */
  ctx=GWEN_XmlCtx_new(flags);
  assert(ctx);

  /* create and assign extension */
  GWEN_NEW_OBJECT(AE_DBXMLCTX, xctx);
  assert(xctx);
  GWEN_INHERIT_SETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx, xctx,
		       AE_DbXmlCtx_FreeData);
  xctx->book=b;
  xctx->doOpenAndCloseTables=doOpenAndCloseTables;

  /* set virtual functions */
  GWEN_XmlCtx_SetStartTagFn(ctx, AE_DbXmlCtx_StartTag);
  GWEN_XmlCtx_SetEndTagFn(ctx, AE_DbXmlCtx_EndTag);
  GWEN_XmlCtx_SetAddDataFn(ctx, AE_DbXmlCtx_AddData);
  GWEN_XmlCtx_SetAddCommentFn(ctx, AE_DbXmlCtx_AddComment);
  GWEN_XmlCtx_SetAddAttrFn(ctx, AE_DbXmlCtx_AddAttr);

  /* create initial group */
  g=AE_DbGroup_Document_new("DB_FILE", NULL, ctx);
  assert(g);
  AE_DbXmlCtx_SetCurrentGroup(ctx, g);


  /* return base object */
  return ctx;
}



GWENHYWFAR_CB
void AE_DbXmlCtx_FreeData(void *bp, void *p) {
  AE_DBXMLCTX *xctx;
  AE_DBGROUP *g;

  xctx=(AE_DBXMLCTX*)p;

  g=xctx->currentGroup;
  while (g) {
    AE_DBGROUP *gParent;

    gParent=AE_DbGroup_GetParent(g);
    AE_DbGroup_free(g);
    g=gParent;
  }

  free(xctx->currentTagName);
  GWEN_DB_Group_free(xctx->dbCurrentAttribs);

  GWEN_FREE_OBJECT(xctx);
}



AE_DBGROUP *AE_DbXmlCtx_GetCurrentGroup(const GWEN_XML_CONTEXT *ctx) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  return xctx->currentGroup;
}



void AE_DbXmlCtx_SetCurrentGroup(GWEN_XML_CONTEXT *ctx, AE_DBGROUP *g){
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  xctx->currentGroup=g;
}



const char *AE_DbXmlCtx_GetCurrentTagName(const GWEN_XML_CONTEXT *ctx) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  return xctx->currentTagName;
}



void AE_DbXmlCtx_SetCurrentTagName(GWEN_XML_CONTEXT *ctx, const char *s) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  free(xctx->currentTagName);
  if (s) xctx->currentTagName=strdup(s);
  else xctx->currentTagName=NULL;
}



GWEN_DB_NODE *AE_DbXmlCtx_GetCurrentAttributes(const GWEN_XML_CONTEXT *ctx) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  return xctx->dbCurrentAttribs;
}



AE_BOOK *AE_DbXmlCtx_GetBook(const GWEN_XML_CONTEXT *ctx) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  return xctx->book;
}



int AE_DbXmlCtx_GetDoOpenAndCloseTables(const GWEN_XML_CONTEXT *ctx) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  return xctx->doOpenAndCloseTables;
}



int AE_DbXmlCtx_SanitizeData(GWEN_XML_CONTEXT *ctx,
			       const char *data,
			       GWEN_BUFFER *buf) {
  const uint8_t *p;
  uint8_t *dst;
  uint8_t *src;
  unsigned int size;
  unsigned int i;
  int lastWasBlank;
  uint8_t *lastBlankPos;
  uint32_t bStart=0;

  if (GWEN_Text_UnescapeXmlToBuffer(data, buf)) {
    DBG_INFO(AE_LOGDOMAIN, "here");
    return GWEN_ERROR_BAD_DATA;
  }

  dst=(uint8_t*)GWEN_Buffer_GetStart(buf);
  src=dst;

  /* skip leading blanks */
  while(*src && (*src<33 || *src>=127))
    src++;

  p=src;
  bStart=src-((uint8_t*)GWEN_Buffer_GetStart(buf));
  size=GWEN_Buffer_GetUsedBytes(buf)-bStart;
  lastWasBlank=0;
  lastBlankPos=0;

  for (i=0; i<size; i++) {
    uint8_t c;

    c=*p;
    if (c<32 || c>=127)
      c=32;

    /* remember next loop whether this char was a blank */
    if (c==32) {
      if (!lastWasBlank) {
	/* store only one blank */
	lastWasBlank=1;
	lastBlankPos=dst;
	*(dst++)=c;
      }
    }
    else {
      lastWasBlank=0;
      lastBlankPos=0;
      *(dst++)=c;
    }
    p++;
  }

  /* remove trailing blanks */
  if (lastBlankPos!=0)
    dst=lastBlankPos;

  size=dst-(uint8_t*)GWEN_Buffer_GetStart(buf);
  GWEN_Buffer_Crop(buf, 0, size);

  return 0;
}










int AE_DbXmlCtx_StartTag(GWEN_XML_CONTEXT *ctx, const char *tagName) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  DBG_INFO(AE_LOGDOMAIN, "Starting tag [%s]", tagName);

  /* store for later, do nothing more here */
  AE_DbXmlCtx_SetCurrentTagName(ctx, tagName);
  GWEN_DB_Group_free(xctx->dbCurrentAttribs);
  xctx->dbCurrentAttribs=GWEN_DB_Group_new("attribs");

  return 0;
}



int AE_DbXmlCtx_EndTag(GWEN_XML_CONTEXT *ctx, int closing) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  if (closing) {
    /* just ignore empty tags which are closed immediately */
    DBG_INFO(AE_LOGDOMAIN, "Closing empty tag [%s]",
	     (xctx->currentTagName)?xctx->currentTagName:"<noname>");
    return 0;
  }

  if (xctx->currentTagName==NULL) {
    DBG_INFO(AE_LOGDOMAIN, "No tag name, malformed OFX file");
    return GWEN_ERROR_BAD_DATA;
  }

  DBG_INFO(AE_LOGDOMAIN, "Completed tag [%s]", xctx->currentTagName);

  if (xctx->currentGroup) {
    if (*(xctx->currentTagName)=='/') {
      int rv;

      /* it is a closing tag, call EndTagFn */
      DBG_INFO(AE_LOGDOMAIN,
	       "Calling %s->EndTag(%s)",
	       AE_DbGroup_GetGroupName(xctx->currentGroup),
	       xctx->currentTagName);
      rv=AE_DbGroup_EndTag(xctx->currentGroup, xctx->currentTagName+1);
      if (rv<0) {
	if (rv!=GWEN_ERROR_NOT_IMPLEMENTED) {
	  DBG_INFO(AE_LOGDOMAIN,
		   "Error in EndTag(%s) for [%s]",
		   AE_DbGroup_GetGroupName(xctx->currentGroup),
		   xctx->currentTagName);
	  return rv;
	}
      }
      else if (rv==1) {
        AE_DBGROUP *g;
	AE_DBGROUP *gParent;

	/* pop current group from stack */
	g=xctx->currentGroup;
	gParent=AE_DbGroup_GetParent(g);
	xctx->currentGroup=gParent;
	if (gParent) {
	  DBG_INFO(AE_LOGDOMAIN,
		   "Calling %s->EndSubGroup(%s)",
		   AE_DbGroup_GetGroupName(gParent),
                   AE_DbGroup_GetGroupName(g));
	  AE_DbGroup_EndSubGroup(gParent, g);
	}
	AE_DbGroup_free(g);
	GWEN_XmlCtx_DecDepth(ctx);
      }
    }
    else {
      int rv;

      /* it is an opening tag, call StartTagFn */
      DBG_INFO(AE_LOGDOMAIN,
	       "Calling %s->StartTag(%s)",
	       AE_DbGroup_GetGroupName(xctx->currentGroup),
	       xctx->currentTagName);
      rv=AE_DbGroup_StartTag(xctx->currentGroup, xctx->currentTagName);
      if (rv<0) {
	if (rv!=GWEN_ERROR_NOT_IMPLEMENTED) {
	  DBG_INFO(AE_LOGDOMAIN,
		   "Error in StartTag(%s) for [%s]",
		   AE_DbGroup_GetGroupName(xctx->currentGroup),
		   xctx->currentTagName);
	  return rv;
	}
      }
    }
  }

  return 0;
}






int AE_DbXmlCtx_AddData(GWEN_XML_CONTEXT *ctx, const char *data) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  if (xctx->currentGroup) {
    int rv;
    GWEN_BUFFER *tbuf;

    DBG_INFO(AE_LOGDOMAIN,
	     "Calling %s->AddData()",
	     AE_DbGroup_GetGroupName(xctx->currentGroup));
    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    if (GWEN_Text_UnescapeXmlToBuffer(data, tbuf)) {
      DBG_INFO(AE_LOGDOMAIN, "here");
      GWEN_Buffer_free(tbuf);
      return GWEN_ERROR_BAD_DATA;
    }

    rv=AE_DbGroup_AddData(xctx->currentGroup, GWEN_Buffer_GetStart(tbuf));
    if (rv<0) {
      if (rv!=GWEN_ERROR_NOT_IMPLEMENTED) {
	DBG_INFO(AE_LOGDOMAIN,
		 "Error in AddData(%s)",
		 AE_DbGroup_GetGroupName(xctx->currentGroup));
	GWEN_Buffer_free(tbuf);
	return rv;
      }
    }
    GWEN_Buffer_free(tbuf);
  }

  return 0;
}



int AE_DbXmlCtx_AddComment(GWEN_XML_CONTEXT *ctx, const char *data) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  /* ignore comments */
  return 0;
}



int AE_DbXmlCtx_AddAttr(GWEN_XML_CONTEXT *ctx,
			const char *attrName,
			const char *attrData) {
  AE_DBXMLCTX *xctx;

  assert(ctx);
  xctx=GWEN_INHERIT_GETDATA(GWEN_XML_CONTEXT, AE_DBXMLCTX, ctx);
  assert(xctx);

  if (xctx->currentGroup) {
    assert(xctx->dbCurrentAttribs);
    GWEN_DB_SetCharValue(xctx->dbCurrentAttribs,
			 GWEN_DB_FLAGS_DEFAULT,
			 attrName, attrData);
  }

  /* ignore attributes */
  return 0;
}





