/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "g_column_p.h"
#include "dbxmlctx_l.h"

#include "g_object_l.h"
#include "g_generic_l.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>



GWEN_INHERIT(AE_DBGROUP, AE_DBGROUP_COLUMN)



AE_DBGROUP *AE_DbGroup_Column_new(const char *groupName,
				  AE_DBGROUP *parent,
				  GWEN_XML_CONTEXT *ctx,
				  AQDB_COLUMN *column,
				  int idx,
				  AQDB_OBJECT *o) {
  AE_DBGROUP *g;
  AE_DBGROUP_COLUMN *xg;

  assert(parent);
  assert(column);

  /* create base group */
  g=AE_DbGroup_Generic_new(groupName, parent, ctx);
  assert(g);

  GWEN_NEW_OBJECT(AE_DBGROUP_COLUMN, xg);
  GWEN_INHERIT_SETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g, xg, AE_DbGroup_Column_FreeData);
  xg->column=column;
  xg->index=idx;
  xg->object=o;

  /* set virtual functions */
  AE_DbGroup_SetStartTagFn(g, AE_DbGroup_Column_StartTag);
  AE_DbGroup_SetAddDataFn(g, AE_DbGroup_Column_AddData);

  return g;
}



void GWENHYWFAR_CB AE_DbGroup_Column_FreeData(void *bp, void *p) {
  AE_DBGROUP_COLUMN *xg;

  xg=(AE_DBGROUP_COLUMN*) p;
  GWEN_FREE_OBJECT(xg);
}



int AE_DbGroup_Column_StartTag(AE_DBGROUP *g, const char *tagName) {
  DBG_ERROR(AE_LOGDOMAIN, "Unexpected group [%s]", tagName);
  return GWEN_ERROR_BAD_DATA;
}



static int AE_DbGroup_Column_GetInt(AE_DBGROUP *g, const char *s) {
  AE_DBGROUP_COLUMN *xg;
  long int val;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  if (1!=sscanf(s, "%ld", &val)) {
    DBG_ERROR(AE_LOGDOMAIN, "Bad data in XML file (not an int)");
    return GWEN_ERROR_BAD_DATA;
  }

  return AQDB_Object_SetFieldInt64(xg->object, xg->index, val);
}



static int AE_DbGroup_Column_GetString(AE_DBGROUP *g, const char *s) {
  AE_DBGROUP_COLUMN *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  return AQDB_Object_SetFieldString(xg->object, xg->index, s);
}



static int AE_DbGroup_Column_GetBin(AE_DBGROUP *g, const char *s) {
  AE_DBGROUP_COLUMN *xg;
  GWEN_BUFFER *tbuf;
  int rv;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=GWEN_Text_FromHexBuffer(s, tbuf);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  rv=AQDB_Object_SetField(xg->object, xg->index,
			  (const uint8_t*) GWEN_Buffer_GetStart(tbuf), GWEN_Buffer_GetUsedBytes(tbuf));
  GWEN_Buffer_free(tbuf);
  return rv;
}



static int AE_DbGroup_Column_GetDate(AE_DBGROUP *g, const char *s) {
  AE_DBGROUP_COLUMN *xg;
  GWEN_DATE *d;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  d=GWEN_Date_fromStringWithTemplate(s, "YYYYMMDD");
  if (d) {
    int rv;

    rv=AQDB_Object_SetFieldDate(xg->object, xg->index, d);
    GWEN_Date_free(d);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }
  return 0;
}



static int AE_DbGroup_Column_GetTimestamp(AE_DBGROUP *g, const char *s) {
  AE_DBGROUP_COLUMN *xg;
  GWEN_TIME *d;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  d=GWEN_Time_fromString(s, "YYYYMMDD-hhmmss");
  if (d) {
    int rv;

    rv=AQDB_Object_SetFieldTimestamp(xg->object, xg->index, d);
    GWEN_Time_free(d);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }
  return 0;
}



static int AE_DbGroup_Column_GetAmount(AE_DBGROUP *g, const char *s) {
  AE_DBGROUP_COLUMN *xg;
  AQDB_VALUE *d;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  d=AQDB_Value_fromString(s);
  if (d) {
    int rv;

    rv=AQDB_Object_SetFieldValue(xg->object, xg->index, d);
    AQDB_Value_free(d);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }
  return 0;
}



int AE_DbGroup_Column_AddData(AE_DBGROUP *g, const char *data) {
  AE_DBGROUP_COLUMN *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_COLUMN, g);
  assert(xg);

  switch(AQDB_Column_GetType(xg->column)) {
  case AQDB_DataType_Int:
    return AE_DbGroup_Column_GetInt(g, data);
  case AQDB_DataType_String:
    return AE_DbGroup_Column_GetString(g, data);
  case AQDB_DataType_Bin:
    return AE_DbGroup_Column_GetBin(g, data);
  case AQDB_DataType_Date:
    return AE_DbGroup_Column_GetDate(g, data);
  case AQDB_DataType_Timestamp:
    return AE_DbGroup_Column_GetTimestamp(g, data);
  case AQDB_DataType_Amount:
    return AE_DbGroup_Column_GetAmount(g, data);

  case AQDB_DataType_Unknown:
  case AQDB_DataType_None:
    DBG_ERROR(AE_LOGDOMAIN, "Unhandled data type %d", AQDB_Column_GetType(xg->column));
    return GWEN_ERROR_BAD_DATA;
  }

  return 0;
}



