/***************************************************************************
 begin       : Wed Mar 17 2010
 copyright   : (C) 2010 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "g_table_p.h"
#include "g_object_l.h"
#include "dbxmlctx_l.h"

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>


GWEN_INHERIT(AE_DBGROUP, AE_DBGROUP_TABLE)




AE_DBGROUP *AE_DbGroup_Table_new(const char *groupName,
				 AE_DBGROUP *parent,
				 GWEN_XML_CONTEXT *ctx,
				 const char *tableName) {
  AE_DBGROUP *g;
  AE_DBGROUP_TABLE *xg;
  AE_BOOK *b;
  int rv;

  g=AE_DbGroup_Generic_new(groupName, parent, ctx);
  assert(g);

  GWEN_NEW_OBJECT(AE_DBGROUP_TABLE, xg);
  GWEN_INHERIT_SETDATA(AE_DBGROUP, AE_DBGROUP_TABLE, g, xg, AE_DbGroup_Table_FreeData);

  /* set virtual functions */
  AE_DbGroup_SetStartTagFn(g, AE_DbGroup_Table_StartTag);
  AE_DbGroup_SetEndTagFn(g, AE_DbGroup_Table_EndTag);
  AE_DbGroup_SetEndSubGroupFn(g, AE_DbGroup_Table_EndSubGroup);

  b=AE_DbXmlCtx_GetBook(ctx);
  assert(b);

  xg->tableType=AE_Book_GetTableType(b, tableName);
  if (xg->tableType==AE_Book_TableType_Unknown) {
    DBG_ERROR(AE_LOGDOMAIN, "Invalid table name [%s]", tableName);
    AE_DbGroup_free(g);
    return NULL;
  }

  if (AE_DbXmlCtx_GetDoOpenAndCloseTables(ctx)) {
    rv=AE_Book_OpenTable(b, xg->tableType, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
    if (rv<0) {
      rv=AE_Book_CreateTable(b, xg->tableType, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
      if (rv==0)
	rv=AE_Book_OpenTable(b, xg->tableType, AQDB_ACTION_FLAGS_READ | AQDB_ACTION_FLAGS_WRITE);
    }
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      AE_DbGroup_free(g);
      return NULL;
    }
  }

  xg->columns=AE_Book_GetColumnList(b, xg->tableType);
  if (xg->columns==NULL) {
    DBG_INFO(AE_LOGDOMAIN, "here");
    AE_DbGroup_free(g);
    return NULL;
  }

  return g;
}



void GWENHYWFAR_CB AE_DbGroup_Table_FreeData(void *bp, void *p) {
  AE_DBGROUP_TABLE *xg;

  xg=(AE_DBGROUP_TABLE*) p;

  if (xg->columns)
    AQDB_Column_List_free(xg->columns);

  GWEN_FREE_OBJECT(xg);
}



AQDB_COLUMN_LIST *AE_DbGroup_Table_GetColumns(const AE_DBGROUP *g) {
  AE_DBGROUP_TABLE *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_TABLE, g);
  assert(xg);

  return xg->columns;
}



AE_BOOK_TABLE_TYPE AE_DbGroup_Table_GetTableType(const AE_DBGROUP *g) {
  AE_DBGROUP_TABLE *xg;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_TABLE, g);
  assert(xg);

  return xg->tableType;
}



AQDB_ID AE_DbGroup_Table_GetTableId(const AE_DBGROUP *g) {
  AE_DBGROUP_TABLE *xg;
  GWEN_XML_CONTEXT *ctx;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_TABLE, g);
  assert(xg);

  ctx=AE_DbGroup_GetXmlContext(g);
  return AE_Book_GetTableId(AE_DbXmlCtx_GetBook(ctx), xg->tableType);
}



int AE_DbGroup_Table_StartTag(AE_DBGROUP *g, const char *tagName) {
  AE_DBGROUP *gNew=NULL;
  GWEN_XML_CONTEXT *ctx;

  assert(g);

  ctx=AE_DbGroup_GetXmlContext(g);

  if (strcasecmp(tagName, "object")==0)
    gNew=AE_DbGroup_Object_new(tagName, g, ctx);
  else {
    DBG_WARN(AE_LOGDOMAIN,
	     "Unexpected group [%s]", tagName);
    return GWEN_ERROR_BAD_DATA;
  }

  if (gNew) {
    AE_DbXmlCtx_SetCurrentGroup(ctx, gNew);
    GWEN_XmlCtx_IncDepth(ctx);
  }

  return 0;
}



int AE_DbGroup_Table_EndTag(AE_DBGROUP *g, const char *tagName) {
  AE_DBGROUP_TABLE *xg;
  GWEN_XML_CONTEXT *ctx;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_TABLE, g);
  assert(xg);

  ctx=AE_DbGroup_GetXmlContext(g);

  if (strcasecmp(AE_DbGroup_GetGroupName(g), tagName)!=0) {
    DBG_INFO(AE_LOGDOMAIN,
	     "Tag [%s] does not close [%s], ignoring",
	     tagName, AE_DbGroup_GetGroupName(g));
    return GWEN_ERROR_BAD_DATA;
  }

  if (AE_DbXmlCtx_GetDoOpenAndCloseTables(ctx)) {
    int rv;

    rv=AE_Book_CloseTable(AE_DbXmlCtx_GetBook(ctx), xg->tableType, 0);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 1;
}



int AE_DbGroup_Table_EndSubGroup(AE_DBGROUP *g, AE_DBGROUP *sg) {
  AE_DBGROUP_TABLE *xg;
  GWEN_XML_CONTEXT *ctx;
  const char *s;

  assert(g);
  xg=GWEN_INHERIT_GETDATA(AE_DBGROUP, AE_DBGROUP_TABLE, g);
  assert(xg);

  ctx=AE_DbGroup_GetXmlContext(g);

  s=AE_DbGroup_GetGroupName(sg);
  assert(s && *s);
  if (strcasecmp(s, "object")==0) {
    AE_BOOK *b;
    AQDB_OBJECT *o;
    int rv;

    b=AE_DbXmlCtx_GetBook(ctx);
    assert(b);
    o=AE_DbGroup_Object_GetObject(sg);
    if (o==NULL) {
      DBG_ERROR(AE_LOGDOMAIN, "No object");
      return GWEN_ERROR_INTERNAL;
    }

    rv=AQDB_DB_AddObject(AE_Book_GetDataBase(b),
			 AE_Book_GetTableId(b, xg->tableType),
                         o);
    if (rv<0) {
      DBG_ERROR(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
  }

  return 0;
}




