/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "csv_p.h"

#include <gwenhywfar/syncio_file.h>
#include <gwenhywfar/debug.h>



int AF_Report_Builder_Csv_AddColumn(GWEN_SYNCIO *sio,
                                    AF_REPORT_AXIS *ax,
                                    AF_REPORT_DATAFIELD *df,
                                    const AF_REPORT_CFG_CSV *cfg) {
  int rv;
  const char *units;

  GWEN_SyncIo_WriteString(sio, "\"");

  units=AF_Report_Axis_GetUnits(ax);

  switch(AF_Report_Axis_GetDataType(ax)) {
  case AQDB_DataType_Int: {
    char numbuf[32];

    snprintf(numbuf, sizeof(numbuf)-1, "%d", AF_Report_DataField_GetDataAsInt(df));
    numbuf[sizeof(numbuf)-1]=0;
    GWEN_SyncIo_WriteString(sio, numbuf);

    /* add units */
    if (units && *units) {
      GWEN_SyncIo_WriteString(sio, " ");
      GWEN_SyncIo_WriteString(sio, units);
    }
    break;
  }

  case AQDB_DataType_String: {
    const char *s;

    s=AF_Report_DataField_GetData(df);
    if (s && *s)
      GWEN_SyncIo_WriteString(sio, s);
    break;
  }

  case AQDB_DataType_Amount: {
    const AQDB_VALUE *v;

    v=AF_Report_DataField_GetDataAsValue(df);
    if (v) {
      GWEN_BUFFER *tbuf;

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      if (AF_Report_Cfg_Csv_GetFlags(cfg) & AF_REPORT_CFG_CSV_FLAG_VALUE_LOCAL)
        AQDB_Value_toLocalHumanReadableString(v, tbuf, AF_Report_Cfg_Csv_GetValuePrecision(cfg));
      else
        AQDB_Value_toHumanReadableString(v, tbuf, AF_Report_Cfg_Csv_GetValuePrecision(cfg));
      GWEN_SyncIo_WriteString(sio, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);

      /* add units */
      if (units && *units) {
        GWEN_SyncIo_WriteString(sio, " ");
        GWEN_SyncIo_WriteString(sio, units);
      }
    }
    break;
  }

  case AQDB_DataType_Date: {
    const GWEN_DATE *dt;

    dt=AF_Report_DataField_GetDataAsDate(df);
    if (dt) {
      GWEN_BUFFER *tbuf;
      const char *tmpl;

      tmpl=AF_Report_Cfg_Csv_GetDateFormat(cfg);
      if (!(tmpl && *tmpl))
        tmpl=I18N("YYYY/MM/DD");
      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      rv=GWEN_Date_toStringWithTemplate(dt, tmpl, tbuf);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_Buffer_free(tbuf);
        return rv;
      }
      GWEN_SyncIo_WriteString(sio, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    break;
  }

  default:
    DBG_ERROR(AE_LOGDOMAIN, "Unhandled data type [%d]", AF_Report_Axis_GetDataType(ax))
    break;
  }


  GWEN_SyncIo_WriteString(sio, "\"");

  return 0;
}



int AF_Report_Builder_Csv(const char *path,
                          const AF_REPORT_DATA *rd,
                          const AF_REPORT_CFG_CSV *cfg) {
  AF_REPORT_AXIS_LIST *al;
  AF_REPORT_AXIS *ax;
  AF_REPORT_DATAPAIR_LIST *dpl;
  AF_REPORT_DATAPAIR *dp;
  GWEN_SYNCIO *sio;
  int rv;

  sio=GWEN_SyncIo_File_new(path, GWEN_SyncIo_File_CreationMode_CreateAlways);
  GWEN_SyncIo_AddFlags(sio,
                       GWEN_SYNCIO_FILE_FLAGS_READ |
                       GWEN_SYNCIO_FILE_FLAGS_WRITE |
                       GWEN_SYNCIO_FILE_FLAGS_UREAD |
                       GWEN_SYNCIO_FILE_FLAGS_UWRITE |
                       GWEN_SYNCIO_FILE_FLAGS_GREAD |
                       GWEN_SYNCIO_FILE_FLAGS_GWRITE);
  rv=GWEN_SyncIo_Connect(sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_free(sio);
    return rv;
  }

  dpl=AF_Report_Data_GetDataPairList(rd);
  assert(dpl);

  al=AF_Report_Data_GetAxisList(rd);
  assert(al);

  /* write title line */
  ax=AF_Report_Axis_List_First(al);
  while(ax) {
    const char *s;

    GWEN_SyncIo_WriteString(sio, "\"");
    s=AF_Report_Axis_GetLabel(ax);
    if (s && *s)
      GWEN_SyncIo_WriteString(sio, s);
    GWEN_SyncIo_WriteString(sio, "\"");

    ax=AF_Report_Axis_List_Next(ax);
    if (ax)
      GWEN_SyncIo_WriteString(sio, ";");
  }
  GWEN_SyncIo_WriteString(sio, "\n");

  dp=AF_Report_DataPair_List_First(dpl);
  while(dp) {
    AF_REPORT_DATAFIELD *df;

    ax=AF_Report_Axis_List_First(al);
    df=AF_Report_DataField_List_First(AF_Report_DataPair_GetDataFieldList(dp));
    while(ax && df) {
      rv=AF_Report_Builder_Csv_AddColumn(sio, ax, df, cfg);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_SyncIo_Disconnect(sio);
        GWEN_SyncIo_free(sio);
        return rv;
      }
      ax=AF_Report_Axis_List_Next(ax);
      df=AF_Report_DataField_List_Next(df);
      if (ax && df)
        GWEN_SyncIo_WriteString(sio, ";");
    }
    GWEN_SyncIo_WriteString(sio, "\n");

    /* next */
    dp=AF_Report_DataPair_List_Next(dp);
  }

  rv=GWEN_SyncIo_Disconnect(sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_free(sio);
    return rv;
  }
  GWEN_SyncIo_free(sio);

  return 0;
}





