/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "htmlbase_p.h"

#include <gwenhywfar/syncio_file.h>
#include <gwenhywfar/debug.h>



int AF_Report_Builder_HtmlBase_AddColumn(GWEN_SYNCIO *sio,
                                         AF_REPORT_AXIS *ax,
                                         AF_REPORT_DATAFIELD *df,
                                         const AF_REPORT_CFG_HTMLBASE *cfg) {
  int rv;
  const char *units;

  units=AF_Report_Axis_GetUnits(ax);

  switch(AF_Report_Axis_GetDataType(ax)) {
  case AQDB_DataType_Int: {
    char numbuf[32];

    snprintf(numbuf, sizeof(numbuf)-1, "%d", AF_Report_DataField_GetDataAsInt(df));
    numbuf[sizeof(numbuf)-1]=0;
    GWEN_SyncIo_WriteString(sio, numbuf);

    /* add units */
    if (units && *units) {
      GWEN_SyncIo_WriteString(sio, " ");
      GWEN_SyncIo_WriteString(sio, units);
    }
    break;
  }

  case AQDB_DataType_String: {
    const char *s;

    s=AF_Report_DataField_GetData(df);
    if (s && *s)
      GWEN_SyncIo_WriteString(sio, s);
    break;
  }

  case AQDB_DataType_Amount: {
    const AQDB_VALUE *v;

    v=AF_Report_DataField_GetDataAsValue(df);
    if (v) {
      GWEN_BUFFER *tbuf;

      if (AQDB_Value_IsNegative(v) &&
          (AF_Report_Cfg_HtmlBase_GetFlags(cfg) & AF_REPORT_CFG_HTMLBASE_FLAG_RED_NEGATIVES))
        GWEN_SyncIo_WriteString(sio, "<font color=\"red\">");

      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      if (AF_Report_Cfg_HtmlBase_GetFlags(cfg) & AF_REPORT_CFG_HTMLBASE_FLAG_VALUE_LOCAL)
        AQDB_Value_toLocalHumanReadableString(v, tbuf, AF_Report_Cfg_HtmlBase_GetValuePrecision(cfg));
      else
        AQDB_Value_toHumanReadableString(v, tbuf, AF_Report_Cfg_HtmlBase_GetValuePrecision(cfg));
      GWEN_SyncIo_WriteString(sio, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);

      /* add units */
      if (units && *units) {
        GWEN_SyncIo_WriteString(sio, " ");
        GWEN_SyncIo_WriteString(sio, units);
      }

      if (AQDB_Value_IsNegative(v) &&
          (AF_Report_Cfg_HtmlBase_GetFlags(cfg) & AF_REPORT_CFG_HTMLBASE_FLAG_RED_NEGATIVES))
        GWEN_SyncIo_WriteString(sio, "</font>");

    }
    break;
  }

  case AQDB_DataType_Date: {
    const GWEN_DATE *dt;

    dt=AF_Report_DataField_GetDataAsDate(df);
    if (dt) {
      GWEN_BUFFER *tbuf;
      const char *tmpl;

      tmpl=AF_Report_Cfg_HtmlBase_GetDateFormat(cfg);
      if (!(tmpl && *tmpl))
        tmpl=I18N("YYYY/MM/DD");
      tbuf=GWEN_Buffer_new(0, 256, 0, 1);
      rv=GWEN_Date_toStringWithTemplate(dt, tmpl, tbuf);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_Buffer_free(tbuf);
        return rv;
      }
      GWEN_SyncIo_WriteString(sio, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_free(tbuf);
    }

    break;
  }

  default:
    DBG_ERROR(AE_LOGDOMAIN, "Unhandled data type [%d]", AF_Report_Axis_GetDataType(ax))
    break;
  }


  return 0;
}



int AF_Report_Builder_HtmlBase_toSio(GWEN_SYNCIO *sio,
				     const AF_REPORT_DATA *rd,
				     const AF_REPORT_CFG_HTMLBASE *cfg) {
  AF_REPORT_AXIS_LIST *al;
  AF_REPORT_AXIS *ax;
  AF_REPORT_DATAPAIR_LIST *dpl;
  AF_REPORT_DATAPAIR *dp;
  int rv;
  uint32_t flags;
  const char *s;
  int i;
  int w;

  dpl=AF_Report_Data_GetDataPairList(rd);
  assert(dpl);

  al=AF_Report_Data_GetAxisList(rd);
  assert(al);

  flags=AF_Report_Cfg_HtmlBase_GetFlags(cfg);

  /* write full header and stuff */
  if (flags & AF_REPORT_CFG_HTMLBASE_FLAG_FULL_DOC) {
    GWEN_SyncIo_WriteString(sio,
                            "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" "
                            "\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
                            "<html>\n"
                            "<head>\n"
                            "<meta name=\"generator\" content=\"AqFinance\" />\n"
                            "<meta http-equiv=\"content-type\" content=\"text/html; charset=utf8\" />\n");
    s=AF_Report_Data_GetTitle(rd);
    if (s && *s) {
      GWEN_SyncIo_WriteString(sio, "<title>");
      GWEN_SyncIo_WriteString(sio, s);
      GWEN_SyncIo_WriteString(sio, "</title>\n");
    }
    GWEN_SyncIo_WriteString(sio, "</head>\n<body>\n");
  }


  /* write title line */
  if (flags & AF_REPORT_CFG_HTMLBASE_FLAG_SHOW_TITLE) {
    s=AF_Report_Data_GetTitle(rd);
    if (s && *s) {
      GWEN_SyncIo_WriteString(sio, "<h1>");
      GWEN_SyncIo_WriteString(sio, s);
      GWEN_SyncIo_WriteString(sio, "</h1>\n");
    }

    s=AF_Report_Data_GetSubTitle(rd);
    if (s && *s) {
      GWEN_SyncIo_WriteString(sio, "<h2>");
      GWEN_SyncIo_WriteString(sio, s);
      GWEN_SyncIo_WriteString(sio, "</h2>\n");
    }
  }

  w=AF_Report_Cfg_HtmlBase_GetBorderWidth(cfg);
  if (w>0) {
    char numbuf[32];

    GWEN_SyncIo_WriteString(sio, "<table border=\"");
    snprintf(numbuf, sizeof(numbuf)-1, "%d", w);
    numbuf[sizeof(numbuf)-1]=0;
    GWEN_SyncIo_WriteString(sio, numbuf);
    GWEN_SyncIo_WriteString(sio, "\">");
  }
  else
    GWEN_SyncIo_WriteString(sio, "<table>");
  GWEN_SyncIo_WriteString(sio, "<tr>");


  /* write title line */
  ax=AF_Report_Axis_List_First(al);
  i=0;
  while(ax) {
    const char *s;

    s=AF_Report_Axis_GetLabel(ax);
    GWEN_SyncIo_WriteString(sio, "<th");
    switch(i) {
    case 0: w=AF_Report_Cfg_HtmlBase_GetColumnWidth0(cfg); break;
    case 1: w=AF_Report_Cfg_HtmlBase_GetColumnWidth1(cfg); break;
    case 2: w=AF_Report_Cfg_HtmlBase_GetColumnWidth2(cfg); break;
    case 3: w=AF_Report_Cfg_HtmlBase_GetColumnWidth3(cfg); break;
    default: w=-1;
    }
    if (w>0) {
      char numbuf[32];

      GWEN_SyncIo_WriteString(sio, " width=\"");
      snprintf(numbuf, sizeof(numbuf)-1, "%d", w);
      numbuf[sizeof(numbuf)-1]=0;
      GWEN_SyncIo_WriteString(sio, numbuf);
      GWEN_SyncIo_WriteString(sio, "\"");
    }
    GWEN_SyncIo_WriteString(sio, ">");
    if (s && *s)
      GWEN_SyncIo_WriteString(sio, s);
    GWEN_SyncIo_WriteString(sio, "</th>");

    i++;
    ax=AF_Report_Axis_List_Next(ax);
  }
  GWEN_SyncIo_WriteString(sio, "\n");

  dp=AF_Report_DataPair_List_First(dpl);
  while(dp) {
    AF_REPORT_DATAFIELD *df;

    GWEN_SyncIo_WriteString(sio, "<tr>");
    ax=AF_Report_Axis_List_First(al);
    df=AF_Report_DataField_List_First(AF_Report_DataPair_GetDataFieldList(dp));
    while(ax && df) {
      if (AF_Report_Axis_GetDataType(ax)==AQDB_DataType_Amount)
        GWEN_SyncIo_WriteString(sio, "<td align=\"right\">");
      else
        GWEN_SyncIo_WriteString(sio, "<td>");
      rv=AF_Report_Builder_HtmlBase_AddColumn(sio, ax, df, cfg);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        return rv;
      }
      GWEN_SyncIo_WriteString(sio, "</td>");

      ax=AF_Report_Axis_List_Next(ax);
      df=AF_Report_DataField_List_Next(df);
    }
    GWEN_SyncIo_WriteString(sio, "</tr>\n");

    /* next */
    dp=AF_Report_DataPair_List_Next(dp);
  }

  GWEN_SyncIo_WriteString(sio, "</table>");

  if (flags & AF_REPORT_CFG_HTMLBASE_FLAG_FULL_DOC)
    GWEN_SyncIo_WriteString(sio, "</body>\n</html>\n");

  return 0;
}



int AF_Report_Builder_HtmlBase(const char *path,
                               const AF_REPORT_DATA *rd,
                               const AF_REPORT_CFG_HTMLBASE *cfg) {
  GWEN_SYNCIO *sio;
  int rv;

  sio=GWEN_SyncIo_File_new(path, GWEN_SyncIo_File_CreationMode_CreateAlways);
  GWEN_SyncIo_AddFlags(sio,
                       GWEN_SYNCIO_FILE_FLAGS_READ |
                       GWEN_SYNCIO_FILE_FLAGS_WRITE |
                       GWEN_SYNCIO_FILE_FLAGS_UREAD |
                       GWEN_SYNCIO_FILE_FLAGS_UWRITE |
                       GWEN_SYNCIO_FILE_FLAGS_GREAD |
                       GWEN_SYNCIO_FILE_FLAGS_GWRITE);
  rv=GWEN_SyncIo_Connect(sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_free(sio);
    return rv;
  }

  rv=AF_Report_Builder_HtmlBase_toSio(sio, rd, cfg);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_Disconnect(sio);
    GWEN_SyncIo_free(sio);
    return rv;
  }

  /* done, close SIO */
  rv=GWEN_SyncIo_Disconnect(sio);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_SyncIo_free(sio);
    return rv;
  }
  GWEN_SyncIo_free(sio);

  return 0;
}








