/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2011 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#include "af_graph_p.h"

#include <gwenhywfar/debug.h>


GWEN_LIST_FUNCTIONS(AF_GRAPH, AF_Graph)
GWEN_LIST2_FUNCTIONS(AF_GRAPH, AF_Graph)




#define BORDER       12
#define SMALL_BORDER 8

#define MAX_TICK_LEVELS 5

#define MIN_DIST_FOR_TICKS 5
#define MIN_DIST_FOR_GRIDS 5


#define TICKDRAWFLAGS_MARKS  0x01
#define TICKDRAWFLAGS_GRIDS  0x02
#define TICKDRAWFLAGS_LABELS 0x04






int AF_Graph_CreatePenFromConfig(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig, const char *name) {
  GWEN_DB_NODE *db;
  uint32_t frontColour;
  int lineWidth;
  int dashType;
  int rv;

  db=GWEN_DB_GetGroup(dbConfig, 0, name);
  assert(db);
  frontColour=(uint32_t)GWEN_DB_GetIntValue(db, "frontColour", 0, (int)0x000000ff);
  lineWidth=GWEN_DB_GetIntValue(db, "lineWidth", 0, 1);
  dashType=GWEN_DB_GetIntValue(db, "dashType", 0, 0);

  rv=AF_Graphics_PenCreate(gr->graphics, frontColour, lineWidth, dashType);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return rv;
}



int AF_Graph_CreateFontFromConfig(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig, const char *name, int defaultFontSize) {
  GWEN_DB_NODE *db;
  const char *fontName;
  int fontSize;
  int fontSlant;
  int fontWeight;
  int rv;

  db=GWEN_DB_GetGroup(dbConfig, 0, name);
  assert(db);


  fontName=GWEN_DB_GetCharValue(db, "fontName", 0, "");
  fontSize=GWEN_DB_GetIntValue(db, "fontSize", 0, defaultFontSize);
  fontSlant=AF_Graphics_Slant_fromString(GWEN_DB_GetCharValue(db, "fontSlant", 0, "none"));
  fontWeight=AF_Graphics_Weight_fromString(GWEN_DB_GetCharValue(db, "fontWeight", 0, "none"));

  rv=AF_Graphics_FontCreate(gr->graphics, fontName, fontSize, fontSlant, fontWeight);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return rv;
}




int AF_Graph_CreatePensAndFonts_FromDb(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig) {
  int rv;
  AF_GRAPH_SUBGRAPH_LIST *sgList;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penTitle");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penTitle=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penSubTitle");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penSubTitle=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penAxisLabel");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penAxisLabel=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penAxis");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penAxis=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penGraph");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penGraph=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penGraphBackground");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penGraphBackground=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penGrid");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penGrid=rv;

  rv=AF_Graph_CreatePenFromConfig(gr, dbConfig, "penBackground");
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->penBackground=rv;



  rv=AF_Graph_CreateFontFromConfig(gr, dbConfig, "fontTitle", 16);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->fontTitle=rv;

  rv=AF_Graph_CreateFontFromConfig(gr, dbConfig, "fontSubTitle", 14);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->fontSubTitle=rv;

  rv=AF_Graph_CreateFontFromConfig(gr, dbConfig, "fontAxisLabel", 12);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->fontAxisLabel=rv;

  rv=AF_Graph_CreateFontFromConfig(gr, dbConfig, "fontTickLabel", 10);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }
  gr->fontTickLabel=rv;

  /* create pens for graphs */
  sgList=AF_Graph_DataSet_GetSubGraphList(gr->dataSet);
  if (sgList) {
    AF_GRAPH_SUBGRAPH *sg;

    sg=AF_Graph_SubGraph_List_First(sgList);
    while(sg) {

      rv=AF_Graphics_PenCreate(gr->graphics,
                               AF_Graph_SubGraph_GetColour(sg),
                               AF_Graph_SubGraph_GetLineWidth(sg),
                               AF_Graph_SubGraph_GetDashType(sg));
      if (rv<0) {
        DBG_INFO(0, "here (%d)", rv);
        return rv;
      }

      AF_Graph_SubGraph_SetPenId(sg, rv);

      sg=AF_Graph_SubGraph_List_Next(sg);
    }
  }

  return 0;
}






int AF_Graph_CreatePenFromSettings(AF_GRAPH *gr, const AF_GRAPH_SETTINGS_PEN *sp) {
  int rv;

  rv=AF_Graphics_PenCreate(gr->graphics,
                           AF_Graph_SettingsPen_GetFrontColour(sp),
                           AF_Graph_SettingsPen_GetLineWidth(sp),
                           AF_Graph_SettingsPen_GetDashType(sp));
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return rv;
}



int AF_Graph_CreateFontFromSettings(AF_GRAPH *gr, const AF_GRAPH_SETTINGS_FONT *sf) {
  int rv;

  rv=AF_Graphics_FontCreate(gr->graphics, "",
                            AF_Graph_SettingsFont_GetFontSize(sf),
                            AF_Graph_SettingsFont_GetFontSlant(sf),
                            AF_Graph_SettingsFont_GetFontWeight(sf));
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  return rv;
}




int AF_Graph_CreatePensAndFonts_FromSettings(AF_GRAPH *gr, const AF_GRAPH_SETTINGS *settings) {
  int rv;
  const AF_GRAPH_SETTINGS_PEN *pen;
  const AF_GRAPH_SETTINGS_FONT *fnt;
  const AF_GRAPH_SUBGRAPH_LIST *sgList;

  pen=AF_Graph_Settings_GetPenTitle(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penTitle=rv;
  }

  pen=AF_Graph_Settings_GetPenSubTitle(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penSubTitle=rv;
  }

  pen=AF_Graph_Settings_GetPenAxisLabel(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penAxisLabel=rv;
  }

  pen=AF_Graph_Settings_GetPenAxis(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penAxis=rv;
  }

  pen=AF_Graph_Settings_GetPenGraph(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penGraph=rv;
  }

  pen=AF_Graph_Settings_GetPenGraphBackground(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penGraphBackground=rv;
  }

  pen=AF_Graph_Settings_GetPenGrid(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penGrid=rv;
  }

  pen=AF_Graph_Settings_GetPenBackground(settings);
  if (pen) {
    rv=AF_Graph_CreatePenFromSettings(gr, pen);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->penBackground=rv;
  }



  fnt=AF_Graph_Settings_GetFontTitle(settings);
  if (fnt) {
    rv=AF_Graph_CreateFontFromSettings(gr, fnt);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->fontTitle=rv;
  }

  fnt=AF_Graph_Settings_GetFontSubTitle(settings);
  if (fnt) {
    rv=AF_Graph_CreateFontFromSettings(gr, fnt);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->fontSubTitle=rv;
  }

  fnt=AF_Graph_Settings_GetFontAxisLabel(settings);
  if (fnt) {
    rv=AF_Graph_CreateFontFromSettings(gr, fnt);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->fontAxisLabel=rv;
  }

  fnt=AF_Graph_Settings_GetFontTickLabel(settings);
  if (fnt) {
    rv=AF_Graph_CreateFontFromSettings(gr, fnt);
    if (rv<0) {
      DBG_INFO(0, "here (%d)", rv);
      return rv;
    }
    gr->fontTickLabel=rv;
  }

  /* create pens for graphs */
  sgList=AF_Graph_DataSet_GetSubGraphList(gr->dataSet);
  if (sgList) {
    AF_GRAPH_SUBGRAPH *sg;

    sg=AF_Graph_SubGraph_List_First(sgList);
    while(sg) {

      rv=AF_Graphics_PenCreate(gr->graphics,
                               AF_Graph_SubGraph_GetColour(sg),
                               AF_Graph_SubGraph_GetLineWidth(sg),
                               AF_Graph_SubGraph_GetDashType(sg));
      if (rv<0) {
        DBG_INFO(0, "here (%d)", rv);
        return rv;
      }

      AF_Graph_SubGraph_SetPenId(sg, rv);

      sg=AF_Graph_SubGraph_List_Next(sg);
    }
  }

  return 0;
}



void AF_Graph_ReleasePensAndFonts(AF_GRAPH *gr) {
  if (gr->penTitle>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penTitle);
    gr->penTitle=-1;
  }

  if (gr->penSubTitle>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penSubTitle);
    gr->penSubTitle=-1;
  }

  if (gr->penAxisLabel>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penAxisLabel);
    gr->penAxisLabel=-1;
  }

  if (gr->penAxis>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penAxis);
    gr->penAxis=-1;
  }

  if (gr->penGraph>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penGraph);
    gr->penGraph=-1;
  }

  if (gr->penGraphBackground>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penGraphBackground);
    gr->penGraphBackground=-1;
  }

  if (gr->penGrid>=0) {
    AF_Graphics_PenRelease(gr->graphics, gr->penGrid);
    gr->penGrid=-1;
  }



  if (gr->fontTitle>=0) {
    AF_Graphics_FontRelease(gr->graphics, gr->fontTitle);
    gr->fontTitle=-1;
  }

  if (gr->fontSubTitle>=0) {
    AF_Graphics_FontRelease(gr->graphics, gr->fontSubTitle);
    gr->fontSubTitle=-1;
  }

  if (gr->fontAxisLabel>=0) {
    AF_Graphics_FontRelease(gr->graphics, gr->fontAxisLabel);
    gr->fontAxisLabel=-1;
  }

  if (gr->fontTickLabel>=0) {
    AF_Graphics_FontRelease(gr->graphics, gr->fontTickLabel);
    gr->fontTickLabel=-1;
  }
}



void AF_Graph_GetTickLabelMaxSizes(AF_GRAPH *gr, AF_GRAPH_TICK_LIST *tl, int *pMaxWidth, int *pMaxHeight) {
  if (tl) {
    AF_GRAPH_TICK *t;
    int maxWidth=0;
    int maxHeight=0;

    t=AF_Graph_Tick_List_First(tl);
    while(t) {
      const char *s;

      s=AF_Graph_Tick_GetLabel(t);
      if (s && *s) {
	int i;

	i=AF_Graphics_GetTextWidth(gr->graphics, gr->fontTickLabel, s);
        if (i<0) {
          DBG_ERROR(0, "Invalid text width (%d)", i);
        }
        else {
          if (i>maxWidth)
            maxWidth=i;
        }
        i=AF_Graphics_GetTextHeight(gr->graphics, gr->fontTickLabel, s);
        if (i<0) {
          DBG_ERROR(0, "Invalid text height (%d)", i);
        }
        else {
          if (i>maxHeight)
            maxHeight=i;
        }
      }
      else {
        DBG_ERROR(0, "No tick label");
      }
      t=AF_Graph_Tick_List_Next(t);
    }

    *pMaxWidth=maxWidth;
    *pMaxHeight=maxHeight;
  }
}





void AF_Graph_SampleSizes(AF_GRAPH *gr, AF_GRAPH_DATASET *ds) {
  AF_GRAPH_TICK_LIST *tl;
  const char *s;

  gr->xTickLabelMaxWidth=-1;
  gr->xTickLabelMaxHeight=-1;
  gr->xTickLabelPosY=-1;
  tl=AF_Graph_DataSet_GetXTickList(ds);
  if (tl)
    AF_Graph_GetTickLabelMaxSizes(gr, tl, &(gr->xTickLabelMaxWidth), &(gr->xTickLabelMaxHeight));

  gr->yTickLabelMaxWidth=-1;
  gr->yTickLabelMaxHeight=-1;
  gr->yTickLabelPosX=-1;
  tl=AF_Graph_DataSet_GetYTickList(ds);
  if (tl)
    AF_Graph_GetTickLabelMaxSizes(gr, tl, &(gr->yTickLabelMaxWidth), &(gr->yTickLabelMaxHeight));

  /* title */
  gr->titleWidth=-1;
  gr->titleHeight=-1;
  gr->titlePosX=-1;
  gr->titlePosY=-1;
  s=AF_Graph_DataSet_GetTitle(ds);
  if (s && *s) {
    gr->titleWidth=AF_Graphics_GetTextWidth(gr->graphics, gr->fontTitle, s);
    gr->titleHeight=AF_Graphics_GetTextHeight(gr->graphics, gr->fontTitle, s);
  }

  /* subTitle */
  gr->subTitleWidth=-1;
  gr->subTitleHeight=-1;
  gr->subTitlePosX=-1;
  gr->subTitlePosY=-1;
  s=AF_Graph_DataSet_GetSubTitle(ds);
  if (s && *s) {
    gr->subTitleWidth=AF_Graphics_GetTextWidth(gr->graphics, gr->fontSubTitle, s);
    gr->subTitleHeight=AF_Graphics_GetTextHeight(gr->graphics, gr->fontSubTitle, s);
  }

  /* xLabel */
  gr->xAxisLabelWidth=-1;
  gr->xAxisLabelHeight=-1;
  gr->xAxisLabelPosX=-1;
  gr->xAxisLabelPosY=-1;
  s=AF_Graph_DataSet_GetLabelX(ds);
  if (s && *s) {
    gr->xAxisLabelWidth=AF_Graphics_GetTextWidth(gr->graphics, gr->fontAxisLabel, s);
    gr->xAxisLabelHeight=AF_Graphics_GetTextHeight(gr->graphics, gr->fontAxisLabel, s);
  }

  /* yAxisLabel */
  gr->yAxisLabelWidth=-1;
  gr->yAxisLabelHeight=-1;
  gr->yAxisLabelPosX=-1;
  gr->yAxisLabelPosY=-1;
  s=AF_Graph_DataSet_GetLabelY(ds);
  if (s && *s) {
    gr->yAxisLabelWidth=AF_Graphics_GetTextWidth(gr->graphics, gr->fontAxisLabel, s);
    gr->yAxisLabelHeight=AF_Graphics_GetTextHeight(gr->graphics, gr->fontAxisLabel, s);
  }

  gr->graphAreaWidth=-1;
  gr->graphAreaHeight=-1;
  gr->graphAreaPosX=-1;
  gr->graphAreaPosY=-1;
}



void AF_Graph_Layout(AF_GRAPH *gr, AF_GRAPH_DATASET *ds) {
  int graphAreaMinX;
  int graphAreaMinY;
  int graphAreaMaxX;
  int graphAreaMaxY;

  graphAreaMinX=BORDER;
  graphAreaMaxX=gr->width-BORDER-1;
  graphAreaMinY=BORDER;
  graphAreaMaxY=gr->height-BORDER-1;


  /* title */
  if (gr->titleWidth>0 && gr->titleHeight>0) {
    gr->titlePosX=(gr->width-gr->titleWidth)/2;
    gr->titlePosY=graphAreaMinY;
    graphAreaMinY+=gr->titleHeight+SMALL_BORDER;
  }

  /* subtitle */
  if (gr->subTitleWidth>0 && gr->subTitleHeight>0) {
    gr->subTitlePosX=(gr->width-gr->subTitleWidth)/2;
    gr->subTitlePosY=graphAreaMinY;
    graphAreaMinY+=gr->subTitleHeight+SMALL_BORDER;
  }

  /* y-axis label */
  if (gr->yAxisLabelWidth>0 && gr->yAxisLabelHeight>0) {
    gr->yAxisLabelPosX=BORDER;
    gr->yAxisLabelPosY=graphAreaMinY;
    graphAreaMinY+=gr->yAxisLabelHeight+BORDER;
  }

  /* X-axis label */
  if (gr->xAxisLabelWidth>0 && gr->xAxisLabelHeight>0) {
    gr->xAxisLabelPosX=graphAreaMaxX-gr->xAxisLabelWidth;
    gr->xAxisLabelPosY=gr->height-gr->xAxisLabelHeight-BORDER;
    graphAreaMaxX=gr->xAxisLabelPosX-BORDER;
  }

  /* y-axis ticklabels */
  if (gr->yTickLabelMaxWidth>0 && gr->yTickLabelMaxHeight>0) {
    gr->yTickLabelPosX=BORDER;
    graphAreaMinX+=gr->yTickLabelMaxWidth+BORDER;
  }

  /* x-axis ticklabels */
  if (gr->xTickLabelMaxWidth>0 && gr->xTickLabelMaxHeight>0) {
    gr->xTickLabelPosY=gr->height-gr->xTickLabelMaxHeight-BORDER;
    graphAreaMaxY=gr->xTickLabelPosY-BORDER;
  }

  /* determine graphics area */
  gr->graphAreaWidth=(graphAreaMaxX-graphAreaMinX);
  gr->graphAreaHeight=(graphAreaMaxY-graphAreaMinY);
  gr->graphAreaPosX=graphAreaMinX;
  gr->graphAreaPosY=graphAreaMinY;

}



void AF_Graph_CalcGraphArea(AF_GRAPH *gr, AF_GRAPH_DATASET *ds) {
  double minVal;
  double maxVal;
  double dist;

  /* determine X-offset of graph origin and xFactor */
  minVal=AF_Graph_DataSet_GetMinValueX(ds);
  maxVal=AF_Graph_DataSet_GetMaxValueX(ds);
  gr->graphFactorX=gr->graphAreaWidth/(maxVal-minVal);
  if (minVal<=0.0 && maxVal>=0.0) {
    /* one negative, the other positive */
    dist=-minVal;
    gr->graphOriginX=gr->graphAreaPosX+dist*gr->graphFactorX;
  }
  else if (minVal<0.0 && maxVal<0.0) {
    /* both negative */
    dist=-minVal;
    gr->graphOriginX=gr->graphAreaPosX+dist*gr->graphFactorX;
  }
  else if (minVal>0.0 && maxVal>=0.0) {
    /* both positive */
    dist=minVal;
    gr->graphOriginX=gr->graphAreaPosX-dist*gr->graphFactorX;
  }

  /* determine x-pos of the y-axis */
  if (gr->graphOriginX<gr->graphAreaPosX) {
    /* y-axis on left border */
    gr->yAxisPosX=gr->graphAreaPosX;
  }
  else if (gr->graphOriginX>=gr->graphAreaWidth) {
    /* y-axis on right border */
    gr->yAxisPosX=gr->graphAreaPosX+gr->graphAreaWidth-1;
  }
  else {
    /* y-axis somewhere within the graph area */
    gr->yAxisPosX=gr->graphOriginX;
  }


  /* determine Y-offset of graph origin and yFactor */
  minVal=AF_Graph_DataSet_GetMinValueY(ds);
  maxVal=AF_Graph_DataSet_GetMaxValueY(ds);
  gr->graphFactorY=gr->graphAreaHeight/(maxVal-minVal);
  if (minVal<=0.0 && maxVal>=0.0) {
    /* one negative, the other positive */
    dist=maxVal;
    gr->graphOriginY=gr->graphAreaPosY+(dist*gr->graphFactorY);
  }
  else if (minVal<0.0 && maxVal<0.0) {
    /* both negative */
    dist=maxVal;
    gr->graphOriginY=gr->graphAreaPosY+dist*gr->graphFactorY;
  }
  else if (minVal>0.0 && maxVal>0.0) {
    /* both positive */
    dist=maxVal;
    gr->graphOriginY=gr->graphAreaPosY+(dist*gr->graphFactorY);
  }

  /* determine y-pos of the x-axis */
  if (gr->graphOriginY<gr->graphAreaPosY) {
    /* y-axis on upper border */
    gr->xAxisPosY=gr->graphAreaPosY;
  }
  else if (gr->graphOriginY>=gr->graphAreaHeight) {
    /* x-axis on lower border */
    gr->xAxisPosY=gr->graphAreaPosY+gr->graphAreaHeight-1;
  }
  else {
    /* y-axis somewhere within the graph area */
    gr->xAxisPosY=gr->graphOriginY;
  }

  DBG_ERROR(0, "Origin: %d / %d (Axis: x-axis y=%d / y-axis x=%d)",
            gr->graphOriginX, gr->graphOriginY,
            gr->xAxisPosY, gr->yAxisPosX);
}




void AF_Graph_PrepareGraph(AF_GRAPH *gr) {
  AF_GRAPH_TICK_LIST *tl;

  tl=AF_Graph_DataSet_GetXTickList(gr->dataSet);
  if (tl)
    AF_Graph_Tick_List_SortByValueAndLevel(tl, 1);

  tl=AF_Graph_DataSet_GetYTickList(gr->dataSet);
  if (tl)
    AF_Graph_Tick_List_SortByValueAndLevel(tl, 1);

  AF_Graph_SampleSizes(gr, gr->dataSet);
  AF_Graph_Layout(gr, gr->dataSet);
  AF_Graph_CalcGraphArea(gr, gr->dataSet);
}




AF_GRAPH *AF_Graph_new(AF_GRAPH_DATASET *ds, AF_GRAPHICS *graphics, int width, int height) {
  AF_GRAPH *gr;

  GWEN_NEW_OBJECT(AF_GRAPH, gr);
  GWEN_LIST_INIT(AF_GRAPH, gr)

  gr->dataSet=ds;
  gr->graphics=graphics;
  gr->width=width;
  gr->height=height;

  return gr;
}



void AF_Graph_free(AF_GRAPH *gr) {
  if (gr) {
    GWEN_LIST_FINI(AF_GRAPH, gr)
    AF_Graphics_free(gr->graphics);
    AF_Graph_DataSet_free(gr->dataSet);
    GWEN_FREE_OBJECT(gr);
  }
}



int AF_Graph_Init_fromDb(AF_GRAPH *gr, GWEN_DB_NODE *dbConfig) {
  int rv;
  int i;
  uint32_t flags=0;

  rv=AF_Graph_CreatePensAndFonts_FromDb(gr, dbConfig);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  for (i=0; i<100; i++) {
    const char *s;

    s=GWEN_DB_GetCharValue(dbConfig, "flags", i, NULL);
    if (!(s && *s))
      break;
    if (strcasecmp(s, "grid")==0)
      flags|=AF_GRAPH_FLAGS_WITH_GRID;
  }
  gr->flags=flags;


  return 0;
}



int AF_Graph_Init(AF_GRAPH *gr, const AF_GRAPH_SETTINGS *gs) {
  int rv;

  rv=AF_Graph_CreatePensAndFonts_FromSettings(gr, gs);
  if (rv<0) {
    DBG_INFO(0, "here (%d)", rv);
    return rv;
  }

  gr->flags=AF_Graph_Settings_GetFlags(gs);

  return 0;
}



int AF_Graph_Fini(AF_GRAPH *gr) {
  AF_Graph_ReleasePensAndFonts(gr);

  return 0;
}



void AF_Graph_DrawXTicks(AF_GRAPH *gr) {
  AF_GRAPH_TICK_LIST *tl;

  tl=AF_Graph_DataSet_GetXTickList(gr->dataSet);
  if (tl) {
    AF_GRAPH_TICK *t;
    uint8_t *pMask;
    int level;

    /* reset */
    pMask=(uint8_t*) malloc(gr->graphAreaWidth);
    assert(pMask);
    memset(pMask, 0, gr->graphAreaWidth);

    /* handle each level */
    for (level=1; level<MAX_TICK_LEVELS+1; level++) {
      uint8_t tickDrawFlags=0;
      int maxWidth=0;
      int every=1;

      /* determine width of labels for this level */
      t=AF_Graph_Tick_List_First(tl);
      while(t) {
        if (AF_Graph_Tick_GetLevel(t)==level) {
          const char *s;

          s=AF_Graph_Tick_GetLabel(t);
          if (s && *s) {
            int i;

            //DBG_ERROR(0, "X-Level %d: [%s] = %f", level, s, AF_Graph_Tick_GetValue(t));
            i=AF_Graphics_GetTextWidth(gr->graphics, gr->fontTickLabel, s);
            if (i<0) {
              DBG_ERROR(0, "Invalid text width (%d)", i);
            }
            else {
              if (i>maxWidth)
                maxWidth=i;
            }
          }
          else {
            DBG_ERROR(0, "X-Level %d: No tick label", level);
          }
        }
        t=AF_Graph_Tick_List_Next(t);
      }


      /* check medium width for the current level */
      {
        int first=1;
        int hasSecond=0;
        double lastValue;

        t=AF_Graph_Tick_List_First(tl);
        while(t) {
          if (AF_Graph_Tick_GetLevel(t)==level) {
            double v;

            v=AF_Graph_Tick_GetValue(t);
            if (first) {
              lastValue=v;
              first=0;
            }
            else {
              int dist;

              dist=(int)((v-lastValue)*gr->graphFactorX);
              if (dist>MIN_DIST_FOR_TICKS)
                tickDrawFlags|=TICKDRAWFLAGS_MARKS;
              if (dist>MIN_DIST_FOR_GRIDS)
                tickDrawFlags|=TICKDRAWFLAGS_GRIDS;

              if (dist>maxWidth+SMALL_BORDER) {
                tickDrawFlags|=TICKDRAWFLAGS_LABELS;
              }
              else if ((dist*2)>maxWidth+SMALL_BORDER) {
                tickDrawFlags|=TICKDRAWFLAGS_LABELS;
                every=2;
              }
              else if ((dist*4)>maxWidth+SMALL_BORDER) {
                tickDrawFlags|=TICKDRAWFLAGS_LABELS;
                every=4;
              }

              DBG_ERROR(0, "X-Level %d: dist=%d (flags=%02x)", level, dist, tickDrawFlags);
              hasSecond=1;
              break;
            }
          }

          t=AF_Graph_Tick_List_Next(t);
        }
        if (!first && !hasSecond) {
          int dist;

          dist=gr->graphAreaWidth;
          if (dist>MIN_DIST_FOR_TICKS)
            tickDrawFlags|=TICKDRAWFLAGS_MARKS;
          if (dist>MIN_DIST_FOR_GRIDS)
            tickDrawFlags|=TICKDRAWFLAGS_GRIDS;
          if (dist>maxWidth+SMALL_BORDER)
            tickDrawFlags|=TICKDRAWFLAGS_LABELS;
          DBG_ERROR(0, "X-Level %d: dist=%d (flags=%02x)", level, dist, tickDrawFlags);
        }
      }

      /* now draw according to flags */
      if (tickDrawFlags) {
        double minVal;
        double maxVal;
        int tickIntervalCnt=every;
    
        minVal=AF_Graph_DataSet_GetMinValueX(gr->dataSet);
        maxVal=AF_Graph_DataSet_GetMaxValueX(gr->dataSet);
        t=AF_Graph_Tick_List_First(tl);
        while(t) {
          if (AF_Graph_Tick_GetLevel(t)==level) {
            double v;

            tickIntervalCnt--;
            v=AF_Graph_Tick_GetValue(t);
            if (v>=minVal && v<=maxVal) {
              int x, y;
              int w;

              x=gr->graphOriginX + ( v * gr->graphFactorX );
              y=gr->xAxisPosY;

              /* draw grid line */
              if (tickDrawFlags & TICKDRAWFLAGS_GRIDS) {
                if (gr->flags & AF_GRAPH_FLAGS_WITH_GRID)
                  AF_Graphics_DrawLine(gr->graphics, gr->penGrid, x, gr->graphAreaPosY, x, gr->graphAreaPosY+gr->graphAreaHeight);
              }

              if (tickDrawFlags & TICKDRAWFLAGS_MARKS) {
                switch(level) {
                  case 1:   w=9; break;
                  case 2:   w=5; break;
                  default:
                  case 3:   w=1; break;
                }

                /* draw tick mark */
                AF_Graphics_DrawLine(gr->graphics, gr->penAxis, x, y-w, x, y+w);
              }

              if (tickDrawFlags & TICKDRAWFLAGS_LABELS) {
                const char *s;
      
                /* draw label */
                if (tickIntervalCnt<1) {

                  s=AF_Graph_Tick_GetLabel(t);
                  if (s && *s) {
                    int i;
        
                    i=AF_Graphics_GetTextWidth(gr->graphics, gr->penAxisLabel, s);
                    if (i<0) {
                    }
                    else {
                      if ((x+(i/2) < (gr->graphAreaPosX+gr->graphAreaWidth)) &&
                          (x-(i/2) > gr->graphAreaPosX)) {
                        int j;
                        int doesFit=1;
                        int m1, m2;
  
                        m1=SMALL_BORDER+x-(i/2)-gr->graphAreaPosX;
                        m2=SMALL_BORDER+x+(i/2)-gr->graphAreaPosX;
  
                        for (j=m1; j<m2; j++) {
                          if (pMask[j]!=0) {
                            doesFit=0;
                            break;
                          }
                        }
  
                        if (doesFit) {
                          m1=x-(i/2)-gr->graphAreaPosX;
                          m2=x+(i/2)-gr->graphAreaPosX;
  
                          for (j=m1; j<m2; j++)
                            pMask[j]=1;
                          AF_Graphics_DrawText(gr->graphics, gr->penAxisLabel, gr->fontTickLabel, AF_Graphics_Direction_Horizontal,
                                               x-(i/2), gr->xTickLabelPosY, s);
                        }
                      }
                    }
                  }
                  tickIntervalCnt=every;
                }
              } /* if labels to be drawn */
            } /* if value within limits */
          }
          t=AF_Graph_Tick_List_Next(t);
        } /* while t */
      } /* if tickDrawFlags */


    } /* for level */

    free(pMask);
  } /* if tl */
}



void AF_Graph_DrawYTicks(AF_GRAPH *gr) {
  AF_GRAPH_TICK_LIST *tl;

  tl=AF_Graph_DataSet_GetYTickList(gr->dataSet);
  if (tl) {
    AF_GRAPH_TICK *t;
    uint8_t *pMask;
    int level;

    /* reset */
    pMask=(uint8_t*) malloc(gr->graphAreaHeight);
    assert(pMask);
    memset(pMask, 0, gr->graphAreaHeight);

    /* handle each level */
    for (level=1; level<MAX_TICK_LEVELS+1; level++) {
      uint8_t tickDrawFlags=0;
      int maxHeight=0;

      /* determine width of labels for this level */
      t=AF_Graph_Tick_List_First(tl);
      while(t) {
        if (AF_Graph_Tick_GetLevel(t)==level) {
          const char *s;

          s=AF_Graph_Tick_GetLabel(t);
          if (s && *s) {
            int i;

            //DBG_ERROR(0, "Y-Level %d: [%s] = %f", level, s, AF_Graph_Tick_GetValue(t));
            i=AF_Graphics_GetTextHeight(gr->graphics, gr->fontTickLabel, s);
            if (i<0) {
              DBG_ERROR(0, "Invalid text width (%d)", i);
            }
            else {
              if (i>maxHeight)
                maxHeight=i;
            }
          }
          else {
            DBG_ERROR(0, "No tick label");
          }
        }
        t=AF_Graph_Tick_List_Next(t);
      }


      /* get 1st and 2nd entry for the current level and check width */
      {
        int first=1;
        int hasSecond=0;
        double lastValue;

        t=AF_Graph_Tick_List_First(tl);
        while(t) {
          if (AF_Graph_Tick_GetLevel(t)==level) {
            double v;

            v=AF_Graph_Tick_GetValue(t);
            if (first) {
              lastValue=v;
              first=0;
            }
            else {
              int dist;

              dist=(int)((v-lastValue)*gr->graphFactorY);
              if (dist>MIN_DIST_FOR_TICKS)
                tickDrawFlags|=TICKDRAWFLAGS_MARKS;
              if (dist>MIN_DIST_FOR_GRIDS)
                tickDrawFlags|=TICKDRAWFLAGS_GRIDS;
              if (dist>maxHeight+5)
                tickDrawFlags|=TICKDRAWFLAGS_LABELS;
              DBG_ERROR(0, "Y-Level %d: dist=%d (flags=%02x)", level, dist, tickDrawFlags);
              hasSecond=1;
              break;
            }
          }

          t=AF_Graph_Tick_List_Next(t);
        }

        if (!first && !hasSecond) {
          int dist;

          dist=gr->graphAreaHeight;
          if (dist>MIN_DIST_FOR_TICKS)
            tickDrawFlags|=TICKDRAWFLAGS_MARKS;
          if (dist>MIN_DIST_FOR_GRIDS)
            tickDrawFlags|=TICKDRAWFLAGS_GRIDS;
          if (dist>maxHeight+2)
            tickDrawFlags|=TICKDRAWFLAGS_LABELS;
        }
      }

      /* now draw according to flags */
      if (tickDrawFlags) {
        double minVal;
        double maxVal;
    
        minVal=AF_Graph_DataSet_GetMinValueY(gr->dataSet);
        maxVal=AF_Graph_DataSet_GetMaxValueY(gr->dataSet);
        t=AF_Graph_Tick_List_First(tl);
        while(t) {
          double v;

          v=AF_Graph_Tick_GetValue(t);
          if (AF_Graph_Tick_GetLevel(t)==level) {
            if (v>=minVal && v<=maxVal) {
              int x, y;
              int w;

              x=gr->yAxisPosX;
              y=gr->graphOriginY - ( v * gr->graphFactorY );

              /* draw grid line */
              if (tickDrawFlags & TICKDRAWFLAGS_GRIDS) {
                if (gr->flags & AF_GRAPH_FLAGS_WITH_GRID)
                  AF_Graphics_DrawLine(gr->graphics, gr->penGrid, gr->graphAreaPosX, y, gr->graphAreaPosX+gr->graphAreaWidth, y);
              }

              if (tickDrawFlags & TICKDRAWFLAGS_MARKS) {
                switch(level) {
                  case 1:   w=9; break;
                  case 2:   w=5; break;
                  default:
                  case 3:   w=1; break;
                }

                /* draw tick mark */
                AF_Graphics_DrawLine(gr->graphics, gr->penAxis, x-w, y, x+w, y);
              }

              if (tickDrawFlags & TICKDRAWFLAGS_LABELS) {
                const char *s;
      
                /* draw label */
                s=AF_Graph_Tick_GetLabel(t);
                if (s && *s) {
                  int i;
      
                  i=AF_Graphics_GetTextHeight(gr->graphics, gr->penAxisLabel, s);
                  if (i<0) {
                  }
                  else {
                    if ((y+(i/2) < (gr->graphAreaPosY+gr->graphAreaHeight)) &&
                        (y-(i/2)>= gr->graphAreaPosY)) {
                      int j;
                      int doesFit=1;
                      int m1, m2;

                      m1=SMALL_BORDER+y-(i/2)-gr->graphAreaPosY;
                      m2=SMALL_BORDER+y+(i/2)-gr->graphAreaPosY;
                      for (j=m1; j<m2; j++) {
                        if (pMask[j]!=0) {
                          doesFit=0;
                          break;
                        }
                      }

                      if (doesFit) {
                        m1=y-(i/2)-gr->graphAreaPosY;
                        m2=y+(i/2)-gr->graphAreaPosY;

                        for (j=m1; j<m2; j++)
                          pMask[j]=1;
                        AF_Graphics_DrawText(gr->graphics, gr->penAxisLabel, gr->fontTickLabel, AF_Graphics_Direction_Horizontal,
                                             gr->yTickLabelPosX, y-(i/2), s);
                      }
                    }
                  }
                }
              } /* if labels to be drawn */
            } /* if value within limits */
          }
          t=AF_Graph_Tick_List_Next(t);
        } /* while t */
      } /* if tickDrawFlags */


    } /* for level */

    free(pMask);
  } /* if tl */
}



void AF_Graph_DrawFramework(AF_GRAPH *gr) {
  const char *s;

  /* draw background */
  AF_Graphics_DrawFilledRect(gr->graphics, gr->penBackground,
                             0, 0, gr->width, gr->height);

  /* draw graph area */
  AF_Graphics_DrawFilledRect(gr->graphics, gr->penGraphBackground,
			     gr->graphAreaPosX, gr->graphAreaPosY,
			     gr->graphAreaWidth, gr->graphAreaHeight);

  AF_Graph_DrawXTicks(gr);

  /* draw x-axis */
  AF_Graphics_DrawLine(gr->graphics, gr->penAxis,
		       gr->graphAreaPosX, gr->xAxisPosY,
                       gr->graphAreaPosX+gr->graphAreaWidth, gr->xAxisPosY);

  /* x-axis label */
  s=AF_Graph_DataSet_GetLabelX(gr->dataSet);
  if (s && *s)
    AF_Graphics_DrawText(gr->graphics, gr->penAxisLabel, gr->fontAxisLabel, AF_Graphics_Direction_Horizontal,
                         gr->xAxisLabelPosX, gr->xAxisLabelPosY, s);




  /* draw y ticks */
  AF_Graph_DrawYTicks(gr);

  /* draw y-axis */
  AF_Graphics_DrawLine(gr->graphics, gr->penAxis,
                       gr->yAxisPosX, gr->graphAreaPosY,
                       gr->yAxisPosX, gr->graphAreaPosY+gr->graphAreaHeight);

  /* y-axis label */
  s=AF_Graph_DataSet_GetLabelY(gr->dataSet);
  if (s && *s)
    AF_Graphics_DrawText(gr->graphics, gr->penAxisLabel, gr->fontAxisLabel, AF_Graphics_Direction_Horizontal,
                         gr->yAxisLabelPosX, gr->yAxisLabelPosY, s);






  /* title */
  s=AF_Graph_DataSet_GetTitle(gr->dataSet);
  if (s && *s)
    AF_Graphics_DrawText(gr->graphics, gr->penTitle, gr->fontTitle, AF_Graphics_Direction_Horizontal,
                         gr->titlePosX, gr->titlePosY, s);

  /* subtitle */
  s=AF_Graph_DataSet_GetSubTitle(gr->dataSet);
  if (s && *s)
    AF_Graphics_DrawText(gr->graphics, gr->penSubTitle, gr->fontSubTitle, AF_Graphics_Direction_Horizontal,
                         gr->subTitlePosX, gr->subTitlePosY, s);
}




void AF_Graph_DrawDataAsLine(AF_GRAPH *gr, int level, int penId) {
  AF_GRAPH_DATAPAIR_LIST *dpl;

  dpl=AF_Graph_DataSet_GetDataPairList(gr->dataSet);
  if (dpl) {
    AF_GRAPH_DATAPAIR *dp;
    double minValX;
    double maxValX;
    double minValY;
    double maxValY;
    int lastX;
    int lastY;
    int first=1;

    minValX=AF_Graph_DataSet_GetMinValueX(gr->dataSet);
    maxValX=AF_Graph_DataSet_GetMaxValueX(gr->dataSet);
    minValY=AF_Graph_DataSet_GetMinValueY(gr->dataSet);
    maxValY=AF_Graph_DataSet_GetMaxValueY(gr->dataSet);
    dp=AF_Graph_DataPair_List_First(dpl);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double vX;
        double vY;
        int x, y;

        vX=AF_Graph_DataPair_GetValueX(dp);
        vY=AF_Graph_DataPair_GetValueY(dp);

        if (vX<minValX)
          vX=minValX;
        else if (vX>maxValX)
          vX=maxValX;

        if (vY<minValY)
          vY=minValY;
        else if (vY>maxValY)
          vY=maxValY;

        x=gr->graphOriginX + ( vX * gr->graphFactorX );
        y=gr->graphOriginY - ( vY * gr->graphFactorY );

        /* draw a small star */
        AF_Graphics_DrawLine(gr->graphics, penId, x-3, y-3, x+3, y+3);
        AF_Graphics_DrawLine(gr->graphics, penId, x-3, y+3, x+3, y-3);
        AF_Graphics_DrawLine(gr->graphics, penId, x, y-3, x, y+3);
        AF_Graphics_DrawLine(gr->graphics, penId, x-3, y, x+3, y);

        if (!first)
          AF_Graphics_DrawLine(gr->graphics, penId, lastX, lastY, x, y);
        else
          first=0;

        lastX=x;
        lastY=y;
      }

      dp=AF_Graph_DataPair_List_Next(dp);
    }

  }
}



void AF_Graph_DrawDataAsBars(AF_GRAPH *gr, int level, int penId) {
  AF_GRAPH_DATAPAIR_LIST *dpl;
  double dataWidth;
  int barWidth;
  int rv;

  rv=AF_Graph_DataSet_GetMinValueDistX(gr->dataSet, level, &dataWidth);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return;
  }
  else if (rv==1) {
    /* use full width */
    barWidth=(gr->graphAreaWidth)-10;
  }
  else {
    barWidth=((int)(dataWidth*gr->graphFactorX))-10;
  }

  if (barWidth<1)
    barWidth=1;

  dpl=AF_Graph_DataSet_GetDataPairList(gr->dataSet);
  if (dpl) {
    AF_GRAPH_DATAPAIR *dp;
    double minValX;
    double maxValX;
    double minValY;
    double maxValY;

    minValX=AF_Graph_DataSet_GetMinValueX(gr->dataSet);
    maxValX=AF_Graph_DataSet_GetMaxValueX(gr->dataSet);
    minValY=AF_Graph_DataSet_GetMinValueY(gr->dataSet);
    maxValY=AF_Graph_DataSet_GetMaxValueY(gr->dataSet);
    dp=AF_Graph_DataPair_List_First(dpl);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double vX;
        double vY;
	int x, y;
	int leftX, bw;

        vX=AF_Graph_DataPair_GetValueX(dp);
        vY=AF_Graph_DataPair_GetValueY(dp);

        if (vX<minValX)
          vX=minValX;
        else if (vX>maxValX)
          vX=maxValX;

        if (vY<minValY)
          vY=minValY;
        else if (vY>maxValY)
          vY=maxValY;

        x=gr->graphOriginX + ( vX * gr->graphFactorX );
        y=gr->graphOriginY - ( vY * gr->graphFactorY );

	/* draw the bar */
	if ((x-(barWidth/2))<gr->graphAreaPosX) {
	  leftX=x;
	  bw=barWidth/2;
	}
	else {
	  leftX=x-(barWidth/2);
	  bw=barWidth;
	}
	if ((leftX+bw)>=(gr->graphAreaPosX+gr->graphAreaWidth)) {
	  bw=barWidth/2;
	}

	AF_Graphics_DrawFilledRect(gr->graphics, penId,
				   leftX, gr->xAxisPosY,
				   bw, y-gr->xAxisPosY);
      }

      dp=AF_Graph_DataPair_List_Next(dp);
    }

  }
}



void AF_Graph_DrawDataAsPoints(AF_GRAPH *gr, int level, int penId) {
  AF_GRAPH_DATAPAIR_LIST *dpl;
  double dataWidth;
  int barWidth;
  int rv;

  rv=AF_Graph_DataSet_GetMinValueDistX(gr->dataSet, level, &dataWidth);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return;
  }
  else if (rv==1) {
    /* use full width */
    barWidth=(gr->graphAreaWidth)-10;
  }
  else {
    barWidth=((int)(dataWidth*gr->graphFactorX))-10;
  }

  if (barWidth<1)
    barWidth=1;

  dpl=AF_Graph_DataSet_GetDataPairList(gr->dataSet);
  if (dpl) {
    AF_GRAPH_DATAPAIR *dp;
    double minValX;
    double maxValX;
    double minValY;
    double maxValY;

    minValX=AF_Graph_DataSet_GetMinValueX(gr->dataSet);
    maxValX=AF_Graph_DataSet_GetMaxValueX(gr->dataSet);
    minValY=AF_Graph_DataSet_GetMinValueY(gr->dataSet);
    maxValY=AF_Graph_DataSet_GetMaxValueY(gr->dataSet);
    dp=AF_Graph_DataPair_List_First(dpl);
    while(dp) {
      if (AF_Graph_DataPair_GetLevel(dp)==level) {
        double vX;
        double vY;
	int x, y;

        vX=AF_Graph_DataPair_GetValueX(dp);
        vY=AF_Graph_DataPair_GetValueY(dp);

        if (vX<minValX)
          vX=minValX;
        else if (vX>maxValX)
          vX=maxValX;

        if (vY<minValY)
          vY=minValY;
        else if (vY>maxValY)
          vY=maxValY;

        x=gr->graphOriginX + ( vX * gr->graphFactorX );
        y=gr->graphOriginY - ( vY * gr->graphFactorY );

        /* draw a small star */
        AF_Graphics_DrawLine(gr->graphics, penId, x-3, y-3, x+3, y+3);
        AF_Graphics_DrawLine(gr->graphics, penId, x-3, y+3, x+3, y-3);
        AF_Graphics_DrawLine(gr->graphics, penId, x, y-3, x, y+3);
        AF_Graphics_DrawLine(gr->graphics, penId, x-3, y, x+3, y);
      }

      dp=AF_Graph_DataPair_List_Next(dp);
    }

  }
}



int AF_Graph_Draw(AF_GRAPH *gr) {
  AF_GRAPH_SUBGRAPH_LIST *sgList;

  AF_Graph_PrepareGraph(gr);
  AF_Graph_DrawFramework(gr);

  /* draw all levels */
  sgList=AF_Graph_DataSet_GetSubGraphList(gr->dataSet);
  if (sgList) {
    AF_GRAPH_SUBGRAPH *sg;

    sg=AF_Graph_SubGraph_List_First(sgList);
    while(sg) {
      /* TODO: select graph type  */
      switch(AF_Graph_SubGraph_GetGraphType(sg)) {
      case AF_GRAPH_TYPE_POINTS:
        AF_Graph_DrawDataAsPoints(gr, AF_Graph_SubGraph_GetLevel(sg), AF_Graph_SubGraph_GetPenId(sg));
        break;
      case AF_GRAPH_TYPE_LINE:
	AF_Graph_DrawDataAsLine(gr, AF_Graph_SubGraph_GetLevel(sg), AF_Graph_SubGraph_GetPenId(sg));
	break;
      case AF_GRAPH_TYPE_BARS:
	AF_Graph_DrawDataAsBars(gr, AF_Graph_SubGraph_GetLevel(sg), AF_Graph_SubGraph_GetPenId(sg));
	break;
      case AF_GRAPH_TYPE_NONE:
      case AF_GRAPH_TYPE_UNKNOWN:
	break;
      }
      sg=AF_Graph_SubGraph_List_Next(sg);
    }
  }

  return 0;
}








