/****************************************************************************
 * This file is part of the project AqFinance.
 * AqFinance (c) by 2010 Martin Preuss, all rights reserved.
 *
 * The license for this file can be found in the file COPYING which you
 * should have received along with this file.
 ****************************************************************************/


#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "af_print_p.h"
#include "af_print_doc.h"
#include "af_print_page.h"
#include "af_print_box.h"

#include "af_htmlctx_cairo.h"

#include <aqfinance/engine/book/ae_utils.h>

#include <gwenhywfar/htmlctx_be.h>

#include <gwenhywfar/misc.h>
#include <gwenhywfar/debug.h>
#include <gwenhywfar/directory.h>




AF_PRINT *AF_Print_newForPdf(const char *filename, AF_PRINT_DOC *tmpl) {
  AF_PRINT *pr;
  GWEN_XML_CONTEXT *ctx;
  AF_PRINT_PAGE *pg;

  pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(tmpl), AF_PrintPage_Id_FirstPage);
  if (pg==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "No first page in template");
    return NULL;
  }

  ctx=AF_HtmlCtxCairo_Pdf_new(0, filename,
			      AF_PrintPage_GetWidth(pg),
			      AF_PrintPage_GetHeight(pg));
  if (ctx==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Could not create Cairo context");
    return NULL;
  }

  pr=AF_Print_new(ctx, tmpl);
  GWEN_XmlCtx_free(ctx);
  return pr;
}



AF_PRINT *AF_Print_newForPs(const char *filename, AF_PRINT_DOC *tmpl) {
  AF_PRINT *pr;
  GWEN_XML_CONTEXT *ctx;
  AF_PRINT_PAGE *pg;

  pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(tmpl), AF_PrintPage_Id_FirstPage);
  if (pg==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "No first page in template");
    return NULL;
  }

  ctx=AF_HtmlCtxCairo_Ps_new(0, filename,
			     AF_PrintPage_GetWidth(pg),
			     AF_PrintPage_GetHeight(pg));
  if (ctx==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Could not create Cairo context");
    return NULL;
  }

  pr=AF_Print_new(ctx, tmpl);
  GWEN_XmlCtx_free(ctx);
  return pr;
}



AF_PRINT *AF_Print_new(GWEN_XML_CONTEXT *htmlContext, AF_PRINT_DOC *docTmpl) {
  AF_PRINT *pr;
  AF_PRINT_PAGE *pg;
  const char *s;

  GWEN_NEW_OBJECT(AF_PRINT, pr);
  pr->templateDoc=docTmpl;

  pr->htmlContext=htmlContext;
  GWEN_XmlCtx_Attach(pr->htmlContext);

  pr->currentDoc=AF_PrintDoc_dup(docTmpl);
  s=AF_PrintDoc_GetFileName(pr->currentDoc);
  if (s && *s) {
    GWEN_BUFFER *tbuf;
    char *p;

    tbuf=GWEN_Buffer_new(0, 256, 0, 1);
    GWEN_Buffer_AppendString(tbuf, s);
    p=strrchr(GWEN_Buffer_GetStart(tbuf), GWEN_DIR_SEPARATOR);
    if (p) {
      uint32_t pos;

      GWEN_Buffer_Crop(tbuf, 0, (p-GWEN_Buffer_GetStart(tbuf)));
      GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S);
      pos=GWEN_Buffer_GetPos(tbuf);
      HtmlCtx_AddMediaPath(pr->htmlContext, GWEN_Buffer_GetStart(tbuf));
      GWEN_Buffer_AppendString(tbuf, "media");
      DBG_INFO(AE_LOGDOMAIN, "Adding media path [%s]", GWEN_Buffer_GetStart(tbuf));
      HtmlCtx_AddMediaPath(pr->htmlContext, GWEN_Buffer_GetStart(tbuf));

      GWEN_Buffer_Crop(tbuf, 0, pos);
      GWEN_Buffer_AppendString(tbuf, "pics");
      DBG_INFO(AE_LOGDOMAIN, "Adding media path [%s]", GWEN_Buffer_GetStart(tbuf));
      HtmlCtx_AddMediaPath(pr->htmlContext, GWEN_Buffer_GetStart(tbuf));
    }
    GWEN_Buffer_free(tbuf);
  }

  /* currentDoc still contains all pages from template, clear list and start with
   * first page by using the appropriate page from the template */
  AF_PrintPage_List_Clear(AF_PrintDoc_GetPrintPages(pr->currentDoc));
  pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(docTmpl), AF_PrintPage_Id_FirstPage);
  if (pg==NULL) {
    DBG_WARN(AE_LOGDOMAIN, "No first page, trying following pages");
    pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(docTmpl), AF_PrintPage_Id_FollowingPages);
  }

  if (pg) {
    AF_PRINT_PAGE *npg;

    npg=AF_PrintPage_dup(pg);
    AF_PrintPage_SetId(npg, 0);
    AF_PrintPage_List_Add(npg, AF_PrintDoc_GetPrintPages(pr->currentDoc));
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "Template contains neither FirstPage nor FollowingPages");
    AF_Print_free(pr);
    return NULL;
  }

  return pr;
}



void AF_Print_free(AF_PRINT *pr) {
  if (pr) {
    AF_PrintDoc_free(pr->currentDoc);
    GWEN_XmlCtx_free(pr->htmlContext);
    GWEN_FREE_OBJECT(pr);
  }
}



AF_PRINT_DOC *AF_Print_GetCurrentDoc(const AF_PRINT *pr) {
  assert(pr);
  return pr->currentDoc;
}



void AF_Print_AddMediaPath(AF_PRINT *pr, const char *s) {
  assert(pr);
  HtmlCtx_AddMediaPath(pr->htmlContext, s);
}



int AF_Print_CheckText(AF_PRINT *pr, AF_PRINT_BOX_ID boxId, const char *text) {
  AF_PRINT_PAGE *pg;
  AF_PRINT_BOX *pb;
  int rv;

  pg=AF_PrintPage_List_Last(AF_PrintDoc_GetPrintPages(pr->currentDoc));
  assert(pg);

  pb=AF_PrintBox_List_GetById(AF_PrintPage_GetPrintBoxes(pg), boxId);
  if (pb==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Box %d not found (page %d)", boxId,
	      AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(pr->currentDoc)));
    return GWEN_ERROR_INVALID;
  }

  rv=0;
  AF_HtmlCtxCairo_SetFontSize(pr->htmlContext, AF_PrintBox_GetFontSize(pb));
  HtmlCtx_SetText(pr->htmlContext, text);
  HtmlCtx_Layout(pr->htmlContext, AF_PrintBox_GetWidth(pb)-2, AF_PrintBox_GetHeight(pb)-2);
  if (HtmlCtx_GetWidth(pr->htmlContext)>AF_PrintBox_GetWidth(pb)) {
    DBG_ERROR(0, "Width: %d > %d", HtmlCtx_GetWidth(pr->htmlContext), AF_PrintBox_GetWidth(pb));
    rv|=1;
  }
  if (HtmlCtx_GetHeight(pr->htmlContext)>AF_PrintBox_GetHeight(pb)) {
    DBG_ERROR(0, "Height: %d > %d", HtmlCtx_GetHeight(pr->htmlContext), AF_PrintBox_GetHeight(pb));
    rv|=2;
  }

  return rv;
}



int AF_Print_SetText(AF_PRINT *pr, AF_PRINT_BOX_ID boxId, const char *text) {
  AF_PRINT_PAGE *pg;
  AF_PRINT_BOX *pb;

  pg=AF_PrintPage_List_Last(AF_PrintDoc_GetPrintPages(pr->currentDoc));
  assert(pg);

  pb=AF_PrintBox_List_GetById(AF_PrintPage_GetPrintBoxes(pg), boxId);
  if (pb==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "Box %d not found", boxId);
    return GWEN_ERROR_INVALID;
  }

  AF_PrintBox_SetText(pb, text);
  return 0;
}



int AF_Print_NextPage(AF_PRINT *pr) {
  AF_PRINT_PAGE *pg;

  pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(pr->templateDoc), AF_PrintPage_Id_FollowingPages);
  if (pg==NULL) {
    DBG_WARN(AE_LOGDOMAIN, "No following page, looking for first page");
    pg=AF_PrintPage_List_GetById(AF_PrintDoc_GetPrintPages(pr->templateDoc), AF_PrintPage_Id_FirstPage);
  }
  if (pg) {
    AF_PRINT_PAGE *npg;
    AF_PRINT_PAGE_LIST *pl;

    pl=AF_PrintDoc_GetPrintPages(pr->currentDoc);

    npg=AF_PrintPage_dup(pg);
    AF_PrintPage_SetId(npg, AF_PrintPage_Id_FollowingPages);
    AF_PrintPage_List_Add(npg, pl);
  }

  return 0;
}



int AF_Print_PrintPage(AF_PRINT *pr, const AF_PRINT_PAGE *pg) {
  AF_PRINT_BOX *pb;

  pb=AF_PrintBox_List_First(AF_PrintPage_GetPrintBoxes(pg));
  while(pb) {
    const char *s;

    s=AF_PrintBox_GetText(pb);
    if (s && *s) {
      AF_HtmlCtxCairo_SetFontSize(pr->htmlContext, AF_PrintBox_GetFontSize(pb));
      HtmlCtx_SetText(pr->htmlContext, s);
      HtmlCtx_Layout(pr->htmlContext, AF_PrintBox_GetWidth(pb), AF_PrintBox_GetHeight(pb));
      AF_HtmlCtxCairo_Paint(pr->htmlContext, AF_PrintBox_GetX(pb), AF_PrintBox_GetY(pb));
    }
    pb=AF_PrintBox_List_Next(pb);
  }

  AF_HtmlCtxCairo_EndPage(pr->htmlContext);

  return 0;
}



int AF_Print_Finish(AF_PRINT *pr) {
  AF_PRINT_PAGE *pg;
  int rv;

  pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(pr->currentDoc));
  while(pg) {
    rv=AF_Print_PrintPage(pr, pg);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      return rv;
    }
    pg=AF_PrintPage_List_Next(pg);
  }
  AF_HtmlCtxCairo_Finish(pr->htmlContext);

  return 0;
}




AF_PRINT_DOC *AF_Print_GenerateTemplate_A4(double dpi) {
  AF_PRINT_DOC *doc;
  AF_PRINT_PAGE *pg;
  AF_PRINT_BOX *pb;
  double w;
  double h;
  double rLeft;
  double rRight;
  double rTop;
  double rBottom;

  doc=AF_PrintDoc_new();

  /* first page */
  pg=AF_PrintPage_new();
  w=8.3*dpi;
  h=11.7*dpi;
  rLeft=0.787*dpi;
  rRight=0.787*dpi;
  rTop=0.787*dpi;
  rBottom=0.787*dpi;

  AF_PrintPage_SetId(pg, AF_PrintPage_Id_FirstPage);
  AF_PrintPage_SetWidth(pg, w);
  AF_PrintPage_SetHeight(pg, h);

  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
  AF_PrintBox_SetX(pb, rLeft);
  AF_PrintBox_SetY(pb, rTop);
  AF_PrintBox_SetWidth(pb, w-rLeft-rRight);
  AF_PrintBox_SetHeight(pb, h-rTop-rBottom);
  AF_PrintBox_SetFontSize(pb, dpi/6.250); /* 48 px at 300 dpi */
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

  AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(doc));

  return doc;
}



AF_PRINT_DOC *AF_Print_GenerateTemplate_A4Letter(double dpi) {
  AF_PRINT_DOC *doc;
  AF_PRINT_PAGE *pg;
  AF_PRINT_BOX *pb;
  double w;
  double h;
  double rLeft;
  double rRight;
  double rTop;
  double rBottom;
  double y;

  doc=AF_PrintDoc_new();
  AF_PrintDoc_SetName(doc, I18N("Standard Letter"));
  AF_PrintDoc_SetCategory(doc, I18N("Letters"));

  /* first page */
  pg=AF_PrintPage_new();
  w=8.3*dpi;
  h=11.7*dpi;
  rLeft=0.787*dpi;
  rRight=0.787*dpi;
  rTop=2.0*dpi;
  rBottom=2.0*dpi;

  AF_PrintPage_SetId(pg, AF_PrintPage_Id_FirstPage);
  AF_PrintPage_SetWidth(pg, w);
  AF_PrintPage_SetHeight(pg, h);
  AF_PrintPage_SetLeftBorder(pg, rLeft);
  AF_PrintPage_SetRightBorder(pg, rRight);
  AF_PrintPage_SetTopBorder(pg, rTop);
  AF_PrintPage_SetBottomBorder(pg, rBottom);

  DBG_ERROR(0, "Page dims: %d x %d", AF_PrintPage_GetWidth(pg), AF_PrintPage_GetHeight(pg));

  /* header box */
  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_HeaderBox);
  AF_PrintBox_SetX(pb, rLeft);
  y=.394*dpi;
  AF_PrintBox_SetY(pb, y);
  AF_PrintBox_SetWidth(pb, w-rLeft-rRight-320);
  AF_PrintBox_SetHeight(pb, 1.5*dpi);
  AF_PrintBox_SetFontSize(pb, dpi/6.5);
  DBG_ERROR(0, "Added Header box: %d x %d", AF_PrintBox_GetWidth(pb), AF_PrintBox_GetHeight(pb));
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

  /* logo box */
  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_LogoBox);
  AF_PrintBox_SetX(pb, w-rRight-320);
  AF_PrintBox_SetY(pb, y);
  AF_PrintBox_SetWidth(pb, 320);
  AF_PrintBox_SetHeight(pb, 320);
  AF_PrintBox_SetFontSize(pb, dpi/6.5);
  DBG_ERROR(0, "Added Logo box: %d x %d", AF_PrintBox_GetWidth(pb), AF_PrintBox_GetHeight(pb));
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

  /* sender address box */
  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_SenderAddressBox);
  AF_PrintBox_SetX(pb, rLeft);
  y=1.969*dpi;
  AF_PrintBox_SetY(pb, y);
  AF_PrintBox_SetWidth(pb, 3.150*dpi);
  AF_PrintBox_SetHeight(pb, 0.394*dpi);
  AF_PrintBox_SetFontSize(pb, dpi/10.0);
  DBG_ERROR(0, "Added Sender address box: %d x %d", AF_PrintBox_GetWidth(pb), AF_PrintBox_GetHeight(pb));
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
  y+=AF_PrintBox_GetHeight(pb);

  /* recipient address box */
  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_RecipientAddressBox);
  AF_PrintBox_SetX(pb, rLeft);
  AF_PrintBox_SetY(pb, y);
  AF_PrintBox_SetWidth(pb, 3.150*dpi);
  AF_PrintBox_SetHeight(pb, 1.181*dpi);
  AF_PrintBox_SetFontSize(pb, dpi/6.250); /* 48 px at 300 dpi */
  DBG_ERROR(0, "Added Recipient address box: %d x %d", AF_PrintBox_GetWidth(pb), AF_PrintBox_GetHeight(pb));
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
  y+=AF_PrintBox_GetHeight(pb);

  /* content box */
  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
  AF_PrintBox_SetX(pb, rLeft);
  AF_PrintBox_SetY(pb, y);
  AF_PrintBox_SetWidth(pb, w-rLeft-rRight);
  AF_PrintBox_SetHeight(pb, h-rBottom-y);
  AF_PrintBox_SetFontSize(pb, dpi/6.250); /* 48 px at 300 dpi */
  DBG_ERROR(0, "Added content box: %d x %d", AF_PrintBox_GetWidth(pb), AF_PrintBox_GetHeight(pb));
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));
  y+=AF_PrintBox_GetHeight(pb);

  /* add first page */
  AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(doc));

  /* second page */
  pg=AF_PrintPage_new();
  w=8.3*dpi;
  h=11.7*dpi;
  rLeft=0.787*dpi;
  rRight=0.787*dpi;
  rTop=0.787*dpi;
  rBottom=0.787*dpi;

  AF_PrintPage_SetId(pg, AF_PrintPage_Id_FollowingPages);
  AF_PrintPage_SetWidth(pg, w);
  AF_PrintPage_SetHeight(pg, h);

  pb=AF_PrintBox_new();
  AF_PrintBox_SetId(pb, AF_PrintBox_Id_ContentBox);
  AF_PrintBox_SetX(pb, rLeft);
  AF_PrintBox_SetY(pb, rTop);
  AF_PrintBox_SetWidth(pb, w-rLeft-rRight);
  AF_PrintBox_SetHeight(pb, h-rTop-rBottom);
  AF_PrintBox_SetFontSize(pb, dpi/6.250); /* 48 px at 300 dpi */
  AF_PrintBox_List_Add(pb, AF_PrintPage_GetPrintBoxes(pg));

  AF_PrintPage_List_Add(pg, AF_PrintDoc_GetPrintPages(doc));

  return doc;
}



int AF_Print_LoadDocTemplate(const char *name, AF_PRINT_DOC **pDoc) {
  GWEN_STRINGLIST *sl;
  GWEN_BUFFER *tbuf;

  tbuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(tbuf, AE_GetUserDataFolder());
  GWEN_Buffer_AppendString(tbuf, GWEN_DIR_SEPARATOR_S "templates" GWEN_DIR_SEPARATOR_S "printdocs");
  sl=GWEN_StringList_new();
  GWEN_Directory_GetMatchingFilesRecursively(GWEN_Buffer_GetStart(tbuf), sl, "*.db");
  GWEN_Buffer_free(tbuf);
  if (GWEN_StringList_Count(sl)) {
    GWEN_STRINGLISTENTRY *se;

    se=GWEN_StringList_FirstEntry(sl);
    while(se) {
      const char *s;

      s=GWEN_StringListEntry_Data(se);
      if (s && *s) {
	GWEN_DB_NODE *db;
        int rv;

	db=GWEN_DB_Group_new("doc");
	rv=GWEN_DB_ReadFile(db, s, GWEN_DB_FLAGS_DEFAULT | GWEN_PATH_FLAGS_CREATE_GROUP);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	}
	else {
	  AF_PRINT_DOC *doc;

	  doc=AF_PrintDoc_fromDb(db);
	  if (doc) {
	    const char *n;

	    AF_PrintDoc_SetFileName(doc, s);
	    n=AF_PrintDoc_GetName(doc);
	    if (n && strcasecmp(n, name)==0) {
	      *pDoc=doc;
	      GWEN_StringList_free(sl);
	      GWEN_DB_Group_free(db);
              return 0;
	    }
	    AF_PrintDoc_free(doc);
	  }
	}
        GWEN_DB_Group_free(db);
      }
      se=GWEN_StringListEntry_Next(se);
    }
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "No doc templates found");
  }
  GWEN_StringList_free(sl);

  return GWEN_ERROR_NOT_FOUND;
}



static int AF_Print_ReplaceDocVars(AF_PRINT *pr, GWEN_DB_NODE *dbValues) {
  AF_PRINT_PAGE *pg;
  int rv;
  GWEN_BUFFER *xbuf;
  int i;

  xbuf=GWEN_Buffer_new(0, 256, 0, 1);
  pg=AF_PrintPage_List_First(AF_PrintDoc_GetPrintPages(pr->currentDoc));
  i=1;
  while(pg) {
    AF_PRINT_BOX *pb;

    GWEN_DB_SetIntValue(dbValues, GWEN_DB_FLAGS_OVERWRITE_VARS, "page", i);

    pb=AF_PrintBox_List_First(AF_PrintPage_GetPrintBoxes(pg));
    while(pb) {
      if (AF_PrintBox_GetId(pb)!=AF_PrintBox_Id_ContentBox) {
	const char *s;

	s=AF_PrintBox_GetText(pb);
	if (s && *s) {
	  rv=AE_Utils_Replace_Vars(s, xbuf, dbValues);
	  if (rv<0) {
	    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	    GWEN_Buffer_free(xbuf);
            return rv;
	  }
	  AF_PrintBox_SetText(pb, GWEN_Buffer_GetStart(xbuf));
	  GWEN_Buffer_Reset(xbuf);
	}
      }
      pb=AF_PrintBox_List_Next(pb);
    }

    i++;
    pg=AF_PrintPage_List_Next(pg);
  }
  GWEN_Buffer_free(xbuf);

  return 0;
}



int AF_Print_Job(AF_PRINT *pr,
		 const AF_PRINT_TABLE_DESCR *descr,
		 const AE_COMMODITY_LIST *coList,
                 const AE_TAX_LIST *taxes,
		 const AE_JOB *j) {
  GWEN_BUFFER *tbuf;
  const char *s;
  GWEN_DB_NODE *db;
  GWEN_DB_NODE *dbJob;
  const GWEN_DATE *cdate;
  GWEN_DATE *date;
  int rv;
  const AE_FIELD_DESCRIPTION *fd;

  fd=AE_Utils_GetItemFieldDescriptions();
  db=GWEN_DB_Group_new("vars");

  /* prepare database */
  dbJob=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "job");
  assert(dbJob);

  /* set job info */
  AE_Job_toDb(j, dbJob);

  /* translate dates */
  cdate=AE_Job_GetEnterDate(j);
  if (cdate) {
    GWEN_BUFFER *xbuf;

    xbuf=GWEN_Buffer_new(0, 32, 0, 1);
    GWEN_Date_toStringWithTemplate(cdate, I18N("YYYY/MM/DD"), xbuf);
    GWEN_DB_SetCharValue(dbJob, GWEN_DB_FLAGS_OVERWRITE_VARS, "enterDate", GWEN_Buffer_GetStart(xbuf));
    GWEN_Buffer_free(xbuf);
  }

  cdate=AE_Job_GetDueDate(j);
  if (cdate) {
    GWEN_BUFFER *xbuf;

    xbuf=GWEN_Buffer_new(0, 32, 0, 1);
    GWEN_Date_toStringWithTemplate(cdate, I18N("YYYY/MM/DD"), xbuf);
    GWEN_DB_SetCharValue(dbJob, GWEN_DB_FLAGS_OVERWRITE_VARS, "dueDate", GWEN_Buffer_GetStart(xbuf));
    GWEN_Buffer_free(xbuf);
  }

  date=GWEN_Date_CurrentDate();
  if (date) {
    GWEN_BUFFER *xbuf;

    xbuf=GWEN_Buffer_new(0, 32, 0, 1);
    GWEN_Date_toStringWithTemplate(date, I18N("YYYY/MM/DD"), xbuf);
    GWEN_DB_SetCharValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "currentDate", GWEN_Buffer_GetStart(xbuf));
    GWEN_Buffer_free(xbuf);
    GWEN_Date_free(date);
  }

  tbuf=GWEN_Buffer_new(0, 1024, 0, 1);

  /* possibly write prologue */
  s=AF_PrintTableDescr_GetPrologue(descr);
  if (s && *s) {
    rv=AE_Utils_Replace_Vars(s, tbuf, db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_DB_Group_free(db);
      GWEN_Buffer_free(tbuf);
      return rv;
    }
  }

  /* add table */
  s=AF_PrintTableDescr_GetColumnList(descr);
  if (s && *s) {
    GWEN_STRINGLIST *sl;
    GWEN_BUFFER *headerBuf;
    GWEN_BUFFER *lbuf;
    GWEN_BUFFER *xbuf;
    AE_ITEM *item;
    const AE_FIELD_DESCRIPTION *d;
    GWEN_STRINGLISTENTRY *se;

    sl=GWEN_StringList_fromTabString(s, 0);

    GWEN_Buffer_AppendString(tbuf, "<table>");
    headerBuf=GWEN_Buffer_new(0, 256, 0, 1);
    GWEN_Buffer_AppendString(headerBuf, "<tr>");

    /* write table headers */
    se=GWEN_StringList_FirstEntry(sl);
    while(se) {
      s=GWEN_StringListEntry_Data(se);
      if (s && *s) {
	d=AE_Utils_GetItemFieldDescriptionByName(fd, s);
	if (d) {
	  GWEN_Buffer_AppendString(headerBuf, "<th>");
          if (d->tableHeader)
	    GWEN_Buffer_AppendString(headerBuf, I18N(d->tableHeader));
	  else
	    GWEN_Buffer_AppendString(headerBuf, I18N(d->text));
	  GWEN_Buffer_AppendString(headerBuf, "</th>");
	}
      }
      se=GWEN_StringListEntry_Next(se);
    }
    GWEN_Buffer_AppendString(headerBuf, "</tr>");

    /* append header */
    GWEN_Buffer_AppendString(tbuf, GWEN_Buffer_GetStart(headerBuf));

    /* now append item lines */
    lbuf=GWEN_Buffer_new(0, 256, 0, 1);
    xbuf=GWEN_Buffer_new(0, 256, 0, 1);
    item=AE_Item_List_First(AE_Job_GetItems(j));
    while(item) {
      uint32_t bpos1;
      uint32_t bpos2;
      GWEN_DB_NODE *dbItem;
      const AQDB_VALUE *v;

      dbItem=GWEN_DB_GetGroup(db, GWEN_DB_FLAGS_OVERWRITE_GROUPS, "item");
      assert(dbItem);
      AE_Item_toDb(item, dbItem);

      v=AE_Item_GetBuyPriceValue(item);
      if (v) {
	GWEN_BUFFER *vbuf;

	vbuf=GWEN_Buffer_new(0, 32, 0, 1);
	AE_Utils_AmountToString(v, AE_Item_GetCommodityId(item), coList, vbuf);
	GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "buyPriceValue", GWEN_Buffer_GetStart(vbuf));
        GWEN_Buffer_free(vbuf);
      }
      v=AE_Item_GetBuyTaxSumValue(item);
      if (v) {
	GWEN_BUFFER *vbuf;

	vbuf=GWEN_Buffer_new(0, 32, 0, 1);
	AE_Utils_AmountToString(v, AE_Item_GetCommodityId(item), coList, vbuf);
	GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "buyTaxSumValue", GWEN_Buffer_GetStart(vbuf));
        GWEN_Buffer_free(vbuf);
      }
      v=AE_Item_GetSellPriceValue(item);
      if (v) {
	GWEN_BUFFER *vbuf;

	vbuf=GWEN_Buffer_new(0, 32, 0, 1);
	AE_Utils_AmountToString(v, AE_Item_GetCommodityId(item), coList, vbuf);
	GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "sellPriceValue", GWEN_Buffer_GetStart(vbuf));
        GWEN_Buffer_free(vbuf);
      }
      v=AE_Item_GetSellTaxSumValue(item);
      if (v) {
	GWEN_BUFFER *vbuf;

	vbuf=GWEN_Buffer_new(0, 32, 0, 1);
	AE_Utils_AmountToString(v, AE_Item_GetCommodityId(item), coList, vbuf);
	GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "sellTaxSumValue", GWEN_Buffer_GetStart(vbuf));
        GWEN_Buffer_free(vbuf);
      }
      v=AE_Item_GetValue(item);
      if (v) {
	GWEN_BUFFER *vbuf;

	vbuf=GWEN_Buffer_new(0, 32, 0, 1);
	AQDB_Value_toHumanReadableString(v, vbuf, 2);
	GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "value_value", GWEN_Buffer_GetStart(vbuf));
	s=AE_Utils_ItemUnitsToString(AE_Item_GetUnits(item));
	if (s && *s) {
	  GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "value_units", s);
	  GWEN_Buffer_AppendString(vbuf, " ");
	  GWEN_Buffer_AppendString(vbuf, s);
	}
	GWEN_DB_SetCharValue(dbItem, GWEN_DB_FLAGS_OVERWRITE_VARS, "value", GWEN_Buffer_GetStart(vbuf));
	GWEN_Buffer_free(vbuf);
      }

      /* add item vars */
      GWEN_Buffer_AppendString(xbuf, "<tr>");
      se=GWEN_StringList_FirstEntry(sl);
      while(se) {
	s=GWEN_StringListEntry_Data(se);
	if (s && *s) {
	  GWEN_Buffer_AppendString(xbuf, "<td");
	  if (strcasecmp(s, "buyPriceValue")==0 ||
	      strcasecmp(s, "buyTaxSumValue")==0 ||
	      strcasecmp(s, "sellPriceValue")==0 ||
	      strcasecmp(s, "sellTaxSumValue")==0 ||
	      strcasecmp(s, "value")==0)
            GWEN_Buffer_AppendString(xbuf, " align=\"right\"");
	  GWEN_Buffer_AppendString(xbuf, ">$(item/");
	  GWEN_Buffer_AppendString(xbuf, s);
	  GWEN_Buffer_AppendString(xbuf, ")");
	  GWEN_Buffer_AppendString(xbuf, "</td>");
	}
	se=GWEN_StringListEntry_Next(se);
      }
      GWEN_Buffer_AppendString(xbuf, "</tr>");

      /* translate item vars */
      rv=AE_Utils_Replace_Vars(GWEN_Buffer_GetStart(xbuf), lbuf, db);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_Buffer_free(xbuf);
	GWEN_Buffer_free(lbuf);
	GWEN_Buffer_free(headerBuf);
	GWEN_DB_Group_free(db);
	GWEN_Buffer_free(tbuf);
	return rv;
      }

      /* check whether the line fits into the page */
      bpos1=GWEN_Buffer_GetPos(tbuf);
      GWEN_Buffer_AppendString(tbuf, GWEN_Buffer_GetStart(lbuf));
      bpos2=GWEN_Buffer_GetPos(tbuf);
      GWEN_Buffer_AppendString(tbuf, "</table>");
      if (AF_Print_CheckText(pr, AF_PrintBox_Id_ContentBox, GWEN_Buffer_GetStart(tbuf))!=0) {
	/* doesn't fit, remove last row, send what we got so far */
	GWEN_Buffer_Crop(tbuf, 0, bpos1);
	GWEN_Buffer_AppendString(tbuf, "</table>");
	AF_Print_SetText(pr, AF_PrintBox_Id_ContentBox, GWEN_Buffer_GetStart(tbuf));

	/* next page */
	rv=AF_Print_NextPage(pr);
	if (rv<0) {
	  DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	  GWEN_Buffer_free(xbuf);
	  GWEN_Buffer_free(lbuf);
	  GWEN_Buffer_free(headerBuf);
	  GWEN_DB_Group_free(db);
	  GWEN_Buffer_free(tbuf);
	  return rv;
	}

	/* setup buffer for next page */
	GWEN_Buffer_Reset(tbuf);

	/* add table header */
	GWEN_Buffer_AppendString(tbuf, GWEN_Buffer_GetStart(headerBuf));
	/* add this row to the next page (we don't actually check whether it fits, we assume it) */
	GWEN_Buffer_AppendString(tbuf, GWEN_Buffer_GetStart(lbuf));
      }
      else {
	/* line fits, remove the "</table>" string */
	GWEN_Buffer_Crop(tbuf, 0, bpos2);
      }

      GWEN_Buffer_Reset(xbuf);
      GWEN_Buffer_Reset(lbuf);

      item=AE_Item_List_Next(item);
    }
    GWEN_Buffer_free(xbuf);
    GWEN_Buffer_free(lbuf);
    GWEN_Buffer_AppendString(tbuf, "</table>");
    GWEN_Buffer_free(headerBuf);
  }

  /* possibly write epilogue */
  s=AF_PrintTableDescr_GetEpilogue(descr);
  if (s && *s) {
    GWEN_BUFFER *lbuf;
    uint32_t bpos;

    lbuf=GWEN_Buffer_new(0, 256, 0, 1);

    rv=AE_Utils_Replace_Vars(s, lbuf, db);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_DB_Group_free(db);
      GWEN_Buffer_free(tbuf);
      return rv;
    }

    /* now check whether the epilogue fits */
    bpos=GWEN_Buffer_GetPos(tbuf);
    GWEN_Buffer_AppendString(tbuf, GWEN_Buffer_GetStart(lbuf));
    if (AF_Print_CheckText(pr, AF_PrintBox_Id_ContentBox, GWEN_Buffer_GetStart(tbuf))!=0) {
      /* doesn't fit, remove last row, send what we got so far */
      GWEN_Buffer_Crop(tbuf, 0, bpos);
      AF_Print_SetText(pr, AF_PrintBox_Id_ContentBox, GWEN_Buffer_GetStart(tbuf));

      /* next page */
      rv=AF_Print_NextPage(pr);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_Buffer_free(lbuf);
	GWEN_DB_Group_free(db);
	GWEN_Buffer_free(tbuf);
	return rv;
      }

      /* add this row to the next page (we don't actually check whether it fits, we assume it) */
      GWEN_Buffer_Reset(tbuf);
      GWEN_Buffer_AppendString(tbuf, GWEN_Buffer_GetStart(lbuf));
    }
    GWEN_Buffer_free(lbuf);
  }

  /* set remaining text into content box and finish */
  if (GWEN_Buffer_GetUsedBytes(tbuf))
    AF_Print_SetText(pr, AF_PrintBox_Id_ContentBox, GWEN_Buffer_GetStart(tbuf));

  GWEN_DB_SetIntValue(db, GWEN_DB_FLAGS_OVERWRITE_VARS, "pages",
		      AF_PrintPage_List_GetCount(AF_PrintDoc_GetPrintPages(pr->currentDoc)));

  /* replace vars in all boxes but content boxes */
  rv=AF_Print_ReplaceDocVars(pr, db);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_DB_Group_free(db);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  GWEN_DB_Group_free(db);
  GWEN_Buffer_free(tbuf);

  rv=AF_Print_Finish(pr);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}





