/***************************************************************************
 begin       : Sat Jun 25 2011
 copyright   : (C) 2011 by Martin Preuss
 email       : martin@libchipcard.de

 ***************************************************************************
 *          Please see toplevel file COPYING for license details           *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
# include <config.h>
#endif


#include "af_update_p.h"

#include <aqfinance/engine/book/ae_utils.h>
#include <aqfinance/engine/book/ae_utils.h>

#include <gwenhywfar/debug.h>
#include <gwenhywfar/misc.h>
#include <gwenhywfar/httpsession.h>
#include <gwenhywfar/syncio_file.h>
#include <gwenhywfar/gui.h>
#include <gwenhywfar/directory.h>
#include <gwenhywfar/sar.h>
#include <gwenhywfar/mdigest.h>

#include <zlib.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <errno.h>
#include <string.h>





AF_UPDATE *AF_Update_new(const char *server, const char *dataFolder) {
  AF_UPDATE *upd;

  assert(server);
  assert(dataFolder);

  GWEN_NEW_OBJECT(AF_UPDATE, upd);
  upd->refCount=1;

  upd->server=strdup(server);
  upd->dataFolder=strdup(dataFolder);

  return upd;
}



void AF_Update_Attach(AF_UPDATE *upd) {
  assert(upd);
  assert(upd->refCount);
  upd->refCount++;
}



void AF_Update_free(AF_UPDATE *upd) {
  if (upd) {
    assert(upd->refCount);
    if (upd->refCount==1) {
      free(upd->server);
      free(upd->appName);

      GWEN_Crypt_Key_free(upd->pubKey);

      upd->refCount=0;
      GWEN_FREE_OBJECT(upd);
    }
    else
      upd->refCount--;
  }
}



void AF_Update_SetPubKey(AF_UPDATE *upd, GWEN_CRYPT_KEY *pubKey) {
  assert(upd);
  assert(upd->refCount);
  GWEN_Crypt_Key_free(upd->pubKey);
  upd->pubKey=pubKey;
}



int AF_Update_Init(AF_UPDATE *upd) {
  int rv;

  rv=AF_Update_ReadProgramVersion(upd);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  return 0;
}



int AF_Update_Fini(AF_UPDATE *upd) {
  return 0;
}



int AF_Update_GetProgramFolder(AF_UPDATE *upd, GWEN_BUFFER *buf) {
  FILE *f;
  char current[256];
  int len;
  uint32_t pos;

  GWEN_Buffer_AppendString(buf, upd->dataFolder);
  GWEN_Buffer_AppendString(buf, GWEN_DIR_SEPARATOR_S "program" GWEN_DIR_SEPARATOR_S);
  pos=GWEN_Buffer_GetPos(buf);
  GWEN_Buffer_AppendString(buf, "current");

  /* read content of file "current" */
  f=fopen(GWEN_Buffer_GetStart(buf), "r");
  if (f==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "fopen(%s): %s", GWEN_Buffer_GetStart(buf), strerror(errno));
    return GWEN_ERROR_IO;
  }

  if (NULL==fgets(current, sizeof(current)-1, f)) {
    DBG_ERROR(AE_LOGDOMAIN, "fgets(%s): %s", GWEN_Buffer_GetStart(buf), strerror(errno));
    fclose(f);
    return GWEN_ERROR_IO;
  }
  current[sizeof(current)-1]=0;
  fclose(f);

  len=strlen(current);
  if (len && current[len-1]==10)
    current[len-1]=0;

  GWEN_Buffer_Crop(buf, 0, pos);
  GWEN_Buffer_AppendString(buf, current);

  return 0;
}



int AF_Update_ReadProgramVersion(AF_UPDATE *upd) {
  GWEN_BUFFER *buf;
  int rv;
  GWEN_DB_NODE *db;
  AF_UPDATE_RELEASE *ur;


  buf=GWEN_Buffer_new(0, 256, 0, 1);
  rv=AF_Update_GetProgramFolder(upd, buf);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(buf);
    return rv;
  }

  GWEN_Buffer_AppendString(buf, GWEN_DIR_SEPARATOR_S "version.conf");
  db=GWEN_DB_Group_new("version");
  rv=GWEN_DB_ReadFile(db, GWEN_Buffer_GetStart(buf), GWEN_DB_FLAGS_DEFAULT);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_DB_Group_free(db);
    GWEN_Buffer_free(buf);
    return rv;
  }

  /* create release info from db */
  ur=AF_UpdateRelease_fromDb(db);
  if (ur) {
    const char *s;

    /* appname */
    free(upd->appName);
    upd->appName=NULL;
    s=AF_UpdateRelease_GetName(ur);
    if (s && *s)
      upd->appName=strdup(s);

    /* sysName */
    free(upd->sysName);
    upd->sysName=NULL;
    s=AF_UpdateRelease_GetSystem(ur);
    if (s && *s)
      upd->sysName=strdup(s);

    /* archName */
    free(upd->archName);
    upd->archName=NULL;
    s=AF_UpdateRelease_GetArchitecture(ur);
    if (s && *s)
      upd->archName=strdup(s);

    upd->vMajor=AF_UpdateRelease_GetVersionMajor(ur);
    upd->vMinor=AF_UpdateRelease_GetVersionMinor(ur);
    upd->vPatchLevel=AF_UpdateRelease_GetVersionPatchLevel(ur);
    upd->vBuild=AF_UpdateRelease_GetVersionBuild(ur);
    upd->maturity=AF_UpdateRelease_GetMaturityLevel(ur);
  }
  AF_UpdateRelease_free(ur);

  GWEN_DB_Group_free(db);
  GWEN_Buffer_free(buf);

  return 0;
}



int AF_Update_Download_ReleaseList(AF_UPDATE *upd, GWEN_BUFFER *lbuf) {
  GWEN_HTTP_SESSION *sess;
  GWEN_BUFFER *ubuf;
  char numbuf[32];
  int rv;

  assert(upd);
  assert(upd->refCount);

  ubuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(ubuf, upd->server);
  GWEN_Buffer_AppendString(ubuf, "/");
  GWEN_Buffer_AppendString(ubuf, upd->appName);
  GWEN_Buffer_AppendString(ubuf, "-");
  GWEN_Buffer_AppendString(ubuf, upd->sysName);
  GWEN_Buffer_AppendString(ubuf, "-");
  GWEN_Buffer_AppendString(ubuf, upd->archName);
  GWEN_Buffer_AppendString(ubuf, "-");
  snprintf(numbuf, sizeof(numbuf)-1, "%d", upd->vMajor);
  numbuf[sizeof(numbuf)-1]=0;
  GWEN_Buffer_AppendString(ubuf, numbuf);

  DBG_INFO(0, "Using URL: [%s]", GWEN_Buffer_GetStart(ubuf));
  sess=GWEN_HttpSession_new(GWEN_Buffer_GetStart(ubuf), "http", 80);
  GWEN_Buffer_free(ubuf);

  GWEN_HttpSession_AddFlags(sess, GWEN_HTTP_SESSION_FLAGS_NO_CACHE);

  GWEN_HttpSession_SetHttpUserAgent(sess, "af_update");
  GWEN_HttpSession_SetHttpVMajor(sess, 1);
  GWEN_HttpSession_SetHttpVMinor(sess, 1);

  /* init session */
  rv=GWEN_HttpSession_Init(sess);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  /* send request */
  rv=GWEN_HttpSession_SendPacket(sess, "GET", NULL, 0);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  /* receive response */
  rv=GWEN_HttpSession_RecvPacket(sess, lbuf);
  if (rv<200 || rv>=300) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  /* fini session */
  rv=GWEN_HttpSession_Fini(sess);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  GWEN_HttpSession_free(sess);
  return 0;
}



int AF_Update_Download_ReleaseInfo(AF_UPDATE *upd, const char *rname, GWEN_BUFFER *lbuf) {
  GWEN_HTTP_SESSION *sess;
  GWEN_BUFFER *ubuf;
  int rv;

  assert(upd);
  assert(upd->refCount);

  ubuf=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(ubuf, upd->server);
  GWEN_Buffer_AppendString(ubuf, "/");
  GWEN_Buffer_AppendString(ubuf, upd->appName);
  GWEN_Buffer_AppendString(ubuf, "-");
  GWEN_Buffer_AppendString(ubuf, upd->sysName);
  GWEN_Buffer_AppendString(ubuf, "-");
  GWEN_Buffer_AppendString(ubuf, upd->archName);
  GWEN_Buffer_AppendString(ubuf, "-");
  GWEN_Buffer_AppendString(ubuf, rname);

  sess=GWEN_HttpSession_new(GWEN_Buffer_GetStart(ubuf), "http", 80);
  GWEN_Buffer_free(ubuf);
  GWEN_HttpSession_AddFlags(sess, GWEN_HTTP_SESSION_FLAGS_NO_CACHE);

  GWEN_HttpSession_SetHttpUserAgent(sess, "af_update");
  GWEN_HttpSession_SetHttpVMajor(sess, 1);
  GWEN_HttpSession_SetHttpVMinor(sess, 1);

  /* init session */
  rv=GWEN_HttpSession_Init(sess);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  /* send request */
  rv=GWEN_HttpSession_SendPacket(sess, "GET", NULL, 0);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  /* receive response */
  rv=GWEN_HttpSession_RecvPacket(sess, lbuf);
  if (rv<200 || rv>=300) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  /* fini session */
  rv=GWEN_HttpSession_Fini(sess);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_HttpSession_free(sess);
    return rv;
  }

  GWEN_HttpSession_free(sess);
  return 0;
}



int AF_Update_ExtractNewReleaseList(AF_UPDATE *upd, GWEN_BUFFER *lbuf, GWEN_STRINGLIST *sl) {
  const char delims[]="\n,";
  GWEN_STRINGLIST *sll;
  GWEN_STRINGLISTENTRY *se;
  char vbuf[32];
  int foundThis=0;

  sll=GWEN_StringList_fromString(GWEN_Buffer_GetStart(lbuf), delims, 1);
  if (sll==NULL) {
    DBG_ERROR(AE_LOGDOMAIN, "No releases found");
    return GWEN_ERROR_BAD_DATA;
  }

  snprintf(vbuf, sizeof(vbuf)-1, "%d.%d.%d.%d-%d",
           upd->vMajor,
           upd->vMinor,
           upd->vPatchLevel,
           upd->vBuild,
           upd->maturity);
  vbuf[sizeof(vbuf)-1]=0;

  /* copy all entries after the one for this release into the list */
  se=GWEN_StringList_FirstEntry(sll);
  while(se) {
    const char *s;

    s=GWEN_StringListEntry_Data(se);
    if (s && *s && strcasecmp(vbuf, s)==0)
      foundThis=1;
    else if (foundThis)
      GWEN_StringList_AppendString(sl, s, 0, 1);
    se=GWEN_StringListEntry_Next(se);
  }

  if (!foundThis) {
    DBG_ERROR(AE_LOGDOMAIN, "This release (%s) is not in the list", vbuf);
    GWEN_StringList_free(sll);
    return GWEN_ERROR_NOT_FOUND;
  }

  GWEN_StringList_free(sll);

  return 0;
}



int AF_Update_DownloadNewReleaseInfos(AF_UPDATE *upd, AF_UPDATE_RELEASE_LIST *rl) {
  GWEN_BUFFER *tbuf;
  GWEN_STRINGLIST *sl;
  GWEN_STRINGLISTENTRY *se;
  int rv;

  /* download list of releases */
  tbuf=GWEN_Buffer_new(0, 1024, 0, 1);
  rv=AF_Update_Download_ReleaseList(upd, tbuf);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Buffer_free(tbuf);
    return rv;
  }

  /* extract new releases */
  sl=GWEN_StringList_new();
  rv=AF_Update_ExtractNewReleaseList(upd, tbuf, sl);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_StringList_free(sl);
    GWEN_Buffer_free(tbuf);
    return rv;
  }
  GWEN_Buffer_Reset(tbuf);

  /* download infos about new releases */
  se=GWEN_StringList_FirstEntry(sl);
  while(se) {
    const char *s;

    s=GWEN_StringListEntry_Data(se);
    if (s && *s) {
      GWEN_DB_NODE *db;
      AF_UPDATE_RELEASE *ur;

      /* download release info */
      DBG_INFO(AE_LOGDOMAIN, "Downloading release info [%s]", s);
      rv=AF_Update_Download_ReleaseInfo(upd, s, tbuf);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_StringList_free(sl);
        GWEN_Buffer_free(tbuf);
        return rv;
      }

      /* parse data as db */
      db=GWEN_DB_Group_new("release");
      rv=GWEN_DB_ReadFromString(db,
                                GWEN_Buffer_GetStart(tbuf),
                                GWEN_Buffer_GetUsedBytes(tbuf),
                                GWEN_DB_FLAGS_DEFAULT);
      if (rv<0) {
        DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
        GWEN_DB_Group_free(db);
        GWEN_StringList_free(sl);
        GWEN_Buffer_free(tbuf);
        return rv;
      }

      /* create release info from db */
      ur=AF_UpdateRelease_fromDb(db);
      if (ur) {
        DBG_INFO(AE_LOGDOMAIN, "Adding release [%s]", s);
        AF_UpdateRelease_List_Add(ur, rl);
      }
      GWEN_DB_Group_free(db);
      GWEN_Buffer_Reset(tbuf);
    }

    se=GWEN_StringListEntry_Next(se);
  }

  GWEN_StringList_free(sl);
  GWEN_Buffer_free(tbuf);

  return 0;
}



int AF_Update_DownloadRelease(AF_UPDATE *upd,
			      const AF_UPDATE_RELEASE *rel,
			      const char *fname) {
  const char *url;

  assert(upd);
  assert(upd->refCount);

  assert(fname);

  url=AF_UpdateRelease_GetUrl(rel);
  if (url) {
    GWEN_HTTP_SESSION *sess;
    int rv;

    DBG_ERROR(AE_LOGDOMAIN, "Downloading [%s]", url);
    sess=GWEN_HttpSession_new(url, "http", 80);
    GWEN_HttpSession_AddFlags(sess, GWEN_HTTP_SESSION_FLAGS_NO_CACHE);

    GWEN_HttpSession_SetHttpUserAgent(sess, "af_update");
    GWEN_HttpSession_SetHttpVMajor(sess, 1);
    GWEN_HttpSession_SetHttpVMinor(sess, 1);

    /* init session */
    rv=GWEN_HttpSession_Init(sess);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_HttpSession_free(sess);
      return rv;
    }

    /* send request */
    rv=GWEN_HttpSession_SendPacket(sess, "GET", NULL, 0);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_HttpSession_free(sess);
      return rv;
    }

    /* receive response */
    rv=GWEN_HttpSession_RecvPacketToFile(sess, fname);
    if (rv<200 || rv>=300) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_HttpSession_free(sess);
      return rv;
    }

    /* fini session */
    rv=GWEN_HttpSession_Fini(sess);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_HttpSession_free(sess);
      return rv;
    }

    GWEN_HttpSession_free(sess);
    return 0;
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "No URL given");
    return GWEN_ERROR_INVALID;
  }
}



int AF_Update_UnzipSio2Sio(GWEN_SYNCIO *sioIn, uint64_t inSize, GWEN_SYNCIO *sioOut) {
  int ret;
  unsigned int have;
  z_stream strm;
  unsigned char in[AF_UPDATE_UNZIP_CHUNK];
  unsigned char out[AF_UPDATE_UNZIP_CHUNK];
  uint32_t pid;
  uint64_t bytesDone=0;

  /* allocate inflate state */
  strm.zalloc = Z_NULL;
  strm.zfree = Z_NULL;
  strm.opaque = Z_NULL;
  strm.avail_in = 0;
  strm.next_in = Z_NULL;
  ret = inflateInit2(&strm, 15+32);
  if (ret != Z_OK) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", ret);
    return GWEN_ERROR_BAD_DATA;
  }

  pid=GWEN_Gui_ProgressStart(GWEN_GUI_PROGRESS_DELAY |
			     GWEN_GUI_PROGRESS_SHOW_ABORT |
			     GWEN_GUI_PROGRESS_ALLOW_EMBED |
			     GWEN_GUI_PROGRESS_SHOW_PROGRESS,
			     I18N("File Operation"),
			     I18N("Unzipping release file"),
			     inSize,
			     0);


  /* decompress until deflate stream ends or end of file */
  do {
    uint64_t bs;
    int rv;

    rv=GWEN_SyncIo_Read(sioIn, in, AF_UPDATE_UNZIP_CHUNK);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      (void)inflateEnd(&strm);
      return rv;
    }
    else if (rv==0) {
      if (bytesDone==inSize) {
	DBG_INFO(AE_LOGDOMAIN, "EOF met");
	break;
      }
      else {
	DBG_INFO(AE_LOGDOMAIN, "Premature EOF");
	(void)inflateEnd(&strm);
	GWEN_Gui_ProgressEnd(pid);
	return GWEN_ERROR_BAD_DATA;
      }
    }
    bs=rv;

    strm.avail_in = bs;

    if (strm.avail_in == 0)
      break;
    strm.next_in = in;

    /* run inflate() on input until output buffer not full */
    do {
      strm.avail_out = AF_UPDATE_UNZIP_CHUNK;
      strm.next_out = out;
      ret = inflate(&strm, Z_NO_FLUSH);
      if (ret==Z_STREAM_ERROR) {
	DBG_INFO(AE_LOGDOMAIN, "inflate(): %d", ret);
	(void)inflateEnd(&strm);
	GWEN_Gui_ProgressEnd(pid);
	return GWEN_ERROR_BAD_DATA;
      }

      switch (ret) {
      case Z_NEED_DICT:
	ret = Z_DATA_ERROR;     /* and fall through */
      case Z_DATA_ERROR:
      case Z_MEM_ERROR:
	DBG_INFO(AE_LOGDOMAIN, "Error on inflate(): %d (%s)", ret, strm.msg);
	(void)inflateEnd(&strm);
	GWEN_Gui_ProgressEnd(pid);
	return GWEN_ERROR_BAD_DATA;
      }
      have = AF_UPDATE_UNZIP_CHUNK - strm.avail_out;

      rv=GWEN_SyncIo_WriteForced(sioOut, out, have);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	(void)inflateEnd(&strm);
	GWEN_Gui_ProgressEnd(pid);
	return rv;
      }
    } while (strm.avail_out == 0);

    /* advance progress bar */
    bytesDone+=bs;
    rv=GWEN_Gui_ProgressAdvance(pid, bytesDone);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_Gui_ProgressEnd(pid);
      inflateEnd(&strm);
      return rv;
    }

    /* done when inflate() says it's done */
  } while (ret != Z_STREAM_END);

  /* clean up and return */
  (void)inflateEnd(&strm);
  GWEN_Gui_ProgressEnd(pid);
  return ret == Z_STREAM_END ? 0 : GWEN_ERROR_BAD_DATA;
}






int AF_Update_UnzipFile(AF_UPDATE *upd, const char *inFile, const char *outFile) {
  struct stat st;
  uint64_t inSize;
  int rv;

  rv=stat(inFile, &st);
  if (rv) {
    DBG_ERROR(AE_LOGDOMAIN, "stat(%s): %d (%s)",
	      inFile, errno, strerror(errno));
    return GWEN_ERROR_IO;
  }
  inSize=(uint64_t)(st.st_size);
  if (inSize>0) {
    GWEN_SYNCIO *sioIn;
    GWEN_SYNCIO *sioOut;

    sioIn=GWEN_SyncIo_File_new(inFile, GWEN_SyncIo_File_CreationMode_OpenExisting);
    GWEN_SyncIo_AddFlags(sioIn, GWEN_SYNCIO_FILE_FLAGS_READ);
    rv=GWEN_SyncIo_Connect(sioIn);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sioIn);
      return rv;
    }

    sioOut=GWEN_SyncIo_File_new(outFile, GWEN_SyncIo_File_CreationMode_CreateAlways);
    GWEN_SyncIo_AddFlags(sioOut,
			 GWEN_SYNCIO_FILE_FLAGS_READ |
			 GWEN_SYNCIO_FILE_FLAGS_WRITE |
			 GWEN_SYNCIO_FILE_FLAGS_UREAD |
			 GWEN_SYNCIO_FILE_FLAGS_UWRITE |
			 GWEN_SYNCIO_FILE_FLAGS_GREAD |
			 GWEN_SYNCIO_FILE_FLAGS_GWRITE);
    rv=GWEN_SyncIo_Connect(sioOut);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sioOut);
      GWEN_SyncIo_Disconnect(sioIn);
      GWEN_SyncIo_free(sioIn);
      return rv;
    }

    rv=AF_Update_UnzipSio2Sio(sioIn, inSize, sioOut);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sioOut);
      GWEN_SyncIo_Disconnect(sioIn);
      GWEN_SyncIo_free(sioIn);
      return rv;
    }
    /* close input file */
    GWEN_SyncIo_Disconnect(sioIn);
    GWEN_SyncIo_free(sioIn);

    /* flush output file */
    rv=GWEN_SyncIo_Flush(sioOut);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_Disconnect(sioOut);
      GWEN_SyncIo_free(sioOut);
      return rv;
    }

    /* close output file */
    rv=GWEN_SyncIo_Disconnect(sioOut);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_SyncIo_free(sioOut);
      return rv;
    }

    return 0;
  }
  else {
    DBG_ERROR(AE_LOGDOMAIN, "Empty file");
    return GWEN_ERROR_BAD_DATA;
  }
}



int AF_Update_UnpackFile(AF_UPDATE *upd, const char *inFile) {
  GWEN_SAR *sr;
  int rv;
  const GWEN_SAR_FILEHEADER_LIST *fhl;

  sr=GWEN_Sar_new();
  rv=GWEN_Sar_OpenArchive(sr, inFile,
			  GWEN_SyncIo_File_CreationMode_OpenExisting,
			  GWEN_SYNCIO_FILE_FLAGS_READ);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }

  fhl=GWEN_Sar_GetHeaders(sr);
  if (fhl) {
    const GWEN_SAR_FILEHEADER *fh;

    fh=GWEN_SarFileHeader_List_First(fhl);
    while(fh) {
      rv=GWEN_Sar_ExtractFile(sr, fh);
      if (rv<0) {
	DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
	GWEN_Sar_CloseArchive(sr, 1);
	GWEN_Sar_free(sr);
      }

      fh=GWEN_SarFileHeader_List_Next(fh);
    }
  }

  rv=GWEN_Sar_CloseArchive(sr, 0);
  if (rv<0) {
    fprintf(stderr, "Error closing archive (%d)\n", rv);
    return 2;
  }

  return 0;
}



int AF_Update_InstallRelease(AF_UPDATE *upd, const AF_UPDATE_RELEASE *rel, uint32_t pid) {
  GWEN_BUFFER *tbuf1;
  GWEN_TIME *ti;
  char prgInstallPath[256];
  char prgFile[256];
  char prgUnzippedFile[256];
  char prgCurrent[256];
  char tmpPath[256];
  char dtBuf[32];
  int rv;
  uint32_t pos;
  FILE *f;
  GWEN_MDIGEST *md;
  
  /* get tmp folder */
  rv=GWEN_Directory_GetTmpDirectory(tmpPath, sizeof(tmpPath)-1);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    return rv;
  }
  tmpPath[sizeof(tmpPath)-1]=0;

  /* create unzipped name */
  tbuf1=GWEN_Buffer_new(0, 256, 0, 1);
  GWEN_Buffer_AppendString(tbuf1, tmpPath);
  GWEN_Buffer_AppendString(tbuf1, GWEN_DIR_SEPARATOR_S);
  GWEN_Buffer_AppendString(tbuf1, "update.sar");
  if (GWEN_Buffer_GetUsedBytes(tbuf1)>=(sizeof(prgUnzippedFile)-1)) {
    DBG_ERROR(AE_LOGDOMAIN, "Path too long");
    GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error,
			 I18N("Internal error: Path too long"));
    GWEN_Buffer_free(tbuf1);
    return GWEN_ERROR_INTERNAL;
  }
  strncpy(prgUnzippedFile, GWEN_Buffer_GetStart(tbuf1), sizeof(prgUnzippedFile)-1);
  prgUnzippedFile[sizeof(prgUnzippedFile)-1]=0;

  /* create zipped file name */
  GWEN_Buffer_AppendString(tbuf1, ".gz");
  if (GWEN_Buffer_GetUsedBytes(tbuf1)>=(sizeof(prgFile)-1)) {
    DBG_ERROR(AE_LOGDOMAIN, "Path too long");
    GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Error,
			 I18N("Internal error: Path too long"));
    GWEN_Buffer_free(tbuf1);
    return GWEN_ERROR_INTERNAL;
  }
  strncpy(prgFile, GWEN_Buffer_GetStart(tbuf1), sizeof(prgFile)-1);
  prgFile[sizeof(prgFile)-1]=0;


  /* create folder name */
  ti=GWEN_CurrentTime();
  assert(ti);
  GWEN_Buffer_Reset(tbuf1);
  rv=GWEN_Time_toString(ti, "YYYYMMDDhhmmss", tbuf1);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Time_free(ti);
    GWEN_Buffer_free(tbuf1);
    return rv;
  }
  if (GWEN_Buffer_GetUsedBytes(tbuf1)>=(sizeof(dtBuf)-1)) {
    DBG_ERROR(AE_LOGDOMAIN, "Path too long");
    GWEN_Buffer_free(tbuf1);
    return GWEN_ERROR_INTERNAL;
  }
  strncpy(dtBuf, GWEN_Buffer_GetStart(tbuf1), sizeof(dtBuf)-1);
  dtBuf[sizeof(dtBuf)-1]=0;
  GWEN_Time_free(ti);
  
  /* create installation names */
  GWEN_Buffer_Reset(tbuf1);

  GWEN_Buffer_AppendString(tbuf1, upd->dataFolder);
  GWEN_Buffer_AppendString(tbuf1,
			   GWEN_DIR_SEPARATOR_S
			   "program"
			   GWEN_DIR_SEPARATOR_S);
  pos=GWEN_Buffer_GetPos(tbuf1);

  /* create new version folder */
  GWEN_Buffer_AppendString(tbuf1, dtBuf);
  if (GWEN_Buffer_GetUsedBytes(tbuf1)>(sizeof(prgInstallPath)-1)) {
    DBG_ERROR(AE_LOGDOMAIN, "Path too long");
    GWEN_Buffer_free(tbuf1);
    return GWEN_ERROR_INTERNAL;
  }
  strncpy(prgInstallPath, GWEN_Buffer_GetStart(tbuf1), sizeof(prgInstallPath)-1);
  prgInstallPath[sizeof(prgInstallPath)-1]=0;

  GWEN_Buffer_Crop(tbuf1, 0, pos);
  GWEN_Buffer_AppendString(tbuf1, "current");
  if (GWEN_Buffer_GetUsedBytes(tbuf1)>(sizeof(prgCurrent)-1)) {
    DBG_ERROR(AE_LOGDOMAIN, "Path too long");
    GWEN_Buffer_free(tbuf1);
    return GWEN_ERROR_INTERNAL;
  }
  strncpy(prgCurrent, GWEN_Buffer_GetStart(tbuf1), sizeof(prgCurrent)-1);
  prgCurrent[sizeof(prgCurrent)-1]=0;

  GWEN_Buffer_free(tbuf1);

  /* download release */
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
		       I18N("Downloading update archive..."));
  rv=AF_Update_DownloadRelease(upd, rel, prgFile);
  if (rv!=0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Error downloading update file (%d)"),
			  rv);
    unlink(prgFile);
    return rv;
  }

  /* unzip release */
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
		       I18N("Unzipping update archive..."));
  rv=AF_Update_UnzipFile(upd, prgFile, prgUnzippedFile);
  if (rv!=0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Error unzipping update file (%d)"),
			  rv);
    unlink(prgFile);
    return rv;
  }

  /* prepare install path */
  rv=GWEN_Directory_GetPath(prgInstallPath, GWEN_PATH_FLAGS_CHECKROOT);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Error preparing path for update (%d)"),
			  rv);
    unlink(prgFile);
    unlink(prgUnzippedFile);
    return rv;
  }

  /* check archive */
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
		       I18N("Checking consistency of the update archive..."));
  rv=GWEN_Sar_CheckArchive(prgUnzippedFile);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
                          I18N("Error while checking consistency of the update archive (%d)"),
			  rv);
    unlink(prgFile);
    unlink(prgUnzippedFile);
    return rv;
  }

  if (upd->pubKey) {
    /* verify archive */
    GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
                         I18N("Verifying signature of the update archive..."));
    rv=GWEN_Sar_VerifyArchive(prgUnzippedFile, "AQUAMANIAC", upd->pubKey);
    if (rv<0) {
      DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
      GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
                            I18N("Error while Verifying signature of the update archive (%d)"),
                            rv);
      unlink(prgFile);
      unlink(prgUnzippedFile);
      return rv;
    }
  }

  /* unpack */
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
		       I18N("Unpacking update archive..."));
  rv=GWEN_Sar_UnpackArchive(prgUnzippedFile, prgInstallPath);
  if (rv<0) {
    DBG_INFO(AE_LOGDOMAIN, "here (%d)", rv);
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Error preparing path for update (%d)"),
			  rv);
    unlink(prgFile);
    unlink(prgUnzippedFile);
    return rv;
  }

  /* generate hash list */
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
		       I18N("Verifying update package..."));
  md=GWEN_MDigest_Rmd160_new();
  rv=GWEN_MDigest_CheckFileTree(md, prgInstallPath, "checksums.rmd", 0, pid);
  if (rv<0) {
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Integrity check on update package failed, not updating"));
    GWEN_MDigest_free(md);
    unlink(prgFile);
    unlink(prgUnzippedFile);
    return rv;
  }
  GWEN_MDigest_free(md);
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
                       I18N("Update package is valid."));


  /* symlink "current" */
  GWEN_Gui_ProgressLog(pid, GWEN_LoggerLevel_Notice,
		       I18N("Selecting new version for next start..."));
  f=fopen(prgCurrent, "w");
  if (f==NULL) {
    DBG_ERROR(AE_LOGDOMAIN,
	      "Could not overwrite version file, installation might be broken (%s)",
	      strerror(errno));
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Could not overwrite version file, installation might be broken (%s)"),
			  strerror(errno));
    unlink(prgFile);
    unlink(prgUnzippedFile);
    return GWEN_ERROR_IO;
  }

  fprintf(f, "%s\n", dtBuf);
  if (fclose(f)) {
    DBG_ERROR(AE_LOGDOMAIN,
	      "Could not write into version file, installation might be broken (%s)",
	      strerror(errno));
    GWEN_Gui_ProgressLog2(pid, GWEN_LoggerLevel_Error,
			  I18N("Could not write into version file, installation might be broken (%s)"),
			  strerror(errno));
    unlink(prgFile);
    unlink(prgUnzippedFile);
    return GWEN_ERROR_IO;
  }

  unlink(prgFile);
  unlink(prgUnzippedFile);

  return 0;
}










